/*
	This file contains the DDL to build the ModelGen database.
*/

//Drop the tables if they already exist. We follow a backtrace of dependencies.

NOROLLBACK DROP TABLE Instance;
NOROLLBACK DROP TABLE InstanceSM;
NOROLLBACK DROP TABLE Scheme; //Note that this will fail on a database where the model tables have already been built. It cannot be helped, sinc the tables are not known here (thus thy can't be dropped).
NOROLLBACK DROP TABLE SchemeSM; //The same problem as above applies here too.
NOROLLBACK DROP TABLE MSM_Dependency;
NOROLLBACK DROP TABLE Mapping_MR_SMR;
NOROLLBACK DROP TABLE Mapping_MP_SMP;
NOROLLBACK DROP TABLE Mapping_MC_SMC;
NOROLLBACK DROP TABLE MSM_Reference;
NOROLLBACK DROP TABLE MSM_Property;
NOROLLBACK DROP TABLE MSM_Construct;
NOROLLBACK DROP TABLE MSM_CarSet;
NOROLLBACK DROP TABLE MSM_Car;
NOROLLBACK DROP TABLE MM_Reference;
NOROLLBACK DROP TABLE MM_Property;
NOROLLBACK DROP TABLE MM_Construct;
NOROLLBACK DROP TABLE Model;

//Model related tables:
CREATE TABLE Model(
	OID int identity primary key,
	name varchar not null,
	shortname varchar not null,
	familyOID int,
	isAncestor boolean not null,
	unique (name)
);

CREATE TABLE MM_Construct(
	OID int identity primary key,
	modelOID int,
	name varchar,
	signature varchar not null,
	bonds varchar not null,
	isLexical boolean,
	modelName varchar, //This is in violation of Boyce-Codd Normal Form, but it's just temporary.
	unique (modelOID, name),
	foreign key (modelOID) references Model(OID)
);

CREATE TABLE MM_Property(
	OID int identity primary key,
	constructOID int,
	name varchar,
	type varchar,
	foreign key (constructOID) references MM_Construct(OID)
);

CREATE TABLE MM_Reference(
	OID int identity primary key,
	ownerConstructOID int,
	name varchar,
	type varchar,
	targetConstructOID int,
	foreign key (ownerConstructOID) references MM_Construct(OID),
	foreign key (targetConstructOID) references MM_Construct(OID)
);

CREATE TABLE Family(
	OID int identity primary key,
	name varchar not null,
	modelOID int,
	unique (name),
	foreign key (modelOID) references Model(OID)
);


//Super Model
CREATE TABLE MSM_Construct(
	OID int identity primary key,
	name varchar,
	isLexical boolean
);

CREATE TABLE MSM_Property(
	OID int identity primary key,
	constructOID int,
	name varchar,
	type varchar,
	default varchar,
	foreign key (constructOID) references MSM_Construct(OID)
);

//Characterizing Set (divides construct references in sets)
CREATE TABLE MSM_Carset (
	OID int identity primary key,
	name varchar,
	constructOID int,
	foreign key (constructOID) references MSM_Construct(OID)
);

CREATE TABLE MSM_Reference(
	OID int identity primary key,
	ownerConstructOID int,
	name varchar,
	type varchar,
	targetConstructOID int,
	foreign key (ownerConstructOID) references MSM_Construct(OID),
	foreign key (targetConstructOID) references MSM_Construct(OID)
);

//Characterization. (specifies the owner carset for each reference)
CREATE TABLE MSM_Car (
	OID int identity primary key,
	carSetOID int,
	referenceOID int,
	foreign key (carSetOID) references MSM_CarSet(OID),
	foreign key (referenceOID) references MSM_Reference(OID)
);

//Dependencies:
CREATE TABLE MSM_Dependency(
	parent int,
	child int,
	foreign key (parent) references MSM_Construct(OID),
	foreign key (child) references MSM_Construct(OID),
	unique (parent, child)
);

//Relations between MM and MSM.
CREATE TABLE Mapping_MC_SMC(
	constructOID_SM int,
	constructOID_M int,
	foreign key (constructOID_SM) references MSM_Construct(OID),
	foreign key (constructOID_M) references MM_Construct(OID)
);

CREATE TABLE Mapping_MP_SMP(
	propertyOID_SM int,
	propertyOID_M int,
	foreign key (propertyOID_SM) references MSM_Property(OID),
	foreign key (propertyOID_M) references MM_Property(OID)	
);

CREATE TABLE Mapping_MR_SMR(
	referenceOID_SM int,
	referenceOID_M int,
	foreign key (referenceOID_SM) references MSM_Reference(OID),
	foreign key (referenceOID_M) references MM_Reference(OID)
);

//Scheme related tables
CREATE TABLE Scheme(
	OID int /*identity*/ primary key,
	name varchar not null,
	modelOID int,
	foreign key(modelOID) references Model(OID)
);

CREATE TABLE SchemeSM(
	OID int /*identity*/ primary key,
	name varchar not null,
	modelOID int,
	description varchar,
	sourceSchemeOID int,
	foreign key(modelOID) references Model(OID),
	foreign key(sourceSchemeOID) references SchemeSM(OID)
);

CREATE TABLE InstanceSM(
	OID int primary key,
	name varchar not null,
	schemeOID int,
	foreign key(schemeOID) references SchemeSM(OID)
);

CREATE TABLE Instance(
	OID int primary key,
	name varchar not null,
	schemeOID int,
	foreign key(schemeOID) references Scheme(OID)
);

