
package org.modelgen.importexport.Export;

import org.modelgen.importexport.relational.model.Column;
import org.modelgen.importexport.relational.model.ComponentOfForeignKey;
import org.modelgen.importexport.relational.model.ForeignKey;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.Tuple;
import org.modelgen.importexport.relational.model.TupleComponent;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.datatype.DB2RelationalDataType;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

/**Export a RelationalSchema to a DB2 database.
 * An Exporter is a class which has the rule to translate the given schema of type T
 * in a correct form for the Destination E.
 * For example we can export a XML schema both to a File and a DBMS with XML capabilities;
 * we can threfore define two class: one which has the rule to export to a File and one to a DBMS 
 * 
 * @version 0.9
 */
public class DB2RelationalExporter implements Exporter<RelationalSchema,Destination<Connection>> {
	
	/**Check that the model Given can be export to the Destination.
	 * When we define a class that implements this interface we can export not only
	 * the model it was written for, but also other model with less costructs
	 * For example a class that export a RelationalSchema can also export a RelationalSchema
	 * in which model does not exist FK.
	 * 
	 *  Per maggiori info vedere modello massimale e minimale nella tesi
	 * 
	 * @param model OID of the model as stored in ModelGen db
	 * @return true if the model is between maximal e minimal model
	 * @throws ImportExportException
	 */
	public boolean checkModel(int model) throws ImportExportException {
		boolean exportable=true;
		
/*		Riabiliter questo controllo dopo aver spostato le info richieste sul Facade
 * 		try {
			PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler();
			Model source=ph.loadModel(model,null);
			if(source==null ) {
				System.out.println("Attention: model is null");
				throw new ImportExportException("One of the model does not exist");
			}
//			System.out.println("Questo  il modello da cui voglio partire: "+source.getName()+" "+source.getOID());
			//The collection of construct used in the source model
			Collection<Construct> sourceC=source.getConstructs();

			//the source model has more construct that we can deal with the relational model
			//or less than a significant relationa(for example only aggregation)
			if(sourceC.size()>4 || sourceC.size()<2)  {
				exportable=false;
			}
			else {

				Iterator<Construct> i=sourceC.iterator();
				Construct temp,current;
				boolean a=false,l=false,f=false,c=false;

				while(i.hasNext()) {
					temp=i.next();

					//the supermodel constructs
					current=(Construct)temp.getSuperModelConstructs().iterator().next();
					//if it is one of this ok
					if(current.getName().equalsIgnoreCase(System.getProperty("Aggregation"))) {
						a=true;
					}
					else if (current.getName().equalsIgnoreCase(System.getProperty("ForeignKey"))) {
						f=true;
					}
					else if (current.getName().equalsIgnoreCase(System.getProperty("ComponentOfForeignKey"))) {
						c=true;
					}
					//if it is this check property
					else if (current.getName().equalsIgnoreCase(System.getProperty("LexicalComponentOfAggregation"))) {
						l=true;
						//per devo fare check sulla propriet
						Collection<Property> prop=temp.getProperties();
						int size=prop.size();

						if(size>3 || size<2) {
							l=false;
						}
						//3 or 2 property
						else if(size==2){
							Iterator<Property> iP=prop.iterator();
							Property p;
							//exportable.setNullable(false);	
							//se ci sono 2 propriet o sono quelle sotto(cio isKey e type)=> exportabile=true
							//oppure non ci sono e quindi exportable=false(e quindi neanche vedo questo valore)

							while(iP.hasNext()) {
								p=iP.next();
								if(p.getName().equalsIgnoreCase(System.getProperty("isKey")) ||
										p.getName().equalsIgnoreCase(System.getProperty("type"))) {
									continue;
								}
								else {
									l=false;									
								}										
							}
						}
						else { //3 element
							Iterator<Property> iP=prop.iterator();
							Property p;

							while(iP.hasNext()) {
								p=iP.next();
								if(p.getName().equalsIgnoreCase(System.getProperty("isKey")) ||
										p.getName().equalsIgnoreCase(System.getProperty("type")) ||
										p.getName().equalsIgnoreCase(System.getProperty("isNullable"))) {
									continue;
								}
								else
									l=false;
							}
						}				
					}
					//adesso so dai boolean quali costrutti ci sono e sono eventualmente corretti
					if(a && l && f && c) {
						//exportable.setExportable(true);
						exportable=true;
					}
					else if(a && l && !f && !c){
						//exportable.setExportable(true);
						exportable=true;
					}
					else
						//exportable.setExportable(false);
						exportable=false;
				}			
			}	
							
		} catch (ClassNotFoundException e) {
			exportable=false;
			System.out.println(e.toString());
			e.printStackTrace();
		} catch (Exception e) {
			exportable=false;
			System.out.println(e.toString());
			e.printStackTrace();
		}*/
		
		return exportable;
	}
	
	/**Export the given schema to the given destination.
	 * Based on the type of the Destination this class must export the information contained in the Schema.
	 * For example in case of a RelationalSchema to a DBMS Destination for each table this metod must 
	 * create a SQL command CREATE TABLE ..
	 * 
	 * 
	 * @param schema the schema to be exported
	 * @param destination the destination to which export the schema
	 * @throws ImportExportException any error during the export
	 */
	public void exportSchema(RelationalSchema schema,Destination<Connection> destination) throws ImportExportException {
		/*per prima cosa prendo la destination che in questo caso  una connection*/
		Connection con=destination.getDestination();
		
		/*creo lo schema*/
		String create_schema=new String("CREATE SCHEMA "+schema.getName());
		
		/*adesso cotruisco una lista di comandi SQL da eseguire*/
		Vector<String> comandi=new Vector<String>();
		comandi.add(create_schema);
		
		for (Table table : schema.getTables()) {
			StringBuilder sql = new StringBuilder();
			String newline = System.getProperty("line.separator");
			/*creo semplicemente il codice SQL*/
			sql.append("CREATE TABLE ");
			sql.append(schema.getName());
			sql.append(".");
			sql.append(table.getName());
			sql.append(" (");
			sql.append(newline);
			
			Vector<Column> primaryKey = new Vector<Column>();
			
			for (Column column : table.getColumns()) {
				sql.append(column.getName());
				sql.append(" ");
				String type=DB2RelationalDataType.getRelationalType(column.getType());
				sql.append(type);
				if(!column.isNull()) {
					sql.append(" NOT NULL");
				}
				sql.append(", ");

				if (column.isKey()) 
					primaryKey.add(column);
			}
			
			if(primaryKey.size()>=1){
				sql.append("primary key (");
				for (Column column : primaryKey) {
					sql.append(column.getName());
					sql.append(", ");
				}
				sql.replace(sql.length()-2, sql.length(), "))");
			}	
			else 
				sql.replace(sql.length()-2, sql.length(), ")");
			
			comandi.add(sql.toString());
		}
		/*verifico se per qualche tabella ho delle istanze*/
		boolean istanceLoaded=false;
		for(Table table:schema.getTables()) {
			if(table.getTuples().size()>0) {
				istanceLoaded=true;
			}
		}
		/*se NON ho delle istanze non carico adesso le FK, ma dopo aver messo le istanze*/
		if(!istanceLoaded){
			comandi.addAll(this.foreignKey(schema));
		}
		/*eseguo uno per uno tutti i comandi che ho creato
		 * si potrebbe usare un batch=>chiedere a Paolo*/
		try {
			Statement st = con.createStatement();
			for(String s:comandi) {
				System.out.println(s);
				System.out.println();
				st.executeUpdate(s);
			}
			st.close();		
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new ImportExportException("Errore nell'Export dell schema");
		}		
	}
	
	/**Export the instances of the given shcema to the given location.
	 * Based on the type of the Destination this class must export the information contained in the Schema.
	 * For example in case of a RelationalSchema to a DBMS Destination for each Tuple this metod must 
	 * create a SQL command INSERT INTO ... VALUES (...)
	 * 
	 * @param schema the schema to be exported
	 * @param destination the destination to which export the schema
	 * @throws ImportExportException any error during the export
	 */
	public void exportInstance(RelationalSchema schema,Destination<Connection> destination)throws ImportExportException {
		/*prendo sempre la connessione*/
		Connection con=destination.getDestination();
		/*sempre la lista dei comandi*/
		Vector<String> comandi=new Vector<String>();
		
		for (Table table : schema.getTables()) {
			for(Tuple tuple:table.getTuples()) {
				/*se ho qualche tupla genero il comando SQL
				 * INSERT INTO <nome dello Schema>.<tabella> (<colonne>) VALUES(....)*/
				StringBuilder sb = new StringBuilder();
				sb.append("INSERT INTO ");
				sb.append(schema.getName());
				sb.append(".");
				sb.append(table.getName());
				sb.append(" (");
				/*genero i nomi delle colonne perch non necessariamente in ModelGen ho i component
				 * ordinati in base all'OID della colonna*/
				for(TupleComponent tupleComponent:tuple.getTupleComponents()) {
					sb.append(tupleComponent.getColumn().getName());
					sb.append(", ");
				}
				sb.replace(sb.length()-2, sb.length(), ")");
				/*adesso genero tutti valori*/
				sb.append(" VALUES( ");
				for(TupleComponent tupleComponent:tuple.getTupleComponents()) {
					if(tupleComponent.getValue()==null)
						sb.append("null");
					else {
						String value=DB2RelationalDataType.getRelationalValue(tupleComponent.getColumn().getType(), tupleComponent.getValue());						
						sb.append(value);
					}
					sb.append(", ");
				}
				sb.replace(sb.length()-2, sb.length(), ")\n");
				comandi.add(sb.toString());
			}
		}
		/*aggiungo anche le fk perch tanto sono sicuro che non sono state inserite prima
		 * e i valori in ModelGen li supponiamo corretti*/
		comandi.addAll(this.foreignKey(schema));

		try {
			Statement st = con.createStatement();
			for(String s:comandi) {
				System.out.println(s);
				System.out.println();
				st.executeUpdate(s);
			}
			st.close();		
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new ImportExportException("Errore nell'Export delle istanze");
		}		
	}
	
	/**Crea i comandi SQL per creare le FK
	 * @param schema
	 * @throws ImportExportException
	 */
	private Vector<String> foreignKey(RelationalSchema schema) throws ImportExportException {
		Vector<String> comandi=new Vector<String>();
		
		for(ForeignKey fk : schema.getForeignKeys()) {
			StringBuilder sql=new StringBuilder();
			
			if(fk.getType()!=ForeignKey.TableToTable){
				throw new ImportExportException("Impossibile esportare uno schema relazione che ha un Fk non tra tabelle");
			}
			sql.append("ALTER TABLE ");
			sql.append(schema.getName());
			sql.append(".");
			Table from=schema.getTableByOID(fk.getFromOID());
			if(from==null)
				throw new ImportExportException("Errore nell'export delle FK\nNono trovata la tabella From");
			sql.append(from.getName());
			sql.append(" ADD FOREIGN KEY ( ");
			
			for (ComponentOfForeignKey component : fk.getComponents()) {
				Column column=from.getColumnByOID(component.getFromOID());
				if(column==null)
					throw new ImportExportException("Errore nell'export delle FK\nNono trovata una colonna che appartiene alla FK");
				sql.append(column.getName());
				sql.append(", ");
			}
			sql.replace(sql.length()-2, sql.length(), " ) ");
			sql.append("REFERENCES ");
			
			Table to=schema.getTableByOID(fk.getToOID());
			if(to==null)
				throw new ImportExportException("Errore nell'export delle FK\nNono trovata la tabella To");
			sql.append(schema.getName());
			sql.append(".");
			sql.append(to.getName());
			sql.append(" ( ");
			for (ComponentOfForeignKey component : fk.getComponents()) {
				Column column=to.getColumnByOID(component.getToOID());
				if(column==null)
					throw new ImportExportException("Errore nell'export delle FK\nNono trovata una colonna che appartiene alla FK");
				sql.append(column.getName());
				sql.append(", ");
			}
			sql.replace(sql.length()-2, sql.length(), " )");
			
			comandi.add(sql.toString());
		}

		return comandi;
	}
}
