package org.modelgen.importexport.Export;

import org.modelgen.importexport.utilities.ImportExportException;

/**This interface has the role to define all the metod tha any Exporter must have.
 * An Exporter is a class which has the rule to translate the given schema of type T
 * in a correct form for the Destination E.
 * For example we can export a XML schema both to a File and a DBMS with XML capabilities;
 * we can threfore define two class: one which has the rule to export to a File and one to a DBMS 
 * 
 * @version 0.9
 *
 */
public interface Exporter<T,E> {
	
	/**Check that the model Given can be export to the Destination.
	 * When we define a class that implements this interface we can export not only
	 * the model it was written for, but also other model with less costructs
	 * For example a class that export a RelationalSchema can also export a RelationalSchema
	 * in which model does not exist FK.
	 * 
	 *  Per maggiori info vedere modello massimale e minimale nella tesi
	 * 
	 * @param model OID of the model as stored in ModelGen db
	 * @return true if the model is between maximal e minimal model
	 * @throws ImportExportException
	 */
	public boolean checkModel(int model) throws ImportExportException;
	
	
	/**Export he given schema to the given destination.
	 * Based on the type of the Destination this class must export the information contained in the Schema.
	 * For example in case of a RelationalSchema to a DBMS Destination for each table this metod must 
	 * create a SQL command CREATE TABLE ..
	 * 
	 * 
	 * @param schema the schema to be exported
	 * @param destination the destination to which export the schema
	 * @throws ImportExportException any error during the export
	 */
	public void exportSchema(T schema,E destination) throws ImportExportException;
	
	/**Export the instances of the given shcema to the given location.
	 * Based on the type of the Destination this class must export the information contained in the Schema.
	 * For example in case of a RelationalSchema to a DBMS Destination for each Tuple this metod must 
	 * create a SQL command INSERT INTO ... VALUES (...)
	 * 
	 * @param schema the schema to be exported
	 * @param destination the destination to which export the schema
	 * @throws ImportExportException any error during the export
	 */
	public void exportInstance(T schema,E destination)throws ImportExportException;	
	
}
