package org.modelgen.importexport.Import;

import org.modelgen.importexport.oo.model.Field;
import org.modelgen.importexport.oo.model.OOChild;
import org.modelgen.importexport.oo.model.OOClass;
import org.modelgen.importexport.oo.model.OOGeneralization;
import org.modelgen.importexport.oo.model.OOObject;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.oo.model.ObjectGeneralization;
import org.modelgen.importexport.oo.model.Reference;
import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;
import org.modelgen.importexport.persistence.ConnectionFactory;
import org.modelgen.importexport.persistence.DbConnection;
import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.datatype.DB2ObjectOrientedDataType;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.List;

/**This class permists to Import a Object Oriented schema from a DbConnection.
 * This implementation is DB2-specific since theirs driver does NOT support some metod that
 * are necessary to import an OOSchmema; in particular the driver returns a Resultset always 
 * empty from metodo getSuperTables()
 * 
 * So this implementation has to look for necessary information in the catalog.
 * See thesis for more information 
 * 
 *  @version 0.9
 *  
 */
public class DB2OOImporter implements Importer<OOSchema,DbConnection> {
	/*OID a livello di schema*/
	private int schemeOID,classOID,fieldOID,referenceOID,generalizationOID,childGeneralizationOID;
	/*OID a livello di istanze*/
	private int iFieldOID,iRefOID;
	/*ATTENZIONE: normalmente normal=true significa che in caso di tipo di dato non
	 * OO(tipi complessi questo solleva un'eccezione)
	 * Per far si per che l'importer del'Or non sia ripetitivo ho anche la possibilit di non fargli 
	 * sollevare l'eccezione
	 * */
	private boolean normal;
	
	public DB2OOImporter() throws PersistenceException {
		ImportExportPersistence ie=ImportExportPersistence.getInstance();
		//la serie di OID che mi serviranno a livello di schema	
		schemeOID=ie.getNewOID("Schema");
		classOID=ie.getNewOID("Class");
		fieldOID=ie.getNewOID("Field");
		referenceOID=ie.getNewOID("Reference");
		generalizationOID=ie.getNewOID("Generalization");
		childGeneralizationOID=ie.getNewOID("ChildOfGeneralization");
		//ora gli OID a livello di schema
		iFieldOID=ie.getNewOID("FieldComponent");
		iRefOID=ie.getNewOID("RefComponent");
		//leggi sopra
		normal=true;
	}
	
	/**NON usare pericolo di morte :) serve solo per l'import dell'OR
	 * @param normal
	 * @throws PersistenceException
	 */
	DB2OOImporter(boolean normal) throws PersistenceException {
		ImportExportPersistence ie=ImportExportPersistence.getInstance();
		//la serie di OID che mi serviranno a livello di schema	
		schemeOID=ie.getNewOID("Schema");
		classOID=ie.getNewOID("Class");
		fieldOID=ie.getNewOID("Field");
		referenceOID=ie.getNewOID("Reference");
		generalizationOID=ie.getNewOID("Generalization");
		childGeneralizationOID=ie.getNewOID("ChildOfGeneralization");
		//ora gli OID a livello di schema
		iFieldOID=ie.getNewOID("FieldComponent");
		iRefOID=ie.getNewOID("RefComponent");
		this.normal=normal;
	}

	/**Control the corretness of the source.
	 * 
	 * Verificare se questo metodo sia necessario
	 *  
	 * @return true is the source is corret
	 * @see org.modelgen.importexport.Import.Importer#checkSource(org.modelgen.importexport.Import.Source)
	 */
	public boolean checkSource(Source<DbConnection> source) throws PersistenceException,ImportExportException {
		
		/*ATTENZIONE questo bisogna decidere se farlo!!!*/
		
		return true;
	}
	
	
	/**Imports a OOSchema from a DbConnection.
	 * 
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importSchema(org.modelgen.importexport.Import.Source)
	 */
	public OOSchema importSchema(Source<DbConnection> source) throws PersistenceException,ImportExportException {
		try {
			/*per prima cosa tiro fuori dal wrapper la con e il nome dello schema*/
			Connection con=source.getSource().getConnection();
			String schema=source.getSource().getSchema();
			
			//ottengo i metadati dalla connessione
			DatabaseMetaData metaDati = con.getMetaData();
			
			/*cerco tra gli schemi se ne esiste uno con il nome che ci  stato fornito */
			ResultSet rs=metaDati.getSchemas();
			boolean found=false;
			String scheme="";
			
			while(rs.next() && !found) {
				scheme=rs.getString("TABLE_SCHEM");
				if(scheme.equalsIgnoreCase(schema))
					found=true;
			}
			rs.close();
			
			if(!found) {
				throw new PersistenceException("Lo schema indicato non esiste");
			}
						
			//creo uno schema di tipo OOSchema
			OOSchema ooSchema=new OOSchema(schemeOID,scheme);

			/*adesso cerco le typed table*/
			String sql=new String("SELECT TABNAME,TYPE,ROWTYPENAME FROM SYSCAT.TABLES WHERE ROWTYPESCHEMA=? AND ROWTYPENAME " +
					"IN (SELECT TYPENAME FROM SYSCAT.DATATYPES WHERE TYPESCHEMA=?)");
			PreparedStatement ps=con.prepareStatement(sql);
			ps.setString(1,scheme);
			ps.setString(2,scheme);
			rs=ps.executeQuery();
	
			while(rs.next()) {
				OOClass o=new OOClass(classOID,rs.getString(1));
				classOID++;				
				if(!rs.getString(2).equals("U"))
					throw new PersistenceException("La classe "+o.getName()+" non  User Defined");
				o.setType(rs.getString(3));
				ooSchema.addOOClass(o);
			}
			rs.close();
			ps.close();
				
			/*this is for fields, in pratica dal nome delle colonne risalgo al tipo usato*/
//			String sqlField=new String("SELECT I4.COLNAME,I4.TYPENAME,I4.LENGTH,I4.NULLS,I4.SCOPE_TABSCHEMA,I4.SCOPE_TABNAME,I3.COLSEQ " +
//					"FROM SYSCAT.KEYCOLUSE AS I3 RIGHT JOIN (SELECT I1.TABSCHEMA,I1.TABNAME,I1.COLNAME,I1.TYPENAME,I1.LENGTH,I1.NULLS," +
//					"I1.SCOPE_TABSCHEMA,I1.SCOPE_TABNAME FROM SYSCAT.COLUMNS AS I1," +
//					"(SELECT ATTR_NAME FROM SYSCAT.ATTRIBUTES WHERE TYPESCHEMA=? AND TYPENAME=? AND SOURCE_TYPENAME=TYPENAME) AS I2" +
//					" WHERE I1.TABSCHEMA=? AND I1.TABNAME=? AND I1.COLNAME=I2.ATTR_NAME) AS I4 " +
//					"ON (I3.TABSCHEMA=I4.TABSCHEMA AND I3.TABNAME=I4.TABNAME AND I3.COLNAME=I4.COLNAME)");
			
			/*Modifica del 17 maggio 2007, da verificare*/
			String sqlField=new String("SELECT I4.COLNAME,I4.TYPENAME,I4.LENGTH,I4.NULLS,I4.SCOPE_TABSCHEMA,I4.SCOPE_TABNAME,I3.COLSEQ " +
					"FROM (SELECT I6.TABSCHEMA,I6.TABNAME,I6.COLNAME,I6.COLSEQ FROM SYSCAT.KEYCOLUSE AS I6 WHERE I6.CONSTNAME NOT IN (SELECT CONSTNAME FROM SYSCAT.REFERENCES)) " +
					"AS I3 RIGHT JOIN (SELECT I1.TABSCHEMA,I1.TABNAME,I1.COLNAME,I1.TYPENAME,I1.LENGTH,I1.NULLS," +
					"I1.SCOPE_TABSCHEMA,I1.SCOPE_TABNAME FROM SYSCAT.COLUMNS AS I1," +
					"(SELECT ATTR_NAME FROM SYSCAT.ATTRIBUTES WHERE TYPESCHEMA=? AND TYPENAME=? AND SOURCE_TYPENAME=TYPENAME) AS I2" +
					" WHERE I1.TABSCHEMA=? AND I1.TABNAME=? AND I1.COLNAME=I2.ATTR_NAME) AS I4 " +
					"ON (I3.TABSCHEMA=I4.TABSCHEMA AND I3.TABNAME=I4.TABNAME AND I3.COLNAME=I4.COLNAME)");
			ps=con.prepareStatement(sqlField);
			ps.setString(1,scheme);
			ps.setString(3,scheme);
			
			/*now fields*/
			for(OOClass o: ooSchema.getOOClasses()) {
				ps.setString(2,o.getType().toUpperCase());
				ps.setString(4,o.getName().toUpperCase());
				rs=ps.executeQuery();
				
				while(rs.next()) {
					/*se il tipo  una ref*/
					if(rs.getString("TYPENAME").equalsIgnoreCase("REFERENCE")) {
						/*modifica del 30/04/07 AbstractAttribute non ha pi
						 *isOptional e isIdentifier, ma isNullable */
						boolean isNullable=true;
						if(rs.getString(4).equals("N"))
							isNullable=false;
						/*adesso cerco la classe a cui si riferisce questa Ref
						 * Attenzione: se la classe a cui si riferisce non  n questo schema ho un errore perch la la ref deve 
						 * rivolgersi a classi solo di questo schema*/
						OOClass to=null;
						String toSchema=rs.getString(5);
						String toClass=rs.getString(6);
						/*questi valori sono nulli se non  stato specificato la tabella di rif per
						 * la Ref(e quindi non posso sapere a chi collegarla)*/
						if(toSchema!=null && toClass!=null) {
							toSchema=toSchema.trim();
							toClass=toClass.trim();
						}
						else 
							throw new ImportExportException("La Ref "+rs.getString(1)+" non ha specificato la tabella di destinazione");
						/*se sono state specificate devo trovare la OOClass a cui questa Ref si riferisce*/
						for(OOClass oo: ooSchema.getOOClasses()){
							if(toSchema.equalsIgnoreCase(scheme) && toClass.equalsIgnoreCase(oo.getName()))
								to=oo;
						}
						if(to!=null) {
							Reference r=new Reference(referenceOID,rs.getString(1), isNullable,to);
							referenceOID++;
							o.addOOReference(r);
						}
						else 
							throw new ImportExportException("La Ref "+rs.getString(1)+" non ha specificato la tabella di destinazione(cio NON  scoped)\n" +
									"oppure questa  in un altro schema(vedere le limitazioni)");
						
					}
					/*altrimenti ho un campo banale che per per l'OO pu essere solo uno dei tipi standard definiti nei DBMS non pu
					 * essere un tipo complesso(per quello vedere OR)*/
					else {
						boolean isId;
						boolean isNull;
						/*isId*/
						if(rs.getInt(7)>0) {
							isId=true;
							isNull=false;
						}
						else if(rs.getString(4).equals("N")){
							isId=false;
							isNull=false;
						}
						else {
							isId=false;
							isNull=true;
						}
						/*adesso ho il tipo espresso come stringa, devo vedere se  un tipo standard
						 * Inserisco il tutto in un try-catch perch in condizioni normali se il tipo non  supportato(ad esempio un tipo strutturato
						 * che nell'OO non posso avere) lancio l'eccezione che invece non deve essere essere lanciata se questo metodo  
						 * richiamato dall'importer dell'OR*/
						try {
							String type=DB2ObjectOrientedDataType.getModelGenDataType(rs.getString(2), rs.getInt(3));
							Field f=new Field(fieldOID,rs.getString(1),type,isNull,isId);
							fieldOID++;
							o.addField(f);
						} catch (PersistenceException e) {
							/*adesso in una situzione normale rilancio solamente l'eccezione*/
							if(normal)
								throw e;
						}	
					}
				}
			}	
			rs.close();
			ps.close();
			
			/*query for generalization*/
			sql=new String("SELECT SUB_NAME,SUPER_NAME FROM SYSCAT.HIERARCHIES WHERE METATYPE='U' AND SUB_SCHEMA=? AND SUB_SCHEMA=SUPER_SCHEMA");
			ps=con.prepareStatement(sql);
			ps.setString(1,scheme);
			
			//query for isTotal
			String queryIsTotal;
			Statement st=con.createStatement();
			
			rs=ps.executeQuery();
			while(rs.next()) {
				OOClass from,to;
				from=ooSchema.getClassByName(rs.getString(1));
				to=ooSchema.getClassByName(rs.getString(2));
				
				if(from==null || to==null)
					throw new PersistenceException("Errore nel caricare le Generalizzazioni:\nNon trovata una classe");

				//to know if the generalization isTotal
				boolean isTotal;
				/*per sapere se una gen  totale basta fare una query select [] only; se c' almeno una riga
				 * nel resultset vuol dire che NON  totale*/
				if(normal) {
					queryIsTotal=new String("SELECT * FROM ONLY ("+scheme+"."+to.getName()+")");
					ResultSet rsTotal=st.executeQuery(queryIsTotal);
					if(rsTotal.next())
						isTotal=false;
					else
						isTotal=true;				
					rsTotal.close();
				}
				/*in condizioni Non normali cio quando questo metodo  richiamato dall'importer dell'OR si occupa lui di decidere se una
				 * gen  totale o no*/
				else 
					isTotal=false;
					
				/*verifico se la generalizazione gi  stat inserita; se si aggingo 
				 * solamente il Child*/
				if(ooSchema.checkGeneralization(to)) {
					OOGeneralization gen=ooSchema.getGeneralization(to);
					OOChild child=new OOChild(childGeneralizationOID,from);
					gen.addChild(child);
					childGeneralizationOID++;
				}
				/*altrimenti creo la gen, il child e li aggiungo allo schema*/
				else {
					OOGeneralization gen=new OOGeneralization(generalizationOID,"Gen"+to.getName(),isTotal,to);
					generalizationOID++;
					OOChild child=new OOChild(childGeneralizationOID,from);
					gen.addChild(child);
					childGeneralizationOID++;
					ooSchema.addOOGeneralization(gen);
				}				
			}
			st.close();
			rs.close();
			ps.close();
			
			return ooSchema;
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import da DB2");
		}
	}

	/**Import the istances of a OOSchema from a DbConnection source.
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importInstance(java.lang.Object, org.modelgen.importexport.Import.Source)
	 */
	public OOSchema importInstance(OOSchema schema, Source<DbConnection> source)throws PersistenceException,ImportExportException {
		try {
			/*tiro fuori dal wrapper le due cose che mi servono*/
			Connection con=source.getSource().getConnection();
			String schema_name=schema.getName();
			
			Statement st=con.createStatement();
			ResultSet rs;
			
			for(OOClass ooClass:schema.generateHierachy()) {
				StringBuilder sb=new StringBuilder("SELECT * FROM ");
				sb.append(schema_name);
				sb.append(".");
				sb.append(ooClass.getName());
				
				rs=st.executeQuery(sb.toString());
				
				while(rs.next()){
					/*creo un oggetto
					 * ATTENZIONE: io creo oggetti che hanno come OID il valore
					 * dato dal DBMS, solamente in un secondo tempo(e cio quando faccio lo storeOOInstace)
					 * andr ad aggiornare gli OID degli oggetti rispettando l'univocit necessaria in Modelgen*/
					OOObject object=new OOObject(rs.getInt(1),ooClass);
					ooClass.addOOObject(object);
					for(Field field: ooClass.getFields()){
						String valore=DB2ObjectOrientedDataType.getModelGenValue(field.getType(), rs.getString(field.getName()));
						ValueComponent value=new ValueComponent(iFieldOID,valore,field,object);
						iFieldOID++;
						object.addValueComponent(value);
					}
					for(Reference ref: ooClass.getOOReferences()) {
						ReferenceComponent reference;
						/*se la ref  null */
						if(rs.getString(ref.getName())==null) {
							reference=new ReferenceComponent(iRefOID,null,ref);
						}
						/*devo trovare l'oggetto*/
						else {
							int objectTo=rs.getInt(ref.getName());
							/*l'oggetto che ho */
							reference=new ReferenceComponent(iRefOID,ref.getClassTo().getOOObjectByOID(objectTo),ref);
						}							
						iRefOID++;
						object.addReferenceComponent(reference);					
					}
				}
				rs.close();
			}

			/*ora devo modificare gli OID degli oggetti che sono uguali, 
			 * per rispettando la gererchia */
			for(OOGeneralization gen:schema.getGeneralizations()) {
				OOClass parent=gen.getParent();
				List<OOClass> childs=gen.getOOChilds();
				
				/*per ogni oggetto del padre devo vedere in quale figlio ho l'oggetto corrispondente
				 * quando e se lo trovo(ci sono oggetti che non appartengono a nessuno dei figli)*/
				for(OOObject obj:parent.getOOObjects()) {
					int objOID=obj.getObjectOID();
					
					Iterator<OOClass> it=childs.iterator();
					boolean found=false;
					
					while(!found && it.hasNext()) {						
						OOClass child=it.next();						
						/*cerco l'oggetto dentro questa classe*/
						OOObject objTO=child.getOOObjectByOID(objOID);
						/*se l'oggetto sta in questa classe l'obejct  diverso da null*/
						if(objTO!=null) {
							OOChild genChild=gen.getChild(child);
							ObjectGeneralization objGen=new ObjectGeneralization(gen,genChild,obj,objTO);
							schema.addObjectGeneralization(objGen);
							found=true;
						}
					}
				}				
			}		
			
			return schema;
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import: Sql");
		}			
	}
	
	public static void main(String[] args){
		try {
			DbConnection con=ConnectionFactory.getConnection(ConnectionFactory.DB2, "OO","localhost", 
					50000, "db2admin","db2admin","OOTOTALE");
			Source<DbConnection> source=new Source<DbConnection>(con);
			OOSchema schema=new DB2OOImporter().importSchema(source);
			schema.generateHierachy();
			schema=new DB2OOImporter().importInstance(schema, source);
			System.out.println(schema.toString());
//			ImportExportPersistence.getInstance().storeOOSchema(schema);
//			ImportExportPersistence.getInstance().storeOOInstance(schema);
			
		} catch (PersistenceException e) {
			e.printStackTrace();
		} catch (ImportExportException e) {
			e.printStackTrace();
		}
	}
}
