package org.modelgen.importexport.Import;

import org.modelgen.importexport.oo.model.Field;
import org.modelgen.importexport.oo.model.OOChild;
import org.modelgen.importexport.oo.model.OOClass;
import org.modelgen.importexport.oo.model.OOGeneralization;
import org.modelgen.importexport.oo.model.OOObject;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.oo.model.ObjectGeneralization;
import org.modelgen.importexport.oo.model.Reference;
import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;
import org.modelgen.importexport.or.model.ComplexType;
import org.modelgen.importexport.or.model.ComplexTypeComponent;
import org.modelgen.importexport.or.model.Insertable;
import org.modelgen.importexport.or.model.Nest;
import org.modelgen.importexport.or.model.NestComponent;
import org.modelgen.importexport.or.model.ORClass;
import org.modelgen.importexport.or.model.ORObject;
import org.modelgen.importexport.or.model.ORReference;
import org.modelgen.importexport.or.model.ORReferenceComponent;
import org.modelgen.importexport.or.model.ORSchema;
import org.modelgen.importexport.or.model.ORTable;
import org.modelgen.importexport.or.model.ORTuple;
import org.modelgen.importexport.persistence.ConnectionFactory;
import org.modelgen.importexport.persistence.DbConnection;
import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.relational.model.Column;
import org.modelgen.importexport.relational.model.ComponentOfForeignKey;
import org.modelgen.importexport.relational.model.ForeignKey;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.TupleComponent;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.datatype.DB2ObjectOrientedDataType;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**This class permists to Import a Object Relational schema from a DbConnection.
 * This implementation is DB2-specific since theirs driver does NOT support some metod that
 * are necessary to import an ORSchmema; in particular the driver returns a Resultset always 
 * empty from metod getSuperTables()
 * 
 * So this implementation has to look for necessary information in the catalog.
 * See thesis for more information 
 * 
 *  @version 0.9
 *  
 */
public class DB2ORImporter implements Importer<ORSchema,DbConnection> {
	//OID a livello di schema
	private int schemeOID,complexOID,fieldOID,refOID,nestOID,fkOID,componentFkOID;
	//OID a livello di istanze
	private int tupleOID,fieldComponentOID,refComponentOID,complexComponentOID,nestComponetOID;
	/*serve a numerare le colonne del resultset, nel caso dei complex type infatti non  possibile usare il 
	 * nome dellla ref e del field per accedere al valore*/
	private int numero;

	public DB2ORImporter()throws PersistenceException {
		ImportExportPersistence ie=ImportExportPersistence.getInstance();
		//la serie di OID che mi serviranno a livello di schema	
		schemeOID=ie.getNewOID("Schema");
		fieldOID=ie.getNewOID("Field");
		refOID=ie.getNewOID("Reference");
		complexOID=ie.getNewOID("ComplexType");
		nestOID=ie.getNewOID("Nest");
		fkOID=ie.getNewOID("ForeignKey");
		componentFkOID=ie.getNewOID("ComponentOfForeignKey");
		//la serie di OID che mi serviranno a livello di istanze	
		tupleOID=ie.getNewOID("Tuple");
		fieldComponentOID=ie.getNewOID("FieldComponent");
		refComponentOID=ie.getNewOID("RefComponent");
		complexComponentOID=ie.getNewOID("ComplexComponent");
		nestComponetOID=ie.getNewOID("NestComponent");
		numero=0;
	}
	
	/**Control the corretness of the source.
	 * 
	 * Verificare se questo metodo sia necessario
	 *  
	 * @return true is the source is corret
	 * @see org.modelgen.importexport.Import.Importer#checkSource(org.modelgen.importexport.Import.Source)
	 */
	public boolean checkSource(Source<DbConnection> source) throws PersistenceException, ImportExportException {
		return true;
	}
	
	/**Imports an ORSchema from a DbConnection.
	 * 
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importSchema(org.modelgen.importexport.Import.Source)
	 */
	public ORSchema importSchema(Source<DbConnection> source) throws PersistenceException, ImportExportException {
		try {
			/*per prima cosa tiro fuori dal wrapper la con e il nome dello schema*/
			Connection con=source.getSource().getConnection();
			String schema=source.getSource().getSchema();

			//ottengo i metadati dalla connessione
			DatabaseMetaData metaDati = con.getMetaData();

			/*cerco tra gli schemi se ne esiste uno con il nome che ci  stato fornito */
			ResultSet rs=metaDati.getSchemas();
			boolean found=false;
			String scheme="";

			while(rs.next() && !found) {
				scheme=rs.getString("TABLE_SCHEM");
				if(scheme.equalsIgnoreCase(schema))
					found=true;
			}
			rs.close();

			if(!found) {
				throw new PersistenceException("Lo schema indicato non esiste");
			}

			//creo uno schema di tipo OOSchema
			ORSchema orSchema=new ORSchema(schemeOID,scheme);
			
			/*sfrutto l'import del'OO per creare le typed table e relativi field e ref*/
			DB2OOImporter oo=new DB2OOImporter(false);
			OOSchema temp=oo.importSchema(source);
					
			/*per ogni OOClass ottenute secondo la gerarchia..*/
			for(OOClass ooClass:temp.generateHierachy()) {
				/*1)creo una nuova ORClass che  una "is a" OOClass*/
				ORClass orClass=new ORClass(ooClass.getOid(),ooClass.getName());
				/*devo caricare anche il type*/
				orClass.setType(ooClass.getType());
				/*i field sono gli stessi*/
				orClass.setFields(ooClass.getFields());
				/*per le ref devo trovare semplicemente il nuovo oggetto ORClass*/
				for(Reference ref:ooClass.getReferences()) {
					//questo non potr mai essere NULL altrimenti l'import dell'OO sarebbe fallito
					ORClass to=orSchema.getClassByOID(ref.getClassTo().getOid());
					Reference newRef=new Reference(ref.getOid(),ref.getName(),ref.isNullable(),to);
					orClass.addReference(newRef);
				}
				orSchema.addORClass(orClass);
			}
			/*mi mancano ancora le generalizzazioni*/
			for(OOGeneralization gen:temp.getGeneralizations()) {
				//questo non potr mai essere NULL altrimenti l'import dell'OO sarebbe fallito
				ORClass parent=orSchema.getClassByOID(gen.getParent().getOid());
				/*nell'import dellla parte OO il valore della propriet isTotal della Generalizzazione  stato messo
				 * a false per default(in quel momento non potevo fare la query che invece posso fare adesso)*/
				boolean totality=checkTotality(con, parent, orSchema);
				OOGeneralization newGen=new OOGeneralization(gen.getOID(),gen.getName(),totality,parent);
				orSchema.addOOGeneralization(newGen);
				for(OOChild child:gen.getChilds()) {
					//questo non potr mai essere NULL altrimenti l'import dell'OO sarebbe fallito
					ORClass childClass=orSchema.getClassByOID(child.getChild().getOid());
					OOChild newChild=new OOChild(child.getOID(),childClass);
					newGen.addChild(newChild);
				}
			}
			/*visto che i field sono stati caricati dall'importer dell'OO devo vedere fino a che OID
			 *  stato assegnato*/
			for(ORClass orClass:orSchema.getORClasses()) {
				for(Field f:orClass.getFields()) {
					if(f.getOid()>=fieldOID)
						fieldOID=f.getOid()+1;
				}
				for(Reference ref:orClass.getReferences()) {
					if(ref.getOid()>=refOID)
						refOID=ref.getOid()+1;
				}
			}
			
			//facilitiamo il gc
			temp=null;
			
			/*sfrutto l'import del relazionale per creare le tabelle e tutte le colonne normali*/
			JDBCRelational rel=new JDBCRelational(false);
			RelationalSchema relSchema=rel.importSchema(source);
			
			for(Table table:relSchema.getTables()) {
				//modifico gli OID
				for(Column c:table.getColumns()) {
					c.setOID(fieldOID);
					fieldOID++;
				}
				ORTable t=new ORTable(table.getOID(),table.getName());
				t.setColumns(table.getColumns());
				orSchema.addORTable(t);
			}
			//facilitiamo il gc
			relSchema=null;
			
			/*ora posso fare la parte PRETTAMENTE OR, cio Ref tra Table e Typed Table
			 * pi complex type associati sia a Table che a Typed Table e inseriti in vari Nest
			 * e per ultimo le FK */
			
			//1) Ref tra Table e Typed Table 
			//anche in questo caso il driver JDBC di DB2
			//non implementa correttamente l'interfaccia DatabaseMetaData mah :(
			String queryReference="SELECT COLNAME,NULLS,SCOPE_TABSCHEMA,SCOPE_TABNAME " +
			"FROM SYSCAT.COLUMNS WHERE TABSCHEMA=? AND TABNAME=? AND TYPENAME='REFERENCE'";
			PreparedStatement ps=con.prepareStatement(queryReference);
			ps.setString(1,scheme);	

			for(ORTable t:orSchema.getTables()){
				/*setto nella query il nome della tabella*/
				ps.setString(2,t.getName());
				rs=ps.executeQuery();

				while(rs.next()){
					ORClass to=orSchema.getClassByName(rs.getString(4));
					//se non  null
					if(to!=null){
						boolean isNullable;
						//se la colonna  nullable significa che  anche opzionale
						if(rs.getString(2).equalsIgnoreCase("Y"))
							isNullable=true;
						else
							isNullable=false;
						//creo la REF
						ORReference ref=new ORReference(refOID,rs.getString(1),isNullable,t,to);
						refOID++;
						t.addORReference(ref);
					}			
					else 
						throw new ImportExportException("La REF "+rs.getString(1)+" NON ha la typed table di destinazione specificata");
				}
				rs.close();
			}
			ps.close();
				
			//2)aggiungo i Complex type
			String query=new String("SELECT TABNAME,COLNAME,TYPENAME,NULLS FROM SYSCAT.COLUMNS WHERE TABSCHEMA=? AND TYPESCHEMA=TABSCHEMA");
			ps=con.prepareStatement(query);
			ps.setString(1, scheme);
			/*query per i Field e le Ref del complex type*/
			String queryFields=new String("SELECT ATTR_NAME,ATTR_TYPENAME,LENGTH,TARGET_TYPESCHEMA,TARGET_TYPENAME FROM SYSCAT.ATTRIBUTES " +
					"WHERE TYPESCHEMA=? AND TYPENAME=? AND ATTR_TYPESCHEMA='SYSIBM' ORDER BY ORDINAL");
			PreparedStatement pst=con.prepareStatement(queryFields);
			pst.setString(1, scheme);
			
			rs=ps.executeQuery();
			while(rs.next()) {
				String belongTo=rs.getString("TABNAME");
				boolean isNullable=rs.getString(4).charAt(0)=='Y';
				String type=rs.getString("TYPENAME");
				/*belongTo pu essere sia una table che una typed table*/
				ORTable table=orSchema.getORTableByName(belongTo);
				ORClass orClass=orSchema.getClassByName(belongTo);
				/*solo uno dei non  null, caso table*/
				if(table!=null && orClass==null) {
					/*creo il complex type associato a questa tabella*/
					ComplexType complex=new ComplexType(complexOID,rs.getString("COLNAME"),type,isNullable,isNullable);
					complexOID++;
					table.addComplex(complex);
					/*ora aggiungo i Field e le Ref*/
					getFieldsAndReferences(pst,complex,orSchema);
				}
				/*caso typed table, con la condizione aggiuntiva che il complex non sia stato gi definito per i parent di questa classe*/
				else if(table==null && orClass!=null && !orSchema.existComplexType(orClass, rs.getString("COLNAME"))) {
					/*creo il complex type associato a questa typed table*/
					ComplexType complex=new ComplexType(complexOID,rs.getString("COLNAME"),type,isNullable,isNullable);
					complexOID++;
					orClass.addComplex(complex);
					/*ora aggiungo i Field e le Ref*/
					getFieldsAndReferences(pst,complex,orSchema);
				}				
			}
			rs.close();
			ps.close();
			/*NON chiudere pst ancora*/
			
			//3) query per i Nest			
			/*per prima cosa popolo un vettore con tutti i complex type sia quelli delle tabelle che quelli
			 * delle typed table*/
			Vector<ComplexType> complexes=new Vector<ComplexType>();
			for(ORTable table:orSchema.getTables()) {
				if(table.getComplexes().size()>=1)
					complexes.addAll(table.getComplexes());					
			}
			for(ORClass orClass:orSchema.getORClasses()) {
				if(orClass.getComplexes().size()>=1)
					complexes.addAll(orClass.getComplexes());				
			}
			/*preparo la query da eseguire*/
			query=new String("SELECT ATTR_NAME,ATTR_TYPENAME FROM SYSCAT.ATTRIBUTES WHERE TYPESCHEMA=? AND TYPENAME=? " +
					"AND NOT ATTR_TYPESCHEMA='SYSIBM'");
			ps=con.prepareStatement(query);
			ps.setString(1,scheme);
			
			Vector<ComplexType> complexesTemp=new Vector<ComplexType>();
			while(complexes.size()>0 || complexesTemp.size()>0){
				complexes.addAll(complexesTemp);
				complexesTemp=new Vector<ComplexType>();
				Iterator<ComplexType> i=complexes.iterator();
				
				while(i.hasNext()){
					ComplexType complex=i.next();
					ps.setString(2,complex.getType());
					rs=ps.executeQuery();
					
					while(rs.next()) {
						/*per ogni riga restituita devo creare un nested*/
						ComplexType nested=new ComplexType(complexOID,rs.getString("ATTR_NAME"),rs.getString("ATTR_TYPENAME"),true,true);
						complexOID++;
						complexesTemp.add(nested);
						getFieldsAndReferences(pst, nested, orSchema);
						Nest nest=new Nest(nestOID,complex.getName()+"-"+rs.getString("ATTR_NAME"),complex,nested);
						nestOID++;
						orSchema.addNest(nest);
					}				
					i.remove();
				}
				rs.close();
			}			
			ps.close();
			/*qui finalmente posso chiudere anche pst*/
			pst.close();
			
			//4)mi mancano solamente le FK
			String queryFk=new String("SELECT TABNAME,REFTABNAME,COLCOUNT,FK_COLNAMES,PK_COLNAMES " +
			"FROM SYSCAT.REFERENCES WHERE TABSCHEMA=?");
			ps=con.prepareStatement(queryFk);
			ps.setString(1,schema.toUpperCase());

			rs=ps.executeQuery();

			while(rs.next()){
				/*questi sono i nomi delle tabelle(sia normali che typed table)*/
				String from=rs.getString(1);
				String to=rs.getString(2);
				/*devo scoprire la FK tra che tabelle (sia normali che typed table)*/
				ORTable tableFrom=null,tableTo=null;
				OOClass classFrom=null,classTo=null;
				for(ORTable table:orSchema.getTables()) {
					if(table.getName().equalsIgnoreCase(from))
						tableFrom=table;
					if(table.getName().equalsIgnoreCase(to))
						tableTo=table;
				}
				for(ORClass o:orSchema.getORClasses()) {
					if(o.getName().equalsIgnoreCase(from))
						classFrom=o;
					if(o.getName().equalsIgnoreCase(to))
						classTo=o;
				}
				/*ora posso creare la FK*/
				ForeignKey fk;
				/*sotto ho le quattro possibilit di collegamento di una FK*/
				if(tableFrom!=null && tableTo!=null){
					fk=new ForeignKey(fkOID,tableFrom.getName()+"-"+tableTo.getName(),tableFrom,tableTo);
				}
				else if(tableFrom!=null && classTo!=null){
					fk=new ForeignKey(fkOID,tableFrom.getName()+"-"+classTo.getName(),tableFrom,classTo);
				}
				else if(classFrom!=null && classTo!=null){
					fk=new ForeignKey(fkOID,classFrom.getName()+"-"+classTo.getName(),classFrom,classTo);
				}
				else if(classFrom!=null && tableTo!=null){
					fk=new ForeignKey(fkOID,classFrom.getName()+"-"+tableTo.getName(),classFrom,tableTo);
				}
				else
					throw new ImportExportException("Errore nella creazione delle chiavi esterne\nVerificare che le limitazioni siano rispettate");
				/*dopo averla creata la aggiungo*/
				orSchema.addForeignKey(fk);
				fkOID++;
				
				/*adesso devo creare i componenti di questa chiave esterna*/
				StringTokenizer sT=new StringTokenizer(rs.getString(4));
				StringTokenizer sT1=new StringTokenizer(rs.getString(5));
				if(sT.countTokens()==sT1.countTokens()) {
					while(sT.hasMoreTokens() && sT1.hasMoreTokens()) {
						/*prendo i prossimi due nomi e ne creo i componenti*/
						getComponentofFK(fk,sT.nextToken(),sT1.nextToken(),tableFrom,tableTo,classFrom,classTo);
					}
				}
				else {
					throw new ImportExportException("Errore nella creazione dei componenti della FK "+fk.getName());
				}
				
			}
			rs.close();
			ps.close();
			
			
			return orSchema;
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import");
		}
	}
	
	/**Metodo che verifica se una generalizzazione  totale
	 * @throws PersistenceException
	 */
	private boolean checkTotality(Connection con,ORClass parent,ORSchema orSchema) throws PersistenceException {
		try {
			boolean result;
			/*prima devo trovare 'lidentificativo dellla typed table, cos posso farci la query sopra*/
			String query=new String("SELECT COLNAME FROM SYSCAT.COLUMNS WHERE TABSCHEMA=? AND COLNO=? AND TABNAME=?");
			PreparedStatement ps = con.prepareStatement(query);
			ps.setString(1, orSchema.getName());
			ps.setInt(2, 0);
			ps.setString(3,parent.getName());
			
			ResultSet rs=ps.executeQuery();
			String columnOID="";
			if(rs.next()) {
				columnOID=rs.getString(1);
			}
			rs.close();
			ps.close();
			/*adesso posso usare tale valore per verificare se la generalizzazione  totale oppure no*/
			Statement st=con.createStatement();
			query=new String("SELECT "+columnOID+" FROM ONLY ("+orSchema.getName()+"."+parent.getName()+")");
			rs=st.executeQuery(query);
			if(rs.next())
				result=false;
			else
				result=true;
			rs.close();
			st.close();
			
			return result;
			
		} catch (SQLException e) {
			throw new PersistenceException("Errore nella determinazione della totalit di una Generalizzazione");
		}	
	}
	
	/**Ricava tutti i Field e REF per un qualsiasi tipo complesso
	 */
	private void getFieldsAndReferences(PreparedStatement pst,ComplexType complex,ORSchema orSchema) throws PersistenceException, ImportExportException {
		try {
			pst.setString(2, complex.getType());
			ResultSet rs=pst.executeQuery();
			
			while(rs.next()) {
				/*se il campo  una REF vado a cercare come al solito la classe cui si riferisce*/
				if(rs.getString("ATTR_TYPENAME").equalsIgnoreCase("REFERENCE")) {
					OOClass to=orSchema.getORClassByType(rs.getString("TARGET_TYPENAME"));
					if(to!=null) {
						/*specifico valori di default per isNullable*/
						Reference ref=new Reference(refOID,rs.getString("ATTR_NAME"),true,to);
						refOID++;
						complex.addReference(ref);
					}		
					else 
						throw new ImportExportException("Non  stata trovata la typed table cui " +
								"si riferisce la REF "+rs.getString("ATTR_NAME")+" del tipo complesso "+complex.getName());
				}
				else {
					/*verifico che il tipo del campo sia uno di quelli permessi(cio i soliti standard)*/
					String tipo=DB2ObjectOrientedDataType.getModelGenDataType(rs.getString("ATTR_TYPENAME"), rs.getInt("LENGTH"));
					/*specifico valori di default per lo stesso motivo di sopra*/
					Field field=new Field(fieldOID,rs.getString("ATTR_NAME"),tipo,true,false);
					fieldOID++;
					complex.addField(field);
				}
			}
			rs.close();
			//NON chiudere il pst!!!	
			
		} catch (SQLException e) {
			throw new PersistenceException("Errore nell'import dei Field and Reference del tipo complesso: "+complex.getName());
		}		
	}
		
	/**Metodo di utilit che crea i vari componenti della FK in base al tipo; cio se la FK  tra due tabella
	 * normali cerca le due colonne che appartengono alla FK. Se la FK  tra una table e una typed table
	 * cerca una colonna e un field
	 * @throws PersistenceException 
	 */
	private void getComponentofFK(ForeignKey fk,String from,String to,ORTable tableFrom,ORTable tableTo,OOClass classFrom,OOClass classTo) throws PersistenceException {
		int type=fk.getType();
		if(type==ForeignKey.TableToTable){
			Column Cfrom=null;
			Column Cto=null;

			for(Column c:tableFrom.getColumns()) {
				if(c.getName().equalsIgnoreCase(from))
					Cfrom=c;
			}
			for(Column c:tableTo.getColumns()) {
				if(c.getName().equalsIgnoreCase(to))
					Cto=c;
			}
			if(Cfrom!=null && Cto!=null && Cto.isKey()) {
				ComponentOfForeignKey cfk=new ComponentOfForeignKey(componentFkOID,Cfrom.getName()+"-"+Cto.getName(),Cfrom,Cto);
				componentFkOID++;
				fk.add(cfk);
			}
			else 
				throw new PersistenceException("Errore nel caricamento dei componenti di una chiave esterna\n" +
						"Controllare che le colonne referenziate siano primary key e che i componenti della Fk siano nella stessa typed table");
		}
		else if (type==ForeignKey.TableToAbstract){
			Column Cfrom=null;
			Field Fto=null;

			for(Column c:tableFrom.getColumns()) {
				if(c.getName().equalsIgnoreCase(from))
					Cfrom=c;
			}
			for(Field field:classTo.getFields()) {
				if(field.getName().equalsIgnoreCase(to))
					Fto=field;
			}
			if(Cfrom!=null && Fto!=null && Fto.isIdentifier()) {
				ComponentOfForeignKey cfk=new ComponentOfForeignKey(componentFkOID,Cfrom.getName()+"-"+Fto.getName(),Cfrom,Fto);
				componentFkOID++;
				fk.add(cfk);
			}
			else 
				throw new PersistenceException("Errore nel caricamento dei componenti di una chiave esterna\n" +
				"Controllare che le colonne referenziate siano primary key e che i componenti della Fk siano nella stessa typed table");
		}
		else if (type==ForeignKey.AbstractToTable){
			Field Ffrom=null;
			Column Cto=null;

			for(Field field:classFrom.getFields()) {
				if(field.getName().equalsIgnoreCase(from))
					Ffrom=field;
			}
			for(Column c:tableTo.getColumns()) {
				if(c.getName().equalsIgnoreCase(to))
					Cto=c;
			}
			if(Ffrom!=null && Cto!=null && Cto.isKey()) {
				ComponentOfForeignKey cfk=new ComponentOfForeignKey(componentFkOID,Ffrom.getName()+"-"+Cto.getName(),Ffrom,Cto);
				componentFkOID++;
				fk.add(cfk);
			}
			else 
				throw new PersistenceException("Errore nel caricamento dei componenti di una chiave esterna\n" +
				"Controllare che le colonne referenziate siano primary key e che i componenti della Fk siano nella stessa typed table");							
		}
		else if (type==ForeignKey.AbstractToAbstract){
			Field Ffrom=null;
			Field Fto=null;

			for(Field field:classFrom.getFields()) {
				if(field.getName().equalsIgnoreCase(from))
					Ffrom=field;
			}
			for(Field field:classTo.getFields()) {
				if(field.getName().equalsIgnoreCase(to))
					Fto=field;
			}
			if(Ffrom!=null && Fto!=null && Fto.isIdentifier()) {
				ComponentOfForeignKey cfk=new ComponentOfForeignKey(componentFkOID,Ffrom.getName()+"-"+Fto.getName(),Ffrom,Fto);
				componentFkOID++;
				fk.add(cfk);
			}
			else 
				throw new PersistenceException("Errore nel caricamento dei componenti di una chiave esterna\n" +
				"Controllare che le colonne referenziate siano primary key e che i componenti della Fk siano nella stessa typed table");			
		}							
	}

	
	/**Imports the instances of a ORSchema which means it loads all the tuple of 
	 * normal table and typed tables.
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importInstance(java.lang.Object, org.modelgen.importexport.Import.Source)
	 */
	public ORSchema importInstance(ORSchema schema, Source<DbConnection> source) throws PersistenceException, ImportExportException {
		/*Come al solito tiro fuori le due info dalla sorgente*/
		String schema_name=schema.getName();
		Connection con=source.getSource().getConnection();
		
		try {		
			List<Insertable> ordered=schema.generateHierachy();
			for(Insertable ins:ordered) {
				if (ins instanceof ORTable) {
					ORTable table=(ORTable)ins;
					loadORTuples(table,schema,con,schema_name);
				}
				else if (ins instanceof ORClass) {
					ORClass orClass=(ORClass)ins;
					loadORClass(orClass, schema, con, schema_name);			
				}
				else if (ins instanceof ComplexType) {		
					/*per questi NON devo fare niente perch vengono caricati insieme alle tabelle o alle
					 * classi di appartenenza*/
				}
			}	
			/*ora devo modificare gli OID degli oggetti che sono uguali, 
			 * per rispettando la gererchia */
			for(OOGeneralization gen:schema.getGeneralizations()) {
				OOClass parent=gen.getParent();
				List<OOClass> childs=gen.getOOChilds();
				
				/*per ogni oggetto del padre devo vedere in quale figlio ho l'oggetto corrispondente
				 * quando e se lo trovo(ci sono oggetti che non appartengono a nessuno dei figli)*/
				for(OOObject obj:parent.getOOObjects()) {
					int objOID=obj.getObjectOID();
					
					Iterator<OOClass> it=childs.iterator();
					boolean found=false;
					
					while(!found && it.hasNext()) {						
						OOClass child=it.next();
						
						/*cerco l'oggetto dentro questa classe*/
						OOObject objTO=child.getOOObjectByOID(objOID);
						/*se l'oggetto sta in questa classe l'obejct  diverso da null*/
						if(objTO!=null) {
							OOChild genChild=gen.getChild(child);
							ObjectGeneralization objGen=new ObjectGeneralization(gen,genChild,obj,objTO);
							schema.addObjectGeneralization(objGen);
							found=true;
						}
					}
				}
			}			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore SQL nell'import delle istanze");
		}	
		
		return schema;
	}
	
	/**Metodo che va a fare la query su ogni tabella normale e ne analizza i risultati
	 * @throws SQLException
	 * @throws PersistenceException 
	 */
	private void loadORTuples(ORTable t,ORSchema schema,Connection con,String schema_name) throws SQLException, PersistenceException {
		//faccio la query nell'ordine che voglio io
		StringBuilder sb=new StringBuilder("SELECT ");
		//1)le colonne
		for(Column c : t.getColumns()) {
			sb.append(c.getName());
			sb.append(",");
		}
		//2)le REF tra questa tabella e altre
		for(ORReference ref:t.getReferences()) {
			sb.append(ref.getName());
			sb.append(",");
		}
		//3)TUTTI i ComplexType dipendenti da questo, sia al primo livello che non
		for(ComplexType complex:t.getComplexes()) {
			for(Field field : complex.getFields()) {
				sb.append(complex.getName());
				sb.append("..");
				sb.append(field.getName());
				sb.append(",");
			}
			for(Reference ref:complex.getReferences()) {
				sb.append(complex.getName());
				sb.append("..");
				sb.append(ref.getName());
				sb.append(",");
			}
			/*dovrei fare per tutti i campi e ref del nested(e via via fino a che nested==null)*/
			for(ComplexType nested:schema.getNested(complex)) {
				sb.append(generateNestedSQL(nested, complex.getName()+"..", schema));
			}
		}
		//cancello l'ultima virgola
		sb.delete(sb.length()-1,sb.length());
		sb.append(" FROM ");
		sb.append(schema_name);
		sb.append(".");
		sb.append(t.getName());
		
		//questa  la query e la eseguo
		String queryTuple=sb.toString();
		//System.out.println(queryTuple);
		Statement sTC=con.createStatement();
		ResultSet rs=sTC.executeQuery(queryTuple);

		while(rs.next()) {
			/*devo numerare i vari componenti che creo perch il classico rs.getXXX("nome colonna")
			 * NON funziona con i complex 
			 * */
			numero=1;
			/*creo la tupla*/
			ORTuple tuple=new ORTuple(tupleOID,t);
			tupleOID++;
			t.addORTuple(tuple);
			/*ci aggiungo i vari componenti*/
			for(Column c: t.getColumns()) {
				String value=DB2ObjectOrientedDataType.getModelGenValue(c.getType(), rs.getString(c.getName()));
				TupleComponent tc=new TupleComponent(fieldComponentOID,value,c);
				tuple.addTupleComponent(tc);
				fieldComponentOID++;				
				numero++;
			}			
			for(ORReference ref:t.getReferences()) {
				//creare nuovo componente di tupla per le ref
				ORReferenceComponent componet;
				if(rs.getString(numero)==null) {
					/*significa che il riferimento  null*/
					componet=new ORReferenceComponent(refComponentOID,ref,tuple,null);
				}
				else {
					OOObject dest=ref.getTo().getOOObjectByOID(rs.getInt(ref.getName()));
					componet=new ORReferenceComponent(refComponentOID,ref,tuple,dest);
				}
				refComponentOID++;
				tuple.addORReferenceComponent(componet);
				numero++;
			}
			for(ComplexType complex:t.getComplexes()) {
				/*devo trovare tutti i nested di questo in ordine, poi faccio il solito select tutti
				 * field,ref di tutti*/
				ComplexTypeComponent complexComponent=new ComplexTypeComponent(complexComponentOID,complex);
				complexComponentOID++;
				tuple.addComplexTypeComponent(complexComponent);
				
				for(Field field : complex.getFields()) {
					String valore=DB2ObjectOrientedDataType.getModelGenValue(field.getType(), rs.getString(numero));
					ValueComponent value=new ValueComponent(fieldComponentOID,valore,field,null);
					fieldComponentOID++;
					numero++;
					complexComponent.addValueComponent(value);
				}
				for(Reference ref:complex.getReferences()) {
					ReferenceComponent reference;
					/*valore del riferimento non NULL*/
					if(rs.getString(numero)!=null) {
						OOObject to=ref.getClassTo().getOOObjectByOID(rs.getInt(numero));
						reference=new ReferenceComponent(refComponentOID,to,ref);
					}	
					else {
						reference=new ReferenceComponent(refComponentOID,null,ref);
					}
					refComponentOID++;
					complexComponent.addReferenceComponent(reference);
					numero++;
				}
				/*se adesso questo complex ha dei nested devo creare per tutta la catena vari Complexcomponent e creare 
				 * anche i componenti per I_SM_NEST*/
				ComplexType top=complex;
				ComplexTypeComponent topComponent=complexComponent;
				for(ComplexType nested:schema.getNested(complex)) {
					createNestedomponent(nested, schema, rs, topComponent, schema.getNest(top, nested));
				}
			}
		}
	}
	
	/**Genera dato un Complex la lista dei Field e Ref di tutti i nested che derivano da questo.
	 * @param complex
	 * @param base
	 * @param schema
	 */
	private String generateNestedSQL(ComplexType complex,String base,ORSchema schema) {
		StringBuilder sb=new StringBuilder();
		for(Field field : complex.getFields()) {
			sb.append(base);
			sb.append(complex.getName());
			sb.append("..");
			sb.append(field.getName());
			sb.append(",");
		}
		for(Reference ref:complex.getReferences()) {
			sb.append(base);
			sb.append(complex.getName());
			sb.append("..");
			sb.append(ref.getName());
			sb.append(",");
		}
		
		for(ComplexType nested:schema.getNested(complex)) {
			sb.append(generateNestedSQL(nested,base+complex.getName()+"..",schema));
		}
		
		return sb.toString();
	}
	
	/**Serve a creare ricorsivamente i ComplexTypeComponent e i relativi NestedComponent
	 * @throws SQLException
	 * @throws PersistenceException 
	 */
	private void createNestedomponent(ComplexType complex,ORSchema schema,ResultSet rs,ComplexTypeComponent topComponent,Nest nest) throws SQLException, PersistenceException {
		ComplexTypeComponent nestedComponet=new ComplexTypeComponent(complexComponentOID,complex);
		complexComponentOID++;
		complex.add(nestedComponet);
		
		for(Field field : complex.getFields()) {
			String valore=DB2ObjectOrientedDataType.getModelGenValue(field.getType(), rs.getString(numero));
			ValueComponent valueComponent=new ValueComponent(fieldComponentOID,valore,field,null);
			fieldComponentOID++;
			numero++;
			nestedComponet.addValueComponent(valueComponent);
		}
		for(Reference ref:complex.getReferences()) {
			ReferenceComponent reference;
			//valore del riferimento non NULL
			if(rs.getString(numero)!=null) {
				OOObject to=ref.getClassTo().getOOObjectByOID(rs.getInt(numero));
				reference=new ReferenceComponent(refComponentOID,to,ref);
			}	
			else {
				reference=new ReferenceComponent(refComponentOID,null,ref);
			}
			refComponentOID++;
			nestedComponet.addReferenceComponent(reference);
			numero++;				
		}
		
		NestComponent nestComponent=new NestComponent(nestComponetOID,topComponent,nestedComponet);
		nestComponetOID++;
		nest.add(nestComponent);
		
		for(ComplexType nested:schema.getNested(complex)) {
			createNestedomponent(nested,schema,rs,nestedComponet,schema.getNest(complex, nested));
		}
	}
	
	/**Metodo che fa la query per ogni classe e ne analizza i risultati
	 * 
	 * @throws SQLException
	 * @throws PersistenceException 
	 */
	private void loadORClass(ORClass orClass,ORSchema schema,Connection con,String schema_name) throws SQLException, PersistenceException {
		/*visto che non posso fare select * perch avrei errore sql perch non esiste la traformazione
		 * dal tipo ai suoi dati devo fare select <tutti i campi,tutte ref>; mi manca per il nome
		 * della colonna che identifica gli oggetti della typed table; quindi devo ricavare il nome di 
		 * questa colonna che in DB2  sempre la prima*/
		String query=new String("SELECT COLNAME FROM SYSCAT.COLUMNS WHERE TABSCHEMA=? AND COLNO=? AND TABNAME=?");
		
		PreparedStatement ps=con.prepareStatement(query);
		ps.setString(1, schema_name);
		ps.setInt(2, 0);
		ps.setString(3,orClass.getName());
		
		ResultSet rs=ps.executeQuery();
		String columnOID;
		if(rs.next()) {
			columnOID=rs.getString(1);
		}
		else {
			throw new SQLException("Non identificata la colonna di OID della typed table "+orClass.getName());
		}
		rs.close();
		ps.close();
		
		/*adesso ho il nome posso fare la query 
		 * select OID,nome,[tutti i Field],[tutte le REF],indirizzo..via,indirizzo..numero,bonus from temp.tabella3*/
		StringBuilder sb=new StringBuilder("SELECT ");
		sb.append(columnOID);
		sb.append(",");
		//1)tutti i Field
		for(Field field:orClass.getFields()) {
			sb.append(field.getName());
			sb.append(",");
		}
		//2)tutte le REF
		for(Reference ref:orClass.getReferences()) {
			sb.append(ref.getName());
			sb.append(",");
		}
		//3)tutti i complex
		for(ComplexType complex:orClass.getComplexes()) {
			for(Field field : complex.getFields()) {
				sb.append(complex.getName());
				sb.append("..");
				sb.append(field.getName());
				sb.append(",");
			}
			for(Reference ref:complex.getReferences()) {
				sb.append(complex.getName());
				sb.append("..");
				sb.append(ref.getName());
				sb.append(",");
			}
			/*dovrei fare per tutti i campi e ref del nested(e via via fino a che nested==null)*/
			for(ComplexType nested:schema.getNested(complex)) {
				sb.append(generateNestedSQL(nested, complex.getName()+"..", schema));
			}
		}
		sb.delete(sb.length()-1,sb.length());
		sb.append(" FROM ");
		sb.append(schema_name);
		sb.append(".");
		sb.append(orClass.getName());
		
		/*finito di creare la query ora la vado ad eseguire*/
		Statement st=con.createStatement();
		//System.out.println(sb.toString());
		rs=st.executeQuery(sb.toString());
		
		while(rs.next()) {
			/*questo lo inserisco perch non posso usre il classico rs.getXXX("nome colonna") sui complex type quindi
			 * devo numerare le colonne
			 * ATTENZIONE il valore 2  corretto perch oltre ai campi normali devo numerare anche la colonna
			 * che fa da identitifcativo*/
			numero=2;
			/*creo l'oggetto*/
			ORObject object=new ORObject(rs.getInt(1),orClass);
			orClass.addORObject(object);
			//1)i Field
			for(Field field: orClass.getFields()){
				String valore=DB2ObjectOrientedDataType.getModelGenValue(field.getType(), rs.getString(field.getName()));
				ValueComponent value=new ValueComponent(fieldComponentOID,valore,field,object);
				fieldComponentOID++;
				numero++;
				object.addValueComponent(value);
			}
			//2)le REF
			for(Reference ref: orClass.getOOReferences()) {
				ReferenceComponent reference;
				if(rs.getString(ref.getName())!=null) {
					OOObject to=ref.getClassTo().getOOObjectByOID(rs.getInt(ref.getName()));
					reference=new ReferenceComponent(refComponentOID,to,ref);
				}				
				else {
					reference=new ReferenceComponent(refComponentOID,null,ref);
				}
				refComponentOID++;
				object.addReferenceComponent(reference);
				numero++;
			}
			//3)i complex type
			for(ComplexType complex:orClass.getComplexes()) {
				/*devo trovare tutti i nested di questo in ordine, poi faccio il solito select tutti
				 * field,ref di tutti*/
				ComplexTypeComponent complexComponent=new ComplexTypeComponent(complexComponentOID,complex);
				complexComponentOID++;
				object.addComplexTypeComponent(complexComponent);
				
				for(Field field : complex.getFields()) {
					String valore=DB2ObjectOrientedDataType.getModelGenValue(field.getType(), rs.getString(numero));
					ValueComponent value=new ValueComponent(fieldComponentOID,valore,field,null);
					fieldComponentOID++;
					numero++;
					complexComponent.addValueComponent(value);
				}
				for(Reference ref:complex.getReferences()) {
					ReferenceComponent reference;
					if(rs.getString(numero)!=null) {
						OOObject to=ref.getClassTo().getOOObjectByOID(rs.getInt(numero));
						reference=new ReferenceComponent(refComponentOID,to,ref);
					}				
					else {
						reference=new ReferenceComponent(refComponentOID,null,ref);
					}
					refComponentOID++;
					complexComponent.addReferenceComponent(reference);
					numero++;
				}
				/*se adesso questo complex ha dei nested devo creare per tutta la catena vari Complexcomponent e creare 
				 * anche i componenti per I_SM_NEST*/
				ComplexType top=complex;
				ComplexTypeComponent topComponent=complexComponent;
				for(ComplexType nested:schema.getNested(complex)) {
					createNestedomponent(nested, schema, rs, topComponent, schema.getNest(top, nested));
				}
			}			
		}
		rs.close();
		st.close();		
	}
	
	public static void main(String[] args){
		try {
			DbConnection con=ConnectionFactory.getConnection(ConnectionFactory.DB2, "Temp1","localhost", 
					50000, "db2admin","db2admin","fk");
			Source<DbConnection> source=new Source<DbConnection>(con);
			ORSchema or=new DB2ORImporter().importSchema(source);
			or=new DB2ORImporter().importInstance(or, source);
			System.out.println(or.toString());
			ImportExportPersistence.getInstance().storeSchema(or);
			ImportExportPersistence.getInstance().storeInstance(or);
			
		} catch (PersistenceException e) {
			e.printStackTrace();
		} catch (ImportExportException e) {
			e.printStackTrace();
		}
	}
}
