package org.modelgen.importexport.Import;

import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;

/**This interface has the role to define all the metod tha any Importer must have.
 * An Importer is a class which has the rule to create a Schema of type T from a source of type E.
 * For example we can import a XML schema both from a File and a DBMS with XML capabilities;
 * we can threfore define two class: one which has the rule to import from a File and one from a DBMS
 * 
 * @version 0.9
 *
 */
public interface Importer<T,E> {
	
	
	
	/**This metod check the source to see if it is acceptable
	 * 
	 * Attenzione: ancora da decidere se serve
	 * 
	 * @param source the source to be checked
	 * @return true if the source if OK
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted)
	 */
	public boolean checkSource(Source<E> source) throws PersistenceException,ImportExportException;
	
	
	
	/**Imports a schema from a source.
	 * This metod as the role to create a schema of the correct type and the object related which are in the schema defined 
	 * in the source.
	 * For example it must create a RelationalSchema, a Table for each Table defined in the source, and so on
	 * 
	 * 
	 * @param source the source form which get information about the schema
	 * @return the schema 
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted)
	 */
	public T importSchema(Source<E> source) throws PersistenceException,ImportExportException;
	
	
	
	/**Imports the instances of a schema.
	 * This metod has the role to import the instances of the given schema from a source.
	 * The source has to be consistent with the source given to load the schema.
	 * For example we can import a XML schema from a file and its instances from another file (or many other files)
	 * 
	 * 
	 * @param schema the schema from which we must load the instances
	 * @param source the source form which get information about the instances
	 * @return the schema with instances imported
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted)
	 */
	public T importInstance(T schema,Source<E> source) throws PersistenceException,ImportExportException;

}
