package org.modelgen.importexport.Import;

import org.modelgen.importexport.persistence.ConnectionFactory;
import org.modelgen.importexport.persistence.DbConnection;
import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.relational.model.Column;
import org.modelgen.importexport.relational.model.ComponentOfForeignKey;
import org.modelgen.importexport.relational.model.ForeignKey;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.Tuple;
import org.modelgen.importexport.relational.model.TupleComponent;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.datatype.DB2RelationalDataType;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;


/**This class permists to Import a Relational schema from a DbConnection.
 * Thanks to the interface DatabaseMetaData it can costructs a Relationl schema
 * from nearly any DBMS; it must have only a JDBC driver with this interface implemented
 *  
 *  @version 0.9
 *  
 */

public class JDBCRelational implements Importer<RelationalSchema,DbConnection> {
	/*gli OID richiesti a livello di schema*/
	private int OID,tableOID,columnOID,fkOID,ComponentFkOID;
	/*gli OID richiesti a livello di istanze*/
	private int tupleOID,tupleCOID;
	/*ATTENZIONE: normalmente normal=true significa che in caso di tipo di dato non
	 * relazionale(ref o tipi complessi questo solleva un'eccezione)
	 * Per far si per che l'importer del'Or non sia ripetitivo ho anche la possibilit di non fargli 
	 * sollevare l'eccezione
	 * */
	private boolean normal;
	
	/**Creates a new JDBCRelational.
	 * In particular it gets all the OID which will be necessary for the import
	 * 
	 * @throws PersistenceException Any error during the search of a new OID
	 */
	public JDBCRelational() throws PersistenceException {
		ImportExportPersistence ie=ImportExportPersistence.getInstance();
		//la serie di OID che mi serviranno a livello di schema	
		this.OID=ie.getNewOID("Schema");
		this.tableOID=ie.getNewOID("Table");
		this.columnOID=ie.getNewOID("Column");
		this.fkOID=ie.getNewOID("ForeignKey");
		this.ComponentFkOID=ie.getNewOID("ComponentOfForeignKey");
		//gli OID che mi seviranno a livello di istanze
		this.tupleOID=ie.getNewOID("Tuple");
		this.tupleCOID=ie.getNewOID("TupleComponent");
		//leggi sopra
		this.normal=true;
	}
	
	/**Da NON usare; serve solo per semplificare l'import dell'OR
	 * @param normal
	 * @throws PersistenceException
	 */
	JDBCRelational(boolean normal) throws PersistenceException {
		ImportExportPersistence ie=ImportExportPersistence.getInstance();
		//la serie di OID che mi serviranno a livello di schema	
		this.OID=ie.getNewOID("Schema");
		this.tableOID=ie.getNewOID("Table");
		this.columnOID=ie.getNewOID("Column");
		this.fkOID=ie.getNewOID("ForeignKey");
		this.ComponentFkOID=ie.getNewOID("ComponentOfForeignKey");
		//gli OID che mi seviranno a livello di istanze
		this.tupleOID=ie.getNewOID("Tuple");
		this.tupleCOID=ie.getNewOID("TupleComponent");
		//leggi sopra
		this.normal=false;
	}

	/**Control the corretness of the source.
	 * 
	 * Verificare se questo metodo sia necessario
	 *  
	 * @return true is the source is corret
	 * @see org.modelgen.importexport.Import.Importer#checkSource(org.modelgen.importexport.Import.Source)
	 */
	public boolean checkSource(Source<DbConnection> source) throws PersistenceException, ImportExportException {
		return true;
	}
	
	
	
	/**Imports a SchemaRelationl from a source
	 * 
	 * @return a SchemaRelational 
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importSchema(org.modelgen.importexport.Import.Source)
	 */
	public RelationalSchema importSchema(Source<DbConnection> source) throws PersistenceException, ImportExportException {
		/*una source per l'import di uno shcema relazionale  una DbConnection che  un wrapper
		 * per 1) una String che rappresenta il nome dello schema che si deve importare
		 * 2)una Connection verso quel database
		 * quindi per prima cosa estraggo queste due informazioni 
		 * */
		String schema=source.getSource().getSchema().toUpperCase();
		Connection con=source.getSource().getConnection();

		try {
			//ottengo i metadati dalla connessione
			DatabaseMetaData metaDati = con.getMetaData();

			/*cerco tra gli schemi se ne esiste uno con il nome che ci  stato fornito */
			ResultSet rs=metaDati.getSchemas();
			boolean found=false;
			while(rs.next()) {
				if(rs.getString("TABLE_SCHEM").equalsIgnoreCase(schema))
					found=true;
			}
			rs.close();

			if(!found) {
				throw new PersistenceException("Lo schema indicato non esiste");
			}

			//creo uno schema di tipo Relational
			RelationalSchema result=new RelationalSchema(OID,schema);

			/*vedo le tabelle che appartengono allo schema dato*/
			rs=metaDati.getTables(null, schema, null, null);
			while(rs.next()) {
				if(rs.getString("TABLE_TYPE").equalsIgnoreCase("TABLE")) {
					Table table=new Table(tableOID,rs.getString("TABLE_NAME"));
					tableOID++;
					result.addTable(table);
				}
			}
			rs.close();

			/*adesso creo le colonne; per ogni tabella mi faccio ritornare dai metadati le colonne
			 * cge appartengono alle tabella*/
			rs=metaDati.getColumns(null, schema, null , null);
			while(rs.next()) {
				//nullable=true  espresso come una string "YES" (non so il motivo :) )
				boolean isNullable=rs.getString("IS_NULLABLE").equalsIgnoreCase("YES");
				/*in condizioni normali verifico che il tipo sia uno di quelli ammessi e creo
				 * la colonna; in condizioni non normali(cio questo metodo  richiamato dall'importer dell'OR)
				 * devo creare la colonna a patto che i tipi non siano REF e tipi complessi(li creo direttamente
				 * di la)*/
				if(normal || (!normal && rs.getInt("DATA_TYPE")!=java.sql.Types.REF && rs.getInt("DATA_TYPE")!=java.sql.Types.STRUCT)) {
					/*c' il problem per del tipo, in modelgen supponiamo che la stringa type sia un tipo SQL valido
					 * quindi per prima cosa faccio un controllo sul tipo*/
					String type=DB2RelationalDataType.getModelGenDataType(rs.getInt("DATA_TYPE"), rs.getString("TYPE_NAME"), rs.getInt("COLUMN_SIZE"));
					Column column=new Column(columnOID,rs.getString("COLUMN_NAME"),type,isNullable,false);
					Table table=result.getTableByName(rs.getString("TABLE_NAME"));
					/*questo  necessario perch getColumns ritorna anche le colonne di altri tipi di 
					 * tabelle(ad esempio le typed table)*/
					if(table!=null) {
						table.addColumn(column);
						columnOID++;
					}							
				}
			}
			rs.close();


			/*prima ho caricato tutte le colonne con valore IsIdentifier=false
			 * ora devo vedere tutte le chiavi primarie e modificare corrispondentemente
			 * il valore in quelle colonne
			 * SE nel db di ModelGen verr aggiunto un costrutto Primary Key, si deve intervenire qui
			 * */
			for(Table table:result.getTables()) {
				//sono costretto a fare un query per ogni tabella perch il dirver JDBC per DB2
				//sbarella se metto null al posto del nome della tabella
				rs=metaDati.getPrimaryKeys(null, schema, table.getName());
				while(rs.next()) {
					Column column=table.getColumnByName(rs.getString("COLUMN_NAME"));
					if(column!=null) {
						column.setKey(true);
					}
				}
				rs.close();
			}


			/*ora devo creare le FK; 
			 * per ogni tabella vedo le ImportedKeys stesso motivo di prima :) 
			 * sbarella se metto null al posto del nome della tabella*/
			if(normal) {
				for(Table table:result.getTables()) {
					rs=metaDati.getImportedKeys(null, schema, table.getName());
					ForeignKey fk=null;

					while(rs.next()) {
						//cerco la taabella to della FK
						Table to=result.getTableByName(rs.getString("PKTABLE_NAME"));
						if(to!=null) {
							/*se abbiamo un solo Component KEY_SEQ=1 quindi devo creare una FK
							 * e un un component
							 * se KEY_SEQ>1 significa che la FK corrispondente lh' gi creata devo solo 
							 * aggiungere un nuovo componente*/
							if(rs.getShort("KEY_SEQ")<=1) {
								fk=new ForeignKey(fkOID,table.getName()+"-"+to.getName(),table,to);
								fkOID++;
								result.addForeignKey(fk);
							}
							Column columnFrom=table.getColumnByName(rs.getString("FKCOLUMN_NAME"));
							Column columnTo=to.getColumnByName(rs.getString("PKCOLUMN_NAME"));
							if(columnFrom!=null && columnTo!=null) {
								ComponentOfForeignKey cFK=new ComponentOfForeignKey(ComponentFkOID,columnFrom.getName()+"-"+columnTo.getName(),
										columnFrom,columnTo);								
								ComponentFkOID++;
								if (fk != null) {
									fk.add(cFK);
								}
							}
							else 
								throw new PersistenceException("Non trovate le colonne che appartengono alla FK");
						}
						else 
							throw new PersistenceException("Non trovata la tabella che appartiene alla FK");					
					}				
					rs.close();
				}	
			}					

			/*faccio un controllo sulle FK, in particolare visto che in ModelGen non  presente la possibilit
			 * di specificare chiavi univoche, le FK devono essere NECESSARIAMENTE solo su chiave primaria*/
			for(ForeignKey fk:result.getForeignKeys()) {
				Table to=result.getTableByOID(fk.getToOID());
				if(to!=null) {
					/*prendo tutti le colonne che appartengono come component alla fk*/
					Vector<Column> columnsTo=new Vector<Column>();
					for(ComponentOfForeignKey cKey:fk.getComponents()) {
						Column temp=result.getColumnByOID(cKey.getToOID());
						if(temp!=null) {
							columnsTo.add(temp);
						}
					}
					/*adesso devo verificare che per ogni colonna che appartiene alla FK isKey=true
					 * per TUTTE le altre colonne deve essere isKey=false*/
					for(Column column:to.getColumns()) {
						if(columnsTo.contains(column)) {
							if(!column.isKey())
								throw new ImportExportException("La colonna "+column.getName()+" non  definita come chiave primaria\n e appartiene " +
										"alla FK "+fk.getName());
						}
						else {
							if(column.isKey())
								throw new ImportExportException("La colonna "+column.getName()+"  definita come chiave primaria\n e NON appartiene " +
										"alla FK "+fk.getName());
						}
					}
				}
			}

			return result;

		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import del relazionale");
		}
	}


	/**Import the istances of a SchemaRelational
	 * 
	 * @see org.modelgen.importexport.Import.Importer#importInstance(java.lang.Object, org.modelgen.importexport.Import.Source)
	 */
	public RelationalSchema importInstance(RelationalSchema schema, Source<DbConnection> source) throws PersistenceException, ImportExportException {
		/*una source per l'import di uno schema relazionale  una DbConnection che  un wrapper
		 * per 1) una String che rappresenta il nome dello schema che si deve importare
		 * 2)una Connection verso quel database
		 * quindi per prima cosa estraggo queste due informazioni 
		 * */
		String schema_name=source.getSource().getSchema().toUpperCase();
		Connection con=source.getSource().getConnection();
		
		try {
			/*devo creare le tuple e i vari componenti; per farlo ovviamente devo eseguire 
			 * una query per ogni tabella
			 * la query viene generata a partire dalla colonne che ho caricato per la tabella stessa*/
			for(Table t : schema.getTables())  {				
				//i create the query in the correct order
				StringBuilder sb=new StringBuilder("SELECT ");
				for(Column c : t.getColumns()) {
					sb.append(c.getName());
					sb.append(",");
				}
				sb.delete(sb.length()-1,sb.length());
				sb.append(" FROM ");
				sb.append(schema_name);
				sb.append(".");
				sb.append(t.getName());

				/*ora ho creato la query e la eseguo*/
				String queryTupleC=sb.toString();
				Statement sTC=con.createStatement();
				ResultSet rs=sTC.executeQuery(queryTupleC);
				
				/*per ogni riga del ResultSet devo creare una tupla e vari component quante sono
				 * le colonne; devo ovviamente fare attenzione anche ai valori nulli*/
				while(rs.next()) {
					Tuple tuple=new Tuple(tupleOID,t);
					tupleOID++;
					t.addTuple(tuple);
					for(Column c: t.getColumns()) {
						/*ATTENZIONE il value potrebbe essere un valore null
						 * in questo caso quindi la stringa con cui creo il valore  proprio null,
						 * ma NON  un errore semplicemente quando andr a salvare il valore in ModelGen
						 * questo value sar null*/
						String value=DB2RelationalDataType.getModelGenValue(c.getType(), rs.getString(c.getName()));
						TupleComponent tc=new TupleComponent(tupleCOID,value,c);
						tuple.addTupleComponent(tc);
						tupleCOID++;						
					}					
				}
			}
			
			return schema;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore SQL nell'import delle istanze");
		}	
	}
	
	/*Main di prova*/
	public static void main(String[] args){
		try {
			DbConnection con=ConnectionFactory.getConnection(ConnectionFactory.DB2, "Rel","localhost", 
					50000, "db2admin","db2admin","Relazionale");
			Source<DbConnection> source=new Source<DbConnection>(con);
			RelationalSchema schema=new JDBCRelational().importSchema(source);
			schema=new JDBCRelational().importInstance(schema, source);
			System.out.println(schema.toString());
//			ImportExportPersistence.getInstance().storeSchema(schema);
//			ImportExportPersistence.getInstance().storeInstance(schema);
			
		} catch (PersistenceException e) {
			e.printStackTrace();
		} catch (ImportExportException e) {
			e.printStackTrace();
		}
	}

}
