package org.modelgen.importexport.Import;

import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.XmlInstanceFilter;
import org.modelgen.importexport.utilities.XmlSchemaFilter;
import org.modelgen.importexport.utilities.XmlSchemaSourceDestination;
import org.modelgen.importexport.utilities.XmlSchemaXpathParser;
import org.modelgen.importexport.utilities.datatype.XmlSchemaDataType;
import org.modelgen.importexport.xml.model.ComplexElement;
import org.modelgen.importexport.xml.model.ComplexInstance;
import org.modelgen.importexport.xml.model.ComplexType;
import org.modelgen.importexport.xml.model.Element;
import org.modelgen.importexport.xml.model.ForeignKey;
import org.modelgen.importexport.xml.model.Key;
import org.modelgen.importexport.xml.model.SimpleElement;
import org.modelgen.importexport.xml.model.SimpleInstance;
import org.modelgen.importexport.xml.model.SimpleType;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**This class permists to Import a XmlSchema from a File.
 *  A XmlSchema can be defined in a lot of ways, some of which are not supported by ModelGen
 *  supermodel.
 *  To know which are the limitations of the ModelGen XmlSchema please see the thesis.
 *  
 *  @version 0.9
 *  
 */

public class XMLSchemaImporter implements Importer<XmlSchema,XmlSchemaSourceDestination> {
	/*the only OID i load from ModelGen, all other elements are created with a default OID
	 *which is modified once the will be loaded in ModelGen */
	private int schemeOID;
	//list of the global SimpleType
	private List<SimpleType> simples;
	//list of the gloabl complexType
	private List<ComplexType> complexes;
	//list of the primary key
	private List<Key> primaryKeys;
	//list of the Foreign keys
	private List<Key> fks;
	
	public XMLSchemaImporter() throws PersistenceException {
		schemeOID=ImportExportPersistence.getInstance().getNewOID("Schema");
		
		simples=new ArrayList<SimpleType>();
		complexes=new ArrayList<ComplexType>();
		primaryKeys=new ArrayList<Key>();
		fks=new ArrayList<Key>();
	}

	/**This metod check the source to see if it is acceptable
	 * 
	 * Attenzione: ancora da decidere se serve
	 * 
	 * @param source the source to be checked
	 * @return true if the source if OK
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted) 
	 */
	public boolean checkSource(Source<XmlSchemaSourceDestination> source) throws PersistenceException, ImportExportException {
		return true;
	}

	/**Imports a schema from a source.
	 * This metod as the role to create a schema of the correct type and the object related which are in the 
	 * schema defined in the source.
	 * For example it must create a XmlSchema, all the global simple/complex Type, all the element defined adn so on
	 * 
	 * 
	 * @param source the source form which get information about the schema
	 * @return the schema 
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted)
	 */
	public XmlSchema importSchema(Source<XmlSchemaSourceDestination> source) throws PersistenceException, ImportExportException {
		try {
			/*creo lo schema dal nome del file*/
			String nameFile=new StringTokenizer(source.getSource().getSchema().getName(),".").nextToken();
			if(nameFile==null || nameFile.length()==0)
				throw new ImportExportException("Errore nella creazione dello schema");
			XmlSchema schema=new XmlSchema(schemeOID,nameFile);
			/*prendo la factory*/
			XMLInputFactory inputFactory=XMLInputFactory.newInstance();
			/*creo un primo reader,filtrando le informazioni inutili*/
			XMLStreamReader reader =inputFactory.createFilteredReader(
					inputFactory.createXMLStreamReader(new FileInputStream(source.getSource().getSchema())),new XmlSchemaFilter());

			/*per prima cosa devo vedere tutti i type*/
			while(reader.hasNext()){
				reader.next();
				/*se ho un simple type lo analizzo e lo aggiungo alla lista dei simpletype globali*/
				if(reader.getLocalName().equals("simpleType")){
					String name=getAttributeValue(reader,"name");
					if(name!=null) {
						simples.add(getSimpleType(reader,name));
					}
					else {
						throw new PersistenceException("Il nome di un simpleType globale deve essere specificato");
					}
				}
				/*stessa cosa per i complex(per ora analizzo SOLO il nome)*/
				else if(reader.getLocalName().equals("complexType")){
					String name=getAttributeValue(reader,"name");
					if(name!=null) {
						complexes.add(new ComplexType(name));
						toEndElement(reader, 1);
					}
					else
						throw new PersistenceException("Il nome di un complexType globale deve essere specificato");
				}
				/*per tutte le altre cose vado alla fine*/
				else {
					toEndElement(reader, 1);
				}
			}
			reader.close();

			/*creo un nuovo reader*/
			reader =inputFactory.createFilteredReader(
					inputFactory.createXMLStreamReader(new FileInputStream(source.getSource().getSchema())),new XmlSchemaFilter());
			/*adesso analizzo tutti i complexType global(E' NECESSARIO il doppi passaggio)*/
			while(reader.hasNext()) {
				reader.next();
				
				/*se si tratta di un complex questa volta oltre a prenderne il nome devo vedere che figli ha ecc*/
				if(reader.getLocalName().equals("complexType")){
					String name=getAttributeValue(reader,"name");
					if(name==null)
						throw new PersistenceException("Errore nel parsing del complex type");
					/*analizzo il complex type*/
					getComplexType(reader,name,true);					
				}
				else
					toEndElement(reader, 1);
			}
			reader.close();
			//System.out.println("complessi dopo aver aggiunto gli elementi:\n\n"+complexes.toString()+"\n\n\n");
			
			/*creo un nuovo reader*/
			reader =inputFactory.createFilteredReader(
					inputFactory.createXMLStreamReader(new FileInputStream(source.getSource().getSchema())),new XmlSchemaFilter());
			/*per ultima cosa analizzo gli elementi*/
			while(reader.hasNext()) {
				reader.next();
				
				/*se si tratta di un complex questa volta oltre a prenderne il nome devo vedere che figli ha ecc*/
				if(reader.getLocalName().equals("element")){
					String name=getAttributeValue(reader,"name");
					if(name==null)
						throw new PersistenceException("Errore nel parsing del complex type");
					/*analizzo il complex type*/
					schema.addElement(getElement(reader, false));			
				}
				/*per le altre cose vado alla fine*/
				else
					toEndElement(reader, 1);
			}       
			reader.close();
			//System.out.println("chiavi primarie:\n"+primaryKeys.toString());
			//System.out.println("\n\nchiavi esterne:\n"+fks.toString());
			
			/*qui inserire controllo un solo elemento+tutti quelli sotto unbounded*/
			if(schema.getTopElements().size()>1) 
				throw new ImportExportException("Anche se l'XSD definito  corretto,  richiesto dal modello" +
						"Xsd di ModelGen\n che vi sia un solo elemento radice che contiene tutti element unbounded al primo livello");
			
			for(Element e:schema.getTopElements()) {
				for(Element child:e.getChilds()) {
					if(!(child instanceof ComplexElement))
						throw new ImportExportException("Anche se l'XSD definito  corretto,  richiesto dal modello Xsd di Modelgen\nche " +
								"il primo livello della radice sia formato da soli elementi complessi");
					if(!child.isUnbounded())
						throw new ImportExportException("Anche se l'XSD definito  corretto,  richiesto dal modello Xsd di Modelgen\nche " +
								"l'elemento al primo livello "+child.getName()+" sia definito unbounded");
					if(child.isOptional())
						throw new ImportExportException("Anche se l'XSD definito  corretto,  richiesto dal modello Xsd di Modelgen\nche " +
								"l'elemento al primo livello "+child.getName()+" sia definito isOptional=false, ovvero bisogna togliere minOccurs");
				}				
			}
			
			//System.out.println(schema.toString());
			
			/*adesso per ogni chiave devo trovare l'elemento corrispondente al selector
			 * e mettere  isId=true ai figli identificati dai vari field*/
			for(Key key:primaryKeys) {
				if(schema.getTopElements().size()>=1)
					XmlSchemaXpathParser.setPrimaryKey(key, schema.getTopElements().get(0));
			}
			
			/*verifico che tutti gli elementi corrispondenti alle chiavi siano stati trovati*/
			for(Key key:primaryKeys) {
				//1)non ho trovato il context node
				if(key.getContextNode()==null)
					throw new ImportExportException("La chiave "+key.getName()+" definisce un context node che non  stato trovato");
				//2)il numero fi field definito nella chiave non corrisponde a quelli trovati
				if(key.getFields().size()!=key.getFieldNodes().size())
					throw new ImportExportException("La chiave "+key.getName()+" definisce dei field che non sono stati trovati\n" +
							"Accertarsi che i field definiti siano figli diretti del context node e che siano di tipo semplice");
			}
			
			//System.out.println("\n\n\nchiavi esterne:");
			
			/*adesso devo fare pi o meno la stessa dosa delle chiavi primarie con le chiavi esterna*/
			for(Key fk:fks) {
				Key key=getKeyByName(primaryKeys,fk.getRefer());
				if(key!=null)
					if(schema.getTopElements().size()>=1)
						XmlSchemaXpathParser.getforeignKey(fk, key,schema.getTopElements().get(0),schema);
				else 
					throw new ImportExportException("La Fk "+fk.getName()+" referenzia la chiave " +
							""+fk.getRefer()+" che non  stata trovata");
			}
			
			/*controllo sulle fk*/
			for(ForeignKey fk:schema.getForeignKeys()) {
				if(schema.getTopElements().size()>=1 && (fk.getFrom()==schema.getTopElements().get(0) || 
						fk.getTo()==schema.getTopElements().get(0)))
					throw new ImportExportException("Le Fk definite sono corrette ma definite sull'elemento " +
							"radice che non viene copiato in ModelGen\nSi prega di eliminare/commentare tali Fk");
				if(fk.getFrom()==null || fk.getTo()==null)
					throw new ImportExportException("Non sono stati identificati i complex cui si riferisce " +
							"la Fk di nome "+fk.getName()+"\nVerificare i context node definiti");
				if(!(fk.getComponents().size()>0))
					throw new ImportExportException("La Fk "+fk.getName()+" non ha componenti\nVerificare che " +
							"i field definiti siano simple element e che siano figli diretti del context node");
			}
			
			//System.out.println(schema.toString());
			
			return schema;
		} catch (FileNotFoundException e) {
			new PersistenceException("Il file specificato non esiste");
			return null;
		} catch (XMLStreamException e) {
			e.printStackTrace();
			new PersistenceException("Errore nella lettura del file XML Schema");
			return null;
		}	
	}

	/**Retrive a PrimaryKey from the list given its name.
	 * @param keys
	 * @param refer
	 */
	private Key getKeyByName(List<Key> keys, String refer) {
		for(Key key:keys) {
			if(key.getName().equals(refer))
				return key;
		}
		return null;
	}


	/**Retrive a SimpleType, both local and global. 
	 * 
	 * @param reader
	 * @param name
	 * @throws ImportExportException
	 * @throws PersistenceException
	 */
	private SimpleType getSimpleType(XMLStreamReader reader,String name) throws ImportExportException, PersistenceException {
		/* ho incontrato un simpleType, quindi consumo tutto fino ad arrivare ad un element che si chiami restriction*/
		try {
			while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("simpleType"))) {
				reader.next();
				
				//1)salto le ventuali annotation
				if(reader.getLocalName().equals("annotation"))
					toEndElement(reader, 1);
				//2)ho la restriction e ne prendo il type
				else if(reader.getLocalName().equals("restriction")) {
					String type=getAttributeValue(reader,"base");
					//vado a prendere un type buono per modelgen
					type=XmlSchemaDataType.getXmlDataType(type);
					if(type==null)
						throw new ImportExportException("Errore nel parsing del simpleType "+name);
					else {
						//vado alla fine della restriction e del simpleType e lo ritorno
						toEndElement(reader, 2);
						return new SimpleType(name,type);
					}
				}
				else if(reader.getLocalName().equals("union"))
					throw new ImportExportException("La union non  ammissibile");
				else if(reader.getLocalName().equals("list"))
					throw new ImportExportException("La list non  ammissibile");
			}
		} catch (XMLStreamException e) {
			throw new PersistenceException("Errore nel parsing del file XSD");
		}
		return null;
	}
	
	/**Analizza un complexType sia local che globale.
	 * @param reader
	 * @param name
	 * @param global
	 * @throws ImportExportException
	 * @throws PersistenceException
	 */
	private ComplexType getComplexType(XMLStreamReader reader,String name,boolean global) throws ImportExportException, PersistenceException {
		ComplexType complex=null;
		if(global) {
			complex=getComplex(name);
		}
		else
			complex=new ComplexType(name);
		
		if(complex!=null) {
			/*ora devo verificare che questo complexType globale sia corretto per noi*/
			//1)niente mixed come attributo
			String mixed=getAttributeValue(reader,"mixed");
			if(mixed!=null && !mixed.equalsIgnoreCase("false"))  {
				throw new ImportExportException("Il complex "+complex.getName()+" definisce il suo conenuto mixed=true, che non  consentito");
			}
			/*adesso devo vedere i figli di questo nodo*/
			try {
				while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("complexType"))) {
					reader.next();
					
					/*2)non pu avere figli che siano simple/complexcontent*/
					if((reader.getLocalName().equals("simpleContent")) || reader.getLocalName().equals("complexContent"))
						throw new ImportExportException("Il complexType "+complex.getName()+" definisce un simple/complexContent, che non  ammissibile");
					/*3)non pu usare le choice*/
					else if(reader.getLocalName().equals("choice"))
						throw new ImportExportException("Il complexType "+complex.getName()+" definisce una choice, che non  ammissibile");
					/*4)non pu usare le all*/
					else if(reader.getLocalName().equals("all"))
						throw new ImportExportException("Il complexType "+complex.getName()+" definisce un gruppo all, che non  ammissibile");
					/*finalmente sto alla sequence*/
					else if(reader.getLocalName().equals("sequence")) {
						analizeSequence(reader,complex,global);
					}
				}
			} catch (XMLStreamException e) {
				throw new PersistenceException("Errore nel parsing");
			}		
			return complex;
		} 		
		else
			throw new ImportExportException("Il complex non  stato caricato durante la prima scansione del file");
	}

	/**Retrive a global ComplexType given its name.
	 * @param name
	 */
	private ComplexType getComplex(String name) {
		for(ComplexType complex:complexes) {
			if(complex.getName().equals(name))
				return complex;
		}
		return null;
	}

	/**Retrive the value of the attribute of the given name
	 * 
	 * @param reader
	 * @param name
	 */
	private String getAttributeValue(XMLStreamReader reader,String name) {
		if(!(reader.getEventType()==XMLStreamConstants.START_ELEMENT)) {
			System.out.println("Attenzione errore: "+reader.getLocalName());
			return null;
		}
		for(int i=0; i<reader.getAttributeCount();i++){
			if(reader.getAttributeLocalName(i).equals(name)) {
				return reader.getAttributeValue(i);
			}
		}
		return null;
	}
	
	/**Retrive an Element.
	 * 
	 * @param reader
	 * @param global
	 * @throws PersistenceException
	 * @throws ImportExportException
	 */
	private Element getElement(XMLStreamReader reader,boolean global) throws PersistenceException, ImportExportException {
		/*possibilit:
		 * 1)element semplice con simple type globale type=>creo un element 
		 * 2)element complesso con compleType globale=>creo un element o poi ci copier la roba del complexType globale
		 */
		
		if(getAttributeValue(reader, "ref")!=null)
			throw new ImportExportException("Gli elementi ref non sono ammessi in questa versione di ModelGen");
		
		String name=getAttributeValue(reader, "name");		
		if(name!=null) {
			Element e=getElement(reader,name,global);
			//System.out.println("ritorno un elemento:\n"+e.toString());
			analizeKeys(reader);
			return e;			
		}
		else
			throw new PersistenceException("La sequence definisce un elemento che non  n un riferimento ne un elemento");
	}
	
	/**Retrive all the primary e ForeignKey.
	 * It constructs only the String representation; the corrisponding Element are identified after that all other
	 * computation are done.
	 * 
	 * @param reader
	 * @throws PersistenceException
	 */
	private void analizeKeys(XMLStreamReader reader) throws PersistenceException {
		while(reader.hasName() && !(reader.isEndElement() && reader.getLocalName().equals("element"))) {
			try {
				reader.next();
				/*se ho una chiave*/
				if(reader.getLocalName().equals("key")) {
					String name=getAttributeValue(reader, "name");
					reader.next();
					
					String selector=null;
					if(reader.getLocalName().equals("selector")) {
						selector=getAttributeValue(reader, "xpath");
						toEndElement(reader, 1);
					}
					if(name!=null && selector!=null) {
						Key key=new Key(name,selector);
						while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("key"))) {
							reader.next();
							
							if(reader.getLocalName().equals("field")) {
								String field=getAttributeValue(reader, "xpath");
								if(field!=null)
									key.addField(field);
								toEndElement(reader, 1);
							}
						}
						primaryKeys.add(key);
						//System.out.println(key.toString());
					}					
				}
				/*se ho una chiave esterna*/
				else if(reader.getLocalName().equals("keyref")) {
					String name=getAttributeValue(reader, "name");
					String refer=getAttributeValue(reader, "refer");
					reader.next();
					
					String selector=null;
					if(reader.getLocalName().equals("selector")) {
						selector=getAttributeValue(reader, "xpath");
						toEndElement(reader, 1);
					}

					if(name!=null && refer!=null && selector!=null) {
						Key fk=new Key(name,selector,refer);
						while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("keyref"))) {
							reader.next();
							
							if(reader.getLocalName().equals("field")) {
								String field=getAttributeValue(reader, "xpath");
								if(field!=null)
									fk.addField(field);
								toEndElement(reader, 1);
							}
						}
						fks.add(fk);
						//System.out.println(fk.toString());
					}			
				}
			} catch (XMLStreamException e) {
				throw new PersistenceException("Errore nel parsing del file Xsd, chiavi");
			}	
		}
	}
	
	/**Analize the child sequence of a ComplexType either global or local.
	 * 
	 * @param reader
	 * @param complex
	 * @param global
	 * @throws ImportExportException
	 * @throws PersistenceException
	 */
	private void analizeSequence(XMLStreamReader reader,ComplexType complex,boolean global) throws ImportExportException, PersistenceException {
		/*verifico ora che la sequence sia definita come noi vogliamo:*/
		//1)l'eventuale attributo della sequence minOccurs deve essere 1
		String min=getAttributeValue(reader,"minOccurs");
		if(min!=null && !min.equalsIgnoreCase("1"))
			throw new ImportExportException("La sequence dichiarata all'interno del complex type "+complex.getName()+" dichiara minOccurs!=1");
		//2)stessa cosa per maxOccurs
		String max=getAttributeValue(reader, "maxOccurs");
		if(max!=null && !max.equalsIgnoreCase("1"))
			throw new ImportExportException("La sequence dichiarata all'interno del complex type "+complex.getName()+" dichiara maxOccurs!=1");
		
		/*ora dentro ad una sequence ci pu essere: annotation,group e choice, altre sequence e un numero imprecisato di element*/
		try {
			while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("sequence"))) {
				reader.next();
				
				if(reader.getLocalName().equals("annotation"))
					toEndElement(reader, 1);
				else if(reader.getLocalName().equals("attribute")) {
					toEndElement(reader, 1);
				}
				else if(reader.getLocalName().equals("group")) {
					throw new ImportExportException("Non  possibile usare group all'interno di una sequence");
				}
				else if(reader.getLocalName().equals("choice")) {
					throw new ImportExportException("Non  possibile usare choice all'interno di una sequence");
				}
				else if(reader.getLocalName().equals("any")) {
					throw new ImportExportException("Non  possibile usare any all'interno di una sequence");
				}
				else if(reader.getLocalName().equals("element")) {
					//System.out.println("aggiungo l'elemento al complex");
					complex.addElement(getElement(reader,global));
				}	
			}
		} catch (XMLStreamException e) {
			throw new PersistenceException("Errore nel parsing del file XSD");
		} 
	}
	
	/**Analize an Element.
	 * 
	 * @param reader
	 * @param name
	 * @param global
	 * @throws PersistenceException
	 * @throws ImportExportException
	 */
	private Element getElement(XMLStreamReader reader,String name,boolean global) throws PersistenceException, ImportExportException {
		/*per prima cosa verifico gli attributi di questo element*/
		//1)optional
		boolean optional=false;
		String minOccurs=getAttributeValue(reader, "minOccurs");
		if(minOccurs!=null && minOccurs.equals("0"))
			optional=true;
		//2)nullable
		boolean nullable=false;
		String nillable=getAttributeValue(reader, "nillable");
		if(nillable!=null && nillable.equals("true"))
			nullable=true;
		//3)maxOccurs
		boolean unbounded=false;
		String max=getAttributeValue(reader, "maxOccurs");
		if(max!=null && max.equals("unbounded")) 
			unbounded=true;
		/*ho due sole possibilit per il tipo:
		 * 1) definito l'attributo type(e questo pu essere o simpleType o complexType)*/
		String type=getAttributeValue(reader, "type");
		if(type!=null) {
			//verifico se il tipo  tra i simpleType globali
			SimpleType simple=null;
			for(SimpleType s:simples) {
				if(s.getName().equals(type))
					simple=s;
			}
			//altrimenti sta tra i comeplxType globali
			ComplexType complex=null;
			for(ComplexType c:complexes) {
				if(c.getName().equals(type))
					complex=c;
			}
			
			if(simple!=null && complex==null) {
				Element e =new SimpleElement(name,nullable,optional,false,unbounded,simple.getType());
				return e;
			}
			else if (simple==null && complex!=null) {
				/*creo un ComplexElement che  collegato ad un type globale*/
				//System.out.println("creo un complexElement con tipo globale "+complex.getName());
				ComplexElement complexElement=new ComplexElement(name,optional,nullable,unbounded,complex);
				return complexElement;
			}
			/*non sta tra i tipi globale, provo tra i tipi generici*/
			else if(simple==null && complex==null) {
				type=XmlSchemaDataType.getXmlDataType(type);
				Element e =new SimpleElement(name,nullable,optional,false,unbounded,type);
				return e;
			}					
		}
		/*2)ho un figlio che pu essere o simpleType oppure ComplexType*/
		else {
			try {
				boolean foundType=false;
				while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals("element"))) {
					reader.next();
					
					if(reader.getLocalName().equals("simpleType")) {
						foundType=true;
						String tipo=getSimpleType(reader,"").getType();
						Element e =new SimpleElement(name,nullable,optional,false,unbounded,tipo);
						return e;
					}
					else if(reader.getLocalName().equals("complexType")) {
						foundType=true;
						ComplexType complexType=getComplexType(reader,name,false);
						//System.out.println("creo un complexElement con tipo locale"+complexType.getName());
						ComplexElement complexElement=new ComplexElement(name,optional,nullable,unbounded,complexType);
						return complexElement;
					}
				}
				if(!foundType)
					throw new ImportExportException("L'elemento non specifica nessun tipo ne semplice ne complesso");
			} catch (XMLStreamException e) {
				throw new PersistenceException("Errore nel parsing del file XSD");
			}
		}
		return null;
	}

	/**Utility metod to go to the end of a number of elements.
	 * 
	 * @param reader
	 * @param elementi
	 * @throws PersistenceException
	 */
	private void toEndElement(XMLStreamReader reader,int elementi) throws PersistenceException {
		try {
			while(reader.hasNext() && elementi>0) {
				int type=reader.next();
				/*per ogni nuovo elemento aumento di uno gli elementi da chiudere*/
				if(type==XMLStreamConstants.START_ELEMENT) {
					elementi++;
				}
				/*chiudo uno degli elementi lasciati aperti*/
				else if(type==XMLStreamConstants.END_ELEMENT) {
					elementi--;
				}
			}
		} catch (XMLStreamException e) {
			throw new PersistenceException("Errore nel parsing del file XSD");
		}
	}

	/**Imports the instances of a schema.
	 * This metod has the role to import the instances of the given schema from a source.
	 * The source has to be consistent with the source given to load the schema.
	 * For example we can import a XML schema from a file and its instances from another file (or many other files)
	 * 
	 * 
	 * @param schema the schema from which we must load the instances
	 * @param source the source form which get information about the instances
	 * @return the schema with instances imported
	 * @throws PersistenceException any error connected with the source(e.g. FileNotFound,ConnectionError)
	 * @throws ImportExportException any error connected with the procedure to import (e.g. schema not permitted)
	 */
	public XmlSchema importInstance(XmlSchema schema, Source<XmlSchemaSourceDestination> source) throws PersistenceException, ImportExportException {
		
		try {
			if(source.getSource().getInstance().size()<1)
				throw new ImportExportException("Le istanze devono essere composte da almeno un file");
			
			File instance=source.getSource().getInstance().get(0);
			/*prendo la factory*/
			XMLInputFactory inputFactory=XMLInputFactory.newInstance();
			/*creo un primo reader,filtrando le informazioni inutili*/
			XMLStreamReader reader =inputFactory.createFilteredReader(
					inputFactory.createXMLStreamReader(new FileInputStream(instance)),new XmlInstanceFilter());

			/*per prima cosa devo vedere tutti i type*/
			if(schema.getTopElements().size()>1 || schema.getTopElements().size()==0)
				throw new ImportExportException("Lo schema deve essere composto da un unico elemento radice");
			Element root=schema.getTopElements().get(0);
			if(root==null)
				throw new ImportExportException("L'elemento radice  null");
			
			ComplexInstance instanceRoot=new ComplexInstance(root);
			
			while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals(root.getName()))) {
				if(reader.isStartElement() && reader.getLocalName().equals(root.getName())) {
					//cominciare l'analisi
					schema.addInstance(instanceRoot);
					reader.next();
					parseInstances(reader,root.getChilds(),instanceRoot);
				}				
			}
			reader.close();
			
			/*per i successivi elementi*/
			source.getSource().getInstance().remove(0);
			
			for(File otherInstance:source.getSource().getInstance()) {
				/*creo un primo reader,filtrando le informazioni inutili*/
				reader =inputFactory.createFilteredReader(
						inputFactory.createXMLStreamReader(new FileInputStream(otherInstance)),new XmlInstanceFilter());
			
				while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals(root.getName()))) {
					if(reader.isStartElement() && reader.getLocalName().equals(root.getName())) {
						reader.next();
						parseInstances(reader,root.getChilds(),instanceRoot);
					}				
				}
				reader.close();
			}			
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (FactoryConfigurationError e) {
			e.printStackTrace();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}		
		
		return schema;
	}
	
	/**Esegue il parsing delle istanze contenuto nella sorgente
	 * 
	 * 
	 * @param reader
	 * @param child
	 * @param instance
	 * @throws ImportExportException
	 */
	private void parseInstances(XMLStreamReader reader,List<Element> child,ComplexInstance instance) throws ImportExportException {
		try {
			while(reader.hasNext() && !(reader.isEndElement() && reader.getLocalName().equals(instance.getSchemaElement().getName()))) {
				if(reader.isStartElement()) {
					String name=reader.getLocalName();
					//a partire dela nome dell'elemento identifico l'istanza corrispondente
					Element schemaElement=getElement(child, name);
					
					if(schemaElement instanceof SimpleElement) {
						/*creo semplicemente un simpleInstance*/
						reader.next();
						String value;
						if(reader.isEndElement())
							value=null;
						else
							value=reader.getText();
						SimpleInstance simpleInstance=new SimpleInstance(schemaElement,value);
						instance.addInstance(simpleInstance);
					}
					else if(schemaElement instanceof ComplexElement) {
						/*creo un'istanza complessa e vado ricorsivamente dentro*/
						ComplexInstance complexInstance=new ComplexInstance(schemaElement);
						instance.addInstance(complexInstance);
						reader.next();
						parseInstances(reader,schemaElement.getChilds(),complexInstance);
					}
				}
				reader.next();
			}
		} catch (XMLStreamException e) {
			e.printStackTrace();
			throw new ImportExportException("Errore nel parsing delle istanze");
		}
	}
	
	/**Metodo banale di ricerca tra i figli del nodo attuale
	 * 
	 * @param childs
	 * @param name
	 */
	private Element getElement(List<Element> childs,String name) {
		for(Element e:childs)
			if(e.getName().equals(name))
				return e;
		return null;
	}
}
