package org.modelgen.importexport;

import org.modelgen.facade.ResourceFunctions;

import java.io.File;
import java.io.FileNotFoundException;

import org.java.plugin.Plugin;


/**Represents the Import Export plugin.
 * The Import/Export plugin is used to make usable ModelGen; in fact it
 * imports a Schema form a Source directly in the supermodel of ModelGen.
 * Thus this schema can be manipulated and transformed into a destination model.
 * It also permits to export a schema to a Destination, so that after applying some rules
 * to turn it into a destination model, it can be exported and used in the Destination system.
 * 
 * For example it is possibile to import a Relational Schema from a DBMS directly into ModelGen
 * db, so that can be transformed into a XmlSchema and then in can be exported to a File.
 *
 */
public class ImportExportPlugin extends Plugin {
    public static final String PLUGIN_ID = "org.modelgen.importexport";

	/**Operations done while loading the Import/export plugin.
	 * It only check is the config file is present.
	 * 
	 * @see org.java.plugin.Plugin#doStart()
	 */
	protected void doStart() throws Exception {
		/*verifico che il file di config esista, in modo che il plugin non sia neanche caricato
		 * se questo file non esiste
		 * Attenzione: se si vuole cambiare il nome del file non basta cambiare qui, ma anche 
		 * nel codice del configurator*/
		File f=ResourceFunctions.createFileIfNotExist(this.getClass(), "importExport.config", PLUGIN_ID);
		if(!f.exists()){
			throw new FileNotFoundException("Il file con le propriet per l'ImportExport non" +
					" nella locazione giusta");
		}
	}

	/**Operations to be done while unloading the plugin.
	 * None so far.
	 * 
	 * @see org.java.plugin.Plugin#doStop()
	 */
	protected void doStop() throws Exception {

	}
	
}
