package org.modelgen.importexport;

//import generici
import org.modelgen.importexport.components.ExportSelection;
import org.modelgen.importexport.utilities.Configurator;
import org.modelgen.importexport.utilities.ExportTableModel;
import org.modelgen.jpf.core.commons.Tool;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.LayoutStyle;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**GUI of the plugin.
 * Contains all the code to create and manipulate the visual componets.
 * 
 * @version 0.9
 *
 */
public class ImportExportTool extends JInternalFrame implements Tool,ActionListener, ListSelectionListener{
	private static final long serialVersionUID = 6645035596795893911L;
	
	/*Singleton della GUI*/
	private static ImportExportTool istance;
	/*mappa degli oggetti scambiati tra core e plugin
	 * Per il momento credo nessuno*/
	private Map<String,Object> applicationObjs;
	
	/*componenti grafiche*/
	private JTabbedPane tabs;
	/*tab per l'import*/
	private JPanel importPanel;
	/*panel superiore che contiene sia panello per i modelli che per le implementazioni*/
	private JPanel importTop;
		private JPanel importModels; 			private JPanel importImplementation;
		private JComboBox importModelCombo; 	private JComboBox importImplementazioneCombo;
														private JButton importStart;
	/*pannello  inferiore per inserire eventuali risultati*/
	private JPanel importResult;
	    private JScrollPane importScrollPanel;
	    /*area in cui inserire eventuali messaggi per l'utente*/
	    private JTextArea importText;
	
	/*tab per l'export*/
	private JPanel exportPanel;
	/*panel superiore*/
	private JPanel exportTop;
		private JScrollPane exportTopScroll;
			/*la tabella con gli schemi da esportare*/
			private JTable exportTable;
		/*il bottone per avviare la l'export e per ricaricare gli schemi per l'export*/
		private JButton exportStart;	private JButton exportReload;
	
	/*panel inferiore con i risultati*/
	private JPanel exportResult;
    	private JScrollPane exportScroll; 
    	/*area per stampare alcune informazioni*/
    	private JTextArea exportText;

	/**Creates the GUI of the Import/export plugin
	 * 
	 */
	public ImportExportTool() {
		super("ImportExportTool", true, true, true, true);
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		initComponents();
		importStart.setEnabled(false);
		this.pack();
		istance=this;
	}
	
	/*Singleton*/
	public static ImportExportTool getIstance(){
		/*Il controllo condizionale non  necessario:
		 * - il metodo deve ritornare l'internal-frame solo se questa  stata creata
		 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
		 * dal controllo dell'Application Plugin
		 * */
		return istance;
	}

	/*visto che attualmente non scambio oggetti con il core questi metodi non sono necessari*/
	public void init(Map<String,Object> applicationObjects) {
		this.applicationObjs  = applicationObjects;
	}
	
	public Map<String, Object> getInfo() {		
		return applicationObjs;
	}
	
	/** Il metodo che gestisce tutti gli eventi(tanto sono 4 :) )
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent evt) {
		/*gli eventi che possono essere generati sono:
		 * 1) l'utente ha scelto un modello*/
		if (evt.getSource() instanceof JComboBox) {
			/*e quindi andiamo a modificare la lista delle possibili implementazioni per 
			 * quel modello*/
			JComboBox source=(JComboBox)evt.getSource();
			
			if(source==importModelCombo){
				String modelloScelto=(String)importModelCombo.getSelectedItem();
				importImplementazioneCombo.removeAllItems();
				
				String[] implementations=Configurator.getInstance().getImplementations(modelloScelto);
				for(int i=0;i<implementations.length;i++) {
					importImplementazioneCombo.addItem(implementations[i]);
				}
				if(importModelCombo.getSelectedIndex()!=0)
					importStart.setEnabled(true);
				else 
					importStart.setEnabled(false);
			}
		}
		/*2)l'utente ha cliccato su un bottone*/
		else if(evt.getSource() instanceof JButton) {
			JButton source=(JButton)evt.getSource();
			/*2')ha cliccato su importstart, quindi devo aprire la lista dei parametri*/
			if(source==importStart) {
				/*i parametri dipendono da quale implementazione voglio usare;
				 * ad esempio se uso una Connection mi serviranno indirizzo del server,
				 * porta, ecc.*/
				try {
					String implementazione=(String)importImplementazioneCombo.getSelectedItem();
					String parametetersClass=Configurator.getInstance().getCorrispondenza(implementazione);
					Class.forName(parametetersClass).newInstance();
				} catch (Exception e) {
					JOptionPane.showMessageDialog(this,"It hasn't been possible the configuration of parameters\n" +
							"Please, check configuration's file","Config Error",JOptionPane.ERROR_MESSAGE);
				}
			}
			/*2')ha cliccato su exportStart, quindi devo aprire la lista dei parametri*/
			else if(source==exportStart) {
				new ExportSelection();
			}
			/*2')ha cliccato su exportReload, quindi devo ricaricare la lista di schemi*/
			else if(source==exportReload) {
				ExportTableModel tableModel=new ExportTableModel();
				exportTable.setModel(tableModel);
			}
		}	
	}
	

	/**Metodo che gestisce la selezione su una riga della lista degli schemi da esportare
	 * Se l'utente non ha selezionato nessuna riga(o non ci sono righe:) ) il tasto per eseguire lo start deve essere
	 * disabilitato
	 * 
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		//Ignore extra messages.
		if (e.getValueIsAdjusting()) return;

		ListSelectionModel lsm = (ListSelectionModel)e.getSource();
		if (lsm.isSelectionEmpty()) {
			exportStart.setEnabled(false);
		} else {
			//selezionata una riga
			exportStart.setEnabled(true);
		}
	}
	
	/**Returns the name of the selected Model for import.
	 */
	public String getImportModel() {
		return (String)this.importModelCombo.getSelectedItem();
	}
	
	/**Returns the name of the selected implementations for import.
	 */
	public String getImportImplementation() {
		return (String)this.importImplementazioneCombo.getSelectedItem();
	}
	
	/**Returns the Model selected  by the user for the Export operation
	 * 
	 * @return the model (as stored in ModelGen)
	 */
	public String getExportModel(){
		return getValueOnTableModel(2);
	}
	
	/**Returns the Schema selected  by the user for the Export operation
	 * 
	 * @return the Schema (as stored in ModelGen)
	 */
	public String getExportSchema() {
		return getValueOnTableModel(0);
	}
	
	/**Returns the Istance selected  by the user for the Export operation
	 * 
	 * @return the Istance (as stored in ModelGen)
	 */
	public String getExportIstance() {
		return getValueOnTableModel(3);
	}
	
	/**Appends a String to the Import result
	 * 
	 * @param msg the String to be appended(without \n)
	 */
	public void appendImportResult(String msg) {
		Calendar now=Calendar.getInstance();
		String str=new String(now.get(Calendar.HOUR_OF_DAY)+":"+now.get(Calendar.MINUTE)+":"+
				+now.get(Calendar.SECOND)+":"+now.get(Calendar.MILLISECOND)+" "+msg+"\n");
		importText.append(str);
	}
	
	/**Appends a String to the Export result
	 * 
	 * @param msg the String to be appended(without \n)
	 */
	public void appendExportResult(String msg) {
		Calendar now=Calendar.getInstance();
		String str=new String(now.get(Calendar.HOUR_OF_DAY)+":"+now.get(Calendar.MINUTE)+":"+
				+now.get(Calendar.SECOND)+":"+now.get(Calendar.MILLISECOND)+" "+msg+"\n");
		exportText.append(str);
	}
	
	/**Metodo privato richiamato dagli altri
	 * 
	 * @param column
	 */
	private String getValueOnTableModel(int column) {
		int row=exportTable.getSelectedRow();
		if(row!=-1) {
			return ((ExportTableModel)exportTable.getModel()).getValueAt(row,column);
		}
		else 
			return "";
	}
	
	/**Questo metodo inizilizza i componenti pi grandi, non dovrebbe essere toccato.
	 * Crea i due pannelli di import e di export e li aggiunge
	 * 
	 */
	private void initComponents() {
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		setLocation(400,80);
		
		tabs=new JTabbedPane();
		
		importPanel=new JPanel();
		exportPanel=new JPanel();
				
		tabs.addTab("Import", null ,importPanel , "To Import Schemas");
		tabs.addTab("Export", null ,exportPanel , "To Export Schemas");
		
		initImportPanel();
		initExportPanel();
		
		String[] models=Configurator.getInstance().getModels();
		for(int i=0;i<models.length;i++) {
			importModelCombo.addItem(models[i]);
		}		
		/*la lista delle implementazioni viene caricata dal metodo actionPerformed
		 * a seconda del modello scelto*/
		
		this.add(tabs);		
	}
	
	/**Questo metodo serve ad inizializzare tutti i componenti del panel relativo all'import
	 * NON dovete modificare questo metodo, il tutto  reso parametrico, vede sopra o la tesi
	 * */
	private void initImportPanel() {
		importTop = new JPanel();
		importModels = new JPanel();
		importModelCombo = new JComboBox();
		importImplementation = new JPanel();
		importImplementazioneCombo = new JComboBox();
		importStart = new JButton();
		importResult = new JPanel();
		importScrollPanel = new JScrollPane();
		importText = new JTextArea();

		importModels.setBorder(BorderFactory.createTitledBorder("Models list"));
		importModelCombo.addActionListener(this);
		importModelCombo.setToolTipText("Choice a model to Import");

		GroupLayout importModelsLayout = new GroupLayout(importModels);
		importModels.setLayout(importModelsLayout);
		importModelsLayout.setHorizontalGroup(
				importModelsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(GroupLayout.Alignment.TRAILING, importModelsLayout.createSequentialGroup()
						.addContainerGap()
						.addComponent(importModelCombo, 0, 180, Short.MAX_VALUE)
						.addContainerGap())
		);
		importModelsLayout.setVerticalGroup(
				importModelsLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(importModelsLayout.createSequentialGroup()
						.addGap(36, 36, 36)
						.addComponent(importModelCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addContainerGap(62, Short.MAX_VALUE))
		);

		importImplementation.setBorder(BorderFactory.createTitledBorder("Implementations list"));
		//importImplementazioneCombo.setModel(new DefaultComboBoxModel(new String[] { "Scegliere prima il Modello" }));
		importImplementazioneCombo.setToolTipText("Choice an implementation for selected Model");

		importStart.setText("Start");
		importStart.setToolTipText("Start selected Model/Implementation Import");
		importStart.addActionListener(this);

		GroupLayout importImplementationLayout = new GroupLayout(importImplementation);
		importImplementation.setLayout(importImplementationLayout);
		importImplementationLayout.setHorizontalGroup(
				importImplementationLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(importImplementationLayout.createSequentialGroup()
						.addGroup(importImplementationLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addComponent(importImplementazioneCombo, 0, 273, Short.MAX_VALUE)
								.addGroup(importImplementationLayout.createSequentialGroup()
										.addGap(86, 86, 86)
										.addComponent(importStart, GroupLayout.PREFERRED_SIZE, 72, GroupLayout.PREFERRED_SIZE)))
										.addContainerGap())
		);
		importImplementationLayout.setVerticalGroup(
				importImplementationLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(importImplementationLayout.createSequentialGroup()
						.addGap(37, 37, 37)
						.addComponent(importImplementazioneCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
						.addComponent(importStart)
						.addContainerGap())
		);

		GroupLayout importTopLayout = new GroupLayout(importTop);
		importTop.setLayout(importTopLayout);
		importTopLayout.setHorizontalGroup(
				importTopLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(GroupLayout.Alignment.TRAILING, importTopLayout.createSequentialGroup()
						.addComponent(importModels, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(importImplementation, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
		);
		importTopLayout.setVerticalGroup(
				importTopLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(importImplementation, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(importModels, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
		);

		importResult.setBorder(BorderFactory.createTitledBorder("Risultato"));
		importText.setColumns(20);
		importText.setEditable(false);
		importText.setRows(5);
		importScrollPanel.setViewportView(importText);

		GroupLayout importResultLayout = new GroupLayout(importResult);
		importResult.setLayout(importResultLayout);
		importResultLayout.setHorizontalGroup(
				importResultLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(importScrollPanel, GroupLayout.DEFAULT_SIZE, 505, Short.MAX_VALUE)
		);
		importResultLayout.setVerticalGroup(
				importResultLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(importScrollPanel, GroupLayout.DEFAULT_SIZE, 253, Short.MAX_VALUE)
		);

		GroupLayout layout = new GroupLayout(importPanel);
		importPanel.setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(importTop, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(importResult, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
		);
		layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addComponent(importTop, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(importResult, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
		);
	}

	/**Questo metodo serve ad inizializzare tutti i componenti del panel relativo all'export
	 * NON dovete modificare questo metodo, il tutto  reso parametrico, vede sopra o la tesi
	 * */
	private void initExportPanel() {
		exportTop = new JPanel();
		exportTopScroll = new JScrollPane();
		exportTable = new JTable();
		exportStart = new JButton();
		exportReload = new JButton();
		exportResult = new JPanel();
		exportScroll = new JScrollPane();
		exportText = new JTextArea();

		exportTop.setBorder(BorderFactory.createTitledBorder(""));
		exportTopScroll.setBorder(BorderFactory.createTitledBorder("Supermodel's Schemes list"));
		exportTable.setModel(new ExportTableModel());

		exportTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		exportTable.getSelectionModel().addListSelectionListener(this);
		exportTopScroll.setViewportView(exportTable);

		exportStart.setText("Start");
		exportStart.setToolTipText("Start selected Scheme Export");
		exportStart.addActionListener(this);
		exportStart.setEnabled(false);

		exportReload.setText("Reload");
		exportReload.setToolTipText("Reload Schemes list");
		exportReload.addActionListener(this);

		GroupLayout exportTopLayout = new GroupLayout(exportTop);
		exportTop.setLayout(exportTopLayout);
		exportTopLayout.setHorizontalGroup(
				exportTopLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(exportTopLayout.createSequentialGroup()
						.addGroup(exportTopLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addGroup(exportTopLayout.createSequentialGroup()
										.addContainerGap()
										.addComponent(exportTopScroll, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
										.addGroup(exportTopLayout.createSequentialGroup()
												.addGap(130, 130, 130)
												.addComponent(exportStart, GroupLayout.PREFERRED_SIZE, 80, GroupLayout.PREFERRED_SIZE)
												.addGap(71, 71, 71)
												.addComponent(exportReload, GroupLayout.PREFERRED_SIZE, 81, GroupLayout.PREFERRED_SIZE)))
												.addContainerGap(15, Short.MAX_VALUE))
		);
		exportTopLayout.setVerticalGroup(
				exportTopLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(exportTopLayout.createSequentialGroup()
						.addComponent(exportTopScroll, GroupLayout.PREFERRED_SIZE, 126, GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addGroup(exportTopLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(exportStart)
								.addComponent(exportReload)))
		);

		exportResult.setBorder(BorderFactory.createTitledBorder("Export Result"));
		exportText.setColumns(20);
		exportText.setEditable(false);
		exportText.setRows(5);
		exportScroll.setViewportView(exportText);

		GroupLayout exportResultLayout = new GroupLayout(exportResult);
		exportResult.setLayout(exportResultLayout);
		exportResultLayout.setHorizontalGroup(
				exportResultLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(exportScroll, GroupLayout.Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 491, Short.MAX_VALUE)
		);
		exportResultLayout.setVerticalGroup(
				exportResultLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(exportScroll, GroupLayout.DEFAULT_SIZE, 192, Short.MAX_VALUE)
		);

		GroupLayout layout = new GroupLayout(exportPanel);
		exportPanel.setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(exportResult, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(exportTop, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
		);
		layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addComponent(exportTop, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(exportResult, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
		);

    }


}
