package org.modelgen.importexport.components;

import org.modelgen.importexport.ImportExportTool;
import org.modelgen.importexport.controller.ExportController;
import org.modelgen.importexport.dialog.IEDialog;
import org.modelgen.jpf.core.ToolBoxFrame;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.WindowConstants;

/**
 *
 * @author  LordofDeceits
 */
public class ExportXsdParameter extends IEDialog implements ActionListener {
	private static final long serialVersionUID = -2169635414125843028L;
	
	/*componenti logiche*/
	private File xsdSchema;
	private File xmlInstance;
	private String exporterName;
	
	/*componenti grafiche*/
	//parte dello schema
	private JLabel labelSchema;		private JCheckBox checkSchema;
	private JTextField fileSchema;
	
	//parte delle istanze
	private JLabel labelInstance;	private JCheckBox checkInstance;   
	private JTextField fileIstance;
	
	//bottoni
	private JButton start;			private JButton annulla;
	
	/** Creates new form ExportXsdParameter */
    public ExportXsdParameter(String exporterName) {
		this(ToolBoxFrame.getIstance(), "ExportXsdParameter", true);
		this.exporterName=exporterName;
        initComponents();
        this.setLocation(250,200);
	}

	public ExportXsdParameter(Frame owner, String title, boolean modal) {
		super(ToolBoxFrame.getIstance(), title , true);
	}
    
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==checkSchema && checkSchema.isSelected()) {
			JFileChooser fc = new JFileChooser(new File("."));
            
			int returnVal = fc.showSaveDialog(this);
			
			if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = fc.getSelectedFile();
	            if(!file.getName().endsWith(".xsd"))
	            	file=new File(file.getAbsolutePath()+".xsd");
	            fileSchema.setText(file.getName());
	            xsdSchema=file;
	        }
			else
				this.checkSchema.setSelected(false);
		}
		else if(e.getSource()==checkInstance && checkInstance.isSelected()) {
			JFileChooser fc = new JFileChooser(new File("."));

			int returnVal = fc.showSaveDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				if(!file.getName().endsWith(".xml"))
					file=new File(file.getAbsolutePath()+".xml");
				fileIstance.setText(file.getName());
				xmlInstance=file;
			}
		}
		else if(e.getSource()==start) {
			startexport();
		}
		else if(e.getSource()==annulla) {
			this.dispose();
		}		
	}   
    
    private void startexport() {
    	ImportExportTool plugin=ImportExportTool.getIstance();
    	int schemaOID=Integer.parseInt(plugin.getExportSchema());
		//System.out.println("schemaOID :"+schemaOID);
		int instanceOID=-1;
		if(!plugin.getExportIstance().equals("Inesistente") && checkInstance.isSelected())
			instanceOID=Integer.parseInt(plugin.getExportIstance());
		//System.out.println("instanceOID :"+instanceOID);
    	this.dispose();
    	new ExportController(xsdSchema,xmlInstance,schemaOID,instanceOID,exporterName).start();
	}

	private void initComponents() {
        fileSchema = new JTextField();
        labelSchema = new JLabel();
        checkSchema = new JCheckBox();
        labelInstance = new JLabel();
        fileIstance = new JTextField();
        checkInstance = new JCheckBox();
        start = new JButton();
        annulla = new JButton();

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Parametri Export Xsd");
        fileSchema.setEditable(false);
        fileSchema.setToolTipText("Nome del file che conterr\u00e0 lo Schema");

        labelSchema.setText("Esporta lo Schema: ");

        checkSchema.setToolTipText("Selezionare per specificare il File su cui andare a salvare lo Schema");
        checkSchema.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        checkSchema.setMargin(new java.awt.Insets(0, 0, 0, 0));
        checkSchema.addActionListener(this);

        labelInstance.setText("Esporta l'istanza:");

        fileIstance.setEditable(false);
        fileIstance.setToolTipText("nome del File che conterr\u00e0 le istanze");

        checkInstance.setToolTipText("Selezionare per specificare il file su cui andare a salvare le istanze");
        checkInstance.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        checkInstance.setMargin(new java.awt.Insets(0, 0, 0, 0));
        checkInstance.addActionListener(this);
        
        start.setText("Start");
        start.setToolTipText("Cliccare per avviare l'Export");
        start.addActionListener(this);

        annulla.setText("Annulla");
        annulla.setToolTipText("Cliccare per annullare l'Export");
        annulla.addActionListener(this);

        GroupLayout layout = new GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING, false)
                    .addComponent(fileSchema)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(labelSchema)
                        .addGap(54, 54, 54)
                        .addComponent(checkSchema))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(labelInstance)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(checkInstance))
                    .addComponent(fileIstance)
                    .addGroup(GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(start, GroupLayout.PREFERRED_SIZE, 66, GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(annulla)))
                .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(12, 12, 12)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING)
                    .addComponent(checkSchema)
                    .addComponent(labelSchema))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(fileSchema, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                .addGap(23, 23, 23)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addComponent(labelInstance)
                    .addComponent(checkInstance))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(fileIstance, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
                .addGap(35, 35, 35)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                    .addComponent(annulla)
                    .addComponent(start))
                .addContainerGap(49, Short.MAX_VALUE))
        );
        pack();
    }
}

