package org.modelgen.importexport.components;

import org.modelgen.importexport.controller.ImportController;
import org.modelgen.importexport.dialog.IEDialog;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.jpf.core.ToolBoxFrame;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 *
 * @author  LordofDeceits
 */
public class ImportXsdParameter extends IEDialog implements ActionListener{
	private static final long serialVersionUID = -5909429610486400186L;

	//componenti logiche
	private File xsdSchemaFile;
	private List<File> xsdInstanceFiles;

	/*componenti grafiche*/
	//parte dello schema
	private JLabel xsdLabel;	private JTextField xsdSchema;	private JButton xsdSelectionSchema;

	//parte delle istanze
	private JLabel xsdInstanceLabel;
	private JScrollPane scrollPanelInstances;		private JButton xsdInstanceAdd;
	private JList xsdListInstance;				private JButton xsdRemoveInstance;
	private DefaultListModel xsdInstanceModel;

	//parte per far partire l'import o per chiudere   
	private JButton xsdStartImport;		private JButton xsdCancelImport;    

	/** Creates new form ImportXsdParameter */
	public ImportXsdParameter() {
		this(ToolBoxFrame.getIstance(), "Import XSD", true);
		initComponents();
		this.setLocation(250,200);
		this.setVisible(true);
	}

	public ImportXsdParameter(Frame owner, String title, boolean modal) {
		super(ToolBoxFrame.getIstance(), title , true);
	}

	public void actionPerformed(ActionEvent e) {
		/*1)apri file per lo schema*/
		if(e.getSource()==xsdSelectionSchema) {
			JFileChooser fc = new JFileChooser(new File("."));
			FileNameExtensionFilter filter = new FileNameExtensionFilter("Xml Schema", "xsd");
			fc.setFileFilter(filter);

			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				xsdSchema.setText(file.getName());
				xsdSchemaFile=file;
			}
		}
		/*2)apri file per file Xml*/
		else if(e.getSource()==xsdInstanceAdd) {
			JFileChooser fc = new JFileChooser(new File("."));
			FileNameExtensionFilter filter = new FileNameExtensionFilter("Xml file", "xml");
			fc.setFileFilter(filter);

			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				xsdInstanceModel.addElement(file.getName());
				xsdInstanceFiles.add(file);
			}
		}
		/*3)dopo aver selezionato una riga delle istanze*/
		else if(e.getSource()==xsdRemoveInstance) {
			int selected=xsdListInstance.getSelectedIndex();
			if(selected!=-1 && selected>=0 && selected<=xsdInstanceFiles.size()) {
				xsdInstanceModel.remove(selected);
				xsdInstanceFiles.remove(selected);
			}
		}
		/*4)faccio partire l'import*/
		else if(e.getSource()==xsdStartImport) {
			if(xsdSchemaFile!=null)
				startImport();
			else
				JOptionPane.showMessageDialog(this,"Specificare almeno lo schema","Errore",
						JOptionPane.ERROR_MESSAGE);
		}
		/*5)chiudo*/
		else if(e.getSource()==xsdCancelImport)
			this.dispose();

	}

	private void startImport() {
		try {
			this.dispose();
			new ImportController(xsdSchemaFile,xsdInstanceFiles).start();
		} catch (PersistenceException e) {
			JOptionPane.showMessageDialog(this,"Errore nella persistenza\n"+e.getMessage(),
					"Errore nella persistenza",JOptionPane.ERROR_MESSAGE);
		}		
	}

	private void initComponents() {
		xsdSchema = new JTextField();
		xsdLabel = new JLabel();
		xsdSelectionSchema = new JButton();
		scrollPanelInstances = new JScrollPane();
		xsdListInstance = new JList();
		xsdInstanceLabel = new JLabel();
		xsdInstanceAdd = new JButton();
		xsdStartImport = new JButton();
		xsdCancelImport = new JButton();
		xsdRemoveInstance = new JButton();
		xsdInstanceModel= new DefaultListModel();
		xsdSchemaFile=null; 
		xsdInstanceFiles=new ArrayList<File>();
		xsdSelectionSchema.addActionListener(this);
		xsdInstanceAdd.addActionListener(this);
		xsdRemoveInstance.addActionListener(this);
		xsdStartImport.addActionListener(this);
		xsdCancelImport.addActionListener(this);

		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setTitle("Parametri Xsd");

		xsdLabel.setText("File contenente lo Schema Xsd:");

		xsdSelectionSchema.setText("Apri");

		scrollPanelInstances.setToolTipText("Lista delle istanze collegate allo Schema");
		xsdListInstance.setToolTipText("Lista delle Istanze collegate allo Schema");
		xsdListInstance.setModel(xsdInstanceModel);
		scrollPanelInstances.setViewportView(xsdListInstance);

		xsdInstanceLabel.setText("File contenenti le istanze dello schema:");

		xsdInstanceAdd.setText("Aggiungi");

		xsdStartImport.setText("Start");

		xsdCancelImport.setText("Annulla");

		xsdRemoveInstance.setText("Togli");

		GroupLayout layout = new GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup( GroupLayout.Alignment.LEADING)
								.addGroup(layout.createSequentialGroup()
										.addGap(43, 43, 43)
										.addComponent(xsdStartImport,  GroupLayout.PREFERRED_SIZE, 74,  GroupLayout.PREFERRED_SIZE)
										.addGap(28, 28, 28)
										.addComponent(xsdCancelImport,  GroupLayout.PREFERRED_SIZE, 77,  GroupLayout.PREFERRED_SIZE))
										.addGroup(layout.createSequentialGroup()
												.addContainerGap()
												.addGroup(layout.createParallelGroup( GroupLayout.Alignment.TRAILING)
														.addComponent(scrollPanelInstances,  GroupLayout.Alignment.LEADING,  GroupLayout.DEFAULT_SIZE, 244, Short.MAX_VALUE)
														.addComponent(xsdSchema,  GroupLayout.DEFAULT_SIZE, 244, Short.MAX_VALUE))
														.addPreferredGap( LayoutStyle.ComponentPlacement.RELATED)
														.addGroup(layout.createParallelGroup( GroupLayout.Alignment.TRAILING, false)
																.addComponent(xsdRemoveInstance,  GroupLayout.DEFAULT_SIZE,  GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
																.addComponent(xsdInstanceAdd,  GroupLayout.Alignment.LEADING,  GroupLayout.DEFAULT_SIZE,  GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
																.addComponent(xsdSelectionSchema,  GroupLayout.Alignment.LEADING,  GroupLayout.DEFAULT_SIZE, 79, Short.MAX_VALUE)))
																.addGroup(layout.createSequentialGroup()
																		.addContainerGap()
																		.addComponent(xsdInstanceLabel))
																		.addGroup(layout.createSequentialGroup()
																				.addContainerGap()
																				.addComponent(xsdLabel)))
																				.addContainerGap())
		);
		layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addGap(12, 12, 12)
						.addComponent(xsdLabel)
						.addPreferredGap( LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(xsdSchema, GroupLayout.PREFERRED_SIZE, 20, GroupLayout.PREFERRED_SIZE)
								.addComponent(xsdSelectionSchema))
								.addGap(21, 21, 21)
								.addComponent(xsdInstanceLabel, GroupLayout.PREFERRED_SIZE, 14, GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
										.addComponent(scrollPanelInstances, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
										.addGroup(layout.createSequentialGroup()
												.addComponent(xsdInstanceAdd)
												.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
												.addComponent(xsdRemoveInstance)))
												.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, 19, Short.MAX_VALUE)
												.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
														.addComponent(xsdStartImport)
														.addComponent(xsdCancelImport))
														.addContainerGap())
		);
		pack();
	}     
}

