package org.modelgen.importexport.controller;

import org.modelgen.importexport.ImportExportTool;
import org.modelgen.importexport.Export.DB2OOExporter;
import org.modelgen.importexport.Export.DB2ORExporter;
import org.modelgen.importexport.Export.DB2RelationalExporter;
import org.modelgen.importexport.Export.Destination;
import org.modelgen.importexport.Export.Exporter;
import org.modelgen.importexport.Export.XMLSchemaExporter;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.or.model.ORSchema;
import org.modelgen.importexport.persistence.ConnectionFactory;
import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.XmlSchemaSourceDestination;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.io.File;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

public class ExportController extends Thread {
	private Destination destination;
	private Exporter exporter;
	private int schemaOID;
	private int instanceOID;
	private String exporterName;
	
	@SuppressWarnings("unchecked")
	public ExportController(String exporterName,String dbName,String server,int port,String user,String password,int schemaOID,int instanceOID) {
		try {			
			Connection con=ConnectionFactory.getConnection(ConnectionFactory.DB2,dbName,server,port,user,password,"").getConnection();
			
			try {
				con.setAutoCommit(false);
			} catch (SQLException e) {
			}
			
			destination=new Destination<Connection>(con);
			
			this.schemaOID=schemaOID;
			this.instanceOID=instanceOID;
			this.exporterName=exporterName;
						
			if(exporterName.equals("DB2RelationalExporter")) {	
				exporter=new DB2RelationalExporter();
			}
			else if(exporterName.equals("DB2OOExporter")) {
				exporter=new DB2OOExporter();
			}
			else if(exporterName.equals("DB2ORExporter")) {
				exporter=new DB2ORExporter();
			}
		} catch (PersistenceException e) {
			JOptionPane.showMessageDialog(ImportExportTool.getIstance(),
				    "Errore nella persistenza:\n"+e.getMessage(),
				    "Errore nella persistenza",JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public ExportController(File schema,File instance,int schemaOID,int instanceOID,String exporterName) {
		if(exporterName.equals("XMLSchemaExporter")) {
			
			List<File> instances=new ArrayList<File>();
			instances.add(instance);
			destination=new Destination<XmlSchemaSourceDestination>(new XmlSchemaSourceDestination(schema,instances));
			
			this.schemaOID=schemaOID;
			this.instanceOID=instanceOID;
			this.exporterName=exporterName;
			
			exporter=new XMLSchemaExporter();
		}
	}
	
	@SuppressWarnings("unchecked")
	public void run() {
		ImportExportTool ui=ImportExportTool.getIstance();
		try {
			if(exporterName.equals("DB2RelationalExporter")) {	
				RelationalSchema rel=ImportExportPersistence.getInstance().loadSchema(schemaOID);
				ui.appendExportResult("Caricate lo schema dal db di ModelGen");
				if(instanceOID!=-1) {
					ImportExportPersistence.getInstance().loadInstance(rel, instanceOID);
					ui.appendExportResult("Caricate le istanze dal db di ModelGen");
				}
				exporter.exportSchema(rel, destination);
				ui.appendExportResult("Esportato lo schema verso la destinazione");
				if(instanceOID!=-1) {
					exporter.exportInstance(rel, destination);
					ui.appendExportResult("Esportate le istanze verso la destinazione");
				}
				
				((Connection)destination.getDestination()).rollback();
				
			}
			else if(exporterName.equals("DB2OOExporter")) {
				Exporter<OOSchema,Destination<Connection>> exporter=new DB2OOExporter();
				
				OOSchema ooSchema=ImportExportPersistence.getInstance().loadOOSchema(schemaOID);
				ui.appendExportResult("Caricate lo schema dal db di ModelGen");
				if(instanceOID!=-1) {
					ImportExportPersistence.getInstance().loadOOInstance(ooSchema, instanceOID);
					ui.appendExportResult("Caricate le istanze dal db di ModelGen");
				}
				exporter.exportSchema(ooSchema, destination);
				ui.appendExportResult("Esportato lo schema verso la destinazione");
				if(instanceOID!=-1) {
					exporter.exportInstance(ooSchema, destination);
					ui.appendExportResult("Esportate le istanze verso la destinazione");
				}
				
				((Connection)destination.getDestination()).rollback();
				
			}
			else if(exporterName.equals("DB2ORExporter")) {
				Exporter<ORSchema,Destination<Connection>> exporter=new DB2ORExporter();
				
				ORSchema orSchema=ImportExportPersistence.getInstance().loadORSchema(schemaOID);
				ui.appendExportResult("Caricate lo schema dal db di ModelGen");
				if(instanceOID!=-1) {
					ImportExportPersistence.getInstance().loadORInstance(orSchema, instanceOID);
					ui.appendExportResult("Caricate le istanze dal db di ModelGen");
				}
				//System.out.println(orSchema.toString());
				exporter.exportSchema(orSchema, destination);
				ui.appendExportResult("Esportato lo schema verso la destinazione");
				if(instanceOID!=-1) {
					exporter.exportInstance(orSchema, destination);
					ui.appendExportResult("Esportate le istanze verso la destinazione");
				}
				
				((Connection)destination.getDestination()).rollback();
				
			}
			else if(exporterName.equals("XMLSchemaExporter")) {
				XmlSchema xmlSchema=ImportExportPersistence.getInstance().loadXMLSchema(schemaOID);
				//System.out.println(xmlSchema.toString());
				ui.appendExportResult("Caricato lo schema dal db di ModelGen");
				
				if(instanceOID!=-1) {
					//ancora da implementare
					xmlSchema=ImportExportPersistence.getInstance().loadXmlInstance(xmlSchema, instanceOID);
					ui.appendExportResult("Caricate le istanze dal db di ModelGen");
					System.out.println(xmlSchema.toString());
				}
				exporter.exportSchema(xmlSchema, destination);
				ui.appendExportResult("Esportato lo schema verso la destinazione");
				if(instanceOID!=-1) {
					exporter.exportInstance(xmlSchema, destination);
					ui.appendExportResult("Esportate le istanze verso la destinazione");
				}
			}
		} catch (PersistenceException e) {
			JOptionPane.showMessageDialog(ImportExportTool.getIstance(),
				    "Errore nella persistenza:\nNel caso di Export su db assicurarsi " +
				    "che il db sia\"pulito\"\n"+e.getMessage(),
				    "Errore nella persistenza",JOptionPane.ERROR_MESSAGE);
		} catch (ImportExportException e) {
			JOptionPane.showMessageDialog(ImportExportTool.getIstance(),
				    "Errore nell'Export:\nSi tratta di un errore logico sui dati\n"+e.getMessage(),
				    "Errore nell'Export",JOptionPane.ERROR_MESSAGE);
		} 
		
		/*questo  da cancellare... l'eccezione sul roolback*/
		catch (SQLException e) {
		}
		
	}
}
