package org.modelgen.importexport.controller;

import org.modelgen.importexport.ImportExportTool;
import org.modelgen.importexport.Import.DB2OOImporter;
import org.modelgen.importexport.Import.DB2ORImporter;
import org.modelgen.importexport.Import.Importer;
import org.modelgen.importexport.Import.JDBCRelational;
import org.modelgen.importexport.Import.Source;
import org.modelgen.importexport.Import.XMLSchemaImporter;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.or.model.ORSchema;
import org.modelgen.importexport.persistence.ConnectionFactory;
import org.modelgen.importexport.persistence.DbConnection;
import org.modelgen.importexport.persistence.ImportExportPersistence;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.utilities.Schema;
import org.modelgen.importexport.utilities.XmlSchemaSourceDestination;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.io.File;
import java.util.List;

import javax.swing.JOptionPane;

/**It is the controller for each Import operation.
 * 
 * @version 0.9
 */
public class ImportController extends Thread{
	private Source source;
	private Importer importer;
	private boolean instance;
	
	public ImportController(String dbName,String server,int port,String user,String password,
			String schema_name,boolean instance) throws PersistenceException {	
		ImportExportTool ui=ImportExportTool.getIstance();
		
		DbConnection con;
		if(ui.getImportImplementation().equals("DB2 v.9 from Connection")) {
			con = ConnectionFactory.getConnection(ConnectionFactory.DB2,dbName,server,port,user,password,schema_name);
		}
		else 
			throw new PersistenceException("Non  stata definita il tipo di connessione in ImportController");
		
		source=new Source<DbConnection>(con);
		
		if(ui.getImportModel().equals("Relazionale") && ui.getImportImplementation().equals("DB2 v.9 from Connection")) {
			importer=new JDBCRelational();
		}
		else if(ui.getImportModel().equals("Object Oriented") && ui.getImportImplementation().equals("DB2 v.9 from Connection")) {
			importer=new DB2OOImporter();
		}
		else if(ui.getImportModel().equals("Object Relational") && ui.getImportImplementation().equals("DB2 v.9 from Connection")) {
			importer=new DB2ORImporter();
		}
		else
			throw new PersistenceException("Non  stato definito quele Importer utilizzare in ImportController");
		
		this.instance=instance;
	}
	
	public ImportController(File schema,List<File> instances) throws PersistenceException {
		source=new Source<XmlSchemaSourceDestination>(new XmlSchemaSourceDestination(schema,instances));
		importer=new XMLSchemaImporter();
		
		if(instances.size()>0)
			this.instance=true;
		else
			this.instance=false;
	}
	
	@SuppressWarnings("unchecked")
	public void run() {
		ImportExportPersistence singleton=ImportExportPersistence.getInstance();
		ImportExportTool ui=ImportExportTool.getIstance();		
		
		Schema schema;
		try {
			schema = (Schema)importer.importSchema(source);
			ui.appendImportResult("Caricata la struttura ad oggetti[Schema]");
			if(instance) {
				schema=(Schema)importer.importInstance(schema, source);
				ui.appendImportResult("Caricata la struttura ad oggetti[Istanze]");
			}
			/*purtroppo sono costretto a fare questa schifezza per mancanza di tempo*/
			if (schema instanceof RelationalSchema) {
				singleton.storeSchema((RelationalSchema)schema);
				ui.appendImportResult("Salvata sul database la struttura ad oggetti[Schema]");
				if(instance) {
					singleton.storeInstance((RelationalSchema)schema);
					ui.appendImportResult("Salvata sul database la struttura ad oggetti[Istanze]");
				}
			}
			else if(schema instanceof OOSchema) {
				singleton.storeSchema((OOSchema)schema);
				ui.appendImportResult("Salvata sul database la struttura ad oggetti[Schema]");
				if(instance) {
					singleton.storeInstance((OOSchema)schema);
					ui.appendImportResult("Salvata sul database la struttura ad oggetti[Istanze]");
				}
			}
			else if(schema instanceof ORSchema) {
				singleton.storeSchema((ORSchema)schema);
				ui.appendImportResult("Salvata sul database la struttura ad oggetti[Schema]");
				if(instance) {
					singleton.storeInstance((ORSchema)schema);
					//System.out.println(schema.toString());
					ui.appendImportResult("Salvata sul database la struttura ad oggetti[Istanze]");
				}
			}
			else if(schema instanceof XmlSchema) {
				System.out.println(schema.toString());
				singleton.storeSchema((XmlSchema)schema);
				ui.appendImportResult("Salvata sul database la struttura ad oggetti[Schema]");
				if(instance) {
					singleton.storeInstance((XmlSchema)schema);
					ui.appendImportResult("Salvata sul database la struttura ad oggetti[Istanze]");
				}
			}
			/*qui vanno aggiunti altri schemi*/

			ui.appendImportResult("Schema OID: "+schema.getOID());
			
		} catch (PersistenceException e) {
			JOptionPane.showMessageDialog(ui,"Errore nella persistenza\n"+e.getMessage(),"Errore nella persistenza",JOptionPane.ERROR_MESSAGE);
		} catch (ImportExportException e) {
			JOptionPane.showMessageDialog(ui,"Errore di import\n"+e.getMessage(),"Errore di import",JOptionPane.ERROR_MESSAGE);
		}
	}	
}
