package org.modelgen.importexport.oo.model;

/**Field it a lexical value of a OOCLass or a ComplexType.
 * A Field can be created by the SQL command
 * <code>
 * creaty type([Field] varchar(50),[Field] integer)
 * </code>
 * A Field is a lexical value, hence it can NOT be a ComplexType(e.g create type(domicilio indirizzo)).
 * 
 * @version 0.9
 */
public class Field {
	private int oid;
	private String name;
	//field type: it should be a valid SQL type
	private String type;
	//allows for null values
	private boolean isNullable;
	//is part of the primary key
	private boolean isIdentifier;
	
	/**Create a new Field with the given parameters
	 * @param oid OID as stored in ModelGen
	 * @param name name as stored in ModelGen
	 * @param type type as stored in ModelGen
	 * @param isNullable true if corrispondeing OOObject can have a Null value
	 * @param isIdentifier true if the Field belongs to the Primary Key
	 */
	public Field(int oid, String name, String type, boolean isNullable, boolean isIdentifier) {
		this.oid = oid;
		this.name = name;
		this.type = type;
		this.isNullable = isNullable;
		this.isIdentifier = isIdentifier;
	}
	
	public boolean isIdentifier() {
		return isIdentifier;
	}

	public boolean isNullable() {
		return isNullable;
	}

	public String getType() {
		return type;
	}

	public String getName() {
		return name;
	}

	public int getOid() {
		return oid;
	}
	
	/**Returns a String representation of this Field
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "oid: " + oid + " name: " + name 
			+ " type: " + type + " isNullable: " + isNullable +" isIdentifier: " + isIdentifier; 
	}
}
