

package org.modelgen.importexport.oo.model;

import java.util.Vector;

/**An OOClass corrispond to a typed table.
 * In particular we consider as legal legal OOClass the typed table that are created
 * by defining a type, with a number of lexical Field and Reference to other OOClass
 * Hence is not allowed to create a hierachy between type that are not strictly related
 * to a table
 * 
 * For example it correct:
 * 
 * create type Address (....) * 
 * create table Addresses of Addres (...)
 * 
 * It NOT correct:
 * create type Man
 * create type Manager Under Uomo
 * create type employee Under Uomo
 * 
 * create table Managers
 * create table Employees 
 * 
 * For more information see thesis 
 * 
 * @version 0.9
 *
 */
public class OOClass {
	// table oid
	protected int oid;
	// table name
	protected String name;
	// columns
	protected Vector<Field> fields;
	// reference
	protected Vector<Reference> references;
	// link to the instances of this relation (table schema)
	private Vector<OOObject> objects;	
	//Type
	private String type;	//used only for import
	
	/**Create a new OOClass with the given parameters
	 * @param oid
	 * @param name
	 */
	public OOClass(int oid, String name) {
		this.oid = oid;
		this.name = name;
		this.fields = new Vector<Field>();
		this.references = new Vector<Reference>();
		this.objects = new Vector<OOObject>();
		this.type="";
	}

	public String getName() {
		return name;
	}

	public int getOid() {
		return oid;
	}

	public Vector<Field> getFields() {
		return fields;
	}

	public void addField(Field c) {
		fields.add(c);
	}
	
	public void addReference(Reference r) {
		references.add(r);
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public void setFields(Vector<Field> fields) {
		this.fields = fields;
	}

	public void setReferences(Vector<Reference> references) {
		this.references = references;
	}
	
	public void addOOReference(Reference ooReference) {
		references.add(ooReference);
	}

	public Vector<Reference> getOOReferences() {
		return references;
	}

	/**Return a Field identified by its OID
	 * @param ooFieldOID the OID of the Field
	 * @return the Field if it exists null othrwise
	 */
	public Field getOOFieldByOID(int ooFieldOID) {
		for (Field ooField : fields) {
			if (ooField.getOid() == ooFieldOID)
				return ooField;
		}
		return null;
	}
	
	/**Return a Reference identified by its OID
	 * @param ooReferenceOID the OID of the Reference
	 * @return the Reference if it exists null othrwise
	 */
	public Reference getOOReferenceByOID(int ooReferenceOID) {
		for (Reference ooReference : references) {
			if (ooReference.getOid() == ooReferenceOID)
				return ooReference;
		}
		return null;
	}

	public void addOOObject(OOObject ooObject) {
		objects.add(ooObject);
	}

	public Vector<OOObject> getOOObjects() {
		return objects;
	}
	
	/**Return a OOObject identified by its OID
	 * @param OID the OID of the OOObject
	 * @return the OOObject if it exists null otherwise
	 */
	public OOObject getOOObjectByOID(int OID) {
		for(OOObject o:this.objects) {
			if(o.getObjectOID()==OID)
				return o;
		}
		return null;
	}

	public Vector<Reference> getReferences() {
		return references;
	}
	
	/**Returns a String representation of ONLY the Schema
	 * @return a String representation of this OOClass
	 */
	protected String schemaToString() {
		StringBuilder sb = new StringBuilder("OOClass: " + name+" oid "+this.oid);
		String newline = System.getProperty("line.separator");
	
		if(fields.size()>0) {
			sb.append(newline);
			sb.append("\tFields:");
			sb.append(newline);
			
			for (Field field : fields) {
				sb.append("\t");
				sb.append(field);
				sb.append(newline);
			}
		}
		else {
			sb.append(newline);
			sb.append("\tNO Field");
			sb.append(newline);
		}
		
		if(references.size()>0) {
			sb.append(newline);
			sb.append("\tReferences:");
			sb.append(newline);
			for(Reference ooR: references ) {
				sb.append("\t");
				sb.append(ooR);
				sb.append(newline);
			}
		}
		else{
			sb.append(newline);
			sb.append("\tNO Reference");
			sb.append(newline);
		} 	
		
		return sb.toString();
	}
	
	/** Returns a String representation of the OOClass
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder(this.schemaToString());
		String newline = System.getProperty("line.separator");
	
		if(objects.size()>0) {
			sb.append(newline);
			sb.append("\tInstances" + newline);
			for(OOObject obj:objects){
				sb.append(obj.toString());
			}
		}
		else{
			sb.append(newline);
			sb.append("\tNO Instance");
			sb.append(newline);
		} 

		return sb.toString();
	}
}
