package org.modelgen.importexport.oo.model;

import java.util.Vector;

/**Represents a generalization between a number of OOClasses.
 * Each OOClass can belong to a genralization, which means that it can not exist
 * on his own but need a parent.
 * 
 *  For example:
 *  <code>
 *  create type uomo (..)
 *  create type manager under uomo(..)
 *  </code>
 * 
 * @version 0.9
 */
public class OOGeneralization {
	//OID
	private int OID;
	//name
	private String name;
	//a generalization id total when all the object of the childs are also object of the parent
	private boolean isTotal;
	//parent of this generalization
	private OOClass parent;
	//childs of this generalization
	private Vector<OOChild> childs;
	
	/**Create a Generalization with the given parameter
	 * @param oid OID as stored in ModelGen
	 * @param name name as stored in ModelGen
	 * @param isTotal boolean see above
	 * @param parent OOClass parent of this generalization
	 */
	public OOGeneralization(int oid, String name, boolean isTotal, OOClass parent) {
		OID = oid;
		this.name = name;
		this.isTotal = isTotal;
		this.parent = parent;
		this.childs=new Vector<OOChild>();
	}
	
	public int getOID() {
		return OID;
	}
	
	public String getName() {
		return name;
	}
	
	public boolean isTotal() {
		return isTotal;
	}
	
	public OOClass getParent() {
		return this.parent;
	}
	
	/**Add a child to this Generalization
	 * @param child OOChild child to be added
	 */
	public void addChild(OOChild child) {
		this.childs.add(child);		
	}
	
	/**Returns all the OOClass child of this generalization
	 */
	public Vector<OOClass> getOOChilds() {
		Vector<OOClass> result=new Vector<OOClass>();
		for(OOChild child:childs) {
			result.add(child.getChild());
		}
		return result;
	}
	
	/**Returns all the OOChild of this Generalization
	 */
	public Vector<OOChild> getChilds() {
		return childs;
	}

	/**Returns the OOChild which contains the given OOClass
	 * @return the OOChild which contains the given OOClass; null otherise
	 */
	public OOChild getChild(OOClass ooChild) {
		for(OOChild child:childs) {
			if(child.getChild()==ooChild)
				return child;
		}
		return null;
	}

	/**Return a String representation of this Generalization
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("OID: ");
		sb.append(OID);
		sb.append(" name:");
		sb.append(name);
		sb.append(" isTotal:");
		sb.append(isTotal);
		sb.append(" parent:");
		sb.append(parent.getName());
		
		sb.append("\n\t");
		for(OOChild o:childs) {
			sb.append(" child:(");
			sb.append(o.getOID());
			sb.append(",");
			sb.append(o.getChild().getName());
			sb.append(")");
		}
		sb.append("\n");
		
		return sb.toString();
	}

}
