package org.modelgen.importexport.oo.model;

import java.util.Vector;

/**Represents a tupla in a typed table.
 * A typed table(OOClass) has a number of tuple(OOObject) that represented by this class.
 * 
 * <code>
 * select * from persone
 * OID  Name .... Addres
 * 1	Leo       indirizzo(1)
 * 2	Ale		  indirizzo(3)
 * </code>
 * As we can see an OOObject has ALREADY an integer OID, which means that i do NOT have to load a
 * new OID from ModelGen
 * 
 * Attenzione: questo  l'unico oggetto che ha questa eccezione, per il motivo vedere la tesi
 * In poche parole se gli oggetti sono messi in una Generalizzazione abbiamo due oggetti con lo stesso
 * OID(DB2 li salva come se fosse uno soltanto); questo non  un male ma un bene perch questa 
 * uguaglianza mi serve per cercare gli oggetti con lo stesso OID per poterli poi inserire come istanze
 * della generalizzazione in ModelGen
 * 
 * 
 */
public class OOObject {
	//OID
	protected int objectOID;
	//OOClass this OOObject belongs
	private OOClass ooClass;
	//list of all values
	protected Vector<ValueComponent> valueComponents;
	//list of all references
	protected Vector<ReferenceComponent> referenceComponents;

	/**Create a new OOObejct with the given parameters
	 * @param objectOID OID (attenzione: non univoco in ModelGen)
	 * @param ooClass OOClass this object belongs
	 */
	public OOObject(int objectOID, OOClass ooClass) {
		this.objectOID = objectOID;
		this.ooClass = ooClass;
		this.valueComponents = new Vector<ValueComponent>();
		this.referenceComponents = new Vector<ReferenceComponent>();
	}
	
	public void addValueComponent(ValueComponent ooObjectValueComponent) {
		valueComponents.add(ooObjectValueComponent);
	}
	
	public void addReferenceComponent(ReferenceComponent ooObjectReferenceComponent) {
		referenceComponents.add(ooObjectReferenceComponent);
	}

	public Vector<ValueComponent> getValueComponents() {
		return valueComponents;
	}
	
	public Vector<ReferenceComponent> getReferenceComponents() {
		return referenceComponents;
	}

	public OOClass getOoClass() {
		return ooClass;
	}

	public int getObjectOID() {
		return objectOID;
	}

	public void setOID(int oid) {
		 this.objectOID = oid;
	}
	
	/**Return true if two OOObject are equal
	 * @param o the other OOObject
	 * @return true if the OID of the two object is equal
	 */
	public boolean equals(OOObject o){
		if(this.objectOID==o.getObjectOID())
			return true;
		else
			return false;		
	}
	
	/**Returns a String representation of this object
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("\tObject oid :");
		sb.append(objectOID);
		sb.append("\n");
		
		for(ValueComponent component:valueComponents) {
			sb.append(component.toString());
		}
		
		
		for(ReferenceComponent reference: referenceComponents) {
			sb.append(reference.toString());
		}
		
		return sb.toString();
	}
}
