package org.modelgen.importexport.oo.model;

import org.modelgen.importexport.utilities.Schema;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**It represents a Object Oriented Schema.
 * A OO schema is composed of a number of OOClass; each Class has a number of
 * Field and Reference to other Class.
 * Each Class can also be part of a Generalization
 * 
 * The instances of a OOClass are OOObject.
 * In DB2 each OOObject has a self refering column, which is used for the Refence(see thesis for
 * other detail); this Column is used to create Object and to link the to other Object thanks
 * to Refences
 * 
 *  Attenzione: contrariamente a quanto succede con il resto di questo plug-in gli OID non saranno 
 *  univoci (cio non servono univoci:) ) perch gli verr assegnato un OID univoco nel db di ModelGen
 *  al momento dello store delle istances.
 *  
 *  @version 0.9
 *
 */
public class OOSchema extends Schema{
	//classes belonging to the schema.
	private Vector<OOClass> ooClasses;
	//the generalizations of the schema
	private Vector<OOGeneralization> generalizations;
	//list of objectGeneralization
	private Vector<ObjectGeneralization> objectGeneralization;
	//the OOClass ordered in a hierachy
	private List<OOClass> ordered;
	
	
	/**Creates a new OOSchema with the given parameters 
	 * @param OID the SCHEMEOID as stored in ModelGen
	 * @param name the name of the schema
	 */
	public OOSchema(int OID,String name) {
		super(OID,name);
		this.ooClasses = new Vector<OOClass>();
		this.generalizations=new Vector<OOGeneralization>();
		this.ordered=new ArrayList<OOClass>();
		this.objectGeneralization=new Vector<ObjectGeneralization>();
	}
	
	public int getOID() {
		return OID;
	}

	public Vector<OOClass> getOOClasses() {
		return ooClasses;
	}
	
	public void addOOClass(OOClass ooClass) {
		ooClasses.add(ooClass);
		return;
	}
	
	/**Return true if the given OOClass is a parent in any generalization
	 * @param o the OOCLass to check
	 * @return	true if it is a parent
	 */
	public boolean checkGeneralization(OOClass o) {
		for(OOGeneralization gen:generalizations) {
			if(gen.getParent()==o)
				return true;
		}
		return false;
	}
	
	/**Return the parent of the given OOCLass
	 * @param o the OOCLass to check
	 * @return	its parent if it exist null otherwise
	 */
	public OOClass getParent(OOClass o) {
		for(OOGeneralization gen : generalizations) {
			if(gen.getOOChilds().contains(o))
				return gen.getParent();
		}		
		return null;
	}
	
	/**Return the OOGeneralization to which the given OOClass is the parent
	 * @param parent the parent of the OOGeneralization
	 * @return the OOgenerlization if exists null otherwise
	 */
	public OOGeneralization getGeneralization(OOClass parent) {
		for(OOGeneralization gen:generalizations) {
			if(gen.getParent()==parent) {
				return gen;
			}
		}
		return null;
	}
	
	public Vector<OOGeneralization> getGeneralizations() {
		return generalizations;
	}
	
	public void setGeneralizations(Vector<OOGeneralization> generalizations) {
		this.generalizations = generalizations;
	}

	public void setObjectGeneralization(Vector<ObjectGeneralization> objectGeneralization) {
		this.objectGeneralization = objectGeneralization;
	}

	/**Returns a OOGeneralization identified by its OID
	 * @param OID the OID to look for
	 * @return the Gen if it exists, null othrwise
	 */
	public OOGeneralization getOOGeneralizationByOID(int OID) {
		for(OOGeneralization gen:generalizations) {
			if(gen.getOID()==OID)
				return gen;
		}
		return null;
	}

	public void addOOGeneralization(OOGeneralization o) {
		this.generalizations.add(o);
	}
	
	/**Check if a OOclass belongs to a OOGeneralization both as a parent or child
	 * @param o the OOCLass to check
	 * @return true if belongs false otherwise
	 */
	public boolean isImplicated(OOClass o) {
		for(OOGeneralization gen : generalizations){
			if(gen.getParent()==o)
				return true;
			else {
				for(OOClass child:gen.getOOChilds())
					if(child==o)
						return true;
			}
		}
		return false;
	}
	
	/**Return a Vector which contains the Child of a Parent
	 * @param o the Parent
	 * @return Vector if o if truly a parent null otherwise
	 */
	public Vector<OOClass> getChilds(OOClass o) {
		for(OOGeneralization gen:generalizations) {
			if(o==gen.getParent()){
				return gen.getOOChilds();
			}
		}
		return null;
	}

	public boolean addObjectGeneralization(ObjectGeneralization objGen) {
		return objectGeneralization.add(objGen);
	}

	
	public Vector<ObjectGeneralization> getObjectGeneralization() {
		return objectGeneralization;
	}

	/**Generate the Hierachy of the OOClasses
	 * Each OOClass to be exported needs that both all of its References a OOgeneralization 
	 * are satisfied; each Reference points to another OOClass, which means that this 
	 * OOClass must be loaded earlier.
	 * A OOGeneralization means tha each child can be loaded only after all its parent
	 * 
	 *  For more info see thesis
	 * 
	 * 
	 * @return an ordered List od OOClass
	 */
	public List<OOClass> generateHierachy() {
		if(this.ordered.size()==0){
			List<OOClass> ordered=new ArrayList<OOClass>();
			
			Vector<OOClass> temp=new Vector<OOClass>();
			temp.addAll(this.ooClasses);		
			
			while(temp.size()>0) {
				Iterator<OOClass> i=temp.iterator();
				
				while(i.hasNext()){
					OOClass current=i.next();
					OOClass parent=this.getParent(current);
					
					if(parent==null || ordered.contains(parent)) {
						/*se non ha un padre o la lista ordinata gi contiene il padre
						 * devo verificare che tutte le Ref puntino a OOClass gi ordinate*/
						boolean toInsert=true;
						
						for(Reference ref:current.getReferences()) {
							if(!ordered.contains(ref.getClassTo()))
								toInsert=false;
						}						
						if(toInsert) {
							ordered.add(current);
							i.remove();
						}
					}			
				}
			}
			/*nel momento che ho tolto anche l'ultimo elemento dalla lista temporanea ho finito*/
			this.ordered=ordered;			
			return this.ordered;
		}
		else
			return this.ordered;
	}
	
	/**Return a OOClass identified by its OID
	 * @param ooClassOID the OID to search
	 * @return the OOClass if it exists null otherwise
	 */
	public OOClass getClassByOID(int ooClassOID) {
		for (OOClass ooClass : ooClasses) {
			if (ooClass.getOid() == ooClassOID) return ooClass;
		}
		return null;
	}
	
	/**Return a OOClass identified by its name
	 * @param name the name of the OOClass
	 * @return the OOClass if it exists null otherwise
	 */
	public OOClass getClassByName(String name) {
		for (OOClass ooClass : ooClasses) {
			if (ooClass.getName().equalsIgnoreCase(name)) 
				return ooClass;
		}
		return null;
	}
	
	/**
	 * @param ooClassInstanceToOID
	 */
	public OOObject getClassInstanceByOID(int ooClassInstanceToOID) {
		 for (OOClass ooClass : ooClasses) {
			for (OOObject ooObject : ooClass.getOOObjects()) {
				if (ooObject.getObjectOID() == ooClassInstanceToOID)
					return ooObject;
			}
		}
		return null;
	}
	
	/**Return a String representation of the Schema
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder("schemaName: " + this.name);
		sb.append("\n");
		
		for (OOClass ooClass : generateHierachy()) {
			sb.append(ooClass);
			sb.append("\n");
		}
	
		//generalization
		sb.append("\nGeneralizzazioni:\n\n");
		for(OOGeneralization gen:generalizations) {
			sb.append(gen.toString());	
			sb.append("\n");
		}
		if(this.ordered.size()>0) {
			sb.append("\nGerarchia:\n");
			for(OOClass o:this.ordered) 
				sb.append(o.getName()+" ");
		}	
		/*a meno di fare verifiche questa  inutile vederla*/
		if(this.objectGeneralization.size()>0) {
			sb.append("\n\nCorrispondenze tra oggetti:\n");
			for(ObjectGeneralization objGen:this.objectGeneralization) {
				sb.append(objGen.toString());
				sb.append("\n");
			}
		}
		
		return sb.toString();	
	}
}
