package org.modelgen.importexport.oo.model;

/**Represents an actual value of a Reference.
 * A ReferenceComponent between a ComplexType/OOClass to a OOClass is something like
 * <code>
 * insert into or.persona values (or.persona(1),'leonardo',....,.....,or.indirizzo(8))
 * </code>
 * 
 * @version 0.9
 *
 */
public class ReferenceComponent {
	//OID
	private int oid;
	//the object this refenceComponent points
	private OOObject ooObjectTo;
	//	the scheme-object reference this reference-component is instance of
	private Reference ooReference;
	
	/**Create a ReferenceComponent with the given parameters
	 * @param oid OID as stored in ModelGen
	 * @param ooObjectTo the OOObject this ReferenceComponent points to
	 * @param ooReference the Reference this ReferenceComponent belongs
	 */
	public ReferenceComponent(int oid, OOObject ooObjectTo, Reference ooReference) {
		this.oid = oid;
		this.ooObjectTo = ooObjectTo;
		this.ooReference = ooReference;
	}
	
	/**Return a String representation of this object.
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if(ooObjectTo!=null)
			return "\t\tReference oid: "+this.oid+", ObjectToOID: "+this.ooObjectTo.getObjectOID()+"\n";
		else 
			return "\t\tReference oid: "+this.oid+", ObjectToOID: null\n";
	}

	public int getOid() {
		return oid;
	}

	public OOObject getOoObjectTo() {
		return ooObjectTo;
	}

	public Reference getOoReference() {
		return ooReference;
	}
}
