package org.modelgen.importexport.or.model;

import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;

import java.util.Vector;

/**Represents an instance of a ComplexType.
 * A ComplexType is the representations of a complex attribute.It can have a number of 
 * Field(as for an OOClass) and Reference (also has a OOClass).
 * 
 * For example:
 * <code>
 * create type uomo(nome varchar(20),et integer,domicilio REF(indirizzo))
 * create table .. (..,impiegato uomo,..)
 * insert into table .. values (..,impiegato()..nome('leonardo')..et(25),..)
 * </code>
 *
 */
public class ComplexTypeComponent {
	//OID
	private int OID;
	//the Complex this instance belongs to
	private ComplexType complex;
	//list of the value this instance has
	private Vector<ValueComponent> values;
	//list of the reference this instance has
	private Vector<ReferenceComponent> referencesComponets;
	
	/**Create a new ComplexTypeComponent with the given parameters.
	 * @param oid
	 * @param complex
	 */
	public ComplexTypeComponent(int oid, ComplexType complex) {
		OID = oid;
		this.complex = complex;
		this.values = new Vector<ValueComponent>();
		this.referencesComponets=new Vector<ReferenceComponent>();
	}
	
	public void addValueComponent(ValueComponent value){
		this.values.add(value);		
	}
	
	public void addReferenceComponent(ReferenceComponent reference) {
		this.referencesComponets.add(reference);
	}
	
	public Vector<ReferenceComponent> getReferencesComponets() {
		return referencesComponets;
	}

	public Vector<ValueComponent> getValues() {
		return values;
	}	

	public ComplexType getComplex() {
		return complex;
	}

	public int getOID() {
		return OID;
	}

	/**Returns a String representation of this object.
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("\tComplexComponent: ");
		sb.append(this.OID);
		
		for(ValueComponent value:this.values) {
			sb.append(",valore: ");
			if(value.getValue()==null)
				sb.append(" NULL ");
			else
				sb.append(value.getValue());
		}
		
		for(ReferenceComponent ref:this.referencesComponets) {
			sb.append(" ,REF: ");
			if(ref.getOoObjectTo()==null)
				sb.append(" NULL ");
			else
				sb.append(ref.getOoObjectTo().getObjectOID());
		}
		sb.append("\n");
		
		return sb.toString();
	}
}
