package org.modelgen.importexport.or.model;

import java.util.Vector;

/**Represents a Nest between two ComplexType.
 * 
 * A ComplexType can have has one of its attributes another ComplexType(and so on :)).
 * For example:
 * <code>
 *  create type email(..)
 *	create type manager(..,email_aziendale email)
 *  create type impiegato(nome varchar(20), cognome varchar(20),capo manager)
 * </code>
 * 
 * @version 0.9
 */
public class Nest {
	//OID
	private int OID;
	//name
	private String name;
	//top is the element that contains the other
	private ComplexType top;
	//nested is the element contained
	private ComplexType nested;
	
	//list of instances
	private Vector<NestComponent> nestedComponent;
	
	/**Creates a new Nest with the given parameters
	 * @param oid
	 * @param name
	 * @param top
	 * @param nested
	 */
	public Nest(int oid, String name, ComplexType top, ComplexType nested) {
		OID = oid;
		this.name = name;
		this.top = top;
		this.nested = nested;
		this.nestedComponent=new Vector<NestComponent>();	
	}
	
	public String getName() {
		return name;
	}

	public boolean add(NestComponent o) {
		return nestedComponent.add(o);
	}

	public Vector<NestComponent> getNestedComponent() {
		return nestedComponent;
	}

	public ComplexType getNested() {
		return nested;
	}

	public int getOID() {
		return OID;
	}

	public ComplexType getTop() {
		return top;
	}

	/**Returns a String representation of this objects
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("OID: ");
		sb.append(OID);
		sb.append(" name: ");
		sb.append(name);

		sb.append("\n\t\t");
		for(NestComponent nestComponent:this.nestedComponent) {
			sb.append("ComponentOID:");
			sb.append(nestComponent.getOID());
			sb.append(" (");
			sb.append(nestComponent.getTop().getOID());
			sb.append(",");
			sb.append(nestComponent.getNested().getOID());
			sb.append(")  ");
		}
		
		return sb.toString();
	}
}
