package org.modelgen.importexport.or.model;

import org.modelgen.importexport.oo.model.OOObject;
import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;

import java.util.Vector;

/**Represents an instance of a ORClass.
 * An ORObject "is a" OOObject plus the posibility to have a number of ComplexTypeComponents.
 * For other info see OOObject
 * 
 * for example:
 * <code>
 *  create type persona as (nome varchar(50),domicilio ref(or.indirizzo),residenza or.indirizzo)
 *  create table persone of persona
 *  insert into or.persone values (or.persona(1),'alessandro',or.indirizzo(1),or.indirizzo()..VIA('Via delle Ortenzie')..NUMERO(17));
 *  insert into or.persone values (or.persona(2),'luca',or.indirizzo(1),or.indirizzo()..VIA('Via delle Rose')..NUMERO(19));
 * </code>
 * 
 * @version 0.9
 * 
 */
public class ORObject extends OOObject {
	//the list of ComplexType component
	private Vector<ComplexTypeComponent> complexComponets;

	/**Creates a new ORObject with the given parameters
	 * @param objectOID
	 * @param orClass
	 */
	public ORObject(int objectOID, ORClass orClass) {
		super(objectOID, orClass);
		this.complexComponets=new Vector<ComplexTypeComponent>();
	}

	public Vector<ComplexTypeComponent> getComplexComponets() {
		return complexComponets;
	}

	public boolean addComplexTypeComponent(ComplexTypeComponent o) {
		return complexComponets.add(o);
	}	
	
	public void setValueComponent(Vector<ValueComponent> values) {
		this.valueComponents=values;
	}
	
	public void setReferenceComponents(Vector<ReferenceComponent> references) {
		this.referenceComponents=references;
	}
	
	/**Returns a String representation of this object.
	 * @see org.modelgen.importexport.oo.model.OOObject#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder(super.toString());
		
		for(ComplexTypeComponent complexComponent:this.complexComponets) {
			sb.append("\t");
			sb.append(complexComponent.toString());
		}
		
		return sb.toString();
	}
}
