package org.modelgen.importexport.or.model;

import org.modelgen.importexport.oo.model.OOClass;
import org.modelgen.importexport.relational.model.Table;

/**Represents a REF in a ORTable.
 * 
 * A Table can have only lexical Column; an ORTable can have number of ORReference, which are 
 * normal Reference but between a Table and a OOClass.
 * 
 *  For example:
 * <code>
 * create type indirizzo(..)
 * create typed table indirizzi of indirizzo(..)
 * create table impiegati (nome varchar(20),et int,..,domicilio REF(indirizzo) scope indirizzi)
 * </code>
 * 
 * @version 0.9
 * 
 *
 */
public class ORReference {
	//OID
	private int OID;
	//name
	private String name;
	//isoptional=> true is this ref can be null
	
/*	Modifica del 30/04/07 gli AbstractAttribute non hanno isOptional, ma isNullable
 * private boolean isOptional;	*/
	private boolean isNullable;
	//Table this ref belongs
	private Table from;
	//OOClass to
	private OOClass to;
	
	/**Creates a new ORReference with the given parameters.
	 */
	public ORReference(int oid, String name, boolean isNullable, Table from, OOClass to) {
		OID = oid;
		this.name = name;
		this.isNullable = isNullable;
		this.from = from;
		this.to = to;
	}

	public String getName() {
		return name;
	}

	public OOClass getTo() {
		return to;
	}

	public Table getFrom() {
		return from;
	}

	public int getOID() {
		return OID;
	}
	
	public boolean isNullable() {
		return isNullable;
	}

	/**Returns a String representation of this object
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		StringBuilder sb=new StringBuilder("\tOID: ");
		sb.append(OID);
		sb.append(" name: ");
		sb.append(name);
		sb.append(" isNullable: ");
		sb.append(isNullable);
		sb.append(" from: ");
		sb.append(from.getName());
		sb.append(" to: ");
		if(to!=null)
			sb.append(to.getName());
		else 
			sb.append(" NULL");
		
		return sb.toString();
	}
}
