package org.modelgen.importexport.or.model;

import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.Tuple;
import org.modelgen.importexport.relational.model.TupleComponent;

import java.util.Vector;

/**Represents an instance of an ORTable.
 * An ORTable can have number of ORReference, which are normal Reference but between a 
 * Table and a OOClass.
 * A complexType is a type used as an attribute of a Column(see ComplexType)
 * 
 * For example:
 * <code>
 * 	insert into .. values (Uomo()..nome('leonardo')..indirizzo(or.indirizzo(2)),
 * 					manager()..nome('paolo')..indirizzo(or.indirizzo(1))..bonus(5));
 * </code>
 * 
 * @version 0.9
 * 
 */
public class ORTuple extends Tuple {
	//the instances of the reference
	private Vector<ORReferenceComponent> referenceComponents;
	//the instances of the complexs
	private Vector<ComplexTypeComponent> complexComponents;
	
	/**Create a new ORTuple with the given parameters.
	 * @param tupleOID OID as stored in ModelGen
	 * @param table table this tuple belongs to
	 */
	public ORTuple(int tupleOID, Table table){
		super(tupleOID,table);
		this.referenceComponents=new Vector<ORReferenceComponent>();	
		this.complexComponents= new Vector<ComplexTypeComponent>();
	}

	public Vector<ORReferenceComponent> getReferenceComponents() {
		return referenceComponents;
	}
	
	public void addORReferenceComponent(ORReferenceComponent ref){
		this.referenceComponents.add(ref);
	}
	
	public void addComplexTypeComponent(ComplexTypeComponent component) {
		this.complexComponents.add(component);
	}

	public Vector<ComplexTypeComponent> getComplexComponents() {
		return complexComponents;
	}
	
	public void setTupleComponents(Vector<TupleComponent> components) {
		this.tupleComponents=components;
	}
}
