package org.modelgen.importexport.persistence;

import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.or.model.ORSchema;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.utilities.Configurator;
import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

/**This class is the link between objects structures and the ModelGen database.
 * It translate the informations which are in the object structures into SQL code
 * 
 * @version 0.9
 * 
 */
public class ImportExportPersistence {
	//l'unica istanza
	private static ImportExportPersistence instance;
	
	private ObjectRelational or;
	
	private Connection con;
	
	/**Returns an instance of the one and only ImportExportPersistence.
	 */
	public static ImportExportPersistence getInstance() {
		if(instance==null) {
			instance=new ImportExportPersistence();
			return instance;			
		}
		else
			return instance;
	}
	
	private ImportExportPersistence() {
		this.con = GeneralManagementFacade.getInstance().getConnection();
	}
		
	/**Store a RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeSchema(RelationalSchema schema) throws PersistenceException {	
		try {
			
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			/*i insert the Scheme*/
			this.storeScheme("Relational", schema.getOID(), schema.getName());
			/*this class is an utility to store, which do all the job*/
			Relational temp=new Relational(this.con,this);
			temp.storeSchema(schema);
			/*now i can commit*/
			this.con.commit();
			this.con.setAutoCommit(true);
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}

//	public void storeSchema(Schema schema) throws PersistenceException {	
//		System.out.println("Non devo lanciare questo");
//	}

	/**store di Or
	 * @param schema
	 * @throws PersistenceException
	 */
	public void storeSchema(ORSchema schema) throws PersistenceException {
		try {
			this.con.setAutoCommit(false);
			
			this.storeScheme("ObjectRelational", schema.getOID(), schema.getName());
			/*come al solito*/
			ObjectRelational or=new ObjectRelational(this.con,this);
			or.storeSchema(schema);
			/*now i can commit*/
			this.con.commit();
			this.con.setAutoCommit(true);
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema di tipo OO");
		}		
	}
	
	public void storeInstance(ORSchema schema) throws PersistenceException {
		try {
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			
			int schemeOID=schema.getOID();
			int instanceOID=this.getNewOID("Istanze");
			
			String insertInstance="INSERT INTO INSTANCESM VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertInstance);
			pst.setInt(1,instanceOID);
			pst.setString(2,schema.getName());
			pst.setInt(3,schemeOID);
			
			pst.executeUpdate();
			pst.close();
			
			ObjectRelational temp=new ObjectRelational(this.con,this);
			temp.storeInstance(schema,instanceOID);
			
			this.con.commit();
			this.con.setAutoCommit(true);	
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	/**Store the instances of the given RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeInstance(XmlSchema schema) throws PersistenceException {
		try {
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			
			int instanceOID=this.getNewOID("Istanze");
			
			String insertInstance="INSERT INTO INSTANCESM VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertInstance);
			pst.setInt(1,instanceOID);
			pst.setString(2,schema.getName());
			pst.setInt(3,schema.getOID());
			pst.executeUpdate();
			pst.close();
			
			Xml temp=new Xml(this.con,this);
			temp.storeInstance(schema, instanceOID);
			
			this.con.commit();
			this.con.setAutoCommit(true);			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}

	
	/**Store the instances of the given RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeInstance(RelationalSchema schema) throws PersistenceException {
		try {
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			
			int instanceOID=this.getNewOID("Istanze");
			
			String insertInstance="INSERT INTO INSTANCESM VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertInstance);
			pst.setInt(1,instanceOID);
			pst.setString(2,schema.getName());
			pst.setInt(3,schema.getOID());
			pst.executeUpdate();
			pst.close();
			
			Relational temp=new Relational(this.con,this);
			temp.storeInstance(schema,instanceOID);
			
			this.con.commit();
			this.con.setAutoCommit(true);			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	/**
	 * This method load a schema from the dictionary and instantiate modelgen
	 * classes to export the schema. 
	 * 
	 * @param schemaOID the OID of the scheme as stored in ModelGen
	 * @return the schema loaded
	 * @throws PersistenceException any error with the ModelGen db
	 */
	public RelationalSchema loadSchema(int schemaOID) throws PersistenceException {
		Relational temp=new Relational(this.con,this);
		return temp.loadSchema(schemaOID);		
	}
	
	
	/**Load the instances of the given schema.
	 * @param schema the schema
	 * @param instanceOID the instaceOID as stored in the ModelGen db
	 * @return the schema loaded
	 * @throws PersistenceException any error with the ModelGen db
	 */
	public RelationalSchema loadInstance(RelationalSchema schema,int instanceOID) throws PersistenceException {
		Relational temp=new Relational(this.con,this);
		return temp.loadInstance(schema, instanceOID);
	}
	
	/**This metod must store a OO schema (which has to be create by specific class) in ModelGen database
	 * 
	 */
	public void storeSchema(OOSchema schema) throws PersistenceException {
		try {
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			this.storeScheme("ObjectOriented", schema.getOID(), schema.getName());
			/*this class is an utility to store, which do all the job*/
			ObjectOriented oo=new ObjectOriented(this.con,this);
			oo.storeOOSchema(schema);
			/*now i can commit*/
			this.con.commit();
			this.con.setAutoCommit(true);
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema di tipo OO");
		}			
	}
	
	/**Store the instances of OOSchema in the DB
	 */
	public void storeInstance(OOSchema schema) throws PersistenceException {
		
		try {
			int schemeOID=schema.getOID();
			int instanceOID=this.getNewOID("Istanze");
			
			String insertInstance="INSERT INTO INSTANCESM VALUES(?,?,?)";
			PreparedStatement pst = this.con.prepareStatement(insertInstance);
			pst.setInt(1,instanceOID);
			pst.setString(2,schema.getName());
			pst.setInt(3,schemeOID);
			
			pst.executeUpdate();
			pst.close();
			
			ObjectOriented oo=new ObjectOriented(this.con,this);
			oo.storeOOInstance(schema,instanceOID);
			
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nello store delle istanze OO");
		}		
	}
	
	/**This metod must load a OO schema and instance from ModelGen database
	 * 
	 */
	public OOSchema loadOOSchema(int schemaOID) throws PersistenceException  {	
		ObjectOriented oo=new ObjectOriented(this.con,this);
		return oo.loadOOSchema(schemaOID);
	}
	
	/**Carica le istanze dal DB di modelgen all'interno di OOSchema
	 * 
	 * @param ooSchema lo schema a cui aggiungere le istanze
	 */
	public void loadOOInstance(OOSchema ooSchema,int instanceOID) throws PersistenceException {
		ObjectOriented oo=new ObjectOriented(this.con,this);
		oo.loadOOInstance(ooSchema, instanceOID);
	}
	
	public ORSchema loadORSchema(int schemaOID) throws PersistenceException {
		or=new ObjectRelational(this.con,this);
		return or.loadORSchema(schemaOID);
	}
	
	public ORSchema loadORInstance(ORSchema orSchema,int instanceOID) throws PersistenceException {
		if(or==null)
			throw new PersistenceException("Va caricato prima lo schema");
		return or.loadORInstance(orSchema, instanceOID);
	}
	
	/**Store a XmlSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeSchema(XmlSchema schema) throws PersistenceException {	
		try {
			
			/*i have to store ALL infos in the db otherwise non of them*/
			this.con.setAutoCommit(false);
			/*i insert the Scheme*/
			this.storeScheme("XSD", schema.getOID(), schema.getName());
			/*this class is an utility to store, which do all the job*/
			Xml temp=new Xml(this.con,this);
			temp.storeSchema(schema);
			/*now i can commit*/
			this.con.commit();
			this.con.setAutoCommit(true);
			
		} catch (SQLException e) {
			e.printStackTrace();
			try {
				this.con.rollback();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	/**
	 * This method load a schema from the dictionary and instantiate modelgen
	 * classes to export the schema. 
	 * 
	 * @param schemaOID the OID of the scheme as stored in ModelGen
	 * @return the schema loaded
	 * @throws PersistenceException any error with the ModelGen db
	 */
	public XmlSchema loadXMLSchema(int schemaOID) throws PersistenceException {
		Xml xml=new Xml(this.con,this);
		return xml.loadXMLSchema(schemaOID);	
	}

	public XmlSchema loadXmlInstance(XmlSchema schema, int instanceOID) throws PersistenceException {
		Xml xml=new Xml(this.con,this);
		return xml.loadXmlInstance(schema, instanceOID);
	}

	
	/**Check if a model exist and return its OID
	 * 
	 */
	public int checkModelOID(String model)throws PersistenceException {
		
		int modelOID;
		String query=new String("SELECT OID FROM MODEL WHERE Name='"+model+"'");
		Statement st;
		try {
			st = con.createStatement();
			ResultSet rs=st.executeQuery(query);
			if(rs.next()) {
				modelOID=rs.getInt(1);
			}
			else {
				throw new PersistenceException("Nel databse di ModelGen non  stato trovato il modello indicato\n" +
						"Accertarsi di averlo importato con lo strumento DefineModel");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel trovare il modello");
		}		
		
		return modelOID;
	}
	
	/**Return a OID from the ModelGen db for the entity of the given name
	 *  
	 * Attenzione nel file corrispondenze.txt ho delle coppie
	 * 
	 * Table=Aggregation
	 * ecc.
	 * quindi io faccio la richiesta per un nuovo OID su table che lui mapper in Aggregation * 
	 * 
	 * @param key key of the entity see coorispondenze.txt
	 * @return un intero univoco
	 * @throws PersistenceException 
	 */
	public int getNewOID(String key) throws PersistenceException {
		try {
			//vado a vedere ne file di configurazione a cosa corrisponde
			//la key data vedere corrispondenze.txt
			String table=Configurator.getInstance().getCorrispondenza(key);
			if(table==null)
				throw new PersistenceException("La chiave richiesta non esiste nel file di configurazione: "+key);
			// Prepare the query.
			String query = "SELECT max(OID) FROM " + table + " ;";
			Statement st = con.createStatement();
			// Execute the query.
			ResultSet result = st.executeQuery(query);
			// Return the result.
			int newOID = 0;
			while (result.next()) {
				newOID  = result.getInt(1) + 1;
			}
			return newOID;
		} catch (SQLException sqle) {
			throw new PersistenceException(sqle.toString());
		}
	}
	
	private void storeScheme(String model,int schemaOID,String schemaName) throws PersistenceException, SQLException {
		int modelOID=this.checkModelOID(model);
		
		String insertScheme="INSERT INTO SCHEMESM VALUES(?,?,?,?,null)";
		PreparedStatement pst=this.con.prepareStatement(insertScheme);
		pst.setInt(1,schemaOID);
		pst.setString(2,schemaName);
		pst.setInt(3,modelOID);
		pst.setString(4,"Import di "+schemaName);
		
		pst.executeUpdate();
		pst.close();
	}
	
	
	/**Va a fare lo store nelle tabelle di skolem sempre con il principio delle corrispondenze
	 * @param key
	 * @throws PersistenceException 
	 */
	public void storeSkolem(String key,int OID,int schemeOID) throws PersistenceException {
		String table=Configurator.getInstance().getCorrispondenza(key);
		if(table==null)
			throw new PersistenceException("La chiave richiesta non esiste nel file di configurazione: "+key);

		String quey=new String("SELECT MAX(OID) FROM "+table+"_SK");
		int maxOID=0;

		try {
			Statement st=con.createStatement();
			ResultSet rs=st.executeQuery(quey);
			if(rs.next())
				maxOID=rs.getInt(1);			
		} catch (SQLException e) {
		}

		if(OID>maxOID) {
			String secondColumnName;
			if (table.startsWith("I_")) {
				secondColumnName = "INSTANCEOID";
			} else {
				secondColumnName = "SCHEMEOID";
			}
			String insert=new String("INSERT INTO "+table+"_SK (OID,"+secondColumnName+") VALUES (?,?)"); 

			try {
				PreparedStatement ps = this.con.prepareStatement(insert);
				ps.setInt(1, OID);
				ps.setInt(2, schemeOID);
				ps.execute();				
				ps.close();

			} catch (SQLException e) {
				e.printStackTrace();
				throw new PersistenceException("Errore nell'inserimento nella tabella di Skolem di "+table);
			}	
		}	
	}

	
	/**Ritorna un array di array di String contenenti le informazioni da mostrare nella finestra
	 * di Export
	 */
	public String[][] loadExportSchemas() {
		Vector<Vector<String>> result=new Vector<Vector<String>>();
		String query=new String("SELECT I1.OID,I1.NAME,M.NAME,I1.INSTANCEOID FROM " +
				"(SELECT S.OID AS OID,S.NAME AS NAME,I.OID AS INSTANCEOID,S.MODELOID AS MODELOID " +
				"FROM SCHEMESM S LEFT JOIN INSTANCESM I ON (S.OID=I.SCHEMEOID)) AS I1, MODEL M WHERE I1.MODELOID=M.OID");
		Statement st;
		try {
			st = this.con.createStatement();
			ResultSet rs=st.executeQuery(query);

			while(rs.next()) {
				Vector<String> temp=new Vector<String>();
				temp.add(rs.getString(1));
				temp.add(rs.getString(2));
				temp.add(rs.getString(3));
				temp.add(rs.getString(4));
				result.add(temp);
			}
			rs.close();
			st.close();			
		} catch (SQLException e) {
			//ritorno una lista vuota	
			return new String[0][4];
		}		
		String[][] risultato=new String[result.size()][4];
		for(int i=0;i<result.size();i++) {
			for(int j=0;j<4;j++) {
				risultato[i][j]=result.get(i).get(j);
			}
		}
		
		return risultato;
	}
}
