/**This is merely a utility class to do the dirt job
 * 
 */
package org.modelgen.importexport.persistence;

import org.modelgen.importexport.oo.model.Field;
import org.modelgen.importexport.oo.model.OOChild;
import org.modelgen.importexport.oo.model.OOClass;
import org.modelgen.importexport.oo.model.OOGeneralization;
import org.modelgen.importexport.oo.model.OOObject;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.oo.model.ObjectGeneralization;
import org.modelgen.importexport.oo.model.Reference;
import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;
import org.modelgen.importexport.utilities.PersistenceException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * @author LordofDeceits
 *
 */
class ObjectOriented {
	private Connection con;
	private ImportExportPersistence ie;
	
	public ObjectOriented(Connection con,ImportExportPersistence ie) {
		this.con=con;
		this.ie=ie;
	}
	
	public void storeOOSchema(OOSchema schema) throws PersistenceException {
		try {
			/*come al solito salvo il valore massimo dell'OID per andare poi ad inserire il corretto
			 * valore nelle tabelle di skolem*/
			int max=-1;
			int maxField=-1;
			String insertAggregation="INSERT INTO SM_ABSTRACT VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertAggregation);

			String insertLexical="INSERT INTO SM_LEXICAL VALUES(?,?,?,?,null,?,?,null,null,null,null,?)";
			PreparedStatement pst1=this.con.prepareStatement(insertLexical);

			for(OOClass o:schema.getOOClasses()) {
				if(o.getOid()>max)
					max=o.getOid();
				//inserire sm_abstract
				pst.setInt(1,o.getOid());
				pst.setInt(2,schema.getOID());
				pst.setString(3,o.getName());

				pst.executeUpdate();
				//inserisci lexicalabstract
				for(Field field:o.getFields()) {
					if(field.getOid()>maxField)
						maxField=field.getOid();
					pst1.setInt(1,field.getOid());
					pst1.setInt(2,schema.getOID());
					pst1.setString(3,field.getName());
					pst1.setBoolean(4, field.isNullable());
					pst1.setBoolean(5,field.isIdentifier());
					pst1.setString(6, field.getType());
					pst1.setInt(7,o.getOid());

					pst1.executeUpdate();
				}			
			}		
			pst.close();
			pst1.close();
			
			this.ie.storeSkolem("Class",max,schema.getOID());
			max=-1;
			this.ie.storeSkolem("Field",maxField,schema.getOID());

			String insertAttribute="INSERT INTO SM_ABSTRACTATTRIBUTE VALUES(?,?,?,?,null,null,?,?)";
			pst=this.con.prepareStatement(insertAttribute);

			for(OOClass o:schema.getOOClasses()) {
				//now abstract attribute
				for(Reference ref:o.getOOReferences()) {
					if(ref.getOid()>max)
						max=ref.getOid();
					pst.setInt(1,ref.getOid());
					pst.setInt(2,schema.getOID());
					pst.setString(3,ref.getName());
					pst.setBoolean(4, ref.isNullable());
					pst.setInt(5,o.getOid());
					pst.setInt(6,ref.getClassTo().getOid());

					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("Reference",max,schema.getOID());
			max=-1;

			//now generalization and child of Gen
			String insertGeneralization="INSERT INTO SM_GENERALIZATION VALUES(?,?,?,?,?)";
			pst=this.con.prepareStatement(insertGeneralization);

			for(OOGeneralization gen: schema.getGeneralizations()) {
				if(gen.getOID()>max)
					max=gen.getOID();
				pst.setInt(1, gen.getOID());
				pst.setInt(2, schema.getOID());
				pst.setString(3,gen.getName());
				pst.setBoolean(4, gen.isTotal());
				pst.setInt(5, gen.getParent().getOid());

				pst.executeUpdate();
			}
			pst.close();
			
			this.ie.storeSkolem("Generalization",max,schema.getOID());
			max=-1;
			
			/*now childs*/
			String insertChild="INSERT INTO SM_CHILDOFGENERALIZATION VALUES(?,?,?,?,?)";
			pst=this.con.prepareStatement(insertChild);

			for(OOGeneralization gen: schema.getGeneralizations()) {
				for(OOChild child:gen.getChilds()) {
					if(child.getOID()>max)
						max=child.getOID();
					pst.setInt(1,child.getOID());
					pst.setInt(2, schema.getOID());
					pst.setString(3,"child"+child.getChild().getName());
					pst.setInt(4,gen.getOID());
					pst.setInt(5,child.getChild().getOid());
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("ChildOfGeneralization",max,schema.getOID());
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema di tipo OO");
		} 	
	}

	public void storeOOInstance(OOSchema schema,int instanceOID) throws PersistenceException {
		try {		
			/*come al solito salvo il valore massimo dell'OID per andare poi ad inserire il corretto
			 * valore nelle tabelle di skolem*/
			int max=-1;
			
			String insertAbstract="INSERT INTO I_SM_ABSTRACT VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertAbstract);
			/*gli oggetti che carico dal dbms hanno id univoci, ma solamente a livello
			 * di typed table e in gerarchia hanno lo stesso oid, quindi per inserirli devo
			 * affiornare io gli OID in modo che anche in ModelGen siano univoci*/
			int objectOID=ie.getNewOID("Object");
						
			for(OOClass o:schema.getOOClasses()) {
				for(OOObject object: o.getOOObjects()) {
					pst.setInt(1,objectOID);
					object.setOID(objectOID);
					if(object.getObjectOID()>max)
						max=object.getObjectOID();
					objectOID++;
					pst.setInt(2,o.getOid());
					pst.setInt(3, instanceOID);
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("Object",max,instanceOID);
			max=-1;
			int maxSecond=-1;
			
			String insertLexical="INSERT INTO I_SM_LEXICAL VALUES(?,?,?,?,null,null,null,null,?)";
			pst=this.con.prepareStatement(insertLexical);
			String insertAttribute="INSERT INTO I_SM_ABSTRACTATTRIBUTE VALUES(?,?,?,null,null,?,?)";
			PreparedStatement pst1=this.con.prepareStatement(insertAttribute);
			
			for(OOClass o:schema.getOOClasses()) {
				for(OOObject object: o.getOOObjects()) {
					for(ValueComponent value:object.getValueComponents()) {
						if(value.getOid()>max)
							max=value.getOid();
						pst.setInt(1,value.getOid());
						pst.setInt(2,value.getOoField().getOid());
						pst.setInt(3,instanceOID);
						if(value.getValue()!=null)
							pst.setString(4, value.getValue());
						else
							pst.setNull(4,java.sql.Types.VARCHAR); 
						pst.setInt(5, value.getOwner().getObjectOID());
						pst.executeUpdate();
					}	
					for(ReferenceComponent ref:object.getReferenceComponents()) {
						if(ref.getOid()>maxSecond)
							maxSecond=ref.getOid();
						pst1.setInt(1,ref.getOid());
						pst1.setInt(2,ref.getOoReference().getOid());
						pst1.setInt(3, instanceOID);
						pst1.setInt(4, object.getObjectOID());
						if(ref.getOoObjectTo()!=null)
							pst1.setInt(5, ref.getOoObjectTo().getObjectOID());
						else 
							pst1.setNull(5,java.sql.Types.INTEGER);
						
						pst1.executeUpdate();
					}
				}
			}
			pst.close();
			pst1.close();
			
			this.ie.storeSkolem("FieldComponent",max,instanceOID);
			this.ie.storeSkolem("RefComponent",maxSecond,instanceOID);
			max=-1;
			maxSecond=-1;
			
			int genIOID=ie.getNewOID("ObjectGeneralization");
			int childIOID=ie.getNewOID("ChildObjectGeneralization");
					
			String insertGeneralization="INSERT INTO I_SM_GENERALIZATION VALUES(?,?,?,?)";
			pst=this.con.prepareStatement(insertGeneralization);
			
			String insertChild="INSERT INTO I_SM_CHILDOFGENERALIZATION VALUES(?,?,?,?,?)";
			pst1=this.con.prepareStatement(insertChild);
			
			for(ObjectGeneralization objGen:schema.getObjectGeneralization()) {
				pst.setInt(1, genIOID);
				if(genIOID>max)
					max=genIOID;
				genIOID++;
				pst.setInt(2,objGen.getGeneralization().getOID());
				pst.setInt(3, instanceOID);
				pst.setInt(4, objGen.getParentObject().getObjectOID());
				
				pst.executeUpdate();
				/*contemporaneamente creo pure l'istanza di child*/
				pst1.setInt(1, childIOID);
				if(childIOID>maxSecond)
					maxSecond=childIOID;
				childIOID++;
				pst1.setInt(2, objGen.getChild().getOID());
				pst1.setInt(3, instanceOID);
				pst1.setInt(4, genIOID-1);
				pst1.setInt(5, objGen.getChildObject().getObjectOID());
				
				pst1.executeUpdate();				
			}
			pst.close();
			pst1.close();	
			
			this.ie.storeSkolem("ObjectGeneralization",max,instanceOID);
			this.ie.storeSkolem("ChildObjectGeneralization",maxSecond,instanceOID);
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import");
		} 	
	}

	public OOSchema loadOOSchema(int schemaOID) throws PersistenceException  {
		try {

			String sqlQuery = "SELECT NAME FROM SCHEMESM WHERE OID = " + schemaOID;
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			result.next();
			String schemaName = result.getString("NAME");
			result.close();

			OOSchema ooSchema = new OOSchema(schemaOID,schemaName);

			// now retrieve the classes's'name
			sqlQuery = "SELECT OID, NAME FROM SM_ABSTRACT WHERE SCHEMEOID = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);
			pst.setInt(1, schemaOID);
			result = pst.executeQuery();
			while (result.next()) {
				int ooClassOID = result.getInt("OID");
				String ooClassName = result.getString("NAME");
				OOClass ooClass  = new OOClass(ooClassOID,ooClassName);
				ooSchema.addOOClass(ooClass);
			}
			result.close();
			pst.close();


			//now, for each class retrieve all the fields
			for (OOClass ooClass : ooSchema.getOOClasses()) {
				sqlQuery = "SELECT OID,NAME, ISIDENTIFIER, ISNULLABLE, TYPE" +
				" FROM SM_LEXICAL" +
				" WHERE SCHEMEOID = ? AND ABSTRACTOID = ?;";
				pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, schemaOID);
				pst.setInt(2, ooClass.getOid());
				result = pst.executeQuery();
				while (result.next()) {
					int ooFieldOID = result.getInt("OID");
					String ooFieldName = result.getString("NAME");
					boolean isNullable = result.getBoolean("ISNULLABLE");
					/*modifica 30/04/07 se datalog non crea colonna isNullable, quindi qui
					 * ho null devo mette true*/
					if(result.wasNull())
						isNullable=true;
					boolean isIdentifier = result.getBoolean("ISIDENTIFIER");
					String ooFieldType = result.getString("TYPE");
					Field ooField = new Field(ooFieldOID, ooFieldName, ooFieldType, isNullable, isIdentifier);
					ooClass.addField(ooField);
				}
			}
			result.close();
			pst.close();

			//and now for references between classes
			for (OOClass ooClass : ooSchema.getOOClasses()) {
				sqlQuery = "SELECT OID, NAME, ISNULLABLE, ABSTRACTTOOID" +
				" FROM SM_ABSTRACTATTRIBUTE" +
				" WHERE SCHEMEOID = ? AND ABSTRACTOID = ?;";
				pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, schemaOID);
				pst.setInt(2, ooClass.getOid());
				result = pst.executeQuery();
				while (result.next()) {
					int ooReferenceOID = result.getInt("OID");
					String ooReferenceName = result.getString("NAME");
					boolean isNullable = result.getBoolean("ISNULLABLE");
					int ooClassToOID = result.getInt("ABSTRACTTOOID");
					OOClass ooClassTo = ooSchema.getClassByOID(ooClassToOID);
					Reference ooReference = new Reference(ooReferenceOID, 
							ooReferenceName, isNullable, ooClassTo);
					ooClass.addReference(ooReference);
				}
			}
			result.close();
			pst.close();

			/*now i have to load the generalization*/
			sqlQuery=new String("SELECT * FROM SM_GENERALIZATION WHERE SCHEMEOID=?");
			pst=con.prepareStatement(sqlQuery);
			pst.setInt(1,schemaOID);
			result=pst.executeQuery();

			while(result.next()) {
				int parent=result.getInt(5);
				OOClass to=ooSchema.getClassByOID(parent);
				OOGeneralization gen=new OOGeneralization(result.getInt(1),result.getString(3),result.getBoolean(4),to);
				ooSchema.addOOGeneralization(gen);
			}
			result.close();
			pst.close();

			sqlQuery=new String("SELECT OID,GENERALIZATIONOID,CHILDABSTRACTOID FROM SM_CHILDOFGENERALIZATION WHERE SCHEMEOID=?");
			pst=con.prepareStatement(sqlQuery);
			pst.setInt(1,schemaOID);
			result=pst.executeQuery();

			while(result.next()) {
				OOGeneralization gen=ooSchema.getOOGeneralizationByOID(result.getInt("GENERALIZATIONOID"));
				//gen NON pu essere null a causa del vincolo di integrit referenziale
				OOClass childClass=ooSchema.getClassByOID(result.getInt("CHILDABSTRACTOID"));
				//childClass NON pu essere null a causa del vincolo di integrit referenziale
				OOChild child=new OOChild(result.getInt("OID"),childClass);
				gen.addChild(child);
			}
			result.close();
			pst.close();
			return ooSchema;

		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out);
			throw new PersistenceException(sqle.toString());
		}		
	}

	public void loadOOInstance(OOSchema ooSchema,int instanceOID) throws PersistenceException {
		try {
			//for each class, load its instances
			for (OOClass ooClass : ooSchema.getOOClasses()) {
				String sqlQuery = "SELECT object.OID AS objectOID " 
					+ "FROM I_SM_ABSTRACT object "
					+ "WHERE object.instanceOID = ? AND object.abstractOID = ?;";
				PreparedStatement pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, instanceOID);
				pst.setInt(2, ooClass.getOid());
				ResultSet result = pst.executeQuery();
				while (result.next()) {
					int objectOID = result.getInt("objectOID");
					OOObject ooObject = new OOObject(objectOID, ooClass);
					ooClass.addOOObject(ooObject);
				}
				result.close();
				pst.close();
			}
			
			for (OOClass ooClass : ooSchema.getOOClasses()) {
				for (OOObject ooObject : ooClass.getOOObjects()) {
					String sqlQuery = "SELECT valueObject.OID AS valueObjectOID, "
						+	"valueObject.value AS valueObjectValue, "
						+	"valueObject.LEXICALOID AS ooFieldOID " 
						+ "FROM I_SM_ABSTRACT object JOIN I_SM_LEXICAL valueObject "
						+ "ON object.OID = valueObject.i_abstractOID "
						+ "WHERE object.instanceOID = ? AND valueObject.instanceOID = ? "
						+ "AND valueObject.i_abstractOID = ?;";
					PreparedStatement pst = con.prepareStatement(sqlQuery);
					pst.setInt(1, instanceOID);
					pst.setInt(2, instanceOID);
					pst.setInt(3, ooObject.getObjectOID());
					ResultSet result = pst.executeQuery();
					while (result.next()) {
						int ooValueOID = result.getInt("valueObjectOID");
						String value =  result.getString("valueObjectValue");
						int ooFieldOID = result.getInt("ooFieldOID");
						Field ooField = ooClass.getOOFieldByOID(ooFieldOID); 
						
						/*Aggiunta 25/05/07, controllo valori nulli*/
						if(!ooField.isNullable() && value==null)
							throw new PersistenceException("Un valore dell'oggetto di OID "+ooObject.getObjectOID()+
									" appartenente a "+ooClass.getName()+" ha un valore null, ma il" +
									" campo corrispondente NON ammette valori nulli");
						
						ValueComponent ooValue = 
							new ValueComponent(ooValueOID, value, 
									ooField, ooObject);
						ooObject.addValueComponent(ooValue);						
					}
					result.close();
					pst.close();
					sqlQuery = "SELECT referenceObject.OID AS ooObjectReferenceOID, "
						+	"referenceObject.i_AbstractToOID AS ooClassInstanceToOID, "
						+	"referenceObject.AbstractAttributeOID AS ooReferenceOID " 
						+ "FROM i_SM_Abstract object JOIN i_SM_AbstractAttribute referenceObject "
						+ "ON object.OID = referenceObject.i_abstractOID "
						+ "WHERE object.instanceOID = ? AND referenceObject.instanceOID = ? "
						+ "AND referenceObject.i_abstractOID = ?;";
					pst = con.prepareStatement(sqlQuery);
					pst.setInt(1, instanceOID);
					pst.setInt(2, instanceOID);
					pst.setInt(3, ooObject.getObjectOID());
					result = pst.executeQuery();
					while (result.next()) {
						int ooObjectReferenceOID = result.getInt("ooObjectReferenceOID");
						int ooClassInstanceToOID =  result.getInt("ooClassInstanceToOID");
						int ooReferenceOID = result.getInt("ooReferenceOID");
						Reference ooReference = ooClass.getOOReferenceByOID(ooReferenceOID); 
						OOObject to=ooSchema.getClassInstanceByOID(ooClassInstanceToOID);
						
						/*Aggiunta 25/05/07, controllo valori nulli*/
						if(!ooReference.isNullable() && to==null)
							throw new PersistenceException("Un valore di una referenza dell'oggetto di OID "+ooObject.getObjectOID()+
									" appartenente a "+ooClass.getName()+" ha un valore null, ma la" +
									" Reference corrispondente NON ammette valori nulli");
						
						ReferenceComponent ooReferenceComponent = 
							new ReferenceComponent(ooObjectReferenceOID,
									to, 
									ooReference);
						ooObject.addReferenceComponent(ooReferenceComponent);
					}
					result.close();
					pst.close();
					
				}
			}
			/*ora devo caricare le object generalizations*/
			String sql=new String("SELECT I1.I_PARENTABSTRACTOID,I2.I_CHILDABSTRACTOID " +
					"FROM I_SM_GENERALIZATION AS I1,I_SM_CHILDOFGENERALIZATION AS I2 " +
					"WHERE INSTANCEOID=? AND I1.OID=I2.I_GENERALIZATIONOID AND I1.GENERALIZATIONOID=? AND I2.CHILDOFGENERALIZATIONOID=?");
			PreparedStatement pst=con.prepareStatement(sql);
			pst.setInt(1, instanceOID);
			
			for(OOGeneralization gen:ooSchema.getGeneralizations()) {
				for(OOChild child:gen.getChilds()) {
					pst.setInt(2,gen.getOID());
					pst.setInt(3,child.getOID());
					
					ResultSet rs=pst.executeQuery();
					
					while(rs.next()) {
						int parentOID=rs.getInt("I_PARENTABSTRACTOID");
						int childOID=rs.getInt("I_CHILDABSTRACTOID");
						
						OOObject parentObj=gen.getParent().getOOObjectByOID(parentOID);
						OOObject childObj=child.getChild().getOOObjectByOID(childOID);
						
						ObjectGeneralization objGen=new ObjectGeneralization(gen,child,parentObj,childObj);
						ooSchema.addObjectGeneralization(objGen);
					}
					rs.close();					
				}				
			}
			pst.close();
			
		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out); 
			throw new PersistenceException(sqle.toString());
		} 		
	}
}
