package org.modelgen.importexport.persistence;

import org.modelgen.importexport.oo.model.Field;
import org.modelgen.importexport.oo.model.OOChild;
import org.modelgen.importexport.oo.model.OOClass;
import org.modelgen.importexport.oo.model.OOGeneralization;
import org.modelgen.importexport.oo.model.OOObject;
import org.modelgen.importexport.oo.model.OOSchema;
import org.modelgen.importexport.oo.model.ObjectGeneralization;
import org.modelgen.importexport.oo.model.Reference;
import org.modelgen.importexport.oo.model.ReferenceComponent;
import org.modelgen.importexport.oo.model.ValueComponent;
import org.modelgen.importexport.or.model.ComplexType;
import org.modelgen.importexport.or.model.ComplexTypeComponent;
import org.modelgen.importexport.or.model.Nest;
import org.modelgen.importexport.or.model.NestComponent;
import org.modelgen.importexport.or.model.ORClass;
import org.modelgen.importexport.or.model.ORObject;
import org.modelgen.importexport.or.model.ORReference;
import org.modelgen.importexport.or.model.ORReferenceComponent;
import org.modelgen.importexport.or.model.ORSchema;
import org.modelgen.importexport.or.model.ORTable;
import org.modelgen.importexport.or.model.ORTuple;
import org.modelgen.importexport.relational.model.ComponentOfForeignKey;
import org.modelgen.importexport.relational.model.ForeignKey;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.Tuple;
import org.modelgen.importexport.utilities.PersistenceException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

/**
 * @author LordofDeceits
 *
 */
class ObjectRelational {
	private Connection con;
	private ImportExportPersistence ie;
	private RelationalSchema relSchema;
	private OOSchema ooSchema;
	
	public ObjectRelational(Connection con,ImportExportPersistence ie) {
		this.con=con;
		this.ie=ie;
		this.relSchema=null;
		this.ooSchema=null;
	}
	
	public void storeSchema(ORSchema schema) throws PersistenceException {
		try {
			/*salvo il massimo valore tra gli OID in modo da andare poi a salvare questo valore 
			 * nella tabelle si skolem*/
			int max=-1;
			/*creo uno schema relazionale temporaneo, in questo modo on devo riscrivere
			 * tutti gli insert che ho usato per il relazionale*/
			RelationalSchema relSchema=new RelationalSchema(schema.getOID(),schema.getName());
			for(Table table:schema.getTables()) {
				relSchema.addTable(table);
			}
			//le Fk le carico tutte insieme alla fine
			Relational rel=new Relational(this.con,this.ie);
			rel.storeSchema(relSchema);

			/*faccio la stessa cosa per l'OO*/
			OOSchema ooSchema=new OOSchema(schema.getOID(),schema.getName());
			for(ORClass o:schema.getORClasses()) {
				ooSchema.addOOClass(o);
			}
			ooSchema.setGeneralizations(schema.getGeneralizations());
			ObjectOriented oo=new ObjectOriented(this.con,this.ie);
			oo.storeOOSchema(ooSchema);

			/*mi manca AbstractComponentofAggregation e le StructofAttributes per fare una tabella OR, giusto??*/
			String insertAbstract="INSERT INTO SM_ABSTRACTATTRIBUTE VALUES(?,?,?,?,null,?,null,?)";
			PreparedStatement pst=this.con.prepareStatement(insertAbstract);

			for(ORTable table:schema.getTables()) {
				for(ORReference ref:table.getReferences()) {
					if(ref.getOID()>max)
						max=ref.getOID();
					pst.setInt(1, ref.getOID());
					pst.setInt(2, schema.getOID());
					pst.setString(3, ref.getName());
					pst.setBoolean(4, ref.isNullable());
					pst.setInt(5, ref.getFrom().getOID());
					pst.setInt(6, ref.getTo().getOid());

					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("Reference",max,schema.getOID());
			max=-1;

			/*complex type che appartengono ad una tabella*/
			String insertStruct=new String("INSERT INTO SM_STRUCTOFATTRIBUTES VALUES(?,?,?,null,?,?,null,?)");
			pst=this.con.prepareStatement(insertStruct);
			Vector<ComplexType> complexes=new Vector<ComplexType>();

			for(ORTable table:schema.getTables()) {
				for(ComplexType complex:table.getComplexes()) {
					if(complex.getOID()>max)
						max=complex.getOID();
					pst.setInt(1, complex.getOID());
					pst.setInt(2, schema.getOID());
					pst.setString(3, complex.getName());
					pst.setBoolean(4, complex.isOptional());
					pst.setBoolean(5, complex.isNullable());
					pst.setInt(6, table.getOID());

					complexes.add(complex);					
					pst.executeUpdate();
				}
			}
			pst.close();

			/*questo dei complex type mo lo devo fare pure per gli abstract*/
			insertStruct=new String("INSERT INTO SM_STRUCTOFATTRIBUTES VALUES(?,?,?,null,?,?,?,null)");
			pst=this.con.prepareStatement(insertStruct);

			for(ORClass o:schema.getORClasses()) {
				for(ComplexType complex:o.getComplexes()) {
					if(complex.getOID()>max)
						max=complex.getOID();
					pst.setInt(1, complex.getOID());
					pst.setInt(2, schema.getOID());
					pst.setString(3, complex.getName());
					pst.setBoolean(4, complex.isOptional());
					pst.setBoolean(5, complex.isNullable());
					pst.setInt(6, o.getOid());

					complexes.add(complex);					
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("ComplexType",max,schema.getOID());
			max=-1;

			/*carico gli attributi lessicali e abstract per tutti i complex type*/
			String insertLexical="INSERT INTO SM_LEXICAL VALUES(?,?,?,?,null,?,?,null,null,null,?,null)";
			PreparedStatement insertField=this.con.prepareStatement(insertLexical);
			String insertAttribute="INSERT INTO SM_ABSTRACTATTRIBUTE VALUES(?,?,?,?,?,null,null,?)";
			PreparedStatement insertRef=this.con.prepareStatement(insertAttribute);
			
			max=storeLexical(complexes, schema,insertField,insertRef);		
			this.ie.storeSkolem("Field",max,schema.getOID());
			max=storeAbstract(complexes, schema,insertField,insertRef);
			this.ie.storeSkolem("Reference",max,schema.getOID());
			max=-1;
			
			/*adesso posso fare i nest, prima per devo caricare anche tutti i 
			 * complex type che erano nestati*/
			insertStruct=new String("INSERT INTO SM_STRUCTOFATTRIBUTES VALUES(?,?,?,null,?,?,null,null)");
			pst=this.con.prepareStatement(insertStruct);
			complexes=new Vector<ComplexType>();
			
			for(ComplexType complex:schema.getAllNested()) {
				if(complex.getOID()>max)
					max=complex.getOID();
				pst.setInt(1, complex.getOID());
				pst.setInt(2, schema.getOID());
				pst.setString(3, complex.getName());
				pst.setBoolean(4, complex.isOptional());
				pst.setBoolean(5, complex.isNullable());
					
				complexes.add(complex);
				pst.executeUpdate();
			}
			pst.close();
			
			this.ie.storeSkolem("ComplexType",max,schema.getOID());
			max=-1;
			
			/*devo ancora ricaricare gli attributi di questi nuovi complex*/
			max=storeLexical(complexes, schema,insertField,insertRef);		
			this.ie.storeSkolem("Field",max,schema.getOID());
			max=storeAbstract(complexes, schema,insertField,insertRef);
			this.ie.storeSkolem("Reference",max,schema.getOID());
			max=-1;
			
			/*ora i Nest*/
			String insertNest=new String("INSERT INTO SM_NEST VALUES(?,?,?,?,?)");
			pst=this.con.prepareStatement(insertNest);

			for(Nest nest:schema.getNestes()) {
				if(nest.getOID()>max)
					max=nest.getOID();
				pst.setInt(1, nest.getOID());
				pst.setInt(2, schema.getOID());
				pst.setString(3,nest.getName());
				pst.setInt(4, nest.getTop().getOID());
				pst.setInt(5, nest.getNested().getOID());

				pst.executeUpdate();
			}
			pst.close();	
			
			this.ie.storeSkolem("Nest",max,schema.getOID());
			max=-1;
			
			/*alla fine posso chiudere i due PreparedStatement per gli inserimenti degli attributi dei complex type*/
			insertField.close();
			insertRef.close();
			
			/*come ultima cosa vado a caricare le FK e relativi componenti*/
			String insertFK="INSERT INTO SM_FOREIGNKEY VALUES(?,?,?,?,?,?,?,null,null)";
			pst=con.prepareStatement(insertFK);
			
			for(ForeignKey fk:schema.getForeignKeys()) {
				if(fk.getOID()>max)
					max=fk.getOID();
				pst.setInt(1, fk.getOID());
				pst.setInt(2, schema.getOID());
				pst.setString(3, schema.getName());
				
				if(fk.getType()==ForeignKey.TableToTable) {
					pst.setInt(4, fk.getFromOID());
					pst.setInt(5, fk.getToOID());
					pst.setNull(6, java.sql.Types.INTEGER);
					pst.setNull(7, java.sql.Types.INTEGER);
				}
				else if(fk.getType()==ForeignKey.TableToAbstract) {
					pst.setInt(4, fk.getFromOID());
					pst.setNull(5, java.sql.Types.INTEGER);
					pst.setNull(6, java.sql.Types.INTEGER);
					pst.setInt(7, fk.getToOID());					
				}
				else if(fk.getType()==ForeignKey.AbstractToTable) {
					pst.setNull(4, java.sql.Types.INTEGER);
					pst.setInt(5, fk.getToOID());
					pst.setInt(6, fk.getFromOID());		
					pst.setNull(7, java.sql.Types.INTEGER);
				}
				else if(fk.getType()==ForeignKey.AbstractToAbstract) {
					pst.setNull(4, java.sql.Types.INTEGER);
					pst.setNull(5, java.sql.Types.INTEGER);
					pst.setInt(6, fk.getFromOID());
					pst.setInt(7, fk.getToOID());					
				}
				
				pst.executeUpdate();				
			}
			pst.close();
			
			this.ie.storeSkolem("ForeignKey",max,schema.getOID());
			max=-1;
			
			/*ora i componenti*/
			String insertComponent="INSERT INTO SM_COMPONENTOFFOREIGNKEY VALUES(?,?,?,?,?,?)";
			pst=con.prepareStatement(insertComponent);
			for(ForeignKey fk: schema.getForeignKeys()) {
				for(ComponentOfForeignKey c: fk.getComponents()) {
					if(c.getOID()>max)
						max=c.getOID();
					pst.setInt(1,c.getOID());
					pst.setInt(2,schema.getOID());
					pst.setString(3,c.getName());
					pst.setInt(4,fk.getOID());
					pst.setInt(5, c.getFromOID());
					pst.setInt(6, c.getToOID());					
					
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("ComponentOfForeignKey",max,schema.getOID());
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	private int storeLexical(Vector<ComplexType> complexes,ORSchema schema,PreparedStatement insertField,PreparedStatement insertRef) throws SQLException{
		int max=-1;
		for(ComplexType type:complexes ) {
			//inserisci lexicalabstract
			for(Field field:type.getFields()) {
				if(field.getOid()>max)
					max=field.getOid();
				insertField.setInt(1,field.getOid());
				insertField.setInt(2,schema.getOID());
				insertField.setString(3,field.getName());
				insertField.setBoolean(4, field.isNullable());
				insertField.setBoolean(5,field.isIdentifier());
				insertField.setString(6, field.getType());
				insertField.setInt(7,type.getOID());
				
				insertField.executeUpdate();
			}			
		}		
		return max;
	}
	
	private int storeAbstract(Vector<ComplexType> complexes,ORSchema schema,PreparedStatement insertField,PreparedStatement insertRef) throws SQLException{
		int max=-1;
		for(ComplexType type:complexes ) {
			//now abstract attribute
			for(Reference ref:type.getReferences()) {
				if(ref.getOid()>max)
					max=ref.getOid();
				insertRef.setInt(1,ref.getOid());
				insertRef.setInt(2,schema.getOID());
				insertRef.setString(3,ref.getName());
				insertRef.setBoolean(4, ref.isNullable());
				insertRef.setInt(5,type.getOID());
				insertRef.setInt(6,ref.getClassTo().getOid());

				insertRef.executeUpdate();
			}
		}	
		return max;
	}
	

	
	public void storeInstance(ORSchema schema,int instanceOID) throws PersistenceException {
		try {
			/*sempre massimo per le tabelle di skolem*/
			int max=-1;
			/*come al solito sfrutto gli store gi definiti*/
			RelationalSchema relSchema=new RelationalSchema(schema.getOID(),schema.getName());
			for(Table table:schema.getTables()) {
				relSchema.addTable(table);
			}			
			//faccio lo store
			Relational rel=new Relational(this.con,this.ie);
			rel.storeInstance(relSchema, instanceOID);
			//facilito il gc
			relSchema=null;
			
			/*faccio la stessa cosa per l'OO*/
			OOSchema ooSchema=new OOSchema(schema.getOID(),schema.getName());
			for(ORClass o:schema.getORClasses()) {
				ooSchema.addOOClass(o);
			}
			ooSchema.setGeneralizations(schema.getGeneralizations());
			ooSchema.setObjectGeneralization(schema.getObjectGeneralization());
			ObjectOriented oo=new ObjectOriented(this.con,this.ie);
			oo.storeOOInstance(ooSchema, instanceOID);
			//facilito il gc
			ooSchema=null;			
			
			/*adesso devo fare solamente :) la parte prettamente OR*/
			String insertAbstractComponent=new String("INSERT INTO I_SM_ABSTRACTATTRIBUTE VALUES (?,?,?,null,?,null,?)");
			PreparedStatement pst=this.con.prepareStatement(insertAbstractComponent);
			
			for(ORTable table:schema.getTables()) {
				for(ORTuple tuple : table.getORTuples()) {
					for(ORReferenceComponent ref:tuple.getReferenceComponents()) {
						if(ref.getOID()>max)
							max=ref.getOID();
						pst.setInt(1,ref.getOID());
						pst.setInt(2, ref.getRef().getOID());
						pst.setInt(3, instanceOID);
						pst.setInt(4, tuple.getTupleOID());
						if(ref.getObject()!=null)
							pst.setInt(5, ref.getObject().getObjectOID());
						else
							pst.setNull(5, java.sql.Types.INTEGER);
						
						pst.executeUpdate();
					}
				}
			}
			pst.close();
			
			this.ie.storeSkolem("RefComponent",max,instanceOID);
			max=-1;
						
			/*ora bisogna fare la stessa cosa pure per i complex; questi sono quelli legati alle tabelle*/
			Vector<ComplexTypeComponent> complexComponents=new Vector<ComplexTypeComponent>();
			
			String insertComplex=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES VALUES (?,?,?,null,?)");
			pst=this.con.prepareStatement(insertComplex);
			
			for(ORTable table:schema.getTables()) {
				for(ORTuple tuple:table.getORTuples()) {
					for(ComplexTypeComponent complexComponent:tuple.getComplexComponents()) {
						if(complexComponent.getOID()>max)
							max=complexComponent.getOID();
						pst.setInt(1,complexComponent.getOID());
						pst.setInt(2,complexComponent.getComplex().getOID());
						pst.setInt(3,instanceOID);
						pst.setInt(4,tuple.getTupleOID());
						
						pst.executeUpdate();
						complexComponents.add(complexComponent);
					}
				}
			}
			pst.close();
				
			/*questi sono quelli legati alle classi*/
			insertComplex=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES VALUES (?,?,?,?,null)");
			pst=this.con.prepareStatement(insertComplex);
			
			for(ORClass orClass:schema.getORClasses()) {
				for(ORObject obj:orClass.getORObjects()) {
					for(ComplexTypeComponent complexComponent:obj.getComplexComponets()) {
						if(complexComponent.getOID()>max)
							max=complexComponent.getOID();
						pst.setInt(1,complexComponent.getOID());
						pst.setInt(2,complexComponent.getComplex().getOID());
						pst.setInt(3,instanceOID);
						pst.setInt(4,obj.getObjectOID());
						
						pst.executeUpdate();
						complexComponents.add(complexComponent);
					}
				}
			}
			pst.close();
			
			/*questi sono quelli non legati a niente*/
			insertComplex=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES VALUES (?,?,?,null,null)");
			pst=this.con.prepareStatement(insertComplex);
			
			for(ComplexType complex:schema.getAllNested()) {
				for(ComplexTypeComponent complexComponent:complex.getComponents()) {
					if(complexComponent.getOID()>max)
						max=complexComponent.getOID();
					pst.setInt(1,complexComponent.getOID());
					pst.setInt(2,complexComponent.getComplex().getOID());
					pst.setInt(3,instanceOID);
					
					pst.executeUpdate();
					complexComponents.add(complexComponent);
				}
			}
			pst.close();
			
			this.ie.storeSkolem("ComplexComponent",max,instanceOID);
			max=-1;
						
			/*qui devo caricare ankora tutti gli attributi lessicali e abstract*/
			String insertNest=new String("INSERT INTO I_SM_NEST VALUES (?,?,?,?,?)");
			pst=this.con.prepareStatement(insertNest);	
			
			for(Nest nest:schema.getNestes()) {
				for(NestComponent nestComponent:nest.getNestedComponent()) {
					if(nestComponent.getOID()>max)
						max=nestComponent.getOID();
					pst.setInt(1, nestComponent.getOID());
					pst.setInt(2, nest.getOID());
					pst.setInt(3, instanceOID);
					pst.setInt(4, nestComponent.getTop().getOID());
					pst.setInt(5, nestComponent.getNested().getOID());
					
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("NestComponent",max,instanceOID);
			max=-1;
			
			/*tutti I_Lexical e I_Abstratc per tutti i complex type*/
			String insertComponent=new String("INSERT INTO I_SM_LEXICAL VALUES(?,?,?,?,null,null,null,?,null)");
			pst=this.con.prepareStatement(insertComponent);
			
			for(ComplexTypeComponent component:complexComponents) {
				for(ValueComponent value:component.getValues()) {
					if(value.getOid()>max)
						max=value.getOid();
					pst.setInt(1, value.getOid());
					pst.setInt(2, value.getOoField().getOid());
					pst.setInt(3, instanceOID);
					if(value.getValue()!=null)
						pst.setString(4, value.getValue());
					else 
						pst.setNull(4, java.sql.Types.VARCHAR);
					pst.setInt(5, component.getOID());
					
					pst.executeUpdate();
				}				
			}
			pst.close();
			
			this.ie.storeSkolem("FieldComponent",max,instanceOID);
			max=-1;
			
			/*faccio anche I_Abstract*/
			insertComponent=new String("INSERT INTO I_SM_ABSTRACTATTRIBUTE VALUES(?,?,?,?,null,null,?)");
			pst=this.con.prepareStatement(insertComponent);
			
			for(ComplexTypeComponent component:complexComponents) {
				for(ReferenceComponent ref:component.getReferencesComponets()) {
					if(ref.getOid()>max)
						max=ref.getOid();
					pst.setInt(1, ref.getOid());
					pst.setInt(2, ref.getOoReference().getOid());
					pst.setInt(3, instanceOID);
					pst.setInt(4, component.getOID());
					if(ref.getOoObjectTo()!=null)
						pst.setInt(5, ref.getOoObjectTo().getObjectOID());
					else
						pst.setNull(5, java.sql.Types.INTEGER);
					
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("RefComponent",max,instanceOID);
			max=-1;
			
			/*per le istanze di fk faccio lo stesso ragionamento che avevo fatto per le istanze di fk nel relational, per maggiori
			 * info vedere la tesi*/
			String query=new String("SELECT I1.I_AGGREGATIONOID AS I_AGGREGATIONFROMOID,I2.I_AGGREGATIONOID AS I_AGGREGATIONTOOID," +
					"I1.I_ABSTRACTOID AS I_ABSTRACTFROMOID,I2.I_ABSTRACTOID AS I_ABSTRACTTOOID," +
					"I1.OID AS I_COMPONENTFROM,I2.OID AS I_COMPONENTTO FROM " +
					"(SELECT OID,VALUE,I_AGGREGATIONOID,I_ABSTRACTOID FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I1," +
					"(SELECT OID,VALUE,I_AGGREGATIONOID,I_ABSTRACTOID FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I2 " +
					"WHERE I1.VALUE=I2.VALUE");
			pst=this.con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			pst.setInt(3, instanceOID);
			
			PreparedStatement insertFK=this.con.prepareStatement("INSERT INTO I_SM_FOREIGNKEY VALUES(?,?,?,?,?,?,?,NULL,NULL)");
			PreparedStatement insertComponentFK=this.con.prepareStatement("INSERT INTO I_SM_COMPONENTOFFOREIGNKEY VALUES(?,?,?,?,?,?)");
			int i_fkOId=ie.getNewOID("ForeignKeyInstance");
			int oidOriginario=i_fkOId;
			int i_cfkOId=ie.getNewOID("ComponentOFForeignKeyInstance");
			
			for(ForeignKey fk:schema.getForeignKeys()) {
				boolean first=true;
				Vector<Integer> identificatiInseriti=new Vector<Integer>();
				int temp=0;
				
				for(ComponentOfForeignKey cFK:fk.getComponents()) {					
					pst.setInt(2,cFK.getFromOID());
					pst.setInt(4,cFK.getToOID());
					ResultSet rs=pst.executeQuery();
					
					while(rs.next()) {
						/*inserisco una istanza di FK solo al primo component, quindi*/
						if(first) {
							insertFK.setInt(1,i_fkOId);
							insertFK.setInt(2,fk.getOID());
							insertFK.setInt(3,instanceOID);
							if(fk.getType()==ForeignKey.TableToTable) {
								insertFK.setInt(4,rs.getInt(1));
								insertFK.setInt(5,rs.getInt(2));
								insertFK.setNull(6, java.sql.Types.INTEGER);
								insertFK.setNull(7, java.sql.Types.INTEGER);
							}
							else if(fk.getType()==ForeignKey.TableToAbstract){
								insertFK.setInt(4,rs.getInt(1));
								insertFK.setNull(5, java.sql.Types.INTEGER);
								insertFK.setNull(6, java.sql.Types.INTEGER);
								insertFK.setInt(7,rs.getInt(4));								
							}
							else if(fk.getType()==ForeignKey.AbstractToTable){
								insertFK.setNull(4, java.sql.Types.INTEGER);
								insertFK.setInt(5,rs.getInt(2));		
								insertFK.setInt(6,rs.getInt(3));
								insertFK.setNull(7, java.sql.Types.INTEGER);														
							}
							else if(fk.getType()==ForeignKey.AbstractToAbstract){
								insertFK.setNull(4, java.sql.Types.INTEGER);
								insertFK.setNull(5, java.sql.Types.INTEGER);		
								insertFK.setInt(6,rs.getInt(3));		
								insertFK.setInt(7,rs.getInt(4));																				
							}							
							
							insertFK.executeUpdate();
							identificatiInseriti.add(i_fkOId);
							i_fkOId++;
							
							/*per ogni componente invece faccio l'insert dei component*/
							insertComponentFK.setInt(1,i_cfkOId);
							insertComponentFK.setInt(2,cFK.getOID());
							insertComponentFK.setInt(3,instanceOID);
							insertComponentFK.setInt(4,i_fkOId-1);
							insertComponentFK.setInt(5,rs.getInt(5));
							insertComponentFK.setInt(6,rs.getInt(6));
							
							insertComponentFK.executeUpdate();							
							i_cfkOId++;		
						}
						else {
							/*devo inserire solamente i componenti*/
							/*per ogni componente invece faccio l'insert dei component*/
							insertComponentFK.setInt(1,i_cfkOId);
							insertComponentFK.setInt(2,cFK.getOID());
							insertComponentFK.setInt(3,instanceOID);
							insertComponentFK.setInt(4,identificatiInseriti.get(temp));
							temp++;
							insertComponentFK.setInt(5,rs.getInt(5));
							insertComponentFK.setInt(6,rs.getInt(6));
							
							insertComponentFK.executeUpdate();							
							i_cfkOId++;	
						}						
					}		
					/*ho finito di eseguire le query e relativi inserimenti per il primo 
					 * component, quindi se ce ne dovessero essere altri, non vanno reinserite
					 * le istanze di fk*/
					first=false;
					temp=0;
				}
 			}
			insertFK.close();
			insertComponentFK.close();
			pst.close();
			
			if((i_fkOId-1)>oidOriginario) {
				this.ie.storeSkolem("ForeignKeyInstance",i_fkOId-1, instanceOID);	
				this.ie.storeSkolem("ComponentOFForeignKeyInstance",i_cfkOId-1, instanceOID);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'import");
		} 	
	}
	
	public ORSchema loadORSchema(int schemaOID) throws PersistenceException {
		try {
			//RelationalSchema temp=ie.loadSchema(schemaOID);
			relSchema=ie.loadSchema(schemaOID);

			ORSchema result=new ORSchema(relSchema.getOID(),relSchema.getName());
			/*qui sono costretto a creare altri oggetti perch la classe parent non va bene
			 * cio: in un ORSchema non posso mettere una Table normale(se no non potr ad esempio aggiungere
			 * cose dell'OR), quindi creo una ORTable uguale
			 * Al contrario le cose funzionano perch una ORTable "is a" Table*/
			for(Table table:relSchema.getTables()) {
				ORTable t=new ORTable(table.getOID(),table.getName());
				t.setColumns(table.getColumns());
				result.addORTable(t);
			}
			/*Ho caricato la parte relativa al modelllo Relazionale*/
			ooSchema=ie.loadOOSchema(schemaOID);
			for(OOClass o:ooSchema.getOOClasses()) {
				ORClass oc=new ORClass(o.getOid(),o.getName());
				oc.setFields(o.getFields());
				result.addORClass(oc);
			}
			for(OOClass o:ooSchema.getOOClasses()) {
				for(Reference ref:o.getReferences()) {
					ORClass to=result.getClassByName(ref.getClassTo().getName());
					Reference newRef=new Reference(ref.getOid(),ref.getName(),ref.isNullable(),to);
					ORClass from=result.getClassByName(o.getName());
					from.addReference(newRef);
				}
			}
			for(OOGeneralization gen:ooSchema.getGeneralizations()) {
				OOClass parent=result.getClassByOID(gen.getParent().getOid());
				OOGeneralization newGen=new OOGeneralization(gen.getOID(),gen.getName(),gen.isTotal(),parent);
				for(OOChild child:gen.getChilds()) {
					OOClass childClass=result.getClassByOID(child.getChild().getOid());
					OOChild newChild=new OOChild(child.getOID(),childClass);
					newGen.addChild(newChild);
				}
				result.addOOGeneralization(newGen);
			}
			/*Ho caricato anche il modello OO*/

			/*Mi mancano AbstractComponentofAggregation, tutte le FK e relativi componenti, i Complext Type e i Nest*/
			//1)Ref tra table e typed table
			String query=new String("SELECT OID,NAME,ISNULLABLE,AGGREGATIONOID,ABSTRACTTOOID " +
			"FROM SM_ABSTRACTATTRIBUTE WHERE SCHEMEOID=? AND AGGREGATIONOID IS NOT NULL");
			PreparedStatement ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			ResultSet rs=ps.executeQuery();

			while(rs.next()) {
				ORTable from=result.getTableByOID(rs.getInt(4));
				ORClass to=result.getClassByOID(rs.getInt(5));

				if(from!=null && to!=null) {
					boolean isNullable=rs.getBoolean(3);
					if(rs.wasNull())
						isNullable=true;					
					ORReference ref=new ORReference(rs.getInt(1),rs.getString(2),isNullable,from,to);
					from.addORReference(ref);
				}
			}
			rs.close();
			ps.close();

			//2)FK
			query=new String("SELECT OID,NAME,AGGREGATIONFROMOID,AGGREGATIONTOOID,ABSTRACTFROMOID,ABSTRACTTOOID " +
			"FROM SM_FOREIGNKEY WHERE SCHEMEOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();

			while(rs.next()) {
				int tableFromOID=rs.getInt("AGGREGATIONFROMOID");
				int tableToOID=rs.getInt("AGGREGATIONTOOID");
				int classFromOID=rs.getInt("ABSTRACTFROMOID");
				int classToOID=rs.getInt("ABSTRACTTOOID");

				ForeignKey fk;
				if(tableFromOID!=0 && tableToOID!=0) {
					fk=new ForeignKey(rs.getInt(1),rs.getString(2),result.getTableByOID(tableFromOID),result.getTableByOID(tableToOID));
					result.addForeignKey(fk);
				}
				else if(tableFromOID!=0 && classToOID!=0) {
					fk=new ForeignKey(rs.getInt(1),rs.getString(2),result.getTableByOID(tableFromOID),result.getClassByOID(classToOID));
					result.addForeignKey(fk);
				}
				else if(classFromOID!=0 && tableToOID!=0){
					fk=new ForeignKey(rs.getInt(1),rs.getString(2),result.getClassByOID(classFromOID),result.getTableByOID(tableToOID));
					result.addForeignKey(fk);				
				}
				else if(classFromOID!=0 && classToOID!=0){
					fk=new ForeignKey(rs.getInt(1),rs.getString(2),result.getClassByOID(classFromOID),result.getClassByOID(classToOID));
					result.addForeignKey(fk);	
				}				
			}
			rs.close();
			ps.close();

			//3)components of FK
			query=new String("SELECT OID, NAME, FOREIGNKEYOID, LEXICALFROMOID, LEXICALTOOID " +
					"FROM SM_COMPONENTOFFOREIGNKEY WHERE SCHEMEOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();

			while(rs.next()) {
				int from=rs.getInt("LEXICALFROMOID");
				int to=rs.getInt("LEXICALTOOID");

				ComponentOfForeignKey cFK;
				ForeignKey fk=result.getForeignKeyByOID(rs.getInt("FOREIGNKEYOID"));

				if(fk.getType()==ForeignKey.TableToTable) {
					cFK=new ComponentOfForeignKey(rs.getInt(1),rs.getString(2),result.getColumnByOID(from),result.getColumnByOID(to));
					fk.add(cFK);
				}
				else if(fk.getType()==ForeignKey.TableToAbstract) {
					cFK=new ComponentOfForeignKey(rs.getInt(1),rs.getString(2),result.getColumnByOID(from),result.getFieldByOID(to));
					fk.add(cFK);
				}
				else if(fk.getType()==ForeignKey.AbstractToTable) {
					cFK=new ComponentOfForeignKey(rs.getInt(1),rs.getString(2),result.getFieldByOID(from),result.getColumnByOID(to));
					fk.add(cFK);
				}
				else if(fk.getType()==ForeignKey.AbstractToAbstract) {
					cFK=new ComponentOfForeignKey(rs.getInt(1),rs.getString(2),result.getFieldByOID(from),result.getFieldByOID(to));
					fk.add(cFK);
				}				
			}
			rs.close();
			ps.close();

			/*Mancano ankora complex type e relativi field e reference relativi nest*/
			//4)complex type
			query=new String("SELECT OID,NAME,ISOPTIONAL,ISNULLABLE,ABSTRACTOID,AGGREGATIONOID " +
			"FROM SM_STRUCTOFATTRIBUTES WHERE SCHEMEOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();
			Vector<ComplexType> complexes=new Vector<ComplexType>();
			Vector<ComplexType> tipiSlegati=new Vector<ComplexType>();

			while(rs.next()) {
				ComplexType complex=new ComplexType(rs.getInt(1),rs.getString(2),rs.getBoolean(3),rs.getBoolean(4));
				int classTo=rs.getInt("ABSTRACTOID");
				int tableTo=rs.getInt("AGGREGATIONOID");

				if(tableTo!=0){
					result.getTableByOID(tableTo).addComplex(complex);
					complexes.add(complex);
				}
				else if(classTo!=0) {
					result.getClassByOID(classTo).addComplex(complex);
					complexes.add(complex);
				}
				else if(classTo==0 && tableTo==0){
					tipiSlegati.add(complex);
				}				
			}
			rs.close();
			ps.close();
			
			//5)Field dei tipi complessi
			query=new String("SELECT OID,NAME,ISNULLABLE,ISIDENTIFIER,TYPE,STRUCTOFATTRIBUTESOID " +
			"FROM SM_LEXICAL WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID IS NOT NULL");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();

			while(rs.next()) {
				int comlexOID=rs.getInt("STRUCTOFATTRIBUTESOID");
				boolean isNullable=rs.getBoolean(3);
				if(rs.wasNull())
					isNullable=true;	

				Field field=new Field(rs.getInt(1),rs.getString(2),rs.getString("TYPE"),isNullable,rs.getBoolean(4));
				/*cerco di caricare il field o ai tipi presenti in tabelle o abstract*/
				for(ComplexType complex:complexes) {
					if(complex.getOID()==comlexOID)
						complex.addField(field);
				}
				/*ma anche a quelli che sono slegati(lo saranno fino a che non carico i Nest)*/
				for(ComplexType complex:tipiSlegati) {
					if(complex.getOID()==comlexOID)
						complex.addField(field);
				}
			}
			rs.close();
			ps.close();
			
			//6)reference dei tipi complessi
			query=new String("SELECT OID,NAME,ISNULLABLE,STRUCTOFATTRIBUTESOID,ABSTRACTTOOID " +
			"FROM SM_ABSTRACTATTRIBUTE WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID IS NOT NULL");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();

			while(rs.next()) {
				int complexOID=rs.getInt("STRUCTOFATTRIBUTESOID");
				int classTo=rs.getInt("ABSTRACTTOOID");
				boolean isNullable=rs.getBoolean(3);
				if(rs.wasNull())
					isNullable=true;

				Reference ref=new Reference(rs.getInt(1),rs.getString(2),isNullable,result.getClassByOID(classTo));
				/*stessa cosa dei field*/
				for(ComplexType complex:complexes) {
					if(complex.getOID()==complexOID)
						complex.addReference(ref);
				}			
				for(ComplexType complex:tipiSlegati) {
					if(complex.getOID()==complexOID)
						complex.addReference(ref);
				}
			}
			rs.close();
			ps.close();
			
			//7) Nest
			query=new String("SELECT OID,NAME,STRUCTOFATTRIBUTESTOPOID,STRUCTOFATTRIBUTESNESTEDOID FROM SM_NEST WHERE SCHEMEOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, result.getOID());

			rs=ps.executeQuery();

			while(rs.next()) {
				int topOID=rs.getInt(3);
				int nestedOID=rs.getInt(4);
				ComplexType top=null,nested=null;

				for(ComplexType complex:complexes) {
					if(complex.getOID()==topOID)
						top=complex;
					if(complex.getOID()==nestedOID)
						nested=complex;
				}		
				for(ComplexType complex:tipiSlegati) {
					if(complex.getOID()==topOID)
						top=complex;
					if(complex.getOID()==nestedOID)
						nested=complex;
				}
				Nest nest=new Nest(rs.getInt(1),rs.getString(2),top,nested);
				result.addNest(nest);
			}
			rs.close();
			ps.close();

			return result;
		}	
		catch(SQLException sql){
			sql.printStackTrace();
			throw new PersistenceException("Errore nel load OR");			
		} 
	}

	public ORSchema loadORInstance(ORSchema schema,int instanceOID)throws PersistenceException {
		if(relSchema==null)
			throw new PersistenceException("Va caricato prima lo schema");
		ie.loadInstance(relSchema, instanceOID);
		/*qui sono costretto a creare altri oggetti perch la classe parent non va bene
		 * cio: in un ORSchema non posso mettere una Table normale(se no non potr ad esempio aggiungere
		 * cose dell'OR), quindi creo una ORTable uguale
		 * Al contrario le cose funzionano perch una ORTable "is a" Table*/
		for(Table table:relSchema.getTables()) {
			ORTable orTable=schema.getTableByOID(table.getOID());
			for(Tuple tuple:table.getTuples()) {
				ORTuple orTuple=new ORTuple(tuple.getTupleOID(),orTable);
				orTuple.setTupleComponents(tuple.getTupleComponents());
				orTable.addORTuple(orTuple);
			}
		}
	
		/*stessa cosa del Relazionale*/
		if(ooSchema==null)
			throw new PersistenceException("Va caricato prima lo schema");
		ie.loadOOInstance(ooSchema, instanceOID);
		for(OOClass o:ooSchema.getOOClasses()) {
			ORClass oc=schema.getClassByOID(o.getOid());
			for(OOObject obj:o.getOOObjects()) {
				ORObject orObj=new ORObject(obj.getObjectOID(),oc);
				orObj.setValueComponent(obj.getValueComponents());
				orObj.setReferenceComponents(obj.getReferenceComponents());
				oc.addORObject(orObj);
			}			
		}
		for(ObjectGeneralization objGen:ooSchema.getObjectGeneralization()) {
			ObjectGeneralization newObjeGen=new ObjectGeneralization(objGen.getGeneralization(),
					objGen.getChild(),objGen.getParentObject(),objGen.getChildObject());
			schema.addObjectGeneralization(newObjeGen);			
		}
		
		/*adesso devo caricare le istanze di Ref tra table e typed table, le istanze degli struct of 
		 * attributes e relativi valori e le istanze di Nest(le istanze di FK non mi servono)*/
		
		try {
			//1)I_SM_ABSTRACT
			String query=new String("SELECT OID,ABSTRACTATTRIBUTEOID,I_AGGREGATIONOID," +
					"I_ABSTRACTTOOID FROM I_SM_ABSTRACTATTRIBUTE WHERE INSTANCEOID=? AND I_AGGREGATIONOID IS NOT NULL");
			PreparedStatement pst=con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			
			ResultSet rs=pst.executeQuery();
			while(rs.next()) {
				ORReference ref=schema.getORReferenceByOID(rs.getInt(2));
				ORTuple tuple=schema.getORTuple(rs.getInt(3));
				OOObject to=ref.getTo().getOOObjectByOID(rs.getInt(4));
				
				/*Aggiunta 25/05/07, controllo valori nulli*/
				if(!ref.isNullable() && to==null)
					throw new PersistenceException("Un valore della tupla di OID "+tuple.getTupleOID()+
							" appartenente alla tabella "+tuple.getTable().getName()+" ha una reference null, ma la" +
							" Reference corrispondente NON ammette valori nulli");
				
				ORReferenceComponent refComponent=new ORReferenceComponent(rs.getInt(1),ref,tuple,to);
				tuple.addORReferenceComponent(refComponent);
			}
			rs.close();
			pst.close();
			
			//2)istanze di struct
			query=new String("SELECT OID,STRUCTOFATTRIBUTESOID,I_ABSTRACTOID,I_AGGREGATIONOID " +
					"FROM I_SM_STRUCTOFATTRIBUTES WHERE INSTANCEOID=?");
			pst=con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			Vector<ComplexTypeComponent> componenti=new Vector<ComplexTypeComponent>();
			
			rs=pst.executeQuery();
			while(rs.next()) {
				ComplexType complex=schema.getComplexTypeByOID(rs.getInt(2));
				ComplexTypeComponent complexComponent=new ComplexTypeComponent(rs.getInt(1),complex);
				componenti.add(complexComponent);
				/*ora devo capire a che appartiene questa istanza di complex type*/
				int tupleOID=rs.getInt(4);
				int objOID=rs.getInt(3);
				
				if(tupleOID!=0 && objOID==0) {
					ORTuple tuple=schema.getORTuple(tupleOID);
					tuple.addComplexTypeComponent(complexComponent);
				}
				else if(tupleOID==0 && objOID!=0) {
					ORObject obj=schema.getORObjectByOID(objOID);
					obj.addComplexTypeComponent(complexComponent);
				}
				else {
					complex.add(complexComponent);
				}
			}
			rs.close();
			pst.close();
			
			//3) caricare i valori dei componenti
			query=new String("SELECT OID,LEXICALOID,VALUE,I_STRUCTOFATTRIBUTESOID " +
					"FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID IS NOT NULL");
			pst=con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			
			rs=pst.executeQuery();
			while(rs.next()) {
				ComplexTypeComponent component=getComplexComponent(componenti,rs.getInt(4));
				if(component!=null) {
					Field to=schema.getFieldByOID(rs.getInt(2));
					
					/*Aggiunta 25/05/07, controllo valori nulli*/
					if(!to.isNullable() && rs.getString(3)==null)
						throw new PersistenceException("Un valore del campo dell'istanza " +
								"di un elemento complesso "+component.getComplex().getName()+" ha un valore null, ma il" +
								"campo corrispondente NON ammette valori nulli");
					
					ValueComponent value=new ValueComponent(rs.getInt(1),rs.getString(3),to,null);
					component.addValueComponent(value);
				}
			}
			rs.close();
			pst.close();
			
			//3')carico le eventuali Ref 
			query=new String("SELECT OID,ABSTRACTATTRIBUTEOID,I_STRUCTOFATTRIBUTESOID,I_ABSTRACTTOOID FROM I_SM_ABSTRACTATTRIBUTE" +
					" WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID IS NOT NULL");
			pst=con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			
			rs=pst.executeQuery();
			while(rs.next()) {
				ComplexTypeComponent component=getComplexComponent(componenti,rs.getInt(3));
				Reference ref=schema.getReferenceByOID(rs.getInt(2));
				
				/*Aggiunta 25/05/07, controllo valori nulli*/
				if(!ref.isNullable() && rs.getString(4)==null)
					throw new PersistenceException("Un valore della referenza dell'istanza " +
							"di un elemento complesso "+component.getComplex().getName()+" ha un valore null, ma la" +
							"Reference corrispondente NON ammette valori nulli");
				
				ReferenceComponent refComponent;
				if(rs.getString(4)==null) {
					refComponent=new ReferenceComponent(rs.getInt(1),null,ref);
					component.addReferenceComponent(refComponent);
				}
				else {
					refComponent=new ReferenceComponent(rs.getInt(1),schema.getORObjectByOID(rs.getInt(4)),ref);
					component.addReferenceComponent(refComponent);
				}					
			}
			rs.close();
			pst.close();
			
			//4)istanze di nest
			query=new String("SELECT OID,NESTOID,I_STRUCTOFATTRIBUTESTOPOID,I_STRUCTOFATTRIBUTESNESTEDOID" +
					" FROM I_SM_NEST WHERE INSTANCEOID=?");
			pst=con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			
			rs=pst.executeQuery();
			while(rs.next()) {
				Nest nest=schema.getNestByOID(rs.getInt(2));
				NestComponent nestComponent=new NestComponent(rs.getInt(1),getComplexComponent(componenti, rs.getInt(3)),
						getComplexComponent(componenti, rs.getInt(4)));
				nest.add(nestComponent);
			}
			rs.close();
			pst.close();
			
			/*le istanze delle FK non vanno caricate*/
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel load delle istanze");
		}		
		
		return schema;
	}
	
	private ComplexTypeComponent getComplexComponent(Vector<ComplexTypeComponent> componenti,int OID) {
		for(ComplexTypeComponent component:componenti){
			if(component.getOID()==OID)
				return component;
		}
		return null;
	}
}

