package org.modelgen.importexport.persistence;

import org.modelgen.importexport.relational.model.Column;
import org.modelgen.importexport.relational.model.ComponentOfForeignKey;
import org.modelgen.importexport.relational.model.ForeignKey;
import org.modelgen.importexport.relational.model.RelationalSchema;
import org.modelgen.importexport.relational.model.Table;
import org.modelgen.importexport.relational.model.Tuple;
import org.modelgen.importexport.relational.model.TupleComponent;
import org.modelgen.importexport.utilities.PersistenceException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

/**This is merely a utility class to do the "dirt" job
 * 
 * Normalmente questa classe non dovrebbe essere neanche vista a meno che non sia cambiata la struttura
 * del database di ModelGen
 *
 */
class Relational {
	private Connection con;
	private ImportExportPersistence ie;
	
	public Relational(Connection con,ImportExportPersistence ie) {
		this.con=con;
		this.ie=ie;
	}
	
	/**
	 * This method load a schema from the dictionary and instantiate modelgen
	 * classes to export the schema. 
	 * 
	 * @param schemaOID the OID of the scheme as stored in ModelGen
	 * @return the schema loaded
	 * @throws PersistenceException any error with the ModelGen db
	 */
	public RelationalSchema loadSchema(int schemaOID) throws PersistenceException {
		try {			
			String sqlQuery = "SELECT * FROM SchemeSM WHERE OID = " + schemaOID+ ";";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			if(!result.next())
				throw new PersistenceException("Non esiste lo schema indicato");
			String schemaName = result.getString("name");
			result.close();
			RelationalSchema relationalSchema = new RelationalSchema(schemaOID,schemaName);

			// now retrieve the table's'name
			sqlQuery = "SELECT oid, name FROM SM_Aggregation WHERE schemeOID = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);
			pst.setInt(1, schemaOID);
			result = pst.executeQuery();
			while (result.next()) {
				int tableOID = result.getInt("OID");
				String tableName = result.getString("name");
				Table t = new Table(tableOID, tableName);
				relationalSchema.addTable(t);
			}
			result.close();
			pst.close();
			
			// now, for each table retrieve all the columns
			for (Table table : relationalSchema.getTables()) {
				sqlQuery = "SELECT oid, name, ISIDENTIFIER, isNullable, type FROM SM_LEXICAL" +
						" WHERE schemeOID = ? AND aggregationOID = ?;";
				pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, schemaOID);
				pst.setInt(2, table.getOID());
				result = pst.executeQuery();
				while (result.next()) {
					int columnOID = result.getInt("oid");
					String columnName = result.getString("name");
					/*Attenzione: nota del 30/04/07
					 * se le procedure non hanno portato isNullable, cio 
					 * la colonna isNullable  null devo mettere controllo in pi*/
					boolean isNull = result.getBoolean("isNullable");
					if(result.wasNull())
						isNull=true;
					boolean isKey = result.getBoolean("ISIDENTIFIER");
					// here we will need a type cast with the target system
					String columnType = result.getString("type");
					Column column = new Column(columnOID, columnName,columnType,isNull,isKey);
					table.addColumn(column);
				}
			}
			
			// now instantiate the foreign keys
			// first: retrieve all the fk!
			sqlQuery = "SELECT fk.oid AS fkOID, fk.name AS fkName, " +
					"fk.aggregationFromOID AS fkTableFromOID, fk.aggregationToOID AS fkTableToOID " +
					"FROM SM_ForeignKey fk " + 
					"WHERE fk.schemeOID = ? AND AGGREGATIONFROMOID IS NOT NULL AND AGGREGATIONTOOID IS NOT NULL";
			pst = con.prepareStatement(sqlQuery);
			pst.setInt(1, schemaOID);
			result = pst.executeQuery();
			while (result.next()) {
				int fkOID = result.getInt("fkOID");
				String fkName =  result.getString("fkName");
				int tableFromOID = result.getInt("fkTableFromOID");
				int tableToOID = result.getInt("fkTableToOID");
				
				Table tableFrom = relationalSchema.getTableByOID(tableFromOID);
				Table tableTo = relationalSchema.getTableByOID(tableToOID);
				// modify/refact following 3 lines of code
				if (tableFrom==null || tableTo==null) 
					throw new PersistenceException("Unresolved load of a FK: " +
						"target table seems not to exists!");
				ForeignKey fk = new ForeignKey(fkOID, fkName,tableFrom,tableTo);
				relationalSchema.addForeignKey(fk);
			}
			result.close();
			pst.close();
			
			// now the component of foreign key
			// for each table, go throu all the fk

			for (ForeignKey fk : relationalSchema.getForeignKeys()) {
				sqlQuery = "SELECT cfk.oid AS cfkOID, cfk.name AS cfkName, " +
				"cfk.LEXICALFROMOID AS cfkColumnFromOID, cfk.LEXICALTOOID AS cfkColumnToOID " + 
				"FROM SM_ComponentOfForeignKey cfk " +
				"WHERE cfk.schemeOID = ? AND cfk.foreignKeyOID = ?;";
				pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, schemaOID);
				pst.setInt(2, fk.getOID());
				result = pst.executeQuery();
				while (result.next()) {
					int cfkOID = result.getInt("cfkOID");
					String cfkName = result.getString("cfkName");
					int cfkColumnFromOID = result.getInt("cfkColumnFromOID");
					int cfkColumnToOID = result.getInt("cfkColumnToOID");
					Column columnFrom = relationalSchema.getColumnByOID(cfkColumnFromOID);
					Column columnTo = relationalSchema.getColumnByOID(cfkColumnToOID);
					ComponentOfForeignKey cfk = 
						new ComponentOfForeignKey(cfkOID, cfkName, columnFrom,columnTo);
					fk.add(cfk);
				}
				pst.close();
				result.close();
			}
			return relationalSchema;
			
		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out);
			throw new PersistenceException(sqle.toString());
		}		
	}
	
	/**Load the instances of the given schema.
	 * @param schema the schema
	 * @param instanceOID the instaceOID as stored in the ModelGen db
	 * @return the schema loaded
	 * @throws PersistenceException any error with the ModelGen db
	 */
	public RelationalSchema loadInstance(RelationalSchema schema,int instanceOID) throws PersistenceException {
		try {
			// for each table in the loaded schema, load its instances
			for (Table table : schema.getTables()) {
				String sqlQuery = "SELECT tuple.OID AS tupleOID " 
						+ "FROM i_SM_Aggregation tuple "
						+ "WHERE tuple.instanceOID = ? AND tuple.aggregationOID = ?;";
				PreparedStatement pst = con.prepareStatement(sqlQuery);
				pst.setInt(1, instanceOID);
				pst.setInt(2, table.getOID());
				ResultSet result = pst.executeQuery();
				Tuple tuple = null;
				while (result.next()) {
					int tupleOID = result.getInt("tupleOID");
					tuple = new Tuple(tupleOID, table);
					table.addTuple(tuple);
				}
				result.close();
				pst.close();
			}
			
			for (Table table : schema.getTables()) {
				for (Tuple tuple : table.getTuples()) {
					String sqlQuery = "SELECT cTuple.OID AS cTupleOID, cTuple.value AS cTupleValue, " +
							"cTuple.LEXICALOID AS cTupleColumn " 
					+ "FROM i_SM_Aggregation tuple JOIN I_SM_LEXICAL cTuple "
					+ "ON tuple.OID = cTuple.i_aggregationOID "
					+ "WHERE tuple.instanceOID = ? AND cTuple.instanceOID = ? "
					+ "AND cTuple.i_aggregationOID = ?;";
					PreparedStatement pst = con.prepareStatement(sqlQuery);
					pst.setInt(1, instanceOID);
					pst.setInt(2, instanceOID);
					pst.setInt(3, tuple.getTupleOID());
					ResultSet result = pst.executeQuery();
					while (result.next()){
						int cTupleOID = result.getInt("cTupleOID");
						String cTupleValue = result.getString("cTupleValue");
						//value potrebbe essere null, ma non  un problema
						Column column=table.getColumnByOID(result.getInt("cTupleColumn"));
						
						/*Aggiunta 25/05/07, controllo valori nulli*/
						if(!column.isNull() && cTupleValue==null)
							throw new PersistenceException("Un valore della tupla di OID "+tuple.getTupleOID()+" " +
									"appartenente alla tabella "+table.getName()+" ha un valore null, ma la" +
									" colonna corrispondente NON ammette valori nulli");
						
						TupleComponent cTuple = new TupleComponent(cTupleOID, cTupleValue, column);
						tuple.addTupleComponent(cTuple);
					}
				}
			}
			return schema;
			
		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out);
			throw new PersistenceException(sqle.toString());
		} 	
	}
	
	/**Store a RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeSchema(RelationalSchema schema) throws PersistenceException {	
		try {
			//memorizzo il valore massimo in modo da poter poi inserire 
			//l'oid massimo usato nelle tabelle di skolem
			int max=-1;	
			/*per ogni Table una Aggregation*/
			String insertAggregation="INSERT INTO SM_AGGREGATION VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertAggregation);

			for(Table t: schema.getTables()) {
				if(t.getOID()>max)
					max=t.getOID();
				pst.setInt(1,t.getOID());
				pst.setInt(2,schema.getOID());
				pst.setString(3,t.getName());
				pst.executeUpdate();
			}		
			pst.close();
			
			this.ie.storeSkolem("Table",max,schema.getOID());
			max=-1;

			String insertLexical="INSERT INTO SM_LEXICAL VALUES(?,?,?,?,null,?,?,null,null,?,null,null)";
			pst=this.con.prepareStatement(insertLexical);

			for(Table t: schema.getTables()) {
				/*adesso per ogni colonna di table un lexicalComponentofAggregation*/
				for(Column c: t.getColumns()) {
					if(c.getOID()>max)
						max=c.getOID();
					pst.setInt(1,c.getOID());
					pst.setInt(2,schema.getOID());
					pst.setString(3,c.getName());
					pst.setBoolean(4, c.isNull());
					pst.setBoolean(5, c.isKey());
					pst.setString(6,c.getType());
					
					pst.setInt(7,t.getOID());
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("Column",max,schema.getOID());
			max=-1;
			
			/*ora le Fk a livello di schema e relativi componenti*/
			String insertFK="INSERT INTO SM_FOREIGNKEY VALUES(?,?,?,?,?,null,null,null,null)";
			pst=this.con.prepareStatement(insertFK);
			String insertComponent="INSERT INTO SM_COMPONENTOFFOREIGNKEY VALUES(?,?,?,?,?,?)";
			PreparedStatement pst1=this.con.prepareStatement(insertComponent);


			for(ForeignKey fk: schema.getForeignKeys()) {
				if(fk.getOID()>max)
					max=fk.getOID();
				pst.setInt(1,fk.getOID());
				pst.setInt(2,schema.getOID());
				pst.setString(3,fk.getName());
				pst.setInt(4,fk.getFromOID());
				pst.setInt(5,fk.getToOID());
				pst.executeUpdate();
			}
			
			this.ie.storeSkolem("ForeignKey",max,schema.getOID());
			max=-1;

			for(ForeignKey fk: schema.getForeignKeys()) {
				for(ComponentOfForeignKey c: fk.getComponents()) {
					if(c.getOID()>max)
						max=c.getOID();
					pst1.setInt(1,c.getOID());
					pst1.setInt(2,schema.getOID());
					pst1.setString(3,c.getName());
					pst1.setInt(4,fk.getOID());
					pst1.setInt(5,c.getFromOID());
					pst1.setInt(6,c.getToOID());
					pst1.executeUpdate();
				}
			}
			pst.close();
			pst1.close();
			
			this.ie.storeSkolem("ComponentOfForeignKey",max,schema.getOID());
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	/**Store the instances of the given RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeInstance(RelationalSchema schema,int instanceOID) throws PersistenceException {
		try {
			int max=-1;
			String insertTuple="INSERT INTO I_SM_AGGREGATION VALUES(?,?,?)";
			PreparedStatement pst=this.con.prepareStatement(insertTuple);

			for(Table t: schema.getTables()) {
				for(Tuple tuple : t.getTuples()) {
					if(tuple.getTupleOID()>max)
						max=tuple.getTupleOID();
					pst.setInt(1,tuple.getTupleOID());
					pst.setInt(2,tuple.getTable().getOID());
					pst.setInt(3,instanceOID);
					pst.executeUpdate();
				}
			}
			pst.close();
			
			this.ie.storeSkolem("Tuple",max, instanceOID);
			max=-1;
			
			String insertComponent="INSERT INTO I_SM_LEXICAL VALUES(?,?,?,?,null,null,?,null,null)";
			pst=this.con.prepareStatement(insertComponent);

			for(Table t: schema.getTables()) {
				for(Tuple tuple : t.getTuples()) {
					for(TupleComponent tC: tuple.getTupleComponents()) {
						if(tC.getOid()>max)
							max=tC.getOid();
						pst.setInt(1,tC.getOid());
						pst.setInt(2,tC.getColumn().getOID());
						pst.setInt(3,instanceOID);
						if(tC.getValue()!=null)
							pst.setString(4,tC.getValue());
						else
							pst.setNull(4, java.sql.Types.VARCHAR);
						//il valore  sempre una stringa per me
						pst.setInt(5,tuple.getTupleOID());
						pst.executeUpdate();
					}
				}
			}
			pst.close();
			
			this.ie.storeSkolem("TupleComponent",max, instanceOID);
			max=-1;

			/*Ora devo inserire I_SM_FK e relativi componenti;
			 * non sono caricate a livello di codice ad oggetti per questioni di efficenza 
			 * si  infatti preferito sfruttare le caratteristiche dei DBMS e dei loro indici piuttosto che
			 * fare una ricerca lineare per ogni tupla per vedere le corrispondenze tra gli oggetti
			 * vedere la tesi per maggiori dettagli*/
			int i_fkOId=ie.getNewOID("ForeignKeyInstance");
			int i_cfkOId=ie.getNewOID("ComponentOFForeignKeyInstance");
			String query=new String("SELECT I1.I_AGGREGATIONOID AS I_AGGREGATIONFROMOID,I2.I_AGGREGATIONOID AS " +
					"I_AGGREGATIONFROMOID,I1.OID AS I_COMPONENTFROM,I2.OID AS I_COMPONENTTO FROM " +
					"(SELECT * FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I1, " +
					"(SELECT * FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I2" +
					" WHERE I1.VALUE=I2.VALUE");
			pst=this.con.prepareStatement(query);
			pst.setInt(1,instanceOID);
			pst.setInt(3,instanceOID);
			
			PreparedStatement insertFK=this.con.prepareStatement("INSERT INTO I_SM_FOREIGNKEY VALUES(?,?,?,?,?,NULL,NULL,NULL,NULL)");
			PreparedStatement insertComponentFK=this.con.prepareStatement("INSERT INTO I_SM_COMPONENTOFFOREIGNKEY VALUES(?,?,?,?,?,?)");
			
			for(ForeignKey fk:schema.getForeignKeys()) {
				boolean first=true;
				Vector<Integer> identificatiInseriti=new Vector<Integer>();
				int temp=0;
				
				for(ComponentOfForeignKey cFK:fk.getComponents()) {					
					pst.setInt(2,cFK.getFromOID());
					pst.setInt(4,cFK.getToOID());
					ResultSet rs=pst.executeQuery();
					
					while(rs.next()) {
						/*inserisco una istanza di FK solo al primo component, quindi*/
						if(first) {
							insertFK.setInt(1,i_fkOId);
							insertFK.setInt(2,fk.getOID());
							insertFK.setInt(3,instanceOID);
							insertFK.setInt(4,rs.getInt(1));
							insertFK.setInt(5,rs.getInt(2));
							
							insertFK.executeUpdate();
							identificatiInseriti.add(i_fkOId);
							i_fkOId++;
							
							/*per ogni componente invece faccio l'insert dei component*/
							insertComponentFK.setInt(1,i_cfkOId);
							insertComponentFK.setInt(2,cFK.getOID());
							insertComponentFK.setInt(3,instanceOID);
							insertComponentFK.setInt(4,i_fkOId-1);
							insertComponentFK.setInt(5,rs.getInt(3));
							insertComponentFK.setInt(6,rs.getInt(4));
							
							insertComponentFK.executeUpdate();							
							i_cfkOId++;		
						}
						else {
							/*devo inserire solamente i componenti*/
							/*per ogni componente invece faccio l'insert dei component*/
							insertComponentFK.setInt(1,i_cfkOId);
							insertComponentFK.setInt(2,cFK.getOID());
							insertComponentFK.setInt(3,instanceOID);
							insertComponentFK.setInt(4,identificatiInseriti.get(temp));
							temp++;
							insertComponentFK.setInt(5,rs.getInt(3));
							insertComponentFK.setInt(6,rs.getInt(4));
							
							insertComponentFK.executeUpdate();							
							i_cfkOId++;	
						}						
					}		
					/*ho finito di eseguire le query e relativi inserimenti per il primo 
					 * component, quindi se ce ne dovessero essere altri, non vanno reinserite
					 * le istanze di fk*/
					first=false;
					temp=0;
				}
 			}
			insertFK.close();
			insertComponentFK.close();
			pst.close();
			
			this.ie.storeSkolem("ForeignKeyInstance",i_fkOId-1, instanceOID);	
			this.ie.storeSkolem("ComponentOFForeignKeyInstance",i_cfkOId-1, instanceOID);
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel salvataggio dell'istanza");
		}			
	}
}
