package org.modelgen.importexport.persistence;

import org.modelgen.importexport.utilities.PersistenceException;
import org.modelgen.importexport.xml.model.ComplexElement;
import org.modelgen.importexport.xml.model.ComplexInstance;
import org.modelgen.importexport.xml.model.ComplexType;
import org.modelgen.importexport.xml.model.ComponentOfForeignKey;
import org.modelgen.importexport.xml.model.Element;
import org.modelgen.importexport.xml.model.ForeignKey;
import org.modelgen.importexport.xml.model.InstanceElement;
import org.modelgen.importexport.xml.model.SimpleElement;
import org.modelgen.importexport.xml.model.SimpleInstance;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**Classe di utilit per fare il load e lo store di uno schema XSD
 *
 * @version 0.9
 *
 */
public class Xml {
	private Connection con;
	private ImportExportPersistence ie;
	
	/*OID a livello di schema*/
	private int topElementOID,simpleOID,complexOID,nestOID,fkOID,cFkOID;
	/*OID a livello di istanze*/
	private int topInstanceOID,lexicalInstanceOID,complexInstanceOID,
	nestInstanceOID,fkInstanceOID,cFKInstanceOID;
	
	/**
	 * @param con
	 * @param ie
	 * @throws PersistenceException 
	 */
	public Xml(Connection con, ImportExportPersistence ie) throws PersistenceException {
		super();
		this.con = con;
		this.ie = ie;
		
		topElementOID=ie.getNewOID("TopElement");		
		simpleOID=ie.getNewOID("SimpleElement");
		complexOID=ie.getNewOID("ComplexElement");
		nestOID=ie.getNewOID("Nest");
		fkOID=ie.getNewOID("ForeignKey");
		cFkOID=ie.getNewOID("ComponentOfForeignKey");
		
		//oid a livello di istanze
		topInstanceOID=ie.getNewOID("TopElementeInstance");	
		lexicalInstanceOID=ie.getNewOID("LexicalInstance");
		complexInstanceOID=ie.getNewOID("ComplexInstance");
		checkGreatest();		
		nestInstanceOID=ie.getNewOID("NestInstance");
		fkInstanceOID=ie.getNewOID("ForeignKeyInstance");
		cFKInstanceOID=ie.getNewOID("ComponentOFForeignKeyInstance");
	}

	/**Store a RelationalSchema in the ModelGen db
	 * @param schema the schema to be stored
	 * @throws PersistenceException any error with the db
	 */
	public void storeSchema(XmlSchema schema) throws PersistenceException {	
		try {
			Element top=null;
			if(schema.getTopElements().size()>=1) {
				top=schema.getTopElements().get(0);
			}
			else
				throw new PersistenceException("E' inutile salvare uno schema che non ha almeno un elemento radice");
			
			/*questo  SOLO per il primo livello*/
			List<Element> primoLivello=top.getChilds();
			List<ComplexElement> complexElements=new ArrayList<ComplexElement>();
			
			/*per ogni elemento primo livello un abstrat*/
			String insert=new String("INSERT INTO SM_ABSTRACT (OID,SCHEMEOID,NAME) VALUES (?,?,?)");
			PreparedStatement pst=this.con.prepareStatement(insert);
						
			for(Element e:primoLivello) {
				pst.setInt(1,topElementOID);
				e.setOID(topElementOID);
				topElementOID++;				
				pst.setInt(2, schema.getOID());
				pst.setString(3, e.getName());
				
				pst.executeUpdate();
			}
			pst.close();
			
			/*lexical collegati ad un abstract */
			insert=new String("INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?,?)");
			PreparedStatement lexicalAbstract=con.prepareStatement(insert);
			
			/*struct collegate ad un abstract*/
			insert=new String("INSERT INTO SM_STRUCTOFATTRIBUTES (OID,SCHEMEOID,NAME,ISSET,ISOPTIONAL,ISNULLABLE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?)");
			PreparedStatement insertStruct=this.con.prepareStatement(insert);
			
			/*lexical collegati ad una struct*/
			insert=new String("INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?,?,?,?)");
			PreparedStatement lexicalStruct=con.prepareStatement(insert);
			
			for(Element e:primoLivello) {
				for(Element child:e.getChilds()) {
					if(child instanceof SimpleElement) {
						SimpleElement simple=(SimpleElement)child;
						if(!simple.isUnbounded()) {
							/*insert lexical puro*/
							//System.out.println("qui lexical puro "+simple.getName()+" OID: "+simple.getOID());
							insertLexicalBounded(lexicalAbstract, simple,e.getOID(),schema.getOID());
						}
						else {
							/*insert di una struct vuota*/
							//System.out.println("qui lexical unbounded "+simple.getName()+" OID: "+simple.getOID());
							insertLexicalUnBounded(insertStruct, lexicalStruct, simple,e.getOID(), schema.getOID());
						}	
					}
					else if (child instanceof ComplexElement) {
						/*insert di un struct collegata al abstract*/
						//System.out.println("qui complex "+e.getName()+" OID: "+e.getOID());
						insertStructAbstract(insertStruct,(ComplexElement)child,e.getOID(),schema.getOID());
						complexElements.add((ComplexElement)child);
					}
				}		
			}
			/*chiudo quelli che non mi serviranno pi, lexicalStruct mi servir ANCORA*/
			lexicalAbstract.close();
			
			/*i nest*/
			insert=new String("INSERT INTO SM_NEST (OID,SCHEMEOID,NAME,STRUCTOFATTRIBUTESTOPOID,STRUCTOFATTRIBUTESNESTEDOID) VALUES (?,?,?,?,?)");
			PreparedStatement insertNest=con.prepareStatement(insert);
			
			while(complexElements.size()>0) {
				Iterator<ComplexElement> i=complexElements.iterator();
				List<ComplexElement> temp=new ArrayList<ComplexElement>();
				
				while(i.hasNext()) {
					ComplexElement complex=i.next();					
					
					/*a questa struct devo collegare i lexical diretti, i lexical unbounded e altri complex*/
					for(Element child:complex.getChilds()) {
						if(child instanceof SimpleElement) {
							SimpleElement simple=(SimpleElement)child;
							if(!simple.isUnbounded()) {
								/*insert lexical puro*/
								//System.out.println("qui lexical puro "+simple.getName()+" OIDcomplex: "+complex.getOID());
								insertLexical(lexicalStruct,simple,complex.getOID(),schema.getOID());
							}
							else {
								/*insert di una struct vuota*/
								//System.out.println("qui lexical unbounded "+simple.getName()+" OID: "+simple.getOID());
								lexicalUnBounded(insertStruct,lexicalStruct,insertNest,simple,complex,schema.getOID());
							}					
						}
						else if (child instanceof ComplexElement) {
							/*insert di un struct collegata al abstract(e poi i figli??)*/
							//System.out.println("qui complex "+child.getName()+" OID: "+child.getOID());
							insertStruct(insertStruct,insertNest,complex,(ComplexElement)child,schema.getOID());
							temp.add((ComplexElement)child);
						}
					}
					i.remove();
				}
				complexElements.addAll(temp);
			}
			/*chiudo tutto*/
			insertStruct.close();
			lexicalStruct.close();
			insertNest.close();
			
			/*ora mi mancano solo le Fk*/
			
			/*fk*/
			insert=new String("INSERT INTO SM_FOREIGNKEY (OID,SCHEMEOID,NAME,ABSTRACTFROMOID,ABSTRACTTOOID,STRUCTFROMOID,STRUCTTOOID) VALUES (?,?,?,?,?,?,?)");
			PreparedStatement insertfk=con.prepareStatement(insert);
					
			/*relativi componenti*/
			insert=new String("INSERT INTO SM_COMPONENTOFFOREIGNKEY (OID,SCHEMEOID,NAME,FOREIGNKEYOID,LEXICALFROMOID,LEXICALTOOID) VALUES (?,?,?,?,?,?)");
			PreparedStatement insertComponent=con.prepareStatement(insert);
			
			for(ForeignKey fk:schema.getForeignKeys()) {
				boolean abstractFrom=false;
				boolean abstractTo=false;
				
				ComplexElement complexFrom=fk.getFrom();
				ComplexElement complexTo=fk.getTo();
				for(Element e:primoLivello) {
					if(e==complexFrom)
						abstractFrom=true;
					if(e==complexTo)
						abstractTo=true;
				}
				//System.out.println("oid from: "+fk.getFrom().getOID()+" oid to: "+fk.getTo().getOID());
				insertFk(insertfk,fk,schema.getOID(),abstractFrom,abstractTo);
				
				for(ComponentOfForeignKey cFk:fk.getComponents()) {
					insertComponent(insertComponent,cFk,(fkOID-1),schema.getOID());
				}
			}
			insertfk.close();
			insertComponent.close();
			
			/*faccio pure le tabelle di skolem e ho finito*/
			this.ie.storeSkolem("TopElement",(topElementOID-1),schema.getOID());
			this.ie.storeSkolem("SimpleElement",(simpleOID-1),schema.getOID());
			this.ie.storeSkolem("ComplexElement",(complexOID-1),schema.getOID());
			this.ie.storeSkolem("Nest",(nestOID-1),schema.getOID());
			this.ie.storeSkolem("ForeignKey",(fkOID-1),schema.getOID());
			this.ie.storeSkolem("ComponentOfForeignKey",(cFkOID-1),schema.getOID());
			
		} catch (SQLException e) {
			e.printStackTrace();
			new PersistenceException("Errore nello store dello schema");
		}
	}
	
	public void storeInstance(XmlSchema schema,int instanceOID) throws PersistenceException {
		if(schema.getInstances().size()>1) 
			throw new PersistenceException("Lo schema deve contenere un'istanza complessa che racchiude tutte le atre");
		
		if(schema.getInstances().size()==1) {
			try {
				InstanceElement root=schema.getInstances().get(0);
				if(!(root instanceof ComplexInstance))
					throw new PersistenceException("L'istanza radice non  una istanza complessa");
				
				List<InstanceElement> primoLivello=((ComplexInstance)root).getInstances();
				
				/*per ogni istanza di primo livello*/
				
				/*1)creo istanza i_sm_abstract*/
				String insert=new String("INSERT INTO I_SM_ABSTRACT (OID,ABSTRACTOID,INSTANCEOID) VALUES (?,?,?)");
				PreparedStatement ps=this.con.prepareStatement(insert);
				ps.setInt(3, instanceOID);
				
				for(InstanceElement instance:primoLivello) {
					if(instance instanceof ComplexInstance) {
						instance.setOID(topInstanceOID);
						
						ps.setInt(1, instance.getOID());
						ps.setInt(2, instance.getSchemaElement().getOID());
						
						ps.executeUpdate();					
						topInstanceOID++;
					}
					else
						System.out.println("Non dovrebbe succedere");					
				}	
				ps.close();
				
				/*2)collego le istanze semplici in cui l'elemento  semplice ma non unbounded direttamente a i_sm_abstract*/
				insert=new String("INSERT INTO I_SM_LEXICAL (OID,LEXICALOID,INSTANCEOID,VALUE,I_ABSTRACTOID) VALUES (?,?,?,?,?)");
				ps=this.con.prepareStatement(insert);
				ps.setInt(3, instanceOID);
				
				for(InstanceElement instance:primoLivello) {
					if(instance instanceof ComplexInstance) {
						for(InstanceElement value:((ComplexInstance)instance).getInstances()) {
							/*cmq aggiorno il valore dell'oid*/
							if(value instanceof SimpleInstance) {
								value.setOID(lexicalInstanceOID);
								lexicalInstanceOID++;
							}
							/*se l'elemento NON era unbounded lo collego diretto*/
							if(value instanceof SimpleInstance && !value.getSchemaElement().isUnbounded()) {
								ps.setInt(1, value.getOID());
								ps.setInt(2, value.getSchemaElement().getOID());
								/*3 fisso ed  istanceOID*/
								if(((SimpleInstance)value).getValue()!=null)
									ps.setString(4, ((SimpleInstance)value).getValue());
								else
									ps.setNull(4, java.sql.Types.VARCHAR);
								ps.setInt(5, instance.getOID());
								
								ps.executeUpdate();
							}							
						}
					}					
				}
				ps.close();
				
				/*3)collego le istanze semplici ma elemento  unbounded, prima cerco l'struct a cui  collegato il lexical 
				 * e creo un'istanza di questo struct e collego l'istanza lexical a quest'ultimo*/
				insert=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES (OID,STRUCTOFATTRIBUTESOID," +
						"INSTANCEOID,I_ABSTRACTOID) VALUES (?,?,?,?)");
				ps=this.con.prepareStatement(insert);
				ps.setInt(3, instanceOID);
				
				insert=new String("INSERT INTO I_SM_LEXICAL (OID,LEXICALOID,INSTANCEOID,VALUE,I_STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?)");
				PreparedStatement pst=this.con.prepareStatement(insert);
				pst.setInt(3, instanceOID);
				
				for(InstanceElement instance:primoLivello) {
					if(instance instanceof ComplexInstance) {
						for(InstanceElement value:((ComplexInstance)instance).getInstances()) {
							/*se l'elemento era unbounded*/
							if(value instanceof SimpleInstance && value.getSchemaElement().isUnbounded()) {
								int structOID=getLexicalUndoundedStruct(value.getSchemaElement().getOID());
								
								ps.setInt(1, complexInstanceOID);
								ps.setInt(2, structOID);
								/*3 sempre instance oid*/
								ps.setInt(4, instance.getOID());
								
								ps.executeUpdate();
								
								pst.setInt(1, value.getOID());
								pst.setInt(2, value.getSchemaElement().getOID());
								/*3 sempre instance oid*/
								if(((SimpleInstance)value).getValue()!=null)
									pst.setString(4, ((SimpleInstance)value).getValue());
								else
									pst.setNull(4, java.sql.Types.VARCHAR);
								pst.setInt(5, complexInstanceOID);
								
								pst.executeUpdate();
								complexInstanceOID++;
							}
						}
					}
				}
				ps.close();
				pst.close();
				
				/*4)collego le istanze complesse a i_sm_abstract e le inserisco in una lista per fare la ricorsione*/
				List<ComplexInstance> complexes=new ArrayList<ComplexInstance>();
				
				insert=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES (OID,STRUCTOFATTRIBUTESOID,INSTANCEOID,I_ABSTRACTOID) VALUES (?,?,?,?)");
				ps=this.con.prepareStatement(insert);
				ps.setInt(3, instanceOID);
				
				for(InstanceElement instance:primoLivello) {
					if(instance instanceof ComplexInstance) {
						for(InstanceElement complex:((ComplexInstance)instance).getInstances()) {
							if(complex instanceof ComplexInstance) {
								complex.setOID(complexInstanceOID);
								complexInstanceOID++;
								
								ps.setInt(1, complex.getOID());
								ps.setInt(2, complex.getSchemaElement().getOID());
								/*3 sempre instance oid*/
								ps.setInt(4, instance.getOID());
								
								ps.executeUpdate();
								complexes.add((ComplexInstance)complex);
							}
						}
					}
				}
				ps.close();
				
				/*5)parte la ricorsione*/
				while(complexes.size()>0) {
					List<ComplexInstance> temp=new ArrayList<ComplexInstance>();
					
					Iterator<ComplexInstance> i=complexes.iterator();
					
					while(i.hasNext()) {
						ComplexInstance complex=i.next();
						
						/*6)istanze semplici collegate direttamente a i_sm_struct*/
						insert=new String("INSERT INTO I_SM_LEXICAL (OID,LEXICALOID,INSTANCEOID,VALUE,I_STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?)");
						ps=this.con.prepareStatement(insert);
						ps.setInt(3, instanceOID);
						
						for(InstanceElement instance:complex.getInstances()) {
							if(instance instanceof SimpleInstance && !instance.getSchemaElement().isUnbounded()) {
								instance.setOID(lexicalInstanceOID);
								lexicalInstanceOID++;
								
								ps.setInt(1, instance.getOID());
								ps.setInt(2, instance.getSchemaElement().getOID());
								/*3 sempre instance oid*/
								if(((SimpleInstance)instance).getValue()!=null)
									ps.setString(4, ((SimpleInstance)instance).getValue());
								else
									ps.setNull(4, java.sql.Types.VARCHAR);
								ps.setInt(5, complex.getOID());
								
								ps.executeUpdate();
							}
						}
						ps.close();
						
						
						/*7)istanze semplici UNBOUNDED collegate tramite i_sm_nest*/
						insert=new String("INSERT INTO I_SM_LEXICAL (OID,LEXICALOID,INSTANCEOID,VALUE,I_STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?)");
						ps=this.con.prepareStatement(insert);
						ps.setInt(3, instanceOID);
						
						insert=new String("INSERT INTO I_SM_STRUCTOFATTRIBUTES (OID,STRUCTOFATTRIBUTESOID,INSTANCEOID) VALUES (?,?,?)");
						pst=this.con.prepareStatement(insert);
						pst.setInt(3, instanceOID);
						
						insert=new String("INSERT INTO I_SM_NEST (OID,NESTOID,INSTANCEOID,I_STRUCTOFATTRIBUTESTOPOID," +
								"I_STRUCTOFATTRIBUTESNESTEDOID) VALUES (?,?,?,?,?)");
						PreparedStatement iNest=this.con.prepareStatement(insert);
						iNest.setInt(3, instanceOID);
						
						for(InstanceElement instance:complex.getInstances()) {
							if(instance instanceof SimpleInstance && instance.getSchemaElement().isUnbounded()) {
								/*creo la struct*/
								pst.setInt(1, complexInstanceOID);
								int structOID=getLexicalUndoundedStruct(instance.getSchemaElement().getOID());
								pst.setInt(2, structOID);
								/*3 sempre instance oid*/
								pst.executeUpdate();
								
								/*creo l'istanza collegata alla struct*/
								instance.setOID(lexicalInstanceOID);
								lexicalInstanceOID++;
								ps.setInt(1, instance.getOID());
								ps.setInt(2, instance.getSchemaElement().getOID());
								/*3 sempre instance oid*/
								if(((SimpleInstance)instance).getValue()!=null)
									ps.setString(4, ((SimpleInstance)instance).getValue());
								else
									ps.setNull(4, java.sql.Types.VARCHAR);
								ps.setInt(5, complexInstanceOID);
								ps.executeUpdate();
								
								/*creo istanza nest per collegare l'istanza complessa attuale all'istanza di struct fittizzia*/
								iNest.setInt(1, nestInstanceOID);
								nestInstanceOID++;
								int nestOID=getNestLexicalUnbounded(structOID);
								iNest.setInt(2, nestOID);
								/*3 sempre instance oid*/
								iNest.setInt(4, complex.getOID());
								iNest.setInt(5, complexInstanceOID);
								iNest.executeUpdate();
								
								complexInstanceOID++;
							}
						}
						ps.close();
						
						/*8)istanze complesse da inserire nella lista temporanea*/				
						for(InstanceElement instance:complex.getInstances()) {
							if(instance instanceof ComplexInstance) {
								instance.setOID(complexInstanceOID);
								complexInstanceOID++;
								
								temp.add((ComplexInstance)instance);
								
								pst.setInt(1, instance.getOID());
								pst.setInt(2, instance.getSchemaElement().getOID());
								/*3 sempre instance oid*/
								pst.executeUpdate();
								
								iNest.setInt(1, nestInstanceOID);
								nestInstanceOID++;
								int nestOID=getNestComplexes(complex.getSchemaElement().getOID(), instance.getSchemaElement().getOID());
								iNest.setInt(2, nestOID);
								/*3 sempre instance oid*/
								iNest.setInt(4, complex.getOID());
								iNest.setInt(5, instance.getOID());
								iNest.executeUpdate();
							}
						}
						pst.close();
						iNest.close();
						
						i.remove();
					}
					complexes.addAll(temp);
				}	
				
				/*6)fk e relative componenti*/
				String query=new String("SELECT I1.I_ABSTRACTOID,I1.I_STRUCTOFATTRIBUTESOID,I2.I_ABSTRACTOID," +
						"I2.I_STRUCTOFATTRIBUTESOID,I1.OID,I2.OID FROM " +
						"(SELECT OID,VALUE,I_ABSTRACTOID,I_STRUCTOFATTRIBUTESOID FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I1," +
						"(SELECT OID,VALUE,I_ABSTRACTOID,I_STRUCTOFATTRIBUTESOID FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND LEXICALOID=?) AS I2 " +
						"WHERE I1.VALUE=I2.VALUE");
				pst=this.con.prepareStatement(query);
				pst.setInt(1, instanceOID);
				pst.setInt(3, instanceOID);
				
				PreparedStatement insertFK=this.con.prepareStatement("INSERT INTO I_SM_FOREIGNKEY VALUES(?,?,?,NULL,NULL,?,?,?,?)");
				PreparedStatement insertComponentFK=this.con.prepareStatement("INSERT INTO I_SM_COMPONENTOFFOREIGNKEY VALUES(?,?,?,?,?,?)");
				
				for(ForeignKey fk:schema.getForeignKeys()) {
					boolean first=true;
					Vector<Integer> identificatiInseriti=new Vector<Integer>();
					int temp=0;
					
					for(ComponentOfForeignKey cFK:fk.getComponents()) {
						pst.setInt(2,cFK.getFrom().getOID());
						pst.setInt(4,cFK.getTo().getOID());
						
						ResultSet rs=pst.executeQuery();
						
						while(rs.next()) {
							/*inserisco una istanza di FK solo al primo component, quindi*/
							if(first) {
								insertFK.setInt(1,fkInstanceOID);
								insertFK.setInt(2,fk.getOID());
								insertFK.setInt(3,instanceOID);
								int from=rs.getInt(1);
								if(rs.wasNull()) {
									from=rs.getInt(2);
									insertFK.setNull(4, java.sql.Types.INTEGER);
									insertFK.setInt(6,from);
								}
								else {
									insertFK.setInt(4,from);
									insertFK.setNull(6, java.sql.Types.INTEGER);									
								}											
								int to=rs.getInt(3);
								if(rs.wasNull()) {
									to=rs.getInt(4);
									insertFK.setNull(5, java.sql.Types.INTEGER);
									insertFK.setInt(7,to);
								}
								else {
									insertFK.setInt(5,to);		
									insertFK.setNull(7, java.sql.Types.INTEGER);														
								}	
								
								insertFK.executeUpdate();
								identificatiInseriti.add(fkInstanceOID);
								fkInstanceOID++;
								
								/*per ogni componente invece faccio l'insert dei component*/
								insertComponentFK.setInt(1,cFKInstanceOID);
								insertComponentFK.setInt(2,cFK.getOID());
								insertComponentFK.setInt(3,instanceOID);
								insertComponentFK.setInt(4,fkInstanceOID-1);
								insertComponentFK.setInt(5,rs.getInt(5));
								insertComponentFK.setInt(6,rs.getInt(6));
								
								insertComponentFK.executeUpdate();							
								cFKInstanceOID++;	
							}
							else {
								/*devo inserire solamente i componenti*/
								/*per ogni componente invece faccio l'insert dei component*/
								insertComponentFK.setInt(1,cFKInstanceOID);
								insertComponentFK.setInt(2,cFK.getOID());
								insertComponentFK.setInt(3,instanceOID);
								insertComponentFK.setInt(4,identificatiInseriti.get(temp));
								temp++;
								insertComponentFK.setInt(5,rs.getInt(5));
								insertComponentFK.setInt(6,rs.getInt(6));
								
								insertComponentFK.executeUpdate();							
								cFKInstanceOID++;	
							}						
						}		
						/*ho finito di eseguire le query e relativi inserimenti per il primo 
						 * component, quindi se ce ne dovessero essere altri, non vanno reinserite
						 * le istanze di fk*/
						first=false;
						temp=0;
					}
				}
				insertFK.close();
				insertComponentFK.close();
				pst.close();
				
				
				/*7)tabelle di skolem*/		
				this.ie.storeSkolem("TopElementeInstance",(topInstanceOID-1),instanceOID);
				this.ie.storeSkolem("LexicalInstance",(lexicalInstanceOID-1),instanceOID);
				this.ie.storeSkolem("ComplexInstance",(complexInstanceOID-1),instanceOID);
				this.ie.storeSkolem("NestInstance",(nestInstanceOID-1),instanceOID);
				this.ie.storeSkolem("ForeignKeyInstance",(fkInstanceOID-1),instanceOID);
				this.ie.storeSkolem("ComponentOFForeignKeyInstance",(cFKInstanceOID-1),instanceOID);
				
				
			} catch (SQLException e) {
				e.printStackTrace();
			}			
		}		
	}
	
	
	/**Load a XmlSchema from ModelGen.
	 * 
	 * @param schemaOID
	 * @throws PersistenceException
	 */
	public XmlSchema loadXMLSchema(int schemaOID) throws PersistenceException {
		XmlSchema schema=null;
		
		try {
			String query=new String("SELECT * FROM SCHEMESM WHERE OID = " + schemaOID);
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(query);
			if(!result.next())
				throw new PersistenceException("Non esiste lo schema indicato");
			String schemaName = result.getString("name");
			result.close();
			
			schema=new XmlSchema(schemaOID,schemaName);
			
			/*per prima cosa carico tutti i top element che DEVONO essere in modelgen abstract
			 * e corrisponderanno a tutti elementi complessi e unbounded*/
			Vector<ComplexType> complexTypes=new Vector<ComplexType>();
			Vector<SimpleElement> simpleElements=new Vector<SimpleElement>();
			Vector<ComplexElement> complexElements=new Vector<ComplexElement>();
			Vector<ComplexElement> tempComplex=new Vector<ComplexElement>();
			Vector<ComplexType> temp=new Vector<ComplexType>();
			
			query=new String("SELECT OID,NAME FROM SM_ABSTRACT WHERE SCHEMEOID=?");
			PreparedStatement ps=con.prepareStatement(query);
			ps.setInt(1, schema.getOID());
			
			ResultSet rs=ps.executeQuery();
			while(rs.next()) {
				ComplexType complexType=new ComplexType(rs.getInt(1),"");
				ComplexElement complex=new ComplexElement(rs.getInt(1),rs.getString(2),false,false,true,complexType);
				schema.addElement(complex);
				complexElements.add(complex);
			}
			rs.close();
			ps.close();
			
			for(ComplexElement complex:complexElements) {
				/*1) carico i SimpleElement collegati direttamente all'abstract*/
				query=new String("SELECT OID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE FROM SM_LEXICAL " +
				"WHERE ABSTRACTOID=? AND SCHEMEOID=?");
				ps=con.prepareStatement(query);
				ps.setInt(1, complex.getOID());
				ps.setInt(2, schema.getOID());

				rs=ps.executeQuery();
				while(rs.next()) {
					/*unbounded  false perch in questa prima versione, il fatto che un elemento sia unbounded viene specificato
					 *passando per una Struct con isSet=true*/
					boolean isNullable=rs.getBoolean(3);
					if(rs.wasNull())
						isNullable=true;
					SimpleElement simple=new SimpleElement(rs.getInt(1),rs.getString(2),isNullable,rs.getBoolean(4),rs.getBoolean(5),false,rs.getString(6));
					complex.getComplexType().addElement(simple);
					simpleElements.add(simple);
				}
				rs.close();
				ps.close();
				
				/*2)carico i lexical collegati attraverso una struct, ovvero unbounded*/
				query=new String("SELECT OID FROM SM_STRUCTOFATTRIBUTES WHERE SCHEMEOID=? AND ISSET=TRUE AND ABSTRACTOID=?");
				ps=con.prepareStatement(query);
				ps.setInt(1, schema.getOID());
				ps.setInt(2, complex.getOID());
				
				rs=ps.executeQuery();
				String queryLexical=new String("SELECT COUNT(*) FROM SM_LEXICAL WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID=?");
				PreparedStatement pst=this.con.prepareStatement(queryLexical);
				pst.setInt(1, schema.getOID());				
				
				while(rs.next()) {
					pst.setInt(2, rs.getInt(1));
					
					ResultSet rsLex=pst.executeQuery();
					rsLex.next();
					int rowNumber=rsLex.getInt(1);
					rsLex.close();
					
					if(rowNumber==1) {
						queryLexical=new String("SELECT OID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE " +
												"FROM SM_LEXICAL WHERE SCHEMEOID="+schema.getOID()+" AND STRUCTOFATTRIBUTESOID="+rs.getInt(1));
						Statement stm=this.con.createStatement();
						
						rsLex=stm.executeQuery(queryLexical);
						while(rsLex.next()) {
							boolean isNullable=rsLex.getBoolean(3);
							if(rs.wasNull())
								isNullable=true;
							SimpleElement simple=new SimpleElement(rsLex.getInt(1),rsLex.getString(2),isNullable,rsLex.getBoolean(4),rsLex.getBoolean(5),true,rsLex.getString(6));
							complex.getComplexType().addElement(simple);
							simpleElements.add(simple);
						}
						rsLex.close();		
						stm.close();
					}					
				}
				pst.close();			
				rs.close();
				ps.close();

				
				/*3)carico gli struct direttamente collegati al abstract*/
				query=new String("SELECT OID,NAME,ISSET,ISOPTIONAL,ISNULLABLE FROM SM_STRUCTOFATTRIBUTES " +
						"WHERE SCHEMEOID=? AND ABSTRACTOID=?"); 	// AND NAME<>''
				ps=con.prepareStatement(query);
				ps.setInt(1, schema.getOID());
				ps.setInt(2, complex.getOID());				

				rs=ps.executeQuery();
				pst=this.con.prepareStatement("SELECT COUNT(*) FROM SM_LEXICAL WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID=?");
				pst.setInt(1, schema.getOID());
				
				while(rs.next()) {					
					pst.setInt(2, rs.getInt(1));
					
					ResultSet count=pst.executeQuery();
					count.next();
					int rowCount=count.getInt(1);
					count.close();					
					
					if(!(rs.getBoolean(3) && rowCount==1)) {
						/*questi sono complexelement*/
						boolean isNullable=rs.getBoolean(4);
						if(rs.wasNull())
							isNullable=true;
						
						ComplexType complexType=new ComplexType(rs.getInt(1),"");
						ComplexElement element=new ComplexElement(rs.getInt(1),rs.getString(2),isNullable,rs.getBoolean(5),rs.getBoolean(3),complexType);
						complex.getComplexType().addElement(element);
						complexTypes.add(complexType);
						tempComplex.add(element);
					}					
				}
				rs.close();
				ps.close();
				pst.close();
			}
			
			complexElements.addAll(tempComplex);
			tempComplex=null;
			
			
			/*in questo momento ho finito il primo livello ovvero gli elementi collegati agli abstract; devo per ancora ricorsivamente scendere di livello
			 * fino alla foglie dell'albero*/
			while(complexTypes.size()>0) {
				Iterator<ComplexType> i=complexTypes.iterator();
				temp=new Vector<ComplexType>();

				while(i.hasNext()) {
					ComplexType complex=i.next();
					int complexOID=complex.getOID();

					/*1) carico i SimpleElement collegati direttamente a questa struct*/
					query=new String("SELECT OID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE FROM SM_LEXICAL " +
					"WHERE STRUCTOFATTRIBUTESOID=? AND SCHEMEOID=?");
					ps=con.prepareStatement(query);
					ps.setInt(1, complexOID);
					ps.setInt(2, schema.getOID());

					rs=ps.executeQuery();
					while(rs.next()) {
						/*unbounded  false perch in questa prima versione, il fatto che un elemento sia unbounded viene specificato
						 *passando per una Struct con isSet=true*/
						boolean isNullable=rs.getBoolean(3);
						if(rs.wasNull())
							isNullable=true;
						SimpleElement simple=new SimpleElement(rs.getInt(1),rs.getString(2),isNullable,rs.getBoolean(4),rs.getBoolean(5),false,rs.getString(6));
						complex.addElement(simple);
						simpleElements.add(simple);
					}
					rs.close();
					ps.close();

					
					/*elementi semplici "unbounded"*/
					query=new String("SELECT I1.OID FROM SM_STRUCTOFATTRIBUTES AS I1, (SELECT STRUCTOFATTRIBUTESNESTEDOID " +
							"FROM SM_NEST WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESTOPOID=?) AS I2" +
							" WHERE I1.OID=I2.STRUCTOFATTRIBUTESNESTEDOID AND I1.ISSET=TRUE"); 
					ps=this.con.prepareStatement(query);
					ps.setInt(1, schema.getOID());
					ps.setInt(2, complexOID);

					rs=ps.executeQuery();
					query=new String("SELECT COUNT(*) FROM SM_LEXICAL WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID=? " +
							"AND STRUCTOFATTRIBUTESOID NOT IN (SELECT STRUCTOFATTRIBUTESTOPOID FROM SM_NEST)");
					PreparedStatement pst=this.con.prepareStatement(query);
					pst.setInt(1, schema.getOID());				
					
					while(rs.next()) {
						pst.setInt(2, rs.getInt(1));
						
						ResultSet count=pst.executeQuery();
						count.next();
						int rowNumber=count.getInt(1);
						count.close();
						
						if(rowNumber==1) {
							query=new String("SELECT OID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE " +
									"FROM SM_LEXICAL WHERE SCHEMEOID="+schema.getOID()+" AND STRUCTOFATTRIBUTESOID="+rs.getInt(1));
							Statement stm=this.con.createStatement();

							ResultSet lex=stm.executeQuery(query);
							while(lex.next()) {
								boolean isNullable=lex.getBoolean(3);
								if(rs.wasNull())
									isNullable=true;
								SimpleElement simple=new SimpleElement(lex.getInt(1),lex.getString(2),isNullable,lex.getBoolean(4),lex.getBoolean(5),true,lex.getString(6));
								complex.addElement(simple);
								simpleElements.add(simple);
							}
							lex.close();		
							stm.close();
						}						
					}
					pst.close();
					rs.close();
					ps.close();					
					
					
					/*3)adesso devo collegare quegli elementi che sono a loro volta complessi*/
					query=new String("SELECT I1.OID,I1.NAME,I1.ISSET,I1.ISOPTIONAL,I1.ISNULLABLE FROM SM_STRUCTOFATTRIBUTES AS I1, " +
							"(SELECT STRUCTOFATTRIBUTESNESTEDOID FROM SM_NEST WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESTOPOID=?) AS I2 " +
							"WHERE I1.OID=I2.STRUCTOFATTRIBUTESNESTEDOID");
					ps=this.con.prepareStatement(query);
					ps.setInt(1, schema.getOID());
					ps.setInt(2, complexOID);
					
					rs=ps.executeQuery();
					query=new String("SELECT COUNT(*) FROM SM_LEXICAL WHERE SCHEMEOID=? AND STRUCTOFATTRIBUTESOID=? " +
							"AND STRUCTOFATTRIBUTESOID NOT IN (SELECT STRUCTOFATTRIBUTESTOPOID FROM SM_NEST)");
					pst=this.con.prepareStatement(query);
					pst.setInt(1, schema.getOID());				
					
					while(rs.next()) {
						pst.setInt(2, rs.getInt(1));
					
						ResultSet count=pst.executeQuery();
						count.next();
						int rowCount=count.getInt(1);
						count.close();					
						
						if(!(rs.getBoolean(3) && rowCount==1)) {
							boolean isNullable=rs.getBoolean(5);
							if(rs.wasNull())
								isNullable=true;
							
							ComplexType complexType=new ComplexType(rs.getInt(1),"");
							ComplexElement element=new ComplexElement(rs.getInt(1),rs.getString(2),rs.getBoolean(4),isNullable,rs.getBoolean(3),complexType);
							complex.addElement(element);
							complexElements.add(element);
							temp.add(complexType);
						}
					}
					rs.close();
					ps.close();
					pst.close();
					
					
					
					/*appena ho finito un ciclo mi preparo per il prossimo*/
					i.remove();	
				}			
				complexTypes.addAll(temp);
			}
			
			/*in questo momento ho finito TUTTI gli elementi a qualsiasi livello di profondit, mi mancano solamente le FK
			 * e i relativi componenti*/
			query=new String("SELECT OID,NAME,ABSTRACTFROMOID,ABSTRACTTOOID," +
					"STRUCTFROMOID,STRUCTTOOID FROM SM_FOREIGNKEY WHERE SCHEMEOID=?");
			ps=con.prepareStatement(query);
			ps.setInt(1, schema.getOID());
			
			rs=ps.executeQuery();
			while(rs.next()) {
				ComplexElement from=null;
				ComplexElement to=null;
				
				/*trovo il from*/
				if(rs.getString(3)!=null) {
					//System.out.println("devo trovare un abstractFrom con OID: "+rs.getInt(3));
					for(Element e:schema.getTopElements()) {
						if(e.getOID()==rs.getInt(3) && e instanceof ComplexElement)
							from=(ComplexElement)e;
					}
				}
				else {
					//System.out.println("devo trovare un complexFrom con OID: "+rs.getInt(5));
					for(ComplexElement complex:complexElements) {
						if(complex.getOID()==rs.getInt(5) && !checkComplex(complex, schema.getTopElements()))
							from=complex;
					}
				}				
				/*trovo il to*/
				if(rs.getString(4)!=null) {
					//System.out.println("devo trovare un abstractTo con OID: "+rs.getInt(4));
					for(Element e:schema.getTopElements()) {
						if(e.getOID()==rs.getInt(4) && e instanceof ComplexElement)
							to=(ComplexElement)e;
					}
				}
				else {
					//System.out.println("devo trovare un complexTo con OID: "+rs.getInt(6));
					for(ComplexElement complex:complexElements) {
						if(complex.getOID()==rs.getInt(6) && !checkComplex(complex, schema.getTopElements()))
							to=complex;
					}
				}
				if(from!=null && to!=null) {
					ForeignKey fk=new ForeignKey(rs.getInt(1),rs.getString(2),from,to);
					schema.addForeignKey(fk);
				}		
				else {
					throw new PersistenceException("Non sono stati trovati i complexElement cui si riferisce la Fk con OID "+rs.getInt(1));
				}
			}
			rs.close();
			ps.close();
			
			/*ora i componenti*/
			query=new String("SELECT OID,NAME,FOREIGNKEYOID,LEXICALFROMOID,LEXICALTOOID FROM SM_COMPONENTOFFOREIGNKEY WHERE SCHEMEOID=?");
			ps=con.prepareStatement(query);
			ps.setInt(1, schema.getOID());
			
			rs=ps.executeQuery();
			while(rs.next()) {
				ForeignKey fk=schema.getForeignKeyByOID(rs.getInt(3));
				if(fk!=null) {
					int fromOID=rs.getInt(4);
					SimpleElement from=null;
					int toOID=rs.getInt(5);
					SimpleElement to=null;
					
					for(SimpleElement s:simpleElements) {
						if(s.getOID()==fromOID)
							from=s;
						if(s.getOID()==toOID)
							to=s;
					}
					
					if(from!=null && to!=null) {
						ComponentOfForeignKey cFk=new ComponentOfForeignKey(rs.getInt(1),rs.getString(2),from,to);
						fk.addComponent(cFk);
					}
					else
						throw new PersistenceException("Non sono stati trovati i simpleElement relativi al component "+rs.getInt(1));
				}
				else {
					throw new PersistenceException("Il componentofFk con OID "+rs.getInt(1)+"  collegato ad una FK che non  stata caricata");
				}
			}
			rs.close();
			ps.close();

			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel caricamento dello schema XML ");
		}	
		
		return schema;	
	}
	
	
	public XmlSchema loadXmlInstance(XmlSchema schema,int instanceOID) throws PersistenceException {
		
		try {	
			/*mi serve per i livelli succissivi al primo*/
			Vector<ComplexInstance> complexInstances=new Vector<ComplexInstance>();			
			
			//1) le istanze per ogni elemento caricato a livello di schema
			String query=new String("SELECT OID FROM I_SM_ABSTRACT WHERE ABSTRACTOID=? AND INSTANCEOID=?");
			PreparedStatement ps=this.con.prepareStatement(query);
			ps.setInt(2, instanceOID);
			
			for(Element e:schema.getTopElements()) {
				ps.setInt(1, e.getOID());
				
				ResultSet rs=ps.executeQuery();
				while(rs.next()) {
					ComplexInstance instance=new ComplexInstance(rs.getInt(1),e);
					schema.addInstance(instance);
				}
				rs.close();
			}
			ps.close();
			
			
			//2) i lexical collegati direttamente alle istanze di sopra
			query=new String("SELECT OID,LEXICALOID,VALUE FROM I_SM_LEXICAL WHERE I_ABSTRACTOID=? AND INSTANCEOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(2, instanceOID);
			
			for(InstanceElement e:schema.getInstances()) {
				ps.setInt(1, e.getOID());
				
				ResultSet rs=ps.executeQuery();
				while(rs.next()) {
					SimpleElement schemaElement=findSimpleChild(e.getSchemaElement(), rs.getInt(2));
					if(schemaElement!=null) {
						SimpleInstance simple=new SimpleInstance(rs.getInt(1),schemaElement,rs.getString(3));
						//condizione sempre vera ma meglio farla
						if(e instanceof ComplexInstance)
							((ComplexInstance)e).addInstance(simple);
					}
				}
				rs.close();
			}
			ps.close();
			
			//2) devo caricare le istanze di lexical unbounded e le istanze di struct
			/*carico le i_smStruct collegate direttamente all i_sm_abstract dove per il corrispettivo isset=true a verifico il numero
			 * di i_sm_lexical collegati*/
			query=new String("SELECT I1.OID,I1.STRUCTOFATTRIBUTESOID,I2.ISSET FROM I_SM_STRUCTOFATTRIBUTES AS I1,SM_STRUCTOFATTRIBUTES AS I2 " +
					"WHERE I1.INSTANCEOID=? AND I2.SCHEMEOID=? AND I1.STRUCTOFATTRIBUTESOID=I2.OID AND I_ABSTRACTOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, instanceOID);
			ps.setInt(2, schema.getOID());
			
			query=new String("SELECT COUNT(*) FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID=?");
			PreparedStatement pst=this.con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			
			query=new String("SELECT OID,LEXICALOID,VALUE FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID=?");
			PreparedStatement lex=this.con.prepareStatement(query);
			lex.setInt(1, instanceOID);
			
			for(InstanceElement e:schema.getInstances()) {
				ps.setInt(3, e.getOID());
			
				ResultSet rs=ps.executeQuery();
				while(rs.next()) {
					/*con questa query ho sia istanze di lexical che istanze vere e proprie di lexical unbounded quindi devo 
					 * verificare il numero*/
					pst.setInt(2, rs.getInt(1));
					
					ResultSet lexical=pst.executeQuery();
					lexical.next();
					int rowCount=lexical.getInt(1);
					lexical.close();
					
					/*se iset=true and numero lexical collegati=1  un'istanza semplice*/
					if(rs.getBoolean(3) && rowCount==1) {
						lex.setInt(2, rs.getInt(1));
						
						ResultSet rsLex=lex.executeQuery();
						if(rsLex.next()){
							SimpleElement schemaElement=findSimpleChild(e.getSchemaElement(), rsLex.getInt(2));
							if(schemaElement!=null) {
								SimpleInstance simple=new SimpleInstance(rsLex.getInt(1),schemaElement,rsLex.getString(3));
								//condizione sempre vera ma meglio farla
								if(e instanceof ComplexInstance)
									((ComplexInstance)e).addInstance(simple);
							}
						}
						rsLex.close();						
					}
					/*altrimenti complessa*/
					else {
						ComplexElement complex=findComplexChild(e.getSchemaElement(), rs.getInt(2));
						if(complex!=null) {
							ComplexInstance complexInstance=new ComplexInstance(rs.getInt(1),complex);
							//condizione sempre vera ma meglio farla
							if(e instanceof ComplexInstance) {
								((ComplexInstance)e).addInstance(complexInstance);
								complexInstances.add(complexInstance);
							}
						}
					}
				}
				rs.close();
			}
			ps.close();
			pst.close();
			lex.close();
			
			//System.out.println("il vettore dei complessi : "+complexInstances.size());	
			
			/*adesso devo fare la parte finch non hai finito*/
			Vector<ComplexInstance> temp;
			
			/*query lexical "puri"*/
			query=new String("SELECT OID,LEXICALOID,VALUE FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID=?");
			ps=this.con.prepareStatement(query);
			ps.setInt(1, instanceOID);
			
			/*query per struct annidati (sia lexical unbounded che struct veri)*/
			query=new String("SELECT I1.OID,I1.STRUCTOFATTRIBUTESOID,I2.ISSET FROM I_SM_STRUCTOFATTRIBUTES AS I1," +
					"SM_STRUCTOFATTRIBUTES AS I2,I_SM_NEST AS I3 WHERE I1.INSTANCEOID=? AND I2.SCHEMEOID=? AND " +
					"I1.STRUCTOFATTRIBUTESOID=I2.OID AND I3.I_STRUCTOFATTRIBUTESTOPOID=? AND I1.OID=I3.I_STRUCTOFATTRIBUTESNESTEDOID");
			pst=this.con.prepareStatement(query);
			pst.setInt(1, instanceOID);
			pst.setInt(2, schema.getOID());
			
			/*query per i count*/
			query=new String("SELECT COUNT(*) FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID=? " +
					"AND I_STRUCTOFATTRIBUTESOID NOT IN (SELECT I_STRUCTOFATTRIBUTESTOPOID FROM I_SM_NEST)");
			PreparedStatement count=this.con.prepareStatement(query);
			count.setInt(1, instanceOID);
			
			/*query per i lexical*/
			query=new String("SELECT OID,LEXICALOID,VALUE FROM I_SM_LEXICAL WHERE INSTANCEOID=? AND I_STRUCTOFATTRIBUTESOID=?");
			lex=this.con.prepareStatement(query);
			lex.setInt(1, instanceOID);
			
			while(complexInstances.size()>0) {
				temp=new Vector<ComplexInstance>();
				Iterator<ComplexInstance> i=complexInstances.iterator();
				
				while(i.hasNext()) {
					ComplexInstance complex=i.next();
					
					/*per ognuno caricare i lexical collegati direttamente, +lexical unbound+complex nestati*/
					//System.out.println("dovrei trovare i lex+unbounded+complex nestati " +
						//	"di: "+complex.getOID()+" "+complex.getSchemaElement().getName()+"\n");
					
					/*1) lexical collegati direttamente con l'istanza*/
					ps.setInt(2, complex.getOID());
					
					ResultSet rs=ps.executeQuery();
					while(rs.next()) {
						SimpleElement schemaElement=findSimpleChild(complex.getSchemaElement(), rs.getInt(2));
						if(schemaElement!=null) {
							SimpleInstance simple=new SimpleInstance(rs.getInt(1),schemaElement,rs.getString(3));
							complex.addInstance(simple);
						}
					}
					rs.close();
					
					/*2) istanze di lexical unbounded e struct*/
					pst.setInt(3, complex.getOID());
					
					rs=pst.executeQuery();
					while(rs.next()) {
						count.setInt(2, rs.getInt(1));
						
						ResultSet rsCount=count.executeQuery();
						rsCount.next();
						int rowCount=rsCount.getInt(1);
						rsCount.close();
						
						if(rs.getBoolean(3) && rowCount==1) {
							lex.setInt(2, rs.getInt(1));
							
							ResultSet rsLex=lex.executeQuery();
							if(rsLex.next()) {
								SimpleElement schemaElement=findSimpleChild(complex.getSchemaElement(), rsLex.getInt(2));
								if(schemaElement!=null) {
									SimpleInstance simple=new SimpleInstance(rsLex.getInt(1),schemaElement,rsLex.getString(3));
									complex.addInstance(simple);
								}
							}
							rsLex.close();
						}
						else {
							ComplexElement child=findComplexChild(complex.getSchemaElement(), rs.getInt(2));
							if(child!=null) {
								ComplexInstance complexInstance=new ComplexInstance(rs.getInt(1),child);
								complex.addInstance(complexInstance);
								temp.add(complexInstance);
							}
						}
					}
					rs.close();
					
					i.remove();
				}
				complexInstances.addAll(temp);
			}
			ps.close();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel caricamento delle istanze dello schema XSD\n" +
					"Controllare il dizionario di ModelGen");
		}			
		return schema;
	}
	
	
	private SimpleElement findSimpleChild(Element e,int childOID) {
		Vector<Element> childs=e.getChilds();
		
		for(Element child:childs) {
			if (child instanceof SimpleElement && child.getOID()==childOID) {
				return (SimpleElement)child;
			}
		}
		return null;
	}
	
	private ComplexElement findComplexChild(Element e,int childOID) {
		Vector<Element> childs=e.getChilds();
		
		for(Element child:childs) {
			if (child instanceof ComplexElement && child.getOID()==childOID) {
				return (ComplexElement)child;
			}
		}
		return null;
	}
	
	
	/**
	 * 
	 * METODI DI UTLITA' BANALI
	 * 
	 * */
	
	/**Returns true if the list contains the ComplexElement
	 * 
	 * @param e
	 * @param complexes
	 */
	private boolean checkComplex(ComplexElement e,Vector<Element> complexes) {
		for(Element complex:complexes) {
			if(complex==e)
				return true;
		}
		return false;
	}

	private void insertComponent(PreparedStatement insertComponent, ComponentOfForeignKey cFk, int fkOID, int schemaOID) throws PersistenceException {
		//INSERT INTO SM_COMPONENTOFFOREIGNKEY (OID,SCHEMEOID,NAME,FOREIGNKEYOID,LEXICALFROMOID,LEXICALTOOID)
		try {
			insertComponent.setInt(1, cFkOID);	
			cFk.setOID(cFkOID);
			cFkOID++;
			insertComponent.setInt(2, schemaOID);
			insertComponent.setString(3, cFk.getName());
			insertComponent.setInt(4, fkOID);
			insertComponent.setInt(5, cFk.getFrom().getOID());
			insertComponent.setInt(6, cFk.getTo().getOID());
			
			insertComponent.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nello store di un componente di una Fk");
		}		
	}

	private void insertFk(PreparedStatement insertfk, ForeignKey fk,int schemaOID, boolean abstractFrom, boolean abstractTo) throws PersistenceException {
		//INSERT INTO SM_FOREIGNKEY (OID,SCHEMEOID,NAME,ABSTRACTFROMOID,ABSTRACTTOOID,STRUCTFROMOID,STRUCTTOOID)
		try {
			insertfk.setInt(1, fkOID);
			fk.setOID(fkOID);
			fkOID++;
			insertfk.setInt(2, schemaOID);
			insertfk.setString(3, fk.getName());
			if(abstractFrom)
				insertfk.setInt(4, fk.getFrom().getOID());
			else
				insertfk.setNull(4, java.sql.Types.INTEGER);
			if(abstractTo)
				insertfk.setInt(5, fk.getTo().getOID());
			else
				insertfk.setNull(5, java.sql.Types.INTEGER);
			
			if(!abstractFrom)
				insertfk.setInt(6, fk.getFrom().getOID());
			else
				insertfk.setNull(6, java.sql.Types.INTEGER);
			if(!abstractTo)
				insertfk.setInt(7, fk.getTo().getOID());
			else
				insertfk.setNull(7, java.sql.Types.INTEGER);
			
			insertfk.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nello store di una Fk");
		}		
	}

	private void insertStruct(PreparedStatement insertStruct,PreparedStatement insertNest,ComplexElement parent, ComplexElement child, int schemaOID) throws PersistenceException {
		//INSERT INTO SM_STRUCTOFATTRIBUTES (OID,SCHEMEOID,NAME,ISSET,ISOPTIONAL,ISNULLABLE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?)
		//INSERT INTO SM_NEST (OID,SCHEMEOID,NAME,STRUCTOFATTRIBUTESTOPOID,STRUCTOFATTRIBUTESNESTEDOID) VALUES (?,?,?,?,?)
		try {
			insertStruct.setInt(1, complexOID);
			child.setOID(complexOID);
			complexOID++;
			insertStruct.setInt(2, schemaOID);
			insertStruct.setString(3, child.getName());
			insertStruct.setBoolean(4, child.isUnbounded());
			insertStruct.setBoolean(5, child.isOptional());
			insertStruct.setBoolean(6, child.isNullable());
			insertStruct.setNull(7, java.sql.Types.INTEGER);
			
			insertNest.setInt(1,nestOID);
			nestOID++;
			insertNest.setInt(2, schemaOID);
			insertNest.setString(3, parent.getName()+child.getName());
			insertNest.setInt(4, parent.getOID());
			insertNest.setInt(5, child.getOID());			
				
			insertStruct.executeUpdate();
			insertNest.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'inserimento di un Lexical");
		}			
	}

	private void lexicalUnBounded(PreparedStatement insertStruct, PreparedStatement lexicalStruct,PreparedStatement insertNest,SimpleElement simple, ComplexElement parent, int schemaOID) throws PersistenceException {
		//INSERT INTO SM_STRUCTOFATTRIBUTES (OID,SCHEMEOID,NAME,ISSET,ISOPTIONAL,ISNULLABLE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?)
		//INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?,?,?,?)
		//INSERT INTO SM_NEST (OID,SCHEMEOID,NAME,STRUCTOFATTRIBUTESTOPOID,STRUCTOFATTRIBUTESNESTEDOID) VALUES (?,?,?,?,?)
		try {
			insertStruct.setInt(1, complexOID);
			insertStruct.setInt(2, schemaOID);
			insertStruct.setString(3, "");
			insertStruct.setBoolean(4, true);
			insertStruct.setBoolean(5, false);
			insertStruct.setBoolean(6, false);
			insertStruct.setNull(7, java.sql.Types.INTEGER);
						
			lexicalStruct.setInt(1, simpleOID);
			simple.setOID(simpleOID);
			simpleOID++;
			lexicalStruct.setInt(2, schemaOID);
			lexicalStruct.setString(3, simple.getName());
			lexicalStruct.setBoolean(4, simple.isNullable());
			lexicalStruct.setBoolean(5, simple.isOptional());
			lexicalStruct.setBoolean(6, simple.isIdentifier());
			lexicalStruct.setString(7, simple.getType());
			lexicalStruct.setInt(8, complexOID);
			
			insertNest.setInt(1,nestOID);
			nestOID++;
			insertNest.setInt(2, schemaOID);
			insertNest.setString(3, "unbound"+simple.getName());
			insertNest.setInt(4, parent.getOID());
			insertNest.setInt(5, complexOID);
			
			complexOID++;			
			insertStruct.executeUpdate();
			lexicalStruct.executeUpdate();
			insertNest.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'inserimento di un Lexical");
		}		
		
	}

	private void insertLexical(PreparedStatement lexicalStruct, SimpleElement simple,int complexOID, int schemaOID) throws PersistenceException {
		//INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?,?,?,?)
		try {
			lexicalStruct.setInt(1, simpleOID);
			simple.setOID(simpleOID);
			simpleOID++;
			lexicalStruct.setInt(2, schemaOID);
			lexicalStruct.setString(3, simple.getName());
			lexicalStruct.setBoolean(4, simple.isNullable());
			lexicalStruct.setBoolean(5, simple.isOptional());
			lexicalStruct.setBoolean(6, simple.isIdentifier());
			lexicalStruct.setString(7, simple.getType());
			lexicalStruct.setInt(8, complexOID);
			
			lexicalStruct.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'inserimento di un Lexical");
		}			
	}

	private void insertStructAbstract(PreparedStatement insertStruct, ComplexElement element,int abstractOID, int schemaOID) throws PersistenceException {
		//INSERT INTO SM_STRUCTOFATTRIBUTES (OID,SCHEMEOID,NAME,ISSET,ISOPTIONAL,ISNULLABLE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?)
		try {
			insertStruct.setInt(1, complexOID);
			element.setOID(complexOID);
			complexOID++;
			insertStruct.setInt(2, schemaOID);
			insertStruct.setString(3, element.getName());
			insertStruct.setBoolean(4, element.isUnbounded());
			insertStruct.setBoolean(5, element.isOptional());
			insertStruct.setBoolean(6, element.isNullable());
			insertStruct.setInt(7, abstractOID);
			
			insertStruct.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nel salvataggio di una struct");
		}
	}

	private void insertLexicalBounded(PreparedStatement insertLexical, SimpleElement simple,int abstractOID,int schemaOID) throws PersistenceException {
		//INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?,?)
		
		try {
			insertLexical.setInt(1, simpleOID);
			simple.setOID(simpleOID);
			simpleOID++;
			insertLexical.setInt(2, schemaOID);
			insertLexical.setString(3, simple.getName());
			insertLexical.setBoolean(4, simple.isNullable());
			insertLexical.setBoolean(5, simple.isOptional());
			insertLexical.setBoolean(6, simple.isIdentifier());
			insertLexical.setString(7, simple.getType());
			insertLexical.setInt(8, abstractOID);
			
			insertLexical.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'inserimento di un Lexical puro collegato ad un abstract");
		}		
	}
	
	private void insertLexicalUnBounded(PreparedStatement insertStruct,PreparedStatement insertLexicalUnbounded, 
			SimpleElement simple,int abstractOID,int schemaOID) throws PersistenceException {
		//INSERT INTO SM_STRUCTOFATTRIBUTES (OID,SCHEMEOID,NAME,ISSET,ISOPTIONAL,ISNULLABLE,ABSTRACTOID) VALUES (?,?,?,?,?,?,?)
		//INSERT INTO SM_LEXICAL (OID,SCHEMEOID,NAME,ISNULLABLE,ISOPTIONAL,ISIDENTIFIER,TYPE,STRUCTOFATTRIBUTESOID) VALUES (?,?,?,?,?,?,?,?)
		try {
			insertStruct.setInt(1, complexOID);
			insertStruct.setInt(2, schemaOID);
			insertStruct.setString(3, "");
			insertStruct.setBoolean(4, true);
			insertStruct.setBoolean(5, false);
			insertStruct.setBoolean(6, false);
			insertStruct.setInt(7, abstractOID);
			
			insertLexicalUnbounded.setInt(1, simpleOID);
			simple.setOID(simpleOID);
			simpleOID++;
			insertLexicalUnbounded.setInt(2, schemaOID);
			insertLexicalUnbounded.setString(3, simple.getName());
			insertLexicalUnbounded.setBoolean(4, simple.isNullable());
			insertLexicalUnbounded.setBoolean(5, simple.isOptional());
			insertLexicalUnbounded.setBoolean(6, simple.isIdentifier());
			insertLexicalUnbounded.setString(7, simple.getType());
			insertLexicalUnbounded.setInt(8, complexOID);
			
			complexOID++;			
			insertStruct.executeUpdate();
			insertLexicalUnbounded.executeUpdate();
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new PersistenceException("Errore nell'inserimento di un Lexical unbounded collegato ad un abstract");
		}		
	}
	
	private int getLexicalUndoundedStruct(int lexicalOID) throws PersistenceException {
		try {
			String query=new String("SELECT STRUCTOFATTRIBUTESOID FROM SM_LEXICAL WHERE OID=?");
			PreparedStatement ps=this.con.prepareStatement(query);
			ps.setInt(1, lexicalOID);
			
			ResultSet rs=ps.executeQuery();
			if(rs.next()) {
				int structOID=rs.getInt(1);
				if(rs.wasNull())
					throw new PersistenceException("Errore nella ricerca dello Struct cui  collegato il Lexical unbounded di OID: "+lexicalOID);
				else
					return structOID;
			}
			
		} catch (SQLException e) {
			throw new PersistenceException("Errore nella ricerca dello Struct cui  collegato il Lexical unbounded di OID: "+lexicalOID);
		}
		return 0;
	}
	
	private int getNestLexicalUnbounded(int structOID) throws PersistenceException {
		try {
			String query=new String("SELECT OID FROM SM_NEST WHERE STRUCTOFATTRIBUTESNESTEDOID=?");
			PreparedStatement ps=this.con.prepareStatement(query);
			ps.setInt(1, structOID);
			
			ResultSet rs=ps.executeQuery();
			if(rs.next()) {
				int nestOID=rs.getInt(1);
				if(rs.wasNull())
					throw new PersistenceException("Errore nella ricerca del Nest cui  collegato il Lexical unbounded collegato allo Struct: "+structOID);
				else
					return nestOID;
			}
			
		} catch (SQLException e) {
			throw new PersistenceException("Errore nella ricerca del Nest cui  collegato il Lexical unbounded collegato allo Struct: "+structOID);
		}		
		return 0;
	}
	
	private int getNestComplexes(int complexTop, int complexNested) throws PersistenceException {
		try {
			String query=new String("SELECT OID FROM SM_NEST WHERE STRUCTOFATTRIBUTESTOPOID=? AND STRUCTOFATTRIBUTESNESTEDOID=?");
			PreparedStatement ps=this.con.prepareStatement(query);
			ps.setInt(1, complexTop);
			ps.setInt(2, complexNested);
			
			ResultSet rs=ps.executeQuery();
			if(rs.next()) {
				int nestOID=rs.getInt(1);
				if(rs.wasNull())
					throw new PersistenceException("Errore nella ricerca del Nest tra i due struct "+complexTop+" e "+complexNested);
				else
					return nestOID;
			}
			
		} catch (SQLException e) {
			throw new PersistenceException("Errore nella ricerca del Nest tra i due struct "+complexTop+" e "+complexNested);
		}		
		return 0;
	}
	
	/**Questo metodo serve perch a livello di oggetti non distinguiamo tra element aappartenenti al primo
	 * livello ed element a livelli inferiori; purtroppo in ModelGen questo avviene e quindi in metodo pi sepmplice
	 * senza modificare la rappresentazione ad oggetti, che  corretta verifico quale  il massimo e faccio partire tutti
	 * da quel valore
	 * 
	 */
	private void checkGreatest() {
		if(topElementOID>=simpleOID && topElementOID>=complexOID) {
			simpleOID=topElementOID;
			complexOID=topElementOID;
		}
		else if(simpleOID>=topElementOID && simpleOID>=complexOID) {
			topElementOID=simpleOID;
			complexOID=simpleOID;
		}
		else if(complexOID>=topElementOID && complexOID>=simpleOID) {
			simpleOID=complexOID;
			topElementOID=complexOID;			
		}		
		
		if(topInstanceOID>=lexicalInstanceOID && topInstanceOID>=complexInstanceOID) {
			lexicalInstanceOID=topInstanceOID;
			complexInstanceOID=topInstanceOID;
		}
		else if(lexicalInstanceOID>=topInstanceOID && lexicalInstanceOID>=complexInstanceOID) {
			complexInstanceOID=lexicalInstanceOID;
			topInstanceOID=lexicalInstanceOID;
		}
		else if(complexInstanceOID>=lexicalInstanceOID && complexInstanceOID>=topInstanceOID) {
			topInstanceOID=complexInstanceOID;
			lexicalInstanceOID=complexInstanceOID;
		}
	}
}
