package org.modelgen.importexport.relational.model;

import org.modelgen.importexport.oo.model.OOClass;

import java.util.Vector;

/**This class represents a Foreign Key.
 * It is not only a Relational FK; it can be used also as a Foreign Key between
 * Tables and OOClass and viceversa
 * 
 * @version 0.9
 */
public class ForeignKey {
	/*questi sono campi statici che servono a verificare che tipo specifico di FK sia
	 * Non  stata fatta una gerarchia con 4 classi perch alla fine l'unica cosa che le diversifica
	 *  che al momento dell'insert nel db di ModelGen bisogna cambiare stringa di inseimento
	 * vedere Relational.storeSchema()
	 *  */
	public static final int TableToTable=1;
	public static final int TableToAbstract=2;
	public static final int AbstractToTable=3;
	public static final int AbstractToAbstract=4;
	
	//campi istanza, uguali al db di ModelGen
	private int OID;
	private String name;
	private int fromOID;
	private int toOID;
	//this represents the type of this FK 
	//and it one of the static field above based on the costructor
	private int type;	
	
	Vector<ComponentOfForeignKey> components;

	/**It creates a NORMAL Fk, that is a FK between two Table
	 * @param OID the FK OID as in ModelGen
	 * @param name the FK name as in ModelGen
	 * @param from Table from
	 * @param to Table to
	 */
	public ForeignKey(int OID, String name,Table from,Table to) {
		this.OID = OID;
		this.name = name;
		fromOID=from.getOID();
		toOID=to.getOID();
		type=TableToTable;
		components = new Vector<ComponentOfForeignKey>();
	}
	
	/**It creates a Fk between a Table and a OOCLass
	 * @param OID the FK OID as in ModelGen
	 * @param name the FK name as in ModelGen
	 * @param from Table from
	 * @param to OOClass to
	 */
	public ForeignKey(int OID, String name,Table from,OOClass to) {
		this.OID = OID;
		this.name = name;
		fromOID=from.getOID();
		toOID=to.getOid();
		type=TableToAbstract;
		components = new Vector<ComponentOfForeignKey>();
	}
	
	/**It creates a Fk between a OOCLass and a Table  
	 * @param OID the FK OID as in ModelGen
	 * @param name the FK name as in ModelGen
	 * @param from OOClass from
	 * @param to Table to
	 */
	public ForeignKey(int OID, String name,OOClass from,Table to) {
		this.OID = OID;
		this.name = name;
		fromOID=from.getOid();
		toOID=to.getOID();
		type=AbstractToTable;
		components = new Vector<ComponentOfForeignKey>();
	}
	
	/**It creates a Fk between two OOCLass
	 * @param OID the FK OID as in ModelGen
	 * @param name the FK name as in ModelGen
	 * @param from OOCLass from
	 * @param to OOCLass to
	 */
	public ForeignKey(int OID, String name,OOClass from,OOClass to) {
		this.OID = OID;
		this.name = name;
		fromOID=from.getOid();
		toOID=to.getOid();
		type=AbstractToAbstract;
		components = new Vector<ComponentOfForeignKey>();
	}


	public int getOID() {
		return OID;
	}
	
	/**Add a ComponentOfForeignKey to this FK
	 * @param cfk
	 */
	public void add(ComponentOfForeignKey cfk){
		components.add(cfk);
	}

	public Vector<ComponentOfForeignKey> getComponents() {
		return components;
	}

	public String getName() {
		return name;
	}

	public int getFromOID() {
		return fromOID;
	}

	public int getToOID() {
		return toOID;
	}

	public int getType() {
		return type;
	}
	
	/**Return a String representation of this Table
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		StringBuilder sb=new StringBuilder("Name: ");
		sb.append(this.name);
		sb.append(" OID: ");
		sb.append(this.OID);		
		sb.append("\n\tFK_FROM: ");
		sb.append(this.fromOID);
		sb.append(" FK_TO: ");
		sb.append(this.toOID);
		for(ComponentOfForeignKey c: components) {
			sb.append("\n\tOID: "+c.getOID()+", nome: "+c.getName()+" OID from: " +c.getFromOID()+
					", OID to:  "+c.getToOID());
		}
		sb.append("\n");
		return sb.toString();
	}
}
