
package org.modelgen.importexport.relational.model;

import org.modelgen.importexport.utilities.Schema;

import java.util.Vector;

/**It represents a Relational schema.
 * A RelationalSchema is composed of a number of Table and eventually Foreign Key between
 * this Table.
 * 
 * The instances of a Relational schema are a number of Tuple each related to a single Table
 * and composed of single value Component that are TupleComponents
 * 
 * @version 0.9
 * 
 */
public class RelationalSchema extends Schema{
	//tables belonging to the schema.
	private Vector<Table> tables;
	//foreign keys
	private Vector<ForeignKey> foreignKeys;	
	
	
	/**Create a new Relational schema with OID and name given
	 * @param OID this corrisponds to SCHEMEOID in ModelGen
	 * @param name this corrisponds to the NAME in ModelGen
	 */
	public RelationalSchema(int OID,String name) {
		super(OID,name);
		this.tables = new Vector<Table>();
		this.foreignKeys=new Vector<ForeignKey>();
	}
	
	public Vector<Table> getTables() {
		return tables;
	}

	/**Adds a new Table to this schema
	 * @param t the new table
	 */
	public void addTable(Table t) {
		tables.add(t);
	}
	
	/**Returns the Table identified by the given OID, null if it does not exist
	 * @param tableOID the OID of the table
	 * @return the Table identified by the given OID, null if it does not exist
	 */
	public Table getTableByOID(int tableOID) {
		for (Table table : tables) {
			if (table.getOID() == tableOID) return table;
		}
		return null;
	}
	
	/**Returns the Table identified by the given Name, null if it does not exist
	 * @param name the name of the table
	 * @return the Table identified by the given OID, null if it does not exist
	 */
	public Table getTableByName(String name) {
		for (Table table : tables) {
			if(table.getName().equalsIgnoreCase(name))
				return table;
		}
		return null;
	}
	
	/**Returns the Column identified by the given OID, null if it does not exist
	 * @param columnOID the OID of the Column
	 * @return the Table identified by the given OID, null if it does not exist
	 */
	public Column getColumnByOID(int columnOID) {
		for (Table table : tables) {
			if (table.getColumnByOID(columnOID) != null) 
				return table.getColumnByOID(columnOID);
		}
		return null;
	}

	public int getOID() {
		return OID;
	}

	/**Adds a new Foreign Key to this Schema
	 * @param fk the FK to be added
	 */
	public void addForeignKey(ForeignKey fk){
		this.foreignKeys.add(fk);
	}

	public Vector<ForeignKey> getForeignKeys() {
		return foreignKeys;
	}
	
	/**Return a String representation of this Schema
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder("\nName: " + this.name);
		sb.append(" OID: ");
		sb.append(this.OID);
		sb.append("\n");
		for (Table table : tables) {
			sb.append(table);
			sb.append("\n");
		}
		sb.append("\nForeign Keys:\n\n");
		for(ForeignKey fk: foreignKeys){
			sb.append(fk.toString());
			sb.append("\n");
		}
		return sb.toString();	
	}	
}
