package org.modelgen.importexport.relational.model;

import java.util.Vector;

/**It represents a Table which has an OID a name and a list of Columns
 *
 * @version 0.9
 */
public class Table {
//	table oid
	protected int OID;
//	table name
	protected String name;
//	columns
	protected Vector<Column> columns;
	
//	link to the instances of this relation (table schema)
	private Vector<Tuple> tuples;

	/**It creates a new Table with the given parameter
	 * @param oid the OID of the table as in ModelGen
	 * @param name the name of the table as in ModelGen
	 */
	public Table(int oid, String name) {
		this.OID = oid;
		this.name = name;
		this.columns = new Vector<Column>();
		this.tuples = new Vector<Tuple>();
	}

	public String getName() {
		return name;
	}

	public int getOID() {
		return OID;
	}

	public Vector<Column> getColumns() {
		return columns;
	}
	
	/**Returns the Column identified by the given name, null if it does not exist
	 * @param name the name of the Column
	 * @return the Column identified by the given OID, null if it does not exist
	 */
	public Column getColumnByName(String name) {
		for(Column c:columns) {
			if(c.getName().equalsIgnoreCase(name))
				return c;
		}
		return null;
	}
	
	/**Adds a new Column to this Table
	 * @param c the new Column
	 */
	public void addColumn(Column c){
		columns.add(c);
	}
	
	public void setColumns(Vector<Column> columns) {
		this.columns = columns;
	}

	public void setTuples(Vector<Tuple> tuples) {
		this.tuples = tuples;
	}
	
	/**Returns the Column identified by the given OID, null if it does not exist
	 * @param columnOID the OID of the Column
	 * @return the Table identified by the given OID, null if it does not exist
	 */
	public Column getColumnByOID(int columnOID) {
		for (Column column : columns) {
			if (column.getOID() == columnOID) 
				return column;
		}
		return null;
	}
	
	public void addTuple(Tuple tuple) {
		tuples.add(tuple);
	}

	public Vector<Tuple> getTuples() {
		return tuples;
	}

	/**Return a String representation of this Table
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder("\nTable: " + name + " oid: "+this.OID+" (");
		String newline = System.getProperty("line.separator");
		sb.append(newline);
		for (Column column : columns) {
			sb.append(column);
			sb.append(newline);
		}
		sb.append(")" + newline);
		
//		now let's print the instances
		sb.append(newline);
		if(tuples.size()==0)
			sb.append("No instances");
		else {
			sb.append("Instances:" + newline);
			for (Tuple tuple : tuples) {
				sb.append("\tTuple OID: ");
				sb.append(tuple.getTupleOID());
				sb.append(newline);
				for (TupleComponent cTuple : tuple.getTupleComponents()) {
					sb.append(cTuple.toString());
				}
				sb.append(newline);
			}
		}		
		return sb.toString();
	}
}
