package org.modelgen.importexport.utilities;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * Configuratore: Configura un'applicazione in base ai parametri di
 * un file di configurazione il cui indirizzo (sottoforma di stringa
 * o di file) viene indicato all'inizio.
 *
 * @version 1.0
 * 
 */
public class Configurator {
	private static Configurator instance;
	private Properties properties;	//la lista delle propriet che mi servono
	
	/**Restituisce l'istanza del Configurator
	 */
	public static Configurator getInstance() {
		if(instance==null) 
			instance=new Configurator();
		return instance;
	}

	/**
	 * Crea un nuovo configuratore a partire dall'indirizzo del file di configurazione.
	 * @throws IOException 
	 */
	private Configurator () {
		File f=new File("data"+File.separator+"org.modelgen.importexport"+
				File.separator+"importExport.config");
		properties = new Properties();
		try {
			properties.load(new FileReader(f));
		} catch (Exception e) {
			/*non gestisto la file not found perch la verifica che il file di config esista
			 *  gestita nello start del plugin */
			 /* potrebbe per essere scritto male il file di config, quindi va controllato*/
			e.printStackTrace();
			System.out.println("Il file di config non  stato letto correttamente");
		}	
	}
	
	/**Returns an array containing ALL the models defined in the config file
	 * 
	 * @return a String array
	 */
	public String[] getModels(){
		//sono memorizzati nella propriet modelli
		String model=getInstance().getCorrispondenza("modelli");
		
		/*sono separati da virgole, quindi devo sapararli*/
		StringTokenizer tokenizer=new StringTokenizer(model,",");
		String[] result=new String[tokenizer.countTokens()];
		
		int i=0;
		while(tokenizer.hasMoreTokens()) {
			result[i]=tokenizer.nextToken();
			i++;
		}
		return result;
	}
	
	/**Returns an array containing ALL the implementations defined in the config file 
	 * for the given model
	 * 
	 * @param model
	 */
	public String[] getImplementations(String model) {
		String implementations=getInstance().getCorrispondenza(model);
		
		/*sono separati da virgole, quindi devo sapararli*/
		StringTokenizer tokenizer=new StringTokenizer(implementations,",");
		String[] result=new String[tokenizer.countTokens()];
		
		int i=0;
		while(tokenizer.hasMoreTokens()) {
			result[i]=tokenizer.nextToken();
			i++;
		}
		return result;
	}
	
	public String[] getExporterClass() {
		String exporters=getCorrispondenza("Exporter");
		
		/*sono separati da virgole, quindi devo sapararli*/
		StringTokenizer tokenizer=new StringTokenizer(exporters,",");
		String[] result=new String[tokenizer.countTokens()];
		
		int i=0;
		while(tokenizer.hasMoreTokens()) {
			result[i]=tokenizer.nextToken();
			i++;
		}
		return result;
	}
    
    public String getCorrispondenza(String key) {
    	if(this.properties.containsKey(key))
    		return this.properties.getProperty(key);
    	else
    		return null;
    }
}

