package org.modelgen.importexport.utilities;

import org.modelgen.importexport.xml.model.ComplexElement;
import org.modelgen.importexport.xml.model.ComponentOfForeignKey;
import org.modelgen.importexport.xml.model.Element;
import org.modelgen.importexport.xml.model.ForeignKey;
import org.modelgen.importexport.xml.model.Key;
import org.modelgen.importexport.xml.model.SimpleElement;
import org.modelgen.importexport.xml.model.XmlSchema;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

/**A parser for Xpath expression on Key and ForeignKey.
 * Due to ModelGen limitation a Xpath expression for selector must :
 * 
 * 1)begin with "."
 * 2)for each element travelled it must append "/nameElement"
 * 
 * Other expressione like *,// are not accepted.
 * 
 * The xpath expression for each field must contain only a DIRECT child of the 
 * context node, it can't specify a gran-child and so on.
 * The only accpeted expression is so "nameOfTheElement" 
 * 
 * 
 * @version 0.9
 * 
 */
public class XmlSchemaXpathParser {
	
	/**Set to true the property isIdentifier for the SimpleElement of the given Key.
	 * 
	 * First it search the context node; once found for each field of the Key look 
	 * for SimpleElement with the given name
	 *  
	 * @param key
	 * @param e
	 * @throws ImportExportException
	 */
	public static void setPrimaryKey(Key key,Element e) throws ImportExportException {
		String selector=key.getSelector();
		
		if(selector.startsWith(".")) {
			List<String> nodiDaAttraversare=getNodiDaAttraversare(selector);
			//System.out.println("nodi da attraversare:"+nodiDaAttraversare);
			
			if(nodiDaAttraversare.size()>0) {
				for(Element child:e.getChilds())
					setIdentifier(key,child,nodiDaAttraversare,0);
			}
			else {
				//System.out.println("setto come context node dellla chiave "+key.getName()+" il complex "+e.getName());
				if(e instanceof ComplexElement) {
					key.setContextNode((ComplexElement)e);
					setIdentifier(key,e);
				}
				else
					throw new ImportExportException("L'elemento a cui corrisponde il selector della chiave "+key.getName()+
							"  stato trovato, ma non  un elemento complesso");				
			}
		}
		else {
			throw new ImportExportException("Le key e keyref sono ammesso solamente dall'elemento radice\n" +
					"e devono iniziare con il punto");
		}
	}
	
	/**Add the correct ForeignKey corrisponding to the "String" Fk.
	 * 
	 * First it search the context node; once found for each field of the Key look 
	 * for SimpleElement with the given name
	 * 
	 * @param fk
	 * @param key
	 * @param e
	 * @param schema
	 * @throws ImportExportException
	 */
	public static void getforeignKey(Key fk,Key key,Element e,XmlSchema schema) throws ImportExportException {
		String selector=fk.getSelector();
		
		if(selector.startsWith(".")) {
			List<String> nodiDaAttraversare=getNodiDaAttraversare(selector);
			//System.out.println("nodi da attraversare fk:"+nodiDaAttraversare);
			
			if(nodiDaAttraversare.size()>0) {
				for(Element child:e.getChilds())
					getforeignKey(fk,key,child,schema,nodiDaAttraversare,0);
			}
			else {
				if(e instanceof ComplexElement) {
					//System.out.println("aggiungo una fk");
					schema.addForeignKey(createFk(fk, key, e));
				}
				else
					throw new ImportExportException("L'elemento "+e.getName()+" non  complesso quindi " +
							"non pu appartenere ad una chiave esterna");	
			}
		}
		else {
			throw new ImportExportException("Le key e keyref sono ammesso solamente dall'elemento radice\n" +
					"e devono iniziare con './'");
		}
	}
	
	/**Utility metod which use ricorsion to look for the context node of the Fk.
	 * 
	 * @param fk
	 * @param key
	 * @param e
	 * @param schema
	 * @param nodiDaAttraversare
	 * @param depth
	 * @throws ImportExportException
	 */
	private static void getforeignKey(Key fk,Key key,Element e,XmlSchema schema,List<String> nodiDaAttraversare,int depth) throws ImportExportException {
		if(nodiDaAttraversare.get(depth).equals(e.getName())) {
			if(nodiDaAttraversare.size()==(depth+1) && e instanceof ComplexElement) {
				//System.out.println("aggiungo fk");
				schema.addForeignKey(createFk(fk, key, e));
			}
			else {
				for(Element child:e.getChilds()) {
					getforeignKey(fk, key, child, schema, nodiDaAttraversare, depth+1);
				}
			}
		}
	}
	
	/**Utility metod which create the Fk oonce found the context node.
	 * 
	 * @param fk
	 * @param key
	 * @param e
	 * @throws ImportExportException
	 */
	private static ForeignKey createFk(Key fk,Key key,Element e) throws ImportExportException {
		ForeignKey foreignKey=new ForeignKey(fk.getName(),(ComplexElement)e,key.getContextNode());
		//System.out.println("setto il from "+e.getName()+" to: "+key.getContextNode().getName());
		if(fk.getFields().size()==key.getFields().size()) {
			Vector<Element> childs=e.getChilds();
			
			int i=0;
			while(i<fk.getFields().size()) {
				String s=fk.getFields().get(i);
				//System.out.println("devo trovare il field"+s+" per la fk "+foreignKey.getName());
				
				for(Element child:childs) {
					if(child.getName().equals(s) && child instanceof SimpleElement) {
						SimpleElement simple=(SimpleElement)child;
						ComponentOfForeignKey cFK=new ComponentOfForeignKey(fk.getFields().get(i)+"-"+
									key.getFields().get(i),simple,key.getFieldNodeByName(key.getFields().get(i)));
						foreignKey.addComponent(cFK);
					}
				}							
				i++;
			}
		}
		else {
			throw new ImportExportException("Il numero di Field della chiave e della foreign key non coincide");
		}
		return foreignKey;		
	}
	
	/**Separate the nodes to be crossed to reach the context node.
	 * 
	 * @param selector
	 */
	private static List<String> getNodiDaAttraversare(String selector) {
		List<String> nodi=new ArrayList<String>();
		StringTokenizer st=new StringTokenizer(selector,"./");
		
		while(st.hasMoreTokens()) {
			nodi.add(st.nextToken());
		}
		
		return nodi;
	}	
	
	/**Utility metod which set to true the IsIdentifer property of the right Element.
	 * 
	 * @param key
	 * @param e
	 * @throws ImportExportException
	 */
	private static void setIdentifier(Key key, Element e) throws ImportExportException {
		Vector<Element> childs=e.getChilds();
		
		for(String s:key.getFields()) {
			if(s.contains("./"))
				throw new ImportExportException("Un field NON pu specificare child che non " +
						"appartengono al nodo selezionato dal path del selector");
			
			boolean found=false;
			
			for(Element child:childs) {
				if(child.getName().equals(s) && child instanceof SimpleElement) {
					SimpleElement simple=(SimpleElement)child;
					found=true;
					simple.setIdentifier(true);
					//System.out.println("setto isId=true per il simple "+simple.getName());
					key.addFieldNode(simple);
				}
			}
			if(!found)
				throw new ImportExportException("Il field "+s+" NON  stato trovato tra i figli del " +
						"nodo\nAccertarsi che i field specificati siano figli diretti del context node");
		}
	}	
	
	/**Utility metod which set to true the IsIdentifer property of the right Element.
	 * @param key
	 * @param e
	 * @param nodiDaAttraversare
	 * @param depth
	 * @throws ImportExportException
	 */
	private static void setIdentifier(Key key, Element e, List<String> nodiDaAttraversare,int depth) throws ImportExportException {
		if(nodiDaAttraversare.get(depth).equals(e.getName())) {
			if(nodiDaAttraversare.size()==(depth+1) && e instanceof ComplexElement) {
				//System.out.println("setto come context node dellla chiave "+key.getName()+" il complex "+e.getName());
				key.setContextNode((ComplexElement)e);
				setIdentifier(key, e);
			}
			else {
				for(Element child:e.getChilds()) 
					setIdentifier(key, child, nodiDaAttraversare, depth+1);
			}
		}
	}
}
