package org.modelgen.importexport.utilities.datatype;

import org.modelgen.importexport.utilities.ImportExportException;
import org.modelgen.importexport.utilities.PersistenceException;

/**
 * @author LordofDeceits
 *
 */
public class DB2RelationalDataType {
	/**Lunghezza di default delle stringhe*/
	private static final int LENGTH=50;
	
	/**Questo metodo fa un check sul tipo verificando che siano tipi accettabili per uno schema
	 * di tipo relazionale ed inoltre nel caso siano validi ritorna una stringa che poi andr 
	 * salvata come tipo nel db di ModelGen in quanto si tratta di un tipo SQL valido a se stante
	 * 
	 * Vedere l'appendice della tesi: tipi ammissibili
	 * 
	 * @throws PersistenceException nel caso di tipo non supportato
	 */
	public static String getModelGenDataType(int SQLDataType,String typeName,int length) throws PersistenceException {
		/*DB2 NON ha il tipo boolean :)
		 * if(SQLDataType==java.sql.Types.BOOLEAN)
			return "boolean";*/
		if(SQLDataType==java.sql.Types.INTEGER)
			return "integer";
		else if(SQLDataType==java.sql.Types.DOUBLE)
			return "double";
		else if(SQLDataType==java.sql.Types.CHAR || SQLDataType==java.sql.Types.VARCHAR)
			return "string";
		else {
			throw new PersistenceException("Tipo non ammesso per una tabella Relazionale: SQLDataType: "+SQLDataType+" di nome: "+typeName
					+"\nSi prega di consultare la tabella dei tipi ammessi per il modello");
		}		
	}
	
	/**Questo metodo in base al tipo della colonna verifica che il valore sia all'interno del range ammesso per i valori
	 * di tale tipo.
	 * 
	 *  Vedere l'appendice della tesi: tipi ammissibili
	 * 
	 * @param type
	 * @param value
	 * @throws PersistenceException
	 */
	public static String getModelGenValue(String type,String value) throws PersistenceException {
		/*DB2 NON ha il tipo boolean :)
		 * if(type.equals("boolean") && value.equalsIgnoreCase("true"))
			return "true";
		else if(type.equals("boolean") && value.equalsIgnoreCase("false"))
			return "false";*/
		if(type.equals("integer"))
			return value;
		else if(type.equals("double"))
			return value.replaceAll(",",".");
		else if(type.equals("string"))
			return value;
		
		else {
			throw new PersistenceException("Valore non ammesso per una tabella Relazionale: Type: "+type+" di valore: "+value);
		}
	}
	
	
	/**Trasforma il tipo contenuto in ModelGen in un tipo SQL valido
	 * 
	 * @param type
	 */
	public static String getRelationalType(String type) throws ImportExportException {
		if(type.equals("string"))
			return "varchar("+LENGTH+")";
		else if(type.equalsIgnoreCase("boolean"))
			throw new ImportExportException("DB2 non supporta il tipo boolean");
		else 
			return type;
	}
	
	/**Trasforma il valore del lexical contenuto in ModelGen in un valore sql valido
	 * 
	 * @param type
	 * @param value
	 */
	public static String getRelationalValue(String type,String value) {
		if(type.equals("string"))
			return "'"+value+"'";
		else
			return value;
	}

}
