package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents a ComplexElement.
 * A Element in XmlSchema can be complex because its type is a global complexType or it has a child complexType
 * which contains a list of other Element.
 * For more information see compleType or thesis
 * 
 * @version 0.9
 *
 */

public class ComplexElement extends Element {
	//the link to the complexType 
	private ComplexType complexType;

	/**Creates a ComplexElement with the given parameters.
	 * 
	 * @param oid
	 * @param name
	 * @param isOptional
	 * @param isNullable
	 * @param complexType
	 */
	public ComplexElement(int oid, String name, boolean isOptional, boolean isNullable,boolean unbounded, ComplexType complexType) {
		super(oid, name);
		this.isOptional = isOptional;
		this.isNullable = isNullable;
		this.unbounded = unbounded;
		this.complexType = complexType;
	}
	
	/**Creates a ComplexElement with the given parameters.
	 * 
	 * @param name
	 * @param isOptional
	 * @param isNullable
	 * @param complexType
	 */
	public ComplexElement(String name, boolean isOptional, boolean isNullable,boolean unbounded, ComplexType complexType) {
		super(Numberable.NO_NUMBERED, name);
		this.isOptional = isOptional;
		this.isNullable = isNullable;
		this.unbounded = unbounded;
		this.complexType = complexType;
	}

	public boolean isNullable() {
		return isNullable;
	}

	public boolean isOptional() {
		return isOptional;
	}	

	public boolean isUnbounded() {
		return unbounded;
	}

	public ComplexType getComplexType() {
		return complexType;
	}
	
	/**Returns the list of child of this element, which are those defined in the complexType.
	 * 
	 * @see org.modelgen.importexport.xml.model.Element#getChilds()
	 */
	public Vector<Element> getChilds() {
		return this.complexType.getElements();
	}	
	
	/**Returns a String representation of this ComplexElement considering also the indentation.
	 * 
	 * @param base the indentation to be used
	 */
	public String getRepresentation(String base) {
		StringBuilder sb=new StringBuilder(base);
		sb.append("ComplexElement :");
		if(this.OID!=Numberable.NO_NUMBERED)
			sb.append(this.OID);
		sb.append(" name :");
		sb.append(this.name);
		sb.append(" isOptional :");
		sb.append(this.isOptional);
		sb.append(" isNullable :");
		sb.append(this.isNullable);
		sb.append(" unbounded :");
		sb.append(this.unbounded);
		sb.append("\n");
		sb.append(complexType.getRepresentation(base));	
		
		return sb.toString();
	}

	/**Returns a String representation of this ComplexElement.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getRepresentation("");
	}
}
