package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents an instance of a ComplexElement.
 * 
 * a Complex element is something like:
 * <complexElement>
 * 		<instance1>..</instance1>
 * 		<instance2>..</instance2>
 * 		<instance3>
 * 				....
 * 		</instance3>
 * <complexElement>
 */
public class ComplexInstance extends InstanceElement {
	private Vector<InstanceElement> instances;

	/**Creates a ComplexInstance.
	 * 
	 * @param oid
	 * @param e
	 */
	public ComplexInstance(int oid,Element e) {
		super(oid,e);
		this.instances = new Vector<InstanceElement>();
	}
	
	public ComplexInstance(Element e) {
		super(e);
		this.instances = new Vector<InstanceElement>();
	}
	
	public void addInstance(InstanceElement e) {
		this.instances.add(e);
	}
	
	public Vector<InstanceElement> getInstances() {
		return instances;
	}
	
	/**Returns a String representation of this ComplexElement considering also the indentation.
	 * 
	 * @param base the indentation to be used
	 */
	public String getRepresentation(String base) {
		StringBuilder sb=new StringBuilder(base);
		sb.append("ComplexInstance :");
		if(this.OID!=Numberable.NO_NUMBERED)
			sb.append(this.OID);
		sb.append(" Elemento :");
		sb.append(this.schemaElement.getName());
		sb.append("\n");
		for(InstanceElement element:this.instances) {
			sb.append(element.getRepresentation(base+"\t"));
			sb.append("\n");
		}
		sb.append("\n");
		
		return sb.toString();
	}

	/** Returns a String representation of this Object.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getRepresentation("");
	}
}
