package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents a XmlSchema ComplexType.
 * A XmlSchema complexType can have a name if it is global. It contains a <code>sequence</code>
 * of Element which can be Simple/ComplexElement and global or local.
 * In XmlSchema a ComplexType can have also other child like:
 * 1)all
 * 2)choice
 * 3)attribute
 * these element are ignored because they don't have a corrisponding costruct in ModelGen
 * 
 * @version 0.9
 * 
 */

public class ComplexType {
	//OID used only for export
	private int OID;
	//name of the type if it's global
	private String name;
	//ho una lista di elementi, sia lessicali che ref(no le ref sono state tolte dal modello)
	private Vector<Element> elements;
	
	/**Creates a ComplexType with the given name.
	 * 
	 * @param name
	 */
	public ComplexType(String name) {
		this.OID=-1;
		this.name = name;
		this.elements=new Vector<Element>();
	}
	
	/**Create a ComplexType with the given name. USED only to simplify export
	 * @param name
	 */
	public ComplexType(int OID,String name) {
		this.OID=OID;
		this.name = name;
		this.elements=new Vector<Element>();
	}
	
	/**Add a Element e to the list of the child element of this complex type
	 * 
	 * @param e
	 */
	public void addElement(Element e) {
		this.elements.add(e);
	}

	public String getName() {
		return name;
	}

	public int getOID() {
		return OID;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public Vector<Element> getElements() {
		return elements;
	}
	
	/**Returns a String representation of this ComplexType considering also the indentation.
	 * 
	 * @param base the indentation to be used
	 */
	public String getRepresentation(String base) {
		StringBuilder sb=new StringBuilder("");
		for(Element e:this.elements) {
			if(e!=null) {
				sb.append(base);
				sb.append(e.getRepresentation(base+"\t"));
				sb.append("\n");
			}
			else
				sb.append("elemnto nullo");
		}		
		return sb.toString();
	}
	
	/**Returns a String representation of this ComplexType.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("complesso "+this.name+ "OID: "+this.OID);
		for(Element e:this.elements) {
			sb.append(e.toString());
			sb.append("\n");
		}		
		return sb.toString();
	}
}

