package org.modelgen.importexport.xml.model;

/**Represents the field of a ForeignKey in XmlSchema.
 * A ForeignKey in XmlSchema has a number of field which are part of the ForeignKey.
 * They are what in a Relational Schema are the Columns that belongs to a ForeignKey.
 * In pure XmlSchema
 * 
 * @version 0.9
 *
 */

public class ComponentOfForeignKey {
	//OID as stored in ModelGen
	private int OID;
	//name
	private String name;
	//Simple from
	private SimpleElement from;
	//Simple to
	private SimpleElement to;
	
	/**Creates a ComponentOfForeignKey between the given SimpleElement.
	 * 
	 * @param oid
	 * @param name
	 * @param from
	 * @param to
	 */
	public ComponentOfForeignKey(int oid, String name, SimpleElement from, SimpleElement to) {
		super();
		OID = oid;
		this.name = name;
		this.from = from;
		this.to = to;
	}

	/**Creates a ComponentOfForeignKey between the given SimpleElement.
	 * 
	 * @param name
	 * @param from
	 * @param to
	 */
	public ComponentOfForeignKey(String name, SimpleElement from, SimpleElement to) {
		super();
		this.OID= Numberable.NO_NUMBERED;
		this.name = name;
		this.from = from;
		this.to = to;
	}

	public SimpleElement getFrom() {
		return from;
	}

	public String getName() {
		return name;
	}

	public int getOID() {
		return OID;
	}

	public void setOID(int oid) {
		OID = oid;
	}

	public SimpleElement getTo() {
		return to;
	}
	
	/**Returns a String representation of this object.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("\t");
		if(this.OID!=Numberable.NO_NUMBERED) {
			sb.append("OID: "); 
			sb.append(this.OID);
		}
		sb.append(" name: ");
		sb.append(this.name);
		sb.append(" fromElement: ");
		sb.append(this.from.getName());
		sb.append(" toElement: ");
		sb.append(this.to.getName());		
		
		return sb.toString();
	}

}
