package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents a base Element.
 * It's a "pure fabrication" object to summarize some common attributes.
 * 
 * @version 0.9
 *
 */
public abstract class Element {
	//OID as stored in ModelGen
	protected int OID;
	//name
	protected String name;
	//property isNullable
	protected boolean isNullable;
	//property isOptional
	protected boolean isOptional;
	//property unbounded
	protected boolean unbounded;
		
	/**Initializes the OID and the name of the Element
	 * @param oid
	 * @param name
	 */
	public Element(int oid, String name) {
		super();
		OID = oid;
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getOID() {
		return OID;
	}

	public void setOID(int oid) {
		OID = oid;
	}
	
	public boolean isNullable() {
		return isNullable;
	}

	public boolean isUnbounded() {
		return unbounded;
	}

	public boolean isOptional() {
		return isOptional;
	}

	/**Returns a list containing all the child of this Element.
	 */
	public abstract Vector<Element> getChilds();
	
	/**Returns a String representation of this Element considering also the indentation.
	 * 
	 * @param base the indentation to be used
	 */
	public String getRepresentation(String base) {
		return base+toString();
	}
	
	/**Returns a String representation of this Element.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Element:"+this.name+" OID:"+this.OID;
	}
}
