package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents a ForeignKey in XmlSchema.
 * In XmlSchema a ForeignKey is defined by a name and a Key it refers to.
 * A ForeignKey is hence defined by its name and the ComplexElements it refers to.
 * It's also composend by a number of ComponentOfForeignKey which specify which SimpleElement
 * are the field of the ForeignKey.
 * 
 * @version 0.9
 *
 */

public class ForeignKey {
	private int OID;
	private String name;
	private ComplexElement from;
	private ComplexElement to;
	private Vector<ComponentOfForeignKey> components;
	
	/**Creates a ForeignKey between the two ComplexElement.
	 * 
	 * @param oid
	 * @param name
	 * @param from
	 * @param to
	 */
	public ForeignKey(int oid, String name, ComplexElement from, ComplexElement to) {
		super();
		OID = oid;
		this.name = name;
		this.from = from;
		this.to = to;
		this.components = new Vector<ComponentOfForeignKey>();
	}	
	
	/**Creates a ForeignKey between the two ComplexElement.
	 * 
	 * @param name
	 * @param from
	 * @param to
	 */
	public ForeignKey(String name, ComplexElement from, ComplexElement to) {
		super();
		this.OID=Numberable.NO_NUMBERED;
		this.name = name;
		this.from = from;
		this.to = to;
		this.components = new Vector<ComponentOfForeignKey>();
	}
	
	/**Add a ComponentOfForeignKey to this ForeignKey
	 * 
	 * @param component the component to be added
	 */
	public void addComponent(ComponentOfForeignKey component) {
		this.components.add(component);
	}

	public ComplexElement getFrom() {
		return from;
	}
	
	public String getName() {
		return name;
	}
	
	public int getOID() {
		return OID;
	}
	
	public void setOID(int oid) {
		OID = oid;
	}
	
	public ComplexElement getTo() {
		return to;
	}
	
	public Vector<ComponentOfForeignKey> getComponents() {
		return components;
	}

	/**Returns a String representation of this object.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("Fk:");
		if(this.OID!=Numberable.NO_NUMBERED) {
			sb.append(this.OID);
		}
		sb.append(" name: ");
		sb.append(this.name);
		sb.append(" fromComplex: ");
		sb.append(this.from.getName());
		sb.append(" toComplex: ");
		sb.append(this.to.getName());	
		sb.append("\n");
		
		for(ComponentOfForeignKey cFk:components) {
			sb.append(cFk.toString());
			sb.append("\n");
		}
		
		return sb.toString();
	}
}
