package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Representes a "pure" XmlSchema primary key and ForeignKey(used only during Import).
 * In xmlSchema both key and keyref have the same structure(except for the refer attribute of the keyref)
 * As you can see from the code of the importer, first of all, all keys and keyref and their property
 * are loaded as string; after all Elements are loaded the String are converted in their corrispondending 
 * values; for slector we look for the ComplexElement the string represents; foeach field we look for 
 * the SimpleElement they represent.
 * 
 *
 *@version 0.9
 *
 */
public class Key {
	//name of the key		//string refer
	private String name;	private String refer;
		//string selector
		private String selector;
		//list of the string of the fields
		private Vector<String> fields;
	
	//the ComplexElement corrisponding to the selector path
	private ComplexElement contextNode;
	//the list of SimpleElement corrispondonding to the fields defined
	private Vector<SimpleElement> fieldNode;
	
	/**Creates a primary key
	 * @param name
	 * @param selector
	 */
	public Key(String name, String selector) {
		super();
		this.name = name;
		this.selector = selector;
		this.fields = new Vector<String>();
		this.fieldNode = new Vector<SimpleElement>();
	}	
	
	/**Create a Foreign Key
	 * @param name
	 * @param selector
	 * @param refer
	 */
	public Key(String name, String selector, String refer) {
		super();
		this.name = name;
		this.selector = selector;
		this.refer = refer;
		this.fields = new Vector<String>();
		this.fieldNode = new Vector<SimpleElement>();
	}

	/**Add a String field
	 * 
	 * @param field
	 */
	public void addField(String field) {
		this.fields.add(field);
	}
	
	/**Add a simpleElement corrisponding to a field of the Key.
	 * 
	 * @param field
	 */
	public void addFieldNode(SimpleElement field) {
		this.fieldNode.add(field);
	}
	
	public ComplexElement getContextNode() {
		return contextNode;
	}

	/**Set the contextNode corrisponding to the xpath attribute of the selector defined for the Key.
	 * 
	 * @param contextNode
	 */
	public void setContextNode(ComplexElement contextNode) {
		this.contextNode = contextNode;
	}

	/**Returns a list of string of the Fields.
	 */
	public Vector<String> getFields() {
		return fields;
	}

	public String getName() {
		return name;
	}

	public String getRefer() {
		return refer;
	}

	public String getSelector() {
		return selector;
	}
	
	public Vector<SimpleElement> getFieldNodes() {
		return fieldNode;
	}

	/**Return a SimpleElement corrisponding to a field of the Key given its name.
	 * 
	 * @param fieldName the name of the SimpleElement
	 */
	public SimpleElement getFieldNodeByName(String fieldName) {
		for(SimpleElement simple:this.fieldNode) {
			if(simple.getName().equals(fieldName))
				return simple;
		}
		return null;
	}

	/**Returns a String representation of this object.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder("chiave: ");
		sb.append(this.name);
		if(this.refer!=null) {
			sb.append(" refer: ");
			sb.append(this.refer);
		}		
		sb.append(" selector: ");
		sb.append(this.selector);
		sb.append("\n");
		
		for(String s:this.fields) {
			sb.append("\t");
			sb.append(s);
		}
		sb.append("\n");
		
		return sb.toString();
	}
}
