package org.modelgen.importexport.xml.model;

import java.util.Vector;

/**Represents an Element which has a type that is simple.
 * A SimpleElement can be defined specifing a simpleType global using the attribute type or
 * declaring a simpleType child with a restriction.
 *  
 * @version 0.9
 *
 */

public class SimpleElement extends Element {
	//a simpleElement can also be part of a key
	private boolean isIdentifier;
	//the string representing the type
	private String type;
	
	/**Creates a SimpleElement with the given parameters.
	 * 
	 * @param oid
	 * @param name
	 * @param isNullable
	 * @param isIdentifier
	 * @param type
	 */
	public SimpleElement(int oid, String name,boolean isNullable,boolean isOptional,boolean isIdentifier,boolean unbounded,String type) {
		super(oid, name);
		this.isNullable = isNullable;
		this.isOptional= isOptional;
		this.unbounded=unbounded;
		this.isIdentifier = isIdentifier;
		this.type = type;
	}
	
	/**Creates a SimpleElement with the given parameters.
	 * 
	 * @param name
	 * @param isNullable
	 * @param isIdentifier
	 * @param type
	 */
	public SimpleElement(String name,boolean isNullable,boolean isOptional,boolean isIdentifier,boolean unbounded,String type) {
		super(Numberable.NO_NUMBERED, name);
		this.isNullable = isNullable;
		this.isOptional= isOptional;
		this.unbounded=unbounded;
		this.isIdentifier = isIdentifier;
		this.type = type;
	}
	
	public String getType() {
		return type;
	}
	
	public boolean isIdentifier() {
		return isIdentifier;
	}
	
	/**Set the SimpleElement to be part of a primary key.
	 * 
	 * @param isIdentifier
	 */
	public void setIdentifier(boolean isIdentifier) {
		this.isIdentifier = isIdentifier;
	}

	/**Returns an empty list because a SimpleElement cannot have child.
	 * 
	 * @see org.modelgen.importexport.xml.model.Element#getChilds()
	 */
	public Vector<Element> getChilds() {
		return new Vector<Element>();
	}	

	/**Returns a String representation of this SimpleElement considering also the indentation.
	 * 
	 * @param base the indentation to be used
	 */
	public String getRepresentation(String base) {
		StringBuilder sb=new StringBuilder("SimpleElement :");
		if(this.OID!=Numberable.NO_NUMBERED) { 
			sb.append(this.OID);
		}
		sb.append(" name :");
		sb.append(this.name);
		sb.append(" isNullable ");
		sb.append(isNullable);
		sb.append(" isId ");
		sb.append(isIdentifier);
		sb.append(" unbounded ");
		sb.append(unbounded);
		sb.append(" type ");
		sb.append(type);

		return base+sb.toString();
	}

	/**Returns a String representation of this SimpleElement.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getRepresentation("");
	}
}
