package org.modelgen.importexport.xml.model;

import org.modelgen.importexport.utilities.Schema;

import java.util.Vector;

/**Represents a Xml schema.
 * A XmlSchema is composed of a number of Simpletype/ComplexType and a number of Element which can be
 * SimpleElement if their type is simple or ComplexElement if their type is Complex.
 * A Xml schema has also a number of Foreign Key between its ComplexElement; each Foreign Key specify 
 * a number of Component which are the field of the foreign key which referes a Primary Key
 * 
 * @version 0.9
 *
 */

public class XmlSchema extends Schema {
	//lista dei top element ( una lista per scelta originaria), ormai il top  solo uno
	private Vector<Element> topElements;
	//le fk
	private Vector<ForeignKey> foreignKeys;
	//lista istanze
	private Vector<InstanceElement> instances;
	
	/**Creates a XmlSchema with the given parameters.
	 * 
	 * @param OID the schemeOID as stored in ModelGen
	 * @param name the schema name as stored in ModelGen
	 */
	public XmlSchema(int OID,String name) {
		super(OID,name);
		this.topElements=new Vector<Element>();
		this.foreignKeys=new Vector<ForeignKey>();
		this.instances=new Vector<InstanceElement>();
	}	
	
	/**Add a ForeignKey to the list of the ForeignKey of this schema.
	 * 
	 * @param fk the ForeignKey to be added
	 */
	public void addForeignKey(ForeignKey fk) {
		this.foreignKeys.add(fk);
	}
	
	/**Add an Element to the list of the Element of this schema.
	 * 
	 * @param e the Element to be added
	 */
	public void addElement(Element e) {
		this.topElements.add(e);
	}
	
	public Vector<Element> getTopElements() {
		return topElements;
	}
	
	public void addInstance(InstanceElement e) {
		this.instances.add(e);
	}
	
	public Vector<InstanceElement> getInstances() {
		return instances;
	}

	/**Return a ForeignKey identified by its OID.
	 * 
	 * @param fkOID the OID to look for
	 * @return the ForeignKey if it exist null otherwise
	 */
	public ForeignKey getForeignKeyByOID(int fkOID) {
		for(ForeignKey fk:this.foreignKeys) {
			if(fk.getOID()==fkOID)
				return fk;
		}
		return null;
	}
	
	/**Returns the List of the ForeignKey of this Schema where the from is the given Element.
	 * 
	 * @param e the Element from
	 * @return the List of the ForeignKey of this Schema where the from is the given Element
	 */
	public Vector<ForeignKey> getFk(ComplexElement e) {
		Vector<ForeignKey> fks= new Vector<ForeignKey>();
		
		for(ForeignKey fk:this.foreignKeys) {
			if(fk.getFrom()==e)
				fks.add(fk);
		}
		
		return fks;
	}
	
	public Vector<ForeignKey> getForeignKeys() {
		return foreignKeys;
	}

	/**Returns a String representation of this Schema.
	 * 
	 * @see org.modelgen.importexport.utilities.Schema#toString()
	 */
	public String toString() {
		StringBuilder sb=new StringBuilder(super.toString());
		sb.append("\n");
		
		for(Element e:topElements) {
			sb.append(e.toString());
			sb.append("\n");
		}
		
		for(ForeignKey fk:this.foreignKeys) {
			sb.append(fk.toString());
			sb.append("\n");
		}
		
		for(InstanceElement instance:this.instances) {
			sb.append(instance.toString());
			sb.append("\n");
		}
		
		return sb.toString();
	}
}
