/*
 * This class implements the main wondow for ApplyInstanceRules plugin 
 */
package org.modelgen.jpf.applyInstanceRules;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.applyInstanceRules.apply.PanelInfo;
import org.modelgen.jpf.applyInstanceRules.apply.PanelRules;
import org.modelgen.jpf.core.commons.Tool;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *  @author kristina
 *  This class implements the main wondow for ApplyInstanceRules plugin
 */
public class ApplyInstanceRulesTool extends JInternalFrame implements Tool {
	private static final long serialVersionUID = 4933945007817074751L;
	
	/* Singleton */
	private static ApplyInstanceRulesTool istance;
	
	/* Mappa condivisa */
	/**
	 * @uml.property  name="applicationObjs"
	 * @uml.associationEnd  qualifier="key:java.lang.Object java.lang.Object"
	 */
	private Map<String,Object> applicationObjs;
	
	// Layout manager
	private GridBagConstraints generalLayout;
	
	// General panel of the window
	private JPanel pnlGeneral;
	
	
	// Menu
	private MenuApplyInstanceRule menu;
	// ToolBar
	private ToolBarApplyInstanceRule tool;
	// Panel for general info to apply rules
	private PanelInfo pnlInfo;
	// Panel rules
	private PanelRules pnlRules;
	
	
	
	/**
	 * Class constructor
	 * @throws PersistenceException
	 */
	public ApplyInstanceRulesTool() throws PersistenceException{
		super("ApplyInstanceRulesTool",true,true,true,true);
		checkConnection();
		initProperties();
		// Creates general panel
		this.pnlGeneral = new JPanel();
		this.setLayout();
				
		// Creates panel info
		this.pnlInfo = PanelInfo.getInstance();
		// Creates panel rules
		this.pnlRules = PanelRules.getInstance(this.pnlInfo);
		
		
		// Creates and shows the component
		this.createShow();
		this.pack();
		/*Consente di applicare il pattern singleton nonostante la classe 
		 * venga istanziata dal CorePlugin*/
		istance = this;
	}

	/**
	 * Check the connection
	 * @throws PersistenceException
	 */
	private void checkConnection() throws PersistenceException{	
		/*Inizializza il PH condiviso e se il server non  avviato provoca 
		 *un eccezione intercettata dalla ToolActivationAction
		 * */
		GeneralManagementFacade.getInstance().initPesistenceHandler();
	}
	
	/**
	 * Initialize the properties of the windows
	 */
	private void initProperties() {
		// L'icona
		ImageIcon imageIcon = ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png");
		setFrameIcon(imageIcon);
		// Alla chiusura
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);			
		Toolkit kit = Toolkit.getDefaultToolkit();
		/* Take x and y coordinates of the screnn */
		int y = (int)kit.getScreenSize().getHeight();
		int x = (int)kit.getScreenSize().getWidth();
		/* Sets the dimension of the internal frame */
		this.setSize(x - 10, y - 150);
		setVisible(true);
	}
	
	/**
	 * Return an instance of the class
	 * @return  ApplyInstanceRulesTool
	 * @uml.property  name="istance"
	 */
	/* Singleton */
	public static ApplyInstanceRulesTool getIstance(){
		/*Il controllo condizionale non  necessario:
		 * - il metodo deve ritornare l'internal-frame solo se questa  stata creata
		 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
		 * dal controllo dell'Application Plugin
		 * */
		return istance;
	}	
	
	public String toString(){
		return "ApplyInstanceRulesTool";
	}

	public void init(Map<String,Object> applicationObjects) {
		this.applicationObjs  = applicationObjects;
	}
	
	public Map<String, Object> getInfo() {
		return this.applicationObjs;
	}
	
	/**
	 * Sets the general layout
	 */
	private void setLayout() {
		// Setto il layout
		this.pnlGeneral.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
	}
	
	/**
	 * Creates and shows the window
	 */
	private void createShow() {
		
		// Creates menu
		this.menu = MenuApplyInstanceRule.getInstance(this,  this.pnlRules);
		this.setJMenuBar(this.menu);		
		// Creates tool bar
		this.tool = ToolBarApplyInstanceRule.getInstance( this.pnlRules); 
		this.getContentPane().add(this.tool, BorderLayout.PAGE_START);
		
		// Creates panel info
		this.pnlInfo.setLayout();
		this.pnlInfo.createShowPanel();		
		// Creates panel rules
		this.pnlRules.setLayout();
		this.pnlRules.createShowPanel();
		
		/*********************** Add to main panel***************************/
		
		// Component in (1,0)
		this.generalLayout.gridx = 1;
		this.generalLayout.gridy = 0;
		this.generalLayout.anchor = GridBagConstraints.EAST;
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.weightx = 1;
		this.pnlGeneral.add(this.pnlInfo, this.generalLayout);
		
		// Component in (0,1)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 1;
		this.generalLayout.gridwidth = 2;
		this.generalLayout.weighty = 20;
		this.generalLayout.weightx = 1;
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.pnlGeneral.add(this.pnlRules, this.generalLayout);
		
		// Add general panel to container
		this.getContentPane().add(new JScrollPane(this.pnlGeneral));
		
		this.setVisible(true);
	}

	/**
	 * @throws Exception 
	 * @throws ClassNotFoundException 
	 * 
	 */
	public void updateGuiData() throws Exception {
		String[] list = GeneralManagementFacade.getInstance().populateCmbScheme("superModel");
		this.pnlInfo.refreshSchemasList(list);
		list = GeneralManagementFacade.getInstance().populateCmbInstance();
		this.pnlInfo.refreshInstanceList(list);
	}
}