/*
 * Menu for apply instance rules 
 */
package org.modelgen.jpf.applyInstanceRules;

import org.modelgen.jpf.applyInstanceRules.apply.PanelRules;
import org.modelgen.jpf.applyInstanceRules.complex.ComplexRules;
import org.modelgen.jpf.applyInstanceRules.util.FactoryIcon;

import java.awt.Toolkit;
import java.awt.event.ActionListener;

import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

/**
 * @author kristina
 * Menu for apply instance rules
 */
class MenuApplyInstanceRule extends JMenuBar {
	private static final long serialVersionUID = 8105446230570226130L;
	// Singleton
	private static MenuApplyInstanceRule instance;
	// Internal frame
	private JInternalFrame main;
	// Listener
	private ActionListener lstAction;
	// References to pnlRules
	private PanelRules pnlRules;
	// Component of menu
	private JMenu file;
	protected JMenuItem open;
	protected JMenuItem save;
	protected JMenuItem execute;
	protected JMenuItem exit;
	
	/**
	 * Class constructor
	 * @param main The internal frame under this menu
	 * @param pnlCopy The references to the panel with copy function
	 * @param pnlRules The references to the panel with apply instance rules function
	 */
	private MenuApplyInstanceRule(JInternalFrame main,  
			PanelRules pnlRules) {   
		this.main = main;
		this.pnlRules = pnlRules;
		// Creates listener
		this.lstAction = new ActionListenerMenu(this,  pnlRules);
		// Creates menu and components
		this.file = new JMenu("File");
		this.open = new JMenuItem("Open");
		this.open.setIcon(FactoryIcon.createOpenIcon());
		this.open.setAccelerator(KeyStroke.getKeyStroke(
				'O', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.open.addActionListener(this.lstAction);
		this.save = new JMenuItem("Save");
		this.save.setIcon(FactoryIcon.createSaveIcon());
		this.save.setAccelerator(KeyStroke.getKeyStroke(
				'S', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.save.addActionListener(this.lstAction);
		this.execute = new JMenuItem("Execute");
		this.execute.setIcon(FactoryIcon.createDownIcon());
		this.execute.setAccelerator(KeyStroke.getKeyStroke(
				'E', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.execute.addActionListener(this.lstAction);
		this.exit = new JMenuItem("Exit");
		this.exit.setIcon(FactoryIcon.createCloseIcon());
		this.exit.setAccelerator(KeyStroke.getKeyStroke(
				'Q', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		// Add to menu
		this.file.addSeparator();
		this.file.add(this.open);
		this.file.add(this.save);
		this.file.add(this.execute);
		this.file.addSeparator();
		this.file.add(this.exit);
		// Add to menu bar
		this.add(this.file);
	}
	
	/**
	 * Return an instance of class
	 * @param main The internal frame under this menu
	 * @param pnlCopy The references to the panel with copy function
	 * @param pnlRules The references to the panel with apply instance rules function
	 * @return MenuApplyInstanceRule
	 */
	protected static MenuApplyInstanceRule getInstance(JInternalFrame main,  
			PanelRules pnlRules) { 
		/* Singleton */
		if(instance == null)
			instance = new MenuApplyInstanceRule(main,  pnlRules); 
		return instance;
	}
	
	/**
	 * Open the file chooser to save the complex ruels
	 */
	
	protected void openSave() {
		ComplexRules save = ComplexRules.getInstance(this.pnlRules);
		save.openSave();
	}
	
	/**
	 * Open the file chooser to load the complex ruels
	 */
	
	protected void openLoad() {
		ComplexRules save = ComplexRules.getInstance(this.pnlRules);
		save.openLoad();
	}
	
	/**
	 * Close this plugin
	 */
	protected void close() {
		this.main.doDefaultCloseAction();
	}
}