/*
 * This class implements a panel for insert general info to apply rules 
 */
package org.modelgen.jpf.applyInstanceRules.apply;

import org.modelgen.facade.GeneralManagementFacade;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @author kristina
 * This class implements a panel for insert general info to apply instance rules
 */
public class PanelInfo extends JPanel {
	private static final long serialVersionUID = -3293928973480210757L;
	// Default text for txtName
	protected static final String TXT_DEFAULT = "Enter a name (optional)";
	/* Singleton */
	private static PanelInfo instance;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Mouse lister
	private MouseListener lstMouse;
	// Key listener
	private KeyListener lstKey;
	// Components for gui
	protected JComboBox cmbInstance;
	protected JComboBox cmbScheme;
	protected JTextField txtTraslated;
	
	/**
	 * Class constructor
	 */
	private PanelInfo() {
		// Creates listener
		this.lstKey = new KeyListenerRules(this);
		this.lstMouse = new MouseListenerRules(this);
		
		try {
			this.cmbInstance = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbInstance()); 
		} catch (Exception e) {}
		try {
			this.cmbScheme = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbScheme("superModel"));  
		} catch (Exception e) {}
		
		this.txtTraslated = new JTextField(30);
		this.txtTraslated.addKeyListener(this.lstKey);
		this.txtTraslated.addMouseListener(this.lstMouse);
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelInfo
	 * @uml.property  name="instance"
	 */
	public static PanelInfo getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelInfo();
		return instance;
	}
	
	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}
	
	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlGeneralInfo = new JPanel();
		// Initialize gui elements
		this.initializePanel();		
		// Create panel general info
		pnlGeneralInfo.setLayout(new GridBagLayout());
		pnlGeneralInfo.setBorder(
				new javax.swing.border.TitledBorder(
				"General info for apply instance rules"));
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.HORIZONTAL;
		consGeneralInfo.anchor = GridBagConstraints.NORTH;
		consGeneralInfo.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 0;
		//consGeneralInfo.gridwidth = 2;
		consGeneralInfo.anchor = GridBagConstraints.EAST;
		pnlGeneralInfo.add(this.cmbInstance, consGeneralInfo);
		// Component in (1,0)
		consGeneralInfo.gridx = 1;
		consGeneralInfo.gridy = 0;
		consGeneralInfo.gridwidth = 2;
		consGeneralInfo.anchor = GridBagConstraints.WEST;
		pnlGeneralInfo.add(this.cmbScheme, consGeneralInfo);
		// Component in (0,1)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 1;
		consGeneralInfo.gridwidth = 4;
		pnlGeneralInfo.add(this.txtTraslated, consGeneralInfo);   
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlGeneralInfo, this.generalLayout);		
	}

	/**
	 * Initialize the panel
	 */
	private void initializePanel() {
		this.txtTraslated.setText(TXT_DEFAULT);
	}
	
	/**
	 * Return the selected index instance
	 * @return int
	 */
	protected int getInstanceSelectedIndex() {
		return this.cmbInstance.getSelectedIndex();
	}
	
	/**
	 * Return the selected index of the scheme
	 * @return int
	 */
	protected int getSchemeSelectedIndex() {
		return this.cmbScheme.getSelectedIndex();
	}
	
	/**
	 * Return the instance name
	 * @return String
	 */
	protected String getSelectedInstance() {
		return (String)this.cmbInstance.getSelectedItem();
	}
	
	/**
	 * Return the scheme name
	 * @return String
	 */
	protected String getSelectetSchema() {
		return (String)this.cmbScheme.getSelectedItem();
	}

	/**
	 * @param list
	 */
	public void refreshSchemasList(String[] list) {
		int items = this.cmbScheme.getItemCount();
		for (int i = items; i < list.length; i++) {
			this.cmbScheme.addItem(list[i]);
		}
	}

	/**
	 * @param list
	 */
	public void refreshInstanceList(String[] list) {
		int items = this.cmbInstance.getItemCount();
		for (int i = items; i < list.length; i++) {
			this.cmbInstance.addItem(list[i]);
		}
	}
}