/*
 * This class implements a panel for applying rules 
 */
package org.modelgen.jpf.applyInstanceRules.apply;

import org.modelgen.commons.beans.instance.Instance;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.applyInstanceRules.ApplyInstanceRulesTool;
import org.modelgen.jpf.applyInstanceRules.load.RulesLoader;
import org.modelgen.jpf.applyInstanceRules.util.FactoryIcon;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * @author kristina
 * This class implements a panel for applying rules
 */
public class PanelRules extends JPanel {
	private static final long serialVersionUID = 8663841166780574529L;
	// Singleton
	private static PanelRules instance;
	// Facade
	private GeneralManagementFacade facade;
	// References to pnlInfo
	private PanelInfo pnlInfo;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Action listener
	private ActionListener lstAction;
	// Mouse listener
	private MouseListener lstMouse;
	// Components for gui
	private JLabel lblRules;
	// Components for gui
	private JLabel lblUserRules;
	protected JList lstRules;
	protected JList lstUserRules;
	protected JButton btnUp;
	protected JButton btnDown;
	protected JButton btnAdd;
	protected JButton btnAddAll;
	protected JButton btnExecute;
	protected JButton btnRemove;
	protected JButton btnRemoveAll;

	/**
	 * Class constructor
	 * @param pnlInfo 
	 */
	private PanelRules(PanelInfo pnlInfo) {
		super();
		this.facade = GeneralManagementFacade.getInstance();

		this.pnlInfo = pnlInfo;
		// Listener
		this.lstAction = new ActionListenerRules(this);
		this.lstMouse = new MouseListenerRules(this);
		// Creates components
		this.lblRules = new JLabel("Select the translation rules to apply");
		this.lblUserRules = new JLabel("Confirm the rules to apply");
		this.lstRules = new JList();
		this.lstRules.addMouseListener(this.lstMouse);
		this.lstUserRules = new JList();
		this.lstUserRules.addMouseListener(this.lstMouse);
		this.btnAdd = new JButton(FactoryIcon.createAddIcon());
		this.btnAdd.addActionListener(this.lstAction);
		this.btnAdd.setToolTipText("Add rule(s)");
		this.btnAddAll = new JButton(FactoryIcon.createAddAllIcon());
		this.btnAddAll.addActionListener(this.lstAction);
		this.btnAddAll.setToolTipText("Add all rules");
		this.btnRemove = new JButton(FactoryIcon.createRemoveIcon());
		this.btnRemove.addActionListener(this.lstAction);
		this.btnRemove.setToolTipText("Remove rule(s)");
		this.btnRemoveAll = new JButton(FactoryIcon.createRemoveAllIcon());
		this.btnRemoveAll.addActionListener(this.lstAction);
		this.btnRemoveAll.setToolTipText("Remove all rules");
		this.btnExecute = new JButton("Execute");
		this.btnExecute.setToolTipText("Execute");
		this.btnExecute.addActionListener(this.lstAction);
		this.btnUp = new JButton(FactoryIcon.createUpIcon());
		this.btnUp.addActionListener(this.lstAction);
		this.btnDown = new JButton(FactoryIcon.createDownIcon());
		this.btnDown.addActionListener(this.lstAction);
	}

	/**
	 * Return an instance of the class
	 * @param pnlInfo 
	 * @return PanelRules
	 */
	public static PanelRules getInstance(PanelInfo pnlInfo) {
		/* Singleton */
		if(instance == null)
			instance = new PanelRules(pnlInfo);
		return instance;
	}

	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}

	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlAddRules = new JPanel();
		JPanel pnlBtnRules, pnlBtnUserRules;
		// Initialize gui elements
		this.initializePanel();		   
		// Panel argument
		pnlAddRules.setLayout(new GridBagLayout());
		pnlAddRules.setBorder(
				new javax.swing.border.TitledBorder(
				"Add rule(s) to apply"));
		
        // Set the parameter for the GrigBagConstrains
		GridBagConstraints consAddRules = new GridBagConstraints();
		consAddRules.fill = GridBagConstraints.NONE;
		consAddRules.anchor = GridBagConstraints.NORTH;
		consAddRules.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consAddRules.gridx = 0;
		consAddRules.gridy = 0;
		consAddRules.anchor = GridBagConstraints.EAST;
		pnlAddRules.add(this.lblRules, consAddRules);
		// Component in (0,1)
		consAddRules.gridx = 0;
		consAddRules.gridy = 1;
		consAddRules.anchor = GridBagConstraints.CENTER;
		consAddRules.fill = GridBagConstraints.BOTH;
		consAddRules.gridheight = 4;
		consAddRules.gridwidth = 1;
		JScrollPane scrRule = new JScrollPane(this.lstRules,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrRule.setPreferredSize(new Dimension(300, 300));
		pnlAddRules.add(scrRule, consAddRules);
		// Panel btn rules
		pnlBtnRules = new JPanel();
		pnlBtnRules.setLayout(new GridBagLayout());
		GridBagConstraints consBtnRules = new GridBagConstraints();
		// Component in (0,0)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 0;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnAdd, consBtnRules);
		// Component in (0,1)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 1;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnAddAll, consBtnRules);
		// Component in (0,2)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 2;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnRemove, consBtnRules);
		// Component in (0,3)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 3;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnRemoveAll, consBtnRules);
		// Component in (1,1)
		consAddRules.gridx = 1;
		consAddRules.gridy = 1;
		pnlAddRules.add(pnlBtnRules, consAddRules);
		// Component in (2,0)
		consAddRules.gridx = 2;
		consAddRules.gridy = 0;
		consAddRules.anchor = GridBagConstraints.WEST;
		pnlAddRules.add(this.lblUserRules, consAddRules);
		// Component in (2,1)
		consAddRules.gridx = 2;
		consAddRules.gridy = 1;
		consAddRules.gridheight = 4;
		consAddRules.gridwidth = 1;
		consAddRules.fill = GridBagConstraints.BOTH;
		consAddRules.anchor = GridBagConstraints.CENTER;
		JScrollPane scrUserRule = new JScrollPane(this.lstUserRules,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrUserRule.setPreferredSize(new Dimension(300, 300));
		pnlAddRules.add(scrUserRule, consAddRules);
		// Component in (3,2)
		consAddRules.gridx = 3;
		consAddRules.gridy = 2;
		consAddRules.fill = GridBagConstraints.NONE;
		consAddRules.anchor = GridBagConstraints.NORTH;
		pnlAddRules.add(this.btnUp, consAddRules);
		// Panel button for swicth user rules
		pnlBtnUserRules = new JPanel();
		pnlBtnUserRules.setLayout(new GridBagLayout());
		GridBagConstraints consBtnUserRules = new GridBagConstraints();
		// Component in (0,0)
		consBtnUserRules.gridx = 0;
		consBtnUserRules.gridy = 0;
		consBtnUserRules.anchor = GridBagConstraints.CENTER;
		pnlBtnUserRules.add(this.btnUp, consBtnUserRules);
		// Component in (0,1)
		consBtnUserRules.gridx = 0;
		consBtnUserRules.gridy = 1;
		consBtnUserRules.anchor = GridBagConstraints.CENTER;
		pnlBtnUserRules.add(this.btnDown, consBtnUserRules);
		// Component in (3,0)
		consAddRules.gridx = 3;
		consAddRules.gridy = 1;
		consAddRules.fill = GridBagConstraints.NONE;
		consAddRules.anchor = GridBagConstraints.CENTER;
		pnlAddRules.add(pnlBtnUserRules, consAddRules);
		// Component in (1,5)
		consAddRules.gridx = 1;
		consAddRules.gridy = 5;
		consAddRules.anchor = GridBagConstraints.CENTER;
		pnlAddRules.add(this.btnExecute, consAddRules);
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlAddRules, this.generalLayout);		
	}

	
	
	/**
	 * Initialize the panel
	 */
	
	private void initializePanel() {
		// Initialize rules
		this.initializeRules();
	}
    
	/**
	 * Initialize rules' list
	 */
	
	private void initializeRules() {
		// Take the rules
		Collection<String> rules = null;
		rules = RulesLoader.load();
		// Creates the default list model
		DefaultListModel listModel =  new DefaultListModel();
		DefaultListModel listModelUser = new DefaultListModel();
		// Iterator
		Iterator<String> it = rules.iterator();
		// Add to list model 
		while(it.hasNext()) {
			String rule = it.next();
			listModel.addElement(rule);
		}
		this.lstRules.setModel(listModel);
		this.lstUserRules.setModel(listModelUser);
	}
	
	/**
	 * Re-order rules' list after the user move the rule(s) from his list to rules' 
	 * list
	 */
	
	private void orderList() {
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		DefaultListModel model = (DefaultListModel)this.lstRules.getModel();
		model.removeAllElements();
		// Take the collection of the rules
		Collection<?> rules = null;
		rules = RulesLoader.load();
		Enumeration<?> elementsInUser = modelUser.elements();
		// Remove the rules in lstUserRules from the collection of rule 
		while(elementsInUser.hasMoreElements()) {
			Object rule = elementsInUser.nextElement();
			rules.remove(rule);
		}
		// Iterator 
		Iterator<?> it = rules.iterator();
		while(it.hasNext()) {
			// Update the model of lstRule with the new rules
			model.addElement(it.next());
		}
		this.lstRules.setModel(model);
	}
	
	/**
	 * Move from list rules to user list rules
	 */
	
	protected void moveToUserList() {
		// Selected objects
		Object[] selectedRules = this.lstRules.getSelectedValues();
		for(int i = 0; i < selectedRules.length; i++) {
			// Must be a DefaultListModel
			((DefaultListModel)		// REMOVE
					this.lstRules.getModel()).removeElement(selectedRules[i]);
			((DefaultListModel)		// ADD
					this.lstUserRules.getModel()).addElement(selectedRules[i]);
		}
		this.updateUI();
	}

	/**
	 * Move from user list rules to list rules
	 */
	
	protected void moveToList() {
		// Take a selected rules into list
		Object[] selectedRules = this.lstUserRules.getSelectedValues();
		for(int i = 0; i < selectedRules.length; i++) {
			// Must be a DefaultListModel
			((DefaultListModel)		// ADD
					this.lstRules.getModel()).addElement(selectedRules[i]);
			((DefaultListModel)		// REMOVE
					this.lstUserRules.getModel()).removeElement(selectedRules[i]);
		}
		this.orderList();
		this.updateUI();
	}
	
	/**
	 * Move from list rules to user list rules
	 */
	
	protected void addAllRules() {
		// Take a size of the list
		DefaultListModel model = (DefaultListModel)this.lstRules.getModel();
		int capacity = model.getSize();
		String currentElement = null;
		// Copy all element in user list
		for(int i = 0; i < capacity; i++) {
			currentElement = (String)model.get(i);
			((DefaultListModel)		// ADD
					this.lstUserRules.getModel()).addElement(currentElement);
		}
		/* If i put model.remove(model.elementAt(i)) instruction in for cicle, the 
		 * size of model list associated at lstRules always change and so i don't take 
		 * the elements in correct order, 0,1,2,...,n.It's mean that i take first 
		 * element, then increase i and the size of model is changed (it's n-i element); 
		 * so the second element is in reality the third element. Put removeAllElemets
		 * instruction after for cicle and this problem is resolved */
		
		model.removeAllElements();
		this.updateUI();
	}
	
	/**
	 * Move from user list rules to list rules
	 */
	
	protected void removeAllRules() {
		// Take a size of the list
		DefaultListModel model = (DefaultListModel)this.lstUserRules.getModel();
		int capacity = model.getSize();
		String currentElement = null;
		// Copy all element in user list
		for(int i = 0; i < capacity; i++) {
			currentElement = (String)model.elementAt(i);
			((DefaultListModel)		// ADD
					this.lstRules.getModel()).addElement(currentElement);
		}
		model.removeAllElements();
		this.orderList();
		this.updateUI();
	}
	
	/**
	 * Move up a selected rele
	 */
	
	protected void moveUp() {
		DefaultListModel model = (DefaultListModel)this.lstUserRules.getModel();
		// Index of element to move 
		int indexToMove = this.lstUserRules.getSelectedIndex();
		// Index of previous element
		int indexPrevious = this.lstUserRules.getSelectedIndex() - 1;
		if(indexToMove > 0) {
			// The element to move
			String toMove = (String)model.remove(indexToMove);
			// The previous element of the element to move 
			String previous = (String)model.remove(indexPrevious);
			model.add(indexPrevious, toMove);
			model.add(indexToMove, previous);
			this.lstUserRules.setModel(model);
			this.lstUserRules.setSelectedIndex(indexPrevious);
		} 
	}
	
	/**
	 * Move down a selected rele
	 */
	
	protected void moveDown() {
		DefaultListModel model = (DefaultListModel)this.lstUserRules.getModel();
		// Index of element to move 
		int indexToMove = this.lstUserRules.getSelectedIndex();
		// Index of next element 
		int indexNext = this.lstUserRules.getSelectedIndex() + 1;
		if(indexNext < model.getSize()) {
			// The element to move 
			String toMove = (String)model.getElementAt(indexToMove);
			//The next element of the element to move 
			String next = (String)model.getElementAt(indexNext);
			model.removeElement(toMove);
			model.add(indexNext, toMove);
			model.removeElement(next);
			model.add(indexToMove, next);
			this.lstUserRules.setModel(model);
			this.lstUserRules.setSelectedIndex(indexNext);
		} 
	}
	
	/**
	 * Remove the selection from the current selected object in the list
	 */
	
	protected void removeSelection() {
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		this.lstUserRules.setModel(modelUser);
	}
	
	/**
	 * Return the enumeration of the user selected rules
	 * @return Enumeration<?>
	 */
	@SuppressWarnings("unchecked")
	
	public Enumeration<String> getAllUserRules() {
		DefaultListModel model = (DefaultListModel)this.lstUserRules.getModel();
	
		return (Enumeration<String>)model.elements();
	}
	
	/**
	 * Apply the specify rules
	 */
	
	public void applyRules() {
		int selectedSchemeIndex = this.pnlInfo.getSchemeSelectedIndex();
		int selectedInstanceIndex = this.pnlInfo.getInstanceSelectedIndex();
		if(selectedSchemeIndex==0 && selectedInstanceIndex==0)
			JOptionPane.showMessageDialog(this, "Select a scheme and an instance, please",
					"Apply Instance Rule", JOptionPane.ERROR_MESSAGE);
		else if(selectedSchemeIndex==0)
			JOptionPane.showMessageDialog(this, "Select a scheme, please",
					"Apply Instance Rule", JOptionPane.ERROR_MESSAGE);
		else if(selectedInstanceIndex==0)
			JOptionPane.showMessageDialog(this, "Select an instance, please",
					"Apply Instance Rule", JOptionPane.ERROR_MESSAGE);
		else {
			int targetSchemaOID = -1, sourceInstanceOID = -1;
			// Scheme name and OID 
			String selectedSchema = this.pnlInfo.getSelectetSchema();
			targetSchemaOID = Integer.parseInt(
					selectedSchema.substring(0,selectedSchema.indexOf(':')));
			// Instance name and OID 
			String selectedInstance = this.pnlInfo.getSelectedInstance();
			sourceInstanceOID = Integer.parseInt(
					selectedInstance.substring(0,selectedInstance.indexOf(':')));
//			System.out.println("INSTANCE OID " + sourceInstanceOID);
			Instance instance = null;
			try {
				instance = this.facade.getInstanceByOID(sourceInstanceOID);
				sourceInstanceOID = instance.getOID();
				int sourceSchemaOID = instance.getSOID();
				// Takes a user rules 
				DefaultListModel listModel = 
					(DefaultListModel)this.lstUserRules.getModel();
				Object[] ruless = this.lstUserRules.getSelectedValues();
				// There aren't selected rules, takes all rules in the list 
				if(ruless.length == 0)
					/* Take all rules */
					ruless = listModel.toArray();
				String[] rules = new String[ruless.length];
				int k = 0;
				// Put into array of string 
				while(k<ruless.length) {
					rules[k] = (String)ruless[k];
					k++;
				}
				// Check the name, because it's optional 
				// Take the content of txt field
				String name = this.pnlInfo.txtTraslated.getText();
				if(name.equals("") || name.equals(PanelInfo.TXT_DEFAULT))
					// If don't insert name or name is equals to defautl text 
					name = "";
				// Execute the rules 
				try {
//					System.out.println("Apply Instance Rules Arguments:");
//					System.out.println("Name " + name + " sourceSchema " + sourceSchemaOID
//							+ "targetSchema " + targetSchemaOID + " sourceInstance " + sourceInstanceOID);
					this.facade.applyInstanceRules(rules, name, sourceSchemaOID, targetSchemaOID, sourceInstanceOID);
					ApplyInstanceRulesTool.getIstance().updateGuiData();
					JOptionPane.showMessageDialog(this, "Translated successfully", 
							"Apply Instance Rules", JOptionPane.INFORMATION_MESSAGE);
				} catch (FileNotFoundException e) {
					JOptionPane.showMessageDialog(this, e.getMessage(), 
							"Apply Instance Rules", JOptionPane.INFORMATION_MESSAGE);
				} catch (IOException e) {
					JOptionPane.showMessageDialog(this, e.getMessage(), 
							"Apply Instance Rules", JOptionPane.INFORMATION_MESSAGE);
				} catch (Exception e) {
					JOptionPane.showMessageDialog(this, e.getMessage(), 
							"Apply Instance Rules", JOptionPane.INFORMATION_MESSAGE);
				}
			} catch (PersistenceException e) {
//				 TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
	}
     
	/**
	 * Set the opened rules from file into user rules list
	 * @param rules The list of rules opened
	 */
	
	public void setOpenedRules(List<?> rules) {
		// Take the models
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		modelUser.removeAllElements();
		Iterator<?> itRules = rules.iterator();
		// Add to modelRules 
		while(itRules.hasNext()) {
			Object rule = itRules.next();
			modelUser.addElement(rule);
		}
		this.orderList();
	}
	
}