/*
 * This class implements a file chooser to save a complex rules 
 */
package org.modelgen.jpf.applyInstanceRules.complex;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.jpf.applyInstanceRules.apply.PanelRules;
import org.modelgen.jpf.applyInstanceRules.util.FileFilter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JPanel;

/**
 * @author kristina
 * This class implements a file chooser to save a complex rules
 */
public class ComplexRules extends JFileChooser {
	private static final long serialVersionUID = 7899800547347825088L;
	// Instance for Singleton
	private static ComplexRules instance;
	// The fileter
	private FileFilter filter;
	// PanelRules
	private PanelRules pnlRules;
	
	/**
	 * Class constructor
	 * @param pnlRules The panel that contains the rules to save
	 */
	private ComplexRules(PanelRules pnlRules) {
		this.pnlRules = pnlRules;
		// Take the path
		String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
		String complexRulesPath = ModelgenConfigurationManager.getIstance().getProperty("complexrules", "", "paths");
		File complexRulesDir = new File(rootnode + complexRulesPath);
		// Set the path
		this.setCurrentDirectory(complexRulesDir);
		// Take the filter
		this.filter = FileFilter.getInstance();
		this.filter.setExstension(FileFilter.CMP, FileFilter.DESCRIPTION_CMP);
		this.setFileFilter(this.filter);
	}
	
	/**
	 * Return an instance of the class
	 * @param pnlRules The panel that contains the rules to save
	 * @return SaveComplexRules
	 */
	public static ComplexRules getInstance(PanelRules pnlRules) {
		if(instance == null)
			instance = new ComplexRules(pnlRules);
		return instance;
	}
	
	/**
	 * Open the file chooser
	 */
	public void openSave() {
		// Open the file chooser to save
		int returnValue = this.showSaveDialog(new JPanel());
		if(returnValue == JFileChooser.APPROVE_OPTION)
			this.save(this.getSelectedFile().getPath());
	}
	
	/**
	 * Save into file
	 * @param fileName The file that contains the complex rules
	 */
	private void save(String fileName) {
		// Take all rules
		Enumeration<String> rules = this.pnlRules.getAllUserRules();
		PrintWriter fout; // To write
		FileWriter file = null;	
		try {
			file = new FileWriter(fileName + FileFilter.CMP, true);
		} catch (IOException e) {}
		fout = new PrintWriter(file);
		// If have another rules, save on the file
		
		while(rules.hasMoreElements()) {
			fout.println(rules.nextElement() + ";");
		}
		
		fout.close();
	}
	
	/**
	 * Open file chooser to load a complex rules
	 */
	public void openLoad() {
		int returnValue = this.showOpenDialog(new JPanel());
		if(returnValue == JFileChooser.APPROVE_OPTION)
			this.open(this.getSelectedFile());
	}
	
	/**
	 * Open the file
	 * @param file2open The file to open to import a complex rule
	 */
	@SuppressWarnings("unchecked")
	private void open(File file2open) {
		RandomAccessFile raf = null;
		try {
			raf = new RandomAccessFile(file2open, "r");
			byte[] byteStringa = null;
			List rules = new ArrayList();
			/* Read the macro rules from file */
			byteStringa = new byte[(int)raf.length()];
			for (int i = 0; i < raf.length(); i++) {
				int valoreLetto = raf.read();
				byteStringa[i] = (byte)valoreLetto;
			}
			String complexRules = new String(byteStringa);
			/* Execute the parser of the complexRules to create one string for each macro
			 * rule */
			int index = 0, lastIndex = 0;
			for(index = 0; index < complexRules.length(); index++) {
				/* If the character is ';', then this is a first macro rules in the 
				 * complex rule file */
				if(complexRules.charAt(index) == ';') {
					// Creates a new string that contains the first macro rule
					String macroRule = complexRules.substring(lastIndex, index);
					// Add to the list of complex rules
					rules.add(macroRule.trim());
					// Update last index
					lastIndex = index + 1;
				}	
			}
			this.pnlRules.setOpenedRules(rules);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}