/*
 * This class implements the main wondow for ApplyRules plugin 
 */
package org.modelgen.jpf.applyRules;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.applyRules.apply.PanelInfo;
import org.modelgen.jpf.applyRules.apply.PanelInstance;
import org.modelgen.jpf.applyRules.apply.PanelRules;
import org.modelgen.jpf.applyRules.copy.PanelCopy;
import org.modelgen.jpf.core.commons.Tool;

/**
 * @author  gabriele  This class implements the main window for ApplyRules plugin
 */
public class ApplyRulesTool extends JInternalFrame implements Tool {
	private static final long serialVersionUID = 4933945007817074751L;
	/* Singleton */
	private static ApplyRulesTool istance;
	/* Mappa condivisa */
	/**
	 * @uml.property  name="applicationObjs"
	 * @uml.associationEnd  qualifier="key:java.lang.Object java.lang.Object"
	 */
	private Map<String,Object> applicationObjs;
	// Menu
	private MenuApplyRule menu;
	// ToolBar
	private ToolBarApplyRule tool;
	// General panel of the window
	private JPanel pnlGeneral;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Panel copy
	private PanelCopy pnlCopy;
	// Panel for general info to apply rules
	private PanelInfo pnlInfo;
	// Panel rules
	private PanelRules pnlRules;
//	Panel Instance
	private PanelInstance pnlInstance;
	
	
	/**
	 * Class constructor
	 * @throws PersistenceException
	 */
	public ApplyRulesTool() throws PersistenceException{
		super("ApplyRulesTool",true,true,true,true);
		checkConnection();
		initProperties();
		
		this.pnlGeneral = null;
		this.pnlCopy = null;
		this.pnlInfo = null;
		this.pnlRules = null;
		this.pnlInstance = null;
		
		// Creates general panel
		this.pnlGeneral = new JPanel();
		this.setLayout();
		// Creates panel copy
		this.pnlCopy = PanelCopy.getInstance();
		// Creates panel info
		this.pnlInfo = PanelInfo.getInstance();
//		Creates panel instance
		this.pnlInstance = PanelInstance.getInstance();
		// Creates panel rules
		this.pnlRules = PanelRules.getInstance(this.pnlInfo, this.pnlInstance);
		
		// Creates and shows the component
		this.createShow();
		this.pack();
		/*Consente di applicare il pattern singleton nonostante la classe 
		 * venga istanziata dal CorePlugin*/
		istance = this;
	}

	/**
	 * Check the connection
	 * @throws PersistenceException
	 */
	private void checkConnection() throws PersistenceException{	
		/*Inizializza il PH condiviso e se il server non � avviato provoca 
		 *un eccezione intercettata dalla ToolActivationAction
		 * */
//		DefineModelFacade.getInstance().initPesistenceHandler();
		GeneralManagementFacade.getInstance().initPesistenceHandler();
	}
	
	/**
	 * Initialize the properties of the windows
	 */
	private void initProperties() {
		// L'icona
		ImageIcon imageIcon = ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png");
		setFrameIcon(imageIcon);
		// Alla chiusura
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);			
		Toolkit kit = Toolkit.getDefaultToolkit();
		/* Take x and y coordinates of the screnn */
		int y = (int)kit.getScreenSize().getHeight();
		int x = (int)kit.getScreenSize().getWidth();
		/* Sets the dimension of the internal frame */
		this.setSize(x - 10, y - 150);
		setVisible(true);
	}
	
	/**
	 * Return an instance of the class
	 * @return  ApplyRulesTool
	 * @uml.property  name="istance"
	 */
	/* Singleton */
	public static ApplyRulesTool getIstance(){
		/*Il controllo condizionale non � necessario:
		 * - il metodo deve ritornare l'internal-frame solo se questa � stata creata
		 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
		 * dal controllo dell'Application Plugin
		 * */
		return istance;
	}	
	
	public String toString(){
		return "ApplyRulesTool";
	}

	public void init(Map<String,Object> applicationObjects) {
		this.applicationObjs  = applicationObjects;
	}
	
	public Map<String, Object> getInfo() {
		return this.applicationObjs;
	}
	
	/**
	 * Sets the general layout
	 */
	private void setLayout() {
		// Setto il layout
		this.pnlGeneral.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
	}
	
	/**
	 * Creates and shows the window
	 */
	private void createShow() {
		this.pnlCopy = PanelCopy.getInstance();
		this.pnlInfo = PanelInfo.getInstance();
		this.pnlInstance = PanelInstance.getInstance();
		this.pnlRules = PanelRules.getInstance(this.pnlInfo, this.pnlInstance);
		
		// Creates menu
		this.menu = MenuApplyRule.getInstance(this, this.pnlCopy, this.pnlRules);
		this.setJMenuBar(this.menu);
		// Creates tool bar
		this.tool = ToolBarApplyRule.getInstance(this.pnlCopy, this.pnlRules);
		this.getContentPane().add(this.tool, BorderLayout.PAGE_START);
		// Creates panel copy
		this.pnlCopy.setLayout();
		this.pnlCopy.createShow();
		// Creates panel info
		this.pnlInfo.setLayout();
		this.pnlInfo.createShowPanel();
		// Creates panel rules
		this.pnlRules.setLayout();
		this.pnlRules.createShowPanel();
//		 Creates panel instance
		this.pnlInstance.setLayout();
		this.pnlInstance.createShowPanel();
		// Add to main panel
		// Component in (0,0)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.generalLayout.weighty = 1;
		this.generalLayout.fill = GridBagConstraints.BOTH;
		this.generalLayout.anchor = GridBagConstraints.CENTER;
		this.pnlGeneral.add(this.pnlCopy, this.generalLayout);
		// Component in (0,1)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 1;
		this.generalLayout.anchor = GridBagConstraints.CENTER;
		this.pnlGeneral.add(this.pnlInfo, this.generalLayout);
		// Component in (0,3)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 3;
		this.generalLayout.gridwidth = 2;
		//this.generalLayout.weighty = 20;
		this.pnlGeneral.add(this.pnlRules, this.generalLayout);
//		 Component in (0,2)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 2;
		this.generalLayout.anchor = GridBagConstraints.CENTER;
		this.pnlGeneral.add(this.pnlInstance, this.generalLayout);
		
		// Add general panel to container
		this.getContentPane().add(new JScrollPane(this.pnlGeneral));
		this.setVisible(true);
	}

	/**
	 * @throws Exception 
	 * @throws ClassNotFoundException 
	 * 
	 */
	public void updateGuiData() throws Exception {
		String[] models = GeneralManagementFacade.getInstance().populateCmbModel();
		this.pnlInfo.refreshModelList(models,0);
		
		String[] schemas = GeneralManagementFacade.getInstance().populateCmbScheme("model");
		this.pnlCopy.refreshModelSchemasList(schemas);
		
		schemas = GeneralManagementFacade.getInstance().populateCmbScheme("SuperModel");
		this.pnlCopy.refreshSuperModelSchemasList(schemas);
		this.pnlInfo.refreshSuperModelSchemasList(schemas,0);
		
		String[] instances = GeneralManagementFacade.getInstance().populateCmbInstance();
		
		/* refreshes every instance */
		for(int i=0;i<this.pnlInfo.getSchemesNumber();i++)
			this.pnlInstance.refreshInstancesList(instances,i);
		
	}
}