/*
 * This class implements a panel for insert general info to apply rules 
 */
package org.modelgen.jpf.applyRules.apply;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.util.*;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.modelgen.facade.GeneralManagementFacade;

/**
 * @author  gabriele  This class implements a panel for insert general info to apply rules
 */
public class PanelInfo extends JPanel {
	public static final int SOURCE_SCHEMES_NUMBER = 1; // number of source schemes created by default 
	public static final int SOURCE_SCHEMES_LIMIT = 4; // the maximum for the source schemes
	private static final long serialVersionUID = -3293928973480210757L;
	// Default text for txtName
	protected static final String TXT_DEFAULT = "Enter a name (optional)";
	/* Singleton */
	private static PanelInfo instance;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Mouse lister
	private MouseListener lstMouse;
	private JPanel pnlGeneralInfo; // panel for source schemes
	// Key listener
	private KeyListener lstKey;
	// Components for gui
	private JButton addParameterButton; // to add a source scheme
	private PanelInstance pnli; // instance panel to be updated when a new source scheme is inserted
	
	
	
	public List<JComboBox> cmbSupermodelList;
	// Components for gui
	public List<JComboBox> cmbModelList;
	protected JTextField txtTraslated;
	
	/**
	 * Class constructor
	 */
	private PanelInfo() {
		// Creates listener
		this.lstKey = new KeyListenerPanelInfo(this);
		this.lstMouse = new MouseListenerPanelInfo(this);
		
		/* parameters list */
		/* list of the source schemes */
		cmbSupermodelList = new LinkedList<JComboBox>();
		cmbModelList = new LinkedList<JComboBox>();
		
		
		
		this.txtTraslated = new JTextField(30);
		this.txtTraslated.addKeyListener(this.lstKey);
		this.txtTraslated.addMouseListener(this.lstMouse);
		this.addParameterButton = new JButton("Add another source scheme");
		addParameterButton.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						/* if it goes above the max do nothing */
						if(getSchemesNumber()<PanelInfo.SOURCE_SCHEMES_LIMIT) {	
							/* adds a source scheme */
							addNewSourceScheme();
							/* and a source instance */
							pnli.addSourceScheme();
						}
						
						
					}
				});
	}
	
	/**
	 * sets the current instance panel
	 * @param pnli
	 */
	public void setInstancePanel(PanelInstance pnli) {
		this.pnli = pnli;
	}

	/**
	 * adds a source scheme
	 */
	private void addSourceScheme() {
		try {
			JComboBox cmbModel = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbModel());
			cmbModelList.add(cmbModel);
		} catch (Exception e) {}
		try {			
			JComboBox cmbSupermodel = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbScheme("superModel"));
			cmbSupermodelList.add(cmbSupermodel);
			
		} catch (Exception e) {}
		
		
		
		
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelInfo
	 * @uml.property  name="instance"
	 */
	public static PanelInfo getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelInfo();
		return instance;
	}
	
	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}
	
	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		
		pnlGeneralInfo = new JPanel();
		// Initialize gui elements
		this.initializePanel();		
		// Create panel general info
		pnlGeneralInfo.setLayout(new GridBagLayout());
		pnlGeneralInfo.setBorder(
				new javax.swing.border.TitledBorder(
				"General info for apply rules"));
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.HORIZONTAL;
		consGeneralInfo.anchor = GridBagConstraints.NORTH;
		consGeneralInfo.insets = new Insets(5,5,5,5);	
				
		/* adds the components for the list */
		
		int j=0;
	
		// text field
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 0;
		consGeneralInfo.gridwidth = 2;
		pnlGeneralInfo.add(this.txtTraslated, consGeneralInfo);
		
		j++;
		
		// add parameter button
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = j;
		consGeneralInfo.gridwidth = 2;
		pnlGeneralInfo.add(this.addParameterButton, consGeneralInfo);
		
		j++;
		
	
		/* default source schemes */
		for(int i =0; i<SOURCE_SCHEMES_NUMBER ;i++) {
			addNewSourceScheme();
			
			
		}
						
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlGeneralInfo, this.generalLayout);
		
		
		
	}
	
	/**
	 * adds a source scheme as reaction to button pression
	 *
	 */
	public void addNewSourceScheme() {	
		
		
		/* creates JCombo */
		addSourceScheme();
		/* adds JCombo */
		
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.NONE;
		consGeneralInfo.anchor = GridBagConstraints.NORTH;
		consGeneralInfo.insets = new Insets(5,5,5,5);	
				
		// last jcb
		JComboBox jcb = cmbModelList.get(cmbModelList.size()-1);
		
		int j = cmbModelList.size()+1;
		
		/* adds the component */
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = j;
		consGeneralInfo.anchor = GridBagConstraints.WEST;
		pnlGeneralInfo.add(jcb,consGeneralInfo);
		consGeneralInfo.anchor = GridBagConstraints.WEST;
		consGeneralInfo.gridx = 1;
		pnlGeneralInfo.add(this.cmbSupermodelList.get(this.cmbModelList.indexOf(jcb)),consGeneralInfo);
		
		
		/* repaints the frame */
		this.setVisible(false);
		this.setVisible(true);
		
		
	}

	/**
	 * Initialize the panel
	 */
	private void initializePanel() {
		this.txtTraslated.setText(TXT_DEFAULT);
	}
	
	/**
	 * Return the selected index model
	 * @param modelNumber the number if the required model
	 * @return int
	 */
	protected int getModelSelectedIndex(int modelNumber) {
		return this.cmbModelList.get(modelNumber).getSelectedIndex();
	}
	
	/**
	 * Return thie selected index of the scheme
	 * @return int
	 */
	protected int getSchemeSelectedIndex(int modelNumber) {
		return this.cmbSupermodelList.get(modelNumber).getSelectedIndex();
	}
	

	public int getSchemesNumber() {
		return this.cmbModelList.size();
	}
	
	/**
	 * Return the model name
	 * @return String
	 */
	protected String getModelName(int modelNumber) {
		String modelNameWithShortName = (String)this.cmbModelList.get(modelNumber).getSelectedItem();
		int index = modelNameWithShortName.indexOf("(")-1;
		return modelNameWithShortName.substring(0,index);
	}
	
	/**
	 * Return the scheme name
	 * @return String
	 */
	protected String getSchemeName(int modelNumber) {
		return (String)this.cmbSupermodelList.get(modelNumber).getSelectedItem();
	}

	
	/**
	 * @param schemas
	 */
	public void refreshSuperModelSchemasList(String[] schemas, int modelNumber) {
		int items = this.cmbSupermodelList.get(modelNumber).getItemCount();
		for (int i = items; i < schemas.length; i++) {
			this.cmbSupermodelList.get(modelNumber).addItem(schemas[i]);
//			System.out.println("Added " + schemas[i] + " pnlInfo-SM");
		}
	}

	/**
	 * @param models
	 */
	public void refreshModelList(String[] models, int modelNumber) {
		int items = this.cmbModelList.get(modelNumber).getItemCount();
		for (int i = items; i < models.length; i++) {
			this.cmbModelList.get(modelNumber).addItem(models[i]);
		}
	}
}