/*
 * This class implements a panel for insert general info to apply rules 
 */
package org.modelgen.jpf.applyRules.apply;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.modelgen.facade.GeneralManagementFacade;

/**
 * @author  gabriele  This class implements a panel for insert general info to apply rules
 */
public class PanelInstance extends JPanel {
	private static final long serialVersionUID = -3293928973480210757L;
	// Default text for txtName
	protected static final String TXT_DEFAULT = "Enter a name (optional)";
	/* Singleton */
	private static PanelInstance instance;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Mouse lister
	private MouseListener lstMouse;
	// Key listener
	private KeyListener lstKey;
	// Components for gui
	private JPanel pnlGeneralInfo;
	// if this panel is enabled or not
	private boolean isEnabled = false;
	
	protected List<JComboBox> cmbInstanceList;
	protected List<JTextField> txtInstanceTranslatedList;
	

	
	/**
	 * Class constructor
	 */
	private PanelInstance() {
		
		cmbInstanceList = new LinkedList<JComboBox>();
		txtInstanceTranslatedList = new LinkedList<JTextField>();
		
		pnlGeneralInfo = new JPanel();
		
		// Creates listener
		this.lstKey = new KeyListenerPanelInstance(this);
		this.lstMouse = new MouseListenerPanelInstance(this);
		
	
	}
	

	public boolean getIsEnabled() {
		return this.isEnabled;
		
	}

	/**
	 * adds JCombo and text for source instance
	 *
	 */
	private void createSourceInstance() {
		
		JComboBox cmbInstance = null;
		try {
			
			cmbInstance = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbInstance());
			cmbInstanceList.add(cmbInstance);
			cmbInstance.setEnabled(false);
		} catch (Exception e) {}
		
		JTextField txtInstanceTranslated = new JTextField(PanelInstance.TXT_DEFAULT,30);
		
		txtInstanceTranslated.addKeyListener(this.lstKey);
		txtInstanceTranslated.addMouseListener(this.lstMouse);
		
		txtInstanceTranslated.setEnabled(false);
		txtInstanceTranslatedList.add(txtInstanceTranslated);
		
		this.setEnabled(false);
		
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelInfo
	 * @uml.property  name="instance"
	 */
	public static PanelInstance getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelInstance();
		return instance;
	}
	
	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.BOTH;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}
	
	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		
		/* adds the default source instances */
		for(int i =0;i<PanelInfo.SOURCE_SCHEMES_NUMBER;i++)
			createSourceInstance();
		
		// Initialize gui elements
		this.initializePanel();		
		// Create panel general info
		pnlGeneralInfo.setLayout(new GridBagLayout());
		pnlGeneralInfo.setBorder(
				new javax.swing.border.TitledBorder(
				"Apply Instance Rules"));
		
		
		/* show all the default combo */
		for(int i=0;i<PanelInfo.SOURCE_SCHEMES_NUMBER;i++) {
			addInstanceComponent(i,false);
			
		}
		
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlGeneralInfo, this.generalLayout);
		this.setEnabled(false);
		
	}

	/**
	 * adds component in this panel
	 * @param pnlGeneralInfo
	 * @param consGeneralInfo
	 * @param i
	 */
	private void addInstanceComponent(int i, boolean enable) {
		
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.HORIZONTAL;
		consGeneralInfo.anchor = GridBagConstraints.NORTH;
		consGeneralInfo.insets = new Insets(5,5,5,5);
		
		// Component in (0,0)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = (i+1);
		consGeneralInfo.anchor = GridBagConstraints.EAST;
		pnlGeneralInfo.add(cmbInstanceList.get(i), consGeneralInfo);

		consGeneralInfo.gridx = 1;
		consGeneralInfo.gridy = (i+1);
		consGeneralInfo.gridwidth = 4;
		pnlGeneralInfo.add(txtInstanceTranslatedList.get(i), consGeneralInfo);
		
		cmbInstanceList.get(i).setEnabled(enable);
		this.txtInstanceTranslatedList.get(i).setEnabled(enable);
	}
	
	/**
	 * adds a new source scheme
	 */
	public void addSourceScheme() {
		createSourceInstance();
		addInstanceComponent(this.cmbInstanceList.size()-1,this.isEnabled);
		this.setVisible(false);
		this.setVisible(true);
		
		
	}
	
	/**
	 * enables or disables this panel
	 */
	protected void switchOnOff(boolean b) {
		for(JTextField jtf : this.txtInstanceTranslatedList)
			jtf.setEnabled(b);
		
		for(JComboBox jcb: this.cmbInstanceList)
			jcb.setEnabled(b);
		
		this.isEnabled = b;
		this.setEnabled(b);
		
	}

	/**
	 * Initialize the panel
	 */
	private void initializePanel() {
		for(JTextField jtf: this.txtInstanceTranslatedList)
			jtf.setText(TXT_DEFAULT);
	}
	

	public void refreshInstancesList(String[] instances, int instanceNumber) {
		int items = this.cmbInstanceList.get(instanceNumber).getItemCount();
		for (int i = items; i < instances.length; i++) {
			this.cmbInstanceList.get(instanceNumber).addItem(instances[i]);
//			System.out.println("Added " + schemas[i] + " pnlInfo-SM");
		}
	}


	public String getSelectedSourceInstance(int instanceNumber) {
		return (String)this.cmbInstanceList.get(instanceNumber).getSelectedItem();
	}
	
	public JTextField getTxtInstance(int instanceNumber) {
		return this.txtInstanceTranslatedList.get(instanceNumber);
	}
	


	public int getInstanceSelectedIndex(int instanceNumber) {
		return this.cmbInstanceList.get(instanceNumber).getSelectedIndex();
	}
}