/*
 * This class implements a panel in which there are the functions to copy a model 
 * scheme into the super-model and to copy a super-model scheme into the model  
 */
package org.modelgen.jpf.applyRules.copy;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.applyRules.ApplyRulesTool;
import org.modelgen.jpf.applyRules.copy.report.ReportCopy;
import org.modelgen.jpf.applyRules.util.FactoryIcon;

/**
 * @author  gabriele  This class implements a panel in which there are the functions to copy a model   scheme into the super-model and to copy a super-model scheme into the model
 */
public class PanelCopy extends JPanel {
	private static final long serialVersionUID = 4085895857128235213L;
	// Singleton
	private static PanelCopy instance;
	// Facade
	private GeneralManagementFacade facade;
	// Listener 
	private ActionListener lstAction;
	// Layout
	private GridBagConstraints generalLayout;
	// Gui components
	protected JComboBox cmbCopyModel;
	protected JComboBox cmbCopySupermodel;
	protected JButton btnCopyModel;
	protected JButton btnCopySupermodel;
	protected JButton btnReportCopyIntoSuperModel;
	protected JButton btnReportCopyIntoModel;
	
	/**
	 * Class constructor
	 */
	private PanelCopy() {
		// Creates facade
		this.facade = GeneralManagementFacade.getInstance();
		// Creates listener
		this.lstAction = new ActionListenerCopy(this);
		// Creates components
		try {
			this.cmbCopyModel = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbScheme("model"));
			this.cmbCopyModel.addActionListener(this.lstAction);
		} catch (Exception e) {}
		this.btnCopyModel = new JButton("Copy into SuperModel scheme");
		this.btnCopyModel.addActionListener(this.lstAction);
		this.btnReportCopyIntoModel = new JButton(FactoryIcon.createReportIcon());
		this.btnReportCopyIntoModel.addActionListener(this.lstAction);
		this.btnReportCopyIntoModel.setToolTipText("View report");
		try {
			this.cmbCopySupermodel = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbScheme("superModel"));
			this.cmbCopySupermodel.addActionListener(this.lstAction);
		} catch (Exception e) {}
		this.btnCopySupermodel = new JButton("Copy into Model");
		this.btnCopySupermodel.addActionListener(this.lstAction);
		this.btnReportCopyIntoSuperModel = new JButton(FactoryIcon.createReportIcon());
		this.btnReportCopyIntoSuperModel.addActionListener(this.lstAction);
		this.btnReportCopyIntoSuperModel.setToolTipText("View report");
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelCopy
	 * @uml.property  name="instance"
	 */
	public static PanelCopy getInstance() {
		if(instance == null)
			instance = new PanelCopy();
		return instance;
	}
	
	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.NONE;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
	}
	
	/**
	 * Creates and shows gui
	 */
	public void createShow() {
		// Create panel general info
		setBorder(new javax.swing.border.TitledBorder(
				"Copy' functions"));
		// Component in (0,0)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;		
		this.generalLayout.anchor = GridBagConstraints.WEST;
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		add(this.cmbCopyModel, this.generalLayout);
		// Component in (1,0)
		this.generalLayout.gridx = 1;
		this.generalLayout.gridy = 0;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		add(this.btnCopyModel, this.generalLayout);
		// Component in (2,0)
		this.generalLayout.gridx = 2;
		this.generalLayout.gridy = 0;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		add(this.btnReportCopyIntoModel, this.generalLayout);
		// Panel copy super model
		// Component in (0,1)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 1;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		add(this.cmbCopySupermodel, this.generalLayout);
		// Component in (1,1)
		this.generalLayout.gridx = 1;
		this.generalLayout.gridy = 1;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		
		add(this.btnCopySupermodel, this.generalLayout);		
		// Component in (2,1)
		this.generalLayout.gridx = 2;
		this.generalLayout.gridy = 1;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		add(this.btnReportCopyIntoSuperModel, this.generalLayout);
	}
	
	/**
	 * Copy model into supermodel
	 */
	public void copyModelIntoSuperModel() {
		if(this.cmbCopyModel.getSelectedIndex() == 0) {
			JOptionPane.showMessageDialog(this, "Select a schema, please.", 
					"Copy function", JOptionPane.ERROR_MESSAGE);
		} else {
			String scheme = (String)this.cmbCopyModel.getSelectedItem();
			int schemeOID = Integer.parseInt(scheme.substring(0,
					scheme.indexOf(":")));
			// Copy
			try {
				// the third parameter is ugly to see. Introduces just to
				// solve the GUI refresh without implementing the observer
				// that require a LOT of refactoring... and, as usual, we don't
				// have time!
				this.facade.copyScheme("model", schemeOID,this.cmbCopySupermodel);
				ApplyRulesTool.getIstance().updateGuiData();
				JOptionPane.showMessageDialog(this, "Copy successfully", 
						"Copy function", JOptionPane.INFORMATION_MESSAGE);
			} catch (PersistenceException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			} catch (ClassNotFoundException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	/**
	 * Copy model into supermodel
	 */
	public void copySuperModelIntoModel() {
		if(this.cmbCopySupermodel.getSelectedIndex() == 0) {
			JOptionPane.showMessageDialog(this, "Select a schema, please.", 
					"Copy function", JOptionPane.ERROR_MESSAGE);
		} else {
			String scheme = (String)this.cmbCopySupermodel.getSelectedItem();
			int schemeOID = Integer.parseInt(scheme.substring(0,
					scheme.indexOf(":")));
			// Copy
			try {
				this.facade.copyScheme("superModel", schemeOID,this.cmbCopyModel);
				ApplyRulesTool.getIstance().updateGuiData();
				JOptionPane.showMessageDialog(this, "Copy successfully", 
						"Copy function", JOptionPane.INFORMATION_MESSAGE);
			} catch (PersistenceException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			} catch (ClassNotFoundException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Copy model", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	/**
	 * Create a report for model
	 */
	protected void reportModel() {
		if(this.cmbCopySupermodel.getSelectedIndex() == 0) {
			JOptionPane.showMessageDialog(this, "Select a schema, please.", 
					"Copy function", JOptionPane.ERROR_MESSAGE);
		} else {
			String scheme = (String)this.cmbCopySupermodel.getSelectedItem();
			int schemeOID = Integer.parseInt(scheme.substring(0,
					scheme.indexOf(":")));
			// Launch frame of report
			ReportCopy report = null;
			try {
				report = ReportCopy.getInstance(
						this.facade.getSchemeForReport(schemeOID));
				report.createReport();
			} catch (PersistenceException e) {
//				 TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Create a report for super model
	 */
	protected void reportSuperModel() {
		if(this.cmbCopySupermodel.getSelectedIndex() == 0) {
			JOptionPane.showMessageDialog(this, "Select a schema, please.", 
					"Copy function", JOptionPane.ERROR_MESSAGE);
		} else {
			String scheme = (String)this.cmbCopySupermodel.getSelectedItem();
			int schemeOID = Integer.parseInt(scheme.substring(0,
					scheme.indexOf(":")));
			// Launch frame of report
			ReportCopy report = null;
			try {
				report = ReportCopy.getInstance(
						this.facade.getSchemeForReport(schemeOID));
				report.createReport();
			} catch (PersistenceException e) {
//				 TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * @param schemas
	 */
	public void refreshModelSchemasList(String[] schemas) {
		int items = this.cmbCopyModel.getItemCount();
		for (int i = items; i < schemas.length; i++) {
			this.cmbCopyModel.addItem(schemas[i]);
//			System.out.println("Added " + schemas[i] + " pnlCopy-M");
		}
	}
	
	/**
	 * @param schemas
	 */
	public void refreshSuperModelSchemasList(String[] schemas) {
		int items = this.cmbCopySupermodel.getItemCount();
		for (int i = items; i < schemas.length; i++) {
//			System.out.println("Added " + schemas[i] + " pnlCopy-SM");
			this.cmbCopySupermodel.addItem(schemas[i]);
		}
	}	

}