/*
 * This class implements a frame with the report of copy of model 
 */
package org.modelgen.jpf.applyRules.copy.report;

import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;

import java.util.Collection;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * @author  gabriele  This class implements a frame with the report of copy of model
 */
public class ReportCopy extends JFrame {
	private static final long serialVersionUID = -5946405592669306281L;
	// Instance
	private static ReportCopy instance;
	// The panel
	private JPanel panel;
	// The scheme to view
	private Scheme scheme;
	
	/**
	 * Class constructor
	 * @param scheme
	 */
	private ReportCopy(Scheme scheme) {
		this.scheme = scheme;
		this.panel = new JPanel();
		this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
	}
	
	/**
	 * Return an instance of the class
	 * @param scheme The scheme to view
	 */
	public static ReportCopy getInstance(Scheme scheme) {
		if(instance == null)
			instance = new ReportCopy(scheme);
		return instance;
	}
	
	/**
	 * Creates a report
	 */
	public void createReport() {		
		String schemeModelName = scheme.getMetaInfo().getName();
		JLabel label0 = new JLabel("<html><font size=5><b>Report of scheme <font color=#64C864>"+ scheme.getName()+ "</font> (instance of model: <font color=#8E00C2>" + schemeModelName+")</font></b></font></html>");
		panel.add(label0);
		JLabel label00 = new JLabel(" ");
		panel.add(label00);
		
		for (Iterator<SchemeObject> objects = scheme.getAbstractConstructs().iterator(); objects.hasNext(); ) {
			SchemeObject object = objects.next();
			JLabel label1 = new JLabel("<html>Object: <font color=#48D031><b>"+object.getName()+"</b></font> instance of <b>"+object.getMetaInfo().getName()+"</b></html>");
			panel.add(label1);
			
			//The properties of this object:
			JLabel label2 = new JLabel("Properties: ");
			panel.add(label2);
			Collection<SchemeObjectProperty> prop = object.getProperties();
			if (prop.size()==0) {
				JLabel label3 = new JLabel("       none");
				panel.add(label3);
			} else {
				for (Iterator<SchemeObjectProperty> properties = prop.iterator(); properties.hasNext(); ) {
					SchemeObjectProperty property = properties.next();
					JLabel label3 = new JLabel("       "+property.getMetaInfo().getName()+": "+property.getValue());
					panel.add(label3);
				}
			}
			
			JLabel label4 = new JLabel("References: ");
			panel.add(label4);
			Collection<SchemeObjectReference> ref = object.getReferences();
			if(ref.size()==0) {
				JLabel label5 = new JLabel("       none");
				panel.add(label5);
			} else {
				//The references of this object:
				for (Iterator<SchemeObjectReference> references = object.getReferences().iterator(); references.hasNext(); ) {
					SchemeObjectReference reference = references.next();
					// paoloc 20060511
					// handling null (optional) references
					if (reference.getValue()==null)
					{
						JLabel label6 = new JLabel("       "+reference.getMetaInfo().getName() + ": "+ SchemeObject.MDM_UNSPECIFIED);
						panel.add(label6);
					}
					else
					{
					JLabel label6 = new JLabel("       "+reference.getMetaInfo().getName() + ": "+reference.getValue().getName());
					panel.add(label6);
					}
				}
			}	
		JLabel label7 = new JLabel(" ");
		panel.add(label7);
		}
		// Add panel to frame
		this.getContentPane().add(panel);
		this.pack();
		this.setVisible(true);
	}
}