package org.modelgen.jpf.automaticSignatureApplication.applicationView;

import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.jpf.automaticSignatureApplication.applicationView.action.ActionListenerRules;
import org.modelgen.jpf.automaticSignatureApplication.facade.FindTranslationFacade;
import org.modelgen.signature.application.macroruleapplication.SignatureApplicationUtility;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * @author Fabrizio Celli
 * Pannello per visualizzare le macroregole applicabili e gli effetti
 * della loro applicazione
 */
public class PanelRules extends JPanel {
	private static final long serialVersionUID = 8663841166780574529L;
	// Singleton
	private static PanelRules instance;
	// References to pnlInfo
	private PanelInfo pnlInfo;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Action listener
	private ActionListener lstAction;
	// Components for gui
	private JLabel lblRules;
	// Components for gui
	protected JList lstRules;
	protected JButton btnExecute;

	//metodi getter
	public JButton getBtnExecute() {
		return btnExecute;
	}

	public JList getLstRules() {
		return lstRules;
	}

	/**
	 * Class constructor
	 * @param pnlInfo 
	 */
	private PanelRules(PanelInfo pnlInfo) {
		super();
		this.pnlInfo = pnlInfo;
		// Listener
		this.lstAction = new ActionListenerRules(this);
		// Creates components
		this.lblRules = new JLabel("Result of the Translation:");
		this.lstRules = new JList();
		this.btnExecute = new JButton("Translate");
		this.btnExecute.setToolTipText("Translate");
		this.btnExecute.addActionListener(this.lstAction);
	}


	/**
	 * Return an instance of the class
	 * @param pnlInfo 
	 * @return PanelRules
	 */
	public static PanelRules getInstance(PanelInfo pnlInfo) {
		/* Singleton */
		if(instance == null)
			instance = new PanelRules(pnlInfo);
		return instance;
	}

	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}

	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlAddRules = new JPanel();
		JPanel pnlBtnRules;	 
		//inizializzo le regole
		this.initializeRules();
		// Panel argument
		pnlAddRules.setLayout(new GridBagLayout());
		pnlAddRules.setBorder(
				new javax.swing.border.TitledBorder(
				"View macrorule obtained from automatic translation"));

		// Set the parameter for the GrigBagConstrains
		GridBagConstraints consAddRules = new GridBagConstraints();
		consAddRules.fill = GridBagConstraints.NONE;
		consAddRules.anchor = GridBagConstraints.NORTH;
		consAddRules.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consAddRules.gridx = 0;
		consAddRules.gridy = 0;
		consAddRules.anchor = GridBagConstraints.WEST;
		pnlAddRules.add(this.lblRules, consAddRules);
		// Component in (0,1)
		consAddRules.gridx = 0;
		consAddRules.gridy = 1;
		consAddRules.anchor = GridBagConstraints.CENTER;
		consAddRules.fill = GridBagConstraints.BOTH;
		consAddRules.gridheight = 4;
		consAddRules.gridwidth = 4;
		JScrollPane scrRule = new JScrollPane(this.lstRules,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrRule.setPreferredSize(new Dimension(500, 160));
		pnlAddRules.add(scrRule, consAddRules);
		// Panel btn rules
		pnlBtnRules = new JPanel();
		pnlBtnRules.setLayout(new GridBagLayout());
		// Component in (1,1)
		consAddRules.gridx = 1;
		consAddRules.gridy = 1;
		pnlAddRules.add(pnlBtnRules, consAddRules);
		//	Component in (2,5)
		consAddRules.gridx = 2;
		consAddRules.gridy = 5;
		consAddRules.anchor = GridBagConstraints.CENTER;
		pnlAddRules.add(this.btnExecute, consAddRules);
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlAddRules, this.generalLayout);		
	}

	/*
	 * Inizializza i pannelli con le regole
	 */
	private void initializeRules(){
		DefaultListModel listModel =  new DefaultListModel();
		this.lstRules.setModel(listModel);
	}


	/**
	 * Restituisce le macroregole trovate, cio il percorso che ha portato alla scelta 
	 * delle macroregole
	 */
	public void execute() {
		//effettuo la traduzione automatica
		Scheme sourceScheme = this.pnlInfo.getSource();
		Model targetModel = this.pnlInfo.getTarget();
		List<String> regoleTrovate = null;
		//se sono stati selezionati modelle e schema
		if(sourceScheme!= null && targetModel!=null){
			if(SignatureApplicationUtility.getInstance().isCompatibile(targetModel.getModelSignature(), sourceScheme.getModelSignature())){
				regoleTrovate = new LinkedList<String>();
				regoleTrovate.add("SOURCE SCHEME BELONGS TO TARGET MODEL");
			}
			else{
				try {
					regoleTrovate = FindTranslationFacade.getInstance().translate(sourceScheme, targetModel);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		else{
			regoleTrovate = new LinkedList<String>();
			regoleTrovate.add("SELECT TARGET MODEL AND SOURCE SCHEME");
		}

		//aggiorno la vista
		DefaultListModel modelList =  (DefaultListModel)this.lstRules.getModel();
		modelList.removeAllElements();

		if(regoleTrovate!=null){
			modelList.addElement("THE AUTOMATIC TRANSLATION HAS SELECTED THE FOLLOWING MACRORULES:");
			modelList.addElement(" ");
			if(regoleTrovate.size()==0)
				modelList.addElement("NO MACRORULE SELECTED");
			else{
				for(String macro: regoleTrovate){
					modelList.addElement(macro);
				}
			}
		}
		else {
			modelList.addElement("-: THE AUTOMATIC TRANSLATION");
			modelList.addElement("-: IS NOT AVAILABLE FOR SCHEME OR MODEL SELECTED");
		}
	}


}
