package org.modelgen.jpf.defineModel.Components;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.defineModel.DefineModelTool;
import org.modelgen.signature.util.ToolFormuleAsStringhe;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.border.EmptyBorder;

public class ModelReport extends JScrollPane implements ActionListener{
	private static final long serialVersionUID = 1L;

	private boolean isNew;
	//Current Schem Model
	private Model model;
	//All models reported
	private static Map<String,ModelReport> reports = new HashMap<String,ModelReport>();


	private static ModelReport instance;

	public static ModelReport getInstance(Model m){
		String name = m.getName();
		ModelReport report = reports.get(name);
		if (report!=null){
			instance = report;
			instance.isNew = false;
		}else{
			instance = new ModelReport(m);
			reports.put(name,instance);
		}
		return instance;
	}

	public ModelReport(Model m){
		/**Passandogli questo modello se il modello viene cancellato o se operazioni comportano la
		 * cancellazione del modello dal workspace, viene anche informato il report*/
		//model = InfoServer.getModelByName(m.getName());
		model = GeneralManagementFacade.getInstance().getModelCachedByName(m.getName());

		setModelReportPanel();	
		isNew = true;
	}

	private void setModelReportPanel() {
		setViewportView(printModel());
		setViewportBorder(new EmptyBorder(0,10,0,0));

	}

	public void refresh(){
		JTabbedPane tabbed = ((JTabbedPane)getParent());

		/***********Finezza: per cambiare il titolo del tab in caso di rinominazione del modello ************/
		int index = tabbed.getSelectedIndex();
		tabbed.setTitleAt(index,model.getName());
		/************************/

		setModelReportPanel();
		updateUI();

	}

	private JPanel printModel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		if (model==null){
			panel.add(new JLabel("No model"));
			return panel;
		}
		panel.add(createTabBar(),BorderLayout.PAGE_START);	
		panel.add(createContent(),BorderLayout.CENTER);

		return panel;
	}

	private Component createContent() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
		JLabel modelName = new JLabel("<html><font size=5><b>Report of model <font color=#00B4FF>"+model.getName()+"</font> (short name: <font color=#00B4FF>"+model.getShortName()+"</font>)</b></html>");
		panel.add(modelName);
		//informazioni sulle famiglie
		JLabel infoFamily;
		if(model.getFamilyOid()==0)
			infoFamily = new JLabel("There are no rule to generate an automatic translation from/to this model");
		else
			infoFamily = new JLabel("This model can be automatic translated");
		panel.add(infoFamily);
		
		//++++++
		JLabel label00 = new JLabel("\n");
		panel.add(label00);
		//+++++++

		for (Iterator<Construct> i = model.getAbstractConstructs().iterator(); i.hasNext(); ) {
			Construct c = i.next();
			JLabel constructName = new JLabel("<html>Construct: <font color=#2F72FB><b>" + c.getName() + "</b></html>");
			panel.add(constructName);
			JLabel mappingConstructs = new JLabel("<html>Metaconstruct:</html>");
			panel.add(mappingConstructs);
			for (Iterator<Construct> itSMC = c.getSuperModelConstructs().iterator(); itSMC.hasNext(); ) {
				Construct smc = itSMC.next();
				JLabel label3 = new JLabel("       " + smc.getName());
				label3.setFont(new Font("Arial",Font.BOLD,12));
				panel.add(label3);
			}

			JLabel label4 =new JLabel("Properties: ");
			panel.add(label4);
			if (c.getProperties().size() != 0) {
				for (Iterator<Property> j = c.getProperties().iterator(); j.hasNext(); ) {
					Property p = j.next();
					JLabel label5 = new JLabel("       "+p.getName() + " (" + p.getType() + ")");
					panel.add(label5);
				}
			} else {
				JLabel label5 = new JLabel("       None");
				panel.add(label5);
			}

			JLabel label9 =new JLabel("Bonds: ");
			panel.add(label9);
			if (c.getUserBonds()!=null && !c.getUserBonds().equalsIgnoreCase("true")) {
				JLabel label10 = new JLabel("       "+ToolFormuleAsStringhe.mantainsOnlyOrParenthesis(c.getUserBonds()));
				panel.add(label10);
			} else {
				JLabel label10 = new JLabel("       No Bond Constrained");
				panel.add(label10);
			}		

			JLabel label6 = new JLabel("References: ");
			panel.add(label6);

			if (c.getReferences().size() != 0) {
				for (Iterator<Reference> j = c.getReferences().iterator(); j.hasNext(); ) {
					Reference r = j.next();
					if (r.getTargetConstruct()!=null)
					{
						JLabel label7 = new JLabel("       "+r.getName() + " (of type " + r.getTypeAsString() + "),  to " + r.getTargetConstruct().getName());
						panel.add(label7);
					}
				}
			} else {
				JLabel label7 = new JLabel("       None");
				panel.add(label7);
			}	

			JLabel label8 = new JLabel(" ");
			panel.add(label8);

		}
		return panel;
	}

	private Component createTabBar() {
//		/*******************************************************************/
		JToolBar barra = new JToolBar("Tab Bar");
		JButton refresh = new JButton(ResourceFunctions.openImageIcon(this.getClass(), "refresh.gif"));
		refresh.setActionCommand("refresh");		
		refresh.addActionListener(this);

		JButton close = new JButton(ResourceFunctions.openImageIcon(this.getClass(), "button_cancel.png"));
		close.setActionCommand("close");
		close.addActionListener(DefineModelTool.getReportsTab());
		close.addActionListener(this);			

		Color bcolor = this.getBackground();
		refresh.setBackground(bcolor);
		close.setBackground(bcolor);

		/******************************/
//		close.putClientProperty("report",this);
		/******************************/


		barra.setLayout(new FlowLayout(FlowLayout.RIGHT));
		barra.setFloatable(false);
		barra.add(close);
		barra.add(refresh);
//		/********************************************************************/
		return barra;
	}

	public boolean isNew() {
		return isNew;
	}

	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();
		if (command.equals("refresh"))
			refresh();	
		else
			removeReportFromMap();
	}

	private void removeReportFromMap() {
//		System.out.println("ModelReport.removeReportFromMap");
		reports.remove(model.getName());		
	}

}
