package org.modelgen.jpf.defineModel;


import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.commons.Tool;
import org.modelgen.jpf.defineModel.Components.ConsoleScroll;
import org.modelgen.jpf.defineModel.Components.ProblemsScroll;
import org.modelgen.jpf.defineModel.Components.ReportTabbedPane;
import org.modelgen.jpf.defineModel.Components.StatusUtility;
import org.modelgen.jpf.defineModel.Tree.WorkSpaceTree;
import org.modelgen.jpf.defineModel.Tree.Popup.AddConstructPopupListener;
import org.modelgen.jpf.defineModel.Tree.Popup.AddPropertyPopupListener;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.CloneModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.CreateBondAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.DeleteBondAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.DeleteConstructAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.DeleteModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.DeletePropertyAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.ExportModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.ImportModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.ModelReportAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.NewModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.RenameConstructAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.RenameModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.RenamePropertyAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.RenameReferenceAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.StoreModelAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.VoidAddConstructAction;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.VoidAddPropertyAction;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

public class DefineModelTool extends JInternalFrame implements Tool{
	private static final long serialVersionUID = 1L;
	
		/*L'ordine e l'indentazione rispecchiano la relazione di contenimento del Frame*/
		private JMenuBar menuBar;
			private JMenu file;
				private JMenuItem newModelMenuItem;	
				private JMenuItem saveModelMenuItem;
				
			private JMenu model;
				private JMenuItem renameModelMenuItem;
				private JMenuItem deleteModelMenuItem;
				private JMenuItem cloneModelMenuItem;
				private JMenuItem reportModelMenuItem;
				
			private JMenu construct;
				private JMenuItem deleteConstructMenuItem;
				private JMenuItem renameConstructMenuItem;
				private JMenuItem renamePropertyMenuItem;
				private JMenuItem deletePropertyMenuItem;
				private JMenuItem addViewBondMenuItem;
				private JMenuItem deleteBondMenuItem;
				private JMenuItem renameReferenceMenuItem;
				
				
			private JMenu strumenti;
				private JMenuItem importModelMenuItem;
				private JMenuItem exportModelMenuItem;
				
		private JToolBar toolBar;
			private JButton newModelToolButton;
			private JButton cloneModelToolButton;
			private JButton renameModelToolButton;
			private JButton saveModelToolButton;
			private JButton deleteModelToolButton;
			private JButton reportModelToolButton;
			
			
			private JButton importModelToolButton;
			private JButton exportModelToolButton;
			
			private JButton newConstructToolMenu;	
			private JButton deleteConstructToolButton;
			private JButton renameConstructToolButton;
		
			private JButton newPropertyToolMenu;
			private JButton deletePropertyToolButton;
			private JButton renamePropertyToolButton;
			
			private JButton addViewBondToolMenu;
			private JButton deleteBondToolMenu;
			
			private JButton renameReferenceToolButton;
			
		private JSplitPane coreSplit;
			private JScrollPane treeScroll;
			private JSplitPane reportSplit;
				
				public static ReportTabbedPane reportsTabs;
				public static JTabbedPane infoTabs;

		/*Singleton*/
		private static DefineModelTool istance;
		
		private Map<String,Object> applicationObjs;
		
		public DefineModelTool() throws PersistenceException{
			super("DefineModelTool",true,true,true,true);
			checkConnection();
			initProperties();
			initStructure();
			initComponetsProperties();
			composeStructure();
			/*Consente di applicare il pattern singleton nonostante la classe 
			 * venga istanziata dal CorePlugin*/
			istance = this;
		}

		private void checkConnection() throws PersistenceException{	
			/*Inizializza il PH condiviso e se il server non  avviato provoca 
			 *un eccezione intercettata dalla ToolActivationAction
			 * */
			GeneralManagementFacade.getInstance().initPesistenceHandler();
		}

		/*Singleton*/
		public static DefineModelTool getIstance(){
			/*Il controllo condizionale non  necessario:
			 * - il metodo deve ritornare l'internal-frame solo se questa  stata creata
			 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
			 * dal controllo dell'Application Plugin
			 * */
			return istance;
		}

		public void init(Map<String,Object> applicationObjects) {
			this.applicationObjs  = applicationObjects;
		}
	
		
		private void initProperties() {
			ImageIcon imageIcon = ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png");
			setFrameIcon(imageIcon);
			
			setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
			
			Dimension dim = getToolkit().getScreenSize();
				int larghezza = (int) (dim.getWidth()*4/5);
				int altezza = (int) (dim.getHeight()*3/4);
				int bordoSinistro = (int) ((dim.getWidth() - larghezza)/2);
				int bordoSuperiore = (int) ((dim.getHeight()-altezza)/4)-altezza/22;
				setBounds(bordoSinistro,bordoSuperiore,larghezza,altezza);
		}
	
		private void initStructure() {
			menuBar = new JMenuBar();
			toolBar = new JToolBar("ToolBar");
					
			reportsTabs = new ReportTabbedPane();
			infoTabs = new JTabbedPane();
				infoTabs.addTab("Problems",ResourceFunctions.openImageIcon(this.getClass(), "error.png"),ProblemsScroll.getInstance(),"ModelGen application problems");
				infoTabs.addTab("Console",ResourceFunctions.openImageIcon(this.getClass(), "task.png"),ConsoleScroll.getInstance(),"ModelGen use actions");
				
			
				reportSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,reportsTabs,infoTabs);
			
			
			treeScroll = WorkSpaceTree.getIstance();
			coreSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,treeScroll,reportSplit);

		}
	
		private void initComponetsProperties() {
			Dimension dim = getToolkit().getScreenSize();
			int altezza = (int) (dim.getHeight()/(2.5));
			reportsTabs.setPreferredSize(new Dimension(0,altezza));
		}
	
		private void composeStructure() {
				
			addMenuBarItems();
			setJMenuBar(menuBar);
				
			addToolBarButtons();
			add(toolBar,BorderLayout.PAGE_START);
			
			add(coreSplit,BorderLayout.CENTER);
		}
	
		private void addMenuBarItems() {
			file = new JMenu("File");	
				newModelMenuItem = new JMenuItem(NewModelAction.getIstance());
					newModelMenuItem.setText(NewModelAction.ACTION_NAME);
				saveModelMenuItem = new JMenuItem(StoreModelAction.getIstance());
					saveModelMenuItem.setText(StoreModelAction.ACTION_NAME);
//				saveAllModelsMenuItem = new JMenuItem(StoreAllModelsAction.getIstance());
//					saveAllModelsMenuItem.setText(StoreAllModelsAction.ACTION_NAME);
				
			file.add(newModelMenuItem);
				file.addSeparator();
			file.add(saveModelMenuItem);
//			file.add(saveAllModelsMenuItem);	
				
			model = new JMenu("Model");
				renameModelMenuItem = new JMenuItem(RenameModelAction.getIstance());
					renameModelMenuItem.setText(RenameModelAction.ACTION_NAME);
				deleteModelMenuItem = new JMenuItem(DeleteModelAction.getIstance());
					deleteModelMenuItem.setText(DeleteModelAction.ACTION_NAME);
				cloneModelMenuItem = new JMenuItem(CloneModelAction.getIstance());
					cloneModelMenuItem.setText(CloneModelAction.ACTION_NAME);
				reportModelMenuItem = new JMenuItem(ModelReportAction.getIstance());
					reportModelMenuItem.setText(ModelReportAction.ACTION_NAME);
					
			model.add(renameModelMenuItem);
			model.addSeparator();
			model.add(deleteModelMenuItem);
			model.addSeparator();
			model.add(cloneModelMenuItem);
			model.addSeparator();
			model.add(reportModelMenuItem);
				
			construct = new JMenu("Construct");	
				deleteConstructMenuItem = new JMenuItem(DeleteConstructAction.getIstance());
					deleteConstructMenuItem.setText(DeleteConstructAction.ACTION_NAME);
				renameConstructMenuItem = new JMenuItem(RenameConstructAction.getIstance());
					renameConstructMenuItem.setText(RenameConstructAction.ACTION_NAME);
				
				renamePropertyMenuItem = new JMenuItem(RenamePropertyAction.getIstance());
					renamePropertyMenuItem.setText(RenamePropertyAction.ACTION_NAME);
				deletePropertyMenuItem = new JMenuItem(DeletePropertyAction.getIstance());
					deletePropertyMenuItem.setText(DeletePropertyAction.ACTION_NAME);
				
				addViewBondMenuItem = new JMenuItem(CreateBondAction.getIstance());
					addViewBondMenuItem.setText(CreateBondAction.ACTION_NAME);
				deleteBondMenuItem = new JMenuItem(DeleteBondAction.getIstance());
					deleteBondMenuItem.setText(DeleteBondAction.ACTION_NAME);
					
				renameReferenceMenuItem = new JMenuItem(RenameReferenceAction.getIstance());
					renameReferenceMenuItem.setText(RenameReferenceAction.ACTION_NAME);
			
     		construct.add(renameConstructMenuItem);
			construct.addSeparator();
			construct.add(renamePropertyMenuItem);
			construct.add(deletePropertyMenuItem);
			construct.addSeparator();
			construct.add(addViewBondMenuItem);
			construct.add(deleteBondMenuItem);
			construct.addSeparator();
			construct.add(renameReferenceMenuItem);
			construct.addSeparator();
			construct.add(deleteConstructMenuItem);
				
			strumenti = new JMenu("Tools");
				importModelMenuItem = new JMenuItem(ImportModelAction.getIstance());
					importModelMenuItem.setText(ImportModelAction.ACTION_NAME);
				exportModelMenuItem = new JMenuItem(ExportModelAction.getIstance());
					exportModelMenuItem.setText(ExportModelAction.ACTION_NAME);
				
				
				
			strumenti.add(importModelMenuItem);
			strumenti.add(exportModelMenuItem);		
				
			menuBar.add(file);
			menuBar.add(model);
			menuBar.add(construct);
			menuBar.add(strumenti);
		}
	
		private void addToolBarButtons() {
			newModelToolButton = new JButton(NewModelAction.getIstance());
				newModelToolButton.putClientProperty("relativeNode","root");
			cloneModelToolButton = new JButton(CloneModelAction.getIstance());
				cloneModelToolButton.putClientProperty("relativeNode","model");
			renameModelToolButton = new JButton(RenameModelAction.getIstance());
				renameModelToolButton.putClientProperty("relativeNode","model");
			saveModelToolButton = new JButton(StoreModelAction.getIstance());
				saveModelToolButton.putClientProperty("relativeNode","model");
			deleteModelToolButton = new JButton(DeleteModelAction.getIstance());
				deleteModelToolButton.putClientProperty("relativeNode","model");
			reportModelToolButton = new JButton(ModelReportAction.getIstance());
				reportModelToolButton.putClientProperty("relativeNode","allModel");
			importModelToolButton = new JButton(ImportModelAction.getIstance());
				importModelToolButton.putClientProperty("relativeNode","root");
			exportModelToolButton = new JButton(ExportModelAction.getIstance());
				exportModelToolButton.putClientProperty("relativeNode","allModel");
			
			renameConstructToolButton = new JButton(RenameConstructAction.getIstance());
				renameConstructToolButton.putClientProperty("relativeNode","construct");
			deleteConstructToolButton = new JButton(DeleteConstructAction.getIstance());
				deleteConstructToolButton.putClientProperty("relativeNode","construct");
			newConstructToolMenu = new JButton(VoidAddConstructAction.getIstance());
				/************************************************/
				newConstructToolMenu.addMouseListener(new AddConstructPopupListener());
			    /************************************************/
				newConstructToolMenu.putClientProperty("relativeNode","model");
				
			renamePropertyToolButton = new JButton(RenamePropertyAction.getIstance());
				renamePropertyToolButton.putClientProperty("relativeNode","property");
			deletePropertyToolButton = new JButton(DeletePropertyAction.getIstance());
				deletePropertyToolButton.putClientProperty("relativeNode","property");		
			newPropertyToolMenu = new JButton(VoidAddPropertyAction.getIstance());
				/************************************************/
				newPropertyToolMenu.addMouseListener(new AddPropertyPopupListener());
			    /************************************************/
				newPropertyToolMenu.putClientProperty("relativeNode","construct");
			
			addViewBondToolMenu = new JButton(CreateBondAction.getIstance());
				addViewBondToolMenu.putClientProperty("relativeNode","construct");
			deleteBondToolMenu = new JButton(DeleteBondAction.getIstance());
				deleteBondToolMenu.putClientProperty("relativeNode","construct");
				
				
			renameReferenceToolButton = new JButton(RenameReferenceAction.getIstance());
				renameReferenceToolButton.putClientProperty("relativeNode","reference");
			
			Dimension sep = new Dimension(90,0);
			toolBar.add(newModelToolButton);	
			toolBar.add(cloneModelToolButton);	
			toolBar.add(saveModelToolButton);
			toolBar.add(renameModelToolButton);
			toolBar.add(deleteModelToolButton);
			toolBar.add(reportModelToolButton);
			toolBar.addSeparator(sep);
			
			toolBar.add(newConstructToolMenu);
			toolBar.add(renameConstructToolButton);
			toolBar.add(deleteConstructToolButton);
			
			toolBar.add(newPropertyToolMenu);
			toolBar.add(renamePropertyToolButton);
			toolBar.add(deletePropertyToolButton);
			
			toolBar.add(addViewBondToolMenu);
			toolBar.add(deleteBondToolMenu);
			
			toolBar.add(renameReferenceToolButton);
			
			toolBar.addSeparator(new Dimension(140,0));
			
			toolBar.add(importModelToolButton);
			toolBar.add(exportModelToolButton);
			
			StatusUtility statusUtil = new StatusUtility(toolBar);
			statusUtil.enableStatus("root");
			statusUtil.disableOthers();
		}
	
		public String toString(){
			return "DefineModelTool";
		}
		public JToolBar getToolBar(){
			return toolBar;
		}

		public static ReportTabbedPane getReportsTab() {
			return reportsTabs;
		}

		public Map<String, Object> getInfo() {
			return this.applicationObjs;
		}





}


