package org.modelgen.jpf.defineModel.Dialog.dialogIMPL;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.defineModel.Dialog.DMDialog;
import org.modelgen.jpf.defineModel.utility.BondHandler;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;

public class CreateBondDialog extends DMDialog implements ActionListener{

	private static final long serialVersionUID = 406090964547135408L;

	private Construct construct;
	private String superModelConstructName;
	private List<Property> properties;
	//mappa nome propriet - JComboBox
	private Map<String, JComboBox> combo;
	
	//vincoli
	private String bond;
	//signature
	private String signature;

	private JPanel infoPanel;
	private JScrollPane scrollPane;
	private JList bonds;
	private JPanel choicePanel;

	/*
	 * Costruttori
	 */
	public CreateBondDialog(Construct construct) {
		this(ToolBoxFrame.getIstance(), "Create bonds", true);
		this.initializeItem(construct);
		defineStructure(this.getContentPane());
	}

	public CreateBondDialog(Frame owner, String title, boolean modal) {
		super(ToolBoxFrame.getIstance(), title , true);
	}

	/*
	 * Inizializza le variabili d'istanza della classe
	 * @param construct
	 */
	private void initializeItem(Construct construct){
		this.construct = construct;

		//prendiamo solo le propriet di tipo boolean
		this.properties = new LinkedList<Property>();
		Collection<Property> allProperties = this.construct.getProperties();
		for(Property p: allProperties){
			if(p.getType().equalsIgnoreCase("boolean"))
				this.properties.add(p);
		}
		
		//nome nel costrutto a livello SM
		Collection<Construct> smConstructs = this.construct.getSuperModelConstructs();
		Iterator<Construct> it = smConstructs.iterator();
		//il SM  inserito alla fine della lista
		while(it.hasNext())
			this.superModelConstructName = it.next().getName();

		//inizializzo vincoli e segnature
		this.bond = construct.getUserBonds();
		this.signature = construct.getSignature();
	}

	/*
	 * Crea la struttura dell'interfaccia grafica
	 */
	private void defineStructure(Container contentPane) {
		super.setSize(new Dimension(400,400));
		ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "addBond.gif");

		JLabel messageLabel = new JLabel("Bonds creation \n",icon,SwingConstants.LEFT);
		contentPane.add(BorderLayout.NORTH,messageLabel);

		//pannello principale
		infoPanel = new JPanel();

		//pannello con la signature scelta
		this.bonds = new JList();
		this.createListBonds();

		//pannello con i vincoli da imporre
		this.createChoicePanel();

		infoPanel.add(BorderLayout.NORTH, scrollPane);
		infoPanel.add(BorderLayout.SOUTH, choicePanel);

		contentPane.add(BorderLayout.CENTER,infoPanel);

		//Serve a dare lo spazio dal margine sinistro
		contentPane.add(BorderLayout.EAST,new JLabel("  "));

		JPanel controlPanel = new JPanel();
		JButton okButton = new JButton("add bond");
		okButton.setActionCommand("commit");
		okButton.addActionListener(this);
		JButton exitButton = new JButton("exit");
		exitButton.setActionCommand("exit");
		exitButton.addActionListener(this);
		controlPanel.add(okButton);
		controlPanel.add(exitButton);
		contentPane.add(BorderLayout.SOUTH,controlPanel);

	}

	//gestisce la pressione dei bottoni
	public void actionPerformed(ActionEvent ae) {
		JButton actionButton = (JButton) ae.getSource();
		String actionCommand = actionButton.getActionCommand();
		if(actionCommand.equals("commit"))
			commit();
		else
			exit();
	}

	/*
	 * Aggiorna il bond e la signature del costrutto
	 */
	private void commit() {
		this.calculateBond();
		Map<String,String> values= new HashMap<String,String>();
		values.put("bond",bond);
		values.put("signature",signature);
		super.setValues(values);		
		this.setVisible(false);
	}

	/*
	 * Esce senza modificare nulla
	 */
	private void exit() {
		Map<String,String> values= null;
		super.setValues(values);		
		this.setVisible(false);
	}

	
	/*
	 * Calcola i vincoli
	 */
	private void calculateBond(){
		String vincolo = null;
		String sign = null;
		for(String property: this.combo.keySet()){
			Property p = this.findProperty(property);
			String propSMnAme = p.getMetaInfo().getName();
			String value = (String)this.combo.get(property).getSelectedItem();
			vincolo = BondHandler.getInstance().createBond(vincolo, property, value);
			sign = BondHandler.getInstance().createSignature(sign, propSMnAme, value, this.superModelConstructName);
		}
		//aggiorno il bond
		if(vincolo!=null){
			if(this.bond==null || this.bond.length()==0 || this.bond.equalsIgnoreCase("true"))
				this.bond = vincolo;
			else
				this.bond = "("+this.bond+" OR "+vincolo+")";
		}
		//aggiorno la signature
		if(sign!=null){
			if(this.signature==null || this.signature.length()==0 || this.signature.equalsIgnoreCase("true"))
				this.signature = sign;
			else
				this.signature = "("+this.signature+" OR "+sign+")";
		}
	}

	/*
	 * Crea il pannello per scegliere i vincoli sulle properties
	 */
	private void createChoicePanel(){
		int size = properties.size();
		this.combo = new HashMap<String, JComboBox>();
		String[] option = populateCombo();
		GridLayout layout = new GridLayout(size, 2, 2, 2);
		this.choicePanel = new JPanel(layout);
		JLabel lab;
		JComboBox c;
		for(Property p: properties){
			lab = new JLabel(p.getName());	
			c = new JComboBox(option);
			combo.put(p.getName(), c);
			choicePanel.add(lab);
			choicePanel.add(c);
		}
	}

	/*
	 * Crea la visualizzazione dei vincoli gi decisi dall'utente
	 */
	private void createListBonds(){
		String bond = this.construct.getUserBonds();
		DefaultListModel listModel =  new DefaultListModel();
		//aggiungiamo lo spazio per questioni stilistiche
		listModel.addElement("BONDS Created:");
		listModel.addElement("");
		if(bond!=null && !bond.equalsIgnoreCase("true")){
			List<String> vincoli = BondHandler.getInstance().analizeBonds(bond);
			for(String s: vincoli){
				listModel.addElement(s);
				listModel.addElement("        OR");
			}
			if(listModel.get(listModel.getSize()-1).equals("        OR"))
				listModel.remove(listModel.getSize()-1);
		}
		else{
			listModel.addElement("- NO BOND CONSTRAINED -");
		}
		this.bonds = new JList(listModel);

		//creo lo ScrollPane
		this.scrollPane = new JScrollPane(this.bonds, 
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, 
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		this.scrollPane.setPreferredSize(new Dimension(380,150));	
	}

	/*
	 * Dato il nome della property, riporta l'oggetto con quel
	 * nome
	 */
	private Property findProperty(String property) {
		for(Property p: properties)
			if(p.getName().equals(property))
				return p;
		return null;
	}

	/*
	 * Vincoli possibili sulle properties
	 */
	private String[] populateCombo(){
		return new String[]{"     free", "     true", "     false"};
	}
}
