package org.modelgen.jpf.defineModel.Dialog.dialogIMPL;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.defineModel.Components.ProblemsScroll;
import org.modelgen.jpf.defineModel.Dialog.DMDialog;
import org.modelgen.jpf.defineModel.utility.ErrorTable;
import org.modelgen.jpf.defineModel.utility.XMLFileFilter;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileFilter;


public class ImportModelDialog extends DMDialog implements ActionListener{
	private static final long serialVersionUID = -5674702359489920416L;
	
	private JFormattedTextField fileName = null;
	
	private JRadioButton flat;
	private JRadioButton nested;
	
	public ImportModelDialog() {
		this(ToolBoxFrame.getIstance(), "Import Model", true);
	}

	public ImportModelDialog(Frame owner, String title, boolean modal) {
		super(ToolBoxFrame.getIstance(), title, true);
		defineStructure(this.getContentPane());
	}

	private void defineStructure(Container contentPane) {
		super.setSize(new Dimension(350,200));
		ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "import.gif");
		
		
		JLabel messageLabel = new JLabel("Model importation \n          ",icon,SwingConstants.LEFT);
		contentPane.add(BorderLayout.NORTH,messageLabel);
		
		JPanel infoPanel = new JPanel();
			infoPanel.setLayout(new GridLayout(2,1));
		JPanel firstRow = new JPanel();
		LayoutManager layout = new FlowLayout(FlowLayout.CENTER,30,0);
			firstRow.setLayout(layout);
		JPanel secondRow = new JPanel();
		LayoutManager layoutLeft = new FlowLayout(FlowLayout.LEFT,30,0);
			secondRow.setLayout(layoutLeft);
		
		// Artifici di separazione
		Dimension textDimension = new Dimension(150,20);
		
		fileName = new JFormattedTextField();
			fileName.setPreferredSize(textDimension);
		JButton browser = new JButton("search");
			browser.setActionCommand("browse");
			browser.addActionListener(this);
			firstRow.add(fileName);
			firstRow.add(browser);
		
		flat = new JRadioButton("flat",true);
		nested = new JRadioButton("Nested");
		ButtonGroup gruop = new ButtonGroup();
			gruop.add(flat);
			gruop.add(nested);
			secondRow.add(flat);
			secondRow.add(nested);
		
		infoPanel.add(firstRow);
		infoPanel.add(secondRow);

		contentPane.add(BorderLayout.CENTER,infoPanel);

		//Serve a dare lo spazio dal margine sinistro
		contentPane.add(BorderLayout.EAST,new JLabel("  "));
		
		JPanel controlPanel = new JPanel();
			JButton okButton = new JButton("Ok");
				okButton.setActionCommand("commit");
				okButton.addActionListener(this);
			JButton resetButton = new JButton("reset");
//				resetButton.setActionCommand("reset");
				resetButton.addActionListener(this);
			controlPanel.add(okButton);
			controlPanel.add(resetButton);
		contentPane.add(BorderLayout.SOUTH,controlPanel);
		
	}

	public void actionPerformed(ActionEvent ae) {
		JButton actionButton = (JButton) ae.getSource();
		String actionCommand = actionButton.getActionCommand();
		if(actionCommand.equals("commit"))
			commit();
		if(actionCommand.equals("browse"))
			browse();
		if(actionCommand.equals("reset"))
			reset();
		
	}

	private void browse() {
		String xmlRepositoryDir = ModelgenConfigurationManager.getIstance().getProperty("xmlRepositoryPath", System.getProperty("user.home"), "paths");
		
		JFileChooser chooser = new JFileChooser(xmlRepositoryDir);
	    	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	    FileFilter filter = XMLFileFilter.getInstance();
	    chooser.setFileFilter(filter);
	    
	    int returnVal = chooser.showOpenDialog(ToolBoxFrame.getIstance());
	    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	String file = chooser.getSelectedFile().getAbsolutePath();
	    	fileName.setText(file);
	    }
		
	}

	private void reset() {
		fileName.setText("");	
	}

	private void commit() {
		String file = fileName.getText();
		String importMode;
		
		if (flat.isSelected()){
			importMode = "flat";
		}else{
			importMode= "nested";
		}
		
		
		ErrorTable errors = new ErrorTable();
		if (file.equals("")){
			errors.addError("Select a file");
		}
		if (!"xml".equals(XMLFileFilter.getInstance().getExtension(file))){
			errors.addError("The selected file is not a xml file");
		}
		if (!errors.isEmpty()){
			JOptionPane.showMessageDialog(this,"Errors:\n"+errors);
			ProblemsScroll.getInstance().logWarning("Check data","Import Model", "");
			this.getToolkit().beep();
			reset();
			return;
			// Non viene eseguito this.setVisible(true) quindi il controllo resta sulla Dialog corrente
		}
		

		Map<String,String> values= new HashMap<String,String>();
			values.put("xmlFile",file);
			values.put("importMode",importMode);
		super.setValues(values);		
		this.dispose();;
	}
}
