package org.modelgen.jpf.defineModel.Tree.ObjectDescription;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.facade.ResourceFunctions;

import java.awt.Component;
import java.util.Iterator;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

public class DMTreeCellRender extends DefaultTreeCellRenderer {
	private static final long serialVersionUID = 4958306164706894750L;
	
	private static DMTreeCellRender instance = null;
	public static DMTreeCellRender getInstance(){
		if (instance==null)
			instance = new DMTreeCellRender();
		return instance;
	}

    private DMTreeCellRender() {
    }

    public Component getTreeCellRendererComponent(
                        JTree tree,
                        Object value,
                        boolean sel,
                        boolean expanded,
                        boolean leaf,
                        int row,
                        boolean hasFocus) {

        super.getTreeCellRendererComponent(
                        tree, value, sel,
                        expanded, leaf, row,
                        hasFocus);
        
        if (isModelNode(value)) {
	        if (leaf){
	       		 setIcon(ResourceFunctions.openImageIcon(this.getClass(), "modelOut.gif"));
	       	}else{
	       		 setIcon(ResourceFunctions.openImageIcon(this.getClass(), "model.gif"));
	       	}
	        Model m = getNodeInfo(value).getModel();
            setToolTipText("Name: "+ m.getName()+"\nShortName"+m.getShortName());  
        } 
//        else {
//            setToolTipText(null); 
//        } 
        
        if (isConstructNode(value)) {
        	if (leaf){
        		 setIcon(ResourceFunctions.openImageIcon(this.getClass(), "construct.gif"));
        	}else{
        		 setIcon(ResourceFunctions.openImageIcon(this.getClass(), "constructOut.gif"));
        	}
            Construct c = getNodeInfo(value).getConstruct();
//            Collection cs = c.getSuperModelConstructs();
            Iterator<Construct> it = c.getSuperModelConstructs().iterator();
            String mappings = "";
            while(it.hasNext()){
            	mappings = mappings.concat(it.next().getName());
            }
            setToolTipText("Name: "+ c.getName()+"\nMapping: "+mappings);
        } 
//        else {
//            setToolTipText(null); 
//        } 
        
        if (isPropertyNode(value)) {
            setIcon(ResourceFunctions.openImageIcon(this.getClass(), "property.gif"));
            
            Property p = getNodeInfo(value).getProperty();
            setToolTipText("Name: "+p.getName()+"\nType: "+p.getType()+"\nMapping: "+p.getMetaInfo().getName());
        } 
        else {
            setToolTipText(null); 
        } 
        if (isReferenceNode(value)) {
            setIcon(ResourceFunctions.openImageIcon(this.getClass(), "reference.png"));
            Reference r = getNodeInfo(value).getReference();
            setToolTipText("Name: "+r.getName()+"\nType: "+r.getTypeAsString()+"\nMapping: "+r.getMetaInfo().getName());
        } else {
            setToolTipText(null); 
            
        } 
        if (isRoot(value)){
        	setIcon(ResourceFunctions.openImageIcon(this.getClass(), "workspace.png"));
        }

        return this;
    }

   private boolean isRoot(Object value) {
	   return ((DefaultMutableTreeNode)value).isRoot();
	}

   private NodeInfo getNodeInfo(Object value) {
		return ((NodeInfo)(((DefaultMutableTreeNode)value).getUserObject()));
	}

   private boolean isReferenceNode(Object value) {
    	return ((NodeInfo)((DefaultMutableTreeNode)value).getUserObject()).getReference()!=null;
	}

   private boolean isPropertyNode(Object value) {
	   return ((NodeInfo)((DefaultMutableTreeNode)value).getUserObject()).getProperty()!=null;
   }

   private boolean isConstructNode(Object value) {
	   return ((NodeInfo)((DefaultMutableTreeNode)value).getUserObject()).getConstruct()!=null;
   }

   private boolean isModelNode(Object value) {	
	   return ((NodeInfo)((DefaultMutableTreeNode)value).getUserObject()).getModel()!=null;
   }

}



