package org.modelgen.jpf.defineModel.Tree.Popup.PopupActions;

import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.defineModel.Components.ConsoleScroll;
import org.modelgen.jpf.defineModel.Components.ProblemsScroll;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

public class StoreModelAction extends AbstractAction {
	private static final long serialVersionUID = 1L;

	public static final String ACTION_NAME = "Save...";
	
	private static StoreModelAction istance;
	
	public static StoreModelAction getIstance(){
		if(istance == null)
			istance = new StoreModelAction();
		return istance;
	}	
	
	private StoreModelAction(){
		putValue(Action.SHORT_DESCRIPTION,ACTION_NAME);
		ImageIcon imageIcon = ResourceFunctions.openImageIcon(this.getClass(), "save.png");
		putValue(Action.SMALL_ICON,imageIcon);
	}

	
	public void actionPerformed(ActionEvent arg0) {
		ConsoleScroll.getInstance().logTask("Saving model",Action.NAME,"model");
		Model model = ActionUtility.getModel();
		
		if (model==null){
			ProblemsScroll.getInstance().logWarning("Select a model", "Save model", "model");
			return;
		}
		/**************************************************/ 

		try {
			//carico le informazioni sulla famiglia
			model.setFamilyOid(GeneralManagementFacade.getInstance().findFamily(model.getModelSignature()));
			model.setAncestor(false);
			//se il modello non appartiene ad alcuna famiglia
			if(model.getFamilyOid()==0){
				int n = JOptionPane.showConfirmDialog(
					    ToolBoxFrame.getIstance(),
					    "There are no rules to execute an automatic tranlation to/from this model",
					    "Do you want to save this model?",
					    JOptionPane.YES_NO_OPTION);
				if(n==JOptionPane.NO_OPTION){
					//mi blocco
					return;
				}
			}
			//salvo il modello
			GeneralManagementFacade.getInstance().storeModel(model);
			ConsoleScroll.getInstance().logInformation("Model saved: [Name: "+model.getName()+ ", ShortName: "+model.getShortName()+"]",Action.NAME,"model");
		} catch (PersistenceException e) {
			ProblemsScroll.getInstance().logError("Errors saving model: ["+e.getMessage()+"]",Action.NAME,"model");
		}
		
		
		/*Performs RefreshAction*/
		RefreshAction.getIstance().actionPerformed(arg0);
	}

}