package org.modelgen.jpf.defineModel.Tree;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.jpf.defineModel.DefineModelTool;
import org.modelgen.jpf.defineModel.Components.StatusUtility;
import org.modelgen.jpf.defineModel.Tree.ObjectDescription.NodeInfo;
import org.modelgen.jpf.defineModel.Tree.ObjectDescription.TreeContent;
import org.modelgen.jpf.defineModel.Tree.Popup.PopupActions.ActionUtility;

import java.awt.Color;
import java.awt.Dimension;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;


public class WorkSpaceTree extends JScrollPane implements TreeSelectionListener{
	private static final long serialVersionUID = -7299012683270433845L;
	int i = 0;

	private static TreeContent tree;

	private static WorkSpaceTree istance = null;

	private static TreePath lastSelectedPath = null;
	private static List<TreePath> lastExpandedPaths = null;

	public static WorkSpaceTree getIstance(){
		if (istance==null){
			istance = new WorkSpaceTree();
		}
		return istance;
	}



	private WorkSpaceTree(){
		super();

		Dimension screenDim = getToolkit().getScreenSize();
		int thisWidth = (int) (screenDim.getWidth()/(5.4));
		setMinimumSize(new Dimension(thisWidth,0)); 

		JPanel columnHeader = new JPanel();
		columnHeader.setBackground(Color.WHITE);
		columnHeader.setPreferredSize(new Dimension(0,5));
		setColumnHeaderView(columnHeader);

		tree = TreeContent.createsTreeContent();
		if (tree!=null){
			paintWorkSpace(tree);
		}else{
			paintError();
		} 		
	}


	private void paintWorkSpace(JComponent treeNew) {
		tree.addTreeSelectionListener(this);
		setViewportView(treeNew);
		updateUI();
	}

	private void paintError() {
		JPanel panel = new JPanel();
		JLabel errL = new JLabel("Errore nel DataBase");
		panel.add(errL);
		paintWorkSpace(panel);
	}

	public void refresh() {
//		oldTree = tree;
		tree = TreeContent.createsTreeContent();		
		if (tree!=null){
			paintWorkSpace(tree);
		}else{
			paintError();
		}
	}

	public TreeContent getTree() {
		return tree;
	}

	public void valueChanged(TreeSelectionEvent arg0){
		JToolBar toolBar = DefineModelTool.getIstance().getToolBar();
		StatusUtility statusUtil = new StatusUtility(toolBar);

		statusUtil.enableStatus("root");
		statusUtil.disableOthers();

		/**
			Eccezione sollevata da me con i refresh, perch dopo ogni operazione lo stato dei 
			pulsanti rimane abilitato anche se (come dopo ogni operazione) non c nessun nodo 
			selezionato
		 */
		if(arg0==null){
			return;
		}

		Object objPath = tree.getLastSelectedPathComponent();

		if (objPath==null){
			return;
		}

		DefaultMutableTreeNode node = (DefaultMutableTreeNode) objPath;

		/*Controllo importante (node==null prima di node.isRoot())*/	
		if(node.isRoot()){;
		return;
		}
		/***********************/
		Object obj = node.getUserObject();
		NodeInfo nodeInfo;
		if(obj==null){
			return;
		}

		nodeInfo = (NodeInfo) obj;


		Object dataObject = nodeInfo.getDataObject();

		if(dataObject instanceof Model){
			Model model = (Model) dataObject;
			statusUtil.enableStatus("allModel");
			if(!model.isAncestor()){
				statusUtil.enableStatus("model");
				statusUtil.disableOthers();
			}
			return;
		}

		if(dataObject instanceof Construct){
			Model model = ActionUtility.getConstructModel();
			if(!model.isAncestor()){
				statusUtil.enableStatus("construct");
				statusUtil.disableOthers();
			}
			return;
		}

		if(dataObject instanceof Property){
			Model model = ActionUtility.getPropertyModel();
			if(!model.isAncestor()){
				statusUtil.enableStatus("property");
				statusUtil.disableOthers();
			}
			return;
		}

		if(dataObject instanceof Reference){
			Model model = ActionUtility.getPropertyModel();
			if(!model.isAncestor()){
				statusUtil.enableStatus("reference");
				statusUtil.disableOthers();
			}
			return;
		}
	}



	public static List<TreePath> getLastExpandedPaths() {
		return lastExpandedPaths;
	}



	public static void setLastExpandedPaths(List<TreePath> lastExpandedPaths) {
		WorkSpaceTree.lastExpandedPaths = lastExpandedPaths;
	}



	public static TreePath getLastSelectedPath() {
		return lastSelectedPath;
	}



	public static void setLastSelectedPath(TreePath lastSelectedPath) {
		WorkSpaceTree.lastSelectedPath = lastSelectedPath;
	}

}

