package org.modelgen.jpf.defineScheme.Components;


import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.defineScheme.DefineSchemeTool;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.JViewport;
import javax.swing.border.EmptyBorder;

public class TextReport extends JScrollPane implements ActionListener{
	private static final long serialVersionUID = 1L;
	private boolean isNew;
	//Current Schem Model
	private Scheme scheme;
	//All models reported
	private static Map<String,TextReport> reports = new HashMap<String,TextReport>();
	
	
	private static TextReport instance;
	
	public static TextReport getInstance(Scheme s){
		String name = s.getName();
		TextReport report = reports.get(name);
		if (report!=null){
			instance = report;
			instance.isNew = false;
		}else{
			instance = new TextReport(s);
			reports.put(name,instance);
		}
		return instance;
	}

	 private TextReport(Scheme s){
		/**Passandogli questo modello se il modello viene cancellato o se operazioni comportano la
		 * cancellazione del modello dal workspace, viene anche informato il report*/
//		model = InfoServer.getModelByName(m.getName());
		scheme = GeneralManagementFacade.getInstance().getSchemeByName(s.getName());
		setSchemeReportView();	
		setViewportBorder(new EmptyBorder(0,10,0,0));
		isNew = true;
	}
	
	 



	private void refresh(){
		JTabbedPane tabbed = ((JTabbedPane)getParent());
		
		/***********Finezza: per cambiare il titolo del tab in caso di rinominazione del modello ************/
		int index = tabbed.getSelectedIndex();
		tabbed.setTitleAt(index,scheme.getName());
		/************************/
		
		setSchemeReportView();
		updateUI();
		
	}
	
	private void setSchemeReportView() {
		JViewport port = null;
		try{
			port = new JViewport();
			JPanel p = new JPanel();p.setLayout(new BorderLayout());
				p.add(createTabBar(),BorderLayout.PAGE_START);
				p.add(createTextReport(),BorderLayout.WEST);
				port.add(p);

		}catch (RuntimeException e) {
			e.printStackTrace();
			port = new JViewport();port.add(new JLabel("Errore nella visualizzazione dello schema" +scheme.getName()));
		}
		setViewportView(port);
		
//		setViewportBorder(new EmptyBorder(0,10,0,0));
	}

	private Component createTextReport() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
		//Scheme scheme = Interfaccia_split.getScheme();
		String schemeModelName = scheme.getMetaInfo().getName();
		JLabel label0 = new JLabel("<html><font size=5><b>Report of scheme <font color=#64C864>"+ scheme.getName()+ "</font> (instance of model: <font color=#8E00C2>" + schemeModelName+")</font></b></font></html>");
		panel.add(label0);
		JLabel label00 = new JLabel(" ");
		panel.add(label00);
		
		for (Iterator<SchemeObject> objects = scheme.getAbstractConstructs().iterator(); objects.hasNext(); ) {
			SchemeObject object = objects.next();
			JLabel label1 = new JLabel("<html>Object: <font color=#48D031><b>"+object.getName()+"</b></font> instance of <b>"+object.getMetaInfo().getName()+"</b></html>");
			panel.add(label1);
			
			//The properties of this object:
			JLabel label2 = new JLabel("Properties: ");
			panel.add(label2);
			Collection<SchemeObjectProperty> prop = object.getProperties();
			if (prop.size()==0) {
				JLabel label3 = new JLabel("       none");
				panel.add(label3);
			} else {
				for (Iterator<SchemeObjectProperty> properties = prop.iterator(); properties.hasNext(); ) {
					SchemeObjectProperty property = properties.next();
					JLabel label3 = new JLabel("       "+property.getMetaInfo().getName()+": "+property.getValue());
					panel.add(label3);
				}
			}
			
			JLabel label4 = new JLabel("References: ");
			panel.add(label4);
			Collection<SchemeObjectReference> ref = object.getReferences();
			if(ref.size()==0) {
				JLabel label5 = new JLabel("       none");
				panel.add(label5);
			} else {
				//The references of this object:
				for (Iterator<SchemeObjectReference> references = object.getReferences().iterator(); references.hasNext(); ) {
					SchemeObjectReference reference = references.next();
					// paoloc 20060511
					// handling null (optional) references
//					JLabel label6 = new JLabel("       "+reference.getModelReference().getName() + ": "+reference.getValue().getName());
					if (reference.getValue()==null)
					{
						JLabel label6 = new JLabel("       "+reference.getMetaInfo().getName() + ": "+ SchemeObject.MDM_UNSPECIFIED);
						panel.add(label6);
					}
					else
					{
					JLabel label6 = new JLabel("       "+reference.getMetaInfo().getName() + ": "+reference.getValue().getName());
					panel.add(label6);
					}
				}
			}	
		JLabel label7 = new JLabel(" ");
		panel.add(label7);
		}
		return panel;
	}
	
	private Component createTabBar() {
//		/*******************************************************************/
		JToolBar barra = new JToolBar("Tab Bar");
		JButton refresh = new JButton(ResourceFunctions.openImageIcon(this.getClass(), "refresh.gif"));
			refresh.setActionCommand("refresh");		
				refresh.addActionListener(this);
			
		JButton close = new JButton(ResourceFunctions.openImageIcon(this.getClass(), "button_cancel.png"));
			close.setActionCommand("close");
				close.addActionListener(DefineSchemeTool.getIstance().getReportsTab());
				close.addActionListener(this);			
			
		Color bcolor = this.getBackground();
			refresh.setBackground(bcolor);
			close.setBackground(bcolor);
			
			/******************************/
//			close.putClientProperty("report",this);
			/******************************/
			
			
		barra.setLayout(new FlowLayout(FlowLayout.RIGHT));
		barra.setFloatable(false);
			barra.add(close);
			barra.add(refresh);
//		/********************************************************************/
		return barra;
	}

	public boolean isNew() {
		return isNew;
	}

	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();
		if (command.equals("refresh"))
			refresh();	
		else
			removeReportFromMap();
	}

	private void removeReportFromMap() {
//		System.out.println("ModelReport.removeReportFromMap");
		reports.remove(scheme.getName());		
	}

	public Scheme getScheme() {
		return scheme;
		
	}
}





