package org.modelgen.jpf.defineScheme;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.commons.Tool;
import org.modelgen.jpf.defineScheme.Components.ConsoleScroll;
import org.modelgen.jpf.defineScheme.Components.ProblemsScroll;
import org.modelgen.jpf.defineScheme.Components.ReportTabbedPane;
import org.modelgen.jpf.defineScheme.Components.StatusUtility;
import org.modelgen.jpf.defineScheme.Tree.WorkSpaceTree;
import org.modelgen.jpf.defineScheme.Tree.Popup.NewSchemePopupListener;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.CloneSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.DeleteSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.DeleteSchemeObjectAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ExportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ImportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ModifySchemeObjectAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.NewSchemeObjectPopupListener;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.RenameSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.StoreSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.TextReportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.VisualReportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.VoidNewSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.VoidNewSchemeObjectAction;
import org.modelgen.jpf.defineScheme.utility.MenuGenerator;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

public class DefineSchemeTool extends JInternalFrame implements Tool{
	private static final long serialVersionUID = 2589258630687298962L;
		/*L'ordine e l'indentazione rispecchiano la relazione di contenimento del Frame*/
		private JMenuBar menuBar;
			private JMenu file;			
				private JMenuItem saveSchemeMenuItem;
				
			private JMenu scheme;
				private JMenuItem renameSchemeMenuItem;
				private JMenuItem cloneSchemeMenuItem;
				private JMenuItem deleteSchemeMenuItem;
				
			private JMenu schemeObject;
				private JMenuItem deleteSchemeObjectMenuItem;
				private JMenuItem modifySchemeObjectMenuItem;
				
			private JMenu strumenti;
				private JMenuItem importSchemeMenuItem;
				private JMenuItem exportSchemeMenuItem;
				
		private JToolBar toolBar;
			private JButton newSchemeToolButton;
			private JButton renameSchemeToolButton;
			private JButton saveSchemeToolButton;
			private JButton cloneSchemeToolButton;
			private JButton deleteSchemeToolButton;
			private JButton visualReportSchemeToolButton;
			private JButton textReportSchemeToolButton;
			
			
			private JButton importSchemeToolButton;
			private JButton exportSchemeToolButton;
			
			private JButton newSchemeObjectToolButton;	
			private JButton deleteSchemeObjectToolButton;
			private JButton modifySchemeObjectToolButton;
			
		private JSplitPane coreSplit;
			private JScrollPane treeScroll;
			private JSplitPane reportSplit;
				
				public ReportTabbedPane reportsTabs;
				public JTabbedPane infoTabs;

		/*Singleton*/
		private static DefineSchemeTool istance;
		
		private Map<String,Object> applicationObjs;
		
		public DefineSchemeTool() throws PersistenceException{
			super("DefineSchemeTool",true,true,true,true);
			checkConnection();
			initProperties();
			initStructure();
			initComponetsProperties();
			composeStructure();
			/*Consente di applicare il pattern singleton nonostante la classe 
			 * venga istanziata dal CorePlugin*/
			istance = this;
		}

		private void checkConnection() throws PersistenceException{	
			/*Inizializza il PH condiviso e se il server non  avviato provoca 
			 *un eccezione intercettata dalla ToolActivationAction
			 * */
			GeneralManagementFacade.getInstance().initPesistenceHandler();
		}

		/*Singleton*/
		public static DefineSchemeTool getIstance(){
			/*Il controllo condizionale non  necessario:
			 * - il metodo deve ritornare l'internal-frame solo se questa  stata creata
			 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
			 * dal controllo dell'Application Plugin
			 * */
			return istance;
		}

		public void init(Map<String,Object> applicationObjects) {
			this.applicationObjs  = applicationObjects;
		}
	
		
		private void initProperties() {
			ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png");
			setFrameIcon(icon);
			
			setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
			
			Dimension dim = getToolkit().getScreenSize();
				int larghezza = (int) (dim.getWidth()*4/5);
				int altezza = (int) (dim.getHeight()*3/4);
				int bordoSinistro = (int) ((dim.getWidth() - larghezza)/2);
				int bordoSuperiore = (int) ((dim.getHeight()-altezza)/4)-altezza/22;
				setBounds(bordoSinistro,bordoSuperiore,larghezza,altezza);
		}
	
		private void initStructure() {
			menuBar = new JMenuBar();
			toolBar = new JToolBar("ToolBar");
					
			reportsTabs = new ReportTabbedPane();
			infoTabs = new JTabbedPane();
				infoTabs.addTab("Problems",ResourceFunctions.openImageIcon(this.getClass(), "error.png"),ProblemsScroll.getInstance(),"ModelGen application problems");
				infoTabs.addTab("Console",ResourceFunctions.openImageIcon(this.getClass(), "task.png"),ConsoleScroll.getInstance(),"ModelGen actions");
				
			
				reportSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,reportsTabs,infoTabs);
			
			
			treeScroll = WorkSpaceTree.getIstance();
			coreSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,treeScroll,reportSplit);

		}
	
		private void initComponetsProperties() {
			Dimension dim = getToolkit().getScreenSize();
			int altezza = (int) (dim.getHeight()/(2.5));
			reportsTabs.setPreferredSize(new Dimension(0,altezza));
		}
	
		private void composeStructure() {
				
			addMenuBarItems();
			setJMenuBar(menuBar);
				
			addToolBarButtons();
			add(toolBar,BorderLayout.PAGE_START);
			
			add(coreSplit,BorderLayout.CENTER);
		}
	
		private void addMenuBarItems() {
			file = new JMenu("File");	
				/*Notare che in questo modo i modelli vengono caricati solo durante la costruzione
				 * (mentre nei popup menu del woorkspace vengono automaticamente caricati)
				 */
				file.add(MenuGenerator.getIstance().createAvailableModelsMenu());
				file.addSeparator();
				saveSchemeMenuItem = new JMenuItem(StoreSchemeAction.getIstance());
					saveSchemeMenuItem.setText(StoreSchemeAction.ACTION_NAME);
			
			file.add(saveSchemeMenuItem);
			
			scheme = new JMenu("Scheme");
				renameSchemeMenuItem = new JMenuItem(RenameSchemeAction.getIstance());
					renameSchemeMenuItem.setText(RenameSchemeAction.ACTION_NAME);
			    deleteSchemeMenuItem = new JMenuItem(DeleteSchemeAction.getIstance());
			    	deleteSchemeMenuItem.setText(DeleteSchemeAction.ACTION_NAME);
			    cloneSchemeMenuItem = new JMenuItem(CloneSchemeAction.getIstance());
				   	cloneSchemeMenuItem.setText(CloneSchemeAction.ACTION_NAME);
				
			scheme.add(MenuGenerator.getIstance().createReportMenu());
			scheme.addSeparator();
			scheme.add(renameSchemeMenuItem);
			scheme.addSeparator();
			scheme.add(deleteSchemeMenuItem);
			scheme.addSeparator();
			scheme.add(cloneSchemeMenuItem);
//				
			schemeObject = new JMenu("Object");	
				deleteSchemeObjectMenuItem = new JMenuItem(DeleteSchemeObjectAction.getIstance());
					deleteSchemeObjectMenuItem.setText(DeleteSchemeObjectAction.ACTION_NAME);
				modifySchemeObjectMenuItem = new JMenuItem(ModifySchemeObjectAction.getIstance());
					modifySchemeObjectMenuItem.setText(ModifySchemeObjectAction.ACTION_NAME);
				schemeObject.add(modifySchemeObjectMenuItem);
				schemeObject.add(deleteSchemeObjectMenuItem);
	
			strumenti = new JMenu("Tools");
				importSchemeMenuItem = new JMenuItem(ImportSchemeAction.getIstance());
					importSchemeMenuItem.setText(ImportSchemeAction.ACTION_NAME);
				exportSchemeMenuItem= new JMenuItem(ExportSchemeAction.getIstance());
					exportSchemeMenuItem.setText(ExportSchemeAction.ACTION_NAME);
				
				
				
			strumenti.add(importSchemeMenuItem);
			strumenti.add(exportSchemeMenuItem);		
				
			menuBar.add(file);
			menuBar.add(scheme);
			menuBar.add(schemeObject);
			menuBar.add(strumenti);
		}
	
		private void addToolBarButtons() {
			newSchemeToolButton = new JButton(VoidNewSchemeAction.getIstance());
			/************************************************/
			newSchemeToolButton.addMouseListener(new NewSchemePopupListener());
		    /************************************************/
			newSchemeToolButton.putClientProperty("relativeNode","root");
			
			saveSchemeToolButton = new JButton(StoreSchemeAction.getIstance());
			saveSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			cloneSchemeToolButton = new JButton(CloneSchemeAction.getIstance());
			cloneSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			deleteSchemeToolButton = new JButton(DeleteSchemeAction.getIstance());
			deleteSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			renameSchemeToolButton = new JButton(RenameSchemeAction.getIstance());
			renameSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			visualReportSchemeToolButton = new JButton(VisualReportSchemeAction.getIstance());
			visualReportSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			textReportSchemeToolButton = new JButton(TextReportSchemeAction.getIstance());
			textReportSchemeToolButton.putClientProperty("relativeNode","scheme");
			
			newSchemeObjectToolButton = new JButton(VoidNewSchemeObjectAction.getIstance());
			/************************************************/
			newSchemeObjectToolButton.addMouseListener(new NewSchemeObjectPopupListener());
		    /************************************************/
			newSchemeObjectToolButton.putClientProperty("relativeNode","scheme");
			
			deleteSchemeObjectToolButton = new JButton(DeleteSchemeObjectAction.getIstance());
			deleteSchemeObjectToolButton.putClientProperty("relativeNode","object");
			
			modifySchemeObjectToolButton = new JButton(ModifySchemeObjectAction.getIstance());
			modifySchemeObjectToolButton.putClientProperty("relativeNode","object");
			
			importSchemeToolButton = new JButton(ImportSchemeAction.getIstance());
				importSchemeToolButton.putClientProperty("relativeNode","root");
			exportSchemeToolButton = new JButton(ExportSchemeAction.getIstance());
				exportSchemeToolButton.putClientProperty("relativeNode","scheme");

			Dimension sep = new Dimension(90,0);
			
			toolBar.add(newSchemeToolButton);
			toolBar.add(saveSchemeToolButton);
			toolBar.add(cloneSchemeToolButton);
			toolBar.add(renameSchemeToolButton);
			toolBar.add(deleteSchemeToolButton);
				toolBar.addSeparator(sep);
			toolBar.add(visualReportSchemeToolButton);
			toolBar.add(textReportSchemeToolButton);
				toolBar.addSeparator(sep);
			
			toolBar.add(newSchemeObjectToolButton);
			toolBar.add(modifySchemeObjectToolButton);
			toolBar.add(deleteSchemeObjectToolButton);
			
				toolBar.addSeparator(sep);
			
			toolBar.add(importSchemeToolButton);
			toolBar.add(exportSchemeToolButton);
	
			
			StatusUtility statusUtil = new StatusUtility(toolBar);
			statusUtil.enableStatus("root");
			statusUtil.disableOthers();
		}
	
		public String toString(){
			return "DefineSchemeTool";
		}
		public JToolBar getToolBar(){
			return toolBar;
		}

		public ReportTabbedPane getReportsTab() {
			return reportsTabs;
		}

		public Map<String, Object> getInfo() {
			return this.applicationObjs;
		}





}


