package org.modelgen.jpf.defineScheme.Dialog.dialogIMPL;


import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.defineScheme.Components.ProblemsScroll;
import org.modelgen.jpf.defineScheme.Components.SpringUtilities;
import org.modelgen.jpf.defineScheme.Dialog.DSDialog;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ActionUtility;
import org.modelgen.jpf.defineScheme.utility.ErrorTable;
import org.modelgen.jpf.defineScheme.utility.SignatureHandler;

import org.modelgen.signature.application.macroruleapplication.SignatureApplicationUtility;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;


public class AddSchemeObjectDialog extends DSDialog implements ActionListener{
	private static final long serialVersionUID = -8524084161233940594L;

	//Costrutto associato al nuovo SchemeObject
	private Construct construct;

	//Da qui si reperiscono le info sugli oggetti da creare
	private JFormattedTextField textName = null;
	private List<JComboBox> refCombos = new LinkedList<JComboBox>();;
	private List<JCheckBox> propCheckBoxs = new LinkedList<JCheckBox>();;
	private List<JFormattedTextField> propTextFields = new LinkedList<JFormattedTextField>();;

	//Utilit di formattazione
	String leftSpace = "        ";

	public AddSchemeObjectDialog(Construct c) {
		super(ToolBoxFrame.getIstance(), "Add Scheme Object", true,Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2,Toolkit.getDefaultToolkit().getScreenSize().getWidth()/3.5);
		this.construct = c;
		defineStructure(this.getContentPane());
	}


	private void defineStructure(Container contentPane) {
		ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "addConstruct.gif");

		JLabel messageLabel = new JLabel("SchemeObject Adding \n          ",icon,SwingConstants.LEFT);
		contentPane.add(BorderLayout.NORTH,messageLabel);

		JPanel infoPanel = new JPanel();
//		infoPanel.setLayout(new GridLayout(3,1,15,10));
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.setBorder(BorderFactory.createEmptyBorder(0,5,0,5));

		JPanel namePanel = createNamePanel();
		infoPanel.add(namePanel);

		JScrollPane bondPanel = createBondPanel();
		infoPanel.add(bondPanel);

		Collection<Property> properties = construct.getProperties();
		if(!properties.isEmpty()){
			JPanel propPanel = createPropertiesPanel((List<Property>)properties);
//			propPanel.setPreferredSize(namePanel.getSize());
			infoPanel.add(propPanel);
		}
		Collection<Reference> references = construct.getReferences();
		if(!references.isEmpty()){
			infoPanel.add(createReferencesPanel((List<Reference>)references));
		}


		JScrollPane infoScroll = new JScrollPane(infoPanel);
		infoScroll.setBorder(BorderFactory.createEmptyBorder());
		contentPane.add(BorderLayout.CENTER,infoScroll);

		JPanel controlPanel = new JPanel();
		JButton okButton = new JButton("Ok");
		okButton.setActionCommand("commit");
		okButton.addActionListener(this);
		JButton resetButton = new JButton("reset");
		resetButton.setActionCommand("reset");
		resetButton.addActionListener(this);
		controlPanel.add(okButton);
		controlPanel.add(resetButton);
		contentPane.add(BorderLayout.SOUTH,controlPanel);

	}

	private JPanel createReferencesPanel(List<Reference> references) {
		JPanel p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.LEFT));
		p.setBorder(BorderFactory.createTitledBorder("References"));
		Dimension textDimension = new Dimension(150,20);


		JPanel pSpring = new JPanel();
		pSpring.setLayout(new SpringLayout());

		for(Reference ref:references){
			JLabel nameLabel = new JLabel(ref.getName()+"           ",JLabel.LEFT);

			Vector<String> v = new Vector<String>();
			List<SchemeObject> compatibleTargetObjects = new LinkedList<SchemeObject>();


			if(ref.getTargetConstruct()!=null) {
				String targetConstruct = ref.getTargetConstruct().getName();
				List<SchemeObject> schemeObjects = ActionUtility.getScheme().getAbstractConstructs();
				Iterator<SchemeObject> i = schemeObjects.iterator();
				SchemeObject o = null;
				while(i.hasNext()) {
					o = i.next();
					String referenceConstruct = o.getMetaInfo().getName();
					if(targetConstruct.equals(referenceConstruct)){
						compatibleTargetObjects.add(o);
						v.add(o.getName());
					}
				}
			}

//			we should always consider to have a null reference
			SchemeObject nullReferenceObject = null;
			String nullReferenceName = SchemeObject.MDM_UNSPECIFIED;
			compatibleTargetObjects.add(nullReferenceObject);
			v.add(nullReferenceName);

			JComboBox combo = new JComboBox(v);
			combo.setPreferredSize(textDimension);
			/*Consente di reperire il riferimento e i possibili targetObject*/
			combo.putClientProperty("reference",ref);
			combo.putClientProperty("compatibleTargetObjects",compatibleTargetObjects);
			refCombos.add(combo);

			pSpring.add(nameLabel);

			pSpring.add(combo);

		}

		SpringUtilities.makeCompactGrid(pSpring,
				references.size(), 2, 
				6, 6,        
				6, 6);
		p.add(pSpring);
		return p;
	}

	private JPanel createPropertiesPanel(List<Property> properties) {
		JPanel p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.LEFT));
		p.setBorder(BorderFactory.createTitledBorder("Properties"));
		Dimension textDimension = new Dimension(150,20);


		JPanel pSpring = new JPanel();
		pSpring.setLayout(new SpringLayout());

		int a = 0;
		int i = 0;

		for(Property prop:properties){
			if(prop.getType().equals("bit") | prop.getType().equals("boolean")){
				a++;

				JCheckBox box = new JCheckBox();
				box.putClientProperty("property",prop);
				propCheckBoxs.add(box);

				JLabel bLabel = new JLabel(prop.getName());
				bLabel.setLabelFor(box);	
				box.setSelected(false);

				pSpring.add(bLabel);
				pSpring.add(box);
			}
			if(prop.getType().equals("varchar")){	
				i++;
				JLabel nameLabel = new JLabel(prop.getName()+"           ",JLabel.LEFT);
				JFormattedTextField textValue = new JFormattedTextField();
				textValue.setPreferredSize(textDimension);

				/*Consente di reperire la propriet associata al campo testo*/
				textValue.putClientProperty("property",prop);
				propTextFields.add(textValue);

				pSpring.add(nameLabel);

				pSpring.add(textValue);

			}
		}

		SpringUtilities.makeCompactGrid(pSpring,
				a+i, 2, 
				6, 6,        
				6, 6);
		p.add(pSpring);
		return p;
	}

	private JPanel createNamePanel() {
		JPanel p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder("General"));
		p.setLayout(new FlowLayout(FlowLayout.LEFT));


		Dimension textDimension = new Dimension(150,20);

		JLabel nameLabel = new JLabel(leftSpace+"Name        ");
		textName = new JFormattedTextField();
		textName.setPreferredSize(textDimension);

		p.add(nameLabel);
		p.add(textName);

		return p;
	}

	/*
	 * Fabrizio Celli: introduciamo la gestione di vincoli sui costrutti dei modelli
	 */
	private JScrollPane createBondPanel() {

		//vincoli del costrutti
		String bond = this.construct.getUserBonds();

		//creo una JList
		DefaultListModel listModel =  new DefaultListModel();
		//aggiungiamo lo spazio per questioni stilistiche
		listModel.addElement("BONDS Created:");
		listModel.addElement("");
		if(bond!=null && !bond.equalsIgnoreCase("true")){
			List<String> vincoli = SignatureHandler.getInstance().analizeBonds(bond);
			for(String s: vincoli){
				listModel.addElement(s);
				listModel.addElement("        OR");
			}
			if(listModel.get(listModel.getSize()-1).equals("        OR"))
				listModel.remove(listModel.getSize()-1);
		}
		else{
			listModel.addElement("- NO BOND CONSTRAINED -");
		}
		JList lista = new JList(listModel);

		//creo lo ScrollPane
		JScrollPane scrollPane = new JScrollPane(lista, 
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, 
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setBorder(BorderFactory.createTitledBorder("Bonds"));
		scrollPane.setPreferredSize(new Dimension(150,100));

		return scrollPane;
	}

	public void actionPerformed(ActionEvent ae) {
		JButton actionButton = (JButton) ae.getSource();
		String actionCommand = actionButton.getActionCommand();
		if(actionCommand.equals("commit"))
			commit();
		else
			reset();

	}

	private void reset() {
		textName.setText("");
		//Seleziona il primo elemento dei combos(riferimento)
		for(JComboBox refCombo: refCombos){
			refCombo.setSelectedIndex(0);
		}
		//Deseleziona i chech delle propriet
		for(JCheckBox check: propCheckBoxs){
			check.setSelected(false);
		}
		//Deseleziona i chech delle propriet
		for(JFormattedTextField propText: propTextFields){
			propText.setText("");
		}
	}

	@SuppressWarnings("unchecked")
	private void commit() {

		//prendo nome nel costrutto a livello SM
		String superModelConstructName = "";
		Collection<Construct> smConstructs = this.construct.getSuperModelConstructs();
		Iterator<Construct> it = smConstructs.iterator();
		//il SM  inserito alla fine della lista
		while(it.hasNext())
			superModelConstructName = it.next().getName();

		//verifico se i vincoli imposti sulle propriet sono compatibili con quelli del modello
		String constructSignature = this.construct.getSignature();
		String newSignature = "true";
		for(JCheckBox chech:propCheckBoxs){
			Property p = (Property) chech.getClientProperty("property");
			boolean value = chech.isSelected();
			newSignature = SignatureHandler.getInstance().createSignature(newSignature, p.getMetaInfo().getName(), value, superModelConstructName);
		}
		//System.out.println(constructSignature);
		//System.out.println(newSignature);

		//se i vincoli richiesti sono compatibili con quelli del costrutto, creo lo SchemeObject
		if(SignatureApplicationUtility.getInstance().isCompatibile(constructSignature, newSignature)){

			ErrorTable errors = new ErrorTable();

			String objName = textName.getText();
			if (objName.equals("") | objName.contains(" ") | !SchemeObject.isValidName(objName)){
				errors.addError("invalid scheme-object name");
			}
			//Vanno controllate solo le propriet, perch i riferimenti hanno sempre un valore  valido
			//e le prop booleane sono sempre a false
			for(JFormattedTextField propText:propTextFields){
				String propTextValue = propText.getText();
				if (propTextValue.equals("") | propTextValue.contains(" ")){
					errors.addError("invalid property value");
				}
			}

			if (!errors.isEmpty()){
				JOptionPane.showMessageDialog(this,"Errors:\n"+errors);
				ProblemsScroll.getInstance().logWarning("Check data","New SchemeObject", "");
				this.getToolkit().beep();
				reset();
				return;
				// Non viene eseguito this.setVisible(true) quindi il controllo resta sulla Dialog corrente
			}

			/* Aggiungiamo i tre paramentri relativi al nome, le propriet e i targetObject dei riferimenti
			 * */
			Map<String,Object> values= new HashMap<String,Object>();
			values.put("objName",objName);

			Map<Property,String> objPropertyMap = new HashMap<Property,String>();
			for(JCheckBox chech:propCheckBoxs){
				objPropertyMap.put((Property) chech.getClientProperty("property"),(new Boolean(chech.isSelected())).toString());
			}
			for(JFormattedTextField propTextValue:propTextFields){
				objPropertyMap.put((Property) propTextValue.getClientProperty("property"),propTextValue.getText());
			}
			values.put("objProperties",objPropertyMap);

			Map<Reference,SchemeObject> objReferenceMap = new HashMap<Reference,SchemeObject>();
			for(JComboBox combo:refCombos){
				int index= combo.getSelectedIndex();
				SchemeObject targetObject = ((List<SchemeObject>)combo.getClientProperty("compatibleTargetObjects")).get(index);
//				if targetObject is Null then it is a reference to MDM_UNSPECIFIED
				System.out.println("XYZ: " + targetObject);
				if (targetObject!=null)
					objReferenceMap.put((Reference) combo.getClientProperty("reference"),targetObject);
			}
			values.put("objReferences",objReferenceMap);

			super.setValues(values);		
			this.dispose();
		}
		else{
			JOptionPane.showMessageDialog(
					ToolBoxFrame.getIstance(),
					"Properties' signs you have choosen don't match with Construct's bonds",
					"Warning",
					JOptionPane.OK_OPTION);
			ProblemsScroll.getInstance().logWarning("Check properties' signs","New SchemeObject", "");
			this.getToolkit().beep();
		}
	}
}

