package org.modelgen.jpf.defineScheme.Tree.ObjectDescription;

import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineScheme.Tree.WorkSpaceTree;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupListener;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

public class TreeContent extends JTree /*implements TreeSelectionListener*/ {
	private static final long serialVersionUID = 6973697478599435970L;

	private static DefaultMutableTreeNode top;
	
	private static List<Scheme> ssList = null;
	
	private static List<SchemeObject> soLs;
	
	public static TreeContent createsTreeContent() {	
		top = new DefaultMutableTreeNode(new NodeInfo("Schemes",null));
        try {
			createsNodes();
		} catch (Exception e) {
			e.printStackTrace();
			/*Vedi gestione del Panel di error in WorkSpaceTree*/
			return null;
		}
		return new TreeContent(top);
	}

	
	/*Private for Singleton*/
	private TreeContent(DefaultMutableTreeNode topAttrib){
		 super(topAttrib);	
		 
		 /*Init Tree Properties*/
		 getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		 addMouseListener(new PopupListener());
		 
		 setRowHeight(21);
		 setCellRenderer(DSTreeCellRender.getInstance());
		 
		 /*Path nel tree precedente*/
//		 TreePath lastPath = WorkSpaceTree.getLastSelectedPath();
//		 if (lastPath!=null){
//			 /*getNodePath(lastPath): path nel tree dopo l'aggiornamento*/
//			 expandPath(createExpNodePath(lastPath));
//		 }
		 
		 List<TreePath> lastPaths = WorkSpaceTree.getLastExpandedPaths();
		 if (lastPaths!=null){
			 for(TreePath lastPath: lastPaths){ 
				 expandPath(getExpandedNodePath(lastPath));
			 }
		}
		 TreePath lastPath = WorkSpaceTree.getLastSelectedPath();
		 if (lastPath!=null){
			 addSelectionPath(getExpandedNodePath(lastPath));
		 }
	}

	private TreePath getExpandedNodePath(TreePath tp) {
//		System.out.println("-getExpNodePath2");
		TreeNode lastNode = null;
		List<DefaultMutableTreeNode> oldNodes = new LinkedList<DefaultMutableTreeNode>();
			oldNodes.add((DefaultMutableTreeNode) this.getModel().getRoot());
		for(Object newNode: tp.getPath()){
			lastNode = findNode(oldNodes,(DefaultMutableTreeNode)newNode);
		}

		return new TreePath(((DefaultTreeModel)this.getModel()).getPathToRoot(lastNode));
	}
	
	private TreeNode findNode(List<DefaultMutableTreeNode> oldNodes, DefaultMutableTreeNode newNode) {
		DefaultMutableTreeNode foundNode = null; 
		for(DefaultMutableTreeNode n: oldNodes){
			if(n!=null){ //Controllo dovuto alle cancellazioni
				if(((NodeInfo)n.getUserObject()).getDataObject()==((NodeInfo)newNode.getUserObject()).getDataObject()){
					foundNode = n;
					break;
				}
			}
		}
		if(foundNode==null){
			oldNodes.clear();
			return top;
		}
		//Crea la lista dei figli
		LinkedList<DefaultMutableTreeNode> children = new LinkedList<DefaultMutableTreeNode>();
		int i = 0;
		while(i<foundNode.getChildCount()){
			children.add((DefaultMutableTreeNode) foundNode.getChildAt(i));
			i++;
		}

		oldNodes.clear();
		//Sostituisce alla lista dei vecchi nodi quella dei figli del nodo trovato
		oldNodes.addAll(new LinkedList<DefaultMutableTreeNode>(children));
		return foundNode;
	}


//	private TreePath createExpNodePath2(TreePath tp) {
//		TreePath path = null;
//		JTree tree = WorkSpaceTree.getIstance().getTree();
//		String name = ((NodeInfo)((DefaultMutableTreeNode)tp.getLastPathComponent()).getUserObject()).getName();
//		TreeNode node = (TreeNode) tree.getNextMatch(name,0,Position.Bias.Forward).getLastPathComponent();
//		path = new TreePath(((DefaultTreeModel)tree.getModel()).getPathToRoot(node));
//		System.out.println(path.hashCode());
//		return path;
//	}


	private static void createsNodes() throws Exception {
		/*Questo controllo avviene alla prima richiesta da parte del WorkSpaceTree per caricare i modelli
		 * datta base di dati
		 * Nota: nei successivi "refresh" la mappa sar� ormai inizializzata e il controllo false
		 * */
		if (ssList==null) {
			ssList = GeneralManagementFacade.getInstance().getSchemesLoaded();
		}
		
		for(Scheme s: ssList){
			top.add(createsSchemeNode(s));
		}		
	}

	private static DefaultMutableTreeNode createsSchemeNode(Scheme s) {
		DefaultMutableTreeNode sNodo = new DefaultMutableTreeNode(new NodeInfo(s.getName()+": "+s.getMetaInfo().getName(),s));
		soLs = new LinkedList<SchemeObject>();
//		for(SchemeObject obj: (List<SchemeObject>)s.getObjects()){
//			sNodo.add(creaNodoSchemeObject(obj));
//			System.out.println("------------------");
//		}
		
		List<DefaultMutableTreeNode> soNodes = new LinkedList<DefaultMutableTreeNode>();
		for(SchemeObject obj: s.getAbstractConstructs()){
			soNodes.add(createsSchemeObjectNode(obj));
		}
		
		for(DefaultMutableTreeNode soN: reorganize(soNodes)){
			sNodo.add(soN);
		}
		return sNodo;
	}


	private static List<DefaultMutableTreeNode> reorganize(List<DefaultMutableTreeNode> soNodes) {
//		System.out.println("Init Reorganize");
		List<DefaultMutableTreeNode> soNodesTemp = new LinkedList<DefaultMutableTreeNode>(soNodes); 
//		List<DefaultMutableTreeNode> soNodesRemoveble= new LinkedList<DefaultMutableTreeNode>(); 
//	//	Collections.reverse(reverse);
		List<SchemeObject> soLsCopy = new LinkedList<SchemeObject>(soLs);
		Collections.reverse(soLs);
			for(SchemeObject so: soLs){
//				System.out.println(so.getName());
				for(SchemeObjectReference r: (List<SchemeObjectReference>)so.getReferences()){
//					we need to handle null references. In case of a null reference do nothing!
					if (r.getValue()!=null) {
						if(r.getMetaInfo().getType()==Reference.ReferenceType.DESCRIPTION){
							for(SchemeObject soTarget: soLsCopy){
								if(r.getValue().getKey().equals(soTarget.getKey())){
	//								System.out.println(so.getName()+" --> "+soTarget.getName());
									DefaultMutableTreeNode nT = soNodesTemp.get(soLsCopy.indexOf(soTarget));
									/* attenzione se ci sono due padri per lo stesso oggetto non va */
									//TODO: tener conto degli insiemi caratterizzanti nell'interfaccia grafica per avere sempre referenze coerenti.
									
									try {
										DefaultMutableTreeNode nF = soNodesTemp.remove(soLsCopy.indexOf(so));
										
										// modifica riccardop: evita di inserire se stesso e creare cicli
										if(nT != nF) {
											nT.add(nF);
										}
									
									/* l'OutOfBoundsException viene sollevata quando si definisce */
									/* uno scheme object con referenze incoerenti */
									/* todo ancora non fatto. In questo modo il disegno dell'albero */
									/* è casuale sia per troppe o poche referenze specificate */
										
									} catch(IndexOutOfBoundsException ex) {
										
									}
									
	//								soNodesRemoveble.add(n);
									break;
								}
							}
						}
					}
					
				}
			}
		return soNodesTemp;
	}


	private static DefaultMutableTreeNode createsSchemeObjectNode(SchemeObject obj) {
		soLs.add(obj);
		DefaultMutableTreeNode sObjNodo = new DefaultMutableTreeNode(new NodeInfo(obj.getName()+": "+obj.getMetaInfo().getName()/*+" ["+ obj.getKey().toString()+"]"*/,obj));
		
		for(SchemeObjectProperty objProp: (List<SchemeObjectProperty>)obj.getProperties()){
			sObjNodo.add(createsPropertyNode(objProp));
		}
		for(SchemeObjectReference objRef: (List<SchemeObjectReference>)obj.getReferences()){
			sObjNodo.add(createsReferenceNode(objRef));
		}
		return sObjNodo;
	}


	private static DefaultMutableTreeNode createsPropertyNode(SchemeObjectProperty objProp) {
		DefaultMutableTreeNode ObjPropNodo = new DefaultMutableTreeNode(new NodeInfo(objProp.getMetaInfo().getName()+": "+objProp.getMetaInfo().getType()+" ["+ objProp.getValue()+"]",objProp));
		return ObjPropNodo;
	}

	private static DefaultMutableTreeNode createsReferenceNode(SchemeObjectReference objRef) {
		String target;
		target = objRef.getValue()!=null?objRef.getValue().getName():"null";
		DefaultMutableTreeNode ObjRefNodo = new DefaultMutableTreeNode(new NodeInfo(objRef.getMetaInfo().getName()+": "+objRef.getMetaInfo().getTargetConstruct().getName()+ "["+target+"]",objRef));
//		System.out.println(objRef.getModelReference().getTypeAsString());
		return ObjRefNodo;
	}

	/*Serve a trovare il treepath del nodo da espandere nel nuovo tree, creato con il refresh*/
//	public TreePath createExpNodePath(TreePath path){
//		TreePath modelPath = null;
//		
//		int count = path.getPathCount();
//		if (count == 1){
//			modelPath = new TreePath(top.getPath());
//		}
//		if (count == 2){
//			modelPath = getSchemePath(path);
//		}
//		if (count == 3){
//			modelPath = getSOPath(getSchemePath(path.getParentPath()),(DefaultMutableTreeNode) path.getLastPathComponent());
//		}
//		return modelPath;
//	}
	
	
//	public TreePath createExpNodePath2(TreePath path){
//		TreePath buildingPath = null;
//
//		buildingPath = new TreePath(top.getPath());
//		
//		Object[] objs = path.getPath();
//		int i;
//		for(i=1; i<objs.length;i++){
//			DefaultMutableTreeNode node = (DefaultMutableTreeNode) objs[i];
//			NodeInfo info = (NodeInfo) node.getUserObject();
//			Object obj = info.getDataObject();
////		    if(obj instanceof Scheme){
////		    	System.out.println("Scheme");
////		    	buildingPath = buildingPath.pathByAddingChild(new DefaultMutableTreeNode(obj));
////		    }else if(obj instanceof SchemeObject){
////		    	System.out.println("SchemeObject");
////		    }else if(obj instanceof SchemeObjectProperty){
////		    	System.out.println("SchemeObjectProperty");
////		    }else if(obj instanceof SchemeObjectReference){
////		    	System.out.println("SchemeObjectReference");
////		    }
//			buildingPath = buildingPath.pathByAddingChild(new DefaultMutableTreeNode(obj));
//		}
//		return buildingPath;
//	}
	
	
	
//	private TreePath getSchemePath(TreePath path){
//		DefaultMutableTreeNode last = (DefaultMutableTreeNode) path.getLastPathComponent();
//		Scheme scheme = ((NodeInfo)last.getUserObject()).getScheme();
//		
//		TreePath schemePath = null;
//		List<DefaultMutableTreeNode> children = (List<DefaultMutableTreeNode>)Collections.list(top.children());
//		
////		System.out.println(children);
//		for(DefaultMutableTreeNode node: children){
//			Scheme s = ((NodeInfo)((DefaultMutableTreeNode)node).getUserObject()).getScheme();
//			if(s.getName().equals(scheme.getName())){
//				schemePath = (new TreePath(top)).pathByAddingChild(node);
//			}
//		}
////		System.out.println(modelPath);
//		return schemePath;
//	}
	
//	private TreePath getSOPath(TreePath schemePath, DefaultMutableTreeNode oldSONode) {
//		TreePath schemeObjectPath = null;	
//
//		SchemeObject oldSO = ((NodeInfo)oldSONode.getUserObject()).getSchemeObject();
//		
//		DefaultMutableTreeNode last = (DefaultMutableTreeNode) schemePath.getLastPathComponent();
//		List<DefaultMutableTreeNode> children = (List<DefaultMutableTreeNode>) Collections.list(last.children());
//		
////		System.out.println(children);
//		for(DefaultMutableTreeNode node: children){
//			SchemeObject obj = ((NodeInfo)((DefaultMutableTreeNode)node).getUserObject()).getSchemeObject();
//			if(obj.getName().equals(oldSO.getName())){
//				schemeObjectPath = schemePath.pathByAddingChild(node);
//			}
//		}
////		System.out.println("SO path; " + schemeObjectPath);
//		
//		/*Controllo se il refresh segue l'eliminazione di un costrutto, e quindi il nodo corrispondente al
//		 *costrutto non � presente*/
//		if (schemeObjectPath==null){
//			/*Cos� espandiamo almeno il modello associato al costrutto*/
//			schemeObjectPath = schemePath;
//		}
//		return schemeObjectPath;
//	}
	
//    public void notificaAggiornamentoSelezione() {
//        this.fireValueChanged(null);
//    }
}


