package org.modelgen.jpf.defineScheme.Tree.Popup;

import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineScheme.Tree.ObjectDescription.NodeInfo;
import org.modelgen.jpf.defineScheme.Tree.ObjectDescription.TreeContent;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.DeleteSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.DeleteSchemeObjectAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ExportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ImportSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.ModifySchemeObjectAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.RenameSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.StoreSchemeAction;
import org.modelgen.jpf.defineScheme.Tree.Popup.PopupActions.VoidNewSchemeObjectAction;
import org.modelgen.jpf.defineScheme.utility.MenuGenerator;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Collections;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.tree.DefaultMutableTreeNode;


public class PopupListener extends MouseAdapter {
	MouseEvent evt = null;
	
    public void mousePressed(MouseEvent e) {
    	evt = e;
        showPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
    	evt = e;
        showPopup(e);
    }

    private void showPopup(MouseEvent e) {
    /*   public boolean isPopupTrigger() Returns whether or not this mouse event is the popup menu trigger event 
    *   for the platform. 
    *   
    *   Note: Popup menus are triggered differently on different systems. Therefore, isPopupTrigger should be 
    *   checked in both mousePressed and mouseReleased for proper cross-platform functionality. 
    *   */   	
    	JPopupMenu popup = null;
        if (e.isPopupTrigger()) {
        	popup = createPopup(e);
            popup.show(e.getComponent(),e.getX(), e.getY());
        }
    }


	private JPopupMenu createPopup(MouseEvent e) {
		JPopupMenu popup = new JPopupMenu();
		TreeContent tree = (TreeContent)e.getSource();
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();		
		if (node == null){
			JMenuItem menuItem = new JMenuItem("Select a component");
		    popup.add(menuItem);
		    return popup;
		}

		if(node.isRoot()){
			return rootPopupMenu(node);
		}
		NodeInfo nodeInfo = (NodeInfo)node.getUserObject();
		Object data = nodeInfo.getDataObject();
		
		/*Processa in base al parametro*/
		if (data instanceof Scheme) {
			return dataObjectPopup((Scheme) data);
		}
		if (data instanceof SchemeObject) {
			return dataObjectPopup((SchemeObject) data);	
		}
		
		return popup;
	}

	private JPopupMenu dataObjectPopup(SchemeObject object) {
		JPopupMenu pmenu = new JPopupMenu();
		
		JMenuItem modifySchemeItem = new JMenuItem(ModifySchemeObjectAction.getIstance());
			modifySchemeItem.setText(ModifySchemeObjectAction.ACTION_NAME);
		
			pmenu.add(modifySchemeItem);
		
			pmenu.addSeparator();
	
		JMenuItem deleteSchemeItem = new JMenuItem(DeleteSchemeObjectAction.getIstance());
			deleteSchemeItem.setText(DeleteSchemeObjectAction.ACTION_NAME);
			
			pmenu.add(deleteSchemeItem);
		return pmenu;
	}

	private JPopupMenu dataObjectPopup(Scheme s) {
		JPopupMenu pmenu = new JPopupMenu();
		
		pmenu.add(MenuGenerator.getIstance().createReportMenu());
		pmenu.addSeparator();
		
		JMenuItem exportSchemeItem = new JMenuItem(ExportSchemeAction.getIstance());
			exportSchemeItem.setText(ExportSchemeAction.ACTION_NAME);
			
			pmenu.add(exportSchemeItem);
			pmenu.addSeparator();
		

		if (!s.getAvailableConstructs().isEmpty()){
			JMenu addSchemeObjectMenu = MenuGenerator.getIstance().createAvailableSchemeObjectsMenu(s);
				addSchemeObjectMenu.setText(VoidNewSchemeObjectAction.ACTION_NAME);
			pmenu.add(addSchemeObjectMenu);
		}
		
		JMenuItem renameSchemeItem = new JMenuItem(RenameSchemeAction.getIstance());
			renameSchemeItem.setText(RenameSchemeAction.ACTION_NAME);
		
			pmenu.add(renameSchemeItem);

		
		JMenuItem deleteS = new JMenuItem(DeleteSchemeAction.getIstance());
			deleteS.setText(DeleteSchemeAction.ACTION_NAME);
		JMenuItem saveS = new JMenuItem(StoreSchemeAction.getIstance());
			saveS.setText(StoreSchemeAction.ACTION_NAME);
			pmenu.add(saveS);
			pmenu.addSeparator();
			pmenu.add(deleteS);

		
		return pmenu;
	}

	private JPopupMenu rootPopupMenu(DefaultMutableTreeNode node) {
		JPopupMenu pmenu = new JPopupMenu();
		
		JMenuItem importSchemeItem = new JMenuItem(ImportSchemeAction.getIstance());
			importSchemeItem.setText(ImportSchemeAction.ACTION_NAME);
			
			pmenu.add(importSchemeItem);
			pmenu.addSeparator();
		
		Collection<Model> availableModels=Collections.emptyList();
		try {
			availableModels = GeneralManagementFacade.getInstance().getModelsLoaded();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (!availableModels.isEmpty()){
			JMenu addSchemeMenu = MenuGenerator.getIstance().createAvailableModelsMenu();

				pmenu.add(addSchemeMenu);
		}
		
		return pmenu;
	}
}
