package org.modelgen.jpf.defineScheme.utility;

import org.modelgen.signature.util.ToolFormuleAsStringhe;

import java.util.LinkedList;
import java.util.List;


/**
 * Singleton that provides methods to manage signatures (of a Construct)
 * @author Fabrizio Celli
 */
public class SignatureHandler {

	//singleton
	private static SignatureHandler instance;

	public static SignatureHandler getInstance(){
		if(instance == null)
			instance = new SignatureHandler();
		return instance;
	}
	
	/**
	 * Analizza una stringa del tipo
	 * 
	 * 		VINCOLO1 OR VINCOLO2 OR .. OR VINCOLON
	 * 
	 * creando una lista di vincoli. 
	 * Per ipotesi, "bond"  non nullo
	 */
	public List<String> analizeBonds(String bond){
		bond = ToolFormuleAsStringhe.removeAllParenthesis(bond);
		List<String> vincoli = new LinkedList<String>();
		String s = bond;
		int orPos = s.indexOf(" OR ");
		while(orPos!=-1){
			vincoli.add(s.substring(0, orPos));
			//se abbiamo trovato OR, sicuramente la stringa ha almeno altri 4 cratteri
			s = s.substring(orPos+4);
			orPos = s.indexOf(" OR ");
		}
		vincoli.add(s);
		return vincoli;
	}


	/**
	 * Data una stringa (signature esistente), la estende con una nuova propriet,
	 * mantenendo la seguente sintassi:
	 * -	il nome della propriet  a livello SM
	 * -	il nome della propriet  preceduto da quello SM del costrutto pi "."
	 * -	per le negazioni:	(NOT costr.prop)
	 * -	per le estensioni:	(signature AND newProp)
	 */
	public String createSignature(String sign, String propertySM, boolean value, String constrSM){
		//determina il segno della propriet
		if(!value)
			propertySM = "(NOT "+constrSM+"."+propertySM+")";
		else
			propertySM = constrSM+"."+propertySM;
		if(sign==null || sign.length()==0 ||sign.equalsIgnoreCase("true")){
			sign = propertySM;
		} else
			sign = "("+sign + " AND "+propertySM+")";
		return sign;
	}

}
