/*
 * Classe che gestisce le funzioni di Skolem 
 */
package org.modelgen.jpf.defineSkolem;

import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.commons.Tool;
import org.modelgen.jpf.defineSkolem.exportXml.PanelExport;
import org.modelgen.jpf.defineSkolem.importXml.PanelImport;
import org.modelgen.jpf.defineSkolem.newSkolem.PanelNewSkolem;
import org.modelgen.jpf.defineSkolem.report.PanelPrintSuperModel;
import org.modelgen.jpf.defineSkolem.table.SkolemTable;
import org.modelgen.jpf.defineSkolem.table.filter.FilterTable;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.util.Collection;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * @author  gabriele  Classe che gestisce le funzioni di Skolem
 */
public class ManagementSkolemTool extends JInternalFrame implements Tool {
	private static final long serialVersionUID = 4933945007817074751L;	
	/* Singleton */
	private static ManagementSkolemTool istance;
	/* Mappa condivisa per l'implementazione dell'interfaccia Tool */
	/**
	 * @uml.property  name="applicationObjs"
	 * @uml.associationEnd  qualifier="key:java.lang.Object java.lang.Object"
	 */
	private Map<String,Object> applicationObjs;
	// Toolbar
	private ToolBarSkolem toolBar;
	// Menu
	private MenuSkolem menu;
	// Skolem table
	private SkolemTable skolemTable;
	// The panel to create a new Skolem function
	private PanelNewSkolem pnlNewSkolem;
	// The panel to import a Skolem function
	private PanelImport pnlImport;
	// The panel to export a Skolem function
	private PanelExport pnlExport;
	// The panel to print a super model
	private PanelPrintSuperModel pnlPrintSuperModel;
	// The main panel
	private JPanel pnlGeneral;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Filter
	private FilterTable filter;
	
	/**
	 * Costruttore della classe
	 * @throws Exception 
	 * @throws ClassNotFoundException 
	 */
	public ManagementSkolemTool() throws ClassNotFoundException, Exception {
		super("DefineSkolemTool",true,true,true,true);
		// Controllo la connessione
		checkConnection();
		// Create main panel
		this.pnlGeneral = new JPanel();
		// Set over this the layout manager
		this.setLayout();
		// Inizializzo i componenti
		// Creates toolbar e menu
		this.toolBar = ToolBarSkolem.getInstance(this);
		this.menu = MenuSkolem.getInstance(this);
		this.createShow();
		initProperties();
		/* Consente di applicare il pattern singleton nonostante la classe 
		 * venga istanziata dal CorePlugin */
		istance = this;
	}

	/**
	 * Controlla la connessione verso la base di dati
	 * @throws Exception
	 */
	private void checkConnection() throws Exception {	
		/* Inizializza il PH condiviso e se il server non  avviato provoca 
		 * un eccezione intercettata dalla ToolActivationAction */
		GeneralManagementFacade.getInstance().initPesistenceHandler();
	}
	
	/**
	 * Inizializza la finestra
	 */
	private void initProperties() {
		// L'icona
		ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png");
		setFrameIcon(icon);
		// Alla chiusura
		setDefaultCloseOperation(JInternalFrame.EXIT_ON_CLOSE);			
		Toolkit kit = Toolkit.getDefaultToolkit();
		/* Take x and y coordinates of the screnn */
		int y = (int)kit.getScreenSize().getHeight();
		int x = (int)kit.getScreenSize().getWidth();
		/* Sets the dimension of the internal frame */
		this.setSize(x - 10, y - 150);
		setVisible(true);
	}
	
	/* Singleton */
	/**
	 * @return  istance
	 * @uml.property  name="istance"
	 */
	public static ManagementSkolemTool getIstance(){
		/* Il controllo condizionale non  necessario:
		 * - il metodo deve ritornare l'internal-frame solo se questa  stata creata
		 * dal CorePlugin; in questo modo si evita di poter creare il frame fuori
		 * dal controllo dell'Application Plugin */
		return istance;
	}	
	
	/**
	 * Metodo toString
	 */
	public String toString(){
		return "ManagementSkolemTool";
	}

	/**
	 * Implementazione per l'interfaccia Tool
	 */
	public void init(Map<String,Object> applicationObjects) {
		this.applicationObjs  = applicationObjects;
	}
	
	/**
	 * Implementazione per l'interfaccia Tool
	 */
	public Map<String, Object> getInfo() {
		return this.applicationObjs;
	}
	
	/**
	 * Sets the general layout
	 */
	private void setLayout() {
		// Setto il layout
		this.pnlGeneral.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		this.pnlGeneral.setBorder(new javax.swing.border.TitledBorder(
				"All Skolem functions"));
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 1;
	}
	
	/**
	 * Creates and shows the gui
	 */
	private void createShow() {
		// Sets menu bar
		this.setJMenuBar(this.menu);
		// Sets tool bar
		this.add(this.toolBar, BorderLayout.PAGE_START);
		// Create table
		this.createPanelTable();
		// Add to container
		JScrollPane pane = new JScrollPane(this.pnlGeneral);
		this.getContentPane().add(pane);
		// Shows gui
		this.setVisible(true);
		this.pack();
	}
	
	/**
	 * Create a table with all Skolem functions
	 */
	private void createPanelTable() {
		this.skolemTable = SkolemTable.getInstance();
		this.skolemTable.setManagementSkolemTool(this);
		this.skolemTable.createTable();
		try {
			this.skolemTable.addFunction(GeneralManagementFacade.getInstance().getSkolemFunctions());
		} catch (Exception e) {}
		// Filter
		this.filter = FilterTable.getInstance();
		this.filter.createPanel();
		// Add table to main panel at (0,0)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.generalLayout.anchor = GridBagConstraints.WEST;
		this.pnlGeneral.add(this.filter, this.generalLayout);
		// Add table to main panel at (0,1)
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 1;
		this.generalLayout.fill = GridBagConstraints.BOTH;
		this.generalLayout.weighty = 150;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.pnlGeneral.add(new JScrollPane(this.skolemTable.getTable(),
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), this.generalLayout);
	}
	
	/**
	 * Add a new Skolem panel to general panel
	 */
	public void addPanelNewSkolem() {
		// Disable buttons insert
		this.menu.mniInsert.setEnabled(false);
		this.toolBar.btnInsert.setEnabled(false);
		this.skolemTable.enableBtnInsert(false);
		// Create the panel
		this.pnlNewSkolem = PanelNewSkolem.getInstance(this);
		// Set layout
		pnlNewSkolem.setLayout();
		// Create and show
		pnlNewSkolem.createShowPanel();
		// Add to panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 2;
		this.pnlGeneral.add(new JScrollPane(this.pnlNewSkolem), this.generalLayout);
		this.updateUI();
	}
	
	/**
	 * Remove the panel new Skolem from the general panel
	 */
	public void removePanelNewSkolem() {
		// Remove all from new Skolem function
		this.pnlNewSkolem.removeAll();
		// Remove
		this.pnlGeneral.remove(2);
		// Enable buttons insert
		this.menu.mniInsert.setEnabled(true);
		this.toolBar.btnInsert.setEnabled(true);
		this.skolemTable.enableBtnInsert(true);
		// Update UI
		this.pnlGeneral.updateUI();
	}
	
	/**
	 * Add an import Skolem panel to general panel
	 */
	protected void addPanelImport() {
		// Create the panel
		this.pnlImport = PanelImport.getInstance();
		// Opern file choose
		Collection<SkolemFunction> newSkolem = null;
		newSkolem = this.pnlImport.openFileChooser();
		this.skolemTable.addFunction(newSkolem);
		this.pnlGeneral.updateUI();
	}
	
	/**
	 * Add an export Skolem panel to general panel
	 */
	protected void addPanelExport() {
		// Create the panel
		this.pnlExport = PanelExport.getInstance();
		// Launch
		pnlExport.openFileChooser();
	}
	
	/**
	 * Add a pritn super model panel to general panel
	 */
	protected void addPanelPrintSuperModel() {
		// Create the panel
		this.pnlPrintSuperModel = PanelPrintSuperModel.getInstance();
		// Set layout
		pnlPrintSuperModel.createReport();
	}
	
	/**
	 * Remove the panel print super model from the general panel
	 */
	public void removePanelPrintSuperModel() {
		// Remove all from new Skolem function
		this.pnlNewSkolem.removeAll();
		// Remove
		this.pnlGeneral.remove(1);
		// Update UI
		this.pnlGeneral.updateUI();
	}
	
	/**
	 * Close plugin
	 */
	protected void close() {
		this.doDefaultCloseAction();
	}
}