/* 
 * Menu for this plug in
 */
package org.modelgen.jpf.defineSkolem;

import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.table.SkolemTable;
import org.modelgen.jpf.defineSkolem.util.FactoryIcon;

import java.awt.Toolkit;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;

/**
 * @author  gabriele  Menu for this plug in
 */
class MenuSkolem extends JMenuBar {
	private static final long serialVersionUID = -7926265467788153104L;
	// Singleton instance
	private static MenuSkolem instance;
	// Facade
	private GeneralManagementFacade facade;
	// Table
	private SkolemTable table;
	// Listener
	private ActionListener lstMenu;
	// Main window
	private ManagementSkolemTool mngSkolem;
	// Menu objet
	private JMenu mnuFile;
	// Menu objet
	private JMenu mnuOperation;
	/* Menu item for menu file */
	protected JMenuItem mniInsert;
	/* Menu item for menu file */
	protected JMenuItem mniDelete;
	/* Menu item for menu file */
	protected JMenuItem mniExit;
	protected JMenuItem mniImport;
	protected JMenuItem mniExport;
	protected JMenuItem mniPrintSuperModel;
	protected JMenuItem mniVersion;
	protected JMenuItem mniAutor;
	protected JMenuItem mniHelp;
	
	/**
	 * Class constructor
	 * @param ManagementSkolemTool The window where this menu is used
	 */
	private MenuSkolem(ManagementSkolemTool mngSkolem) {
		this.mngSkolem = mngSkolem;
		// Facade 
		this.facade = GeneralManagementFacade.getInstance();
		this.table = SkolemTable.getInstance();
		// Creates listener
		this.lstMenu = new ActionListenerMenu(this, this.mngSkolem);
		/* Menu File */
		this.mnuFile = new JMenu("File");
		// Creates menu item and add to menu
		this.mniInsert = new JMenuItem("New Skolem function");
		this.mniInsert.setAccelerator(KeyStroke.getKeyStroke(
				'N', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniInsert.setIcon(FactoryIcon.createInsertIcon());
		this.mniInsert.addActionListener(this.lstMenu);
		this.mniDelete = new JMenuItem("Delete");
		this.mniDelete.setAccelerator(KeyStroke.getKeyStroke(
				'D', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniDelete.setIcon(FactoryIcon.createDeleteIcon());
		this.mniDelete.addActionListener(this.lstMenu);
		this.mniExit = new JMenuItem("Exit");
		this.mniExit.setAccelerator(KeyStroke.getKeyStroke(
				'Q', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniExit.setIcon(FactoryIcon.createCloseIcon());
		this.mniExit.addActionListener(this.lstMenu);
		this.mnuFile.add(this.mniInsert);
		this.mnuFile.add(this.mniDelete);
		this.mnuFile.add(new JSeparator());
		this.mnuFile.add(this.mniExit);
		/* Operation menu */
		this.mnuOperation = new JMenu("Operation");
		// Creates menu item and add to menu
		this.mniImport = new JMenuItem("Import");
		this.mniImport.setAccelerator(KeyStroke.getKeyStroke(
				'I', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniImport.setIcon(FactoryIcon.createImportIcon());
		this.mniImport.addActionListener(this.lstMenu);
		this.mniExport = new JMenuItem("Export");
		this.mniExport.setAccelerator(KeyStroke.getKeyStroke(
				'E', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniExport.setIcon(FactoryIcon.createExportIcon());
		this.mniExport.addActionListener(this.lstMenu);
		this.mniExport.addActionListener(this.lstMenu);
		this.mniPrintSuperModel = new JMenuItem("Print Super Model");
		this.mniPrintSuperModel.setAccelerator(KeyStroke.getKeyStroke(
				'P', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		this.mniPrintSuperModel.setIcon(FactoryIcon.createPrintSuperModelIcon());
		this.mniPrintSuperModel.addActionListener(this.lstMenu);
		this.mnuOperation.add(this.mniImport);
		this.mnuOperation.add(this.mniExport);
		this.mnuOperation.add(new JSeparator());
		this.mnuOperation.add(this.mniPrintSuperModel);
		// Add to main
		this.add(this.mnuFile);
		this.add(this.mnuOperation);
	}
	
	/**
	 * Return an instance of the class
	 * @param mngSkolem The window where this menu is used
	 * @return MenuSkolem
	 */
	public static MenuSkolem getInstance(ManagementSkolemTool mngSkolem) {
		if(instance == null)
			instance = new MenuSkolem(mngSkolem);
		return instance;
	}
	
	/**
	 * Delete Skolem function
	 */
	protected void deleteSkolem() {
		int[] row = this.table.getTable().getSelectedRows();
		if(row.length == 0) {
			JOptionPane.showMessageDialog(new JPanel(), "No selected rows.", 
					"Delete Skolem", JOptionPane.ERROR_MESSAGE);
		} else {
			int value = JOptionPane.showConfirmDialog(this, "Are you sure to delete all " + 
					"selected rows?", "Delete Skolem", JOptionPane.YES_NO_OPTION);
			if(value ==  JOptionPane.YES_OPTION) {
				String function = null;
				String construct = null;
				for(int i = 0; i <row.length ; i++) {
					function = (String)this.table.getTable().getValueAt(row[i], 0);
					construct = (String)this.table.getTable().getValueAt(row[i], 1);
					try {
						this.facade.deleteSkolemFunction(construct, function);
					} catch (Exception e) {}
				}
				JOptionPane.showMessageDialog(this, "Delete successfully", "Delete Skolem",
						JOptionPane.INFORMATION_MESSAGE);
				try {
					this.table.reload();
				} catch (ClassNotFoundException e) {} catch (Exception e) {}
			}
		}
	}
}