/*
 * Tool bar for this plug in 
 */
package org.modelgen.jpf.defineSkolem;

import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.table.SkolemTable;
import org.modelgen.jpf.defineSkolem.util.FactoryIcon;

import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;

/**
 * @author  gabriele  Tool bar for this plug in
 */
public class ToolBarSkolem extends JToolBar {
	private static final long serialVersionUID = 7544650067428341335L;
	// Instance
	private static ToolBarSkolem instance;
	// Facade
	private GeneralManagementFacade facade;
	// Manin window
	private ManagementSkolemTool mngSkolem;
	// Listener
	private ActionListener lstAction;
	// Buttons
	protected JButton btnInsert;
	// Buttons
	protected JButton btnDelete;
	// Buttons
	protected JButton btnImport;
	// Buttons
	protected JButton btnExport;
	// Buttons
	protected JButton btnPrintSuperModel;
	// Table
	private SkolemTable skolemTable;
	
	/**
	 * Class constructor
	 * @param mngSkolem
	 */
	private ToolBarSkolem(ManagementSkolemTool mngSkolem) {
		super("ToolBarSkolem", JToolBar.HORIZONTAL);
		this.mngSkolem = mngSkolem;
		// Facade 
		this.facade = GeneralManagementFacade.getInstance();
		// Creates table
		this.skolemTable = SkolemTable.getInstance();
		// Listener
		this.lstAction = new ActionListenerToolBar(this, this.mngSkolem);
		// Creates component
		this.btnInsert = new JButton(FactoryIcon.createInsertIcon());
		this.btnInsert.setToolTipText("Create new Skolem function");
		this.btnInsert.addActionListener(this.lstAction);
		this.btnDelete = new JButton(FactoryIcon.createDeleteIcon());
		this.btnDelete.setToolTipText("Delete Skolem function");
		this.btnDelete.addActionListener(this.lstAction);
		this.btnImport = new JButton(FactoryIcon.createImportIcon());
		this.btnImport.setToolTipText("Import Skolem function");
		this.btnImport.addActionListener(this.lstAction);
		this.btnExport = new JButton(FactoryIcon.createExportIcon());
		this.btnExport.setToolTipText("Export Skolem function");
		this.btnExport.addActionListener(this.lstAction);
		this.btnPrintSuperModel = new JButton(FactoryIcon.createPrintSuperModelIcon());
		this.btnPrintSuperModel.setToolTipText("Print super model");
		this.btnPrintSuperModel.addActionListener(this.lstAction);
		// Add
		this.add(this.btnInsert);
		this.add(this.btnDelete);
		this.add(this.btnImport);
		this.add(this.btnExport);
		this.add(this.btnPrintSuperModel);
	}
	
	/**
	 * Return an instance of the class
	 * @return ToolBarSkolem
	 */
	public static ToolBarSkolem getInstance(ManagementSkolemTool mngSkolem) {
		if(instance == null)
			instance = new ToolBarSkolem(mngSkolem);
		return instance;
	}
	
	/**
	 * Delete Skolem function
	 */
	protected void deleteSkolem() {
		int[] row = this.skolemTable.getTable().getSelectedRows();
		if(row.length == 0) {
			JOptionPane.showMessageDialog(new JPanel(), "No selected rows.", 
					"Delete Skolem", JOptionPane.ERROR_MESSAGE);
		} else {
			int value = JOptionPane.showConfirmDialog(this, "Are you sure to delete all " + 
					"selected rows?", "Delete Skolem", JOptionPane.YES_NO_OPTION);
			if(value ==  JOptionPane.YES_OPTION) {
				String function = null;
				String construct = null;
				for(int i = 0; i <row.length ; i++) {
					function = (String)this.skolemTable.getTable().getValueAt(row[i], 0);
					construct = (String)this.skolemTable.getTable().getValueAt(row[i], 1);
					try {
						this.facade.deleteSkolemFunction(construct, function);
					} catch (Exception e) {}
				}
				JOptionPane.showMessageDialog(this, "Delete successfully", "Delete Skolem",
						JOptionPane.INFORMATION_MESSAGE);
				try {
					this.skolemTable.reload();
				} catch (ClassNotFoundException e) {} catch (Exception e) {}
			}
		}
	}
}