/*
 * This class implements a panel to export into xml file a Skolem function 
 */
package org.modelgen.jpf.defineSkolem.exportXml;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.util.FileFilter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.xml.bind.JAXBException;

/**
 * @author  gabriele  This class implements a panel to export into xml file a Skolem function
 */
public class PanelExport extends JPanel {
	private static final long serialVersionUID = -5497356560355580002L;
	// Singleton
	private static PanelExport instance;
	// Facade
	private GeneralManagementFacade facade;
	// File chooser
	private JFileChooser fileChooser;
	// File filter
	private FileFilter filter;
	// Xml function
	private String xmlFunction;
	
	/**
	 * Class constructor
	 */
	private PanelExport() {
		// Creates facade
		this.facade = GeneralManagementFacade.getInstance();
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelExport
	 * @uml.property  name="instance"
	 */
	public static PanelExport getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelExport();
		return instance;
	}
	
	/**
	 * Load file chooser to export
	 */
	public void openFileChooser() {
		// Take a xml function
		try {
			this.xmlFunction = this.facade.getXmlSkolemFunctions();
		} catch (PersistenceException e1) {
			JOptionPane.showMessageDialog(this,
        			"Can't export Skolem function: error loading skolem from db.",
        			"Export Skolem",JOptionPane.ERROR_MESSAGE);
		} catch (JAXBException e1) {
			JOptionPane.showMessageDialog(this,
        			"Can't export Skolem function: error exporting to xml file (JAXB).",
        			"Export Skolem",JOptionPane.ERROR_MESSAGE);
		}
		// Creates file chooser
		this.fileChooser = new JFileChooser("modelgenDATA/xmlRepository/Skolem");
		// Creates filter
	    this.filter = FileFilter.getInstance();
	    // Parameter for filter
	    this.filter.setExstension(FileFilter.XML, FileFilter.DESCRIPTION_XML);
	    this.fileChooser.setFileFilter(this.filter);
	    int returnValue = this.fileChooser.showSaveDialog(this);
	    if (returnValue == JFileChooser.APPROVE_OPTION) {
	    	/* Take file */
        	File file = this.fileChooser.getSelectedFile();
        	/* To write the file */
        	FileWriter writer = null;
			try {
				writer = new FileWriter(file,true);
				writer.write(this.xmlFunction);
	        	writer.flush();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this,
	        			"Can't export Skolem function",
	        			"Export Skolem",JOptionPane.ERROR_MESSAGE);
			}
        	JOptionPane.showMessageDialog(this,
        			"Set of Skolem functions successfully exported",
        			"Export Skolem",JOptionPane.INFORMATION_MESSAGE);
        	
	    }
	}
}