/*
 * Implements a panel to import an xml file 
 */
package org.modelgen.jpf.defineSkolem.importXml;

import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.util.FileFilter;

import java.io.File;
import java.util.Collection;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * @author  gabriele  Implements a panel to import an xml file
 */
public class PanelImport extends JPanel {
	private static final long serialVersionUID = -7556505919059395115L;
	// Singleton
	private static PanelImport instance;
	// Facade
	private GeneralManagementFacade facade;
	// The file chooser
	private JFileChooser fileChooser;
	// Filter class
	private FileFilter filter;
	// The file to import
	private File file2import;
	
	/**
	 * Constructor of the class
	 */
	private PanelImport() {
		// Initialize facade
		this.facade = GeneralManagementFacade.getInstance();
	}
	
	/**
	 * Return an instance of the class
	 * @return  PanelImport
	 * @uml.property  name="instance"
	 */
	public static PanelImport getInstance() {
		if(instance == null)
			instance = new PanelImport();
		return instance;
	}
	
	/**
	 * Open file chooser
	 */
	public Collection<SkolemFunction> openFileChooser() {
		// File chooser
		String xmlRepositoryDir = ModelgenConfigurationManager.getIstance().getProperty("xmlRepositoryPath", System.getProperty("user.home"), "paths");
		this.fileChooser = new JFileChooser(xmlRepositoryDir);
//		this.fileChooser = new JFileChooser("modelgenDATA/xmlRepository/Skolem");
		// Creates filter
	    this.filter = FileFilter.getInstance();
	    // Parameter for filter
	    this.filter.setExstension(FileFilter.XML, FileFilter.DESCRIPTION_XML);
	    this.fileChooser.setFileFilter(this.filter);
	    // Return value
	    int returnVal = fileChooser.showOpenDialog(this);
	    Collection<SkolemFunction> insertedSkolem = null;
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	this.file2import = this.fileChooser.getSelectedFile();
	    	try {
				insertedSkolem = this.facade.importSkolemFunctions(this.file2import.getPath());
				JOptionPane.showMessageDialog(this, "Skolem function was imported", 
						"Import Skolem function", JOptionPane.INFORMATION_MESSAGE);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), 
						"Import Skolem function", 
						JOptionPane.ERROR_MESSAGE);
			}
	    }
	    return insertedSkolem;
	}
}