/*
 * This class implements a panel to create a new Skolem's function 
 */
package org.modelgen.jpf.defineSkolem.newSkolem;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.commons.beans.skolem.SkolemFunctionArgument;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.ManagementSkolemTool;
import org.modelgen.jpf.defineSkolem.table.filter.FilterTable;
import org.modelgen.jpf.defineSkolem.util.FactoryIcon;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.AncestorListener;

/**
 * @author  gabriele  
 * This class implements a pa  nel to create a new Skolem's function
 */
public class PanelNewSkolem extends JPanel {
	private static final long serialVersionUID = 6185517444142534240L;
	// Array for TYPE ARGUMENT
	private static final String[] TYPE_ARGUMENT = {"integer", "varchar"};
	// Singleton's instance
	private static PanelNewSkolem instance;
	// Facade
	private GeneralManagementFacade facade;
	// Main window
	private ManagementSkolemTool mngSkolem;
	// Current function over work
	private SkolemFunction currentFunction;
	// Listener for combo
	private ItemListener lstCombo;
	// Listener for action
	private ActionListener lstAction;
	// Listener for keyboard
	private KeyListener lstKey;
	// Listener for mouse
	private AncestorListener lstAncestor;
	// Layout manager
	private GridBagConstraints generalLayout;
	// The table filter
	private FilterTable filter;
	// Component of the panel
	private JLabel lblConstruct;
	protected JComboBox cmbConstruct;
	private JLabel lblFunctionName;
	protected JTextField txtFunctionName;
	private JList lstArgument;
	private JLabel lblArgumentName;
	private JLabel lblArgumentType;
	protected JTextField txtArgumentName;
	private JComboBox cmbArgumentType;
	//private JLabel lblSourceConstruct;
	private JComboBox cmbSourceConstruct;
	protected JButton btnArgumentOk;
	protected JButton btnArgumentReset;
	protected JButton btnSaveFunction;
	protected JButton btnClosePanel;
	// The list of the added argument
	private Collection<String> argument;

	/**
	 * Constructor of the class
	 * @param ManagementSkolemFunction The main window
	 */
	private PanelNewSkolem(ManagementSkolemTool mngSkolem) {
//		starting the skolem panel, I start to define a function
		this.currentFunction = null;
		this.mngSkolem = mngSkolem;
		// Create facade
		this.facade = GeneralManagementFacade.getInstance();
		// Create listeners
		this.lstCombo = new ItemListenerPanelNewSkolem(this);
		this.lstAction = new ActionListenerPanelNewSkolem(this);
		this.lstKey = new KeyListenerPanelNewSkolem(this);
		this.lstAncestor = new AncestorListenerPanelNewSkolem(this);
		// Create filter
		this.filter = FilterTable.getInstance();
		this.filter.createPanel();
		// Create component
		this.addAncestorListener(this.lstAncestor);
		this.lblConstruct = new JLabel("Select a construct from the list");
		this.cmbConstruct = new JComboBox();
		this.lblFunctionName = new JLabel("Name of the new Skolem function");
		this.txtFunctionName = new JTextField(25);
		this.txtFunctionName.addKeyListener(this.lstKey);
		this.lblArgumentName = new JLabel("Argument name: ");
		this.lblArgumentType = new JLabel("Argument type: ");
		//this.lblSourceConstruct = new JLabel("Select a construct from the list");
		this.cmbSourceConstruct = new JComboBox();
		this.txtArgumentName = new JTextField(25);
		this.txtArgumentName.addKeyListener(this.lstKey);
		this.lstArgument = new JList();
		this.cmbArgumentType = new JComboBox(TYPE_ARGUMENT);
		this.btnArgumentOk = new JButton("Ok");
		this.btnArgumentOk.addActionListener(this.lstAction);
		this.btnArgumentReset = new JButton("Reset");
		this.btnArgumentReset.addActionListener(this.lstAction);
		this.btnSaveFunction = new JButton("Save function");
		this.btnSaveFunction.addActionListener(this.lstAction);
		this.btnClosePanel = new JButton(FactoryIcon.createCloseIcon());
		this.btnClosePanel.setToolTipText("Exit");
		this.btnClosePanel.addActionListener(this.lstAction);
	}

	/**
	 * Return an instance of the class
	 * @param mngSkolem The main window
	 * @return PanelNewSkolem
	 */
	public static PanelNewSkolem getInstance(ManagementSkolemTool mngSkolem) {
		/* Implements Singleton */
		if(instance == null)
			instance = new PanelNewSkolem(mngSkolem);
		return instance;
	}

	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.BOTH;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}

	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlGeneralInfo = new JPanel();
		JPanel pnlAddArgument = new JPanel();
		// Initialize gui elements
		this.initializePanel();		
		// Create panel general info
		pnlGeneralInfo.setLayout(new GridBagLayout());
		pnlGeneralInfo.setBorder(
				new javax.swing.border.TitledBorder(
						"General info for Skolem function"));
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.HORIZONTAL;
		consGeneralInfo.anchor = GridBagConstraints.NORTHWEST;
		consGeneralInfo.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 0;
		pnlGeneralInfo.add(this.lblConstruct, consGeneralInfo);
		// Component in (1,0)
		consGeneralInfo.gridx = 1;
		consGeneralInfo.gridy = 0;
		pnlGeneralInfo.add(this.cmbConstruct, consGeneralInfo);
		// Component in (2,0)
		consGeneralInfo.gridx = 2;
		consGeneralInfo.gridy = 0;
		pnlGeneralInfo.add(this.lblFunctionName, consGeneralInfo);
		// Component in (3,0)
		consGeneralInfo.gridx = 3;
		consGeneralInfo.gridy = 0;
		pnlGeneralInfo.add(this.txtFunctionName, consGeneralInfo);   
		// Panel argument
		pnlAddArgument.setLayout(new GridBagLayout());
		pnlAddArgument.setBorder(
				new javax.swing.border.TitledBorder(
						"Add argument(s) to Skolem function"));
		GridBagConstraints consAddArgument = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consAddArgument.fill = GridBagConstraints.NONE;
		consAddArgument.anchor = GridBagConstraints.NORTHWEST;
		consAddArgument.insets = new Insets(5,5,5,5);
		disableAddArgument();
		// Component in (0,0)
		consAddArgument.gridx = 0;
		consAddArgument.gridy = 0;
		consAddArgument.anchor = GridBagConstraints.EAST;
		pnlAddArgument.add(this.lblArgumentName, consAddArgument);
		// Component in (1,0)
		consAddArgument.gridx = 1;
		consAddArgument.gridy = 0;
		consAddArgument.anchor = GridBagConstraints.WEST;
		pnlAddArgument.add(this.txtArgumentName, consAddArgument);
		// Component in (0,1)
		consAddArgument.gridx = 0;
		consAddArgument.gridy = 1;
		consAddArgument.anchor = GridBagConstraints.EAST;
		pnlAddArgument.add(this.lblArgumentType, consAddArgument);
		// Component in (1,1)
		consAddArgument.gridx = 1;
		consAddArgument.gridy = 1;
		consAddArgument.anchor = GridBagConstraints.WEST;
		pnlAddArgument.add(this.cmbArgumentType, consAddArgument);
//		 Component in (1,2)
		consAddArgument.gridx = 1;
		consAddArgument.gridy = 2;
		consAddArgument.anchor = GridBagConstraints.WEST;
		pnlAddArgument.add(this.cmbSourceConstruct, consAddArgument);
		// Component in (1,3)
		consAddArgument.gridx = 1;
		consAddArgument.gridy = 3;
		consAddArgument.anchor = GridBagConstraints.WEST;
		pnlAddArgument.add(this.btnArgumentOk, consAddArgument);
		// Component in (0,3)
		consAddArgument.gridx = 0;
		consAddArgument.gridy = 4;
		consAddArgument.anchor = GridBagConstraints.EAST;
		consAddArgument.fill = GridBagConstraints.NONE;
		pnlAddArgument.add(this.btnSaveFunction, consAddArgument);
		// Component in (1,3)
		consAddArgument.gridx = 1;
		consAddArgument.gridy = 4;
		consAddArgument.anchor = GridBagConstraints.WEST;
		pnlAddArgument.add(this.btnArgumentReset, consAddArgument);
		// Component in (2,0)
		consAddArgument.gridx = 2;
		consAddArgument.gridy = 0;
		consAddArgument.gridheight = 4;
		consAddArgument.gridwidth = 1;
		pnlAddArgument.add(new JScrollPane(this.lstArgument), consAddArgument);
		// Component in (5,6)
		consAddArgument.gridx = 2;
		consAddArgument.gridy = 4;
		consAddArgument.anchor = GridBagConstraints.CENTER;
		consAddArgument.fill = GridBagConstraints.NONE;
		consAddArgument.gridheight = 0;
		consAddArgument.gridwidth = 0;
		pnlAddArgument.add(this.btnClosePanel, consAddArgument);
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlGeneralInfo, this.generalLayout);
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 1;
		this.add(pnlAddArgument, this.generalLayout);		
	}

	/**
	 * Initialize the cmb for the construct
	 */
	private void initializeCmbConstructs(JComboBox cmbConstruct) {
		if(cmbConstruct.getItemCount() != 0)
			cmbConstruct.setSelectedIndex(0);
		else {
			// First element of combo box
			cmbConstruct.addItem(".: Select :.");
			// Take a collection of contructs
			Collection<Construct> construct = null;
			try {
				construct = GeneralManagementFacade.getInstance().populateCmbConstruct(false);
				// Iterator over collection
				for(Construct c : construct) {
					// Add to combo box
					cmbConstruct.addItem(c.getName());
				}
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, "", e.getMessage(), JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	/**
	 * Initialize panel' components
	 */
	@SuppressWarnings("unchecked")
	protected void initializePanel() {
		this.initializeCmbConstructs(this.cmbConstruct);
		this.initializeCmbConstructs(this.cmbSourceConstruct);
		this.txtFunctionName.setEnabled(false);
		this.cmbConstruct.addItemListener(this.lstCombo);
		this.cmbConstruct.setEnabled(true);
		this.cmbConstruct.setSelectedIndex(0);
//		this.cmbSourceConstruct.addItemListener(this.lstComboSource);
//		this.cmbSourceConstruct.setEnabled(true);
//		this.cmbSourceConstruct.setSelectedIndex(0);
		this.lstArgument.setPreferredSize(new Dimension(255, 70));
		this.txtFunctionName.setText("");
		this.txtArgumentName.setText("");
		/* Clear the list */
		if(this.argument != null) {
			this.argument.removeAll(this.argument);
			/* Clear jlist */
			this.lstArgument.setListData(this.argument.toArray());
		} else {
			this.argument = new LinkedList();
			this.lstArgument.setListData(this.argument.toArray());
		}
		this.cmbArgumentType.setSelectedIndex(0);
	}

	/**
	 * Disable the component of pnlAddArgument
	 */
	private void disableAddArgument() {
		this.lstArgument.setEnabled(false);
		this.txtArgumentName.setEnabled(false);
		this.cmbArgumentType.setEnabled(false);
		this.cmbSourceConstruct.setEnabled(false);
		this.btnArgumentOk.setEnabled(false);
		this.btnArgumentReset.setEnabled(false);
		this.btnSaveFunction.setEnabled(false);
	}

	/**
	 * Enable the component of pnlAddArgument and disable component of pnlGeneralInfo
	 */
	protected void enableAddArgument() {
		if(!this.txtFunctionName.equals("")) {
			this.txtArgumentName.setEnabled(true);
			this.cmbArgumentType.setEnabled(true);
			this.cmbSourceConstruct.setEnabled(true);
			this.btnArgumentReset.setEnabled(true);
			this.cmbConstruct.setEnabled(true);
			this.cmbSourceConstruct.setEnabled(true);
		} else {
			this.txtArgumentName.setEnabled(false);
			this.cmbArgumentType.setEnabled(false);
			this.btnArgumentReset.setEnabled(false);
			this.cmbSourceConstruct.setEnabled(false);
		}
		// Create the argument's list if not null
		if(this.argument == null)
			this.argument = new ArrayList<String>();
	}
	
	/**
	 * Remove the panel to the window
	 */
	protected void closePanel() {
		this.mngSkolem.removePanelNewSkolem();
		this.filter.setCmbConstruct(0);
		this.filter.setTxtFunction("");
	}
	
	/**
	 * Notify the filter of the combo's change
	 */
	protected void notifyFilter() {
		this.filter.setCmbConstruct(this.cmbConstruct.getSelectedIndex());
//		this.filter.setTxtFunction(this.txtFunctionName.getText());
		this.filter.resetFilter();
		this.filter.runFilter();
	}

	/**
	 * Enable text field for function's name
	 */
	protected void enableTxtFunction() {
		if(!this.txtFunctionName.isEnabled())
			this.txtFunctionName.setEnabled(true);
		if(this.cmbConstruct.getSelectedIndex() == 0) {
			this.txtFunctionName.setEnabled(false);
			// Disable argument
			this.disableAddArgument();
			this.argument = null;
		}
	}

	/**
	 * Create a new Skolem function
	 */
	protected void createNewSkolemFunction() {
		// Create the new funcion
		String functionsName = this.txtFunctionName.getText();
		String constructName = (String)this.cmbConstruct.getSelectedItem();
		try {
			if(functionsName.equals(""))
				this.currentFunction = 
					this.facade.createNewSkolemFunction("noname", constructName);
			else
				this.currentFunction = 
					this.facade.createNewSkolemFunction(functionsName, constructName);
		} catch (PersistenceException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "New Skolem function", 
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Check is function name is already in the list
	 */
	protected void checkFunctionName() {
		if(this.txtFunctionName.getText().equals("")) {
			this.txtFunctionName.setBackground(Color.WHITE);
		} else {
			// 	Check if function is alreay in the list
			if(GeneralManagementFacade.getInstance().checkSkolemFunction(
					this.txtFunctionName.getText())) {
				this.txtFunctionName.setBackground(Color.RED);
			} else {
				this.txtFunctionName.setBackground(Color.WHITE);
			}
		}
	}

	/**
	 * Enable/disable ok button for argument
	 */
	protected void enableBtnArgumentOk() {
		if(!this.txtArgumentName.getText().equals(""))
			this.btnArgumentOk.setEnabled(true);
		else
			this.btnArgumentOk.setEnabled(false);
	}	

	/**
	 * Check if argument is already in the list
	 */
	protected void checkArgument() {
		String argument = this.txtArgumentName.getText();
		if(GeneralManagementFacade.getInstance().checkSkolemFunctionArgument(argument, this.argument)) {
			this.txtArgumentName.setBackground(Color.RED);
			this.btnArgumentOk.setEnabled(false);
		} else {
			this.txtArgumentName.setBackground(Color.WHITE);
			this.btnArgumentOk.setEnabled(true);
		}
	}

	/**
	 * Add argument to Skolem function
	 */
	protected void addArgumentToFunction() {
		String argumentName = this.txtArgumentName.getText();
		String type = (String)this.cmbArgumentType.getSelectedItem();
		String constructName = (String)this.cmbSourceConstruct.getSelectedItem();
		// check if the skolem's arguments is OK. this check should not be
		// done here.
		
		// Add
		
		try {
			this.facade.addSkolemArgument(argumentName, type, constructName);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Cannot insert function argument.", 
					"New Skolem function", JOptionPane.INFORMATION_MESSAGE);
			e.printStackTrace();
		}
	}

	/**
	 * Add a new argument
	 */
	@SuppressWarnings("unchecked")
	protected void addArgument() {
		String argName, argType, argSourceConstruct;
		argName = this.txtArgumentName.getText();
		argType = (String)this.cmbArgumentType.getSelectedItem();
		argSourceConstruct = (String) this.cmbSourceConstruct.getSelectedItem();
		String arg = argName + ", " + argType + "(" + argSourceConstruct +  ")";
		this.argument.add(arg);
		this.lstArgument.removeAll();
		this.lstArgument.setListData(this.argument.toArray());
		this.txtArgumentName.setText("");
		this.cmbArgumentType.setSelectedIndex(0);
		this.cmbSourceConstruct.setSelectedIndex(0);
		this.btnArgumentOk.setEnabled(false);
		this.btnSaveFunction.setEnabled(true);
	}
	
	/**
	 * Ask to the user if he wants preserve the argument already insert, if he 
	 * changes the current function
	 */
	@SuppressWarnings("unchecked")
	protected void changeArgument() {
		if(this.argument.size() > 0) {
			int value = JOptionPane.showConfirmDialog(this, 
					"The current Skolem function is changed; do you want" + 
					" preserve the argument(s) already insert?", 
					"New Skolem function", JOptionPane.YES_NO_OPTION);
			if(value == JOptionPane.NO_OPTION) {
				// Remove to collection
				this.argument.removeAll(this.argument);
				// Update jlist
				this.lstArgument.setListData(this.argument.toArray());
			} else {
				// TODO check: this.argument = this.currentFunction.getArguments();
			    	Collection<SkolemFunctionArgument> argumentTemp = this.currentFunction.getArguments();
				Collection<String> c = new LinkedList<String>();
				String strArg = null;
				// Iterator
				Iterator<SkolemFunctionArgument> it = argumentTemp.iterator();
				while(it.hasNext()) {
					SkolemFunctionArgument arg = it.next();
					strArg = arg.getName() + " (" + arg.getType() + ")";
					c.add(strArg);
				}
				// Update jlist
				this.lstArgument.setListData(c.toArray());
			}
		}
	}

	/**
	 * Store Skolem function
	 */
	protected void saveFunction() {
		String functionName = this.txtFunctionName.getText();
		String constructName = (String)this.cmbConstruct.getSelectedItem();
		if(!functionName.equals("")) {
			try {
				this.currentFunction = this.facade.createNewSkolemFunction(functionName, 
						constructName);
				if (this.facade.isSkolemFunctionSafe(this.currentFunction)) {
					this.facade.storeSkolemFunction();
					JOptionPane.showMessageDialog(this, "Insert successfully", 
						"New Skolem function", JOptionPane.INFORMATION_MESSAGE);
				}
				else {
					JOptionPane.showMessageDialog(this, "Cannot insert skolem function: name or " +
							"\n\rarguments repeated in the dictionary.", 
							"New Skolem function", JOptionPane.INFORMATION_MESSAGE);
				}
				this.currentFunction = null;
				this.argument = new LinkedList<String>();
				this.facade.resetSkolemFunction();
			} catch (PersistenceException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), "New Skolem function", 
						JOptionPane.ERROR_MESSAGE);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), "New Skolem function", 
						JOptionPane.ERROR_MESSAGE);
			} finally {
				this.currentFunction = null;
				this.argument = null;
				this.removeAll();
				this.mngSkolem.removePanelNewSkolem();
				this.filter.setCmbConstruct(0);
				this.filter.setTxtFunction("");
				this.filter.resetFilter();
			}
		} else {
			JOptionPane.showMessageDialog(this, "Insert function name; this field " + 
					"cannot be empty", "New Skolem function",
					JOptionPane.ERROR_MESSAGE);
		}
	}
}