/*
 * This class implements a report of super model 
 */
package org.modelgen.jpf.defineSkolem.report;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * @author  gabriele  This class implements a report of super model
 */
public class PanelPrintSuperModel extends JFrame {
	private static final long serialVersionUID = -848447555131287635L;
	// Singleton
	private static PanelPrintSuperModel instance;
	// Facade
	private GeneralManagementFacade facade;
	// Text area for report
	private JTextArea txaReport;
	// Layout manager
	private GridBagConstraints generalLayout;
	
	/**
	 * Class construct
	 */
	private PanelPrintSuperModel() {
		super("SuperModel report");
		// Creates la text area
		this.txaReport = new JTextArea();
		this.txaReport.setText("");
		// Create facade
		this.facade = GeneralManagementFacade.getInstance();
		// Sets layout
		this.setLayout();
	}
	
	/**
	 * Sets the general layout
	 */
	private void setLayout() {
		this.getContentPane().setLayout(new GridBagLayout());
		// Setto il layout
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.BOTH;
		this.generalLayout.anchor = GridBagConstraints.CENTER;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 1;
	}
	
	/**
	 * Return an instance of class
	 * @return  PanelPrintSuperModel
	 * @uml.property  name="instance"
	 */
	public static PanelPrintSuperModel getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelPrintSuperModel();
		return instance;
	}
	
	/**
	 * Creates report
	 */
	public void createReport() {
		try {
			Model superModel = this.facade.getSuperModel();
//			 Take a collection of constructs
			Collection<Construct> constructs = superModel.getAbstractConstructs();
			// Iterator 
			Iterator<Construct> itConstruct = constructs.iterator();
			while(itConstruct.hasNext()) {
				Construct construct = itConstruct.next();
				this.txaReport.append("Construct: " + construct.getName().toUpperCase() + 
						"\n");
				this.txaReport.append("Properties: " + "\n");
				if(construct.getProperties().size() != 0) {
					for(Iterator<Property> j = construct.getProperties().iterator(); j.hasNext();) {
						Property p = j.next();
						this.txaReport.append(
								"       "+p.getName() + " (" + p.getType() + ")" + "\n\n");
					}
				}
				else {
					this.txaReport.append("       (has no properties)" + "\n\n");
				}
				this.txaReport.append("References: " + "\n");		
				if(construct.getReferences().size() != 0) {
					for(Iterator<Reference> j = construct.getReferences().iterator(); j.hasNext();) {
						Reference r = j.next();
						this.txaReport.append(
								"       "+r.getName() + " (of type " + 
								r.getTypeAsString() + "), pointing to " + 
								r.getTargetConstruct().getName() + "\n\n");
					}			
				} else {
					this.txaReport.append("       (has no references)" + "\n\n");
				}
			}
			this.addReport();
		} catch (PersistenceException e) {}
	}
	
	/**
	 * Add report
	 */
	private void addReport() {
		this.generalLayout.gridx = 1;
		this.generalLayout.gridy = 1;
		JScrollPane pane = new JScrollPane(this.txaReport,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		pane.setPreferredSize(new Dimension(600, 600));
		this.getContentPane().add(pane, this.generalLayout);
		this.setVisible(true);
		this.pack();
		this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
	}
}