/*
 * This class implements an abstract model for Skolem table 
 */
package org.modelgen.jpf.defineSkolem.table;

import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.commons.beans.skolem.SkolemFunctionArgument;
import org.modelgen.facade.GeneralManagementFacade;

import java.util.Collection;
import java.util.Iterator;

import javax.swing.table.AbstractTableModel;

/**
 * @author  gabriele  This class implements an abstract model for Skolem table
 */
class ModelSkolemTable extends AbstractTableModel {
	private static final long serialVersionUID = 4547579822288670620L;
	// Singleton
	private static ModelSkolemTable instance;
	// Array of columns name and data
	private String[][] data = null;
	private String[] columnNames = {"Function name", "Construct", "Arguments" };
	
	/**
	 * Class constructor
	 */
	private ModelSkolemTable() {
	}
	
	/**
	 * Return an instance of the class
	 * @return  ModelSkolemTable
	 * @uml.property  name="instance"
	 */
	protected static ModelSkolemTable getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new ModelSkolemTable();
		return instance;
	}

	/**
	 * Return column count
	 * @return int
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * Return row count
	 * @return int
	 */
	public int getRowCount() {
		return data.length;
	}

	/**
	 * Return a value at the specify position
	 * @return Object
	 */
	public Object getValueAt(int row, int column) {
		return data[row][column];
	}
	
	/**
	 * Return the name of specify column
	 * @param col The column
	 */
	public String getColumnName(int col) {
		return this.columnNames[col];
	}
	
	/**
	 * Add a collection of functions to the table
	 * @param functions The collection of the functions
	 */
	protected void addFunction(Collection<SkolemFunction> functions) {
		// Index to create model
		int k = 0; int max = 0;
		// Max dimension of array
		max = functions.size();
		// Creates data array
		data = new String[max][3];
		for (Iterator<SkolemFunction> itFunctions = functions.iterator(); itFunctions.hasNext();) {
			SkolemFunction function = itFunctions.next();
			data[k][0] = function.getName();
			//The construct
			data[k][1] = function.getOwnerConstruct().getName();
			//The arguments
			String arguments = "";
			int i=0;
			for (Iterator<SkolemFunctionArgument> itArguments = function.getArguments().iterator(); itArguments.hasNext();) {
				SkolemFunctionArgument argument = itArguments.next();
				if(i==0)
					arguments = argument.getName() + "(" + argument.getSourceConstruct().getName() + ")";
				else
					arguments = arguments +", "+argument.getName() + "(" + argument.getSourceConstruct().getName() + ")";
			i++;
			}		
			data[k][2] = arguments;
			k++;
		}
		// Update table
		fireTableDataChanged();
	}
	
	/**
	 * Reload the table
	 * @throws ClassNotFoundException
	 * @throws Exception
	 */
	protected void reload() throws Exception {
		this.addFunction(GeneralManagementFacade.getInstance().getSkolemFunctions());
	}
}