/*
 * This class implements a table for Skolem' functions 
 */
package org.modelgen.jpf.defineSkolem.table;

import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.jpf.defineSkolem.ManagementSkolemTool;
import org.modelgen.jpf.defineSkolem.table.popup.PopupTable;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JTable;

/**
 * @author  gabriele  This class implements a table for Skolem' functions
 */
public class SkolemTable {
	// Singleton instance
	private static SkolemTable instance;
	// Reference to abstract table model
	private static ModelSkolemTable model;
	// Main window
	private ManagementSkolemTool mngSkolem;
	// Listener table
	private MouseListener lstMouse;
	// The table
	private JTable table;
	// Popup menu
	private PopupTable popTable;
	
	/**
	 * Costructor of the class
	 */
	private SkolemTable() {
		// Create listener
		this.lstMouse = new MouseListenerTable(this);
		// Create a table model
		model = ModelSkolemTable.getInstance();
	}
	
	/**
	 * Return an instance of the class
	 * @return  SkolemTable
	 * @uml.property  name="instance"
	 */
	public static SkolemTable getInstance() {
		/* Singleto */
		if(instance == null)
			instance = new SkolemTable();
		return instance;
	}
	
	/**
	 * Sets the main windows
	 * @param mngSkolem The main window
	 */
	public void setManagementSkolemTool(ManagementSkolemTool mngSkolem) {
		this.mngSkolem = mngSkolem;
		// Creates a popup
		this.popTable = PopupTable.getInstance(this, this.mngSkolem);
	}
	
	/**
	 * Create the table
	 */
	public void createTable() {
		TableSorter sorter = new TableSorter(model); 
		this.table = new JTable(sorter);             
		sorter.setTableHeader(this.table.getTableHeader());
		this.table.getTableHeader().setToolTipText(
				"Click to specify sorting; Control-Click to specify secondary sorting");
		// Add to listener
		this.table.addMouseListener(this.lstMouse);
	}
	
	/**
	 * Show popup
	 */
	protected void showPopup(Point point) {
		this.popTable.showPopup(point);
	}
	
	/**
	 * Return the table
	 * @return  JTable
	 * @uml.property  name="table"
	 */
	public JTable getTable() {
		this.table.setPreferredScrollableViewportSize(new Dimension(800, 150));
		return this.table;
	}
	
	/**
	 * Enable/disable btnInsert of popup menu
	 * @param enable
	 */
	public void enableBtnInsert(boolean enable) {
		this.popTable.enableBtnInsert(enable);
	}	
	
	/**
	 * Add to the table
	 * @param functions The collection of Skolem function
	 */
	public void addFunction(Collection<SkolemFunction> functions) {
		model.addFunction(functions);
	}
	
	/**
	 * Unload the elements in the table
	 */
	public void unloadTable() {
		/* This method is invoked when the tabbed changes */
		model.addFunction(new LinkedList<SkolemFunction>());
	}
	
	/**
	 * Run the filter
	 * @param c The collection of Skolem function
	 */
	public void runFilter(Collection<SkolemFunction> c) {
		model.addFunction(new LinkedList<SkolemFunction>());
		model.addFunction(c);
	}
	
	/**
	 * Reload the table
	 * @throws Exception 
	 * @throws ClassNotFoundException 
	 */
	public void reload() throws ClassNotFoundException, Exception {
		model.reload();
	}
}