/*
 * Implements a filter for the table 
 */
package org.modelgen.jpf.defineSkolem.table.filter;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.table.SkolemTable;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemListener;
import java.awt.event.KeyListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @author  gabriele  Implements a filter for the table
 */
public class FilterTable extends JPanel {
	private static final long serialVersionUID = -5398892605674808128L;
	// Singleton
	private static FilterTable instance;
	// The list of function already defined
	private Collection<SkolemFunction> copyFunctions;
	// Listener for filter
	private ItemListener lstItem;
	private KeyListener lstKey;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Component for filtering
	protected JComboBox cmbFilter;
	protected JTextField txtFilter;
	// Skolem table
	private SkolemTable skolemTable;
	
	/**
	 * Class constructor
	 */
	@SuppressWarnings("unchecked")
	private FilterTable() {
		// Take a list of already defined functions
		try {
			this.copyFunctions = GeneralManagementFacade.getInstance().getSkolemFunctions();
		} catch (Exception e) {}
		// Creates Skolem table
		this.skolemTable = SkolemTable.getInstance();
		// Filter component
		this.cmbFilter = new JComboBox();
		this.initializeCmbConstructs();
		this.txtFilter = new JTextField(35);
		// Create listeners
		this.lstItem = new ItemListenerCombo(this);
		this.lstKey = new KeyListenerField(this);
		this.cmbFilter.addItemListener(this.lstItem);
		this.txtFilter.addKeyListener(this.lstKey);
		// Sets layout
		this.setLayout();
	}
	
	/**
	 * Return an instance of the class
	 * @return  FilterTable
	 * @uml.property  name="instance"
	 */
	public static FilterTable getInstance() {
		/* Singleto */
		if(instance == null)
			instance = new FilterTable();
		return instance;
	}
	
	/**
	 * Sets the general layout
	 */
	private void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.NONE;
		this.generalLayout.anchor = GridBagConstraints.WEST;
	}
	
	/**
	 * Creates a panel
	 */
	public void createPanel() {
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(new JLabel("Select a construct"), this.generalLayout);
		this.generalLayout.gridx = 1;
		this.generalLayout.gridy = 0;
		this.add(this.cmbFilter, this.generalLayout);
		this.generalLayout.gridx = 2;
		this.generalLayout.gridy = 0;
		this.add(new JLabel("Function name"), this.generalLayout);
		this.generalLayout.gridx = 3;
		this.generalLayout.gridy = 0;
		this.add(this.txtFilter, this.generalLayout);
	}
	
	/**
	 * Initialize the cmb for the construct
	 */
	private void initializeCmbConstructs() {
		if(this.cmbFilter.getItemCount() != 0)
			this.cmbFilter.setSelectedIndex(0);
		else {
			// First element of combo box
			this.cmbFilter.addItem(".: Select :.");
			// Take a collection of contructs
			Collection<Construct> construct = null;
			try {
				construct = GeneralManagementFacade.getInstance().populateCmbConstruct(false);
				// Iterator over collection
				for(Construct c : construct) {
					// Add to combo box
					this.cmbFilter.addItem(c.getName());
				}
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this, "", e.getMessage(), JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	/* This method are implemented to have the combo and the text field that execute
	 * table filter also in the panel to create a new Skolem function, in accord with
	 * the request that while the user inserts a new skolem function, the table update
	 * itself like the user uses the filter */
	
	/**
	 * Set the item of the cmb for construct
	 * @param index The index to set 
	 */
	public void setCmbConstruct(int index) {
		this.cmbFilter.setSelectedIndex(index);
	}
	
	/**
	 * Set the text of the txt function name
	 * @param text The text to set
	 */
	public void setTxtFunction(String text) {
		this.txtFilter.setText(text);
	}
	
	/**
	 * Run the filter
	 */
	public void runFilter() {
		/* Take construnct name and function name */
		String constructName, functionName;
		if(this.cmbFilter.getSelectedIndex() != 0) {
			constructName = (String)this.cmbFilter.getSelectedItem();
		} else 
			constructName = "";
		functionName = this.txtFilter.getText();
		this.skolemTable.runFilter(this.runFilter(constructName, functionName));
		
	}
	
	/**
	 * Run the filter
	 * @param constructName The construct name
	 * @param functionName The function name
	 * @return Collection
	 */
	private Collection<SkolemFunction> runFilter(String constructName, String functionName) {
		// Iterator 
		Iterator<SkolemFunction> it = this.copyFunctions.iterator();
		// Skolem function
		SkolemFunction sklFunction = null;
		/* Don't pass a construcName; dynamic filterig of function name */
		while(it.hasNext()) {
			sklFunction = it.next();
			if(!functionName.equals("")) { 
				if(constructName.equals("")) {
					if(!sklFunction.getName().toLowerCase().startsWith(
							functionName.toLowerCase())) {
						it.remove();
					}
				} else {
					/* Take all function with this construct name */
					if(!sklFunction.getName().toLowerCase().startsWith(
							functionName.toLowerCase()) ||
							!sklFunction.getOwnerConstruct().getName().equals(
							constructName)) {
						it.remove();
					}
				} 
			} else {
				if(constructName.equals("")) {
					this.resetFilter();
				} else {
					/* Take all function with this construct name */
					if(!sklFunction.getOwnerConstruct().getName().equals(
							constructName)) {
						it.remove();
					}
				}
			}
		}
		return this.copyFunctions;
	}
	
	/**
	 * Reset the filter
	 */
	public void resetFilter() {
		try {
			this.copyFunctions = GeneralManagementFacade.getInstance().getSkolemFunctions();
		} catch (Exception e) {}
		this.skolemTable.addFunction(this.copyFunctions);
	}
}