/*
 * Popup menu over the table 
 */
package org.modelgen.jpf.defineSkolem.table.popup;

import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.defineSkolem.ManagementSkolemTool;
import org.modelgen.jpf.defineSkolem.table.SkolemTable;
import org.modelgen.jpf.defineSkolem.util.FactoryIcon;

import java.awt.Point;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

/**
 * @author  gabriele  Popup menu over the table
 */
public class PopupTable extends JPopupMenu {
	private static final long serialVersionUID = 7700594408224955646L;
	// Singleton
	private static PopupTable instance;
	// Facade
	private GeneralManagementFacade facade;
	// Listener
	private ActionListener lstAction;
	// Component of popup
	protected JMenuItem btnInsert;
	protected JMenuItem btnDelete;
	// The table
	private SkolemTable table;
	
	/**
	 * Class constructor
	 * @param skolemTable The table of Skolem function
	 * @param mngSkolem The main window
	 */
	private PopupTable(SkolemTable skolemTable, ManagementSkolemTool mngSkolem) {
		this.table = skolemTable;
		// Creates facade
		this.facade = GeneralManagementFacade.getInstance();
		// Creates listener
		this.lstAction = new ActionListenerPopup(this, mngSkolem);
		// Creates button
		this.btnInsert = new JMenuItem("New Skolem function");
		this.btnInsert.setIcon(FactoryIcon.createInsertIcon());
		this.btnInsert.addActionListener(this.lstAction);
		this.btnDelete = new JMenuItem("Delete");
		this.btnDelete.setIcon(FactoryIcon.createDeleteIcon());
		this.btnDelete.addActionListener(this.lstAction);
	}
	
	/**
	 * Return an instance of the class
	 * @param skolemTable The table of Skolem function
	 * @param mngSkolem The main window
	 * @return PopupTable
	 */
	public static PopupTable getInstance(SkolemTable skolemTable, 
			ManagementSkolemTool mngSkolem) {
		/* Singleton */
		if(instance == null)
			instance = new PopupTable(skolemTable, mngSkolem);
		return instance;
	}
	
	/**
	 * Class constructor
	 * @param mngSkolem The main window
	 */
	private PopupTable(ManagementSkolemTool mngSkolem) {
		// Creates facade
		this.facade = GeneralManagementFacade.getInstance();
		// Creates listener
		this.lstAction = new ActionListenerPopup(this, mngSkolem);
		// Creates button
		this.btnInsert = new JMenuItem("New Skolem function");
		this.btnInsert.setIcon(FactoryIcon.createInsertIcon());
		this.btnInsert.addActionListener(this.lstAction);
		this.btnDelete = new JMenuItem("Delete");
		this.btnDelete.setIcon(FactoryIcon.createDeleteIcon());
		this.btnDelete.addActionListener(this.lstAction);
	}
	
	/**
	 * Return an instance of the class
	 * @param mngSkolem The main window
	 * @return PopupTable
	 */
	public static PopupTable getInstance(ManagementSkolemTool mngSkolem) {
		/* Singleton */
		if(instance == null)
			instance = new PopupTable(mngSkolem);
		return instance;
	}
	
	/**
	 * Show popup
	 * @param point The point where mouse clicked
	 */
	public void showPopup(Point point) {
		this.add(this.btnInsert);
		this.add(this.btnDelete);
		this.show(this.table.getTable(), (int)point.getX(), (int)point.getY());
		this.setVisible(true);
		this.pack();
	}
	
	/**
	 * Enable/disable btnInsert in this popup menu
	 * @param enable True, if enable
	 */
	public void enableBtnInsert(boolean enable) {
		this.btnInsert.setEnabled(enable);
	}
	
	/**
	 * Delete Skolem function
	 */
	protected void deleteSkolemFunction() {
		int[] row = this.table.getTable().getSelectedRows();
		if(row.length == 0) {
			JOptionPane.showMessageDialog(new JPanel(), "No selected rows.", 
					"Delete Skolem", JOptionPane.ERROR_MESSAGE);
		} else {
			int value = JOptionPane.showConfirmDialog(this, "Are you sure to delete all " + 
					"selected rows?", "Delete Skolem", JOptionPane.YES_NO_OPTION);
			if(value ==  JOptionPane.YES_OPTION) {
				String function = null;
				String construct = null;
				for(int i = 0; i <row.length ; i++) {
					function = (String)this.table.getTable().getValueAt(row[i], 0);
					construct = (String)this.table.getTable().getValueAt(row[i], 1);
					try {
						this.facade.deleteSkolemFunction(construct, function);
					} catch (Exception e) {}
				}
				JOptionPane.showMessageDialog(this, "Delete successfully", "Delete Skolem",
						JOptionPane.INFORMATION_MESSAGE);
				try {
					this.table.reload();
				} catch (ClassNotFoundException e) {} catch (Exception e) {}
			}
		}
	}
}