package org.modelgen.jpf.opgen;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;

import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.commons.Tool;
import org.modelgen.jpf.opgen.common.OpGen;
import org.modelgen.regole.MacroRegola;


/**
 * Tool to generate model management operators in datalog rules form
 * @author luigi, francesca
 *
 */
public class OpGenTool extends JInternalFrame implements Tool {

	private static final long serialVersionUID = 6645035596795893910L;
	
	/* Singleton for GUI */
	private static OpGenTool istance;
	/* for object exchange with the core plugin */	 
	private Map<String,Object> applicationObjs;
	
	private JPanel jContentPane = null;
	private JToolBar jToolBar = null;
	private JButton jButton = null;
	private JButton jButton1 = null;
	private JButton jButton2 = null;	
	private JScrollPane jScrollPane1 = null;
	private JTextPane jTextPane1 = null;
	private JLabel jLabel = null;
	private JTextField jTextField = null;
	
	/* the currently generated macrorule */
	private MacroRegola currentMacroRule = null;
		
	/* domain access Facade */
	private OpGen opGenFacade;
                 	
	
	public OpGenTool() {
		super("OpGenTool", true, true, true, true);
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		initComponents();
		
		/* initializes the facade. It is not necessary to initialize */
		/* it right now, we could do this as soon as we need to access, */
		/* however it has safeness puropose. */
		opGenFacade = OpGen.getInstance();		
		
		istance=this;
	}
	
	/**
	 * returns the intanciated facade
	 * @return the intanciated facade
	 */
	public OpGen getOpGenFacade() {
		return this.opGenFacade;
	}
	
	/**
	 * resets the current macrorule
	 *
	 */
	public void resetRule() {
		this.currentMacroRule = null;
		/* forces garbage collection to free the memory */
		/* eventually occupied by the prevoius rules. */
		Runtime.getRuntime().gc();
	}
	
	/**
	 * returns if a macrorule has been generated
	 * @return if a macrorule has been generated
	 */
	public String getMacroRuleName() {
		return this.jTextField.getText();
	}
	
	/**
	 * sets the current macrorule
	 * @param mr
	 */
	public void setMacroRule(MacroRegola mr) {
		this.currentMacroRule = mr;
	}
	
	/**
	 * returns the eventually generated macrorule
	 * @return the eventually generated macrorule
	 */
	public MacroRegola getMacroRule() {
		return this.currentMacroRule;
	}
	
	/**
	 * if a macrorule has been generated
	 * @return if a macrorule has been generated
	 */
	public boolean isMacroRuleSet() {
		return this.currentMacroRule!=null;
	}
	
	/**
	 * displays the currently generated macrorule
	 * 
	 */
	public void updateView() {		
		
		/* asynchronously updates the jTextPane */
		Runnable upDate = new Runnable() {
			public void run() {				
				OpGenTool inst = OpGenTool.getIstance();				
				inst.getTextPane().setText(inst.getMacroRule().toString());
			}};
		
		upDate.run();
			
	}
	
	/**
	 * returns the text field
	 * @return the text field
	 */
	public JTextPane getTextPane() {
		return this.jTextPane1;
	}
	
	/**
	 * returns the rule name in a textField
	 * @return if a macrorule has been generated
	 */
	public JTextField getRuleName() {
		return this.jTextField;
	}	
	
	
	/* non necessari poiché non c'è scambio di oggeti con il core */	
	public void init(Map<String,Object> applicationObjects) {
		this.applicationObjs  = applicationObjects;
	}
	
	public Map<String, Object> getInfo() {		
		return applicationObjs;
	}
	
	/* singleton */
	public static OpGenTool getIstance() {		
		return istance;
	}
	
	/**
	 * initializes GUI	 *
	 */
	private void initComponents() {			
			setProperties();		
			this.setVisible(true);
		
	}
	
	/**
	 * sets properties of this frame	 
	 */
	private void setProperties() {
		
		
		setContentPane(getJContentPane());
		setTitle("OpGenTool");
		
		Dimension dim = getToolkit().getScreenSize();
		int larghezza = (int) (dim.getWidth()*4/5);
		int altezza = (int) (dim.getHeight()*3/4);
		int bordoSinistro = (int) ((dim.getWidth() - larghezza)/2);
		int bordoSuperiore = (int) ((dim.getHeight()-altezza)/4)-altezza/22;
		setBounds(bordoSinistro,bordoSuperiore,larghezza,altezza);
		
				
		
		/* impostazione dell'icona del frame */		
		setFrameIcon(ResourceFunctions.openImageIcon(this.getClass(), "toolFrame.png"));
		
		
	}

	/**
	 * This method initializes jContentPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJToolBar(), BorderLayout.NORTH);			
			jContentPane.add(getJScrollPane1(), BorderLayout.CENTER);
		}
		return jContentPane;
	}
	/**
	 * This method initializes jToolBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private JToolBar getJToolBar() {
		if (jToolBar == null) {
			jLabel = new JLabel();			
			jLabel.setText("     Macrorule name     ");
			jToolBar = new JToolBar();
			jToolBar.add(getJButton());
			jToolBar.add(getJButton1());
			jToolBar.add(getJButton2());
			jToolBar.add(jLabel);
			jToolBar.add(getJTextField());
		}
		return jToolBar;
	}
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (jButton == null) {			
			jButton = new JButton();
			
			/* icona diff */			
			
			jButton.setIcon(ResourceFunctions.openImageIcon(this.getClass(), "diff.png"));
			
			
			jButton.setToolTipText("Click here to genereate diff macrorule");
			jButton.setText("Diff");
			jButton.setName("Diff");
			jButton.addActionListener(new ButtonActionListener(this));
		}
		return jButton;
	}
	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			
			
			/* icona merge */			
			jButton1.setIcon(ResourceFunctions.openImageIcon(this.getClass(), "merge.png"));
			
			
			
			jButton1.setToolTipText("Click here to genereate merge macrorule");
			jButton1.setText("Merge");
			jButton1.setName("Merge");
			jButton1.addActionListener(new ButtonActionListener(this));
		}
		return jButton1;
	}
	/**
	 * This method initializes jButton2	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton2() {
		if (jButton2 == null) {
			jButton2 = new JButton();
			jButton2.setToolTipText("Click here to save generated macrorule");
			jButton2.setText("Save");
			jButton2.setName("Save");
			
			/* icona save */			
			jButton2.setIcon(ResourceFunctions.openImageIcon(this.getClass(), "save.png"));
			
			jButton2.addActionListener(new ButtonActionListener(this));
		}
		return jButton2;
	}
	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTextPane1());
		}
		return jScrollPane1;
	}
	/**
	 * This method initializes jTextPane1	
	 * 	
	 * @return javax.swing.JTextPane	
	 */
	private JTextPane getJTextPane1() {
		if (jTextPane1 == null) {			
			jTextPane1 = new JTextPane();
			jTextPane1.setEditable(false);			
			this.jScrollPane1.add(jTextPane1);
		}
		return jTextPane1;
	}
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setToolTipText("Write here macrorule name before saving");
		}
		return jTextField;
	}
	
	/**
	 * shows a message in a dialog given the text and the type.
	 * @param text
	 * @param type
	 */
	public void showMessage(String title, String text, int type) {
		JOptionPane.showMessageDialog(null,text,title,type);
		
	}
	
	/**
	 * asks the user for a YES no QUESTION
	 * @param title
	 * @param text
	 * @return YES or NO
	 */
	public int showQuestionMessage(String title, String text) {
		return JOptionPane.showOptionDialog(null, text, title, JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);
	}
	
	/**
	 * shows an hourglass when a long task is being performed.
	 * It also restores the original cursor.
	 */
	public void showHourglass(boolean enable) {
		
		if(enable)
			this.setCursor(new java.awt.Cursor(java.awt.Cursor.WAIT_CURSOR));
		else
			this.setCursor(java.awt.Cursor.getDefaultCursor());
		
	}
	
	/**
	 * sets a customized text in the main jTextPane
	 * @param text
	 */
	public void setBodyText(String text) {
		this.jTextPane1.setText(text);
	}
	

	
	
	

}
