package org.modelgen.jpf.opgen.common;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * represents a MsmCarset, a set which identifies a K-Variant for a construct.
 * @author luigi, francesca
 *
 */
@Entity
@Table(name="MSM_CARSET")
public class MsmCarset implements Serializable {
	@Id
	private int oid;

	private String name;
	
	@ManyToOne
	@JoinColumn(name="CONSTRUCTOID")
	private MsmConstruct constructoid;

	@OneToMany(mappedBy="carsetoid")
	private Set<MsmCar> msmCarCollection;
	

	private static final long serialVersionUID = 1L;

	public MsmCarset() {
		super();
		
	}

	public int getOid() {
		return this.oid;
	}

	public void setOid(int oid) {
		this.oid = oid;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	
	public MsmConstruct getConstructoid() {
		return this.constructoid;
	}

	public void setConstructoid(MsmConstruct constructoid) {
		this.constructoid = constructoid;
	}
	

	public Set<MsmCar> getMsmCarCollection() {
		return this.msmCarCollection;
	}
	
	public Set<MsmReference> getMsmReferenceCollection() {
		

		/* msmReferenceCollection which is a derived field */
		/* masked by the manyToMany relationship between */
		/* CarSet and References. This relationship is */
		/* translated in the supermodel by means */
		/* of the relation MsmCar */
		Set<MsmReference> msmReferenceCollection = new HashSet<MsmReference>();
		for(MsmCar c: this.getMsmCarCollection())
			msmReferenceCollection.add(c.getReferenceoid());
		
		return msmReferenceCollection;
	}

	public void setMsmCarCollection(Set<MsmCar> msmCarCollection) {
		this.msmCarCollection = msmCarCollection;
	}
	
	/**
	 * returns the KVariant corresponding to this Carset
	 * @return the KVariant corresponding to this Carset
	 */
	public KVariant getKVariant() {
		KVariant kv = new KVariant(this,name,constructoid);
		
		return kv;
		
		
	}

}
