package org.modelgen.jpf.opgen.common;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
/**
 * represents a MsmConstruct
 * @author luigi, francesca
 *
 */
@Entity
@Table(name="MSM_CONSTRUCT")
public class MsmConstruct implements Serializable {
	@Id
	private int oid;

	private long islexical;

	private String name;
	
	@Transient
	private List<KVariant> variants;

	@OneToMany(mappedBy="ownerconstructoid")
	private Set<MsmReference> msmReferenceCollection;

	@OneToMany(mappedBy="targetconstructoid")
	private Set<MsmReference> msmReferenceCollection2;

	@OneToMany(mappedBy="constructoid")
	private Set<MsmSkolemfunction> msmSkolemfunctionCollection;

	@OneToMany(mappedBy="sourceconstructoid")
	private Set<MsmSkolemfunctionargument> msmSkolemfunctionargumentCollection;

	@OneToMany(mappedBy="constructoid")
	private Set<MsmProperty> msmPropertyCollection;

	@OneToMany(mappedBy="constructoid")
	private Set<MsmCarset> msmCarsetCollection;

	private static final long serialVersionUID = 1L;

	public MsmConstruct() {
		super();
	}

	public int getOid() {
		return this.oid;
	}

	public void setOid(int oid) {
		this.oid = oid;
	}

	public long getIslexical() {
		return this.islexical;
	}

	public void setIslexical(long islexical) {
		this.islexical = islexical;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Set<MsmReference> getMsmReferenceCollection() {
		return this.msmReferenceCollection;
	}

	public void setMsmReferenceCollection(Set<MsmReference> msmReferenceCollection) {
		this.msmReferenceCollection = msmReferenceCollection;
	}

	public Set<MsmReference> getMsmReferenceCollection2() {
		return this.msmReferenceCollection2;
	}

	public void setMsmReferenceCollection2(Set<MsmReference> msmReferenceCollection2) {
		this.msmReferenceCollection2 = msmReferenceCollection2;
	}

	public Set<MsmSkolemfunction> getMsmSkolemfunctionCollection() {
		return this.msmSkolemfunctionCollection;
	}

	public void setMsmSkolemfunctionCollection(Set<MsmSkolemfunction> msmSkolemfunctionCollection) {
		this.msmSkolemfunctionCollection = msmSkolemfunctionCollection;
	}

	public Set<MsmSkolemfunctionargument> getMsmSkolemfunctionargumentCollection() {
		return this.msmSkolemfunctionargumentCollection;
	}

	public void setMsmSkolemfunctionargumentCollection(Set<MsmSkolemfunctionargument> msmSkolemfunctionargumentCollection) {
		this.msmSkolemfunctionargumentCollection = msmSkolemfunctionargumentCollection;
	}

	public Set<MsmProperty> getMsmPropertyCollection() {
		return this.msmPropertyCollection;
	}

	public void setMsmPropertyCollection(Set<MsmProperty> msmPropertyCollection) {
		this.msmPropertyCollection = msmPropertyCollection;
	}

	public Set<MsmCarset> getMsmCarsetCollection() {
		return this.msmCarsetCollection;
	}

	public void setMsmCarsetCollection(Set<MsmCarset> msmCarsetCollection) {
		this.msmCarsetCollection = msmCarsetCollection;
	}
	
	/**
	 * returns the variant construct corresponding to the variants of this MsmConstruct
	 * @return the variant construct corresponding to the variants of this MsmConstruct
	 */
	public List<VariantConstruct> getVariantConstructs() {
		List<VariantConstruct> result = new LinkedList<VariantConstruct>();
		for(KVariant kv: this.variants)
			result.addAll(kv.getVariantConstruct());
		
		return result;
	}
	
	/**
	 * generates KVariants for this MsmConstruct
	 *
	 */
	public List<KVariant> generateKVariants() {
		
		if(variants==null) { // kVariants caching
			variants = new LinkedList<KVariant>();
			
			/* asks the characterizing set for the corresponding kvariant */
			KVariant kv = null;
			for(MsmCarset cs : msmCarsetCollection) {
				kv = cs.getKVariant();
				kv.setMsmConstruct(this);
				variants.add(kv);
			} 
			/* if there aren't msmCarset, generate a first level empty variant */
			if(msmCarsetCollection.isEmpty()) {
				/* first level kv have the MsmConstruct name */
				KVariant kvfl = new KVariant(null,name,this); 
				kvfl.setIsFirstLevel(true);
				variants.add(kvfl);
			}
		}
		
		return variants;
		
		
	}
	
	/**
	 * returns the names of the properties associated with the 
	 * MSM construct
	 * @return the names of the properties associated with the MSM construct
	 */
	public List<String> getPropertiesNames() {
		
		List<String> lista = new LinkedList<String>();
		for(MsmProperty p : this.msmPropertyCollection)
			lista.add(p.getName());
		return lista;
	}
	
	/**
	 * returns the skolemFunction with the given name
	 * @param name
	 * @return the skolemFunction with the given name
	 */
	public MsmSkolemfunction getSkolemFunctionByName(String name){
		MsmSkolemfunction sk1 = null;
		for(MsmSkolemfunction sk: this.msmSkolemfunctionCollection)
			if(sk.getName().equals(name))
				sk1 = sk;
		return sk1;		
	}
	
	/**
	 * two constructs are equal when they have the same name
	 * @param c
	 * @return equality condition
	 */
	public boolean equals(MsmConstruct c) {
		return this.name == c.name;
	}

}
