package org.modelgen.jpf.opgen.common;

import java.io.Serializable;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
/**
 * represents a MsmReference
 * @author luigi, francesca
 *
 */
@Entity
@Table(name="MSM_REFERENCE")
public class MsmReference implements Serializable {
	@Id
	private int oid;

	private String name;

	private String type;

	@ManyToOne
	@JoinColumn(name="OWNERCONSTRUCTOID")
	private MsmConstruct ownerconstructoid;

	@ManyToOne
	@JoinColumn(name="TARGETCONSTRUCTOID")
	private MsmConstruct targetconstructoid;

	@OneToMany(mappedBy="referenceoid")
	private Set<MsmCar> msmCarCollection;
	
	private static final long serialVersionUID = 1L;

	public MsmReference() {
		super();
		
	}

	public int getOid() {
		return this.oid;
	}

	public void setOid(int oid) {
		this.oid = oid;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getType() {
		return this.type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public MsmConstruct getOwnerconstructoid() {
		return this.ownerconstructoid;
	}

	public void setOwnerconstructoid(MsmConstruct ownerconstructoid) {
		this.ownerconstructoid = ownerconstructoid;
	}

	public MsmConstruct getTargetconstructoid() {
		return this.targetconstructoid;
	}
	
	public Set<MsmCarset> getMsmCarsetCollection() {
		
		Set<MsmCarset> msmCarsetCollection = new HashSet<MsmCarset>();	
		/* msmCarsetCollection which is a derived field */
		/* masked by the manyToMany relationship between */
		/* CarSet and References. This relationship is */
		/* translated in the supermodel by means */
		/* of the relation MsmCar */
	
		for(MsmCar c: this.getMsmCarCollection())
			msmCarsetCollection.add(c.getCarsetoid());
		
		return msmCarsetCollection;
	}

	public void setTargetconstructoid(MsmConstruct targetconstructoid) {
		this.targetconstructoid = targetconstructoid;
	}

	public Set<MsmCar> getMsmCarCollection() {
		return this.msmCarCollection;
	}

	public void setMsmCarCollection(Set<MsmCar> msmCarCollection) {
		this.msmCarCollection = msmCarCollection;
	}
	
	/**
	 * returns the variantConstructs of the MsmConstruct referred by
	 * this reference
	 * @return the variantConstructs of the MsmConstruct referred by this reference
	 */
	public List<VariantConstruct> getVariantConstructs() {
		return this.targetconstructoid.getVariantConstructs();
		
	}
	

}
