package org.modelgen.jpf.opgen.common;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.modelgen.jpf.opgen.ormHandling.ORMHandler;
import org.modelgen.regole.MacroRegola;
import org.modelgen.regole.RegolaDatalog;

/**
 * the Facade for the domain level
 * @author luigi, francesca
 *
 */
public class OpGen {
	
	private static final String SOURCE_3 = "[SOURCE_3]";
	private static final String DEST_1 = "[DEST_1]";
	private static final String SOURCE_2 = "[SOURCE_2]";
	private static final String SOURCE_1 = "[SOURCE_1]";
	
	private static OpGen instance;
	private Map<String,MsmConstruct> constructs;
	/* kvariants of all constructs */
	private List<KVariant> variants;
	/* variantConstructs of all constructs */
	private List<VariantConstruct> variantConstructs;
	
	
	public static OpGen getInstance() {
		if(instance==null)			
				instance = new OpGen();		
						
		return instance;
			
	}
	
	/**
	 * generates diff macrorule
	 *
	 */
	public MacroRegola generaDiff() {
		
		/* generates KVariant and VariantConstructs */
		generateVariants();
		/* generates the macrorule */
		MacroRegola macro = generateDiffMacroRule();
		
		
		return macro;
		
	}
	
	/**
	 * generates merge macrorule
	 * @return the merge macrorule
	 */
	public MacroRegola generaMerge() {

		/* generates KVariant and VariantConstructs */
		generateVariants();
		/* generates the macrorule */
		MacroRegola macro = generateMergeMacroRule();
				
		return macro;
		
	}
	
	
	
	/**
	 * generates diff macrorule
	 * @return the diff macrorule
	 */
	private MacroRegola generateDiffMacroRule() {
		
		MacroRegola mc = new MacroRegola();
		
		/* equivalence TMP */
		for(VariantConstruct vc: this.variantConstructs)
			/* collects TMP equivalence rules for each variant construct */
			mc.addRegola(vc.getRegolaTMP(SOURCE_1, SOURCE_2, DEST_1, false, ""));
		/* copyrule 1 */
		for(VariantConstruct vc: this.variantConstructs){
			mc.addRegola(vc.generateCopyRule(false, true));
			mc.addRegola(vc.generateCopyRule(true, true));
		}
		/* copyrule 2 */
		for(VariantConstruct vc: this.variantConstructs){
			mc.addRegola(vc.generateCopyRule(false, false));
			mc.addRegola(vc.generateCopyRule(true, false));
		}
		
		return mc;
			
		
		
	}
	
	/**
	 * generates merge macrorule
	 * @return the merge macrorule
	 */
	private MacroRegola generateMergeMacroRule() {
		
		MacroRegola mc = new MacroRegola();
		
		/* equivalence TMP (first rule) */
		for(VariantConstruct vc: this.variantConstructs) {
			/* collects TMP equivalence rules for each variant construct between S1 and S3'- */
			mc.addRegola(vc.getRegolaTMP(SOURCE_1, SOURCE_2, DEST_1, true, "_1"));
		}
		/* equivalence TMP (second rule) */
		for(VariantConstruct vc: this.variantConstructs) {		
			/* collects TMP equivalence rules for each variant construct between S1 and S3+' */
			mc.addRegola(vc.getRegolaTMP(SOURCE_1, SOURCE_3, DEST_1, true, "_2"));
		}
		
		/* copyrule 1 S1 -> S2 */
		for(VariantConstruct vc: this.variantConstructs) {		
			/* copies every construct from S1 to S2 if some condition below is satisfied */
			mc.addRegola(vc.getMergeCopyRule(CopyRuleCase.NotPresentNotPresent));
			mc.addRegola(vc.getMergeCopyRule(CopyRuleCase.NotPresentStub));
			mc.addRegola(vc.getMergeCopyRule(CopyRuleCase.StubNotPresent));
			mc.addRegola(vc.getMergeCopyRule(CopyRuleCase.StubStub));
		}
		
		/* copyrule 2 S3'+ -> S2 */
		/* for each variant construct retrieves the list of the */
		/* corresponding copy rules */
		List<RegolaDatalog> regoleList = new LinkedList<RegolaDatalog>();
		for(VariantConstruct vc: this.variantConstructs) 
			regoleList.addAll(vc.getCopyRules2List());
			
		/* adds every generated rule to the macrorule */
		for(RegolaDatalog rd: regoleList)
			mc.addRegola(rd);
		
		
			
		return mc;
	}
	
	
	
	/**
	 * generates KVariante and CostruttoVariante lists
	 *
	 */
	private void generateVariants() {
		
		if(variants==null) { // variants caching 
			variants = new LinkedList<KVariant>();
			
			/* generate variants for every construct */
			Set<String> keys = constructs.keySet();
			for(String k: keys)
				variants.addAll(constructs.get(k).generateKVariants());
			
			/* sorts kvariants according to references */
			KVariantCollections kvc = new KVariantCollections();
			
			variants = kvc.topologicalSorting(variants);
			//Collections.sort(variants);
		
		} 
		
	
		
		
		if(variantConstructs==null) { // variant constructs caching
			
			variantConstructs = new LinkedList<VariantConstruct>();
			/* generate variant construct for each kvariant */
			for(KVariant kv: variants) {
				variantConstructs.addAll(kv.getVariantConstruct());
			}
			
		}
		
	}
	
	private OpGen() {
		/* initializes constructs map */
		try {
			constructs = ORMHandler.getInstance().getConstructMap();			
		} catch(Exception ex) {
			System.out.println("OpGen: Exception occurred loading PersistenceHandler");
			System.out.println("Message: "+ ex.getMessage());
			System.out.println("OpGen: Constructs set will be empty");
			constructs = new HashMap<String,MsmConstruct>();
		}
		
			
	}
	
	public String toString() {
		
		StringBuffer sb = new StringBuffer();
		
		sb.append("KVariants \n");
		for(KVariant kv: this.variants) 
			sb.append(kv.toString()+"\n");
		
		sb.append("Variant Constructs: \n");
		for(VariantConstruct vc: this.variantConstructs)
			sb.append(vc.toString()+"\n");
		
		return sb.toString();
	}

}
