
package org.modelgen.jpf.opgen.ormHandling;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;


import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.regole.MacroRegola;
import org.modelgen.regole.RegolaDatalog;

/**
 * saver for macrorules
 * @author luigi, francesca
 *
 */
public class MacroSaver {
	
	private static final String MACRORULE_EXTENSION = ".txt";
	private static final String DATALOG_EXTENSION = ".dtr";
	private static final String PATH_SEPARATOR = "/";
	private static final String NAME_SEPARATOR = "_";	
	private MacroRegola mr;
	
	public MacroSaver(MacroRegola mr) {
		this.mr = mr;		
		
	}
	
	/**
	 * saves a macrorule given its name
	 * @param name
	 */
	public void save(String name) {
		
		// Uncomment for JUnit Test
		// String simplePath = " [...]/ModelGenJPF_Application/ModelGenApplicationEnvironment/modelgenDATA/rules/simplerules/";
		// String macroPath = "[...]/ModelGenJPF_Application/ModelGenApplicationEnvironment/modelgenDATA/rules/macrorules/";
		
		/* fetch paths from configuration */
		
		String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
	    String macroRulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
	    String simpleRulePath = ModelgenConfigurationManager.getIstance().getProperty("simplerules", "", "paths");
		
		String macroPath = rootnode + macroRulePath;
		String simplePath = rootnode + simpleRulePath;
		
		/* for rule's toString() */
		String regola;
		byte buffer[];
		
		/* streams */
		OutputStream o = null, o1 = null;
		int contatore = 1;
		
		/* creates macrorule directory */		
		File f = new File(simplePath + name);
			
		f.mkdir();
		File f1 = null,f2 = null;
				
		List<RegolaDatalog> regole = mr.getRegole();
		
		
		/* stream for macrorule file */
		try {
			/* macrorule file */
			f2 = new File(macroPath + name + MACRORULE_EXTENSION);
			/* stream for macrorule file */
			o1 = new FileOutputStream(f2);
		} catch(FileNotFoundException ex) {
			System.out.println("OPGEN: File not found");
		}
		
		for(RegolaDatalog r : regole) {
			regola = r.toString();
			buffer = regola.getBytes();
			
			try {
				/* simple rule file */
				f1 = new File(simplePath + name + PATH_SEPARATOR + name + NAME_SEPARATOR + r.getTesta().getPredicateName().toLowerCase() + NAME_SEPARATOR + contatore + DATALOG_EXTENSION);
											
				/* stream for simple rule file */
				o = new FileOutputStream(f1);
			} catch(FileNotFoundException ex) {
				System.out.println("OPGEN: Error creating simple rule file");
			}
			
			
			/* write each entry for the macrorule (the name is composed by the */
			/* macro rulename and first literal predicate name) */
			try {
				saveSimpleRule(name,name + NAME_SEPARATOR + r.getTesta().getPredicateName().toLowerCase() , buffer, o, o1, contatore);
			} catch(IOException ex) {
				System.out.println("OPGEN: Error writing datalog rule file");
			}
			
			contatore++;
		}
		
		/* close streams */
		try {
			if(o!=null)
				o.close();
			if(o1!=null)
				o1.close();
		} catch(IOException ex) {
			System.out.println("OPGEN: Error writing datalog rule file");
		}
		
				
	}

	/**
	 * writes a single simple rule invokation in a macrorule and the corresponding simplerule
	 * @param name
	 * @param path
	 * @param buffer
	 * @param o
	 * @param o1
	 * @param contatore
	 * @throws IOException
	 */
	private void saveSimpleRule(String path, String name, byte[] buffer, OutputStream o, OutputStream o1, int contatore) throws IOException {
		/* write simple rule */
		o.write(buffer, 0, buffer.length);
		/* write macrorule entry */
		o1.write((path +  PATH_SEPARATOR + name + NAME_SEPARATOR + contatore + DATALOG_EXTENSION+"();\n").getBytes());
	}
	
}

