package org.modelgen.jpf.opgen.ormHandling;

import java.util.HashMap;
import java.util.Map;

import org.modelgen.jpf.opgen.common.MsmConstruct;

/**
 * Manages ORM persistence. Implements functions to access the supermodel.
 * @author luigi, francesca
 */
public class ORMHandler {
		
	/* Singleton access */
	private static ORMHandler ph;
	
	/* the entity manager */
	private EntityManagerProxy em;
	
	/* to prevent construction */
	private ORMHandler() {
		
		/* initializes entity manager with PU persistence unit */
		/* persistence.xml stores information about it and it is in META-INF/persistence.xml */
		/* build.xml packages it into meta.jar and copies this file to modelgen libs. */
		/* run.sh includes meta.jar in the classpath. */
		try {
			em = ORMProxy.getInstance().createEntityManagerFactoryProxy("PU").createEntityManagerProxy();
		} catch(SwitchingException se) {
			this.die();
		}
	}

		
	public static ORMHandler getInstance() {
		if(ph==null)
			ph = new ORMHandler();
		return ph;
		
	}
	
	/**
	 * returns name to MsmConstruct map.
	 * @return  name to MsmConstruct map
	 */
	public Map<String,MsmConstruct> getConstructMap() {
		
		Map<String,MsmConstruct> consMap = new HashMap<String,MsmConstruct>();
		
		int OID = 0;
		
		MsmConstruct cons = null;
		
		/* extracts construct until OID denotes a valid primary key */
		do {
			try {
				cons = em.find(MsmConstruct.class,OID);
			} catch(SwitchingException se) {
				this.die();
			}
			if(cons!=null)
				consMap.put(cons.getName(),cons);
			OID++;
		} while(cons!=null);
		
		
		
		
		return consMap;
		
	}
	
	/**
	 * destroys this persistence handler	 
	 */
	public void die() {
		ph = null;
	}
	

}
