package org.modelgen.jpf.opgen.ormHandling;

/**
 * behavior to perform a ClassLoader switch to be
 * shared among proxies.
 * @author luigi, francesca
 *
 */
public abstract class Switcher {
	
	/* display debug output or not */
	final private boolean DEBUG = false;
	
	/* to store ClassLoader to restore after switching. Initialized with default */
	/* ClassLoader of the current thread */
	protected ClassLoader threadLoader = Thread.currentThread().getContextClassLoader();
	
	public Switcher(ClassLoader threadLoader) {
		this.threadLoader = threadLoader;
	}
	
	/**
	 * switches between default java ClassLoader and PluginClassLoader. When invoking
	 * JPA libraries, the thread must use default loader, for every other operation
	 * the plugin loader is needed. This method has to be invoked at the beginning and 
	 * at the end of each JPA operation	 
	 */
	protected final void switchClassLoader() {
		
		if(DEBUG) {
			System.out.println("DEBUG INFO: My ClassLoader is "+ this.getClass().getClassLoader());	
			System.out.println("DEBUG: CURRENT THREAD Classloader is "+ Thread.currentThread().getContextClassLoader());
		}
		
		ClassLoader pluginLoader = this.getClass().getClassLoader();
		
		if(!Thread.currentThread().getContextClassLoader().equals(pluginLoader))
			Thread.currentThread().setContextClassLoader(pluginLoader);
		else
			Thread.currentThread().setContextClassLoader(this.threadLoader);
			
		/* this allows toplink to use PluginClassLoader instead of standard java ClassLoader */
		/* whithout this line toplink cannot find classes belonging to OpGen plugin */	
		
	}
	

}
