package org.modelgen.jpf.signatureApplication.applicationView;

import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.jpf.signatureApplication.applicationView.action.ActionListenerCmb;
import org.modelgen.signature.ModelSignature;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Pannello per prendere in input lo schema sorgente e il modello target.
 * Contiene anche due aree di testo per mostrare le segnature degli schemi scelti.
 * @author Fabrizio Celli
 */
public class PanelInfo extends JPanel {
	private static final long serialVersionUID = -3293928973480210757L;
	/* Singleton */
	private static PanelInfo instance;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Components for gui
	protected JComboBox cmbSchemes;
	protected JComboBox cmbModels;
	protected JTextArea scheme;
	protected JTextArea model;
	//listener per combo
	private ActionListenerCmb alCmb;
	//deve conoscere
	private ModelSignature source = null;
	private ModelSignature target = null;
	private PanelRules prule;

	//metodi getter
	public JComboBox getCmbSchemes() {
		return cmbSchemes;
	}

	public JComboBox getCmbModels() {
		return cmbModels;
	}
	
	public JTextArea getModel() {
		return model;
	}

	public JTextArea getScheme() {
		return scheme;
	}

	/**
	 * Class constructor
	 */
	private PanelInfo() {
		this.alCmb = new ActionListenerCmb(this);
		try {
			this.cmbSchemes = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbModel());
					//GeneralManagementFacade.getInstance().populateCmbScheme("superModel"));
			this.cmbModels = new JComboBox(
					GeneralManagementFacade.getInstance().populateCmbModel());
		} catch (Exception e) {}
		this.cmbSchemes.addActionListener(alCmb);
		this.cmbModels.addActionListener(alCmb);
		this.initializePanel();
	}

	/**
	 * Return an instance of the class
	 * @return  PanelInfo
	 * @uml.property  name="instance"
	 */
	public static PanelInfo getInstance() {
		/* Singleton */
		if(instance == null)
			instance = new PanelInfo();
		return instance;
	}

	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}

	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlGeneralInfo = new JPanel();	
		// Create panel general info
		pnlGeneralInfo.setLayout(new GridBagLayout());
		pnlGeneralInfo.setBorder(
				new javax.swing.border.TitledBorder(
				"General info for signature application"));
		GridBagConstraints consGeneralInfo = new GridBagConstraints();
		// Set the parameter for the GrigBagConstrains
		consGeneralInfo.fill = GridBagConstraints.HORIZONTAL;
		consGeneralInfo.anchor = GridBagConstraints.NORTH;
		consGeneralInfo.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 0;
		consGeneralInfo.gridwidth = 3;
		consGeneralInfo.anchor = GridBagConstraints.EAST;
		pnlGeneralInfo.add(this.cmbSchemes, consGeneralInfo);
		// Component in (3,0)
		consGeneralInfo.gridx = 3;
		consGeneralInfo.gridy = 0;
		consGeneralInfo.gridwidth = 3;
		consGeneralInfo.anchor = GridBagConstraints.WEST;
		pnlGeneralInfo.add(this.cmbModels, consGeneralInfo);
		// Component in (0,3)
		consGeneralInfo.gridx = 0;
		consGeneralInfo.gridy = 3;
		consGeneralInfo.gridwidth = 3;

		JScrollPane scrScheme = new JScrollPane(this.scheme,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrScheme.setPreferredSize(new Dimension(350, 200));
		
		pnlGeneralInfo.add(scrScheme, consGeneralInfo); 
		// Component in (2,2)
		consGeneralInfo.gridx = 3;
		consGeneralInfo.gridy = 3;
		consGeneralInfo.gridwidth = 3;

		JScrollPane scrModel = new JScrollPane(this.model,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrModel.setPreferredSize(new Dimension(350, 200));
		
		pnlGeneralInfo.add(scrModel, consGeneralInfo);  
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlGeneralInfo, this.generalLayout);		
	}

	/**
	 * Initialize the panel
	 */
	private void initializePanel() {
		//Create a text area.
		this.scheme = new JTextArea("no source scheme selected");
		this.scheme.setPreferredSize(new Dimension(250, 220));
		this.scheme.setFont(new Font("Serif", Font.ROMAN_BASELINE, 12));
		this.scheme.setLineWrap(true);
		this.scheme.setWrapStyleWord(true);
		this.scheme.setEditable(false);
		
		this.model = new JTextArea("no target model selected");
		this.model.setFont(new Font("Serif", Font.ROMAN_BASELINE, 12));
		this.model.setPreferredSize(new Dimension(250, 220));
		this.model.setLineWrap(true);
		this.model.setWrapStyleWord(true);
		this.model.setEditable(false);
	}

	/**
	 * Return the selected index instance
	 * @return int
	 */
	protected int getSchemaSelectedIndex() {
		return this.cmbSchemes.getSelectedIndex();
	}

	/**
	 * Return the selected index of the scheme
	 * @return int
	 */
	protected int getModelSelectedIndex() {
		return this.cmbModels.getSelectedIndex();
	}

	/**
	 * Return the instance name
	 * @return String
	 */
	public String getSelectedSchema() {
		return (String)this.cmbSchemes.getSelectedItem();
	}

	/**
	 * Return the scheme name
	 * @return String
	 */
	public String getSelectetModel() {
		return (String)this.cmbModels.getSelectedItem();
	}

	//setter e getter
	public ModelSignature getSource() {
		return source;
	}

	public void setSource(ModelSignature source) {
		this.source = source;
		if(this.source!=null && this.target!=null)
			this.prule.createListUsefulRules();
	}

	public ModelSignature getTarget() {
		return target;
	}

	public void setTarget(ModelSignature target) {
		this.target = target;
		if(this.source!=null && this.target!=null)
			this.prule.createListUsefulRules();
	}
	
	/**
	 * pattern Observer: la PanelRules si abbona a questo oggetto
	 */
	public void addPanelRule(PanelRules p){
		this.prule = p;
	}
}