package org.modelgen.jpf.signatureApplication.applicationView;

import org.modelgen.jpf.signatureApplication.applicationView.action.ActionListenerRules;
import org.modelgen.jpf.signatureApplication.applicationView.action.MouseListenerRules;
import org.modelgen.jpf.signatureApplication.facade.SignatureApplicationFacade;
import org.modelgen.jpf.signatureApplication.load.FactoryIcon;
import org.modelgen.signature.DiffContainer;
import org.modelgen.signature.MacroRuleSignature;
import org.modelgen.signature.ModelSignature;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * @author Fabrizio Celli
 * Pannello per visualizzare le macroregole applicabili e gli effetti
 * della loro applicazione
 */
public class PanelRules extends JPanel {
	private static final long serialVersionUID = 8663841166780574529L;
	// Singleton
	private static PanelRules instance;
	// References to pnlInfo
	private PanelInfo pnlInfo;
	// Layout manager
	private GridBagConstraints generalLayout;
	// Action listener
	private ActionListener lstAction;
	// Mouse listener
	private MouseListener lstMouse;
	// Components for gui
	private JLabel lblRules;
	// Components for gui
	protected JList lstRules;
	protected JList lstUserRules;
	protected JButton btnAdd;
	protected JButton btnExecute;
	protected JButton btnRemove;
	protected JButton btnFinish;
	
	//da conoscere
	protected Map<MacroRuleSignature, DiffContainer> mappaUtili;

	//metodi getter
	public JButton getBtnAdd() {
		return btnAdd;
	}

	public JButton getBtnExecute() {
		return btnExecute;
	}

	public JButton getBtnFinish() {
		return btnFinish;
	}

	public JButton getBtnRemove() {
		return btnRemove;
	}

	public JList getLstRules() {
		return lstRules;
	}

	public JList getLstUserRules() {
		return lstUserRules;
	}

	/**
	 * Class constructor
	 * @param pnlInfo 
	 */
	private PanelRules(PanelInfo pnlInfo) {
		super();
		this.pnlInfo = pnlInfo;
		// Listener
		this.lstAction = new ActionListenerRules(this);
		this.lstMouse = new MouseListenerRules(this);
		// Creates components
		this.lblRules = new JLabel("Select the macrorule to apply");
		this.lstRules = new JList();
		this.lstRules.addMouseListener(this.lstMouse);
		this.lstUserRules = new JList();
		this.lstUserRules.addMouseListener(this.lstMouse);
		this.btnAdd = new JButton(FactoryIcon.createAddIcon());
		this.btnAdd.addActionListener(this.lstAction);
		this.btnAdd.setToolTipText("Add macrorule information");
		this.btnRemove = new JButton(FactoryIcon.createRemoveIcon());
		this.btnRemove.addActionListener(this.lstAction);
		this.btnRemove.setToolTipText("Deselect macrorule");
		this.btnExecute = new JButton("Execute");
		this.btnExecute.setToolTipText("Execute");
		this.btnExecute.addActionListener(this.lstAction);
		this.btnFinish = new JButton("Finish");
		this.btnFinish.setToolTipText("Finish");
		this.btnFinish.addActionListener(this.lstAction);
	}


	/**
	 * Return an instance of the class
	 * @param pnlInfo 
	 * @return PanelRules
	 */
	public static PanelRules getInstance(PanelInfo pnlInfo) {
		/* Singleton */
		if(instance == null)
			instance = new PanelRules(pnlInfo);
		return instance;
	}

	/**
	 * Sets the general layout
	 */
	public void setLayout() {
		// Setto il layout
		this.setLayout(new GridBagLayout());
		this.generalLayout = new GridBagConstraints();
		// Setto i parametri
		this.generalLayout.insets = new Insets(5,5,5,5);
		this.generalLayout.fill = GridBagConstraints.HORIZONTAL;
		this.generalLayout.anchor = GridBagConstraints.NORTH;
		this.generalLayout.weightx = 1;
		this.generalLayout.weighty = 0;
	}

	/**
	 * Create and show the component of the panel
	 */
	public void createShowPanel() {
		JPanel pnlAddRules = new JPanel();
		JPanel pnlBtnRules;	 
		//inizializzo le regole
		this.initializeRules();
		// Panel argument
		pnlAddRules.setLayout(new GridBagLayout());
		pnlAddRules.setBorder(
				new javax.swing.border.TitledBorder(
				"View macrorule signature application effects"));

		// Set the parameter for the GrigBagConstrains
		GridBagConstraints consAddRules = new GridBagConstraints();
		consAddRules.fill = GridBagConstraints.NONE;
		consAddRules.anchor = GridBagConstraints.NORTH;
		consAddRules.insets = new Insets(5,5,5,5);
		// Component in (0,0)
		consAddRules.gridx = 0;
		consAddRules.gridy = 0;
		consAddRules.anchor = GridBagConstraints.EAST;
		pnlAddRules.add(this.lblRules, consAddRules);
		// Component in (0,1)
		consAddRules.gridx = 0;
		consAddRules.gridy = 1;
		consAddRules.anchor = GridBagConstraints.CENTER;
		consAddRules.fill = GridBagConstraints.BOTH;
		consAddRules.gridheight = 4;
		consAddRules.gridwidth = 1;
		JScrollPane scrRule = new JScrollPane(this.lstRules,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrRule.setPreferredSize(new Dimension(320, 180));
		pnlAddRules.add(scrRule, consAddRules);
		// Panel btn rules
		pnlBtnRules = new JPanel();
		pnlBtnRules.setLayout(new GridBagLayout());
		GridBagConstraints consBtnRules = new GridBagConstraints();
		// Component in (0,0)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 0;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnAdd, consBtnRules);
		// Component in (0,1)
		consBtnRules.gridx = 0;
		consBtnRules.gridy = 1;
		consBtnRules.anchor = GridBagConstraints.CENTER;
		pnlBtnRules.add(this.btnRemove, consBtnRules);

		// Component in (1,1)
		consAddRules.gridx = 1;
		consAddRules.gridy = 1;
		pnlAddRules.add(pnlBtnRules, consAddRules);
		// Component in (2,1)
		consAddRules.gridx = 2;
		consAddRules.gridy = 1;
		consAddRules.gridheight = 2;
		consAddRules.gridwidth = 1;
		consAddRules.fill = GridBagConstraints.BOTH;
		consAddRules.anchor = GridBagConstraints.CENTER;
		JScrollPane scrUserRule = new JScrollPane(this.lstUserRules,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrUserRule.setPreferredSize(new Dimension(320, 180));
		pnlAddRules.add(scrUserRule, consAddRules);
		// Component in (1,5)
		consAddRules.gridx = 1;
		consAddRules.gridy = 5;
		consAddRules.anchor = GridBagConstraints.CENTER;
		pnlAddRules.add(this.btnExecute, consAddRules);
		//	Component in (2,5)
		consAddRules.gridx = 2;
		consAddRules.gridy = 5;
		consAddRules.anchor = GridBagConstraints.CENTER;
		pnlAddRules.add(this.btnFinish, consAddRules);
		// Add to the main panel
		this.generalLayout.gridx = 0;
		this.generalLayout.gridy = 0;
		this.add(pnlAddRules, this.generalLayout);		
	}

	/*
	 * Inizializza i pannelli con le regole
	 */
	private void initializeRules(){
		DefaultListModel listModel =  new DefaultListModel();
		DefaultListModel listModelUser = new DefaultListModel();
		this.lstRules.setModel(listModel);
		this.lstUserRules.setModel(listModelUser);
	}

	/**
	 * Renew rules list
	 */
	public void createListUsefulRules() {
		//select source and target
		ModelSignature source = this.pnlInfo.getSource();
		ModelSignature target = this.pnlInfo.getTarget();
		this.createList(source, target);
		//svuota la lista delle macroregole scelte
		SignatureApplicationFacade.getInstance().resetScelte();
	}

	private void createList(ModelSignature source, ModelSignature target){
		//Creates the default list model
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		modelUser.removeAllElements();
		DefaultListModel model =  (DefaultListModel)this.lstRules.getModel();
		model.removeAllElements();
		SignatureApplicationFacade facade = SignatureApplicationFacade.getInstance();
		this.mappaUtili = facade.viewUsefulMacro(source, target);
		// Take the rules names
		Collection<MacroRuleSignature> regole = mappaUtili.keySet();
		for(MacroRuleSignature regola: regole){
			model.addElement(regola.getFileName());
		}
		this.deselectMacroRules();
	}

	/**
	 * Seleziona la macroregola, visualizzando una anteprima
	 * dei suoi effetti
	 */
	public void viewMacroRuleEffect(){
		// Selected objects
		Object[] selectedRules = this.lstRules.getSelectedValues();
		String regola = (String) selectedRules[0];
		DefaultListModel userModel = ((DefaultListModel)this.lstUserRules.getModel());
		userModel.removeAllElements();
		userModel.addElement("SELECTED MACRORULE: ");
		userModel.addElement(regola);
		userModel.addElement(" ");
		userModel.addElement("CURRENT MODEL: ");
		ModelSignature current = SignatureApplicationFacade.getInstance().getSource();
		List<String> costruttiCurrent = current.getCostruttiAsStringhe();
		for(String costrutto: costruttiCurrent)
			userModel.addElement(costrutto);
		if(SignatureApplicationFacade.getInstance().compatibili())
			userModel.addElement("------matches with target model!!!");
		userModel.addElement(" ");
		MacroRuleSignature macro = null;
		macro = SignatureApplicationFacade.getInstance().getMacroRegola(regola);
		DiffContainer container = this.mappaUtili.get(macro);
		if(container!=null){
			userModel.addElement("ADDED CONSTRUCTS: ");
			userModel.addElement(container.getAggiunti());
			userModel.addElement(" ");
			userModel.addElement("REMOVED CONSTRUCTS: ");
			userModel.addElement(container.getRimossi());
			userModel.addElement(" ");
			userModel.addElement("IGNORED CONSTRUCTS WITH THEIR IGNORED PROPERTIES: ");
			Map<String, String> ignorate = container.getIgnorate();
			Collection<String> nomiCostrutti = ignorate.keySet();
			for(String nome: nomiCostrutti)
				userModel.addElement("{"+nome+": "+ignorate.get(nome)+"}");
			userModel.addElement(" ");
			userModel.addElement("REACHABLE MODEL: ");
			ModelSignature newSource = container.getArrivo();
			List<String> costruttiNewSource = newSource.getCostruttiAsStringhe();
			for(String costrutto: costruttiNewSource)
				userModel.addElement(costrutto);
			userModel.addElement(" ");
			userModel.addElement("TARGET MODEL: ");
			ModelSignature target = SignatureApplicationFacade.getInstance().getTarget();
			List<String> costruttiTarget = target.getCostruttiAsStringhe();
			for(String costrutto: costruttiTarget)
				userModel.addElement(costrutto);
			if(SignatureApplicationFacade.getInstance().isCompatibile(newSource))
				userModel.addElement("->->->matches with target model");
			else
				userModel.addElement("does not match with target model");
		}
		this.updateUI();
	}

	/**
	 * Deseleziona la macroregola selezionata, svuotando la lista con le informazioni
	 */
	public void deselectMacroRules() {
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		modelUser.removeAllElements();
		if(SignatureApplicationFacade.getInstance().getSource()!=null){
			modelUser.addElement("CURRENT MODEL: ");
			ModelSignature source = SignatureApplicationFacade.getInstance().getSource();
			List<String> costruttiSource = source.getCostruttiAsStringhe();
			for(String costrutto: costruttiSource)
				modelUser.addElement(costrutto);
			if(SignatureApplicationFacade.getInstance().compatibili())
				modelUser.addElement("------matches with target model!!!");
		}
		this.updateUI();
	}

	/**
	 * Applicazione della macroregola selezionata: modifica la vista, rinnovando la 
	 * lista delle macroregole applicabili al modello raggiunto
	 */
	public void applyRules() {
		// Selected macrorules
		Object[] selectedRules = this.lstRules.getSelectedValues();
		if(selectedRules!=null){
			String regola = (String) selectedRules[0];
			MacroRuleSignature macro = null;
			macro = SignatureApplicationFacade.getInstance().getMacroRegola(regola);
			DiffContainer container = this.mappaUtili.get(macro);
			ModelSignature newSource = container.getArrivo();
			ModelSignature target = this.pnlInfo.getTarget();
			SignatureApplicationFacade.getInstance().addMacroScelta(macro);
			this.createList(newSource, target);
		}
	}

	/**
	 * Restituisce le scelte fatte, cio il percorso che ha portato alla scelta 
	 * delle macroregole
	 */
	public void finish() {
		List<MacroRuleSignature> scelte = SignatureApplicationFacade.getInstance().getScelte();
		DefaultListModel modelUser = (DefaultListModel)this.lstUserRules.getModel();
		modelUser.removeAllElements();
		DefaultListModel model =  (DefaultListModel)this.lstRules.getModel();
		model.removeAllElements();
		if(SignatureApplicationFacade.getInstance().getSource()!=null){
			modelUser.addElement("CURRENT MODEL: ");
			ModelSignature source = SignatureApplicationFacade.getInstance().getSource();
			List<String> costruttiSource = source.getCostruttiAsStringhe();
			for(String costrutto: costruttiSource)
				modelUser.addElement(costrutto);
			if(SignatureApplicationFacade.getInstance().compatibili())
				modelUser.addElement("------matches with target model!!!");
			modelUser.addElement(" ");
		}
		if(scelte!=null){
			modelUser.addElement("YOU HAVE SELECTED THE FOLLOWING MACRORULES:");
			for(MacroRuleSignature macro: scelte){
				String nameMacro = macro.getFileName();
				if(nameMacro!=null)
					modelUser.addElement(nameMacro);
			}
			SignatureApplicationFacade.getInstance().resetScelte();
		}
		else
			modelUser.addElement("-: NO MACRORULES EXECUTED :-");
	}


}
