package org.modelgen.jpf.visualizer;

import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.core.commons.Tool;
import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.pac.TopLevelPAC;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

/**
 * Class <code>VisualizerTool</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class VisualizerTool extends JInternalFrame implements Tool {
	private static final long serialVersionUID = 1L;
	private Map<String, Object> applicationObjects;

	public VisualizerTool() {
		super("VisualizerTool",true,true,true,true);
		initProperties();
		add(new JPanel(), BorderLayout.CENTER); // di base vuoto
		TopLevelPAC.setParent(this);
		TopLevelPAC top = TopLevelPAC.getInstance();
		JComponent view = top.getView();
		if(view != null) {
			add(view, BorderLayout.CENTER);
		}
		super.toBack();

		/* Imposta icona */
		ImageIcon icon = ToolConfigurationManager.sharedInstance.getIcon("tool_.png");
		if (icon != null) this.setFrameIcon(icon);

		/* A ogni attivazione della finestra, nel VisualizerTool
		 * corrente deve essere attivato il focus */
		final Frame frame = ToolBoxFrame.getIstance();
		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowActivated(WindowEvent e) {
				VisualizerTool.this.requestFocus();
			}
		});
		super.addFocusListener(new FocusAdapter() {
			@Override
			public void focusGained(FocusEvent e) {
				//System.out.println(e);
				for (java.awt.Component child : VisualizerTool.this.getContentPane().getComponents()) {
					//System.out.println("Focus in "+child);
					child.requestFocus();
				}
			}
		});
	}

	public Map<String, Object> getInfo() {
		return this.applicationObjects;
	}

	public void init(Map<String, Object> applicationObjects) {
		this.applicationObjects  = applicationObjects;
	}

	private void initProperties() {
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		Dimension dim = getToolkit().getScreenSize();
		int larghezza = (int) (dim.getWidth()*4/5);
		int altezza = (int) (dim.getHeight()*3/4);
		int bordoSinistro = (int) ((dim.getWidth() - larghezza)/2);
		int bordoSuperiore = (int) ((dim.getHeight()-altezza)/4)-altezza/22;
		setBounds(bordoSinistro,bordoSuperiore,larghezza,altezza);
	}

	public String toString(){
		return "VisualizerTool";
	}


}
