package org.modelgen.jpf.visualizer.debugger;

import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;
import org.modelgen.commons.persistence.database.DBHandler;
import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.SQLException;

import org.hsqldb.Server;
import org.hsqldb.util.ShutdownServer;

/**
 * Class <code>DatabaseClone</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
class DatabaseClone {
    private String newServerUrl;
    private String oldServerUrl;
    private DBHandler oldPersistenceHandler;

    public DatabaseClone() throws Exception {
	try {
		this.oldPersistenceHandler = (DBHandler) PersistenceHandlerFactory.getPersistenceHandler();
		
	    // get info
	    this.oldServerUrl = ModelgenConfigurationManager.getIstance().getProperty("serverUrl","","jdbc");

	    // shutdown current server
	    ShutdownServer.main(new String[]{"-database",oldServerUrl});
	    
	    // prepare to open a clone server
	    File basePathFile = ToolConfigurationManager.sharedInstance.getDatabaseClonePath();
	    String basePath;
	    if (basePathFile != null) {
		basePath = basePathFile.getAbsolutePath();
	    } else {
		basePath = "."; // directory corrente
	    }
	    this.newServerUrl = basePath + File.separator + "clone";

	    // ensure clone server deleted
	    deleteDataBase(newServerUrl);

	    // copy old server data
	    copyDatabase(oldServerUrl, newServerUrl);

	    // open the clone server
	    Server.main(new String[]{"-database",newServerUrl,"-no_system_exit","true"});
	    
	    PersistenceHandlerFactory.reload();
	} catch (Exception e) {
	    this.resetOriginalDatabase();
	    throw e;
	} 
    }


    public void deleteDataBase(String serverUrl){
	System.out.println("********");
	File dataBase = (new File(serverUrl)).getParentFile();
	String prefix = (new File(serverUrl)).getName();
	prefix += ".";
	for (String file : dataBase.list()){
	    // per ogni serverUrl.* elimina
	    if (file.startsWith(prefix)) {
		System.out.println(" - "+file + " deleted ["+(new File(dataBase,file)).delete()+"]");
	    }
	}
	System.out.println("********");
    }


    private void copyDatabase(String serverUrl, String newServerUrl) throws IOException {
	File oldDatabase = (new File(serverUrl)).getParentFile();
	String prefix = (new File(serverUrl)).getName();
	prefix += ".";
	for (String file : oldDatabase.list()){
	    // per ogni serverUrl.* copia in newServerUrl.*
	    if (file.startsWith(prefix)) {
		String extension = file.substring(prefix.length());
		String copyPath = newServerUrl + "." + extension;
		String sourcePath = oldDatabase + File.separator + file;
		File targetFile = new File(copyPath);
		File sourceFile = new File(sourcePath);
		copyFile(sourceFile, targetFile);
		System.out.println(" - "+sourcePath + " copied in " + copyPath);
	    }
	}

    }

    public void copyFile(File in, File out) throws IOException {
	FileInputStream fis = null;
	FileOutputStream fos = null;
	try {
	    fis  = new FileInputStream(in);
	    fos = new FileOutputStream(out);
	    byte[] buf = new byte[1024];
	    int i = 0;
	    while((i=fis.read(buf))!=-1) {
		fos.write(buf, 0, i);
	    }
	} catch (IOException e) {
	    throw e;
	} finally {
	    if (fis != null) fis.close();
	    if (fos != null) fos.close();
	}
    }

    public void resetOriginalDatabase() throws ClassNotFoundException, SQLException {
	 // shutdown current server
	 ShutdownServer.main(new String[]{"-database",newServerUrl});
	 
	 // restart old server
	 Server.main(new String[]{"-database",oldServerUrl,"-no_system_exit","true"});
	 
	 PersistenceHandlerFactory.resetDBHandler(this.oldPersistenceHandler);
    }

    public PersistenceHandler getPersistenceHandler() throws ClassNotFoundException, Exception {
	return PersistenceHandlerFactory.getPersistenceHandler();
    }
}
