package org.modelgen.jpf.visualizer.debugger;

import org.modelgen.commons.persistence.database.DBConnecter;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.util.Pair;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Class <code>SkolemHelper</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class SkolemHelper {
	public static final String PREFIX_SM = "SM_";
	public static final String SUFFIX_SKOLEM = "_SK";

	/**
	 * Calcola i collegamenti che si vengono a creare tramite
	 * le funzioni di skolem
	 * 
	 * @param targetConstruct il costrutto destinazione
	 * @param sourceConstructs i possibili costrutti sorgente
	 * @return la coppia [weakLinks,hardLink], dove i
	 * weakLinks rappresentano i collegamenti riconosciuti solo
	 * tramite oid, mentre l'hardLink e' quel particolare weakLink 
	 * che lessicograficamente risulta piu' probabile 
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 */
	public static Pair<Collection<AbstractConstruct>, AbstractConstruct> calculateLinks(AbstractConstruct targetConstruct, Collection<AbstractConstruct> sourceConstructs) throws SQLException, ClassNotFoundException {
		Map<AbstractConstruct, String> weakLinksDescription = new LinkedHashMap<AbstractConstruct, String>();
		AbstractConstruct hardLink = null;
		AbstractConstruct smConstruct = targetConstruct.getParent();
		String skolemTable = PREFIX_SM + smConstruct.getName() + SUFFIX_SKOLEM;
		String skolemValuesQuery = 
			" SELECT * \n" +
			" FROM " + skolemTable + 
			" WHERE OID = " + targetConstruct.getOID() + "\n"+
			" ;";
		DBConnecter dbc = new DBConnecter();
		Connection con = dbc.getConnection();
		Statement st = con.createStatement();
		ResultSet skolemValues = st.executeQuery(skolemValuesQuery);
		ResultSetMetaData skolemValuesInfo = skolemValues.getMetaData();
		if (skolemValues.next()) { // un solo eventuale risultato
			int numColumns = skolemValuesInfo.getColumnCount();
			// salta le prime due colonne
			for (int i=2; i<numColumns+1; i++) {
				Object objValue = skolemValues.getObject(i);
				if (objValue != null) { // distingui tra '0' e 'null'
					int sourceOID = skolemValues.getInt(i);
					// se il sourceOID coincide con uno dei source, aggiungi
					for (AbstractConstruct sourceConstruct : sourceConstructs) {
						if (sourceConstruct.getOID() == sourceOID) {
							weakLinksDescription.put(sourceConstruct, skolemValuesInfo.getColumnName(i));
						}
					}
				}
			}
		}
		Collection<AbstractConstruct> weakLinks = weakLinksDescription.keySet();

		if (weakLinks.size() == 1) {
			hardLink = weakLinks.iterator().next();
		} else if (weakLinks.size() >= 2) { 
			// se ho piu' weakLinks, discrimina in base al tipo
			for (AbstractConstruct weakLink : weakLinks) {
				String skolemFunctionArg = weakLinksDescription.get(weakLink);
				AbstractConstruct smSourceConstruct = (AbstractConstruct) weakLink.getRootBaseInfo();
				String skolemArgConstructRefQuery = 
					" SELECT DISTINCT arg.oid \n"+ 
				    " FROM MSM_SKOLEMFUNCTION AS sk, MSM_SKOLEMFUNCTIONARGUMENT AS arg \n"+
				    " WHERE lcase(arg.name)=lcase('"+skolemFunctionArg+"') \n"+
				    "\t AND sk.constructoid="+smConstruct.getOID()+"\n"+
				    "\t AND sk.oid=arg.functionoid \n" + // join
				    "\t AND arg.sourceconstructoid="+smSourceConstruct.getOID()+
				    " ;";
				//System.out.println(skolemArgConstructRefQuery);
				ResultSet isSourceValid = st.executeQuery(skolemArgConstructRefQuery);
				if(isSourceValid.next()) { // se il sorgente ha lo stesso costrutto
					hardLink = weakLink;
				}
			}
		}

		// weakLinks non piu' necessari
		weakLinks.clear();
		
		Pair<Collection<AbstractConstruct>, AbstractConstruct> result = new Pair<Collection<AbstractConstruct>, AbstractConstruct>(weakLinks, hardLink);
		return result;
	}

}
