package org.modelgen.jpf.visualizer.modelgenview;

import org.modelgen.commons.beans.model.Reference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;
import org.modelgen.jpf.visualizer.modelgenview.tooltip.ToolTipManager;
import org.modelgen.jpf.visualizer.util.CollectionFunctions;
import org.modelgen.jpf.visualizer.util.HtmlString;
import org.modelgen.jpf.visualizer.util.UnsharedString;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.Vector;

/**
 * Class <code>SchemeModelGenBuilderImp</code> defines
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class MetaConstructsViewStrategy implements ViewStrategy {
	public static final String NEST_NAME = "Nest";
	public static final String NEST_PARENT_ID = "Top"; // se contiene allora parent
	public static final String NEST_CHILD_ID = "Nested"; // se contiene allora child
	public static final String BINARYAGGR_NAME = "BinaryAggregationOfAbstracts";
	public static final String BINARYAGGR_REF1 = "abstract1OID";
	public static final String BINARYAGGR_REF2 = "abstract2OID";
	public static final String BINARYAGGR_ROLEPREFIX = "role";
	public static final String BINARYAGGR_FUNCTIONALPREFIX = "isFunctional";
	public static final String BINARYAGGR_OPTIONALPREFIX = "isOptional";
	public static final String ABSTRACTATTRIBUTE_NAME = "AbstractAttribute";
	public static final String ABSTRACTATTRIBUTE_TO = "abstractToOID";


	public static ViewStrategy instance = new MetaConstructsViewStrategy();

	public ModelGraph doParse(MultiDirectionAbstractStructure structure, Collection<MultiDirectionAbstractStructure> ownersAllowed) throws ClassNotFoundException, Exception {
		ModelGraph modelGraph = new ModelGraph(structure.getName(), structure.getSid());




		/*------------------ Crea grafo ---------------------*/
		Map<BaseInfo<?>, ModelComponent> structureNodes = new LinkedHashMap<BaseInfo<?>, ModelComponent>();

		SortedSet<AbstractConstruct> constructs = structure.getAbstractConstructs();

		/* Crea semplicemente i nodi */
		for (AbstractConstruct c : constructs) {
			AbstractConstruct smConstruct = (AbstractConstruct) c.getRootBaseInfo();
			if(! (smConstruct.getName().equals(NEST_NAME))) {
				ModelComponent node = new ModelComponent(c);
				HtmlString extendedDescription = ToolTipManager.instance.createToolTip(c);
				node.setTooltip(extendedDescription);

				// aggiungi convenzioni sul nome, come (?) o (*)
				ModelGenDescriptionFactory.setHtmlText(node.getHtmlName(), c);

				// aggiungi info sulla forma
				ShapeInfoFlyweightFactory.setShape(node, c);

				structureNodes.put(c, node);
			}
		}

		/* Aggiungi le relazioni di contenimento */
		for (AbstractConstruct owner : constructs) {
			Collection<AbstractReference> parents = new LinkedList<AbstractReference>();
			AbstractConstruct smOwner = (AbstractConstruct) owner.getRootBaseInfo();

			// il caso nest non differenzia tra relation e description
			boolean isNest = false;

			/* Non inserire il nest. Inoltra l'owner con il target del nest. */
			if (smOwner.getName().equals(NEST_NAME)) {
				isNest = true;
				/* Memorizza il riferimento al nest */
				AbstractConstruct nest = owner;

				/* Ottieni il nuovo owner */
				for(AbstractReference ref : owner.getAbstractReferences()) {
					AbstractReference smRef = (AbstractReference) ref.getRootBaseInfo();
					if(smRef.getName().contains(NEST_PARENT_ID) && ref.getTarget() != null) {
						parents.add(ref); /* Ottieni il target */
					} else if(smRef.getName().contains(NEST_CHILD_ID) && ref.getTarget() != null) {
						owner = ref.getTarget(); // inoltra l'owner
						ModelComponent ownerInfo = structureNodes.get(owner);
						HtmlString tip = ownerInfo.getTooltip();
						HtmlString nestToolTip = ToolTipManager.instance.createToolTip(nest);
						nestToolTip.addOuterHtmlTag("BLOCKQUOTE");
						nestToolTip.addOuterHtmlTag("SMALL");
						String nestToolTipHtmlText = nestToolTip.getHtmlText(false);
						tip.setText(tip.getText() + "<small>Nested by:</small>" + nestToolTipHtmlText);
					}
				}
			} else { // caso normale (non nest)
				parents = owner.getAbstractReferences();
			}

			/* Per tutti i reference */
			ModelComponent child = structureNodes.get(owner);

			/* Se e' un set non inserire */
			smOwner = (AbstractConstruct) owner.getRootBaseInfo(); // ricarica
			for (AbstractReference r : parents) {
				AbstractConstruct target = r.getTarget();
				// il caso nest non differenzia tra relation e description
				if ( child != null && (isNest || r.getType() == Reference.ReferenceType.DESCRIPTION)) {
					/* La relazione parent-child si inverte */
					ModelComponent parent = structureNodes.get(target);
					if (parent != null) { // FIXME: se null sarebbe exception
						// i gruppi si inseriscono in coda
						boolean insertLast = !child.isLeaf();
						if (insertLast == false) {
							parent.addChildren(child,false);
						} else if (!parent.isFirstGroupInserted()) {
							parent.addChildren(child,true);
							parent.setFirstGroupInserted(true);
						} else { // gia' inserito un gruppo
							parent.addChildrenConcatenateLast(child);
						}
					}
				}
			}
		}



		/* 
		 * Ora che ho inserito le relazioni di contenimento
		 * sono in grado di aggiungere la colonna 
		 * standard "oid" esclusivamente nella radice.
		 *  
		 * La colonna 'name' non viene inclusa perche' inutile
		 * (il name coincide con il nome del nodo e del tab) 
		 */
		for (AbstractConstruct owner : constructs) {
			ModelComponent root = structureNodes.get(owner);
			if (root != null && root.isRoot()) {
				ModelComponent oid = new ModelComponent(new UnsharedString("oid"), false);
				HtmlString oidTip = ToolTipManager.instance.createToolTip(owner);
				oidTip.setText( "OID of " + oidTip.getText());
				oid.setTooltip(oidTip);
				root.addChildren(oid, false);
				modelGraph.addVertex(oid);
			}
		}

		// aggiungo i nodi creati nel grafo
		for (ModelComponent view : structureNodes.values()) {
			modelGraph.addVertex(view);
		}

		// Aggiungi gli archi grafici
		for (AbstractConstruct owner : constructs) {
			ModelComponent ownerView = structureNodes.get(owner);
			if (ownerView != null) {

				// conta quanti reference di tipo relation possiede
				int referenceRelationCount = 0;
				for (AbstractReference r : owner.getAbstractReferences()) {
					AbstractConstruct target = r.getTarget();
					if (target != null && r.getType() != Reference.ReferenceType.DESCRIPTION) {
						referenceRelationCount++;
						if (referenceRelationCount >= 2) {
							break;
						}
					}
				}
				boolean hasOnlyOneRelationEdge = referenceRelationCount == 1;

				// aggiungi gli archi relation
				for (AbstractReference r : owner.getAbstractReferences()) {
					AbstractConstruct target = r.getTarget();
					if (target != null && r.getType() != Reference.ReferenceType.DESCRIPTION) {
						ModelComponent targetView = structureNodes.get(target);
						ModelComponent edge = new ModelComponent(r,false);
						HtmlString extendedDescription = ToolTipManager.instance.createToolTip(r);
						edge.setTooltip(extendedDescription);
						// FIXME: se uno dei due null sarebbe exception
						if (ownerView != null && targetView != null) {
							modelGraph.addConnection(ownerView, targetView, edge);
							structureNodes.put(r, edge);

							// aggiungi la colonna innestata solo se 
							// innesta piu' di un elemento
							if (ownerView.isRoot() || !hasOnlyOneRelationEdge) {
								ownerView.addChildren(edge);
							}
						}
					}
				}
			}
		}

		// tratta in modo esplicito il caso relationship binario
		for (AbstractConstruct owner : constructs) {
			AbstractConstruct smOwner = (AbstractConstruct) owner.getRootBaseInfo();
			if (smOwner.getName().equals(BINARYAGGR_NAME)) {
				for (AbstractReference r : owner.getAbstractReferences()) {
					int roleNumber;
					AbstractReference metaRef = (AbstractReference) r.getRootBaseInfo(); 
					if (metaRef.getName().equals(BINARYAGGR_REF1)) {
						roleNumber = 1;
					} else if (metaRef.getName().equals(BINARYAGGR_REF2)) {
						roleNumber = 2;
					} else {
						break;
					}
					ModelComponent edge = structureNodes.get(r);
					HtmlString edgeDescription = edge.getHtmlName();
					HtmlString edgeTip = edge.getTooltip();

					// ottieni il nome del ruolo
					String roleName = edgeDescription.getText();
					for (AbstractProperty p : owner.getAbstractProperties()) {
						AbstractProperty metaProp = (AbstractProperty) p.getRootBaseInfo();
						// ottieni il valore della proprieta' role1 o role2
						if (metaProp.getName().equals(BINARYAGGR_ROLEPREFIX + roleNumber)) {
							roleName = p.getValue();
						}
					}

					// imposta il nome del ruolo ottenuto
					edgeDescription.setText(roleName);

					// Modifica provvisoriamente cosa viene
					// interpretato come optional e functional
					final String[] setNamesFalse = ModelGenDescriptionFactory.setNamesFalse;
					final String[] optionalNames = ModelGenDescriptionFactory.optionalNames;
					final String[] underlineProperties = ModelGenDescriptionFactory.underlineProperties;
					ModelGenDescriptionFactory.setNamesFalse = new String[] { BINARYAGGR_FUNCTIONALPREFIX + roleNumber };
					ModelGenDescriptionFactory.optionalNames = new String[] { BINARYAGGR_OPTIONALPREFIX + roleNumber };
					ModelGenDescriptionFactory.underlineProperties = new String[] { };
					ModelGenDescriptionFactory.setHtmlText(edgeDescription, owner);

					// undo
					ModelGenDescriptionFactory.setNamesFalse = setNamesFalse;
					ModelGenDescriptionFactory.optionalNames = optionalNames;
					ModelGenDescriptionFactory.underlineProperties = underlineProperties;

					// imposta il nome del ruolo nel tooltip
					String text = edgeTip.getText();
					edgeTip.setText("<p><u><i>Role name</i></u>: <b>"+edgeDescription.getHtmlText(false)+"</b></p>"+text);
				}
			}
		}



		// ------------------ Aggiungi le istanze ---------------------
		Map<BaseInfo<?>, ModelComponent> instanceNodes = new LinkedHashMap<BaseInfo<?>, ModelComponent>();



		// ------------------ Costruzione colonne ---------------------

		// Crea semplicemente i nodi 
		for (AbstractConstruct column : constructs) {
			for (AbstractConstruct row : column.getChildrensBelongingTo(ownersAllowed)) {
				AbstractConstruct smConstruct = (AbstractConstruct) row.getRootBaseInfo();
				if(! (smConstruct.getName().equals(NEST_NAME))) {
					ModelComponent node = new ModelComponent(row);
					HtmlString extendedDescription = ToolTipManager.instance.createToolTip(row);
					node.setTooltip(extendedDescription);
					ModelGenDescriptionFactory.setHtmlText(node.getHtmlName(), row);
					instanceNodes.put(row, node);
					for (AbstractReference ref : row.getAbstractReferences()) {
						ModelComponent refStruct = new ModelComponent(ref, ref.toString());
						node.addChildren(refStruct);
						instanceNodes.put(ref, refStruct);
					}
				}
			}  
		}

		// Aggiungi le relazioni di contenimento
		for (AbstractConstruct c : constructs) {
			for (AbstractConstruct owner : c.getChildrensBelongingTo(ownersAllowed)) {
				Collection<AbstractReference> parents = new LinkedList<AbstractReference>();
				AbstractConstruct smOwner = (AbstractConstruct) owner.getRootBaseInfo();

				// il caso nest non differenzia tra relation e description
				boolean isNest = false;

				// Non inserire il nest. Inoltra l'owner con il target del nest.
				if (smOwner.getName().equals(NEST_NAME)) {
					isNest = true;
					// Memorizza il riferimento al nest
					AbstractConstruct nest = owner;

					// Ottieni il nuovo owner
					for(AbstractReference ref : owner.getAbstractReferences()) {
						AbstractReference smRef = (AbstractReference) ref.getRootBaseInfo();
						if(smRef.getName().contains(NEST_PARENT_ID) && ref.getTarget() != null) {
							parents.add(ref); // Ottieni il target
						} else if(smRef.getName().contains(NEST_CHILD_ID) && ref.getTarget() != null) {
							owner = ref.getTarget(); // inoltra l'owner
							ModelComponent ownerInfo = instanceNodes.get(owner);
							HtmlString tip = ownerInfo.getTooltip();
							HtmlString nestToolTip = ToolTipManager.instance.createToolTip(nest);
							nestToolTip.addOuterHtmlTag("BLOCKQUOTE");
							nestToolTip.addOuterHtmlTag("SMALL");
							String nestToolTipHtmlText = nestToolTip.getHtmlText(false);
							tip.setText(tip.getText() + "<small>Nested by:</small>" + nestToolTipHtmlText);
						}
					}
				} else { // caso normale (non nest)
					parents = owner.getAbstractReferences();
				}

				ModelComponent child = instanceNodes.get(owner);
				// ricarica
				smOwner = (AbstractConstruct) owner.getRootBaseInfo();
				for (AbstractReference r : parents) {
					AbstractConstruct target = r.getTarget();
					// il caso nest non differenzia tra relation e description
					if ( child != null && (isNest || r.getType() == Reference.ReferenceType.DESCRIPTION)) {
						// La relazione parent-child si inverte
						ModelComponent parent = instanceNodes.get(target);
						if (parent != null) { // FIXME: se null sarebbe exception
							parent.addChildren(child,false);
						}
					}
				}
			}
		}


		// ------------------ Costruzione righe ---------------------

		// Aggiungi il valore della colonna oid
		for (AbstractConstruct columns : constructs) {
			ModelComponent columnsStruct = structureNodes.get(columns);
			if(columnsStruct != null && columnsStruct.isRoot()) {
				List<ModelComponent> rows = columnsStruct.getChildrens();
				for (AbstractConstruct columnsValue : columns.getChildrensBelongingTo(ownersAllowed)) {
					// ottieni la riga corrente
					//System.out.println(columnsValue);
					ModelComponent columnsValueStruct = instanceNodes.get(columnsValue);
					ModelComponent oid = rows.get(0);
					ModelComponent value = new ModelComponent(columnsValue.getOID());
					HtmlString oidTip = ToolTipManager.instance.createToolTip(columnsValue);
					oidTip.setText( "OID of " + oidTip.getText());
					value.setTooltip(oidTip);
					oid.addInstance(columnsValueStruct, value, null);
				}
			}
		}

		// inverti per un corretto ordine dei valori annidati
		SortedSet<AbstractConstruct> constructsReversed = CollectionFunctions.reverse(constructs);
		//SortedSet<AbstractConstruct> constructsReversed = constructs;

		// Aggiungi il valore di tutte le colonne tranne la colonna oid
		for (AbstractConstruct columnSubSet : constructsReversed) {
			ModelComponent columnsSubSetStruct = structureNodes.get(columnSubSet);
			if(columnsSubSetStruct != null && columnsSubSetStruct.isLeaf()) {
				for(AbstractConstruct cell : columnSubSet.getChildrensBelongingTo(ownersAllowed)) {
					ModelComponent cellStruct = instanceNodes.get(cell);
					ModelComponent currentRow = cellStruct.calculateRoot();
					ModelComponent cellValue = new ModelComponent(cell);
					ModelGenDescriptionFactory.setHtmlText(cellStruct.getHtmlName(), cell);
					//System.out.println("Adding cell " + cellValue + " in row " + currentRow);

					// eventuale elemento puntato da calcolare
					AbstractConstruct constructReferenced = null;

					// se un costrutto ha 1 solo arco "relation"
					// allora aggiungi tooltip con preview e cambia la cella
					// per assumere il valore puntato
					AbstractReference relationEdge = null;
					int relationCount = 0;
					for (AbstractReference r : cell.getAbstractReferences()) {
						if (r.getType() == Reference.ReferenceType.RELATION) {
							relationEdge = r;
							relationCount++;
							if (relationCount > 1) break; 
						}
					}

					// se ho un solo arco "relation"

					if(relationCount == 1) {
						if(relationEdge != null) {
							Vector<AbstractConstruct> preview = new Vector<AbstractConstruct>();
							// ottieni l'elemento puntato
							constructReferenced = relationEdge.getTarget();

							// aggiungi anteprima nel tooltip
							ModelComponent rowReferenced = instanceNodes.get(constructReferenced);
							for(ModelComponent child : rowReferenced.getChildrens()) {
								Object info = child.getInfo();
								if (info instanceof AbstractConstruct) {
									AbstractConstruct c = (AbstractConstruct) child.getInfo();
									if (c.getChildrens().isEmpty()) { // se e' una foglia
										preview.add(c); // aggiungi
									}
								}
							}

							ModelComponent rowReferencedInfo = cellStruct.getParent();
							columnsSubSetStruct.addInstance(currentRow, rowReferenced, rowReferencedInfo);
							rowReferenced.setTooltip(ToolTipManager.instance.createToolTip(constructReferenced, preview));
						} 
						
						// non trovo riferimento o valore puntato
						if(relationEdge == null || constructReferenced == null) {
							columnsSubSetStruct.addInstance(currentRow, new ModelComponent(""), null);
							// elemento puntato non risolvibile
							cellValue.setTooltip(ToolTipManager.instance.createToolTip(cell,null));
							HtmlString tip = cellValue.getTooltip();
							if (constructReferenced != null) {
								HtmlString pointingTip = ToolTipManager.instance.createToolTip(constructReferenced);
								pointingTip.addOuterHtmlTag("BLOCKQUOTE");
								pointingTip.addOuterHtmlTag("SMALL");
								String pointingTipHtmlText = pointingTip.getHtmlText(false);
								tip.setText(tip.getText() + "<small>Pointing to:</small>" + pointingTipHtmlText);
							}
						}
					} else { // caso normale: tooltip semplice
						cellValue.setTooltip(ToolTipManager.instance.createToolTip(cell));
						ModelComponent cellValueInfo = cellStruct.getParent();
						columnsSubSetStruct.addInstance(currentRow, cellValue, cellValueInfo);
					}
				}
			} 

			// aggiungi il valore tabulare dei riferimenti relation
			for(AbstractReference refColumnSubSet : columnSubSet.getAbstractReferences()) {
				if (refColumnSubSet.getType() != Reference.ReferenceType.DESCRIPTION) {
					ModelComponent refColumnsSubSetStruct = structureNodes.get(refColumnSubSet);
					// FIXME: se null sarebbe exception
					if(refColumnsSubSetStruct != null && refColumnsSubSetStruct.isLeaf()) {
						for (AbstractReference ref : refColumnSubSet.getChildrensBelongingTo(ownersAllowed)) {
							ModelComponent refStruct = instanceNodes.get(ref);
							ModelComponent refCurrentRow = refStruct.calculateRoot();
							ModelComponent refCellValue;
							// gestisci il caso in cui il reference non abbia il target
							if (ref.getTarget() == null) {
								refCellValue = new ModelComponent("");
								refCellValue.setTooltip(new HtmlString("null target"));
							} else {
								refCellValue = new ModelComponent(ref.getTarget().toString());
								//System.out.println("Adding cell " + refCellValue + " in row " + refCurrentRow);

								// aggiungi il tooltip con preview
								ModelComponent rowReferenced = instanceNodes.get(ref.getTarget());
								Vector<AbstractConstruct> preview = new Vector<AbstractConstruct>();
								for(ModelComponent child : rowReferenced.getChildrens()) {
									Object info = child.getInfo();
									if (info instanceof AbstractConstruct) {
										AbstractConstruct c = (AbstractConstruct) child.getInfo();
										if (c.getChildrens().isEmpty()) { // se e' una foglia
											preview.add(c); // aggiungi
										}
									}
								}
								refCellValue.setTooltip(ToolTipManager.instance.createToolTip(ref.getTarget(), preview));
							}
							ModelComponent refCellValueInfo = refStruct.getParent();
							refColumnsSubSetStruct.addInstance(refCurrentRow, refCellValue, refCellValueInfo);
						}
					}
				}
			}
		}

		return modelGraph;
	}

}
