package org.modelgen.jpf.visualizer.modelgenview;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.util.ModelGenValueConverter;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;
import org.modelgen.jpf.visualizer.modelgenview.tooltip.ToolTipManager;
import org.modelgen.jpf.visualizer.util.HtmlString;
import org.modelgen.jpf.visualizer.util.UnsharedString;

import java.io.Serializable;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

/**
 * Class <code>ModelModelGenBuilderImp</code> defines
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class MetaModelViewStrategy implements ViewStrategy {
    public static ViewStrategy instance = new MetaModelViewStrategy();

    public ModelGraph doParse(MultiDirectionAbstractStructure structure, Collection<MultiDirectionAbstractStructure> ownersAllowed) throws ClassNotFoundException, Exception {
	Map<BaseInfo<?>, ModelComponent> modelGenNodes = new LinkedHashMap<BaseInfo<?>, ModelComponent>();

	ModelGraph modelGraph = new ModelGraph(structure.getName(), structure.getSid());

	
	
	
	/*------------------ Crea grafo ---------------------*/
	
	
	
	Collection<AbstractConstruct> constructs = structure.getAbstractConstructs();
	/* Crea semplicemente i nodi */
	for (AbstractConstruct c : constructs) {
	    ModelComponent parent = new ModelComponent(c, c.getName());
	    HtmlString extendedDescription = ToolTipManager.instance.createToolTip(c);
	    parent.setTooltip(extendedDescription);
	    modelGraph.addVertex(parent);
	    modelGenNodes.put(c, parent); // salva

	    /* Aggiungi all'inizio le colonne "oid" e "name" 
	     * 
	     * Utilizzo il costruttore new UnsharedString("name") e non semplicemente
	     * "name" poiche' l'istruzione "name" genera una new StringBuffer("name")
	     * SOLO SE "name" NON ESISTE GIA'. Di fatto "name" e' una
	     * flyweight factory. Non vogliamo invece istanze condivise.
	     * Per questo creo esplicitamente una nuova istanza */
	    ModelComponent oid = new ModelComponent(new UnsharedString("oid"), false);
	    HtmlString oidTip = ToolTipManager.instance.createToolTip(c);
	    oidTip.setText( "OID of " + oidTip.getText());
	    oid.setTooltip(oidTip);
	    parent.addChildren(oid);
	    
	    ModelComponent name = new ModelComponent(new UnsharedString("name"), false);
	    HtmlString nameTip = ToolTipManager.instance.createToolTip(c);
	    nameTip.setText( "Name of " + nameTip.getText());
	    name.setTooltip(nameTip);
	    parent.addChildren(name);

	    for (AbstractProperty p : c.getAbstractProperties()) {
		ModelComponent child = new ModelComponent(p, p.getName());
		HtmlString propExtendedDescription = ToolTipManager.instance.createToolTip(p);
		child.setTooltip(propExtendedDescription);
		parent.addChildren(child);

		modelGraph.addVertex(child);
	    }

	    modelGraph.addVertex(oid);
	    modelGraph.addVertex(name);

	}

	/* Aggiungi gli archi grafici */
	for (AbstractConstruct owner : constructs) {
	    ModelComponent ownerView = modelGenNodes.get(owner);
	    for (AbstractReference r : owner.getAbstractReferences()) {
		AbstractConstruct target = r.getTarget();
		if (target != null) {
		    ModelComponent targetView = modelGenNodes.get(target);
		    ModelComponent edge = new ModelComponent(r,false); 
		    HtmlString extendedDescription = ToolTipManager.instance.createToolTip(r);
		    edge.setTooltip(extendedDescription);
		    modelGraph.addConnection(ownerView, targetView, edge);
		    ownerView.addChildren(edge);  // aggiungi la colonna
		}
	    }
	}
	
	
	
	
	/*------------------ Aggiungi le istanze ---------------------*/
	

	/* Aggiungi le istanze delle colonne "oid" , "name" e delle property */
	for (AbstractConstruct c : constructs) {
	    ModelComponent columns = modelGenNodes.get(c);
	    for (AbstractConstruct currentRow : c.getChildrensBelongingTo(ownersAllowed)) {
		ModelComponent currentRowComponent = new ModelComponent(currentRow);
		// aggiungi oid
		ModelComponent oid = columns.findOwner(new ModelComponent(new UnsharedString("oid")));
		ModelComponent oidValue = new ModelComponent(currentRow.getOID());
		HtmlString oidTip = ToolTipManager.instance.createToolTip(currentRow);
		oidTip.setText( "OID of " + oidTip.getText());
		oidValue.setTooltip(oidTip);
		oid.addInstance(currentRowComponent, oidValue, null);
		
		// aggiungi name
		ModelComponent name = columns.findOwner(new ModelComponent(new UnsharedString("name")));
		ModelComponent nameValue = new ModelComponent(currentRow.getName());
		HtmlString nameTip = ToolTipManager.instance.createToolTip(currentRow);
		nameTip.setText( "Name of " + nameTip.getText());
		nameValue.setTooltip(nameTip);
		name.addInstance(currentRowComponent, nameValue, null);
		
		// aggiungi property
		for (AbstractProperty p : currentRow.getAbstractProperties()) {
		    ModelComponent column = columns.findOwner(new ModelComponent(p.getParent()));
		    Serializable cellValue = ModelGenValueConverter.convertValue(p);
		    ModelComponent value = new ModelComponent(cellValue);
		    HtmlString tooltip = ToolTipManager.instance.createToolTip(p);
		    value.setTooltip(tooltip);
		    
		    column.addInstance(currentRowComponent, value, null);
		}
	    }
	}

	/* Se a qualche proprieta' non e' stato dato un valore, assegna "" */
	for (AbstractConstruct c : constructs) {
	    ModelComponent columns = modelGenNodes.get(c);
	    for (AbstractConstruct currentRow : c.getChildrensBelongingTo(ownersAllowed)) {
		ModelComponent currentRowComponent = new ModelComponent(currentRow);
		Collection<AbstractProperty> rowParentProperties = new LinkedList<AbstractProperty>();
		for (AbstractProperty p : currentRow.getAbstractProperties()) {
		    rowParentProperties.add(p.getParent());
		}
		for (AbstractProperty p : c.getAbstractProperties()) {
		    if( !rowParentProperties.contains(p)) {
			ModelComponent column = columns.findOwner(new ModelComponent(p));
			ModelComponent value = new ModelComponent("");
			HtmlString tooltip = ToolTipManager.instance.createToolTip(p);
			tooltip.setText(tooltip.getText() + "<P><SMALL><U>not present</U></SMALL></P>");
			value.setTooltip(tooltip);
			column.addInstance(currentRowComponent, value, null);
		    }
		}
	    }
	}

	/* Aggiungi le istanze delle colonne reference */
	for (AbstractConstruct c : constructs) {
	    ModelComponent columns = modelGenNodes.get(c);
	    for (AbstractConstruct currentRow : c.getChildrensBelongingTo(ownersAllowed)) {
		ModelComponent currentRowComponent = new ModelComponent(currentRow);

		for (AbstractReference r : currentRow.getAbstractReferences()) {
		    ModelComponent column = columns.findOwner(new ModelComponent(r.getParent()));
		    ModelComponent cellValue = new ModelComponent(r.getTarget().getDescription());
		    HtmlString tooltip = ToolTipManager.instance.createToolTip(r);
		    cellValue.setTooltip(tooltip);
		    column.addInstance(currentRowComponent, cellValue, null);
		}
	    }
	}
	
	/* Se a qualche reference non e' stato dato un valore, assegna "" */
	for (AbstractConstruct c : constructs) {
	    ModelComponent columns = modelGenNodes.get(c);
	    for (AbstractConstruct currentRow : c.getChildrensBelongingTo(ownersAllowed)) {
		ModelComponent currentRowComponent = new ModelComponent(currentRow);
		Collection<AbstractReference> rowParentProperties = new LinkedList<AbstractReference>();
		for (AbstractReference r : currentRow.getAbstractReferences()) {
		    rowParentProperties.add(r.getParent());
		}
		for (AbstractReference r : c.getAbstractReferences()) {
		    if( !rowParentProperties.contains(r)) {
			ModelComponent column = columns.findOwner(new ModelComponent(r));
			ModelComponent value = new ModelComponent("");
			HtmlString tooltip = ToolTipManager.instance.createToolTip(r);
			tooltip.setText(tooltip.getText() + "<P><SMALL><U>not present</U></SMALL></P>");
			value.setTooltip(tooltip);
			column.addInstance(currentRowComponent, value, null);
		    }
		}
	    }
	}

	return modelGraph;
    }

}
