package org.modelgen.jpf.visualizer.modelgenview;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;

import java.util.Collection;
import java.util.Set;

/**
 * Class <code>GraphBuilder</code> builds a new Graph object
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ModelGenBuilder {

    /**
     * Crea il grafo a partire da una struttura di modelgen
     * @param structure la struttura di modelgen da analizzare
     * @param ownersAllowed quali figli considerare. se null
     * non esegue alcun filtro
     * @param removeEmpty se rimuovere gli elementi che non
     * hanno dati associati
     * @return il grafo creato
     */
    public static ModelGraph parse(MultiDirectionAbstractStructure structure, Collection<MultiDirectionAbstractStructure> ownersAllowed, boolean removeEmpty) throws ClassNotFoundException, Exception {
	ViewStrategy viewStrategy = null;
	if (MultiDirectionAbstractStructure.isSuperModel(structure) ||
		MultiDirectionAbstractStructure.isModel(structure)) {
	    viewStrategy = MetaModelViewStrategy.instance;
	} else if (MultiDirectionAbstractStructure.isScheme(structure) ||
		MultiDirectionAbstractStructure.isInstance(structure)) {
	    viewStrategy = MetaConstructsViewStrategy.instance;
	} else {
	    assert false : "Struttura sconosciuta";
		return null;
	}

	// crea il grafo
	ModelGraph modelGenGraph = viewStrategy.doParse(structure, ownersAllowed);

	// rimuovi i costrutti che non hanno dati associati
	if(removeEmpty) {
	    // Crea un array di appoggio per impedire eccezioni Concurrent
	    Set<ModelComponent> verticesSet = modelGenGraph.getVertices();
	    ModelComponent[] verticesArray = new ModelComponent[verticesSet.size()];
	    verticesArray = verticesSet.toArray(verticesArray);
	    for(ModelComponent c : verticesArray) {
		if(c.getInstances().isEmpty()) {
		    modelGenGraph.remove(c);
		}
	    }
	}
	
	return modelGenGraph;
    }

}
