package org.modelgen.jpf.visualizer.modelgenview;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.util.HtmlString;

import java.util.Arrays;

/**
 * <code>ModelGenDescriptionFactory</code> consente
 * di inserire descrizioni per gli attributi.
 * 
 * TODO: per ora cablato
 * La forma da utilizzare nel file di configurazione 
 * e' la seguente:<PRE>
isIdentifier = "&lt;u&gt;${inner}&lt;/u&gt;"
isNullable = "&lt;i&gt;+${inner}&lt;/i&gt;"
isOptional = "?${inner}"
isFunctional = "*${inner}"
isOptional+isFunctional = "+${inner}"
!isIdentifier = "bo?${inner}"
</PRE>
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ModelGenDescriptionFactory {
    public static String[] optionalNames = { "isOptional" };
    public static String[] setNamesTrue = { "isSet" };
    public static String[] setNamesFalse = { "isFunctional" };
    public static String[] underlineProperties = { "isIdentified", "isIdentifier", "isKey" };
    public static String[] italicProperties = { "isNullable", "isTotal" };
    
    public static String getPrefix(AbstractConstruct c) {
	String result;
	boolean optional = false, set = false;
	
	for(AbstractProperty p : c.getAbstractProperties()) {
	    BaseInfo<AbstractProperty> smProperty = getSuperModelConstruct(p);
	    if(Arrays.binarySearch(optionalNames, smProperty.getName()) >= 0) {
		if (p.getValue().equals("true")) {
		    optional = true;
		}
		break;
	    }
	}
	
	for(AbstractProperty p : c.getAbstractProperties()) {
	    BaseInfo<AbstractProperty> smProperty = getSuperModelConstruct(p);
	    if(Arrays.binarySearch(setNamesTrue, smProperty.getName()) >= 0) {
		if (p.getValue().equals("true")) {
		    set = true;
		}
		break;
	    }
	    if(Arrays.binarySearch(setNamesFalse, smProperty.getName()) >= 0) {
		if (p.getValue().equals("false")) {
		    set = true;
		}
		break;
	    }
	}
	
	if (optional && set) {
	    result = "*";
	} else if (optional) {
	    result = "?";
	} else if (set) { 
	    result = "+";
	} else { // !set && !optional
	    result = null;
	}
	
	return result;
    }
    
    public static void setHtmlText(HtmlString result, AbstractConstruct c) {
	boolean overline = false;
	boolean italic = false;
	
	for(AbstractProperty p : c.getAbstractProperties()) {
	    BaseInfo<AbstractProperty> smProperty = getSuperModelConstruct(p);
	    if(Arrays.binarySearch(underlineProperties, smProperty.getName()) >= 0) {
		if (p.getValue().equals("true")) {
		    overline = true;
		}
		break;
	    }
	}
	
	for(AbstractProperty p : c.getAbstractProperties()) {
	    BaseInfo<AbstractProperty> smProperty = getSuperModelConstruct(p);
	    if(Arrays.binarySearch(italicProperties, smProperty.getName()) >= 0) {
		if (p.getValue().equals("true")) {
		    italic = true;
		}
		break;
	    }
	}
	
	if (overline) {
	    result.addInnerHtmlTag("u");   // sottolinea
	}
	
	if (italic) {
	    result.addInnerHtmlTag("EM"); // corsivo
	    result.addInnerHtmlTag("FONT COLOR=#666666"); // grigio
	}
	
	String cardinality = getPrefix(c);
	if (cardinality != null) {
	    result.setText("(" + cardinality  +") " + result.getText());
	}
    }
    
    public static <T extends BaseInfo<T>> BaseInfo<T> getSuperModelConstruct(BaseInfo<T> c) {
	BaseInfo<T> parent = c.getParent();
	if(parent == null) {
	    return c;
	} else {
	    return getSuperModelConstruct(parent);
	}
    }
}
