package org.modelgen.jpf.visualizer.modelgenview;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ShapeInfo;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.Color;

/**
 * Class <code>ShapeInfoFlyweightFactory</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ShapeInfoFlyweightFactory {
    private static ShapeInfo abstractShape;
    private static ShapeInfo aggregationShape;
    private static ShapeInfo foreignKeyShape;
    private static ShapeInfo generalizationShape;
    private static ShapeInfo aggregationOfAbstractsShape;
    private static ShapeInfo binaryAggregationOfAbstractsShape;

    private static class ShapeMapType extends Pair<String,ShapeInfo> {
	private static final long serialVersionUID = 1L;
	public ShapeMapType(String name, ShapeInfo view) {
	    super(name, view);
	}
    }

    private static ShapeMapType[] fromNameToShape;

    static {
	abstractShape = new ShapeInfo();
	abstractShape.setShapeType(ShapeInfo.ShapeType.RECTANGLE);
	abstractShape.setBorderColor(Color.BLACK);

	aggregationShape = new ShapeInfo();
	//aggregationShape.setBorder(new RoundBorder());
	aggregationShape.setShapeType(ShapeInfo.ShapeType.ROUNDED);
	aggregationShape.setBorderColor(Color.BLACK);

	foreignKeyShape = new ShapeInfo();
	foreignKeyShape.setShapeType(ShapeInfo.ShapeType.RECTANGLE);
	//foreignKeyShape.setBorder(new DashedBorder(Color.LIGHT_GRAY, 8, 8));
	foreignKeyShape.setDashed(true);
	foreignKeyShape.setBorderColor(Color.LIGHT_GRAY);

	generalizationShape = new ShapeInfo();
	generalizationShape.setShapeType(ShapeInfo.ShapeType.OCTAGON);
	generalizationShape.setBorderColor(Color.BLACK);

	aggregationOfAbstractsShape = new ShapeInfo();
	aggregationOfAbstractsShape.setShapeType(ShapeInfo.ShapeType.DIAMOND);
	aggregationOfAbstractsShape.setBorderColor(Color.BLACK);

	binaryAggregationOfAbstractsShape = new ShapeInfo();
	binaryAggregationOfAbstractsShape.setShapeType(ShapeInfo.ShapeType.DIAMOND);
	binaryAggregationOfAbstractsShape.setDashed(true);
	binaryAggregationOfAbstractsShape.setBorderColor(Color.BLACK);


	fromNameToShape = new ShapeMapType[]{
		new ShapeMapType("Abstract", abstractShape),
		new ShapeMapType("Aggregation", aggregationShape),
		new ShapeMapType("ForeignKey", foreignKeyShape),
		new ShapeMapType("Generalization", generalizationShape),
		new ShapeMapType("AggregationOfAbstracts", aggregationOfAbstractsShape),
		new ShapeMapType("BinaryAggregationOfAbstracts", binaryAggregationOfAbstractsShape)
	};
    }

    public static void setShape(ModelComponent modelToShape, AbstractConstruct c) {
	String metaConstructName = c.getRootBaseInfo().getName();
	for (ShapeMapType mapInfo : fromNameToShape) {
	    String availableName = mapInfo.getFirst();
	    if (metaConstructName.equals(availableName)) {
		ShapeInfo shapeInfo = mapInfo.getSecond();
		modelToShape.setShapeInfo(shapeInfo);
		return;
	    }
	}
    }
}
