package org.modelgen.jpf.visualizer.modelgenview.modelgen;

import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfoDatabase;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.StructureModelAdapter;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.StructureSchemeAdapter;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.util.PersistenceFunctions;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * Class <code>AbstractModelGenBuilder</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class AbstractModelGenBuilderFacade {
    private static AbstractModelGenBuilderFacade instance = null;
    protected PersistenceHandler data = null;
    private MultiDirectionAbstractStructure superModel;
    private Map<Integer, MultiDirectionAbstractStructure> modelMap = new HashMap<Integer, MultiDirectionAbstractStructure>();
    
    public static synchronized AbstractModelGenBuilderFacade getInstance() throws ClassNotFoundException, Exception {
	if (instance == null) {
	    instance = new AbstractModelGenBuilderFacade();
	}
	return instance;
    }

    /**
     * Ottiene (ma non carica) l'albero modelGen
     * @throws Exception 
     * @throws ClassNotFoundException 
     */
    @SuppressWarnings("unchecked")
	private AbstractModelGenBuilderFacade() throws ClassNotFoundException, Exception {
	BaseInfo.setNewDatabase(); // nuovo modelgen repository
	this.data = PersistenceHandlerFactory.getPersistenceHandler();
	Collection<Model> models = this.data.getModels();
	Collection<Scheme> schemes = this.data.getSchemes();
	Collection<Scheme> smSchemes = this.data.getSuperModelSchemes();
	
	/* Inserisco il super modello */
	Model sm = this.data.getSuperModel();
	this.superModel = new StructureModelAdapter(null, sm);

	/* Inserisco modelli */
	for (Model model : models) {
	    MultiDirectionAbstractStructure modelStructure = new StructureModelAdapter(superModel, model);
	    this.modelMap.put(model.getOID(), modelStructure);
	}
	
	/* Inserisco schemi collegati a un modello */
	for (Scheme scheme : schemes) {
	    MultiDirectionAbstractStructure model = modelMap.get(scheme.getMetaInfo().getOID());
	    assert model != null;
	    MultiDirectionAbstractStructure schemeStructure = new StructureSchemeAdapter(model, scheme, false);
	    PersistenceFunctions.getInstances(schemeStructure); // risultato notificato a schemeStructure
	}
	for (Scheme smScheme : smSchemes) {
	    //AbstractStructure model = modelMap.get(smScheme.getModel().getOID());
	    //assert model != null;
	    MultiDirectionAbstractStructure schemeStructure = new StructureSchemeAdapter(superModel, smScheme, true);
	    PersistenceFunctions.getSuperModelInstances(schemeStructure); // risultato notificato a schemeStructure
	}
    }
    
    public MultiDirectionAbstractStructure getAbstractStructure(Model model) {
	return this.modelMap.get(model);
    }

    public MultiDirectionAbstractStructure getSuperModel() {
	assert this.superModel != null;
	return this.superModel;
    }
    
    public static AbstractModelGenBuilderFacade reload() throws ClassNotFoundException, Exception {
	instance = new AbstractModelGenBuilderFacade();
	return instance;
    }
    
    public BaseInfoDatabase getDatabaseImage() {
	return BaseInfo.getDatabaseImage();
    }


}
