package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Class <code>ConstructAdapter</code> 
 * consente di trattare in modo omogeneo
 * Construct e SchemeObject
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public abstract class AbstractConstruct extends ProxyBaseInfo<AbstractConstruct> {
    private transient Collection<AbstractProperty> abstractProperties = null;
    private transient Collection<AbstractReference> abstractReferences = null;
    private transient MultiDirectionAbstractStructure structure;
    private Collection<AbstractConstruct> weakLinks = new LinkedList<AbstractConstruct>();
    private AbstractConstruct hardLink;
    protected abstract Collection<AbstractProperty> doCreateAbstractProperties();
    protected abstract Collection<AbstractReference> doCreateAbstractReferences();
    public abstract boolean isLexical();
    public abstract void setLexical(boolean lexical);

    public AbstractConstruct(MultiDirectionAbstractStructure structure, AbstractConstruct parent, int oid, String name) {
	this(structure,parent,null,oid,name);
    }
    public AbstractConstruct(MultiDirectionAbstractStructure structure, AbstractConstruct parent, Object realSubject, int oid, String name) {
	super(parent, realSubject, oid, name);
	this.structure = structure;
    }
    
    public final Collection<AbstractProperty> getAbstractProperties() {
	if (this.abstractProperties == null) {
	    this.abstractProperties = doCreateAbstractProperties();
	}
	return this.abstractProperties;
    }
    
    public final Collection<AbstractReference> getAbstractReferences() {
	if (this.abstractReferences == null) {
	    this.abstractReferences = doCreateAbstractReferences();
	}
	return this.abstractReferences;
    }
    
    @Override
    public MultiDirectionAbstractStructure getStructure() {
        return structure;
    }

    public void setStructure(MultiDirectionAbstractStructure structure) {
        this.structure = structure;
    }
    
    public final void createPropertiesAndReferences() {
    	doCreateAbstractProperties();
    	doCreateAbstractReferences();
    }
    
    @Override
    public TypeID getTypeId() {
	return TypeID.AbstractConstruct;
    }
    
    /**
     * Imposta i costrutti a cui il corrente e' collegato debolmente
     * 
     * Utile ad esempio, per memorizzare il legame che si viene
     * a creare tramite le funzioni di skolem che non e' certo,
     * ma probabile.
     * 
     * Le informazioni non vengono gestite internamente,
     * ma solo memorizzate. La gestione avverra' dall'esterno
     * da parte dell'utente.
     * 
     * @param weakLinks i costrutti a cui e' collegato
     */
    public void setWeakLinks(Collection<AbstractConstruct> weakLinks) {
	this.weakLinks = weakLinks;
    }
    
    /**
     * Imposta il costrutto piu' probabile a cui e' collegato
     * @param hardLink il costrutto piu' probabile a cui e' collegato
     */
    public void setHardLink(AbstractConstruct hardLink) {
	this.hardLink = hardLink;
    }
    
    public AbstractConstruct getHardLink() {
        return hardLink;
    }
    
    /**
     * @return i costrutti a cui e' collegato
     */
    public Collection<AbstractConstruct> getWeakLinks() {
        return weakLinks;
    }
    
}
