package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters;

import org.modelgen.commons.beans.model.Reference.ReferenceType;


/**
 * Class <code>ReferenceAdapter</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public abstract class AbstractReference extends ProxyBaseInfo<AbstractReference> {
    private transient AbstractConstruct owner = null;
    private transient AbstractConstruct target = null;

    public AbstractReference(AbstractReference parent, AbstractConstruct owner, AbstractConstruct target, int oid, String name) {
	this(parent, null, owner, target, oid, name);
    }
    public AbstractReference(AbstractReference parent, Object realSubject, AbstractConstruct owner, AbstractConstruct target, int oid, String name) {
	super(parent, realSubject, oid, name);
	this.owner = owner;
	this.target = target;
    }
    
    public abstract ReferenceType getType();
    public abstract String getTypeAsString();
    
    public void setOwner(AbstractConstruct owner) {
        this.owner = owner;
    }
    public void setTarget(AbstractConstruct target) {
        this.target = target;
    }
    public final AbstractConstruct getOwner() {
	return this.owner;
    }
    
    public final AbstractConstruct getTarget() {
	return this.target;
    }
    
    @Override
    public TypeID getTypeId() {
	return TypeID.AbstractReference;
    }
    
    /* FORZA IL CONFRONTO PER REFERENCE */
    
    @Override
    public boolean equals(Object obj) {
	boolean result;
	if ( obj == null || ! (obj instanceof BaseInfo) ) {
	    result = false;
	} else {
	    result = (this == obj);
	}
	return result;
    }
    
    @Override
    public int hashCode() {
	int result;
	if (realSubject == null) {
	    result = super.hashCode();
	} else {
	    result = realSubject.hashCode();
	}
	return result;
    }
    
    
    @Override
    public String getTypeString() {
	return owner.getTypeString() + "'s Reference";
    }
	
	@Override
    public MultiDirectionAbstractStructure getStructure() {
        return this.owner.getStructure();
    }
    

}