package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters;

import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Consente di caricare i figli su richiesta in modo trasparente
 * @author Riccardo Pietrucci
 */
public abstract class ProxyBaseInfo<T extends ProxyBaseInfo<T>> extends BaseInfo<T> {

	public ProxyBaseInfo(T parent, int oid, String name) {
		super(parent, oid, name);
	}

	public ProxyBaseInfo(T parent, Object real, int oid, String name) {
		super(parent, real, oid, name);
	}

	@Override
	public synchronized SortedSet<T> getChildrens() throws Exception {
		// prima di restituire i figli, carica
		this.getStructure().loadAllChildrens();
		return super.getChildrens();
	}

	public synchronized Collection<T> getChildrensBelongingTo(MultiDirectionAbstractStructure aChild) throws Exception {
		// carica solo il figlio interessato
		aChild.load();
		
		// estrai dai figli finora caricati quelli appartenenti a child
		Collection<T> childrensLoaded = super.getChildrens();
		SortedSet<T> childrensOfStructure = new TreeSet<T>();
		for(T i : childrensLoaded) {
			if (i.getStructure() == aChild) {
				childrensOfStructure.add(i);
			}
		}
		return childrensOfStructure;
	}
	
	public synchronized SortedSet<T> getChildrensBelongingTo(Collection<MultiDirectionAbstractStructure> someChildrens) throws Exception {
		if (someChildrens == null) return this.getChildrens();
		
		SortedSet<T> childrensOfStructures = new TreeSet<T>();
		Collection<T> childrensOfStructure;
		
		for (MultiDirectionAbstractStructure aChild : someChildrens) {
			childrensOfStructure = this.getChildrensBelongingTo(aChild);
			childrensOfStructures.addAll(childrensOfStructure);
		}
		
		return childrensOfStructures;
	}
	

}
