package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Class <code>ConstructAdapter</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ConstructAdapter extends AbstractConstruct {
	private static final long serialVersionUID = 1L;
	private transient Construct construct;
    private transient boolean isSuperModelConstruct;

    public ConstructAdapter(StructureModelAdapter structure, ConstructAdapter parent, Construct construct) {
	super(structure, parent, construct, construct.getOID(), construct.getName());
	this.construct = construct;
	if (structure != null) {
	    this.isSuperModelConstruct = structure.isSuperModel();
	}
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Collection<AbstractProperty> doCreateAbstractProperties() {
	Collection<AbstractProperty> result = new LinkedList<AbstractProperty>();
	Collection<Property> properties = this.construct.getProperties();
	for ( Property p : properties ) {
	    AbstractProperty ap = AbstractConstructFlyweightFactory.sharedInstance.getConstructProperty(p, this); 
	    result.add(ap);
	}
	return result;
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Collection<AbstractReference> doCreateAbstractReferences() {
	Collection<AbstractReference> result = new LinkedList<AbstractReference>();
	Collection<Reference> references = this.construct.getReferences();
	for ( Reference r : references ) {
	    ConstructAdapter target = AbstractConstructFlyweightFactory.sharedInstance.getConstructAdapter(r.getTargetConstruct(), this.getStructure() );
	    AbstractReference ar = AbstractConstructFlyweightFactory.sharedInstance.getConstructReference(r, this, target);
	    result.add(ar);
	}
	return result;
    }

    @Override
    public boolean isLexical() {
	return this.construct.isLexical();
    }

    public void setName(String name) {
	this.construct.setName(name);
    }

    public void setOID(int OID) {
	this.construct.setOID(OID);
    }

    public boolean isSuperModelConstruct() {
        return isSuperModelConstruct;
    }

    @Override
    public void setLexical(boolean lexical) {
	this.construct.setLexical(lexical);
    }

    @Override
    public String getTypeString() {
	return "Construct";
    }
    

}
