package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.util.TypeConverter;

import java.util.Collection;
import java.util.LinkedList;

/**
 * <code>InstanceObject</code> definisce
 * un oggetto istanza.
 * 
 * Il nome ottenibile con {@link #getName()}
 * rappresenta il nome dello schema.
 * 
 * Il possibile valore e' invece ottenibile con {@link #toString()}
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class InstanceObject extends AbstractConstruct {
	private static final long serialVersionUID = 1L;
	private transient boolean lexical;
    private transient Collection<AbstractProperty> abstractProperties = new LinkedList<AbstractProperty>();
    private transient Collection<AbstractReference> abstractReferences = new LinkedList<AbstractReference>();
    private String value;
    private Object type;

    public InstanceObject(MultiDirectionAbstractStructure structure, AbstractConstruct parent, int oid, String name, boolean lexical, Collection<AbstractProperty> abstractProperties, Collection<AbstractReference> abstractReferences) {
	super(structure, parent, oid, name);
	this.lexical = lexical;
	this.value = super.getDescription();
	if (abstractProperties != null) {
	    this.abstractProperties = abstractProperties;
	    /* Se esiste un VALUE, imposta la descrizione dell'istanza a tale
	     * valore, altrimenti conserva il nome dello schemeObject */
	    for (AbstractProperty property : abstractProperties) {
		if (property.getName().equals("type")) {
		    this.type = TypeConverter.getType(property.getValue());
		}
		if (property.getName().equals("value")) {
		    this.value = property.getValue();
		}
	    }
	}
	if (abstractReferences != null) {
	    this.abstractReferences = abstractReferences;
	}
    }

    public InstanceObject(MultiDirectionAbstractStructure structure, AbstractConstruct parent, int oid, String name, boolean lexical) {
	this(structure, parent, oid, name, lexical, null, null);
    }

    @Override
    protected Collection<AbstractProperty> doCreateAbstractProperties() {
	return this.abstractProperties;
    }

    @Override
    protected Collection<AbstractReference> doCreateAbstractReferences() {
	return this.abstractReferences;
    }

    @Override
    public boolean isLexical() {
	return this.lexical;
    }

    /**
     * Fornisce non solo una descrizione testuale, ma riporta
     * anche il valore eventuale associato
     */
    @Override
    public String toString() {
	return this.value;
    }

    @Override
    public void setLexical(boolean lexical) {
	this.lexical = lexical;
    }
    
    public Object getType() {
	return type;
    }
    
    @Override
    public String getTypeString() {
	return "Instance Object";
    }

}
