package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp;

import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Class <code>SchemeObjectAdapter</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class SchemeObjectAdapter extends AbstractConstruct {
	private static final long serialVersionUID = 1L;
	private transient SchemeObject schemeObject;
    private transient boolean isSuperScheme;

    public SchemeObjectAdapter(StructureSchemeAdapter structure, ConstructAdapter parent, SchemeObject schemeObject) {
	super(structure, parent, schemeObject, schemeObject.getOID(), schemeObject.getName());
	this.schemeObject = schemeObject;
	if (structure != null) {
	    this.isSuperScheme = structure.isSuperScheme();
	}
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Collection<AbstractProperty> doCreateAbstractProperties() {
	Collection<AbstractProperty> result = new LinkedList<AbstractProperty>();
	Collection<SchemeObjectProperty> properties = this.schemeObject.getProperties();
	for ( SchemeObjectProperty p : properties ) {
	    AbstractProperty ap = AbstractConstructFlyweightFactory.sharedInstance.getSchemeProperty(p, this); 
	    result.add(ap);
	}
	return result;
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Collection<AbstractReference> doCreateAbstractReferences() {
	Collection<AbstractReference> result = new LinkedList<AbstractReference>();
	Collection<SchemeObjectReference> references = this.schemeObject.getReferences();
	for ( SchemeObjectReference r : references ) {
	    SchemeObjectAdapter target;
	    SchemeObject value = r.getValue();
	    if (value != null) {
		target = AbstractConstructFlyweightFactory.sharedInstance.getSchemeObject(value, this.getStructure());
		AbstractReference ar = AbstractConstructFlyweightFactory.sharedInstance.getSchemeReference(r, this, target); 
		result.add(ar);
	    }
	}
	return result;
    }

    @Override
    public boolean isLexical() {
	return this.schemeObject.getMetaInfo().isLexical();
    }

    public boolean isSuperScheme() {
        return isSuperScheme;
    }

    @Override
    public void setLexical(boolean lexical) {
	this.schemeObject.getMetaInfo().setLexical(lexical);
    }
    
    @Override
    public String getTypeString() {
	return "Scheme Element";
    }

}
