package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.TypeID;

import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Class <code>ModelAdapter</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class StructureModelAdapter extends MultiDirectionAbstractStructure {
	private static final long serialVersionUID = 1L;
	private transient Model model;

    /** 
     * Crea un nuovo <code>StructureModelAdapter</code>
     * @param parent il supermodello. Se e' null, allora il modello
     * stesso creato e' un supermodello (non ha alcun padre)
     */
    public StructureModelAdapter(MultiDirectionAbstractStructure parent, Model model) {
	super(parent, model, model.getOID(), model.getName());
	this.model = model;
    }
    
    @Override
    public MultiDirectionAbstractStructure doLoad(PersistenceHandler handler) throws PersistenceException {
	Model modelLoaded;
	if (this.parent != null) {
	    modelLoaded = this.model;
	} else { // e' un superModello
	    modelLoaded = handler.getSuperModel();
	}
	return new StructureModelAdapter(null, modelLoaded);
    }

    @Override
    @SuppressWarnings("unchecked")
    public SortedSet<AbstractConstruct> doCreateAbstractConstructs(MultiDirectionAbstractStructure proxyStructure) {
	SortedSet<AbstractConstruct> result = new TreeSet<AbstractConstruct>();
	Collection<Construct> constructs = this.model.getAbstractConstructs();
	for (Construct c : constructs) {
	    AbstractConstruct ca = AbstractConstructFlyweightFactory.sharedInstance.getConstructAdapter(c, proxyStructure);
	    result.add(ca);
	}
	return result;
    }
    
    public boolean isSuperModel() {
	return this.parent == null;
    }

    @Override
    public TypeID getTypeId() {
	return TypeID.Model;
    }
    
    
    @Override
    public String getTypeString() {
	String type;
	if (isSuperModel()) {
	    type = "SuperModel";
	} else {
	    type = "Model";
	}
	return type;
    }
    
    

}
