package org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp;

import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.TypeID;

import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Class <code>StructureSchemeAdapter</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class StructureSchemeAdapter extends MultiDirectionAbstractStructure {
	private static final long serialVersionUID = 1L;
	private transient Scheme scheme;
    private transient boolean isSuperScheme;

    public StructureSchemeAdapter(MultiDirectionAbstractStructure parent, Scheme scheme, boolean isSuperScheme) {
	super(parent, scheme, scheme.getOID(), scheme.getName());
	this.scheme = scheme;
	this.isSuperScheme = isSuperScheme; 
    }

    @Override
    public MultiDirectionAbstractStructure doLoad(PersistenceHandler handler) throws PersistenceException {
	Scheme schemeLoaded;
	if ( this.isSuperScheme ) {
	    //schemeLoaded = handler.loadSuperModelScheme(this.scheme.getOID());
		schemeLoaded = this.scheme;
	} else {
	    //schemeLoaded = handler.loadScheme(this.scheme.getOID());
		schemeLoaded = this.scheme;
	}
	return new StructureSchemeAdapter(null, schemeLoaded, this.isSuperScheme);
    }

    @Override
    @SuppressWarnings("unchecked")
    public SortedSet<AbstractConstruct> doCreateAbstractConstructs(MultiDirectionAbstractStructure proxyStructure) {
    	SortedSet<AbstractConstruct> result = new TreeSet<AbstractConstruct>();
	Collection<SchemeObject> schemeObjects = this.scheme.getAbstractConstructs();
	for (SchemeObject s : schemeObjects) {
	    AbstractConstruct ca = AbstractConstructFlyweightFactory.sharedInstance.getSchemeObject(s, proxyStructure);
	    result.add(ca);
	}
	return result;
    }

    public AbstractConstruct getAbstractConstruct(int referenceTargetOID, String name) {
	SchemeObject schemeObject = this.scheme.getObject(referenceTargetOID, name);
	return AbstractConstructFlyweightFactory.sharedInstance.getSchemeObject(schemeObject, this);
    }
    
    public boolean isSuperScheme() {
        return isSuperScheme;
    }

    @Override
    public String toString() {
	String description = super.toString();
	/*if (this.isSuperScheme) {
	    description = "(SM) " + description; 
	}*/
	return description;
    }

    @Override
    public TypeID getTypeId() {
	return TypeID.Scheme;
    }
    
    
    @Override
    public String getTypeString() {
	return "Scheme";
    }

}
