package org.modelgen.jpf.visualizer.modelgenview.modelgen.util;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.database.DBConnecter;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.BaseInfoKeyConvenienceFunctions;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.InstanceObject;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.InstanceProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.InstanceReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.imp.SimpleStructure;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Collection;
import java.util.LinkedList;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Class <code>PersistenceFunctions</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class PersistenceFunctions {
    
    /**
     * Definisce un'istanza. Non si e' realizzata questa classe
     * direttamente nel package "adapters.imp" per il principio
     * ADP (Acyclic Dependencies Principle): 
     * le dipendenze nel grafo dei packages 
     * non devono costituire cicli
     * 
     * @author Riccardo Pietrucci
     * @version 0.1
     */
    private static class StructureInstance extends SimpleStructure {
		private static final long serialVersionUID = 1L;

	public StructureInstance(MultiDirectionAbstractStructure parent, int oid, String name, SortedSet<AbstractConstruct> abstractContructs) {
	    super(parent, oid, name, abstractContructs);
	}

	@Override
	protected MultiDirectionAbstractStructure doLoad(PersistenceHandler handler) throws ClassNotFoundException, Exception {
	    PersistenceFunctions.loadSuperModelInstance(this);
	    return this;
	}
	
    }
    
    public static final String PREFIX_INSTANCE = "I_SM_";
    public static final String PREFIX_REFERENCE = "I_";
    public static final String PREFIX_SM_CONSTRUCT = "SM_";

    public static Collection<SimpleStructure> getSuperModelInstances(MultiDirectionAbstractStructure scheme) throws PersistenceException {
	return getTableInstances(scheme, "instancesm");
    }

    public static Collection<SimpleStructure> getInstances(MultiDirectionAbstractStructure scheme) throws PersistenceException {
	return getTableInstances(scheme, "instance");
    }

    public static void loadInstance(StructureInstance instance) throws ClassNotFoundException, Exception {
	loadTableInstance(instance, "instance");
    }

    public static void loadSuperModelInstance(StructureInstance instance) throws ClassNotFoundException, Exception {
	loadTableInstance(instance, "instancesm");
    }

    private static Collection<SimpleStructure> getTableInstances(MultiDirectionAbstractStructure scheme, String instanceTable) throws PersistenceException {
	Collection<SimpleStructure> schemeInstances = new LinkedList<SimpleStructure>();

	try {/* FIXME: rendere piu' modulare? */
	    DBConnecter dbc = new DBConnecter();
	    Connection con = dbc.getConnection();

	    /* Ottieni le istanze dello schema selezionato */
	    Statement st = con.createStatement();
	    String queryInstances = "SELECT oid, name FROM " + instanceTable + " WHERE schemeOID = " + scheme.getOID() + ";";
	    // System.out.println(queryInstances);

	    /* Esegui la query principale per ottenere le istanze */
	    ResultSet rsInstances = st.executeQuery(queryInstances);
	    while (rsInstances.next()) {
		int OID = rsInstances.getInt("OID");
		String name = rsInstances.getString("name");
		SimpleStructure instance = new StructureInstance(scheme, OID, name, null);
		schemeInstances.add(instance);
	    }
	} catch (Exception e) {
	    throw new PersistenceException(e);
	}
	return schemeInstances;
    }




    private static void loadTableInstance(final StructureInstance instance, String instanceTable) throws ClassNotFoundException, Exception {
    	SortedSet<AbstractConstruct> instanceObjects = new TreeSet<AbstractConstruct>();
	MultiDirectionAbstractStructure scheme = instance.getParent();

	/* Dai vincoli semantici, risulta che ad un modello corrisponde
	 * un unico supermodel-construct. Per come sono definite le
	 * strutture non e' ovvio, poiche' potrebbe esistere piu'
	 * modelConstruct dello stesso modello che utilizzano lo stesso
	 * supermodel-construct. Utilizziamo questo vincolo esclusivamente
	 * semantico per mappare il modelConstruct al relativo (unico)
	 * superModelConstruct.
	 * 
	 * Viceversa, da un'istanceObject non e' possibile risalire al
	 * particolare schemaObject, ma solo al modelConstruct. */
	SortedSet<AbstractConstruct> modelConstructsUsed = new TreeSet<AbstractConstruct>();
	for(AbstractConstruct schemeObject : scheme.getAbstractConstructs()) {
	    AbstractConstruct modelConstruct = schemeObject.getParent();
	    modelConstructsUsed.add(modelConstruct);
	}

	try {/* FIXME: rendere piu' modulare? */
	    DBConnecter dbc = new DBConnecter();
	    Connection con = dbc.getConnection();

	    /* Ottieni le istanze dello schema selezionato */
	    Statement st = con.createStatement();
	    for (AbstractConstruct smConstruct : modelConstructsUsed) {
		/* Dai vincoli semantici, risulta che ai modelConstruct corrisponde
		 * un unico supermodel-construct. Per come sono definite le
		 * strutture non e' ovvio, poiche' potrebbe esistere piu'
		 * modelConstruct dello **stesso** modello che utilizzano lo **stesso**
		 * supermodel-construct. Utilizziamo questo vincolo esclusivamente
		 * semantico per ottenere dai modelConstruct il relativo (unico)
		 * superModelConstruct. */
		String instanceSchemeObjectTable = PREFIX_INSTANCE+smConstruct.getName();
		String smConstructTable = PREFIX_SM_CONSTRUCT+smConstruct.getName();

		/* Ottengo i dati del particolare instanceObject */
		String querySchemeObjectValues = "SELECT\n\tiConstr.oid as ioid, smConstr.oid as soid, * "+
		"\nFROM\n\t"+instanceSchemeObjectTable+" AS iConstr, \n\t"+ smConstructTable + " AS smConstr, \n\t"+instanceTable+" AS i "+
		"\nWHERE \n\tiConstr.instanceoid = "+instance.getOID()+" AND \n\t"+
		"i.oid = iConstr.instanceoid AND \n\tiConstr."+smConstruct.getName()+"oid = smConstr.oid;";
		//System.out.println(querySchemeObjectValues);
		ResultSet rsInstanceObjectValues = st.executeQuery(querySchemeObjectValues);
		while (rsInstanceObjectValues.next()) {
		    int instanceObjectOID = rsInstanceObjectValues.getInt("ioid");
		    int schemeObjectOID = rsInstanceObjectValues.getInt("soid");
		    //Pair<String,String> schemeKey = new Pair<String,String>(smConstruct.getName(), schemeObjectName);
		    AbstractConstruct schemeObject = BaseInfoKeyConvenienceFunctions.getSuperModelSchemeObject(smConstruct.getOID(), schemeObjectOID);
		    //Pair<Integer,Integer> schemeKey = new Pair<Integer,Integer>(smConstruct.getOID(), schemeObjectOID);
		    assert schemeObject != null;

		    /* Copia le proprieta' dal relativo schemeObject all'instanceObject */
		    Collection<AbstractProperty> properties = new LinkedList<AbstractProperty>();
		    for (AbstractProperty schemeProperty : schemeObject.getAbstractProperties()) {
			InstanceProperty ip;
			if (schemeProperty.getName().equals("type")) { // se e' lexical
			    String instanceValue = rsInstanceObjectValues.getString("VALUE");
			    ip = new InstanceProperty(schemeProperty, schemeProperty.getOID(), "value", schemeProperty.getValue(), instanceValue);
			    
			    /* FIXME: questa istruzione e' necessaria a causa di un bug in
			     * ModelGenCore: non gestisce correttamente l'info isLexical */
			    schemeObject.setLexical(true); 
			} else {
			    ip = new InstanceProperty(schemeProperty, schemeProperty.getOID(), schemeProperty.getName(), schemeProperty.getType(), schemeProperty.getValue());
			}
			properties.add(ip);
		    }


		    /* Costruisco un insieme vuoto di reference */
		    Collection<AbstractReference> references = new LinkedList<AbstractReference>();
		    for (AbstractReference reference : schemeObject.getAbstractReferences()) {   
			InstanceReference ir = new InstanceReference(reference, reference.getOID(), reference.getName(), reference.getType());
			references.add(ir);
		    }

		    /* Inserisco l'instanceObject ancora incompleto dei reference */
		    InstanceObject object = new InstanceObject(instance, schemeObject, instanceObjectOID, schemeObject.getName(), schemeObject.isLexical(), properties, references);
		    instanceObjects.add(object);
		}
	    }

	    /* Ora che si hanno tutti gli InstanceObject, inserisci i reference */
	    for (AbstractConstruct ownerInstanceObject : instanceObjects) {
		AbstractConstruct smConstruct = ownerInstanceObject.getParent().getParent();
		String sourceTable = PREFIX_INSTANCE+smConstruct.getName();
		String smSourceTable = PREFIX_SM_CONSTRUCT+smConstruct.getName();
		for (AbstractReference abstractReference : ownerInstanceObject.getAbstractReferences()) {
		    InstanceReference instanceReference = (InstanceReference) abstractReference;
		    AbstractReference smReference = instanceReference.getParent().getParent();
		    AbstractConstruct smTarget = smReference.getTarget();
		    String targetTable = PREFIX_INSTANCE+smTarget.getName();
		    String smTargetTable = PREFIX_SM_CONSTRUCT+smTarget.getName();
		    String referenceSchemeObjectTable = PREFIX_REFERENCE + smReference.getName();
		    String sqlQuery = "SELECT DISTINCT\n\ttarget.oid, smTarget.oid, smConstruct.oid\nFROM"
			+ "\n\t"+sourceTable + " as source, " 
			+ "\n\t"+smSourceTable + " as smSource, " 
			+ "\n\t"+targetTable + " as target, " 
			+ "\n\t"+smTargetTable + " as smTarget, " 
			+ "\n\tMSM_Construct as smConstruct "
			+ "\nWHERE\n\tsource.INSTANCEOID = " + instance.getOID() 
			+ " AND \n\tsource."+smConstruct.getName()+"oid = smSource.oid"
			+ " AND \n\tsource.oid = " +ownerInstanceObject.getOID()
			+ " AND \n\tsource." + referenceSchemeObjectTable + " = target.oid"
			+ " AND \n\tsmTarget.oid = target."+smTarget.getName()+"oid"
			+ " AND \n\tsmConstruct.name = '"+smTarget.getName()+"';";
		    //System.out.println("Instance: " + ownerInstanceObject + ",oid:"+ownerInstanceObject.getOID()+",parentoid:"+ownerInstanceObject.getParent().getOID());
		    //System.out.println(sqlQuery);
		    ResultSet rsReferences = st.executeQuery(sqlQuery);
		    instanceReference.setOwner(ownerInstanceObject);
		    while(rsReferences.next()) {
			int instanceTargetOID = rsReferences.getInt(1);
			int smSchemeObjectOID = rsReferences.getInt(2);
			int smConstructOID = rsReferences.getInt(3);
			
			InstanceObject targetInstanceObject = BaseInfoKeyConvenienceFunctions.getInstanceObject(smConstructOID, smSchemeObjectOID, instanceTargetOID);
			instanceReference.setTarget(targetInstanceObject);
			//System.out.println(ownerInstanceObject + "," + ownerInstanceObject.getOID() + " ---> " + targetInstanceObject + "," + targetInstanceObject.getOID());
		    }
		}
	    }

	} catch (Exception e) {
	    throw new PersistenceException(e);
	}
	
	/* Imposta i costrutti */
	instance.setAbstractConstructs(instanceObjects);
    }

}
