package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.jgraph.graph.GraphCell;

/**
 * Class <code>LayoutManager</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public abstract class LayoutManager<V extends ModelComponent> {
    public abstract void doLayout(JGraphFacade<V> graphInfo);

    @SuppressWarnings("unchecked")
	public static <V extends ModelComponent> LayoutManager<V> load(String filename, JGraphFacade<V> graphInfo) {
	ObjectInputStream in = null;
	try {
	    File basePath = ToolConfigurationManager.sharedInstance.getLayoutsPath();
	    if (basePath != null) { // se esiste preponi path
		filename = basePath + File.separator + filename;
	    }
	    File inFile = new File(filename);
	    in = new ObjectInputStream(new FileInputStream(inFile));
	    LayoutCacheManager<V> layout = (LayoutCacheManager<V>) in.readObject();
	    System.out.println("Layout loaded from " +inFile.getAbsolutePath());
	    if(graphInfo.equals(layout.graphInfo)) {
		layout.setGraphInfo(graphInfo); // carica le info non salvate
		return layout;
	    } else {
		in.close();
		System.err.println("Structure has changed from last layout saving: using generated layout");
		System.err.println(" - "+inFile + " deleted ["+inFile.delete()+"]");
		return new LayoutJGraphManager<V>();
	    }
	} catch (Exception e) {
	    if(in != null) {
		try {
		    in.close();
		} catch (IOException e1) {
		    // TODO Auto-generated catch block
		    e1.printStackTrace();
		}
	    }
	    //System.err.println("Warning: "+e.getMessage());
	    return new LayoutJGraphManager<V>();
	}
    }

    public static <V extends ModelComponent> void save(String filename, JGraphFacade<V> graphInfo) throws IOException {
	File basePath = ToolConfigurationManager.sharedInstance.getLayoutsPath();
	if (basePath != null) { // se esiste preponi path
	    filename = basePath + File.separator + filename;
	}
	File inFile = new File(filename);
	LayoutCacheManager<V> layout = new LayoutCacheManager<V>(graphInfo);
	ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(inFile));
	out.writeObject(layout);
	System.out.println("Layout saved in " +inFile.getAbsolutePath());
    }

    private static class LayoutCacheManager<V extends ModelComponent> extends LayoutManager<V> implements Serializable {
	private static final long serialVersionUID = 1L;
	private JGraphFacade<V> graphInfo;
	private Map<V,Point2D> positions;
	private Map<V,Boolean> isExpandedMap;

	public LayoutCacheManager(JGraphFacade<V> graphInfo) {
	    this.graphInfo = graphInfo;
	    this.isExpandedMap = graphInfo.getAllExpandedStates();
	    
	    // le posizioni si aggiornano solo se collassate
	    graphInfo.jGraph.collapseAll(false);
	    this.positions = graphInfo.getAllPositions();
	    graphInfo.jGraph.resetExpand(this.isExpandedMap);
	}

	@Override
	public void doLayout(JGraphFacade<V> graphInfo) {
	    final Map<GraphCell, Map<V,Point2D>> nestedAttributes = new HashMap<GraphCell, Map<V,Point2D>>();

	    // posiziona
	    for (V v : positions.keySet()) {
		if (v.isRoot()) {
		    Point2D position = positions.get(v);
		    Pair<GraphCell, Map<V,Point2D>> moveAction = this.graphInfo.getPositionAttributeMapToSet(v, position.getX(), position.getY());
		    nestedAttributes.put(moveAction.getFirst(), moveAction.getSecond());
		}
	    }
	    graphInfo.jGraph.graphLayout(nestedAttributes);

	    // espandi se necessario
	    if (this.isExpandedMap != null) {
		graphInfo.jGraph.resetExpand(this.isExpandedMap);
	    }
	}

	public JGraphFacade<?> getGraphInfo() {
	    return graphInfo;
	}

	public void setGraphInfo(JGraphFacade<V> graphInfo) {
	    this.graphInfo = graphInfo;
	}

	public boolean isExpanded(V cell) {
	    if (this.isExpandedMap == null) return false;
	    
	    Boolean expanded = this.isExpandedMap.get(cell);
	    return expanded != null ? expanded.booleanValue() : false;
	}
    }

    private static class LayoutJGraphManager<V extends ModelComponent> extends LayoutManager<V> {
	@Override
	public void doLayout(JGraphFacade<V> graphInfo) {
	    graphInfo.jGraph.graphLayout();
	    graphInfo.jGraph.graphLayout(); // doppio layout necessario
	}
    }
}
