package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ShapeInfo;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.DiamondDashedView;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.DiamondView;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.OctagonalView;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.RectangularDashedView;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.RectangularView;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.vertexviews.RoundedView;

import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.VertexView;

/**
 * Class <code>ModelViewCellFactory</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ModelComponentViewCellFactory extends DefaultCellViewFactory {
    private static final long serialVersionUID = 1L;
    public ModelComponentViewCellFactory() {
	
    }

    @Override
    protected VertexView createVertexView(Object cellObj) {
	VertexView view;
	DefaultGraphCell cell = (DefaultGraphCell) cellObj;
	ModelComponent c = (ModelComponent) cell.getUserObject();
	ShapeInfo s = c.getShapeInfo();
	
	if (s == null) {
	    view = new RectangularView(cell);
	} else {
	    boolean isDashed = s.isDashed();
	    switch(s.getShapeType()) {
	    case ROUNDED:
		view = new RoundedView(cell);
		break;
	    case OCTAGON:
		view = new OctagonalView(cell);
		break;
	    case DIAMOND:
		if (!isDashed) {
		    view = new DiamondView(cell);
		} else {
		    view = new DiamondDashedView(cell);
		}
		break;
	    case RECTANGLE: default:
		if (!isDashed) {
		    view = new RectangularView(cell);
		} else {
		    view = new RectangularDashedView(cell);
		}
		
	    	break;
	    }
	}

	return view;
    }

}
