package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;

import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphCell;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;

/**
 * Class <code>GroupableJGraph</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class GroupableJGraph<CellType extends ModelComponent> extends JGraph {
    private static final long serialVersionUID = 1L;
    private List<JGraphDoubleClickListener> listeners = new LinkedList<JGraphDoubleClickListener>();
    private Map<CellType, Boolean> isExpandedMap = new LinkedHashMap<CellType, Boolean>();
    private Map<Pair<DefaultEdge,Boolean>,DefaultGraphCell> cellPort = new LinkedHashMap<Pair<DefaultEdge,Boolean>, DefaultGraphCell>();
    private Map<CellType, CellView> viewsInfo = new LinkedHashMap<CellType, CellView>();
    private Map<GraphCell, Color> colorsToApply = new LinkedHashMap<GraphCell, Color>();
    private static final int INTERCELL_SPACING = 0; // spazio intercella
    private static final int INTERGROUP_SPACING = 10; // spazio intercella
    private static final int CELLGROUP_SPACING = 5; // spazio intercella

    // Construct the Graph using the Model as its Data Source
    public GroupableJGraph(GraphModel model, GraphLayoutCache cache) {
	super(model, cache);
	
	// Overrides the global vertex renderer
	//VertexView.renderer = new JGraphGroupVertexRenderer();
	EdgeView.renderer = new JGraphGroupEdgeRenderer();
	
	
	// Adds redirector for group collapse/expand
	JGraphFoldingManager foldingManager = new JGraphFoldingManager(this);
	super.addMouseListener(foldingManager);
	super.addMouseMotionListener(foldingManager);
	// tooltip register
	ToolTipManager.sharedInstance().registerComponent(this);
	// Carica il fix per il bug in JGraphParallelRouter
	JGraphParallelRouterInstanceFixer.setGraph(this);
	// Make Ports Visible by Default
	super.setPortsVisible(false);
	// Use the Grid
	super.setGridEnabled(true);
	super.setGridVisible(true);
	// Set the Grid Size to 10 Pixel
	super.setGridSize(10);
	// Set the Tolerance to 2 Pixel
	super.setTolerance(2);
	// Jump to default port on connect
	super.setJumpToDefaultPort(true);
	super.setEditable(false);
	super.setDisconnectable(false);
	super.setAntiAliased(true);
	super.setCloneable(false);
    }

//  Determines if a Cell is a Group
    public boolean isGroup(CellType cell) {
	// Map the Cell to its View
	CellView view = super.getGraphLayoutCache().getMapping(cell, false);
	if (view != null)
	    return !view.isLeaf();
	return false;
    }

    // Brings the Specified Cells to Front
    public void toFront(CellType[] c) {
	super.getGraphLayoutCache().toFront(c);
    }

    // Sends the Specified Cells to Back
    public void toBack(CellType[] c) {
	super.getGraphLayoutCache().toBack(c);
    }
    
    public void expandAll() {
	this.expandAll(true);
    }

    public void expandAll(boolean saveState) {
	CellView[] rootViews = super.graphLayoutCache.getRoots();
	for (CellView rootView : rootViews) {
	    if (DefaultGraphModel.isGroup(super.graphModel, rootView.getCell())) {
		expandAll(rootView, saveState);
	    }
	}
    }

    public void collapseAll(boolean saveState) {
	CellView[] rootViews = super.graphLayoutCache.getRoots();
	for (CellView rootView : rootViews) {
	    if (DefaultGraphModel.isGroup(super.graphModel, rootView.getCell())) {
		collapseAll(rootView, saveState);
	    }
	}
    }
    
    public void expandAll(CellView startView) {
	this.expandAll(startView, true);
    }

    public void expandAll(CellView startView, boolean saveState) {
	expand(startView, saveState); // caso base ricorsione
	CellView[] cellViews = this.getOnlyDirectDescendants(startView);
	for (CellView cellView : cellViews) {
	    if (DefaultGraphModel.isGroup(super.graphModel, cellView.getCell())) {
		expandAll(cellView, saveState);
	    }
	}
    }
    
    public void collapseAll() {
	this.collapseAll(true);
    }
    
    public void collapseAll(CellView startView) {
	this.collapseAll(startView, true);
    }

    public void collapseAll(CellView startView, boolean saveState) {
	CellView[] cellViews = this.getOnlyDirectDescendants(startView);
	for (CellView cellView : cellViews) {
	    if (DefaultGraphModel.isGroup(super.graphModel, cellView.getCell())) {
		collapseAll(cellView, saveState);
	    }
	}
	collapse(startView, saveState); // caso base ricorsione
    }
    
    
    public void expand(CellView cell) {
	this.expand(cell, true);
    }
    
    public void expand(CellType cell) {
	this.expand(cell, true);
    }
    
    public void expand(CellType cell, boolean saveState) {
	CellView cellView = this.viewsInfo.get(cell);
	if (cellView != null) {
	    this.expand(cellView, saveState);
	}
    }

    @SuppressWarnings("unchecked")
	public void expand(CellView cellView, boolean saveState) {
	//System.out.println("\n-------\nexpanding " + cellView.getCell());
	super.graphLayoutCache.expand(new Object[] {cellView.getCell()});
	if (saveState) {
	    this.isExpandedMap.put((CellType) ((DefaultGraphCell) cellView.getCell()).getUserObject(), true);
	}

	/* Ripristina il port. E' dovuto a un bug nel JGraphParallelRouter */
	DefaultGraphCell source = (DefaultGraphCell) cellView.getCell();
	List<DefaultGraphCell> sourceChildPorts = this.getChildPorts(source);
	this.undoChangePort(sourceChildPorts);

	CellView parentView = cellView.getParentView();
	if (parentView == null) {
	    parentView = cellView;
	}
	while (parentView.getParentView() != null) parentView = parentView.getParentView(); 

	calculateGroupBounds(parentView);

	//JGraphDebug.dumpGroupStructure(super.graphModel);

	for(CellView childView : cellView.getChildViews()) {
	    // espandi i figli se gia' espansi
	    if (this.isExpandedMap.containsKey(((DefaultGraphCell) childView.getCell()).getUserObject())) {
		boolean wasExpanded = this.isExpandedMap.get( ((DefaultGraphCell) childView.getCell()).getUserObject()).booleanValue();
		if (wasExpanded) expand(childView, saveState);
	    }

	    // colora all'espansione se bisogna colorare
	    if (this.colorsToApply.containsKey(childView.getCell())) {
		Color colorToApply = this.colorsToApply.get(childView.getCell());
		this.setColor(childView, colorToApply);
		// colorato. elimina dalla coda
		this.colorsToApply.remove(childView.getCell());
	    }
	}
    }
    
    public void collapse(CellView cellView) {
	this.collapse(cellView, true);
    }

    @SuppressWarnings("unchecked")
	public void collapse(CellView cellView, boolean saveState) {
	//System.out.println("\n-------\ncollapsing " + cellView.getCell());
	super.graphLayoutCache.collapse(new Object[] {cellView.getCell()});

	if (saveState) {
	    this.isExpandedMap.put((CellType) ((DefaultGraphCell) cellView.getCell()).getUserObject(), false);
	}

	CellView rootView = cellView.getParentView();
	if (rootView == null) {
	    rootView = cellView;
	}
	while (rootView.getParentView() != null) rootView = rootView.getParentView(); 

	calculateGroupBounds(rootView);
	
	super.graphLayoutCache.expand(new Object[] {cellView.getCell()});
	super.graphLayoutCache.collapse(new Object[] {cellView.getCell()});

	
	// Modifica il port. E' dovuto a un bug nel JGraphParallelRouter
	DefaultGraphCell source = (DefaultGraphCell) cellView.getCell();
	DefaultGraphCell newSourcePort = (DefaultGraphCell) source.getFirstChild();
	List<DefaultGraphCell> sourceChildPorts = this.getChildPorts(source);
	this.changePort(sourceChildPorts, newSourcePort);
	//System.out.println("changing port of " + sourceChildPorts + " to " +newSourcePort);
	
	//JGraphDebug.dumpGroupStructure(super.graphModel);
    }

    private List<DefaultGraphCell> getChildPorts(DefaultGraphCell source) {
	List<DefaultGraphCell> sourceChildPorts = new LinkedList<DefaultGraphCell>();
	for (Object c : source.getChildren()) {
	    if( ! this.graphModel.isPort(c) ) {
		DefaultGraphCell oldSource = (DefaultGraphCell) c;
		DefaultGraphCell oldSourcePort = (DefaultGraphCell) oldSource.getFirstChild();
		sourceChildPorts.add(oldSourcePort);
	    }
	}
	return sourceChildPorts;
    }

    public void addDoubleClickListener(JGraphDoubleClickListener listener) {
	this.listeners.add(listener);
    }

    /**
     * Calcola il bound per il gruppo, impostando
     * ascissa e ordinata secondo la posizione attuale
     * della cella
     * @param cellView la radice di inizio
     */
    private void calculateGroupBounds(CellView cellView) {
	// bugfix: imposta il parallel edge
	JGraphParallelRouterInstanceFixer.setGraph(this);
	
	Rectangle2D bounds = cellView.getBounds();
	double xPosition = bounds.getX();
	double yPosition = bounds.getY();
	calculateGroupBounds(cellView, xPosition, yPosition);
    }

    /**
     * Calcola il bound per il gruppo
     * @param parentView la radice di inizio
     * @param xPosition la ascissa di inizio
     * @param yPosition l'ordinata di inizio 
     */
    private void calculateGroupBounds(CellView parentView, double xPosition, double yPosition) {
	//System.out.println("Posizione di " + parentView.getCell() + ": (" +xPosition +"," + yPosition + ")");
	
	/* Calcola la posizione del child */
	Rectangle2D childBounds = new Rectangle2D.Double(xPosition, yPosition, 0D, 0D);

	/* Modifica la posizione del child */ 
	Map<?,?> childMap = new Hashtable<Object,Object>();
	GraphConstants.setBounds(childMap, childBounds);
	GraphConstants.setAutoSize(childMap, true);

	/* Salva la posizione del child.
	 * La mappa dovrebbe essere di tipo generico:
	 * <CellType,Map>, ma per evitare continui cast
	 * si conserva la forma safe <Object, Map> */
	Map<Object, Map<?,?>> graphMap = new Hashtable<Object, Map<?,?>>(); 
	graphMap.put(parentView.getCell(),childMap); 
	super.graphModel.edit(graphMap, null, null, null);
	super.graphLayoutCache.update(parentView);


	boolean isGroup = DefaultGraphModel.isGroup(super.graphModel, parentView.getCell());
	if (isGroup) { // se e' un gruppo ho il caso ricorsione 
	    /* Di base trasla di inset */
	    double inset = GraphConstants.getInset(parentView.getAllAttributes());
	    yPosition += inset;
	    /* Per tutti i figli */
	    CellView[] childrensView = this.getOnlyDirectDescendants(parentView);
	    CellView lastChild = null;
	    for (CellView childView : childrensView) {
		boolean isGroupOrVertex = DefaultGraphModel.isVertex(super.graphModel, childView.getCell()) || DefaultGraphModel.isGroup(super.graphModel, childView.getCell());
		boolean isDirectChild = (childView.getParentView() == parentView);

		/* Se e' un figlio (gruppo o nodo semplice) */
		if (childView != parentView && isDirectChild && isGroupOrVertex) {
		    inset = GraphConstants.getInset(parentView.getAllAttributes());
		    //System.out.println("Calculating bounds of "+parentView.getCell());
		    //System.out.println("inset: "+inset);

		    if (DefaultGraphModel.isGroup(super.graphModel, childView.getCell())) {
			/* Se ho due gruppi vicini, inserisci lo spazio di inset */
			if(lastChild != null && DefaultGraphModel.isGroup(super.graphModel, lastChild.getCell())) {
			    yPosition += INTERGROUP_SPACING;
			} else if (lastChild != null && DefaultGraphModel.isVertex(super.graphModel, lastChild.getCell())) { // cella e gruppo vicino
			    yPosition += CELLGROUP_SPACING;
			} // else port o null
		    }
		    
		    calculateGroupBounds(childView, xPosition + inset, yPosition);
		    double offset = childView.getBounds().getHeight() + INTERCELL_SPACING;
		    yPosition += offset;
		}
		lastChild = childView;
	    }
	}
    }

    private CellView[] getOnlyDirectDescendants(CellView start) {
	CellView[] allChildrensView = super.graphLayoutCache.getAllDescendants(new CellView[] {start});
	Vector<CellView> directChildrensView = new Vector<CellView>(allChildrensView.length);
	for (CellView childView : allChildrensView) {
	    boolean isDirectChild = (childView.getParentView() == start);
	    boolean isTheSame = childView == start;

	    if (!isTheSame && isDirectChild ) {
		directChildrensView.add(childView);
	    }
	}

	CellView[] result = new CellView[directChildrensView.size()];
	return directChildrensView.toArray(result);
    }

    /** 
     * Esegue il layout del grafo 
     */
    public void graphLayout() {
	Map<?,?> layoutResult = this.doGraphLayout();
	this.graphLayout(layoutResult);
    }

    /** 
     * Salva l'associazione valoreNodo - nodoGrafico
     * che verra' usato in
     * {@link #setColor(ModelComponent, Color)}
     * DA ESEGUIRE ESTERNAMENTE PRIMA DEL LAYOUT
     */ 
    @SuppressWarnings("unchecked")
	public void saveViewsInfo() {
	CellView[] openViews = super.graphLayoutCache.getAllViews();
	for(CellView openView : openViews) {
	    DefaultGraphCell cell = (DefaultGraphCell) openView.getCell();
	    boolean isGroupOrVertex = DefaultGraphModel.isVertex(super.graphModel, cell) || DefaultGraphModel.isGroup(super.graphModel, cell);
	    if (isGroupOrVertex) {
		CellType value = (CellType) cell.getUserObject();
		this.viewsInfo.put(value, openView);
	    }
	}
    }

    public void graphLayout(Map<?,?> layoutResult) {
	this.collapseAll(false); // layout riferito al grafo logico
	// Applica i risultati al grafo corrente
	super.graphLayoutCache.edit(layoutResult);

	/* Necessario per allineare gli archi paralleli */
	//this.expandAll(true);
	this.collapseAll(false);
    }

    /**
     * hook for subclassers
     * 
     * @return il risultato del layout
     */
    protected Map<?,?> doGraphLayout() {
	// Pass the facade the JGraph instance
	JGraphFacade facade = new JGraphFacade(this);
	facade.setIgnoresUnconnectedCells(false);
	facade.setIgnoresHiddenCells(true);
	facade.setIgnoresCellsInGroups(true);
	facade.setEdgePromotion(true);

	// Create an instance of the appropriate layout
	JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout(); 
	//layout.setDeterministic(true);
	layout.setCompactLayout(true);
	layout.setFineTuning(true);
	layout.setFixRoots(false);
	layout.setOrientation(SwingConstants.EAST);
	//layout.setInterHierarchySpacing(500); // spazio verticale
	//layout.setIntraCellSpacing(500); // spazio verticale
	layout.setInterRankCellSpacing(100); // spazio orizzontale
	// Run the layout on the facade. Note that layouts do not implement the Runnable interface, to avoid confusion
	layout.run(facade); 
	// Obtain a map of the resulting attribute changes from the facade
	Map<?,?> layoutResult = facade.createNestedMap(true, true);
	return layoutResult;
    }

    /**
     * Restituisce la radice di un nodo
     * @param cellView il nodo da analizzare
     * @return la radice
     */
    public CellView calculateRoot(CellView cellView) {
	CellView parentView = cellView.getParentView();
	CellView rootView = cellView;
	while (parentView != null) {
	    rootView = parentView;
	    parentView = parentView.getParentView();
	}
	return rootView;
    }

    /** Notifica gli ascoltatori che e' avvenuto un doppio click
     * in un nodo specifico 
     * @param cellClicked il nodo cliccato
     */
    public void fireDoubleClicked(DefaultGraphCell cellClicked) {
	for (JGraphDoubleClickListener listener : listeners) {
	    listener.doubleClickedPerformed(cellClicked);
	}
    }

    public boolean isExpanded(CellView cellView) {
	return isExpanded((DefaultGraphCell) cellView.getCell());
    }
    
    public boolean isExpanded(DefaultGraphCell cell) {
	Boolean expanded = this.isExpandedMap.get(cell.getUserObject());
	return expanded != null ? expanded.booleanValue() : false;
    }


    @Override
    public String getToolTipText(MouseEvent event) {
	String tooltip = null;
	Collection<CellView> toolTipableViews = JGraphFoldingManager.getCellsInPosition(this, event.getPoint(), false);
	CellView leafView = JGraphFoldingManager.getLeaf(toolTipableViews, event.getPoint(), (DefaultGraphModel) super.graphModel);
	if (leafView != null) {
	    DefaultGraphCell cell = (DefaultGraphCell) leafView.getCell();
	    if (cell != null) {
		ModelComponent value = (ModelComponent) cell.getUserObject();
		tooltip = value.getTooltipText();
	    }
	}
	return tooltip;
    }

    private void changePort(List<DefaultGraphCell> sourceChildPorts, DefaultGraphCell newPort) {
	/* Devo modificare tutti i port e tutti gli archi che si connettono
	 * ai port modificati */
	for (int i = 0; i < this.graphModel.getRootCount(); i++) {
	    Object child = this.graphModel.getRootAt (i);
	    if(this.graphModel.isEdge(child)) {
		DefaultEdge edge = (DefaultEdge) child;

		DefaultGraphCell edgeSourcePort = null;
		Pair<DefaultEdge,Boolean> edgeSourceKey = new Pair<DefaultEdge,Boolean>(edge, true);
		if (this.cellPort.containsKey(edgeSourceKey)) {
		    edgeSourcePort = this.cellPort.get(edgeSourceKey);
		} else {
		    /* Se non esiste allora significa che e' "l'originale" */
		    edgeSourcePort = (DefaultGraphCell) edge.getSource();
		    // salva per un successivo undo
		    this.cellPort.put(edgeSourceKey, edgeSourcePort);
		}
		while ( !sourceChildPorts.contains(edgeSourcePort) && edgeSourcePort.getParent().getParent() != null) {
		    edgeSourcePort = (DefaultGraphCell) edgeSourcePort.getParent().getParent().getChildAt(0);
		}
		/* se e' stato trovato un port nella gerarchia */
		if (sourceChildPorts.contains(edgeSourcePort)) {
		    ConnectionSet cSet = new ConnectionSet(edge, newPort, true);
		    this.getGraphLayoutCache().edit(null, cSet, null, null);
		    //System.out.println("Changing in port from " + edgeSourcePort + " to " + newPort);
		}

		DefaultGraphCell edgeTargetPort = null;
		Pair<DefaultEdge,Boolean> edgeTargetKey = new Pair<DefaultEdge,Boolean>(edge, false);
		if (this.cellPort.containsKey(edgeTargetKey)) {
		    edgeTargetPort = this.cellPort.get(edgeTargetKey);
		} else {
		    /* Se non esiste allora significa che e' "l'originale" */
		    edgeTargetPort = (DefaultGraphCell) edge.getTarget();
		    // salva per un successivo undo
		    this.cellPort.put(edgeTargetKey, edgeTargetPort);
		}
		while ( !sourceChildPorts.contains(edgeTargetPort) && edgeTargetPort.getParent().getParent() != null ) {
		    edgeTargetPort = (DefaultGraphCell) edgeTargetPort.getParent().getParent().getChildAt(0);
		}
		/* se e' stato trovato un port nella gerarchia */
		if (sourceChildPorts.contains(edgeTargetPort)) {
		    ConnectionSet cSet = new ConnectionSet(edge, newPort, false);
		    this.getGraphLayoutCache().edit(null, cSet, null, null);
		    //System.out.println("Changing out port from " + edgeTargetPort + " to " + newPort);
		}
	    }
	}
    }

    /**
     * @param sourceChildPorts
     */
    private void undoChangePort(List<DefaultGraphCell> sourceChildPorts) {
	/* Devo modificare tutti i port e tutti gli archi che si connettono
	 * ai port modificati */
	for (int i = 0; i < this.graphModel.getRootCount(); i++) {
	    Object child = this.graphModel.getRootAt (i);
	    if(this.graphModel.isEdge(child)) {
		DefaultEdge edge = (DefaultEdge) child;

		// ottieni lo stato salvato
		Pair<DefaultEdge,Boolean> edgeSourceKey = new Pair<DefaultEdge,Boolean>(edge, true);
		DefaultGraphCell newSourcePort = this.cellPort.get(edgeSourceKey);
		while ( newSourcePort.getParent().getParent() != null && !sourceChildPorts.contains(newSourcePort)) {
		    newSourcePort = (DefaultGraphCell) newSourcePort.getParent().getParent().getChildAt(0);
		}
		if (sourceChildPorts.contains(newSourcePort)) {
		    ConnectionSet cSet = new ConnectionSet(edge, newSourcePort, true);
		    this.getGraphLayoutCache().edit(null, cSet, null, null);
		    //System.out.println("Undoing in port from " + edgeSourcePort + " to " + newSourcePort);
		}

		// ottieni lo stato salvato
		Pair<DefaultEdge,Boolean> edgeTargetKey = new Pair<DefaultEdge,Boolean>(edge, false);
		DefaultGraphCell newTargetPort = this.cellPort.get(edgeTargetKey);
		while ( newTargetPort.getParent().getParent() != null && !sourceChildPorts.contains(newTargetPort)) {
		    newTargetPort = (DefaultGraphCell) newTargetPort.getParent().getParent().getChildAt(0);
		}
		if (sourceChildPorts.contains(newTargetPort)) {
		    ConnectionSet cSet = new ConnectionSet(edge, newTargetPort, false);
		    this.getGraphLayoutCache().edit(null, cSet, null, null);
		    //System.out.println("Undoing out port from " + edgeTargetPort + " to " + newTargetPort);
		}
	    }
	}
    }

    public CellView getCellView(CellType cell) {
	return this.viewsInfo.get(cell);
    }

    public void setColor(CellType cell, Color color) {
	// bugfix: imposta il parallel edge
	GroupableJGraph<?> oldWorkingGraph = JGraphParallelRouterInstanceFixer.getGraph();
	if (oldWorkingGraph != this) JGraphParallelRouterInstanceFixer.setGraph(this);
	CellView cellView = this.getCellView(cell);
	if (cellView != null) {
	    this.setColor(cellView, color);
	}
	
	// undo
	if (oldWorkingGraph != this) JGraphParallelRouterInstanceFixer.setGraph(oldWorkingGraph);
    }

    private void setColor(CellView cellView, Color color) {
	Map<DefaultGraphCell, Map<?,?>> nested = new Hashtable<DefaultGraphCell, Map<?,?>>();

	/* ripristina colore del nodo precedente
	if (this.lastFocusRoot  != null) {
	    Map unfocusMap = new Hashtable();
	    GraphConstants.setGradientColor(unfocusMap, Color.white);
	    nested.put((DefaultGraphCell) lastFocusRoot.getCell(), unfocusMap);
	}*/

	if ( isVisible(cellView.getParentView()) ) { // se e' visibile, colora
	    Map<?,?> focusMap = new Hashtable<Object,Object>();
	    GraphConstants.setGradientColor(focusMap, color);
	    nested.put((DefaultGraphCell) cellView.getCell(), focusMap);
	    this.getGraphLayoutCache().edit(nested, null, null, null);
	} else { 
	    // se non e' espanso, metti in coda per successivo coloring
	    this.colorsToApply.put((GraphCell) cellView.getCell(), color);
	}
    }

    public Map<CellType, Boolean> getAllExpandedStates() {
	return this.isExpandedMap;
    }

    public void resetExpand(Map<CellType, Boolean> isExpandedMap) {
	this.isExpandedMap = isExpandedMap; // imposta la nuova mappa
	CellView[] rootViews = super.graphLayoutCache.getRoots();
	for (CellView rootView : rootViews) {
	    boolean wasExpanded = this.isExpanded(rootView);
	    if (wasExpanded) 
		expand(rootView, false); // espandi seguendo la mappa
	}
    }
    
    /**
     * Un nodo e' visibile se e' espanso se' stesso e tutti i padri 
     */
    private boolean isVisible(CellView cellView) {
	if (cellView == null) return true;
	CellView parentView = cellView.getParentView();
	return isExpanded(cellView) && isVisible(parentView);
    }
    
    

}
