/*
 * $Id: JGraphFoldingManager.java,v 1.13 2007-05-25 17:03:09 paolo Exp $
 * Copyright (c) 2001-2005, Gaudenz Alder
 * 
 * All rights reserved. 
 * 
 * This file is licensed under the JGraph software license, a copy of which
 * will have been provided to you in the file LICENSE at the root of your
 * installation directory. If you are unable to locate this file please
 * contact JGraph sales for another copy.
 */
package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable;

import org.modelgen.jpf.visualizer.pac.TopLevelPAC;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.event.MouseInputAdapter;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.VertexRenderer;

/**
 * Mananges the folding and unfolding of groups
 */
class JGraphFoldingManager extends MouseInputAdapter {
    
    private GroupableJGraph<?> graph;

    public JGraphFoldingManager(GroupableJGraph<?> graph) {
	this.graph = graph;
    }

    /**
     * Called when the mouse button is released to see if a collapse or expand
     * request has been made
     */
    public void mousePressed(MouseEvent e) {
	    Point mousePosition = e.getPoint();
	    Collection<CellView> views = getCellsInPosition(graph, mousePosition, true);

	    /* Ottieni l'elemento piu' annidato */
	    CellView cellView = getLeaf(views, mousePosition, (DefaultGraphModel) graph.getModel());

	    /* Se ho un doppio click, visualizza l'elemento piu' annidato */
	    if (e.getClickCount() == 2 && cellView != null) { // doppio click
		DefaultGraphCell cellClicked = (DefaultGraphCell) cellView.getCell();
		graph.fireDoubleClicked(cellClicked);
	    }
	    
	    if (cellView != null && isClickedGroupHandle(graph, cellView, mousePosition)) {
		graph.setMoveable(false); // impedisci drag se nel pulsante
		//System.out.println("\n\n##### clicked "+cellView.getCell() +" #####\n\n");
		if (cellView.isLeaf()) {
		    if (e.getButton() == MouseEvent.BUTTON1) { // left
			graph.expand( cellView );
		    } else if (e.getButton() == MouseEvent.BUTTON3) { // right
			graph.expandAll( cellView );
		    }
		} else {
		    if (e.getButton() == MouseEvent.BUTTON1) { // left
			graph.collapse( cellView );
		    } else if (e.getButton() == MouseEvent.BUTTON3) { // right
			graph.collapseAll( cellView );
		    }
		}
		
	    }
	    e.consume();
    }

    @Override
    public void mouseReleased(MouseEvent e) {
	boolean moveable = graph.isMoveable();
	if (!moveable) graph.setMoveable(true); // consenti drag dopo il pulsante
	TopLevelPAC.getParent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // reset
    }

    @Override
    public void mouseMoved(MouseEvent e) {
	    Point mousePosition = e.getPoint(); 
	    Collection<CellView> views = getCellsInPosition(graph, mousePosition, true);
	    /* Ottieni l'elemento piu' annidato */
	    CellView cellView = getLeaf(views, mousePosition, (DefaultGraphModel) graph.getModel());
	    boolean mouseInGroupHandle = cellView != null && isClickedGroupHandle(graph, cellView, mousePosition);
	    if (mouseInGroupHandle) {
		graph.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
	    } else {
		graph.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // reset
	    }
    }

    @Override
    public void mouseExited(MouseEvent e) {
	TopLevelPAC.getParent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); //turn off the hand cursor
    }

    private boolean isClickedGroupHandle(GroupableJGraph<?> graph, CellView cellView, Point pt) {
	boolean clickedGroupHandle = false;
	Point2D containerPoint = graph.fromScreen((Point2D) pt.clone());
	Rectangle2D rectBounds = cellView.getBounds();
	containerPoint.setLocation(containerPoint.getX()
		- rectBounds.getX(), containerPoint.getY()
		- rectBounds.getY());
	Component renderer = cellView.getRendererComponent(graph,
		false, false, false);
	if (renderer instanceof JGraphGroupVertexRenderer) {
	    JGraphGroupVertexRenderer group = (JGraphGroupVertexRenderer) renderer;
	    if (group.inHitRegion(containerPoint)) {
		clickedGroupHandle = true;
	    }
	}

	return clickedGroupHandle;

    }

    public static Collection<CellView> getCellsInPosition(GroupableJGraph<?> graph, Point2D pt, boolean onlyVertices) {
	// lista delle celle candidate
	LinkedList<CellView> cellsClicked = new LinkedList<CellView>();
	CellView[] views = graph.getGraphLayoutCache().getCellViews();
	for (int i = 0; i < views.length; i++) {
	    if (views[i].getBounds().contains(pt.getX(), pt.getY())) {
		Component renderer = views[i].getRendererComponent(graph,
			false, false, false);
		/* Necessario perche' nello stesso punto (x,y)
		 * potrebbero coincidere tanti archi e nodi.
		 * Il cast discrimina i 2 casi */
		if (!onlyVertices || renderer instanceof VertexRenderer) {
		    cellsClicked.add(views[i]);
		}
	    }
	}
	return cellsClicked;
    }

    public static CellView getLeaf(Collection<CellView> clickedCellViews, Point clickedPoint, DefaultGraphModel model) {
	LinkedList<CellView> candidateResult = new LinkedList<CellView>();
	/* Filtro le celle cliccate con le celle piu' innestate */ 
	LinkedList<CellView> parents = new LinkedList<CellView>();
	for(CellView cellView : clickedCellViews) { // salvo tutti i padri
	    CellView parentView = cellView.getParentView();
	    if ( parentView != null ) {
		parents.add(parentView);
	    }
	}
	/* Le celle che non risultano tra i padri sono le foglie */
	for(CellView cellView : clickedCellViews) { // salvo tutti i padri
	    if ( ! parents.contains(cellView)) { // && ! model.isPort(cell)) {
		candidateResult.add(cellView);
	    }
	}

	/* Ora ho tutte le celle foglia,
	 * devo valutare il migliore: il piu' vicino al mouse */
	CellView nearestCell = null;
	double minDistance = Double.MAX_VALUE;
	//System.out.println("----------");
	for(CellView currentCell : candidateResult) {
	    Rectangle2D bounds = currentCell.getBounds();

	    double cellViewXDistance = Math.abs(bounds.getCenterX() - clickedPoint.getX());
	    double cellViewYDistance = Math.abs(bounds.getCenterY() - clickedPoint.getY());
	    // teorema di pitagora...
	    double overallDistance = Math.sqrt(Math.pow(cellViewXDistance, 2.0) + Math.pow(cellViewYDistance, 2.0));
	    // debug System.out.println("candidate "+currentCell.getCell()+" distance: "+overallDistance);
	    if(overallDistance < minDistance) {
		minDistance = overallDistance;
		nearestCell = currentCell;
	    }
	}

	return nearestCell;
    }


}
