/*
 * $Id: JGraphGroupVertexRenderer.java,v 1.13 2007-05-25 17:03:09 paolo Exp $
 * Copyright (c) 2001-2005, Gaudenz Alder
 * 
 * All rights reserved. 
 * 
 * This file is licensed under the JGraph software license, a copy of which
 * will have been provided to you in the file LICENSE at the root of your
 * installation directory. If you are unable to locate this file please
 * contact JGraph sales for another copy.
 */
package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.util.HtmlString;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.geom.Point2D;

import javax.swing.JLabel;

import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.VertexRenderer;

/**
 * Renderer for grouped cells, includes '+' '-' to expand/collapse
 */
public class JGraphGroupVertexRenderer extends VertexRenderer { /* ( extends JLabel) */
    private static final long serialVersionUID = 1L;

    public static final int VERTICAL_SPACE_LABEL = 40;

    /**
     * Default handle bounds for renderer, '+' or '-'
     */
    private Rectangle handle = new Rectangle(0, 0, 9, 9);

    /**
     * Specifies whether the current view is a rich text value, and if the image
     * should be stretched.
     */
    protected boolean isGroup = false;

    /**
     * Holds the background and foreground of the graph.
     */
    protected Color handleColor = Color.orange, graphForeground = Color.black;

    private boolean hasChild;


    public JGraphGroupVertexRenderer() {
    }
    
    public void moveHandle(int x, int y) {
	handle.setLocation(x, y);
    }


    /**
     * Overrides the parent implementation to return the value component stored
     * in the user object instead of this renderer if a value component exists.
     * This applies some of the values installed to this renderer to the value
     * component (border, opaque) if the latter is a JComponent.
     * 
     * @return Returns a configured renderer for the specified view.
     */
    public Component getRendererComponent(JGraph graph, CellView view,
	    boolean sel, boolean focus, boolean preview) {
	//handleColor = graph.getHandleColor();
	//graphForeground = graph.getForeground();
	//isGroup = DefaultGraphModel.isGroup(graph.getModel(), view.getCell());
	
	JLabel old = (JLabel) super.getRendererComponent(graph, view, sel, focus, preview);
	
	// il vertice gruppo ha la proprieta' di essere opaco 
	isGroup = old.isOpaque();
	hasChild = DefaultGraphModel.isGroup(graph.getModel(), view.getCell());
	
	DefaultGraphCell cell = (DefaultGraphCell) view.getCell();
	ModelComponent model = (ModelComponent) cell.getUserObject();
	HtmlString name = null;
	if (model != null) {
	    name = model.getHtmlName();
	}
	if (name == null) {
	    name = new HtmlString("<<null>>");
	}
	/*if (isGroup && model.getInfo() instanceof AbstractConstruct) {
	    old.setOpaque(true);
	    AbstractConstruct c = (AbstractConstruct) model.getInfo();
	    AbstractConstruct parent = c.getParent();
	    if (parent != null) {
		// non modificare model
		name = (HtmlString) name.clone();
		name.setText("<center>"+name.getText() + "<br/>" + ""+ parent.getName() + "</center><hr>");
	    }
	}*/
	
	old.setText(name.getHtmlText());

	return old;
    }


    /**
     * renderer paint method
     */
    public void paint(Graphics g) {
	super.paint(g);
	
	if (isGroup && hasChild) {
	    /* Costruisci il quadrato con il '-' */
	    g.setColor(handleColor);
	    g.fill3DRect(handle.x, handle.y, handle.width, handle.height, true);
	    g.setColor(graphForeground);
	    g.drawRect(handle.x, handle.y, handle.width, handle.height);
	    g.drawLine(handle.x + 1, handle.y + handle.height / 2, handle.x
		    + handle.width - 2, handle.y + handle.height / 2);
	    /* Se e' compresso, aggiungi '|' per farlo diventare '+' */
	    if (view.isLeaf())
		g.drawLine(handle.x + handle.width / 2, handle.y + 1, handle.x
			+ handle.width / 2, handle.y + handle.height - 2);
	}
    }

    /**
     * Detect whether or not a point has hit the group/ungroup image
     * 
     * @param pt
     *            the point to check
     * @return whether or not the point lies within the handle
     */
    public boolean inHitRegion(Point2D pt) {
	return isGroup && hasChild && handle.contains(pt.getX(), pt.getY());
    }

}
