package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.List;

import org.jgraph.graph.EdgeView;
import org.jgraph.graph.Edge.Routing;

import com.jgraph.layout.routing.JGraphParallelRouter;

/**
 * Class <code>JGraphParallelRouterFixer</code> compensa
 * un errore in JGraphParallelRouter:
 * la classe ha valori univoci che si riferiscono
 * a livello di classe.
 * 
 * Si vuole invece che tali valori si riferiscano a livello
 * di istanza.
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class JGraphParallelRouterInstanceFixer implements Routing {
    private static JGraphParallelRouterInstanceFixer singleton = new JGraphParallelRouterInstanceFixer();
    private static JGraphParallelRouterInstance currentInstance = null;
    private static final long serialVersionUID = 1L;
    private static double edgeDepartureDefault = 15;
    private static double edgeSeparationDefault = 15;

    public static void setGraph(GroupableJGraph<?> groupableGraph) {
	JGraphParallelRouterInstance router = new JGraphParallelRouterInstance(groupableGraph, edgeDepartureDefault, edgeSeparationDefault);
	groupableGraph.addFocusListener(router);
	router.focusGained(null); // inizializza
    }
    
    public static GroupableJGraph<?> getGraph() {
	return currentInstance.graph;
    }

    public static void setEdgeDeparture(double edgeDeparture) {
	edgeDepartureDefault = edgeDeparture;
    }

    public static void setEdgeSeparation(double edgeSeparation) {
	edgeSeparationDefault = edgeSeparation;
    }

    public static Routing getSharedInstance() {
	return singleton;
    }

    public int getPreferredLineStyle(EdgeView edge) {
	return currentInstance.getPreferredLineStyle(edge);
    }

    public List<?> route(EdgeView edge) {
	List<?> result = null;
	try {
	    result = currentInstance.route(edge);
	} catch (ArrayIndexOutOfBoundsException ex) {
	    /* bug di jgraph: non fare niente */
	}
	return result;
    }


    private static class JGraphParallelRouterInstance extends JGraphParallelRouter implements FocusListener {
		private static final long serialVersionUID = 1L;
	private GroupableJGraph<?> graph;
	private double edgeDeparture;
	private double edgeSeparation;
	public JGraphParallelRouterInstance(GroupableJGraph<?> graph, double edgeDeparture, double edgeSeparation) {
	    this.graph = graph;
	    this.edgeDeparture = edgeDeparture;
	    this.edgeSeparation = edgeSeparation;
	}

	public void focusGained(FocusEvent e) {
	    currentInstance = this;
	    JGraphParallelRouter.setGraph(graph);
	    JGraphParallelRouter.setEdgeDeparture(edgeDeparture);
	    JGraphParallelRouter.setEdgeSeparation(edgeSeparation);
	}

	public void focusLost(FocusEvent e) {
	}
	
    }


}
