package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.prototypes;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ShapeInfo;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable.JGraphParallelRouterInstanceFixer;

import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.SwingConstants;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;

/**
 * Class <code>MyDefaultGraphCellFactory</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class DefaultGraphCellFactory implements GraphCellFactory {
    public static transient final int LABEL_SIZE = 20;
    private static Point2D[] labelPositions;

    static {
	final int EXTRA_LABELS_DISTANCE_RATIO = 16;
	final int EXTRA_LABELS_OFFSET = -5;
	labelPositions = new Point2D[2];
	labelPositions[0] = new Point2D.Double(GraphConstants.PERMILLE*(EXTRA_LABELS_DISTANCE_RATIO - 1)/EXTRA_LABELS_DISTANCE_RATIO, EXTRA_LABELS_OFFSET);
	labelPositions[1] = new Point2D.Double(GraphConstants.PERMILLE/EXTRA_LABELS_DISTANCE_RATIO, EXTRA_LABELS_OFFSET);
    }

    public DefaultGraphCell createDefaultGraphCell(ModelComponent value) {
	DefaultGraphCell cell = new DefaultGraphCell(value);
	// Add a Bounds Attribute to the Map
	GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0D,0D,0D,0D));
	GraphConstants.setAutoSize(cell.getAttributes(), true);
	
	// Add one Floating Port
	DefaultPort port = new DefaultPort(value); 
	//GraphConstants.setOffset(port.getAttributes(), new Point2D.Double(GraphConstants.PERMILLE/2, GraphConstants.PERMILLE/8));
	cell.add(port);
	
	return cell;
    }

    public DefaultEdge createDefaultEdge(ModelComponent value,
	    String fromDescription, String toDescription) {
	DefaultEdge edge = new DefaultEdge(value);

	/*String[] extraLabels = { "", "" };
	if (toDescription != null)
	    extraLabels[0] = toDescription;
	if (fromDescription != null)
	    extraLabels[1] = fromDescription;*/
	//GraphConstants.setExtraLabels(edge.getAttributes(), extraLabels);

	return edge;
    }
    

    public Map<?,?> createCellAttributes(JGraph jGraph, Point2D point, ModelComponent component) {
	Map<?,?> map = new Hashtable<Object,Object>();
	// Snap the Point to the Grid
	if (jGraph != null) {
	    point = jGraph.snap((Point2D) point.clone());
	} else {
	    point = (Point2D) point.clone();
	}
	// Make sure the cell is resized on insert
	GraphConstants.setInset(map, 2);
	GraphConstants.setHorizontalAlignment(map, SwingConstants.LEFT);
	GraphConstants.setHorizontalTextPosition(map, SwingConstants.LEFT);
	GraphConstants.setVerticalAlignment(map, SwingConstants.BOTTOM);
	GraphConstants.setVerticalTextPosition(map, SwingConstants.BOTTOM);
	// Add a nice looking gradient background
	//GraphConstants.setGradientColor(map, Color.blue);
	// Add a Border Color Attribute to the Map
	//GraphConstants.setBorderColor(map, Color.black);
	// Add a White Background
	//GraphConstants.setBackground(map, Color.white);
	// Make Vertex Opaque
	GraphConstants.setOpaque(map, false);
	// Add a Bounds Attribute to the Map
	GraphConstants.setBounds(map, new Rectangle2D.Double(point.getX(),
		point.getY(), 0, 0));
	GraphConstants.setAutoSize(map, true);
	GraphConstants.setSelectable(map, false);
	
	setModelComponentAttributes(map, component, false);
	
	return map;
    }

    public Map<?,?> createEdgeAttributes()
    {
	Map<?,?> map = new Hashtable<Object, Object>();
	// Add a Line End Attribute
	GraphConstants.setLineEnd(map, GraphConstants.ARROW_CLASSIC);
	// Add a label along edge attribute
	GraphConstants.setLabelAlongEdge(map, true);
	GraphConstants.setBendable(map, false);
	GraphConstants.setSelectable(map, false);
	GraphConstants.setDisconnectable(map,false);
	GraphConstants.setEndFill(map, true);
	GraphConstants.setExtraLabelPositions(map,DefaultGraphCellFactory.labelPositions);
	
	/* Attenzione: e' necessario che venga impostata la proprieta'
	 * {@link JGraphParallelRouter#setGraph(JGraph)}	 */
	GraphConstants.setRouting(map, JGraphParallelRouterInstanceFixer.getSharedInstance());
	//GraphConstants.setLineStyle(attributes, GraphConstants.STYLE_SPLINE);
	return map;
    }

    public Map<?,?> createGroupAttributes(DefaultGraphCell group, ModelComponent component) {
	Map<?,?> map = new Hashtable<Object,Object>();
	// Add a Bounds Attribute to the Map
	//Rectangle2D b = GraphConstants.getBounds(group.getAttributes());
	GraphConstants.setBounds(map, new Rectangle2D.Double(0D,0D,0D,0D));
	// Make sure the cell is resized on insert
	GraphConstants.setAutoSize(map, true);
	GraphConstants.setGroupOpaque(map, true);
	GraphConstants.setOpaque(map, true);
	//GraphConstants.setVerticalAlignment(map, SwingConstants.TOP);
	GraphConstants.setChildrenSelectable(map, false);
	GraphConstants.setInset(map, 10);
	Font oldFont = GraphConstants.getFont(map);
	Font newFont = new Font(oldFont.getFamily(), Font.BOLD, oldFont.getSize()+1);
	//Font font = new Font(Font.SANS_SERIF, Font.BOLD, 15);
	GraphConstants.setFont(map, newFont);
	//GraphConstants.setForeground(map, Color.blue);
	
	//DefaultGraphCell port = (DefaultGraphCell) group.getFirstChild();
	//GraphConstants.setOffset(port.getAttributes(), new Point2D.Double(0, 0));
	
	setModelComponentAttributes(map, component, true);
	
	//Color borderColor = GraphConstants.getBorderColor(map);
	//GraphConstants.setBorder(map, new ShadowBorder(borderColor));

	return map;
    }
    
    

    /**
     * Aggiunge informazioni grafiche basandosi sulle informazioni
     * ShapeInfo di ModelComponent
     */
    private void setModelComponentAttributes(Map<?,?> map, ModelComponent c, boolean isGroup) {
	ShapeInfo s = c.getShapeInfo();
	if (s == null) return;
	
	Color background = s.getBackground();
	Color borderColor = s.getBorderColor();
	Font font = s.getFont();
	Icon icon = s.getIcon();
	
	if (background != null) GraphConstants.setGradientColor(map, background);
	if (font != null) GraphConstants.setFont(map, font);
	if (icon != null) GraphConstants.setIcon(map, icon);
	
	if (isGroup && borderColor != null) GraphConstants.setBorderColor(map, borderColor);
	
	
	
	
    }
}
