package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.table;

import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * La classe <code>ColumnHeaderToolTips</code> consente
 * di aggiungere tooltip alle colonne nella seguente forma:
 * 
<pre>ColumnHeaderToolTips tips = new ColumnHeaderToolTips();
    
    // Assign a tooltip for each of the columns
    for (int c=0; c&lt;table.getColumnCount(); c++) {
        TableColumn col = table.getColumnModel().getColumn(c);
        tips.setToolTip(col, "Col "+c);
    }
    table.getTableHeader().addMouseMotionListener(tips);</pre>
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ColumnHeaderToolTips extends MouseMotionAdapter {
    // Current column whose tooltip is being displayed.
    // This variable is used to minimize the calls to setToolTipText().
    TableColumn curCol;

    // Maps TableColumn objects to tooltips
    Map<TableColumn, String> tips = new HashMap<TableColumn, String>();

    // If tooltip is null, removes any tooltip text.
    public void setToolTip(TableColumn col, String tooltip) {
        if (tooltip == null) {
            tips.remove(col);
        } else {
            tips.put(col, tooltip);
        }
    }

    public void mouseMoved(MouseEvent evt) {
        TableColumn col = null;
        JTableHeader header = (JTableHeader)evt.getSource();
        JTable table = header.getTable();
        TableColumnModel colModel = table.getColumnModel();
        int vColIndex = colModel.getColumnIndexAtX(evt.getX());

        // Return if not clicked on any column header
        if (vColIndex >= 0) {
            col = colModel.getColumn(vColIndex);
        }

        if (col != curCol) {
            header.setToolTipText(tips.get(col));
            curCol = col;
        }
    }
}