package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.table;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.rows.multivalue.MultiValueTableModel;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

/**
 * Class <code>NullCellRenderer</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class MultiValueCellRenderer extends DefaultTableCellRenderer {
    private static final long serialVersionUID = 1L;
    private MultiValueTableModel<?> model;
    private Color foreground = null;
    private Color selForeground = null;
    private Color background = null;
    private Color selBackground = null;
    private static final String NULL_TEXT = "(null)";
    
    public MultiValueCellRenderer(MultiValueTableModel<?> model) {
	super();
	this.model = model;
    }
    
    private void saveState(JComponent renderer, boolean isSelected) {
	if (!isSelected) {
	    if(this.background == null && this.foreground == null) {
		this.background = renderer.getBackground();
		this.foreground = renderer.getForeground();
	    }
	} else {
	    if( this.selBackground == null && this.selForeground == null) {
		this.selBackground = renderer.getBackground();
		this.selForeground = renderer.getForeground();
	    }
	}
    }

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
	JComponent renderer = (JComponent) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	boolean emptyValue = false;
	
	/* Inizializza i colori: TODO fix selectionForeground... */
	this.saveState(renderer, isSelected);
	
	// celle non presenti piu' scure
	if(value != null && value.equals("")) {
		emptyValue = true;
	}
	
	// cambia cella (blankValue) in vuota ma non piu' scura
	if(value == BaseInfo.EMPTY_NAME) {
		value = "";
	} else if (value == null) {
		value = NULL_TEXT;
	}


	/* Se il Boolean ha un valore null, viene trattato semplicemente
	 * come un valore false. Per questo viene verificato se ho
	 * un'istanza di DefaultTableCellRenderer */
	if (value == NULL_TEXT) {
	    if (renderer instanceof DefaultTableCellRenderer) {
		DefaultTableCellRenderer jlabelRenderer = (DefaultTableCellRenderer) renderer;
		jlabelRenderer.setToolTipText("null value");
	    }
	} else { // renderer specifico per il tipo primitivo
	    TableCellRenderer cellRenderer = table.getDefaultRenderer(value.getClass());
	    if (cellRenderer != this) { // no ciclo
		renderer = (JComponent) cellRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		/* Inizializza i colori: TODO fix selectionForeground... */
		this.saveState(renderer, isSelected);
	    }
	}
	
	/* Ripristina la vecchia visualizzazione (undo) */
	if (!isSelected) {
	    renderer.setBackground(this.background);
	    renderer.setForeground(this.foreground);
	} else {
	    renderer.setBackground(this.selBackground);
	    renderer.setForeground(this.selForeground);
	}
	
	/* Indipendentemente da null o meno, se e' un figlio
	 * sono tutti dello stesso colore */
	if (this.model.isChildren(row)) {
	    //renderer.setBackground(new Color(240,240,240));
	    Color childrenColor = new Color(255,255,180);
	    if (isSelected) {
		childrenColor = childrenColor.darker();
	    }
	    renderer.setBackground(childrenColor);
	}
	
	if (this.model.isCompositeElement(row)) {
	    //renderer.setBackground(new Color(240,240,240));
	    Color childrenColor = new Color(255,255,0);
	    if (isSelected) {
		childrenColor = childrenColor.darker();
	    }
	    renderer.setBackground(childrenColor);
	}
	
	// se null piu' scuro
	if(emptyValue) {
	    Color background = renderer.getBackground();
	    renderer.setBackground(background.darker());
	}
	
	/* Modifica la visualizzazione */
	this.updateFont(renderer, row, value.toString());
	
	/* Restituisci il renderer modificato */
	return renderer;
    }

    /**
     * Modifica lo stile del componente a seconda del ruolo della tupla
     * (padre, figlio, padre senza figli) 
     */
    private void updateFont(Component c, int row, String textValue) {
    	if(textValue == NULL_TEXT) {
    		Font oldFont = c.getFont();
    	    Font newFont = oldFont.deriveFont(Font.ITALIC);
    	    c.setFont(newFont);
    	}
    	
	if (this.model.isCompositeElement(row)) {
	    Font oldFont = c.getFont();
	    Font newFont = oldFont.deriveFont(Font.BOLD);
	    c.setForeground(Color.BLUE);
	    c.setFont(newFont);
	} else if (this.model.isChildren(row)) {
	    Font oldFont = c.getFont();
	    final float newSizeRatio = 0.9F;
	    float oldSize = oldFont.getSize2D();
	    float newSize = oldSize * newSizeRatio;
	    Font newFont = oldFont.deriveFont(newSize);
	    c.setFont(newFont);
	}
    }
}
