package org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.table;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.columns.groupable.ColumnGroup;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.columns.groupable.GroupableTableColumnModel;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.columns.groupable.GroupableTableHeader;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.rows.multivalue.MultiValueTableModel;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Iterator;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

/**
 * Class <code>MultiSpanCellTable</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class MultiValueGroupableTable extends JTable implements MouseListener, MouseMotionListener {
    private static final long serialVersionUID = 1L;
    private MultiValueTableModel<?> myModel;
    private GroupableTableColumnModel myColumnModel;

    public MultiValueGroupableTable(MultiValueTableModel<?> model) {
	super();
	this.myModel = model;
	this.myColumnModel = new GroupableTableColumnModel();
	super.setColumnModel(this.myColumnModel);
	super.setTableHeader(new GroupableTableHeader(this.myColumnModel));
	//super.setAutoCreateRowSorter(true);
	super.setModel(model); // attenzione all'ordine...
	super.setShowGrid(false);
	super.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	super.setDefaultRenderer(Object.class, new MultiValueCellRenderer(myModel));
	super.setDefaultRenderer(ModelComponent.class, new ModelComponentCellRenderer(myModel));
	super.setIntercellSpacing(new Dimension(0, 0));
	//model.addTableModelListener(this);
	super.addMouseListener(this);
	super.addMouseMotionListener(this);
	super.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	calcColumnWidths(this);
    }


    @SuppressWarnings("unchecked")
    public Iterator<ColumnGroup> columnGroupIterator() {
	return this.myColumnModel.columnGroupIterator();
    }

    /* Espande o comprime le righe al click */
    public void mouseReleased(MouseEvent e) {
	int rowIndex = super.rowAtPoint(e.getPoint());
	this.myModel.collapseOrExpand(rowIndex);
    }

    /* Cambia la forma del mouse in caso di elemento composito */
    public void mouseMoved(MouseEvent e) {
	int rowIndex = super.rowAtPoint(e.getPoint());
	Cursor mouseShape;
	if (rowIndex != -1 && this.myModel.isCompositeElement(rowIndex)) {
	    mouseShape = new Cursor(Cursor.HAND_CURSOR);
	} else {
	    mouseShape = new Cursor(Cursor.DEFAULT_CURSOR);
	}
	setCursor(mouseShape);
    }


    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
    }
    
    public static void calcColumnWidths(JTable table)
    {
        JTableHeader header = table.getTableHeader();

        TableCellRenderer defaultHeaderRenderer = null;

        if (header != null)
            defaultHeaderRenderer = header.getDefaultRenderer();

        TableColumnModel columns = table.getColumnModel();
        TableModel data = table.getModel();

        int margin = columns.getColumnMargin(); // only JDK1.3
        
        final int myMargin = 60;

        int rowCount = data.getRowCount();

        int totalWidth = 0;

        for (int i = columns.getColumnCount() - 1; i >= 0; --i)
        {
            TableColumn column = columns.getColumn(i);
                
            int columnIndex = column.getModelIndex();
                
            int width = -1; 

            TableCellRenderer h = column.getHeaderRenderer();
              
            if (h == null)
                h = defaultHeaderRenderer;
                
            if (h != null) // Not explicitly impossible
            {
                Component c = h.getTableCellRendererComponent
                       (table, column.getHeaderValue(),
                        false, false, -1, i);
                        
                width = c.getPreferredSize().width;
            }
           
            for (int row = rowCount - 1; row >= 0; --row)
            {
                TableCellRenderer r = table.getCellRenderer(row, i);
                     
                Component c = r.getTableCellRendererComponent
                   (table,
                    data.getValueAt(row, columnIndex),
                    false, false, row, i);
            
                    width = Math.max(width, c.getPreferredSize().width);
            }

            if (width >= 0)
                column.setPreferredWidth(width + margin + myMargin); // <1.3: without margin
            else
                ; // ???
                
            totalWidth += column.getPreferredWidth();
        }

}
}
