package org.modelgen.jpf.visualizer.modelgenview.tooltip;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractProperty;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractReference;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.BaseInfo;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.util.HtmlString;

import java.util.Collection;
import java.util.Vector;

/**
 * Class <code>DefaultToolTipFactory</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class DefaultToolTipFactory implements ToolTipFactory {
    public final static String PREVIEW_COLOR_HTML = "Blue";
    
    public HtmlString createToolTip(MultiDirectionAbstractStructure info) {
	String descr = "<i><u>"+ info.getTypeString() + "</u></i>: <b>" + info.getDescription() + "</b>";
	//descr += "<p><i>id <b>"+ info.getSid() +"</b></i></p>";
	BaseInfo<?> parent = info.getParent();
	if (parent != null) {
	    descr += "<p><i>instance of <b>"+ parent.getDescription() +"</b></i></p>";
	}
	
	HtmlString result = new HtmlString(descr);

	return result;
    }
    
    public HtmlString createToolTip(AbstractConstruct info) {
	return this.createToolTip(info, new Vector<AbstractConstruct>());
    }
    
    public HtmlString createToolTip(AbstractConstruct info, Vector<AbstractConstruct> preview) {
	String descr = "";
	if (preview != null && !preview.isEmpty()) {
	    descr += "<P><B>Pointing to:</B></P>";
	}
	descr += "<i><u>"+ info.getTypeString() + "</u></i>: <b>" + info.getDescription() + "</b><br/>";
	//descr += "<p><i>id <b>"+ info.getSid() +"</b></i></p>";
	
	// distinzione tra preview null e vuota:
	// la prima rileva errori nella preview
	// la seconda una preview vuota
	if(preview == null) {
	    descr += "<CENTER><I><font color=red>Preview not available</font></I></CENTER>";
	} else if(!preview.isEmpty()) {
	    descr += "<CENTER><B><FONT COLOR=\""+PREVIEW_COLOR_HTML+"\">"+preview+"</FONT></B></CENTER>";
	}
	descr += "belonging to <b>" + info.getStructure().getDescription() + "</b>";
	BaseInfo<?> parent = info.getParent();
	if (parent != null) {
	    descr += "<p><i>instance of <b>"+ parent.getDescription() +"</b></i></p>";
	}
	
	Collection<AbstractConstruct> weakLinks = info.getWeakLinks();
	AbstractConstruct hardLink = info.getHardLink();
	if (hardLink != null || !weakLinks.isEmpty()) {
	    descr += "<p><i>Linked with: </i></p><BLOCKQUOTE><small>";
	    if (hardLink != null) {
		descr += "<p><b>"+hardLink.getDescription()+"</b></p>";
	    }
	    for (AbstractConstruct weakLink : weakLinks) {
		descr += "<p>"+weakLink.getDescription()+"</p>";
	    }
	    descr += "</small></BLOCKQUOTE></p>";
	}
	
	HtmlString result = new HtmlString(descr);

	return result;
    }
    
    public HtmlString createToolTip(AbstractReference info) {
	String descr = "<i><u>"+ info.getTypeString() + "</u></i>: <b>" + info.getDescription() + "</b><br/>";
	//descr += "<p><i>id <b>"+ info.getSid() +"</b></i></p>";
	descr += "of type <b>"+info.getTypeAsString()+"</b><br/>";
	descr += "belonging to <b>" + info.getOwner().getStructure().getDescription() + "</b>";
	BaseInfo<?> parent = info.getParent();
	if (parent != null) {
	    descr += "<p><i>instance of <b>"+ parent.getDescription() +"</b></i></p>";
	}
	descr += "<p><small><u>  pointing to</u>:</small><BLOCKQUOTE><small>";
	descr += createToolTip(info.getTarget()).getText();
	descr += "</small></BLOCKQUOTE></p>";
	
	HtmlString result = new HtmlString(descr);

	return result;
    }
    
    public HtmlString createToolTip(AbstractProperty info) {
	String descr = "<i><u>"+ info.getTypeString() + "</u></i>: <b>" + info.getDescription() + "</b><br/>";
	//descr += "<p><i>id <b>"+ info.getSid() +"</b></i></p>";
	descr += "belonging to <b>" + info.getAbstractConstruct().getStructure().getDescription() + "</b>";
	BaseInfo<?> parent = info.getParent();
	if (parent != null) {
	    descr += "<p><i>instance of <b>"+ parent.getDescription() +"</b></i></p>";
	}
	
	HtmlString result = new HtmlString(descr);

	return result;
    }
}
