package org.modelgen.jpf.visualizer.pac;

import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.pac.actions.BackAction;
import org.modelgen.jpf.visualizer.pac.actions.NewApplyRuleDebuggerAction;
import org.modelgen.jpf.visualizer.pac.actions.NewTableWindowAction;
import org.modelgen.jpf.visualizer.pac.actions.NewWindowAction;
import org.modelgen.jpf.visualizer.pac.actions.NextAction;
import org.modelgen.jpf.visualizer.pac.actions.OpenPerspectiveDebuggerAction;
import org.modelgen.jpf.visualizer.pac.actions.OpenPerspectiveVisualizerAction;
import org.modelgen.jpf.visualizer.pac.actions.RefreshAction;
import org.modelgen.jpf.visualizer.pac.actions.SaveGraphLayoutAction;
import org.modelgen.jpf.visualizer.pac.top.DebugHandler;
import org.modelgen.jpf.visualizer.pac.top.ErrorHandler;
import org.modelgen.jpf.visualizer.pac.top.ViewsHandler;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyVetoException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

/**
 * Class <code>TopLevelPAC</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class TopLevelPAC {
    public final static String PERSPECTIVE_VIEW = "View"; 
    public final static String PERSPECTIVE_DEBUG = "Debug";
    private static Frame frame = ToolBoxFrame.getIstance();
    private static JInternalFrame parent = null;
    private JPanel view;
    private ViewsHandler viewsHandler;
    private DebugHandler debugHandler;
    private Map<JFrame, JMenuItem> windows = new LinkedHashMap<JFrame, JMenuItem>();
    private JMenu windowMenu;
    private boolean wasSaveActionEnabled;
    private boolean wasNewTableWindowActionEnabled;
    private boolean wasNewWindowActionEnabled;
    private JMenu historyMenu;
    private boolean isViewPerspectiveEnabled;
	private JToggleButton visualizerButton;
	private JToggleButton debuggerButton;
    private static Action actions[] = {
	NewWindowAction.sharedInstance,
	NewTableWindowAction.sharedInstance,
	SaveGraphLayoutAction.sharedInstance,
	null, // separatore
	BackAction.sharedInstance,
	NextAction.sharedInstance,
	RefreshAction.sharedInstance,
	null, // separatore
	NewApplyRuleDebuggerAction.sharedInstance,
    }; // tratta a parte le perspective in createToolBar 

    private static class SingletonHolder {
	private static TopLevelPAC instance = new TopLevelPAC();
    } 

    public static TopLevelPAC getInstance() {
	return SingletonHolder.instance;
    }

    private TopLevelPAC() {
	try {
	    view = new JPanel(new CardLayout());
	    viewsHandler = new ViewsHandler(this);
	    debugHandler = new DebugHandler(this);
	    view.add(viewsHandler.getView(), PERSPECTIVE_VIEW);
	    view.add(debugHandler.getView(), PERSPECTIVE_DEBUG);
	    view.setOpaque(true); //content panes must be opaque
	    if(parent != null) {
		parent.setJMenuBar(createMenuBar());
		JPanel toolBarContainer = new JPanel();
		toolBarContainer.setLayout(new BorderLayout());
		toolBarContainer.add(createToolBar(), BorderLayout.WEST);
		toolBarContainer.add(createPerspectiveToolBar(), BorderLayout.EAST);
		parent.add(toolBarContainer, BorderLayout.PAGE_START);
	    }
	    this.setViewPerspective();
	} catch (Exception e) {
	    ErrorHandler.instance.showError(e);
	    //e.printStackTrace();
	}
    }

    private JToolBar createToolBar() {
	JToolBar toolBar = new JToolBar();
	toolBar.setFloatable(false);
	for(Action action : actions) {
	    if (action != null) {
		toolBar.add(action);
	    } else {
		toolBar.addSeparator();
	    }
	}
	
	return toolBar;
    }
    
    private JToolBar createPerspectiveToolBar() {
	JToolBar toolBar = new JToolBar();
	toolBar.setFloatable(false);
	
	toolBar.addSeparator();
	this.visualizerButton = new JToggleButton(OpenPerspectiveVisualizerAction.sharedInstance);
	
	visualizerButton.setSelected(true);
	this.debuggerButton = new JToggleButton(OpenPerspectiveDebuggerAction.sharedInstance);
	
	toolBar.add(this.visualizerButton);
	toolBar.add(this.debuggerButton);
	
	ButtonGroup buttonGroup = new ButtonGroup();
	buttonGroup.add(this.visualizerButton);
	buttonGroup.add(this.debuggerButton);
	
	return toolBar;
    }

    private JMenuBar createMenuBar() {
	JMenuBar menuBar = new JMenuBar();
	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);
	fileMenu.add(NewWindowAction.sharedInstance);
	fileMenu.add(NewTableWindowAction.sharedInstance);
	fileMenu.addSeparator();
	fileMenu.add(SaveGraphLayoutAction.sharedInstance);
	fileMenu.addSeparator();
	JMenuItem exitItem = new JMenuItem("Exit");
	exitItem.setMnemonic(KeyEvent.VK_E);
	exitItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		try {
		    parent.setClosed(true);
		} catch (PropertyVetoException e1) {
		    // TODO Auto-generated catch block
		    e1.printStackTrace();
		}
	    }
	});
	fileMenu.add(exitItem);
	menuBar.add(fileMenu);
	
	historyMenu = new JMenu("History");
	historyMenu.setMnemonic(KeyEvent.VK_H);
	initializeHistoryMenu();
	
	menuBar.add(historyMenu);
	
	JMenu runMenu = new JMenu("Run");
	runMenu.setMnemonic(KeyEvent.VK_R);
	runMenu.add(NewApplyRuleDebuggerAction.sharedInstance);
	menuBar.add(runMenu);
	
	JMenu viewMenu = new JMenu("View");
	viewMenu.setMnemonic(KeyEvent.VK_V);
	viewMenu.add(RefreshAction.sharedInstance);
	JMenu subViewMenu = new JMenu("Open Perspective");
	subViewMenu.setMnemonic(KeyEvent.VK_O);
	subViewMenu.add(OpenPerspectiveVisualizerAction.sharedInstance);
	subViewMenu.add(OpenPerspectiveDebuggerAction.sharedInstance);
	viewMenu.addSeparator();
	viewMenu.add(subViewMenu);
	menuBar.add(viewMenu);
	
	windowMenu = new JMenu("Window");
	windowMenu.setMnemonic(KeyEvent.VK_W);
	menuBar.add(windowMenu);
	
	return menuBar;
    }

    private void initializeHistoryMenu() {
	historyMenu.add(BackAction.sharedInstance);
	historyMenu.add(NextAction.sharedInstance);
    }

    /**
     * @return la vista creata. null in caso di problemi
     */
    public JPanel getView() {
	return view;
    }

    public void showException(Exception ex) {
	ErrorHandler.instance.showError(ex);
    }

    public void addWindow(final JFrame newWindow) {
	JMenuItem windowItem = new JMenuItem(newWindow.getTitle());
	Image winIcon = newWindow.getIconImage();
	if (winIcon != null) {
	    ImageIcon icon = new ImageIcon(newWindow.getIconImage());
	    windowItem.setIcon(icon);
	}
	windowItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		newWindow.toFront();
	    }
	});
	windowMenu.add(windowItem);
	this.windows.put(newWindow, windowItem);
    }

    public void removeWindow(JFrame newWindow) {
	JMenuItem windowItem = this.windows.get(newWindow);
	if (windowItem != null) {
	    this.windowMenu.remove(windowItem);
	    newWindow.dispose();
	}
    }

    public static void setFrame(Frame myFrame) {
	frame = myFrame;
    }

    public static Frame getFrame() {
	return frame;
    }

    public static void setParent(JInternalFrame parent) {
	TopLevelPAC.parent = parent;
    }

    public static JInternalFrame getParent() {
	return parent;
    }

    public void showWarning(Exception e) {
	ErrorHandler.instance.showWarning(e);
    }
    
    public void showMessage(String message) {
	ErrorHandler.instance.showMessage(message);
    }

    public void newSession(MultiDirectionAbstractStructure sourceScheme, Collection<MultiDirectionAbstractStructure> targetSchemes) {
	try {
	    debugHandler.newSession(sourceScheme, targetSchemes);
	    this.setDebugPerspective();
	} catch (Exception e) {
	    this.showException(e);
	}
	
    }

    public void setDebugPerspective() {
    	this.debuggerButton.setSelected(true);
	if (this.isViewPerspectiveEnabled == false) return;
	
	CardLayout cl = (CardLayout)(view.getLayout());
	cl.show(view, PERSPECTIVE_DEBUG);
	
	// disabilita azioni non necessarie nella prospettiva debug
	this.wasSaveActionEnabled = SaveGraphLayoutAction.sharedInstance.isEnabled();
	this.wasNewTableWindowActionEnabled = NewTableWindowAction.sharedInstance.isEnabled();
	this.wasNewWindowActionEnabled = NewWindowAction.sharedInstance.isEnabled();
	SaveGraphLayoutAction.sharedInstance.setEnabled(false);
	NewTableWindowAction.sharedInstance.setEnabled(false);
	NewWindowAction.sharedInstance.setEnabled(false);
	
	// carica configurazione specifica del debug
	debugHandler.open();
	
	this.isViewPerspectiveEnabled = false;
    }

    public void setViewPerspective() {
    	this.visualizerButton.setSelected(true);
	if (this.isViewPerspectiveEnabled == true) return;
	
	CardLayout cl = (CardLayout)(view.getLayout());
	cl.show(view, PERSPECTIVE_VIEW);
	
	// riabilita azioni precedentemente disabilitate da debug
	SaveGraphLayoutAction.sharedInstance.setEnabled(this.wasSaveActionEnabled);
	NewTableWindowAction.sharedInstance.setEnabled(this.wasNewTableWindowActionEnabled);
	NewWindowAction.sharedInstance.setEnabled(this.wasNewWindowActionEnabled);
	
	// carica configurazione specifica del view
	viewsHandler.open();
	
	this.isViewPerspectiveEnabled = true;
    }

    public void historyActionsUpdated(Collection<Action> historyActions) {
	// resetta history menu
	historyMenu.removeAll();
	initializeHistoryMenu();
	
	if ( ! historyActions.isEmpty() ) {
	    historyMenu.addSeparator();
	}
	
	for (Action historyAction : historyActions) {
	    historyMenu.add(historyAction);
	}
    }
    
}
