package org.modelgen.jpf.visualizer.pac.actions;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.pac.top.views.view.ModelGraphViewHandler;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

/**
 * Class <code>NewWindowAction</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class NewTableWindowAction extends AbstractAction {
    private static final long serialVersionUID = 1L;

    public static final String ACTION_NAME = "New Table Window";
    public static final int ACTION_MNEMONIC = KeyEvent.VK_T;
    
    private ModelComponent receiver;

    private ModelGraphViewHandler viewCreator;

    private ImageIcon icon;

    public static NewTableWindowAction sharedInstance = new NewTableWindowAction();

    private NewTableWindowAction() {
	this.setEnabled(false);
	putValue(Action.NAME,ACTION_NAME);
	putValue(Action.MNEMONIC_KEY,ACTION_MNEMONIC);
	ImageIcon icon = ToolConfigurationManager.sharedInstance.getIcon("new.png");
	if (icon != null) putValue(Action.SMALL_ICON, icon);
	putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("F8"));
    }

    public void setReceiver(ModelComponent receiver) {
	this.receiver = receiver;
	putValue(Action.SHORT_DESCRIPTION,"Open "+receiver.toString()+"'s Data in a New Window");
    }

    public void setViewCreator(ModelGraphViewHandler viewCreator) {
	this.viewCreator = viewCreator;
    }

    public void actionPerformed(ActionEvent event) {
	try {
	    final JFrame newWindow = new JFrame(receiver.toString());
	    newWindow.setMinimumSize(new Dimension(400,400));
	    final ModelComponent clone = new ModelComponent(receiver.getInfo());
	    newWindow.addWindowListener(new WindowAdapter() {
		@Override
		public void windowClosing(WindowEvent e) {
		    viewCreator.removeWindow(clone);
		}
	    });
	    if (icon != null) {
		newWindow.setIconImage(icon.getImage());
	    } // else non carica icona
	    newWindow.pack();
	    newWindow.setExtendedState(Frame.MAXIMIZED_BOTH);
	    newWindow.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    viewCreator.newWindow(receiver, newWindow);
	} catch (Exception e) {
	    viewCreator.showException(e);
	}
    }
    
    @Override
    public void setEnabled(boolean newValue) {
	super.setEnabled(newValue);
	if (newValue == false) {
	    putValue(Action.SHORT_DESCRIPTION,null);
	} else {
	    putValue(Action.SHORT_DESCRIPTION,"Open "+receiver.toString()+"'s Data in a New Window");
	}
    }

}
