package org.modelgen.jpf.visualizer.pac.actions;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.pac.top.ViewsHandler;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

/**
 * Class <code>NewWindowAction</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class NewWindowAction extends AbstractAction {
    private static final long serialVersionUID = 1L;
    private static Action actions[] = {
	SaveGraphLayoutAction.sharedInstance,
	null, // separatore
	NewTableWindowAction.sharedInstance
    };

    public static final String ACTION_NAME = "New Window";
    public static final int ACTION_MNEMONIC = KeyEvent.VK_N;
    private ModelComponent receiver;
    private ImageIcon windowIcon = null;

    private ViewsHandler viewCreator;
    private JFrame windowCreated;

    public static NewWindowAction sharedInstance = new NewWindowAction();

    private NewWindowAction() {
	this.setEnabled(false);
	putValue(Action.NAME,ACTION_NAME);
	putValue(Action.MNEMONIC_KEY,ACTION_MNEMONIC);
	ImageIcon icon = ToolConfigurationManager.sharedInstance.getIcon("window-new.png");
	if (icon != null) putValue(Action.SMALL_ICON,icon);
	putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("F7"));
    }

    public void setReceiver(ModelComponent receiver) {
	this.receiver = receiver;
	putValue(Action.SHORT_DESCRIPTION,"Open "+receiver.toString()+" in a New Window");
    }

    public void setViewCreator(ViewsHandler viewCreator) {
	this.viewCreator = viewCreator;
    }

    public void setWindowIcon(ImageIcon windowIcon) {
	this.windowIcon = windowIcon;
    }

    public void actionPerformed(ActionEvent event) {
	try {
	    final JFrame newWindow = new JFrame(receiver.toString());
	    newWindow.setMinimumSize(new Dimension(400,400));
	    final ModelComponent clone = new ModelComponent(receiver.getInfo());
	    newWindow.addWindowListener(new WindowAdapter() {
		@Override
		public void windowClosing(WindowEvent e) {
		    if (viewCreator != null) {
			viewCreator.removeWindow(clone);
		    }
		}

		@Override
		public void windowActivated(WindowEvent e) {
		    try {
			for (java.awt.Component child : newWindow.getContentPane().getComponents()) {
			    child.requestFocus();
			}
		    } catch (Exception ex) {
			if (viewCreator != null) { 
			    viewCreator.showException(ex);
			}
		    }
		}
	    });
	    if (windowIcon != null) {
		newWindow.setIconImage(windowIcon.getImage());
	    } // else non carica icona
	    
	    newWindow.pack();
	    newWindow.setExtendedState(Frame.MAXIMIZED_BOTH);
	    newWindow.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    newWindow.setJMenuBar(createMenuBar(newWindow));
	    newWindow.add(createToolBar(), BorderLayout.PAGE_START);
	    if (viewCreator != null) {
		viewCreator.newWindow(receiver, newWindow);
	    }
	    this.windowCreated = newWindow;
	} catch (Exception e) {
	    if (viewCreator != null) {
		viewCreator.showException(e);
	    }
	}
    }
    
    public JFrame execute() {
	this.actionPerformed(null);
	return this.windowCreated;
    }
    
    @Override
    public void setEnabled(boolean newValue) {
	super.setEnabled(newValue);
	if (newValue == false) {
	    putValue(Action.SHORT_DESCRIPTION,null);
	} else {
	    putValue(Action.SHORT_DESCRIPTION,"Open "+receiver.toString()+" in a New Window");
	}
    }
    
    
    private JToolBar createToolBar() {
	JToolBar toolBar = new JToolBar();
	for(Action action : actions) {
	    if (action != null) {
		toolBar.add(action);
	    } else {
		toolBar.addSeparator();
	    }
	}
	return toolBar;
    }

    private JMenuBar createMenuBar(final JFrame parent) {
	JMenuBar menuBar = new JMenuBar();
	JMenu fileMenu = new JMenu("File");
	fileMenu.add(SaveGraphLayoutAction.sharedInstance);
	fileMenu.addSeparator();
	JMenuItem exitItem = new JMenuItem("Exit");
	exitItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		parent.dispose();
	    }
	});
	fileMenu.add(exitItem);
	menuBar.add(fileMenu);
	return menuBar;
    }

}
