package org.modelgen.jpf.visualizer.pac.actions;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.pac.top.views.view.ViewGraphJGraph;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

/**
 * Class <code>NewWindowAction</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class SaveGraphLayoutAction extends AbstractAction {
    private static final long serialVersionUID = 1L;

    public static final String ACTION_NAME = "Save Graph Layout";
    public static final int ACTION_MNEMONIC = KeyEvent.VK_S;
    private ViewGraphJGraph receiver;

    public static SaveGraphLayoutAction sharedInstance = new SaveGraphLayoutAction();

    private SaveGraphLayoutAction() {
	this.setEnabled(false);
	putValue(Action.NAME,ACTION_NAME);
	putValue(Action.MNEMONIC_KEY,ACTION_MNEMONIC);
	ImageIcon icon = ToolConfigurationManager.sharedInstance.getIcon("save.png");
	if (icon != null) putValue(Action.SMALL_ICON, icon);
	putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke('S', java.awt.event.InputEvent.CTRL_DOWN_MASK));
    }

    public void setReceiver(ViewGraphJGraph receiver) {
	this.receiver = receiver;
	putValue(Action.SHORT_DESCRIPTION,"Save "+receiver.toString()+"'s graph current layout");
    }

    public void actionPerformed(ActionEvent event) {
	try {
	    receiver.saveLayout();
	    receiver.showMessage(receiver.toString()+ "'s layout has been saved");
	} catch (Exception e) {
	    receiver.showException(e);
	}
    }

    @Override
    public void setEnabled(boolean newValue) {
	this.setEnabled(newValue, null);
    }

    public void setEnabled(boolean newValue, String shortDescription) {
	super.setEnabled(newValue);
	if (newValue == false) {
	    putValue(Action.SHORT_DESCRIPTION,shortDescription);
	} else {
	    if (receiver != null)
		putValue(Action.SHORT_DESCRIPTION,"Save "+receiver.toString()+"'s graph current layout");
	}
    }
    
    

}
