package org.modelgen.jpf.visualizer.pac.actions.wizard;

import org.modelgen.jpf.visualizer.debugger.RuleDebugger;
import org.modelgen.jpf.visualizer.debugger.RuleDebuggerProgressListener;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.pac.TopLevelPAC;
import org.modelgen.jpf.visualizer.pac.actions.NewApplyRuleDebuggerAction;
import org.modelgen.jpf.visualizer.pac.actions.RefreshAction;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.Collection;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ProgressMonitor;
import javax.swing.SwingWorker;
/**
 * Class <code>NewWindowAction</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
class FinishAction extends AbstractAction implements PropertyChangeListener {
    private static final long serialVersionUID = 1L;

    public static final String ACTION_NAME = "Finish";
    public static final String ACTION_DESCRIPTION = "Create a New Apply Rule Debugger Session...";
    private NewApplyRuleDebuggerWizard invoker;

    private ProgressMonitor progressMonitor;
    private int maximumValue;

    private RuleDebugger debugger;

    private boolean gotException = false; 

    public FinishAction(NewApplyRuleDebuggerWizard invoker) {
	this.invoker = invoker;
	putValue(Action.NAME,ACTION_NAME);
	putValue(Action.SHORT_DESCRIPTION,ACTION_DESCRIPTION);
    }

    public void actionPerformed(ActionEvent event) {
	// ottieni le info necessarie
	MultiDirectionAbstractStructure schemeToTranslate = this.invoker.getSchemeToTranslate();
	String ruleToApply = this.invoker.getRuleToApply();
	File macroRuleDir = this.invoker.getMacroRuleDir();

	// azione sul receiver
	try {
	    // esegui
	    debugger = new RuleDebugger(schemeToTranslate, ruleToApply, macroRuleDir);
	    maximumValue = debugger.getStepsNumber();
	    progressMonitor = new ProgressMonitor(TopLevelPAC.getFrame(),
		    "Creating a new session",
		    "", 0, debugger.getStepsNumber());
	    progressMonitor.setMillisToDecideToPopup(0);
	    progressMonitor.setMillisToPopup(0);
	    progressMonitor.setNote("Initializing");
	    progressMonitor.setProgress(0);
	    this.gotException = false;
	    Task debuggerTask = new Task(debugger);
	    debuggerTask.addPropertyChangeListener(this);
	    debuggerTask.execute();
	} catch (Exception e) { // possibile eccezione nel creare il RuleDebugger
	    this.showException(e);
	} // finally  ripristina dallo stato di attesa in propertyChange
    }

    @SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent evt) {
	if ("step" == evt.getPropertyName() ) {
	    Pair<Integer,String> step = (Pair<Integer,String>) evt.getNewValue();
	    int progress = step.getFirst();
	    String message = step.getSecond();
	    progressMonitor.setProgress(progress);
	    progressMonitor.setNote(message);
	} else if ("done" == evt.getPropertyName()) {
	    // fine del progress, ma l'exception potrebbe gia' essere
	    // stata lanciata
	    if (!gotException) {
		MultiDirectionAbstractStructure sourceScheme = this.invoker.getSchemeToTranslate();
		try {
		    Collection<MultiDirectionAbstractStructure> targetSchemes =  (Collection<MultiDirectionAbstractStructure>) evt.getNewValue();
		    if (targetSchemes == null) {
			throw new Exception("Exception in Apply Rule Engine");
		    }
		    TopLevelPAC.getInstance().newSession(sourceScheme, targetSchemes);
		} catch (Exception e) {
		    this.showException(e);
		}
	    }
	} else if ("exception" == evt.getPropertyName()) {
	    this.gotException = true;
	    Exception e = (Exception) evt.getNewValue();
	    this.showException(e);
	}
    }
    
    public void showException(Exception e) {
	// chiudi progress
	progressMonitor.setProgress(debugger.getStepsNumber());
	// mostra eccezione
	invoker.showException(e);
    }

    public class Task extends SwingWorker<Void,Void> implements RuleDebuggerProgressListener  {
	private RuleDebugger debugger;
	private int value;
	private Collection<MultiDirectionAbstractStructure> schemesCreated;

	public Task(RuleDebugger debugger) {
	    this.debugger = debugger;
	    value = 0;
	}

	@Override
	protected Void doInBackground()  {
	    try {
		// metti in stato di attesa
		NewApplyRuleDebuggerAction.sharedInstance.setEnabled(false);
		RefreshAction.sharedInstance.setEnabled(false);
		TopLevelPAC.getParent().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		setProgress(0);
		debugger.addListener(this);
		this.schemesCreated = debugger.startDebug();
	    } catch (Exception e) {
		this.firePropertyChange("exception", null, e);
		e.printStackTrace();
	    } finally {
		// Ripristina dallo stato di attesa
		TopLevelPAC.getParent().setCursor(null); //turn off the wait cursor
		RefreshAction.sharedInstance.setEnabled(true);
		NewApplyRuleDebuggerAction.sharedInstance.setEnabled(true);
	    }
	    return null;
	}

	@Override
	protected void done() {
	    Toolkit.getDefaultToolkit().beep();
	    this.firePropertyChange("done", null, this.schemesCreated);
	}

	public void stepStarting(String message) {
	    value++;
	    double percentageDouble = (double)value / maximumValue;
	    percentageDouble *= 100;
	    Pair<Integer,String> valueToNotify = new Pair<Integer,String>(value, message);
	    this.firePropertyChange("step", null, valueToNotify);
	}


    }

}
