package org.modelgen.jpf.visualizer.pac.actions.wizard;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.pac.TopLevelPAC;
import org.modelgen.jpf.visualizer.pac.actions.wizard.RulesComboBox.RulesComboBoxMemento;
import org.modelgen.jpf.visualizer.pac.actions.wizard.SchemesJTree.SchemesJTreeMemento;
import org.modelgen.jpf.visualizer.pac.top.ErrorHandler;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.border.EmptyBorder;

/**
 * Class <code>NewApplyRuleDebuggerWizard</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class NewApplyRuleDebuggerWizard extends JDialog implements ActionListener {
    private static final long serialVersionUID = 1L;

    public enum ExitStatus { FINISH, CANCEL }
    private JPanel panel;
    private JButton cancelButton;
    private JButton finishButton;
    private RulesComboBox rulesComboBox;
    private SchemesJTree schemesTree;
    private static RulesComboBoxMemento rulesComboBoxState;
    private static SchemesJTreeMemento schemesTreeState;
    private boolean schemesTreeValid = false;
    private boolean rulesComboBoxValid = false;

    private static class SingletonHolder {
	private static Frame owner;
	private static NewApplyRuleDebuggerWizard instance = new NewApplyRuleDebuggerWizard(owner);
	private static void newInstance() {
	    instance = new NewApplyRuleDebuggerWizard(owner);
	}
    }


    public static void newInstance() {
	SingletonHolder.newInstance();
    }

    public static NewApplyRuleDebuggerWizard getInstance(Frame owner) {
	SingletonHolder.owner = owner;
	NewApplyRuleDebuggerWizard instance = SingletonHolder.instance;

	// salva lo stato per un successivo undo in caso di cancel
	if (instance.rulesComboBox != null) {
		rulesComboBoxState = instance.rulesComboBox.saveState();
		}
	if (instance.schemesTree != null) {
		schemesTreeState = instance.schemesTree.saveState();
	}

	// forza il reload
	try {
	    // instance.reloadTree();
	    // forza la riselezione degli elementi salvati
	    //instance.schemesTree.undoState(schemesTreeState);
	} catch (Exception e) {
	    TopLevelPAC.getInstance().showException(e);
	}

	return instance;
    }

    private NewApplyRuleDebuggerWizard(Frame owner) {
	super(owner, "New Debug Session");
	initComponents();
    }

    public void doWizard() {
	this.setVisible(true);
    }

    @Override
    public void setVisible(boolean visible) {
	if (visible == true) {
	    // forza il centramento  a ogni nuova visualizzazione
	    center(this);
	}
	super.setVisible(visible);
    }

    private void initComponents() {
	JPanel buttonPanel = new JPanel();
	Box buttonBox = new Box(BoxLayout.X_AXIS);

	finishButton = new JButton("Finish");
	finishButton.setMargin(new Insets(2, 30, 2, 30));
	Action finishAction = new FinishAction(this);
	finishButton.setAction(finishAction);
	finishButton.setEnabled(false);
	finishButton.addActionListener(this);

	cancelButton = new JButton("Cancel");
	cancelButton.setMargin(new Insets(2, 30, 2, 30));
	cancelButton.addActionListener(this);

	buttonPanel.setLayout(new BorderLayout());
	buttonPanel.add(new JSeparator(), BorderLayout.NORTH);

	buttonBox.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));
	buttonBox.add(finishButton);
	buttonBox.add(Box.createHorizontalStrut(30));
	buttonBox.add(cancelButton);
	buttonPanel.add(buttonBox, BorderLayout.EAST);

	this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
	panel = initMainPanel();
	this.getContentPane().add(panel, BorderLayout.CENTER);
	this.pack();
    }

    public static void center(JDialog dialog) {
	GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
	Point center = ge.getCenterPoint();
	Rectangle bounds = ge.getMaximumWindowBounds();
	int w = Math.max(bounds.width/2, Math.min(dialog.getWidth(), bounds.width));
	int h = Math.max(bounds.height/2, Math.min(dialog.getHeight(), bounds.height));
	int x = center.x - w/2, y = center.y - h/2;
	dialog.setBounds(x, y, w, h);
	//if (w == bounds.width && h == bounds.height) dialog.setExtendedState(Frame.MAXIMIZED_BOTH);
	dialog.validate();
    }

    private JPanel initMainPanel() {
	JPanel panel = new JPanel();
	panel.setLayout(new BorderLayout());
	panel.setBorder(new EmptyBorder(new Insets(0, 0, 0, 0)));

	JPanel titlePanel = new JPanel();
	titlePanel.setLayout(new BorderLayout());
	titlePanel.setBackground(Color.white);

	JLabel textLabel = new JLabel();
	textLabel.setBackground(Color.white);
	textLabel.setFont(new Font("MS Sans Serif", Font.BOLD, 14));
	textLabel.setText("New Debug Session");
	textLabel.setBorder(new EmptyBorder(new Insets(10, 8, 8, 10)));
	textLabel.setOpaque(true);

	titlePanel.add(textLabel, BorderLayout.NORTH);
	JLabel welcomeLabel = new JLabel("Please enter the following info:");
	welcomeLabel.setBorder(new EmptyBorder(new Insets(0, 10, 10, 10)));
	titlePanel.add(welcomeLabel, BorderLayout.CENTER);
	titlePanel.add(new JSeparator(), BorderLayout.SOUTH);

	panel.add(titlePanel, BorderLayout.NORTH);

	JPanel contentPanel;
	try {
	    contentPanel = getContentPanel();
	} catch (Exception e) {
	    contentPanel = new JPanel();
	    contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.PAGE_AXIS));
	    JLabel errorTitle = new JLabel(e.getClass().getSimpleName());
	    errorTitle.setForeground(Color.red);
	    contentPanel.add(errorTitle);
	    contentPanel.add(new JLabel());
	    contentPanel.add(new JLabel(e.getMessage()));
	    //e.printStackTrace();
	}
	contentPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
	panel.add(contentPanel, BorderLayout.CENTER);

	JLabel endLabel = new JLabel("Press Finish to complete the operation");
	endLabel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
	panel.add(endLabel, BorderLayout.SOUTH);

	return panel;
    }

    private JPanel getContentPanel() throws ClassNotFoundException, Exception {
	JPanel result = new JPanel();

	result.setLayout(new BorderLayout());

	// rule select
	JPanel rulePanel = new JPanel();
	rulePanel.setLayout(new GridLayout(0, 1));
	rulePanel.setBorder(BorderFactory.createTitledBorder("Rule"));
	this.rulesComboBox = new RulesComboBox(this);

	rulePanel.add(new JLabel("Choose the rule to debug: "));
	rulePanel.add(rulesComboBox);


	result.add(rulePanel, BorderLayout.NORTH);

	// tree
	JPanel treePanel = new JPanel();
	treePanel.setLayout(new BorderLayout());
	treePanel.setBorder(BorderFactory.createTitledBorder("Scheme"));

	treePanel.add(new JLabel("Choose the working scheme:"), BorderLayout.NORTH);
	this.schemesTree = new SchemesJTree(this);
	treePanel.add(schemesTree.getView(), BorderLayout.CENTER);

	result.add(treePanel, BorderLayout.CENTER);

	return result;
    }

    public void actionPerformed(ActionEvent event) {
	// in caso di cancel ritorna alle vecchie selezioni
	if (this.cancelButton == event.getSource() ) {
	    if (rulesComboBox != null) rulesComboBox.undoState(rulesComboBoxState);
	    if (schemesTree != null) schemesTree.undoState(schemesTreeState);
	}

	// sia in caso di cancel che di finish, chiudi il wizard
	super.setVisible(false);
    }

    public String getRuleToApply() {
	return this.rulesComboBox.getRuleToApply();
    }

    public MultiDirectionAbstractStructure getSchemeToTranslate() {
	return this.schemesTree.getSchemeToTranslate();
    }

    public File getMacroRuleDir() {
        return this.rulesComboBox.getMacroRuleDir();
    }

    public void showException(Exception e) {
	ErrorHandler.instance.showError(e);
    }

    public void isValid(Object wizardComponent, boolean valid) {
	if (wizardComponent == schemesTree) {
	    schemesTreeValid = valid;
	} else {
	    rulesComboBoxValid = valid;
	}

	boolean finishButtonEnabled = rulesComboBoxValid && schemesTreeValid;
	finishButton.setEnabled(finishButtonEnabled);
    }

    public void doConfirm() {
	// se abilitato, clicca
	finishButton.doClick();
    }

}
