package org.modelgen.jpf.visualizer.pac.actions.wizard;

import org.modelgen.commons.utils.ModelgenConfigurationManager;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;

/**
 * Class <code>RulesComboBox</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class RulesComboBox extends JComboBox {
    private static final long serialVersionUID = 1L;
    private File macroRuleDir;

    public RulesComboBox(NewApplyRuleDebuggerWizard mediator) {
	super();
	boolean valid = false;
	//This was from the request, now we need to load the available rules.
	String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
	String macrorulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
	macroRuleDir = new File(rootnode + macrorulePath);

	Vector<String> rules;
    rules = getRulesAvailable();
    if ( rules != null && ! rules.isEmpty() ) {
	valid = true;
	ComboBoxModel model = new DefaultComboBoxModel(rules);
	super.setModel(model);
    }
	mediator.isValid(this, valid);
    }

    /**
     * Return a collection of available rules
     * @return Collection
     */
    @SuppressWarnings("unchecked")
    private Vector<String> getRulesAvailable() {
	Vector<String> rules = new Vector<String>();
	if (macroRuleDir.isDirectory()) {
	    String sa_rules[] = macroRuleDir.list(new FilenameFilter() {
		public boolean accept(File dir, String name) {
		    if (name == null) return false;
		    if (name.endsWith(".txt")) return true;
		    return false;
		}
	    });
	    // All the .txt files are now in the array, but it should be easier to
	    // deal with a LinkedList.
	    if (sa_rules != null)
		for (String rule : sa_rules) {
		    rules.add(rule);
		}
	} else {
	    rules = null;
	}
	return rules;
    }

    public String getRuleToApply() {
	return (String) getSelectedItem();
    }

    public File getMacroRuleDir() {
        return macroRuleDir;
    }

    public RulesComboBoxMemento saveState() {
	return new RulesComboBoxMemento(this.getSelectedIndex());
    }

    public void undoState(RulesComboBoxMemento state) {
	this.setSelectedIndex(state.selectedIndex);
    }

    // per consentire l'undo
    public class RulesComboBoxMemento {
	private int selectedIndex;
	private RulesComboBoxMemento(int selectedIndex) {
	    this.selectedIndex = selectedIndex;
	}
    }
}
