package org.modelgen.jpf.visualizer.pac.actions.wizard;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.AbstractModelGenBuilderFacade;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.tooltip.ToolTipManager;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseEvent;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

/**
 * Class <code>SchemeTree</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class SchemesJTree {
    private DataTree dataTree;
    private JScrollPane view;
    private MultiDirectionAbstractStructure structureSelected;
    private NewApplyRuleDebuggerWizard mediator;

    private static ModelComponent wrap(MultiDirectionAbstractStructure structure) {
	ModelComponent result = new ModelComponent(structure);
	result.setTooltip(ToolTipManager.instance.createToolTip(structure));
	return result;
    }

    public SchemesJTree(NewApplyRuleDebuggerWizard mediator) throws ClassNotFoundException, Exception {
	this.mediator = mediator;
	view = new JScrollPane();
	TreeModel model;
	model = createTreeModel();
	dataTree = new DataTree(model);
	view.setViewportView(dataTree);
	dataTree.repaint();
	dataTree.revalidate();
    }

    private TreeModel createTreeModel() throws ClassNotFoundException, Exception {
	MultiDirectionAbstractStructure rootTree = AbstractModelGenBuilderFacade.getInstance().getSuperModel();

	DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(wrap(rootTree));
	/* Inserisco modelli e relativi schemi */
	for (MultiDirectionAbstractStructure smScheme : rootTree.getChildrens()) {
	    // non inserire i modelli del supermodello
	    if (MultiDirectionAbstractStructure.isScheme(smScheme)) {	
		DefaultMutableTreeNode modelNode = new DefaultMutableTreeNode(wrap(smScheme));
		rootNode.add(modelNode);
	    }
	}

	TreeModel myTreeModel = new DefaultTreeModel(rootNode);
	return myTreeModel;
    }

    public class DataTree extends JTree implements TreeSelectionListener {
	private static final long serialVersionUID = 1L;

	public DataTree(TreeModel model) {
	    super();
	    DefaultTreeCellRenderer renderer = 
		new TreeCellLoadedDisplayRenderer();
	    this.setCellRenderer(renderer);
	    this.addTreeSelectionListener(this);
	    this.setModel(model);
	    this.setToolTipText("");
	}

	public void valueChanged(TreeSelectionEvent e) {
	    DefaultMutableTreeNode node = (DefaultMutableTreeNode)this.getLastSelectedPathComponent();
	    boolean valid;
	    if (node == null) {
		valid = false;
	    } else {
		ModelComponent structureWrapped = (ModelComponent) node.getUserObject();
		SchemesJTree.this.structureSelected = (MultiDirectionAbstractStructure) structureWrapped.getInfo();
		valid = MultiDirectionAbstractStructure.isScheme(structureSelected);
	    }
	    SchemesJTree.this.mediator.isValid(SchemesJTree.this, valid);
	}

	@Override
	public String getToolTipText(MouseEvent evt) {
	    if (getRowForLocation(evt.getX(), evt.getY()) == -1) return null;    
	    TreePath curPath = getPathForLocation(evt.getX(), evt.getY());
	    DefaultMutableTreeNode node = (DefaultMutableTreeNode) curPath.getLastPathComponent();
	    ModelComponent toolTipableNode = (ModelComponent) node.getUserObject();
	    return toolTipableNode.getTooltipText();
	}

	public class TreeCellLoadedDisplayRenderer extends DefaultTreeCellRenderer {
	    private static final long serialVersionUID = 1L;

	    public TreeCellLoadedDisplayRenderer() {
		super();
	    }

	    @Override
	    public Component getTreeCellRendererComponent(JTree tree, Object valueObj, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
		JLabel cellRenderer = (JLabel) super.getTreeCellRendererComponent(tree, valueObj, sel, expanded, leaf,
			row, hasFocus);
		DefaultMutableTreeNode valueNode = (DefaultMutableTreeNode) valueObj;

		/* instanceof necessario: in Windows non serve, in Linux serve.
		 * E' proprio vero: 'write once, debug everywhere' */
		if (valueNode.getUserObject() instanceof ModelComponent) {
		    ModelComponent valueWrapped = (ModelComponent) valueNode.getUserObject();
		    MultiDirectionAbstractStructure value = (MultiDirectionAbstractStructure) valueWrapped.getInfo();
		    ImageIcon icon = loadIcon(value);
		    if (icon != null) this.setIcon(icon);
		}
		return cellRenderer;
	    }

	    @Override
	    public Dimension getPreferredSize() {
		Dimension size = super.getPreferredSize();
		size.width *= 1.15;
		return size;
	    }
	}
    }

    public JScrollPane getView() {
	return view;
    }

    private ImageIcon loadIcon(MultiDirectionAbstractStructure structure) {
	ImageIcon icon = null;
	if (MultiDirectionAbstractStructure.isSuperModel(structure) || MultiDirectionAbstractStructure.isModel(structure)) {
	     icon = ToolConfigurationManager.sharedInstance.getIcon("toolFrame1.png");
	} else {
	    icon = ToolConfigurationManager.sharedInstance.getIcon("toolFrame2.png");
	}
	return icon;

    }

    public MultiDirectionAbstractStructure getSchemeToTranslate() {
	return this.structureSelected;
    }
    
    
    // per consentire l'undo
    public SchemesJTreeMemento saveState() {
	return new SchemesJTreeMemento(dataTree.getSelectionPath());
    }
    
    public void undoState(SchemesJTreeMemento state) {
	dataTree.setSelectionPath(state.selectedNode);
	dataTree.valueChanged(null);
    }
    
    public class SchemesJTreeMemento {
	private TreePath selectedNode;
	private SchemesJTreeMemento(TreePath selectedNode) {
	    this.selectedNode = selectedNode;
	}
    }

    public void reload() throws Exception {
	// salva l'elemento selezionato
	SchemesJTreeMemento selectionState = this.saveState();
	TreeModel treeModel = this.createTreeModel();
	dataTree.setModel(treeModel);
	
	// ripristina l'elemento selezionato
	this.undoState(selectionState);
    }

}
