package org.modelgen.jpf.visualizer.pac.top;

import org.modelgen.jpf.visualizer.pac.TopLevelPAC;
import org.modelgen.jpf.visualizer.pac.actions.BackAction;
import org.modelgen.jpf.visualizer.pac.actions.NextAction;
import org.modelgen.jpf.visualizer.pac.actions.RefreshAction;

import java.util.Collection;
import java.util.LinkedList;

import javax.swing.Action;

/**
 * Class <code>Perspective</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public abstract class AbstractPerspective {
    protected TopLevelPAC mediator;
    protected Collection<Action> historyActions = new LinkedList<Action>();

    public AbstractPerspective(TopLevelPAC mediator) {
	this.mediator = mediator;
    }
    
    /**
     * Azioni da eseguire al momento dell'apertura della
     * prospettiva
     */
    public void open() {
	RefreshAction.sharedInstance.setReceiver(this);
	BackAction.sharedInstance.setReceiver(this);
	NextAction.sharedInstance.setReceiver(this);
	this.updateHistoryActions();
    }

    public TopLevelPAC getMediator() {
        return mediator;
    }

    public void showException(Exception e) {
	mediator.showException(e);
    }
    
    public void showWarning(Exception e) {
	mediator.showWarning(e);
    }
    
    public void showMessage(String message) {
	mediator.showMessage(message);
    }
    
    /**
     * Refresh della prospettiva
     * @throws Exception in caso di errori
     */
    public abstract void refresh() throws Exception;
    
    public abstract void next();
    
    public abstract void back();
    
    public void updateHistoryActions() {
	mediator.historyActionsUpdated(this.historyActions);
    }
    
    public abstract String getRefreshDescription();

    public abstract String getBackActionDescription();

    public abstract String getNextActionDescription();
}
