package org.modelgen.jpf.visualizer.pac.top;

import org.modelgen.jpf.visualizer.facade.VisualizerFacade;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.tooltip.ToolTipManager;
import org.modelgen.jpf.visualizer.pac.TopLevelPAC;
import org.modelgen.jpf.visualizer.pac.actions.BackAction;
import org.modelgen.jpf.visualizer.pac.actions.NextAction;
import org.modelgen.jpf.visualizer.pac.actions.wizard.NewApplyRuleDebuggerWizard;
import org.modelgen.jpf.visualizer.pac.top.debug.LinksShowClickEvent;
import org.modelgen.jpf.visualizer.pac.top.debug.SourceGraph;
import org.modelgen.jpf.visualizer.pac.top.debug.TargetGraph;
import org.modelgen.jpf.visualizer.pac.top.views.view.ModelDoubleClickListener;
import org.modelgen.jpf.visualizer.pac.top.views.view.ViewGraphJGraph;
import org.modelgen.jpf.visualizer.util.HtmlString;
import org.modelgen.jpf.visualizer.util.MoreUnitIncrementScrollPane;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
/**
 * Class <code>DebugHandler</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class DebugHandler extends AbstractPerspective {
    private JComponent view = null;
    private SourceGraph sourceGraph;
    private TargetGraph targetGraph;
    int stepIndex = 0;
    private Map<ViewGraphJGraph, Pair<MultiDirectionAbstractStructure,JComponent>> targetSchemesView;
    private Map<AbstractConstruct, ModelComponent> sourceSchemeInfo;
    private ViewGraphJGraph[] viewGraphs;

    public DebugHandler(TopLevelPAC mediator) {
	super(mediator);
	// crea splitter di graph
	JSplitPane graphSplitter = new JSplitPane();
	graphSplitter.setBorder(null);
	graphSplitter.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
	graphSplitter.setDividerSize(8);
	graphSplitter.setOneTouchExpandable(true);

	// aggiungi graph
	targetGraph = new TargetGraph(this);
	graphSplitter.setRightComponent(targetGraph.getView());
	sourceGraph = new SourceGraph(this);
	graphSplitter.setLeftComponent(sourceGraph.getView());

	// salva il risultato finale
	this.view = graphSplitter;
    }

    public JComponent getView() {
	return view;
    }


    public void newSession(MultiDirectionAbstractStructure sourceScheme, Collection<MultiDirectionAbstractStructure> targetSchemes) throws ClassNotFoundException, Exception {
	// carica il grafo source 
    final ViewGraphJGraph sourceSchemeView = VisualizerFacade.createView(sourceScheme);
	JScrollPane realSourceView = new MoreUnitIncrementScrollPane(sourceSchemeView.getView());
	this.sourceGraph.setView(sourceScheme.getName(), null, realSourceView, sourceScheme.getDescription());

	// salva i construct presenti nel grafo source
	this.sourceSchemeInfo = new LinkedHashMap<AbstractConstruct, ModelComponent>();
	for (ModelComponent v : sourceSchemeView.getModel().getVertices()) {
	    Object info = v.getInfo();
	    if (info instanceof AbstractConstruct) {
		AbstractConstruct c = (AbstractConstruct) info;
		this.sourceSchemeInfo.put(c, v);
	    }
	}

	this.targetSchemesView = new LinkedHashMap<ViewGraphJGraph, Pair<MultiDirectionAbstractStructure,JComponent>>();
	// salva tutti i target step
	for (MultiDirectionAbstractStructure targetScheme : targetSchemes) {
	    // carica il grafo target
	    final ViewGraphJGraph targetSchemeView = VisualizerFacade.createView(targetScheme);
	    
	    JScrollPane realTargetView = new MoreUnitIncrementScrollPane(targetSchemeView.getView());
	    Pair<MultiDirectionAbstractStructure,JComponent> value = new Pair<MultiDirectionAbstractStructure,JComponent>(targetScheme, realTargetView);
	    this.targetSchemesView.put(targetSchemeView, value);

	    // aggiungi azione al click su un nodo
	    ModelDoubleClickListener clickAction = new LinksShowClickEvent(sourceSchemeView, targetSchemeView, this.sourceSchemeInfo);
	    targetSchemeView.addClickEvent(clickAction);
	}

	// salva come array i target per referenziare tramite indice
	this.viewGraphs = new ViewGraphJGraph[targetSchemesView.size()];
	this.viewGraphs = targetSchemesView.keySet().toArray(viewGraphs);
	this.createHistoryActions(this.viewGraphs);

	// carica il primo target
	if ( ! this.targetSchemesView.isEmpty() ) {
	    this.goToStep(0);
	}
    }

    private void createHistoryActions(ViewGraphJGraph[] graphs) {
	class HistoryAction extends AbstractAction {
	    private static final long serialVersionUID = 1L;
	    private int stepIndex;
	    HistoryAction(String name, int stepIndex) {
		super.putValue(Action.NAME, name);
		this.stepIndex = stepIndex;
	    }
	    public void actionPerformed(ActionEvent e) {
		DebugHandler.this.goToStep(stepIndex);
	    }    
	}
	// aggiungi le azioni
	super.historyActions = new LinkedList<Action>();
	for (int i = 0; i < graphs.length; i++) {
	    ViewGraphJGraph graph = graphs[i];
	    Pair<MultiDirectionAbstractStructure,JComponent> viewGraphInfo = this.targetSchemesView.get(graph);
	    MultiDirectionAbstractStructure structureDrawen = viewGraphInfo.getFirst();
	    String counter = (i+1) +"/"+ (this.viewGraphs.length);
	    String name = counter + " " + structureDrawen.getName();
		
	    Action historyAction = new HistoryAction(name, i);
	    super.historyActions.add(historyAction);
	}
	
	// notifica l'avvenuto inserimento
	super.updateHistoryActions();
    }

    private void goToStep(int stepIndex) {
	this.stepIndex = stepIndex;
	loadCurrentTarget();
    }

    private void loadCurrentTarget() {
	ViewGraphJGraph viewGraph = this.viewGraphs[this.stepIndex];
	Pair<MultiDirectionAbstractStructure,JComponent> viewGraphInfo = this.targetSchemesView.get(viewGraph);
	MultiDirectionAbstractStructure structureDrawen = viewGraphInfo.getFirst();
	JComponent realTargetView = viewGraphInfo.getSecond();
	HtmlString tooltip = ToolTipManager.instance.createToolTip(structureDrawen);
	String counter = (this.stepIndex+1) +"/"+ (this.viewGraphs.length);
	this.targetGraph.setView(counter + " " + structureDrawen.getName(), null, realTargetView, tooltip.getHtmlText());
	
	// forza a ricolorare ad ogni cambio di step
	viewGraph.click();
	
	this.updateActionsState();
    }

    private void updateActionsState() {
	if (viewGraphs == null) {
	    BackAction.sharedInstance.setEnabled(false);
	    NextAction.sharedInstance.setEnabled(false);
	    return; // esci
	} // else...

	if (stepIndex == 0) {
	    BackAction.sharedInstance.setEnabled(false);
	    NextAction.sharedInstance.setEnabled(true);
	} else if (stepIndex == viewGraphs.length - 1) {
	    BackAction.sharedInstance.setEnabled(true);
	    NextAction.sharedInstance.setEnabled(false);
	} else {
	    BackAction.sharedInstance.setEnabled(true);
	    NextAction.sharedInstance.setEnabled(true);
	}
    }

    /**
     * Ricarica la sessione di debug con le
     * stesse impostazioni
     */
    public void refresh() {
	// il wizard contiene le stesse impostazioni
	// riesegui semplicemente il pulsante 'confirm'
	NewApplyRuleDebuggerWizard wizard = NewApplyRuleDebuggerWizard.getInstance(TopLevelPAC.getFrame());
	wizard.doConfirm();
    }

    @Override
    public String getRefreshDescription() {
	return "Reload Debug Session";
    }

    @Override
    public String getBackActionDescription() {
	return "Previous Rule Step";
    }

    @Override
    public String getNextActionDescription() {
	return "Next Rule Step";
    }

    @Override
    public void back() {
	this.stepIndex--;
	this.goToStep(stepIndex);
    }

    @Override
    public void next() {
	this.stepIndex++;
	this.goToStep(stepIndex);
    }

    @Override
    public void open() {
	super.open();
	this.updateActionsState();
    }

}
