package org.modelgen.jpf.visualizer.pac.top;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.AbstractModelGenBuilderFacade;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.tooltip.ToolTipManager;
import org.modelgen.jpf.visualizer.pac.TopLevelPAC;
import org.modelgen.jpf.visualizer.pac.actions.wizard.NewApplyRuleDebuggerWizard;
import org.modelgen.jpf.visualizer.pac.top.views.ViewHandler;
import org.modelgen.jpf.visualizer.pac.top.views.ViewsOverview;

import java.awt.Dimension;
import java.awt.Frame;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

/**
 * Class <code>ModelGenViewManager</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ViewsHandler extends AbstractPerspective {
    private static final long serialVersionUID = 1L;
    private Map<ModelComponent,ViewHandler> modelGenViewersLoaded = new LinkedHashMap<ModelComponent,ViewHandler>();
    private JSplitPane view;
    private JPanel modelGenViewer;
    private ViewsOverview viewsOverview;
    private Map<ModelComponent,JFrame> windows = new LinkedHashMap<ModelComponent,JFrame>();
    private JScrollPane tree;

    public ViewsHandler(TopLevelPAC mediator) throws ClassNotFoundException, Exception {
	super(mediator);
	view = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

	TreeModel treeModel = createTreeModel();
	viewsOverview = new ViewsOverview(this, treeModel);

	tree = viewsOverview.getView();
	modelGenViewer = new JPanel();
	view.setLeftComponent(tree);
	view.setRightComponent(this.modelGenViewer);
	view.setOneTouchExpandable(true);
	view.setBorder(null);
	view.setDividerSize(8);
	view.setDividerLocation(200);

	Dimension minimumSize = new Dimension(100, 50);
	tree.setMinimumSize(minimumSize);
	this.modelGenViewer.setMinimumSize(minimumSize);
	view.setPreferredSize(new Dimension(800, 600));
    }

    public ViewHandler open(ModelComponent structureWrapped) throws ClassNotFoundException, Exception {
	ViewHandler viewer = null;
	if ( ! this.modelGenViewersLoaded.containsKey(structureWrapped) ) {
	    //this.loadStructure(structureWrapped);
	    viewer = createView(structureWrapped);
	    this.modelGenViewersLoaded.put(structureWrapped, viewer);
	    //this.modelGenViewer.add(viewer.getView(), BorderLayout.CENTER);
	} else {
	    viewer = this.modelGenViewersLoaded.get(structureWrapped);
	}
	int divider = view.getDividerLocation();
	view.setRightComponent(viewer.getView());
	view.setDividerLocation(divider);
	viewer.getView().grabFocus(); // notifica dell'apertura
	
	return viewer;
    }

    public boolean isOpened(ModelComponent structureWrapped) {
	return this.modelGenViewersLoaded.containsKey(structureWrapped);
    }

    @SuppressWarnings("unchecked")
    private TreeModel createTreeModel() throws ClassNotFoundException, Exception {
	MultiDirectionAbstractStructure rootTree = AbstractModelGenBuilderFacade.getInstance().getSuperModel();

	DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(wrap(rootTree));
	/* Inserisco modelli e relativi schemi */
	for (MultiDirectionAbstractStructure model : rootTree.getChildrens()) {
	    DefaultMutableTreeNode modelNode = new DefaultMutableTreeNode(wrap(model));

	    rootNode.add(modelNode);
	    for (MultiDirectionAbstractStructure scheme : model.getChildrens()) {
		DefaultMutableTreeNode schemeNode = new DefaultMutableTreeNode(wrap(scheme));
		modelNode.add(schemeNode);
		for(MultiDirectionAbstractStructure instance : scheme.getChildrens()) {
		    DefaultMutableTreeNode instanceNode = new DefaultMutableTreeNode(wrap(instance));
		    schemeNode.add(instanceNode);
		}
	    }
	}

	TreeModel myTreeModel = new DefaultTreeModel(rootNode);
	return myTreeModel;
    }

    private static ModelComponent wrap(MultiDirectionAbstractStructure structure) {
	ModelComponent result = new ModelComponent(structure);
	result.setTooltip(ToolTipManager.instance.createToolTip(structure));
	return result;
    }

    public JSplitPane getView() {
	return view;
    }

    public ViewHandler createView(ModelComponent structureWrapped) throws ClassNotFoundException, Exception {
	ViewHandler viewer = new ViewHandler(this, structureWrapped);
	return viewer;
    }

    public void newWindow(ModelComponent structureWrapped, JFrame prototype) throws ClassNotFoundException, Exception {
	JFrame window = prototype;
	if ( ! this.windows.containsKey(structureWrapped) ) {
	    ViewHandler view = this.createView(structureWrapped);
	    prototype.getContentPane().add(view.getView());
	    prototype.setVisible(true);
	    this.windows.put(structureWrapped, prototype);
	    super.mediator.addWindow(prototype);
	} else {
	    window = this.windows.get(structureWrapped);
	}
	int state = window.getExtendedState();
	state &= ~Frame.ICONIFIED; // deiconifica
	window.setExtendedState(state);
	window.toFront();
    }

    public Collection<JFrame> getWindows() {
	return this.windows.values();
    }

    public void removeWindow(ModelComponent id) {
	JFrame window = this.windows.get(id);
	this.windows.remove(id);
	super.mediator.removeWindow(window);
    }

    public void addWindow(JFrame newWindow) {
	super.mediator.addWindow(newWindow);
    }

    public void removeWindow(JFrame window) {
	super.mediator.removeWindow(window);
    }

    public void grabFocus(ModelComponent structureWrapped) {
	ViewHandler viewer = this.modelGenViewersLoaded.get(structureWrapped);
	viewer.getView().grabFocus(); // notifica dell'apertura
    }

    /**
     * Ricarica l'albero
     */
    public void refresh() throws Exception {
	AbstractModelGenBuilderFacade.reload();
	TreeModel treeModel = createTreeModel();
	viewsOverview.setModel(treeModel);
	viewsOverview.clearHistory();
	modelGenViewersLoaded.clear();
	NewApplyRuleDebuggerWizard.newInstance();
    }

    @Override
    public String getRefreshDescription() {
	return "Reload ModelGen Tree";
    }
    
    @Override
    public String getBackActionDescription() {
	return "Previous Selected Tree Node";
    }

    @Override
    public String getNextActionDescription() {
	return "Next Selected Tree Node";
    }

    @Override
    public void back() {
	viewsOverview.back();
    }

    @Override
    public void next() {
	viewsOverview.next();
    }

    @Override
    public void open() {
	super.open();
	// ripristina le azioni della prospettiva
	viewsOverview.open();
    }
    
    

}
