package org.modelgen.jpf.visualizer.pac.top.debug;

import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.AbstractConstruct;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.pac.top.views.view.ModelDoubleClickListener;
import org.modelgen.jpf.visualizer.pac.top.views.view.ViewGraphJGraph;
import org.modelgen.jpf.visualizer.util.HtmlString;
import org.modelgen.jpf.visualizer.util.Pair;

import java.awt.Color;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class LinksShowClickEvent implements ModelDoubleClickListener {
	public static final Color weakLinkNodeColor = Color.yellow;
	public static final Color hardLinkNodeColor = Color.green;
	public static final Color sourceNodeColor = hardLinkNodeColor;
	public static final Color sourceIsolatedNodeColor = Color.yellow;

	public static final String weakLinkTextColor = "Red";
	public static final String hardLinkTextColor = "Blue";
	public static final String sourceIsolatedTextColor = "Red";
	public static final String sourceNodeText = hardLinkTextColor;

	private ViewGraphJGraph sourceSchemeView;
	private ViewGraphJGraph targetSchemeView;
	private Map<AbstractConstruct, ModelComponent> sourceSchemeInfo;
	private Map<ModelComponent, Pair<ModelComponent, Set<ModelComponent>>> fromTargetNodeClickedToNodesSourceToColor;
	private Collection<ModelComponent> sourceVerticesNotColorable;

	// azione da annullare condivisa tra tutti gli step
	private static MacroAction macro = null;
	private static MacroAction initialColorAction = null;

	public LinksShowClickEvent(ViewGraphJGraph sourceSchemeView, ViewGraphJGraph targetSchemeView, Map<AbstractConstruct, ModelComponent> sourceSchemeInfo) {
		this.sourceSchemeView = sourceSchemeView;
		this.targetSchemeView = targetSchemeView;
		this.sourceSchemeInfo = sourceSchemeInfo;
		this.fromTargetNodeClickedToNodesSourceToColor = new LinkedHashMap<ModelComponent, Pair<ModelComponent, Set<ModelComponent>>>();
		for (ModelComponent node : targetSchemeView.getModel().getVertices()) {
			// salva associazione nodo target - coloro sorgente 
			Pair<ModelComponent, Set<ModelComponent>> nodesSourceToColor = saveDoubleClickAction(node);
			if (nodesSourceToColor != null) {
				this.fromTargetNodeClickedToNodesSourceToColor.put(node, nodesSourceToColor);
			}
		}
		
		// salva nodi sorgente isolati
		Collection<ModelComponent> sourceVerticesColorable = new HashSet<ModelComponent>();
		for(Pair<ModelComponent, Set<ModelComponent>> sourceVertices 
				: this.fromTargetNodeClickedToNodesSourceToColor.values()) {
			sourceVerticesColorable.add(sourceVertices.getFirst());
			sourceVerticesColorable.addAll(sourceVertices.getSecond());
		}
		Collection<ModelComponent> sourceVertices = sourceSchemeView.getModel().getVertices();
		this.sourceVerticesNotColorable = new HashSet<ModelComponent>();
		for(ModelComponent sourceVertex : sourceVertices) {
			// se non ha una colorazione e' un nodo isolato
			if(sourceVertex.isStructural() && !sourceVerticesColorable.contains(sourceVertex)) {
				this.sourceVerticesNotColorable.add(sourceVertex);
			}
		}
	}

	/**
	 * Calcola quali sono i nodi da colorare e li salva per 
	 * non ricalcolare le informazioni ad ogni click
	 * @param nodeClicked il nodo cliccato
	 * @return la coppia [hardLink,weakLinks] da colorare
	 */
	public Pair<ModelComponent, Set<ModelComponent>> saveDoubleClickAction(ModelComponent nodeClicked) {
		Pair<ModelComponent, Set<ModelComponent>> nodesToColor;

		// se e' un nodo ausiliario esci
		if ( ! (nodeClicked.getInfo() instanceof AbstractConstruct) ) {
			return null;
		}

		// ottieni il nodo hardLink e i nodi weakLinks
		AbstractConstruct targetConstruct = (AbstractConstruct) nodeClicked.getInfo();
		AbstractConstruct hardLink = targetConstruct.getHardLink();
		Collection<AbstractConstruct> weakLinks = targetConstruct.getWeakLinks();

		// per ogni link, ottieni il corrispondente 
		// nodo nel grafo e salvalo
		Set<ModelComponent> weakNodesToColor = new HashSet<ModelComponent>();
		for (AbstractConstruct weakLink : weakLinks) {
			ModelComponent weakLinkNode = this.sourceSchemeInfo.get(weakLink);
			if (weakLinkNode != null) {
				weakNodesToColor.add(weakLinkNode);
			}
		}

		// salva hard link da colorare
		ModelComponent hardNodeToColor = this.sourceSchemeInfo.get(hardLink);
		assert hardNodeToColor != null;
		//hardNodeToColor = hardNodeToColor.calculateRoot();
		//this.sourceSchemeView.setColorInRoot(hardLinkNode, Color.yellow);

		// se nei nodi weak e' gia' colorato l'hard, tralascia
		weakNodesToColor.remove(hardNodeToColor);

		// salva i nodi da colorare
		nodesToColor = new Pair<ModelComponent, Set<ModelComponent>>(hardNodeToColor, weakNodesToColor);
		
		return nodesToColor;
	}

	/* doppio click su un nodo */
	public void doubleClickedPerformed(ModelComponent nodeClicked) {
		if(nodeClicked == null) { 
			initialColor(); 
			return; 
		}
		
		Pair<ModelComponent, Set<ModelComponent>> nodesToColor;

		// annulla la precedente colorazione
		if (LinksShowClickEvent.macro != null) {
			LinksShowClickEvent.macro.undo();
			initialColorAction.exec();
		}

		// ottieni nodi da colorare
		nodesToColor = this.fromTargetNodeClickedToNodesSourceToColor.get(nodeClicked);

		// se non e' un nodo che prevede colorazione, esci
		if (nodesToColor == null) return;

		ModelComponent hardNodeToColor = nodesToColor.getFirst();
		Set<ModelComponent> weakNodesToColor = nodesToColor.getSecond();

		// se non e' un nodo che prevede colorazione, esci
		if (hardNodeToColor == null) return;

		LinksShowClickEvent.macro = new MacroAction();

		HtmlString clickedName = (HtmlString) nodeClicked.getHtmlName().clone();
		// titolo blu
		clickedName.setText("<font color=\""+sourceNodeText+"\">"+clickedName.getText()+"</font>");
		
		ColorizeAction toClicked = new ColorizeAction(this.targetSchemeView, nodeClicked, 
				sourceNodeColor, clickedName);
		LinksShowClickEvent.macro.addAction(toClicked);


		// colora i weak links
		for (ModelComponent weakNodeToColor : weakNodesToColor) {
			HtmlString weakName = (HtmlString) weakNodeToColor.getHtmlName().clone();
			// titolo rosso
			weakName.setText("<font color=\""+weakLinkTextColor+"\">"+weakName.getText()+"</font>");
			
			ColorizeAction toWeak = new ColorizeAction(this.sourceSchemeView, weakNodeToColor,
					weakLinkNodeColor, weakName);
			LinksShowClickEvent.macro.addAction(toWeak);
		}

		// colora l'hard link
		HtmlString hardName = (HtmlString) hardNodeToColor.getHtmlName().clone();
		// titolo blu
		hardName.setText("<font color=\""+hardLinkTextColor+"\">"+hardName.getText()+"</font>");
		
		ColorizeAction toHard = new ColorizeAction(this.sourceSchemeView, hardNodeToColor, 
				hardLinkNodeColor, hardName);
		LinksShowClickEvent.macro.addAction(toHard);

		// esegui coloring
		LinksShowClickEvent.macro.exec();

	}

	private void initialColor() {
		if(initialColorAction != null) {
			initialColorAction.undo();
			if (LinksShowClickEvent.macro != null) {
				LinksShowClickEvent.macro.undo();
				LinksShowClickEvent.macro = null;
			}
		}
		
		initialColorAction = new MacroAction();
		
		for(ModelComponent v : this.sourceVerticesNotColorable) {
			HtmlString name = (HtmlString) v.getHtmlName().clone();
			// titolo bianco
			name.setText("<font color=\""+sourceIsolatedTextColor+"\">"+name.getText()+"</font>");
			
			ColorizeAction colorizeAction = new ColorizeAction(this.sourceSchemeView, v, 
					sourceIsolatedNodeColor, name);
			initialColorAction.addAction(colorizeAction);
		}
		
		initialColorAction.exec();
		
	}

}