package org.modelgen.jpf.visualizer.pac.top.debug;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.pac.top.views.view.ViewGraphJGraph;
import org.modelgen.jpf.visualizer.util.HtmlString;

import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

interface UndoableAction {
    public void exec();
    public void undo();
}

class ColorizeAction implements UndoableAction {
	
    private ViewGraphJGraph graph;
    private ModelComponent nodeToColor;
    private Color oldColor;
    private static final Color DEFAULT_COLOR = Color.WHITE;
    private Color newColor;
    private HtmlString oldName;
    private HtmlString newName;

    public ColorizeAction(ViewGraphJGraph graph, ModelComponent nodeToColor, Color newColor, HtmlString newName) {
	this.graph = graph;
	this.nodeToColor = nodeToColor;
	this.oldColor = DEFAULT_COLOR;
	this.newColor = newColor;
	this.oldName = (HtmlString) nodeToColor.getHtmlName().clone();
	if (newName == null) {
	    this.newName = this.oldName;
	} else {
	    this.newName = newName;
	}
    }
    
    public void undo() {
	ColorizeAction.setColor(graph, nodeToColor, oldColor);
	nodeToColor.setHtmlName(oldName);
    }
    public void exec() {
	ColorizeAction.setColor(graph, nodeToColor, newColor);
	nodeToColor.setHtmlName(newName);
    }
    
    /**
     * Colora il nodo e tutti i padri.
     * Man mano che salo nella gerarchia, il colore impostato
     * si fa piu' leggero
     */
    public static void setColor(ViewGraphJGraph graph, ModelComponent nodeToColor, Color color) {
	graph.setColor(nodeToColor, color);
	ModelComponent parent = nodeToColor.getParent();
	if (parent != null) {
	    ColorizeAction.setColor(graph, parent, color);
	}
    }
}

class MacroAction implements UndoableAction {
    private List<UndoableAction> actions = new LinkedList<UndoableAction>();
    public void addAction(UndoableAction action) {
	this.actions.add(action);
    }

    public void exec() {
	for (UndoableAction action : actions) {
	    action.exec();
	}
    }

    public void undo() {
	for (UndoableAction action : actions) {
	    action.undo();
	}
    }
    
}

