package org.modelgen.jpf.visualizer.pac.top.views;

import org.modelgen.jpf.visualizer.modelgenview.ModelGenBuilder;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;
import org.modelgen.jpf.visualizer.pac.top.ViewsHandler;
import org.modelgen.jpf.visualizer.pac.top.views.view.ModelGraphViewHandler;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Class <code>ViewHandler</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ViewHandler {
    private JTabbedPane tabbedPane;
    private ViewsHandler mediator;
    
    ModelGraphViewHandler[] tabItems;

    public ViewHandler(ViewsHandler mediator, ModelComponent structureWrapped) throws ClassNotFoundException, Exception {
	tabItems = new ModelGraphViewHandler[2];
	this.mediator = mediator;
	MultiDirectionAbstractStructure structure = (MultiDirectionAbstractStructure) structureWrapped.getInfo();
	ModelGraph forwardModelGraph = null, backwardModelGraph = null;
	
	// non fare il forward se e' un'istanza
	if (!MultiDirectionAbstractStructure.isInstance(structure)) {
	    forwardModelGraph = ModelGenBuilder.parse(structure, null, false);
	}
	
	MultiDirectionAbstractStructure metaStructure = structure.getParent();
	if(metaStructure != null) {
	    Collection<MultiDirectionAbstractStructure> ownerAllowed = new LinkedList<MultiDirectionAbstractStructure>();
	    ownerAllowed.add(structure);
	    backwardModelGraph = ModelGenBuilder.parse(metaStructure, ownerAllowed, true);
	}
	
	tabbedPane = new JTabbedPane();
	final ModelGraphViewHandler forwardHandler = new ModelGraphViewHandler(this, forwardModelGraph, true);
	int index = 0;
	if(forwardModelGraph != null) {
	    JSplitPane forwardSplit = forwardHandler.getView();
	    tabbedPane.insertTab("Forward", null, forwardSplit, "View all childrens's data", index);
	    tabItems[index] = forwardHandler;
	    index++;
	}
	final ModelGraphViewHandler backwardHandler = new ModelGraphViewHandler(this, backwardModelGraph, false);
	if(backwardModelGraph != null) {
	    JSplitPane backwardSplit = backwardHandler.getView();
	    tabbedPane.insertTab("Backward", null, backwardSplit, "View elements used to build this structure", index);
	    tabItems[index] = backwardHandler;
	}
	tabbedPane.setSelectedIndex(0);
	tabbedPane.addFocusListener(new FocusAdapter() {
	    @Override
	    public void focusGained(FocusEvent e) {
		int index = tabbedPane.getSelectedIndex();
		ModelGraphViewHandler handler;
		handler = tabItems[index];
		handler.updateTableWindowAction();
		handler.updateSaveLayoutAction();
	    }
	});
	tabbedPane.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent change) {
		int index = tabbedPane.getSelectedIndex();
		ModelGraphViewHandler handler;
		if (index == 0) {
		    handler = forwardHandler;
		} else {
		    handler = backwardHandler;
		}
		handler.updateTableWindowAction();
		handler.updateSaveLayoutAction();
	    }
	});
    }
    
    public JTabbedPane getView() {
	return tabbedPane;
    }

    /**
     * @param e
     */
    public void showException(Exception e) {
	mediator.showException(e);
    }

    public void addWindow(JFrame newWindow) {
	mediator.addWindow(newWindow);
    }

    public void removeWindow(JFrame window) {
	mediator.removeWindow(window);
    }
    
    public void showMessage(String message) {
	mediator.showMessage(message);
    }
    
}
