package org.modelgen.jpf.visualizer.pac.top.views;

import org.modelgen.jpf.visualizer.commons.ToolConfigurationManager;
import org.modelgen.jpf.visualizer.modelgenview.modelgen.adapters.MultiDirectionAbstractStructure;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.pac.actions.BackAction;
import org.modelgen.jpf.visualizer.pac.actions.NewWindowAction;
import org.modelgen.jpf.visualizer.pac.actions.NextAction;
import org.modelgen.jpf.visualizer.pac.top.ViewsHandler;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Stack;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

/**
 * Class <code>ViewsOverview</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ViewsOverview {

    private Stack<TreePath> backHistory = new Stack<TreePath>();
    private Stack<TreePath> nextHistory = new Stack<TreePath>();
    private DataTree visualPane;
    private JScrollPane view;

    public ViewsOverview(ViewsHandler mediator, TreeModel model) {
	view = new JScrollPane();
	visualPane = new DataTree(mediator, model);
	view.setViewportView(visualPane);
	visualPane.repaint();
	visualPane.revalidate();
    }

    public void setModel(TreeModel treeModel) {
	visualPane.setModel(treeModel);
	visualPane.repaint();
	visualPane.revalidate();
    }

    public class DataTree extends JTree implements TreeSelectionListener, MouseListener {
	private static final long serialVersionUID = 1L;
	private ViewsHandler mediator;
	private TreePath lastSelected;
	private Stack<TreePath> workingStack;
	private Action workingAction;
	private boolean userNodeClick = true;

	public DataTree(ViewsHandler mediator, TreeModel model) {
	    super();
	    DefaultTreeCellRenderer renderer = 
		new TreeCellLoadedDisplayRenderer();
	    this.setCellRenderer(renderer);
	    this.mediator = mediator;
	    this.addTreeSelectionListener(this);
	    this.addMouseListener(this);
	    this.setModel(model);
	    this.setToolTipText("");
	}

	private JPopupMenu createPopupMenu(ModelComponent structureWrapped) {
	    JPopupMenu menu = new JPopupMenu();
	    menu.add(NewWindowAction.sharedInstance);
	    return menu;
	}

	public void valueChanged(TreeSelectionEvent e) {
	    if (userNodeClick) {
		workingStack = backHistory;
		workingAction = BackAction.sharedInstance;
	    }
	    
	    // salva la precedente/successiva posizione 
	    // per consentire il next/back
	    if (lastSelected != null) {
		workingStack.push(lastSelected); // salva in history
		// se e' il primo inserimento abilita l'azione
		workingAction.setEnabled(true);
	    } else {
		workingAction.setEnabled(false);
	    }

	    DefaultMutableTreeNode node = (DefaultMutableTreeNode)this.getLastSelectedPathComponent();
	    if (node == null) return;
	    ModelComponent structureWrapped = (ModelComponent) node.getUserObject();

	    Cursor undoState = this.getCursor();
	    this.setCursor(new Cursor(Cursor.WAIT_CURSOR)); // clessidra
	    try {
		this.mediator.open(structureWrapped);

		/* Modifica l'azione corrispondente */
		NewWindowAction newWindow = NewWindowAction.sharedInstance;
		newWindow.setViewCreator(mediator);
		newWindow.setReceiver(structureWrapped);
		ImageIcon windowIcon = loadIcon((MultiDirectionAbstractStructure) structureWrapped.getInfo());
		newWindow.setWindowIcon(windowIcon);
		newWindow.setEnabled(true);

		JPopupMenu popup = this.createPopupMenu(structureWrapped);
		this.setComponentPopupMenu(popup);

		// salva la selezione
		this.lastSelected = this.getSelectionPath();

		if (userNodeClick) {
		    nextHistory.clear();
		    NextAction.sharedInstance.setEnabled(false);
		} else {
		    // mantieni il nextHistory. Ripristina userNodeClick
		    userNodeClick = true;
		}
		
		// salva le history actions per il toolbar
		updateHistoryActions();

	    } catch (Exception ex) {
		mediator.showException(ex);
	    } finally {
		this.setCursor(undoState); // fine clessidra
	    }
	}

	@Override
	public String getToolTipText(MouseEvent evt) {
	    if (getRowForLocation(evt.getX(), evt.getY()) == -1) return null;    
	    TreePath curPath = getPathForLocation(evt.getX(), evt.getY());
	    DefaultMutableTreeNode node = (DefaultMutableTreeNode) curPath.getLastPathComponent();
	    ModelComponent toolTipableNode = (ModelComponent) node.getUserObject();
	    return toolTipableNode.getTooltipText();
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	/* Seleziona il nodo anche in caso di click destro */
	public void mousePressed(MouseEvent e) {
	    int selRow = this.getRowForLocation(e.getX(), e.getY());
	    if(selRow != -1) this.setSelectionRow(selRow);
	    if (e.getClickCount() == 2) {
		NewWindowAction.sharedInstance.actionPerformed(null); // click
	    }
	    e.consume();
	}
	public void mouseReleased(MouseEvent e) {}

	public class TreeCellLoadedDisplayRenderer extends DefaultTreeCellRenderer {
	    private static final long serialVersionUID = 1L;
	    private Font undoFont;
	    private Color undoForeground;

	    public TreeCellLoadedDisplayRenderer() {
		super();
	    }

	    @Override
	    public Component getTreeCellRendererComponent(JTree tree, Object valueObj, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
		JLabel cellRenderer = (JLabel) super.getTreeCellRendererComponent(tree, valueObj, sel, expanded, leaf,
			row, hasFocus);
		if(this.undoFont == null) {
		    this.undoFont = super.getFont();
		    this.undoForeground = super.getForeground();
		}
		DefaultMutableTreeNode valueNode = (DefaultMutableTreeNode) valueObj;

		/* instanceof necessario: in Windows non serve, in Linux serve.
		 * E' proprio vero: 'write once, debug everywhere' */
		if (valueNode.getUserObject() instanceof ModelComponent) {
		    ModelComponent valueWrapped = (ModelComponent) valueNode.getUserObject();
		    MultiDirectionAbstractStructure value = (MultiDirectionAbstractStructure) valueWrapped.getInfo();

		    Font font = this.undoFont;
		    Color foreground = this.undoForeground;
		    switch(value.getLoadedState()) {
		    case LOADED:
			font = font.deriveFont(Font.ITALIC);
			break;
		    case ERROR_LOADED:
			foreground = Color.red;
			break;
		    default:
			// lascia il default
		    }
		    // se e' gia' stato cliccato in precedenza
		    if(mediator.isOpened(valueWrapped)) {
			font = font.deriveFont(Font.BOLD);
		    }


		    super.validate();
		    cellRenderer.setFont(font);
		    cellRenderer.setForeground(foreground);
		    ImageIcon icon = loadIcon(value);
		    if (icon != null) this.setIcon(icon);
		}
		return cellRenderer;
	    }

	    @Override
	    public Dimension getPreferredSize() {
		Dimension size = super.getPreferredSize();
		size.width *= 1.15;
		return size;
	    }
	}
    }

    public JScrollPane getView() {
	return view;
    }

    private ImageIcon loadIcon(MultiDirectionAbstractStructure structure) {
	ImageIcon icon = null;
	if (MultiDirectionAbstractStructure.isSuperModel(structure) || MultiDirectionAbstractStructure.isModel(structure)) {
	     icon = ToolConfigurationManager.sharedInstance.getIcon("toolFrame1.png");
	} else {
	    icon = ToolConfigurationManager.sharedInstance.getIcon("toolFrame2.png");
	}
	return icon;
    }

    public void back(boolean realExecute) {
	TreePath backValue = backHistory.pop();
	assert backValue != null;
	this.visualPane.workingAction = NextAction.sharedInstance;
	this.visualPane.workingStack = nextHistory;
	this.visualPane.userNodeClick = false;
	this.visualPane.setSelectionPath(backValue);
	if (backHistory.isEmpty()) {
	    BackAction.sharedInstance.setEnabled(false);
	}
    }
    
    public void back() {
	back(true);
    }

    public void next(boolean realExecute) {
	TreePath nextValue = nextHistory.pop();
	assert nextValue != null;
	this.visualPane.workingAction = BackAction.sharedInstance;
	this.visualPane.workingStack = backHistory;
	this.visualPane.userNodeClick = false;
	this.visualPane.setSelectionPath(nextValue);
	if (nextHistory.isEmpty()) {
	    NextAction.sharedInstance.setEnabled(false);
	}
    }
    
    public void next() {
	next(true);
    }

    /**
     * Ripristina lo stato delle azioni 
     */
    public void open() {
	updateActions();
    }
    
    private void updateActions() {
	if (nextHistory.isEmpty()) {
	    NextAction.sharedInstance.setEnabled(false);
	} else {
	    NextAction.sharedInstance.setEnabled(true);
	}
	if (backHistory.isEmpty()) {
	    BackAction.sharedInstance.setEnabled(false);
	} else {
	    BackAction.sharedInstance.setEnabled(true);
	}
    }
    

    public void updateHistoryActions() {
	// TODO
    }


    public void clearHistory() {
	this.backHistory.clear();
	this.nextHistory.clear();
	this.visualPane.lastSelected = null;
	updateActions();
    }

}
