package org.modelgen.jpf.visualizer.pac.top.views.view;

import org.modelgen.jpf.visualizer.ToolTipableTabbedPane;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;
import org.modelgen.jpf.visualizer.pac.actions.NewTableWindowAction;
import org.modelgen.jpf.visualizer.pac.actions.SaveGraphLayoutAction;
import org.modelgen.jpf.visualizer.pac.top.views.ViewHandler;
import org.modelgen.jpf.visualizer.util.MoreUnitIncrementScrollPane;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Class <code>ViewHandler</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ModelGraphViewHandler implements ChangeListener {

    private static final int BOTTOM_HEIGHT = 400;
    private List<AbstractView<?>> views = new LinkedList<AbstractView<?>>();
    private Map<ModelComponent, JScrollPane> viewsComponent = new LinkedHashMap<ModelComponent, JScrollPane>();
    private Map<JScrollPane, ModelComponent> modelsComponent = new LinkedHashMap<JScrollPane, ModelComponent>();
    private ViewGraph viewGraph = null;
    private ViewFactory factory;
    private JSplitPane split;
    private ToolTipableTabbedPane tableContainer;
    private ViewHandler mediator;
    private Map<ModelComponent, JFrame> windows = new LinkedHashMap<ModelComponent, JFrame>();
    private boolean saveLayoutEnabled;
    private ModelComponent lastOpenedTable = null;

    public ModelGraphViewHandler(ViewHandler mediator, ModelGraph modelGraph, boolean saveLayoutEnabled) {
	if (modelGraph == null) return; // non inizializzare
	this.saveLayoutEnabled = saveLayoutEnabled;
	this.mediator = mediator;
	this.factory = Settings.instance.getViewFactory();
	this.factory.setMediator(this);
	this.open(modelGraph);
	tableContainer = new ToolTipableTabbedPane();
	tableContainer.addChangeListener(this);
	split = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	split.setBorder(null);
	split.setDividerSize(8);
	split.setOneTouchExpandable(true);
	
	/* Aggiungi gli elementi allo split */
	JScrollPane scrollStructureView = new MoreUnitIncrementScrollPane();
	scrollStructureView.setViewportView(viewGraph.getView());
	split.setBottomComponent(tableContainer);
	split.setTopComponent(scrollStructureView);

	/* Imposta la posizione dello split */
	Dimension minimumSize = new Dimension(0, 0);
	tableContainer.setMinimumSize(minimumSize);
	scrollStructureView.setMinimumSize(minimumSize);
	int dividerLocation = BOTTOM_HEIGHT;
	//System.out.println(dividerLocation);
	split.setDividerLocation(dividerLocation);
    }

    public JSplitPane getView() {
	return this.split;
    }

    public void open(ModelComponent modelComponent) {
	JScrollPane scrollViewNode;
	if( ! viewsComponent.containsKey(modelComponent)) {
	    ViewComponent viewComponent = factory.createViewComponent(modelComponent);
	    scrollViewNode = new MoreUnitIncrementScrollPane();
	    scrollViewNode.setViewportView(viewComponent.getView());
	    this.views.add(viewComponent);
	    this.viewsComponent.put(modelComponent, scrollViewNode);
	    this.modelsComponent.put(scrollViewNode, modelComponent);
	    tableContainer.add(modelComponent, scrollViewNode);
	    updateTableWindowAction();
	} else {
	    scrollViewNode = this.viewsComponent.get(modelComponent);
	}

	// metti in foreground
	tableContainer.setSelectedComponent(scrollViewNode);
	
	// annulla precedente evidenziazione
	if (this.lastOpenedTable != null) {
	    setColor(this.lastOpenedTable, Color.white);
	}

	// evidenzia nel grafo il nodo che si e' aperto
	this.setColor(modelComponent, Color.yellow);
	
	// salva il nodo evidenziato per permettere il successivo undo
	this.lastOpenedTable = modelComponent;
    }
    
    /** Imposta il colore al nodo e a tutti i figli ricorsivamente */
    private void setColor(ModelComponent component, Color color) {
	this.viewGraph.setColor(component, color);
	for (ModelComponent child : component.getChildrens()) {
	    this.setColor(child, color); // ricorsione
	}
    }

    public void open(ModelGraph modelGraph) {
	if(viewGraph == null) {
	    this.viewGraph = factory.createViewGraph(modelGraph);
	    this.views.add(viewGraph);
	}
    }

    public void closeAll() {
	for(AbstractView<?> view : views) {
	    view.close();
	}
    }


    public void removeWindow(ModelComponent receiver) {
	JFrame window = windows.get(receiver);
	mediator.removeWindow(window);
	windows.remove(receiver);
    }

    public void newWindow(ModelComponent receiver, JFrame prototype) {
	JFrame window = prototype;
	if ( ! windows.containsKey(receiver) ) {
	    ViewComponent viewComponent = factory.createViewComponent(receiver);
	    JScrollPane scrollViewNode = new MoreUnitIncrementScrollPane();
	    scrollViewNode.setViewportView(viewComponent.getView());
	    prototype.add(scrollViewNode);
	    windows.put(receiver, prototype);
	    mediator.addWindow(prototype);
	    prototype.setVisible(true);
	} else {
	    window = this.windows.get(receiver); 
	}
	int state = window.getExtendedState();
        state &= ~Frame.ICONIFIED; // deiconifica
	window.setExtendedState(state);
	window.toFront();
    }

    public void showException(Exception e) {
	mediator.showException(e);
    }

    public void stateChanged(ChangeEvent e) {
	JScrollPane scrollTable = (JScrollPane) tableContainer.getSelectedComponent();
	ModelComponent valueClicked = this.modelsComponent.get(scrollTable);
	this.open(valueClicked);
	this.updateTableWindowAction();
    }
    

    public void updateTableWindowAction() {
	JScrollPane selectedView = (JScrollPane) tableContainer.getSelectedComponent();
	ModelComponent selectedModel = modelsComponent.get(selectedView);
	NewTableWindowAction.sharedInstance.setViewCreator(ModelGraphViewHandler.this);
	if (selectedModel != null) {
	    NewTableWindowAction.sharedInstance.setReceiver(selectedModel);
	    NewTableWindowAction.sharedInstance.setEnabled(true);
	} else {
	    NewTableWindowAction.sharedInstance.setEnabled(false);
	}
    }

    public void updateSaveLayoutAction() {
	if (saveLayoutEnabled) {
	    SaveGraphLayoutAction.sharedInstance.setEnabled(true);
	    SaveGraphLayoutAction.sharedInstance.setReceiver((ViewGraphJGraph) viewGraph);
	} else {
	    SaveGraphLayoutAction.sharedInstance.setEnabled(false, "To save the layout, choose it's forward version");
	}
    }
    
    public void showMessage(String message) {
	mediator.showMessage(message);
    }

}
