package org.modelgen.jpf.visualizer.pac.top.views.view;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.JTableFacade;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.JTableFacade.ColumnsIndexOutOfBoundsException;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.columns.groupable.ColumnGroup;
import org.modelgen.jpf.visualizer.util.Pair;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.table.DefaultTableModel;

/**
 * Class <code>ViewComponent</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ViewComponentJTable extends ViewComponent {
    private static final long serialVersionUID = 1L;
    private LinkedHashMap<ModelComponent,Integer> columnIndexes;

    public ViewComponentJTable(ModelGraphViewHandler mediator, ModelComponent model) {
	super(mediator, model);
    }

    @Override
    protected JComponent doCreateView() {
	columnIndexes = new LinkedHashMap<ModelComponent,Integer>();
	JTableFacade<ModelComponent,ModelComponent> facade = new JTableFacade<ModelComponent,ModelComponent>();

	/* Itera le colonne nel corretto ordine */
	//updateCompositeRelationships(model, cells.get(viewNode));

	/*--------- Aggiunge le colonne foglia -----------*/
	addLeafColumns(this.model, facade); // ricorsione

	/*---------- Aggiunge le colonne composite --------*/
	for (ModelComponent child : model.getChildrens()) {
	    if (!child.isLeaf()) { // isGroup
		addCompositeColumns(child, facade);
	    }
	}

	/*--------- Aggiungi le istanze -------------*/
	this.addRows(facade, model.getInstances());

	return facade.createJTable();
    }

    private void addLeafColumns(ModelComponent start, JTableFacade<ModelComponent, ModelComponent> facade) {
	if (start.isLeaf()) {
	    int index = facade.addColumn(start);
	    columnIndexes.put(start, index);
	}
	for (ModelComponent child : start.getChildrens()) {
	    addLeafColumns(child, facade); // ricorsione
	}
    }

    private ColumnGroup addCompositeColumns(ModelComponent group, JTableFacade<ModelComponent, ModelComponent> facade) {
	Pair<Integer,Integer> indexes = this.getFromAndToIndexes(group, Integer.MAX_VALUE, Integer.MIN_VALUE);
	int fromIndex = indexes.getFirst();
	int toIndex = indexes.getSecond();
	
	// crea la colonna composita contenente colonne semplici
	ColumnGroup columnCreated = null;
	if (fromIndex != Integer.MAX_VALUE || toIndex != Integer.MIN_VALUE) {
	    int columnToSpan = toIndex - fromIndex + 1;
	    columnCreated = facade.addColumnGroup(group,fromIndex,columnToSpan);
	}

	// aggiorna la colonna composita creata con altre colonne composite
	for (ModelComponent child : group.getChildrens()) {
	    if ( !child.isLeaf() ) { // isGroup
		ColumnGroup columnChildCreated = addCompositeColumns(child, facade); // ricorsione
		if (columnCreated != null) {
		    facade.addColumnGroup(columnCreated, columnChildCreated);
		} else { // creala
		    facade.addColumnGroup(group, columnChildCreated);
		}
	    }
	}
	return columnCreated;
    }
    
    private Pair<Integer,Integer> getFromAndToIndexes(ModelComponent group, int fromResult, int toResult) {
	Pair<Integer,Integer> indexes;
	for (ModelComponent child : group.getChildrens()) {
	    if (child.isLeaf()) {
		int index = this.columnIndexes.get(child);
		fromResult = Math.min(fromResult,index);
		toResult = Math.max(toResult, index);
	    }
	}
	indexes = new Pair<Integer,Integer>(fromResult, toResult);
	return indexes;
    }

    @SuppressWarnings("unchecked")
	private void addRows(JTableFacade<ModelComponent, ModelComponent> facade, Vector<Vector<List<ModelComponent>>> rows) {
	/* Esempio di rows: 
	 * [[[1102001], [Andrea], [11111, 22222]]]
	 * cioe' ho: una riga e 3 colonne.
	 * Nella colonna 3 ho 2 valori.
	 */
	for (Vector<List<ModelComponent>> row : rows) {
	    //System.out.println("current row: " +row);
	    Vector<ModelComponent> rowToAdd = new Vector<ModelComponent>();
	    DefaultTableModel multiValueContainer = facade.createMultiValueContainer();
	    int colIndex = 0;
	    for(List<ModelComponent> multiValue : row) { // per ogni colonna
		
		/* Se e' un valore singolo inserisci normalmente */
		if (multiValue == null) {
		    rowToAdd.add(new ModelComponent(""));
		} else if (multiValue.size() == 1) {
		    rowToAdd.add(multiValue.get(0));
		} else { // colonna multivalore
		    /* Di base inserisci la prima riga come riga non composita */
		    rowToAdd.add(multiValue.get(0));

		    /* Crea nel multiValueContainer le righe necessarie
		     * escludendo la prima riga che e' stata gia' inserita */
		    int multiValueSizeTraslated = multiValue.size() - 1;
		    if(multiValueContainer.getRowCount() < multiValueSizeTraslated) {
			int numberOfRowsToAdd = multiValueSizeTraslated - multiValueContainer.getRowCount();
			Object[] emptyRow = new Object[row.size()];
			for(int i = 0; i < emptyRow.length; i++) {
			    emptyRow[i] = new ModelComponent("");
			}
			while(numberOfRowsToAdd-- > 0) {
			    multiValueContainer.addRow(emptyRow);
			}
		    }

		    /* Aggiungi le righe */
		    for (int nestedRow = 0; nestedRow < multiValueSizeTraslated; nestedRow++) {
			//System.out.println("nestedRow = " + nestedRow);
			//System.out.println("colIndex = " + colIndex);
			ModelComponent value = multiValue.get(nestedRow+1); // dalla seconda riga
			multiValueContainer.setValueAt(value, nestedRow, colIndex);
		    }
		} // fine caso colonna multivalore
		colIndex++;
	    } // fine scansione multivalori della riga corrente

	    /* Aggiungi la riga al modello */
	    try {
		//System.out.println("adding row "+rowToAdd);
		facade.addRow(rowToAdd);
		if (multiValueContainer.getRowCount() > 0) {
		    Vector<Vector<ModelComponent>> multirows = multiValueContainer.getDataVector();
		    for(Vector<ModelComponent> multirow : multirows) {
			//System.out.println("adding multirow "+multirow);
			    facade.addCompositeRow(multirow);
		    }
		}
	    } catch (ColumnsIndexOutOfBoundsException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	} // fine scansione della riga
    }

}
