package org.modelgen.jpf.visualizer.pac.top.views.view;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelGraph;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.JGraphFacade;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jgraph.groupable.GroupableJGraph;
import org.modelgen.jpf.visualizer.util.Pair;
import org.modelgen.jpf.visualizer.util.UnsharedString;

import java.awt.Color;
import java.io.IOException;
import java.io.Serializable;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Stack;

import javax.swing.JComponent;

/**
 * Class <code>ViewGraph</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ViewGraphJGraph extends ViewGraph implements Serializable {
    private static final long serialVersionUID = 1L;
    private JGraphFacade<ModelComponent> graphInfo;

    public ViewGraphJGraph(ModelGraphViewHandler mediator, ModelGraph model) {
	super(mediator, model);
    }

    public void addClickEvent(ModelDoubleClickListener clickListener) {
	this.graphInfo.addDoubleClickListener(clickListener);
    }

    @Override
    protected JComponent doCreateView() {
	JGraphFacade<ModelComponent> graphBuilder = new JGraphFacade<ModelComponent>();

	/* Creo i nodi */
	for(ModelComponent m : model) {
	    if (m.isStructural()) {
		//System.out.println("addVertex " + m);
		boolean isLeaf = !m.isRoot() && m.hasStructuralLeaf();
		graphBuilder.addVertex(m,isLeaf);
	    }
	}

	/* Creo gli archi */
	for(ModelComponent source : model) {
	    for (ModelComponent edge : model.getOutModelComponent(source)) {
		for (ModelComponent target : model.getOutModelComponent(edge)) {
		    //System.out.println(source + " ---["+edge+"]---> "+target);
		    graphBuilder.addEdge(edge, source, target);
		}
	    }
	}

	/* Creo i gruppi in ordine di inserimento.
	 * JGraph consente solo l'inserimento di figli
	 * in ordine inverso. Per questo ci appoggiamo
	 * a una coda LIFO per inserire in ordine inverso 
	 * ed avere quindi un ordine diretto. */
	Stack<Pair<ModelComponent,ModelComponent>> groupsToAdd = new Stack<Pair<ModelComponent,ModelComponent>>();
	for(ModelComponent m : model) {
	    if (m.isRoot()) {
		/* Salvo i gruppi da aggiungere */
		createGroup(m, groupsToAdd, graphBuilder);
	    }
	}

	// salvo i gruppi creati
	Set<ModelComponent> groups = new LinkedHashSet<ModelComponent>();

	// inserisco i gruppi in ordine inverso
	while(!groupsToAdd.empty()) {
	    Pair<ModelComponent,ModelComponent> groupToAdd = groupsToAdd.pop();
	    ModelComponent parent = groupToAdd.getFirst();
	    ModelComponent child = groupToAdd.getSecond();
	    //System.out.println("addChildren( " + parent + ", " + child + " )");
	    graphBuilder.addChildrenLast(parent, child);

	    // salva il gruppo creato
	    groups.add(parent);
	}

	
	// inserisco un vertice vuoto per lasciare spazio al titolo
	for (ModelComponent m : groups) {
	    String text = m.getHtmlName().getText();
	    // sostituisci i caratteri in spazi
	    StringBuffer textBuffer = new StringBuffer();
	    textBuffer.setLength(text.length()+6);
	    for (int i = 0; i < textBuffer.length(); i++) {
		textBuffer.setCharAt(i, '_');
	    }

	    // vertice fittizio per lasciare lo spazio
	    ModelComponent child = new ModelComponent(new UnsharedString("<font color=white>"+textBuffer.toString()+"</font>"));
	    child.setParent(m);
	    child.setTooltip(m.getTooltip());
	    graphBuilder.addChildrenLast(m, child);
	}

	/* Creo il grafo */
	GroupableJGraph<ModelComponent> jgraph = graphBuilder.getJGraph(model.getId());

	this.graphInfo = graphBuilder;

	return jgraph;
    }

    private void createGroup(ModelComponent parent, Stack<Pair<ModelComponent, ModelComponent>> groupsToAdd, JGraphFacade<ModelComponent> graphBuilder) {
	for (ModelComponent child : parent.getChildrens()) {
	    if(child.isStructural()) {
		Pair<ModelComponent,ModelComponent> parentChild;
		parentChild = new Pair<ModelComponent,ModelComponent>(parent, child);
		groupsToAdd.push(parentChild);
		createGroup(child, groupsToAdd, graphBuilder);
	    }
	}
    }

    @Override
    public void setColor(ModelComponent modelComponent, Color color) {
	this.graphInfo.getCreatedJGraph().setColor(modelComponent, color);
    }

    public void showException(Exception e) {
	mediator.showException(e);
    }

    public void saveLayout() throws IOException {
	this.graphInfo.saveLayout(model.getId());
    }

    @Override
    public String toString() {
	return model.toString();
    }

    public void showMessage(String message) {
	mediator.showMessage(message);
    }

	public void click() {
		this.graphInfo.click();
	}
}
