package org.modelgen.jpf.visualizer.test;

import org.modelgen.jpf.visualizer.modelgenview.mvc.model.ModelComponent;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.JTableFacade;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.JTableFacade.ColumnsIndexOutOfBoundsException;
import org.modelgen.jpf.visualizer.modelgenview.mvc.view.jtable.columns.groupable.ColumnGroup;
import org.modelgen.jpf.visualizer.util.HtmlString;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

/**
 * Class <code>JExtendedTableTest</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class MultiValueGroupableTableTest {

    public static void main(String[] args) throws ColumnsIndexOutOfBoundsException {
	JTableFacade<ModelComponent,ModelComponent> facade = new JTableFacade<ModelComponent,ModelComponent>();
	
	facade.addColumns(new ModelComponent("First Name"),new ModelComponent("Last Name"),new ModelComponent("Sport"),new ModelComponent("# of Years"),new ModelComponent("Vegetarian"),new ModelComponent("prova"));
	
	facade.addRow(new ModelComponent("Mary"), new ModelComponent("Campione"), new ModelComponent("Snowboarding"), new ModelComponent(new Integer(5)), new ModelComponent(new Boolean(false)));
	facade.addRow(new ModelComponent("Alison"), new ModelComponent("Huml"), new ModelComponent("Rowing"), new ModelComponent(new Integer(3)), new ModelComponent(new Boolean(true)));
	facade.addRow(new ModelComponent("Kathy"), new ModelComponent("Walrath"), new ModelComponent("Knitting"), new ModelComponent(new Integer(2)), new ModelComponent(new Boolean(false)));
	facade.addCompositeRow( new ModelComponent("Nest"), new ModelComponent("nest") );
	facade.addRow(new ModelComponent(""), new ModelComponent("Zakhour"), new ModelComponent("Speed reading"), new ModelComponent(new Integer(20)), new ModelComponent(new Boolean(true)));
	facade.addRow(new ModelComponent("Philip"), new ModelComponent("Milne"), new ModelComponent("Pool"), new ModelComponent(new Integer(10)), new ModelComponent(new Boolean(false)));
	
	facade.addColumn(new ModelComponent("un'altra"));
	facade.addRow(new ModelComponent(new Boolean(false)));
	
	/* Inserisco una riga multivalore */
	facade.addCompositeRow( new ModelComponent("nested1"), new ModelComponent("nested2"), new ModelComponent("a"), new ModelComponent("b" ));
	facade.addCompositeRow( new ModelComponent("nested3"), new ModelComponent("nested4" ));
	
        /* Aggiungo altre righe composite e non */
	ModelComponent provaTooltip = new ModelComponent(new Boolean(true));
	provaTooltip.setTooltip(new HtmlString("Prova tooltip"));
	facade.addRow(provaTooltip);
	facade.addCompositeRow( new ModelComponent("AltroNest"), new ModelComponent("prova 2" ));
	facade.addRow(new ModelComponent(new Boolean(false)));
	
	/* Costruisco le colonne annidate */
	ColumnGroup name = facade.addColumnGroup(new ModelComponent("Name"), 1);
	ColumnGroup second = facade.addColumnGroup(new ModelComponent("Second"), 2, 2);
	facade.addColumnGroup(name, second);
	ColumnGroup language = facade.addColumnGroup(new ModelComponent("Language"), 4);
	ColumnGroup others = facade.addColumnGroup(new ModelComponent("Others"), 5, 2);
	facade.addColumnGroup(language, others);
	
	/* Creo la tabella con il modello costruito */
	JTable table = facade.createJTable();
	
	// Create the scroll pane and add the table to it.
	JScrollPane scrollPane = new JScrollPane(table);

	//Add the scroll pane to this panel.
	JPanel panel = new JPanel();
	panel.add(scrollPane);
	panel.setOpaque(true);

	// Create and set up the window.
	JFrame frame = new JFrame("SimpleTableDemo");
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	frame.setContentPane(scrollPane);

	// Display the window.
	frame.pack();
	frame.setVisible(true);
    }



    public static void printDebugData(JTable table) {
	int numRows = table.getRowCount();
	int numCols = table.getColumnCount();
	javax.swing.table.TableModel model = table.getModel();

	System.out.println("Value of data: ");
	for (int i=0; i < numRows; i++) {
	    System.out.print("    row " + i + ":");
	    for (int j=0; j < numCols; j++) {
		System.out.print("  " + model.getValueAt(i, j));
	    }
	    System.out.println();
	}
	System.out.println("--------------------------");
    }
}
