package org.modelgen.jpf.visualizer.util;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

/**
 * Class <code>HtmlString</code> defines
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class HtmlString implements Cloneable, Serializable {
	private static final long serialVersionUID = 1L;

	@Override
    public Object clone() {
	try {
	    return super.clone();
	} catch (CloneNotSupportedException e) {
	    return null;
	}
    }

    private String text;
    private List<String> htmlTags = new LinkedList<String>();
    private String htmlText = null;

    public HtmlString(String text) {
	this.text = text;
    }
    
    /**
     * Inserisce il tag HTML nella zona piu' interna
     * 
     * nel caso di un tag font color=gray tratta opportunamente
     * la chiusura del tag 
     * @param tag il tag HTML senza i caratteri '&lt;' e '&gt;'
     */
    public void addInnerHtmlTag(String tag) {
	htmlTags.add(0,tag);
	htmlText = null; 
    }
    
    /**
     * Inserisce il tag HTML nella zona piu' esterna
     * 
     * nel caso di un tag font color=gray tratta opportunamente
     * la chiusura del tag 
     * @param tag il tag HTML senza i caratteri '&lt;' e '&gt;'
     */
    public void addOuterHtmlTag(String tag) {
	htmlTags.add(tag);
	htmlText = null;
    }
    
    private String calculateHtmlText() {
	Stack<String> tags = new Stack<String>();
	
	htmlText = "";
	for(String htmlTag : htmlTags) {
	    htmlText += "<"+htmlTag+">";
	    tags.push(htmlTag);
	}
	htmlText += text;
	while(!tags.isEmpty()) {
	    String htmlTag = tags.pop();
	    String[] tagSpaceSeparated = htmlTag.split(" ");
	    String tag = tagSpaceSeparated[0];
	    htmlText += "</" + tag + ">";
	}
	
	return htmlText;
    }
    
    public String getHtmlText(boolean withHtmlTags) {
	if (htmlText == null) {
	    htmlText = this.calculateHtmlText();
	    if (withHtmlTags) {
		htmlText = "<html>" + htmlText + "</html>";
	    }
	}
	return htmlText;
    }
    
    public String getHtmlText() {
	return this.getHtmlText(true);
    }

    @Override
    public String toString() {
	return this.getHtmlText();
    }

    // disponibile per modifiche immediate
    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
	htmlText = null; // si assume che bisogna ricalcolare l'html
    }
    
    
}
