package org.modelgen.jpf.visualizer.util;

import java.util.Collection;
import java.util.Iterator;

/**
 * La classe <code>ReverseIterator</code>
 * definisce un generico iteratore invertito.
 * 
 * La strategia di inversione e' (eccessivamente) semplice:
 * copia la collezione in un'array, su cui itera.
 * 
 * Di conseguenza non e' possibile eliminare elementi
 * tramite {@link #remove()}
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ReverseIterator<E> implements Iterator<E>, Iterable<E> {
    private int valuesIndex;
    private Object[] values;

    public ReverseIterator(Collection<E> collection) {
	values = collection.toArray();
	valuesIndex = values.length - 1;
    }

    public boolean hasNext() {
	return this.valuesIndex >= 0;
    }

    @SuppressWarnings("unchecked")
    public E next() {
	E value = (E) this.values[this.valuesIndex];
	this.valuesIndex--;
	return value;
    }

    /** non elimina nulla */
    public void remove() { /* non elimina nulla */ }

    public Iterator<E> iterator() {
	return this;
    }
    

}
