/* JFlex sintax file for MacroRule language lexer specification */

package org.modelgen.applyrule.reader;

import org.apache.log4j.*;
import org.modelgen.applyrule.*;

import java.io.*; //Exceptions.

%%

%class SimpleRuleReader
%public

/* Scanner customization */
%int
%standalone

%unicode
%ignorecase

/* These will be used to track down errors. */
%line
%column

%init{
	simpleRule = new SimpleRule();
%init}

%{
	private static Category log = Category.getInstance(MacroRuleReader.class.getName());
	private static SimpleRule simpleRule = null;
	
	private void addStatement(String statement, boolean noRollback) {
		simpleRule.addStatement(new Statement(statement, noRollback));
	}
	
	public static SimpleRule read(String filename) throws FileNotFoundException, IOException, Exception {
		simpleRule = new SimpleRule();
		SimpleRuleReader scanner = null;
		FileReader reader = null;

		reader = new FileReader(filename); //Throws IOE, FNFE
		scanner = new SimpleRuleReader(reader);
		while (!scanner.yy_atEOF)
			scanner.yylex(); //Throws Exception
		//Reading has completed.
		simpleRule.setFilename(filename);
		reader.close();
		return simpleRule;
	}
%}

/* Some macro definitions */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]
WhiteSpace     = {LineTerminator} | [ \t\f]

Comment = {TraditionalComment} | {EndOfLineComment} | {DocumentationComment}

TraditionalComment   = "/*" [^*] ~"*/"
EndOfLineComment     = "//" {InputCharacter}* {LineTerminator}
DocumentationComment = "/**" {CommentContent} "*"+ "/"
CommentContent       = ( [^*] | \*+ [^/*] )*

Tail = [^;]* ";"
/*Statement = [^;]* ";"*/
Statement = ("SELECT" | "INSERT" | "DROP" | "UPDATE" | "CREATE" | "ALTER" | "DELETE" ) { Tail }
NoRollbackStatement = "NOROLLBACK " { Statement }

%%

{ Comment }	{ /* Ignore */ }
{ WhiteSpace }	{ /* Ignore */ }

{ NoRollbackStatement } {
	String s = yytext().substring(10).trim(); //This one removes the "NOROLLBACK" part.
	s = CommentRemover.removeComments(s);
	addStatement(s, true);
	log.debug("'No rollback' statement found:\n" + s);
}
{ Statement } {
	String s = yytext().trim();
	s = CommentRemover.removeComments(s);
	addStatement(s, false);
	log.debug("Statement found:\n" + s);
}
. {
	//log.debug("No match found for: " + yytext());
}