/* JFlex sintax file for MacroRule language lexer specification */

/* adattato da: luigi, francesca */

package org.modelgen.regole.reader;

import org.apache.log4j.*;


import java.util.StringTokenizer;

import org.modelgen.regole.*; 
import org.modelgen.regole.Exception.ConditionCreationException;



%%

/* definizione del nome e del modificatore della classe */
%class DatalogRulesReader
%public

/* Scanner customization */
%int

/* crea metodo main. */
%standalone

/*	usa caratteri unicode. */
%unicode
%ignorecase

/* These will be used to track down errors. */
%line
%column

/*	costruttore della classe generata. */
%init{
	regolaDatalog = new RegolaDatalog();
	
	// inizializzo le funzioni di skolem
	// in futuro avvera' dal db
	
	//log.debug("chiamo inizializzazione");
	inizializzazione();
%init}

%eof{
	log.debug("letta la regola datalog nel file: " + nomefile);
	log.debug("stampa della regola letta:");
	log.debug(regolaDatalog);

%eof}

/*	codice copiato "verbatim" nella classe generata */
%{
	private static Category log = Category.getInstance(RegolaDatalog.class.getName());
	private static String nomefile = null;
	
	static String DEST = RegolaDatalog.DEST; 
	static String SOURCE = RegolaDatalog.SOURCE;  
	
	private MappaSkolemFunctorDatalog mappa;
	static private RegolaDatalog regolaDatalog;
	SkolemFunctor skfd;  
	ValueFunction vf;
	boolean isInTesta, isTargetSkolem, isLetteraleNegato, applyDistinct, isLetteraleInstance;
	String predicateName, etichetta, schemaConstraint;
	String costante, variabile, nomeFunzione;
	Letterale letterale = null;
	
	String condizione1, condizione2, operatore;
	
	
	public void inizializzaVariabili() {
		schemaConstraint = null;
		isLetteraleNegato = false;
		condizione1 = null;
		condizione2 = null;
		operatore = null;
	}
	
	
	public void inizializzazione() {
	
		mappa = new MappaSkolemFunctorDatalog();
		
	}
	
	
	/*	stampa le funzioni a disposizione (nella mappa) */
	/*
	private void stampaFunzioni() {
		if (!skolemFunction.isEmpty()) {
				Iterator i = skolemFunction.keySet().iterator();
				while (i.hasNext()) {
					System.out.println(skolemFunction.get(i.next()));
				}
			} 
	}*/
	
	/*	metodo per leggere il file "filename" contenente una regola datlog. */
	static public  RegolaDatalog read(String filename) throws Exception{
		nomefile = filename;
	        DatalogRulesReader scanner = null;
	        try {
	          scanner = new DatalogRulesReader( new java.io.FileReader(filename) );
	          while ( !scanner.zzAtEOF ) scanner.yylex(); 
	        }
	        catch (java.io.FileNotFoundException e) {
	          System.out.println("File not found : \""+filename+"\"");
	          throw e;
	        }
	        catch (java.io.IOException e) {
	          System.out.println("IO error scanning file \""+filename+"\"");
	          System.out.println(e);
	          throw e;
	        }
	        catch (Exception e) {
	          System.out.println("Unexpected exception:");
	          e.printStackTrace();
	          throw e;
	        } 
	        // RegolaDatalog regolaDatalog = null;
	        return regolaDatalog;
	      }

	
%}

/* Some macro definitions */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]
WhiteSpace     = {LineTerminator} | [ \t\f]

Comment = {TraditionalComment} | {EndOfLineComment} | {DocumentationComment}

TraditionalComment   = "/*" [^*] ~"*/"
EndOfLineComment     = "//" {InputCharacter}* {LineTerminator}
DocumentationComment = "/**" {CommentContent} "*"+ "/"
CommentContent       = ( [^*] | \*+ [^/*] )*

String = '? [:jletterdigit:] [:jletterdigit:]* '?
//String = {WhiteSpace}* '? [a-zA-Z] [a-zA-Z0-9_]* '? {WhiteSpace}*


PredicateName = "!"? {WhiteSpace}*{String}{WhiteSpace}*("["|"(")
SchemaConstraint = "[dest]"| "[source]" | "[dest_" {String} "]" | "[source_" {String} "]"
Label = {String} {WhiteSpace}* (":")
Variabile = {String}
Costante = "\""{String}"\""
Funtore = "#" {String} "*"? {WhiteSpace}* "("	
ValueFunction = "@" {String} {WhiteSpace}* "("  
Operatore = [<>=] | "<>"
Distinct = "<*-"
// nell'argomento composto non e' attualmente supportato il funtore
ArgomentoComposto = ({Variabile}|{Costante}) ({WhiteSpace}* "+" {WhiteSpace}* ({Variabile}|{Costante}))+

/*	stati dell'automa */


%state ARGOMENTOETICHETTATO SCHEMACONSTRAINT ARGOMENTO
%state ENDARGOMENTOETICHETTATO END ENDARGOMENTOFUNTORE
%state ARGOMENTOFUNTORE LETTERALE SECONDOMEMBROCONDIZIONE
%state ARGOMENTOVALUE ENDARGOMENTOVALUE 
// modifica LF


%%

{ Comment }	{ /* Ignore */ }
{ WhiteSpace }	{ /* Ignore */ }

<YYINITIAL> {
. {
	// inizializzo le variabili che uso nel parser
	inizializzaVariabili();
	
	// inizio  a leggere la regola datalog
	//log.debug("inizio a leggere la regola datalog");

	// ho iniziato: devo leggere la testa della regola
	// uso una var isInTesta per specificare che mi trovo
	// nella testa della regola
	isInTesta = true;

	//log.debug("devo leggere la testa: variabile testa = '" + isInTesta + "'");
	
	log.debug("leggo la regola datalog nel file: " + nomefile);
	yypushback(1);
	yybegin(LETTERALE);
} // chiudo .
} // chiudo <YYINITIAL>

<LETTERALE> {

{PredicateName}  {
	// il predicate name e' il testo appena letto
	predicateName = yytext().trim();
	
	isLetteraleNegato = false;
	if (predicateName.startsWith("!")) {
		predicateName = predicateName.substring(1);
		isLetteraleNegato = true;
		
	}
	if(predicateName.startsWith("i_"))
		isLetteraleInstance = true;
	else
		isLetteraleInstance = false;
	
	predicateName = predicateName.substring(0, predicateName.length()-1);
	predicateName = predicateName.trim();
	
	
	//dopo il predicate name devo leggere gli argomenti etichettati del letterale
	yypushback(1);
	yybegin(SCHEMACONSTRAINT);
} // chiudo PredicateName

{Variabile}{WhiteSpace}*{Operatore} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	if (isInTesta) {
		log.error("ERROR: le condizioni non possono essere espresse nella testa della regola.");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("definite condizioni nella testa'"+yytext()+"'");
	}
	
	//log.debug("trovata variabile lato sinistro in una condizione");
	condizione1 = yytext().trim();
	//log.debug("condizione1: '" + condizione1 + "'");	
	if (condizione1.endsWith("<>")) {
		condizione1 = condizione1.substring(0,condizione1.length()-2);
		yypushback(2);
	} else {
		condizione1 = condizione1.substring(0,condizione1.length()-1);
		yypushback(1);
	}
	condizione1 = condizione1.trim();
	
	yybegin(SECONDOMEMBROCONDIZIONE);
} // chiudo Variabile

{Costante}{WhiteSpace}*{Operatore} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	if (isInTesta) {
		log.error("ERROR: le condizioni non possono essere espresse nella testa della regola.");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("definite condizioni nella testa'"+yytext()+"'");
	}
	
	//log.debug("trovata costante lato sinistro in una condizione");
	condizione1 = yytext().trim();
	//log.debug("condizione1: '" + condizione1 + "'");
	
	if (condizione1.endsWith("<>")) {
		condizione1 = condizione1.substring(0,condizione1.length()-2); //modificato fl
		yypushback(2);
	}
	else {
		condizione1 = condizione1.substring(0,condizione1.length()-1); // modificato fl
		yypushback(1);
	}
	condizione1 = condizione1.trim();
	condizione1 = condizione1.substring(1,condizione1.length()-1);
	
	yybegin(SECONDOMEMBROCONDIZIONE);	
} // chiudo Costante

. {
	log.error("ERROR: non ho trovato un 'predicate name' o una condizione"
		+ " o l'operatore per la condizione");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <LETTERALE>

<SECONDOMEMBROCONDIZIONE> {
{Operatore} {
	operatore = yytext().trim();
} // chiudo Operatore
{Variabile} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	//log.debug("trovata variabile lato sinistro in una condizione");
	condizione2 = yytext().trim();
	//log.debug("condizione2: '" + condizione2 + "'");
	
	if (operatore == null) {
		log.error("ERROR: manca l'operatore");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("manca l'operatore. '"+yytext()+"'");
	}
	
	try {
		regolaDatalog.aggiungiCondizione(condizione1,condizione2,operatore);
	} catch(ConditionCreationException ex) {
		throw new Error(ex.getMessage());
	}
	
	
	inizializzaVariabili();
	yybegin(END);
} // chiudo Variabile

{Costante} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	// siccome le condizioni possono essere definite solo tra variabili
	// se ho una costante viene generata una condizione fittizia sempre vera.
	
	
	//log.debug("trovata costante lato sinistro in una condizione");
	condizione2 = yytext().trim();
	condizione2 = condizione2.substring(1,condizione2.length()-1);
	//log.debug("condizione2: '" + condizione2 + "'");

	if (operatore == null) {
		log.error("ERROR: manca l'operatore");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("manca l'operatore. '"+yytext()+"'");
	}
	
	try {
		regolaDatalog.aggiungiCondizione(condizione1,condizione2,operatore);
	} catch(ConditionCreationException ex) {
		throw new Error(ex.getMessage());
	}
	
	
	
	inizializzaVariabili();
	yybegin(END);	
} // chiudo Costante
. {
	log.error("ERROR: non ho trovato il secondo membro della condizione");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("manca il secondo membro della condizione. '"+yytext()+"'");
} // chiudo .
} // chiudo <SECONDOMEMBROCONDIZIONE>

<SCHEMACONSTRAINT> {
{SchemaConstraint} {
	//log.debug("trovato lo schema constraint");
	schemaConstraint = yytext().trim();
	//log.debug("lo schema constraint e': '" + schemaConstraint + "'");
} // chiudo SchemaConstraint
"(" {
	if (schemaConstraint==null) {
		//log.debug("lo schema constraint non specificato nel letterale.");
		if (isInTesta) {
			//log.debug("sono nella testa; imposto schema constraint a [dest]");
			schemaConstraint = DEST;
			
		} else {
			//log.debug("sono nel corpo; imposto schema constraint a [source]");
			schemaConstraint = SOURCE;
			
		} 
	} //else log.debug("schemaConstraint esplicitato nel letterale.");
	//log.debug("schemaConstraint: '" + schemaConstraint + "'");
	// inizializzo il letterale

	
		if(!isLetteraleNegato) 
			letterale = new Letterale(predicateName, schemaConstraint);
		else
			letterale = new LetteraleNegato(predicateName, schemaConstraint);
			
		letterale.setIsNegato(isLetteraleNegato);
		letterale.setIsInTesta(isInTesta);
		letterale.setIsInstance(isLetteraleInstance);



	yybegin(ARGOMENTOETICHETTATO);
} // chiudo "("
. {
	log.error("ERROR: specificare [dest] o [source] oppure mancano gli"
		+ " argomenti del letterale.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <SCHEMACONSTRAINT>

<ARGOMENTOETICHETTATO> {
	// mi aspetto di trovare delle coppie etichetta argomento
{Label} {
		// ho trovato una etichetta
		//log.debug("ho trovato una etichetta");
		etichetta = yytext().trim();
		etichetta = etichetta.substring(0,etichetta.length()-1);
		etichetta = etichetta.trim();
		//log.debug("l'etichetta e': '" + etichetta + "'");
		yypushback(1);
} // chiudo Etichetta
":" { yybegin(ARGOMENTO); }
. {
	log.error("ERROR: non trovo l'etichetta o mancano i ':'");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} //chiudo <ARGOMENTOETICHETTATO>

<ARGOMENTO> {
{ArgomentoComposto} {
	log.debug("trovato un argomento composto!!!");
	String argomentoComposto = yytext();
	log.debug("ecco cosa ho letto: " + argomentoComposto);
	StringTokenizer st = new StringTokenizer(argomentoComposto,"+");
	String app = st.nextToken().trim();
	
	Manipolabile ad = null;
	if (app.startsWith("\"")) {
		app = app.substring(1, app.length()-1);
		ad = new CostanteDatalog(app);
	} else ad = regolaDatalog.getVariabile(app,letterale,false);  
	log.debug(app);
	ArgomentoComposto adc = new ArgomentoComposto();  
	adc.addMembro(ad);  
	while (st.hasMoreTokens()) {
		app = st.nextToken().trim();
		
		if (app.startsWith("\"")) {
			app = app.substring(1, app.length()-1);
			ad = new CostanteDatalog(app);
		} else ad = regolaDatalog.getVariabile(app,letterale,false); 
		log.debug(app);
		adc.addMembro(ad);  
	}
	// costruisco l'argomento etichettato
	letterale.inserisciArgomentoEtichettato(etichetta,adc);  


	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	
	// aggiungo l'argomento letto al letterale

	yybegin(ENDARGOMENTOETICHETTATO);
}
{Variabile} {
	//log.debug("trovata una variabile");
	variabile = yytext().trim();
	//log.debug("variabile: '" + variabile + "'");

	// costruisco l'argomento etichettato

	letterale.inserisciArgomentoEtichettato(etichetta,regolaDatalog.getVariabile(variabile,letterale,false)); 
	
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	
	
	// leggo il successivo argomento etichettato
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo Variabile
{Costante} {
	//log.debug("trovata una costante");
	costante = yytext().trim();
	costante = costante.substring(1, costante.length()-1);
	//log.debug("costante: '" + costante + "'");
	
 
		letterale.inserisciArgomentoEtichettato(etichetta, new CostanteDatalog(costante)); 
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	
	// leggo il successivo argomento etichettato
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo Costante

{ValueFunction} { // modifica LF
	nomeFunzione = yytext().trim();
	vf = new ValueFunction();
	yybegin(ARGOMENTOVALUE);
	
} //chiudo funzione
{Funtore} {
	//log.debug("trovato un funtore");
	
	letterale.setHasSkolem(true); 
	
	nomeFunzione = yytext().trim();
	nomeFunzione = nomeFunzione.substring(1, nomeFunzione.length()-1); // toglie il #
	nomeFunzione = nomeFunzione.trim(); 
	
	
	isTargetSkolem = false;
	if (nomeFunzione.endsWith("*")) {
		nomeFunzione = nomeFunzione.substring(0,nomeFunzione.length()-1);
		isTargetSkolem = true;
	}
	//log.debug("funtore: '" + nomeFunzione + "'");
	//log.debug("il funtore e'designato come target skolem: '" + isTargetSkolem + "'");
	
	// recupero la funzione dalla mappa, costruita leggendo dalla base di dati
	SkolemFunction skfn = null;
	
	boolean instance = false;
	
	if(nomeFunzione.toLowerCase().startsWith("i_"))	{
		skfn = ((SkolemFunction)mappa.getFunzione(nomeFunzione.substring(2))).getFunzionePerIstanza(); // carica la funzione per le istanze
		instance = true;
	} else
		skfn = (SkolemFunction)mappa.getFunzione(nomeFunzione); 
	
	
	if (skfn != null) skfd = new SkolemFunctor(skfn, isTargetSkolem); 	
	
	else {
		log.error("ERROR: nome funzione non trovato tra le disponibili"
			+ " nella base di dati");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("funzione di skolem non disponibile. '"+yytext()+"'");
	}
	// leggo gli argomenti del funtore
	yybegin(ARGOMENTOFUNTORE);

} // chiudo Funtore
. {
	log.error("ERROR: non trovo una variabile, una costante o un funtore.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .

} // chiudo <ARGOMENTO>






<ARGOMENTOVALUE> {
{Variabile} {
	// e' un argomento di tipo variabile
	String argomentoFunzione = yytext().trim();
	vf.addVariabile(regolaDatalog.getVariabile(argomentoFunzione, letterale,false)); 
	yybegin(ENDARGOMENTOVALUE);
} // chiudo Variabile

")" {
	//log.debug("trovata chiusura argomenti della funzione.");
	letterale.inserisciArgomentoEtichettato(etichetta,vf);
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"

. {
	log.error("ERROR: errore negli argomenti della funzione oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <ARGOMENTOVALUE>








<ARGOMENTOFUNTORE> {
{Variabile} {
	// e' un argomento di tipo variabile
	String argomentoFuntore = yytext().trim();
	skfd.addArgomento(regolaDatalog.getVariabile(argomentoFuntore, letterale,true)); 
	skfd.setHaAlmenoUnaVariabile(true);
	yybegin(ENDARGOMENTOFUNTORE);
} // chiudo Variabile
{Costante} {
	// e' un argomento di tipo costante
	String argomentoFuntore = yytext().trim();
    argomentoFuntore = argomentoFuntore.substring(1,argomentoFuntore.length()-1); 
	skfd.addArgomento(new CostanteDatalog(argomentoFuntore)); 

	yybegin(ENDARGOMENTOFUNTORE);
} // chiudo Costante
")" {
	//log.debug("trovata chiusura argomenti del funtore.");
	//log.debug("WARNING:  il funtore e' terminato senza argomenti");
	log.warn("WARNING:  il funtore e' terminato senza argomenti");
	letterale.inserisciArgomentoEtichettato(etichetta,skfd);
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"

. {
	log.error("ERROR: errore negli argomenti del funtore oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <ARGOMENTOFUNTORE>






<ENDARGOMENTOVALUE> { // modifica LF
"," {
	//log.debug("trovata virgola: mi aspetto un altro argomento per la funzione");
	yybegin(ARGOMENTOVALUE);
} // chiudo ","
")" {
	//log.debug("trovata chiusura argomenti del funtore: il funtore e' terminato");

	letterale.inserisciArgomentoEtichettato(etichetta,vf); 
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");

	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"
. {
	log.error("ERROR: errore negli argomenti della funzione oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}
} // chiudo ENDARGOMENTOFUNZIONE










<ENDARGOMENTOFUNTORE> {
"," {
	//log.debug("trovata virgola: mi aspetto un altro argomento per il funtore");
	yybegin(ARGOMENTOFUNTORE);
} // chiudo ","
")" {
	//log.debug("trovata chiusura argomenti del funtore: il funtore e' terminato");

	letterale.inserisciArgomentoEtichettato(etichetta,skfd); 
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");

	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"
. {
	log.error("ERROR: errore negli argomenti del funtore oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}
} // chiudo ENDARGOMENTOFUNTORE

<ENDARGOMENTOETICHETTATO> {
"," {
	//log.debug("trovata virgola: mi aspetto un altro argomento etichettato");
	yybegin(ARGOMENTOETICHETTATO);
} // chiudo ","
")" {
	//log.debug("trovata chiusura argomenti etichettati: il letterale e' terminato");
	if (isInTesta) regolaDatalog.setTesta(letterale);  
	else regolaDatalog.addElementoCorpo(letterale);  
	yybegin(END);
} // chiudo ")"

. {
	log.error("ERROR: errore a fine argomento: mi aspetto una ',' oppure una ')'.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}
} // chiudo ENDARGOMENTOETICHETTATO

<END> {
"<-" | {Distinct} {
	if (!isInTesta) {
		log.error("ERROR: troppi '<-'");
		log.error("INVALID SINTAX in file. Error at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	}
	// la testa e' terminata; ora leggo il corpo
	//log.debug("testa terminata; leggo il corpo"); 
	isInTesta = false;
	String testoLetto = yytext();
	
	if (testoLetto.equals("<*-")) {
		log.debug("impostato DISTINCT");
		applyDistinct = true;
	} else applyDistinct = false;
	regolaDatalog.setIsApplyDistinct(applyDistinct);  
	
	inizializzaVariabili();
	yybegin(LETTERALE);
}
"," {
	if (isInTesta) {
		log.error("ERROR: nella testa e' ammesso un solo letterale");
		log.error("INVALID SINTAX: troppi letterali nella testa della regola. " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	}
	//log.debug("trovata virgola: mi aspetto un altro letterale");
	inizializzaVariabili();
	yybegin(LETTERALE);
}
";" {
	if (isInTesta) {
		log.error("ERROR: manca il corpo della regola");
		log.error("INVALID SINTAX: manca il corpo della regola. " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	}
	//log.debug("trovato ';': la regola e' terminata!");
}

. {
	log.error("ERROR: mi aspetto un '<-', un ',' oppure un ';'.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}

} // chiudo END

. {
	log.error("letto: " + yytext());
	log.error("INVALID SINTAX in file. Error at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	//log.error("STATE: " + yystate());
	throw new Error("INVALID SINTAX in file. Error at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
}