/* JFlex sintax file for ParametricDatalogRule language lexer specification */

/* adattato da: luigi, francesca */

package org.modelgen.regole.reader;

import org.apache.log4j.*;

import java.util.List;

import org.modelgen.regole.*;
import org.modelgen.regole.Exception.ConditionCreationException;
import org.jconfig.Configuration; 
import org.jconfig.ConfigurationManager; 


%%

/* definizione del nome e del modificatore della classe */
%class ParametricDatalogRulesReader
%public

/* Scanner customization */
%int

/* crea metodo main. */
%standalone

/*	usa caratteri unicode. */
%unicode
%ignorecase

/* These will be used to track down errors. */
%line
%column

/*	costruttore della classe generata. */
%init{
	
	macroRegola = new MacroRegola();
	inizializzazione();
	inizializzaVariabiliSingolaRPD();
	inizializzaVariabili();
	
%init}

%eof{
	log.debug("raggiunta la fine del file: " + nomefile);

	log.debug("STAMPO la MacroRegolaDatalogParametrica");
	System.out.println(macroRegola);
%eof}

/*	codice copiato "verbatim" nella classe generata */
%{
	
	private static Category log = Category.getInstance(ParametricDatalogRulesReader.class.getName());
	private Configuration applyRuleConfig = ConfigurationManager.getConfiguration("modelgen");
	private static String nomefile = null;
	
	// variabile di appoggio durante la lettura
	static String DEST = RegolaDatalog.DEST;
	static String SOURCE = RegolaDatalog.SOURCE;
	
	private MappaSkolemFunctorDatalog mappa;
	private RegolaDatalog regolaDatalog;
	SkolemFunctor skfd;
	boolean mustIterate, isTargetSkolem, isLetteraleNegato;
	String predicateName, etichetta, schemaConstraint;
	String costante, variabile, nomeFunzione;
	Letterale letterale = null;	
	String condizione1, condizione2, operatore;
	
	List listaParametri;
	
	String simpleFileName;
		
	// variabili per istanziare una macro di regole datalog parametriche
	private static MacroRegola macroRegola;
	private String macroFileName;
	
	
	public void inizializzazione() {
		mappa = new MappaSkolemFunctorDatalog();	
	}
	
	public void inizializzaVariabili() {
		schemaConstraint = null;
		isLetteraleNegato = false;
		condizione1 = null;
		condizione2 = null;
		operatore = null;
	}
	
	private void inizializzaVariabiliSingolaRPD() {
		
		mustIterate = false;
	}	
	
	/*	metodo per leggere il file "filename" contenente una regola datlog. */
	static public  MacroRegola read(String filename) {
		nomefile = filename;
	        ParametricDatalogRulesReader scanner = null;
	        try {
	          scanner = new ParametricDatalogRulesReader( new java.io.FileReader(filename) );
	          while ( !scanner.zzAtEOF ) scanner.yylex(); // obsoleto yy_atEOF correggere in JFlex 1.4.1	          
	        }
	        catch (java.io.FileNotFoundException e) {
	          System.out.println("ERROR: File not found : \""+filename+"\" ERROR");
	        }
	        catch (java.io.IOException e) {
	          System.out.println("ERROR: IO error scanning file \""+filename+"\" ERROR");
	          System.out.println(e);
	        }
	        catch (Exception e) {
	          System.out.println("ERROR: Unexpected exception: ERROR");
	          e.printStackTrace();
	        }
	        return macroRegola;
	      }
%}

/* Some macro definitions */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]
WhiteSpace     = {LineTerminator} | [ \t\f]

Comment = {TraditionalComment} | {EndOfLineComment} | {DocumentationComment}

TraditionalComment   = "/*" [^*] ~"*/"
EndOfLineComment     = "//" {InputCharacter}* {LineTerminator}
DocumentationComment = "/**" {CommentContent} "*"+ "/"
CommentContent       = ( [^*] | \*+ [^/*] )*


/* grammatica della regola datalog. */

FileName = ( "/"? {String} ) ( ([-_./]) {String} )* {WhiteSpace}*("(")
PredicateName = "!"? {WhiteSpace}*{String}{WhiteSpace}*("["|"(")
SchemaConstraint = "[dest]"| "[source]"
Label = {String} {WhiteSpace}* (":")
Variabile = {String}
Costante = "\""{String}"\""
Funtore = "#" {String} "*"? {WhiteSpace}* "("
Operatore = [<>=] | "<>"

String = '? [:jletterdigit:] [:jletterdigit:]* '?
//String = {WhiteSpace}* [a-zA-Z] [a-zA-Z0-9_]* {WhiteSpace}*

/*	stati dell'automa */
%state ARGOMENTOETICHETTATO SCHEMACONSTRAINT ARGOMENTO
%state ENDARGOMENTOETICHETTATO END ENDARGOMENTOFUNTORE
%state ARGOMENTOFUNTORE LETTERALE SECONDOMEMBROCONDIZIONE PRELETTERALE
%%

{ Comment }	{ /* Ignore */ }
{ WhiteSpace }	{ /* Ignore */ }

/*	automa */

<YYINITIAL> {
. {
	log.debug("leggo la macro-regola datalog nel file: " + nomefile);
	inizializzaVariabiliSingolaRPD();
	inizializzaVariabili();
	yypushback(1);
	yybegin(PRELETTERALE);
}
}

<PRELETTERALE> {
	"!!" {
		//log.debug("ITERA");
		if (mustIterate) log.debug("troppi !!!. ma quanto devi iterare?");
		else mustIterate = true;
	}
	{FileName} {
		//log.debug("trovato simpleFileName");
		simpleFileName = yytext().trim();
		simpleFileName = simpleFileName.substring(0,simpleFileName.length()-1);
		log.debug("nome file regola datalog da leggere: '" + simpleFileName + "'");
		
		simpleFileName = applyRuleConfig.getProperty("rootnode","","paths") + applyRuleConfig.getProperty("simplerules","","paths") + simpleFileName;
		
		try {
		    regolaDatalog = DatalogRulesReader.read(simpleFileName); // modificatoLF
		    // la linea successiva e' stata aggiunta a manina, anche sul .java
		    // che viene generato da questo flex. Sarebbe da controllare che
		    // la generazione sia coerente
		    regolaDatalog.setMustIterate(mustIterate);
		} catch(Exception ex) { }
		
		//listaParametri = new LinkedList();
		// modificatoLF
		yybegin(LETTERALE);
	}
	";" {
		// ignora
	}
}	// chiudo PRELETTERALE

<LETTERALE> {
{PredicateName}  {
	// il predicate name e' il testo appena letto
	predicateName = yytext().trim();
	
	isLetteraleNegato = false;
	if (predicateName.startsWith("!")) {
		predicateName = predicateName.substring(1);
		isLetteraleNegato = true;
		//log.debug("sul letterale e' specificata la negazione");
	}
	predicateName = predicateName.substring(0, predicateName.length()-1);
	predicateName = predicateName.trim();
	//log.debug("il predicate name e': '" + predicateName + "'");
	
	//dopo il predicate name devo leggere gli argomenti etichettati del letterale
	yypushback(1);
	yybegin(SCHEMACONSTRAINT);
} // chiudo PredicateName

")" {
	log.debug("nessun parametro per la regola: '" + simpleFileName + "'");
	yypushback(1);
	yybegin(END);
}

{Variabile}{WhiteSpace}*{Operatore} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	//log.debug("trovata variabile lato sinistro in una condizione");
	condizione1 = yytext().trim();
	//log.debug("condizione1: '" + condizione1 + "'");	
	if (condizione1.endsWith("<>")) {
		condizione1 = condizione1.substring(0,condizione1.length()-2);
		yypushback(2);
	} else {
		condizione1 = condizione1.substring(0,condizione1.length()-1);
		yypushback(1);
	}
	condizione1 = condizione1.trim();
	
	yybegin(SECONDOMEMBROCONDIZIONE);
} // chiudo Variabile

{Costante}{WhiteSpace}*{Operatore} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	//log.debug("trovata costante lato sinistro in una condizione");
	condizione1 = yytext().trim();
	//log.debug("condizione1: '" + condizione1 + "'");
	
	if (condizione1.endsWith("<>")) {
		condizione1 = condizione1.substring(0,condizione1.length()-2);
		yypushback(2);
	}
	else {
		condizione1 = condizione1.substring(0,condizione1.length()-1);
		yypushback(1);
	}
	condizione1 = condizione1.trim();
	condizione1 = condizione1.substring(1,condizione1.length()-1); // vengono tolti gli apici
	
	yybegin(SECONDOMEMBROCONDIZIONE);	
} // chiudo Costante

. {
	log.error("ERROR: non ho trovato un 'predicate name' o una condizione"
		+ " o l'operatore per la condizione");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <LETTERALE>

<SECONDOMEMBROCONDIZIONE> {
{Operatore} {
	operatore = yytext().trim();
} // chiudo Operatore
{Variabile} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	//log.debug("trovata variabile lato sinistro in una condizione");
	condizione2 = yytext().trim();
	//log.debug("condizione2: '" + condizione2 + "'");
	
	if (operatore == null) {
		log.error("ERROR: manca l'operatore");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("manca l'operatore. '"+yytext()+"'");
	}

	try {
		regolaDatalog.aggiungiCondizione(condizione1,condizione2,operatore);
	} catch(ConditionCreationException ex) {
		throw new Error(ex.getMessage());
	} 
	
	

	inizializzaVariabili();
	yybegin(END);
} // chiudo Variabile

{Costante} {
	// leggo eventuali condizioni espresse fuori dai letterali.
	// possono esistere esculisavamente nel corpo della regola
	// es.: OID1 > OID2.
	// gli operatori ammessi sulle condizioni aggiuntive sono:
	// <, >, <>, =
	// gli operandi possono essere variabili o costanti
	
	//log.debug("trovata costante lato sinistro in una condizione");
	condizione2 = yytext().trim();
	condizione2 = condizione2.substring(1,condizione2.length()-1); // vengono tolti gli apici
	//log.debug("condizione2: '" + condizione2 + "'");

	if (operatore == null) {
		log.error("ERROR: manca l'operatore");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("manca l'operatore. '"+yytext()+"'");
	}
	
	try {
		regolaDatalog.aggiungiCondizione(condizione1,condizione2,operatore);
	} catch(ConditionCreationException ex) {
		throw new Error(ex.getMessage());
	}
	
	
	
	inizializzaVariabili();
	yybegin(END);	
} // chiudo Costante
. {
	log.error("ERROR: non ho trovato il secondo membro della condizione");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("manca il secondo membro della condizione. '"+yytext()+"'");
} // chiudo .
} // chiudo <SECONDOMEMBROCONDIZIONE>

<SCHEMACONSTRAINT> {
{SchemaConstraint} {
	//log.debug("trovato lo schema constraint");
	schemaConstraint = yytext().trim();
	//log.debug("lo schema constraint e': '" + schemaConstraint + "'");
} // chiudo SchemaConstraint
"(" {
	if (schemaConstraint==null) {
		//log.debug("lo schema constraint non specificato nel letterale.");
		//log.debug("sono nel corpo; imposto schema constraint a [source]");
		schemaConstraint = SOURCE;
		//log.debug("schemaConstraint: '" + schemaConstraint + "'"); 
	} //else log.debug("schemaConstraint esplicitato nel letterale.");
	// inizializzo il letterale
	if (!isLetteraleNegato) {
		letterale = new Letterale(predicateName, schemaConstraint);
	} else {
		letterale = new LetteraleNegato(predicateName, schemaConstraint); 
	}
	
	letterale.setIsNegato(isLetteraleNegato);
	letterale.setIsInTesta(false); 
	
	yybegin(ARGOMENTOETICHETTATO);
} // chiudo "("
. {
	log.error("ERROR: specificare [dest] o [source] oppure mancano gli"
		+ " argomenti del letterale.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <SCHEMACONSTRAINT>

<ARGOMENTOETICHETTATO> {
	// mi aspetto di trovare delle coppie etichetta argomento
{Label} {
		// ho trovato una etichetta
		//log.debug("ho trovato una etichetta");
		etichetta = yytext().trim();
		etichetta = etichetta.substring(0,etichetta.length()-1);
		etichetta = etichetta.trim();
		//log.debug("l'etichetta e': '" + etichetta + "'");
		yypushback(1);
} // chiudo Etichetta
":" { yybegin(ARGOMENTO); }
. {
	log.error("ERROR: non trovo l'etichetta o mancano i ':'");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} //chiudo <ARGOMENTOETICHETTATO>

<ARGOMENTO> {
{Variabile} {
	//log.debug("trovata una variabile");
	variabile = yytext().trim();
	//log.debug("variabile: '" + variabile + "'");

	// costruisco l'argomento etichettato
	/*ArgomentoEtichettato argomentoEtichettato = 
		new ArgomentoEtichettato(etichetta, new VariabileDatalog(variabile)); */
    letterale.inserisciArgomentoEtichettato(etichetta, regolaDatalog.getVariabile(variabile,letterale,false)); 
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	
	// aggiungo l'argomento letto al letterale
	//letterale.addArgument(argomentoEtichettato);
	
	// leggo il successivo argomento etichettato
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo Variabile
{Costante} {
	//log.debug("trovata una costante");
	costante = yytext().trim();
	costante = costante.substring(1, costante.length()-1);
	//log.debug("costante: '" + costante + "'");
	
	// costruisco l'argomento etichettato

	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");

	// aggiungo l'argomento letto al letterale

	letterale.inserisciArgomentoEtichettato(etichetta, new CostanteDatalog(costante)); 
	
	
	// leggo il successivo argomento etichettato
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo Costante
{Funtore} {
	//log.debug("trovato un funtore");
	nomeFunzione = yytext().trim();
	nomeFunzione = nomeFunzione.substring(1, nomeFunzione.length()-1);
	nomeFunzione = nomeFunzione.trim();
	isTargetSkolem = false;


	//log.debug("funtore: '" + nomeFunzione + "'");
	//log.debug("funtore isTargetSkolem: '" + isTargetSkolem + "'");
	
	// recupero la funzione dalla mappa, costruita leggendo dalla base di dati
	SkolemFunction skfn = (SkolemFunction)mappa.getFunzione(nomeFunzione);
	if (skfn != null) skfd = new SkolemFunctor(skfn, isTargetSkolem); //modificatoLF
	else {
		log.error("ERROR: nome funzione non trovato tra le disponibili"
			+ " nella base di dati");
		log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
		throw new Error("funzione di skolem non disponibile. '"+yytext()+"'");
	}
	// leggo gli argomenti del funtore
	yybegin(ARGOMENTOFUNTORE);
	// costruisco l'argomento etichettato

		
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
} // chiudo Funtore
. {
	log.error("ERROR: non trovo una variabile, una costante o un funtore.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <ARGOMENTO>

<ARGOMENTOFUNTORE> {
{Variabile} {
	// e' un argomento di tipo variabile
	String argomentoFuntore = yytext().trim();
	skfd.addArgomento(regolaDatalog.getVariabile(argomentoFuntore, letterale, true));
	yybegin(ENDARGOMENTOFUNTORE);
} // chiudo Variabile
{Costante} {
	// e' un argomento di tipo costante
	String argomentoFuntore = yytext().trim();
	skfd.addArgomento(new CostanteDatalog(argomentoFuntore)); 
	yybegin(ENDARGOMENTOFUNTORE);
} // chiudo Costante
")" {
	//log.debug("trovata chiusura argomenti del funtore.");
	//log.debug("WARNING:  il funtore e' terminato senza argomenti");
	log.warn("WARNING:  il funtore e' terminato senza argomenti");

	letterale.inserisciArgomentoEtichettato(etichetta,skfd); //modificatoLF
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
	//letterale.addArgument(argomentoEtichettato);
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"

. {
	log.error("ERROR: errore negli argomenti del funtore oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
} // chiudo .
} // chiudo <ARGOMENTOFUNTORE>

<ENDARGOMENTOFUNTORE> {
"," {
	//log.debug("trovata virgola: mi aspetto un altro argomento per il funtore");
	yybegin(ARGOMENTOFUNTORE);
} // chiudo ","
")" {
	//log.debug("trovata chiusura argomenti del funtore: il funtore e' terminato");

	// costruisco l'argomento etichettato

	letterale.inserisciArgomentoEtichettato(etichetta, skfd); 
	//log.debug("Argomento Etichettato: '" + argomentoEtichettato + "'");
    //letterale.addArgument(argomentoEtichettato);
	yybegin(ENDARGOMENTOETICHETTATO);
} // chiudo ")"
. {
	log.error("ERROR: errore negli argomenti del funtore oppure non termina"
		+ " correttamente.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}
} // chiudo ENDARGOMENTOFUNTORE

<ENDARGOMENTOETICHETTATO> {
"," {
	//log.debug("trovata virgola: mi aspetto un altro argomento etichettato");
	yybegin(ARGOMENTOETICHETTATO);
} // chiudo ","
")" {
	//log.debug("trovata chiusura argomenti etichettati: il letterale e' terminato");
	regolaDatalog.addElementoCorpo(letterale); 
	yybegin(END);
} // chiudo ")"

. {
	log.error("ERROR: errore nel funtore; mi aspetto una ',' oppure una ')'.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}
} // chiudo ENDARGOMENTOETICHETTATO

<END> {
"," {
	//log.debug("trovata virgola: mi aspetto un altro letterale");
	inizializzaVariabili();
	yybegin(LETTERALE);
}
")" {WhiteSpace}* ";" {	
	//log.debug("trovato terminatore di parametri per la regola: " + simpleFileName);
	//log.debug("la regola ha i seguenti parametri: " + listaParametri);
	
	
	log.debug("regola letta: ");

	macroRegola.addRegola(regolaDatalog); //modificatoLF
	inizializzaVariabiliSingolaRPD();
	yybegin(PRELETTERALE);
}

. {
	log.error("ERROR: mi aspetto un '<-', un ',' oppure un ';'.");
	log.error("at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	throw new Error("Illegal character '"+yytext()+"'");
}

} // chiudo END

. {
	log.error("letto: " + yytext());
	log.error("INVALID SINTAX in file. Error at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
	//log.error("STATE: " + yystate());
	throw new Error("INVALID SINTAX in file. Error at line " + (yyline + 1) + ", column " + (yycolumn + 3) + ".");
}
