/*
 * Created on 10-dic-2003
 *
 */
package org.modelgen.applyrule;

import org.modelgen.applyrule.reader.MacroRuleReader;
import org.modelgen.commons.utils.ModelgenConfigurationManager;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.LinkedList;

import org.apache.log4j.Logger;

/**
 * Main class of the ApplyRule tool.<p>
 * The usage is: java -jar ApplyRule.jar &lt;macroRule_filename&gt; &lt;schemeOID1&gt; &lt;schemeOID2&gt; ...
 * @author Luca
 *
 */
public class RuleApplicator {
	private static Logger log = Logger.getLogger(RuleApplicator.class);
	private static Connection con = null;
	
	/**
	 * Apply rules described in a file with the parameters specified  
	 * @param args arguments in the form: &lt;MacroRuleFilename&gt; [parameter [{, parameter]}]
	 */
	public static void main(String[] args) {
		/*
		log.info("Starting configuration test...");
		testConfiguration();
		log.info("Starting transaction test (w/ commit)...");
		testTransaction(true);
		log.info("Starting transaction test (w/ rollback)...");
		testTransaction(false);
		*/
		/*log.debug("args.length: " + args.length);
		if (true) return; //Dehehihohu*/
		if (args.length < 1) {
			RuleApplicator.printUsage();
		}
		else {
			File fMacroRule = new File(args[0]);
			String macroRuleFilename = null;
			if (fMacroRule.isAbsolute()) {
				//The filename can't be absolute, due to tree hierarchy.
				RuleApplicator.printUsage();
				log.fatal("the macrorule file name must be relative to the central node ('" + ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths") + "').");
				return;
			}
			else {
				//The filename is relative, it's likely to be good. We don't make a deeper check: a FileNotFoundException will be thrown automatically in case the path wasn't good.
				macroRuleFilename = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths")
											 + ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths")
											 + args[0];
											 
				String[] parameters = new String[args.length - 1];
				for (int i = 0; i < parameters.length; i++) {
					parameters[i] = args[i + 1];
				}
				try {
					MacroRule macroRule = MacroRuleReader.read(macroRuleFilename);
					log.debug("calling applyRule().");
					RuleApplicator.applyRule(macroRule, parameters);
					log.debug("returned from applyRule()");
				}
				catch (Exception e) {
					log.fatal(e);
				}
			}
		}
	}

	/**
	 * Logs the usage
	 */
	private static void printUsage() {
		log.fatal("Usage: java -jar applyrule.jar <MacroRuleFilename> [parameter [{, parameter]}]");
	}

	/**
	 * Apply the macrorule
	 * @param macroRule the tree of rules to be applied
	 * @param parameters the array of scheme OIDs (or instance OIDs) that must be replaced in the pseudo SQL statements.
	 * @throws Exception errors in the execution of the real sql in the database
	 */
	public static void applyRule(MacroRule macroRule, String[] parameters) throws Exception {
		//1. Connect to the database and begin a transaction.
		try {
			RuleApplicator.prepareConnection();
		}
		catch (Exception e) {
			e.printStackTrace();
			return; //If we can't begin a transaction, we can't proceed.
		}
		//2. Execute the SQL queries in the macroRule tree.
		try {
			RuleApplicator.applyMacroRule(macroRule, parameters);
			//If everything went good, we can commit the changes to the database.
			log.debug("applyMacroRule reported success, committing changes.");
			con.commit();
		}
		catch (Exception e) {
			//There were unrecoverable errors during the application of (at least) one rule.
			log.fatal(e.getLocalizedMessage());
			log.fatal("Due to unrecoverable errors, we're going to rollback any change made to the db.");
			try {
				con.rollback();
			}
			catch (SQLException sqle) { //This exception does not interest us.
			}
			throw e; //2005-02-01: luca s: this is needed to notify webapplyrule of the failure
		}
		//3. Close the connection to the database.
		RuleApplicator.closeConnection();
	}

	/**
	 * Navigates the MacroRule tree and applies every SimpleRule it finds.
	 * @param macroRule as the tree of rules.
	 * @throws Exception if at least one of the Statements produces an unrecoverable error.
	 */
	private static void applyMacroRule(MacroRule macroRule, String[] parameters) throws Exception {
		LinkedList<Rule> items = macroRule.getItems();
		
		//Begin the iteration.
//		boolean thereWereChanges = false;
//		int conta = 0;
//		do {
//			conta++;
//			thereWereChanges = false;
			for (Rule rule : items) {
				log.debug("looping on the items of a macrorule...");
				try {
					if (rule instanceof SimpleRule) //o can be a SimpleRule or a BlockOfRules
						RuleApplicator.applySimpleRule((SimpleRule) rule, parameters);
//						thereWereChanges = RuleApplicator.applySimpleRule((SimpleRule) o, parameters) || thereWereChanges;
					else if (rule instanceof BlockOfRules)
						RuleApplicator.applyBlockOfRules((BlockOfRules) rule, parameters);
				}
				catch (Exception e) {
					//There were unrecoverable errors during the execution of the simple rule or of an entire block.
					//We can't go on, so let's throw an exception.
					throw new Exception(e);
				}
			}
		//When we get here, the entire tree of rules was applied successfully.
//			System.out.println("There were changes: " + thereWereChanges);
//			System.out.println("To iterate: " + macroRule.mustIterate());
//			System.out.println("Do-While? " + (thereWereChanges && macroRule.mustIterate()) );
//			System.out.println("conta: " + conta);
//		} while (thereWereChanges && macroRule.mustIterate());
		return;
	}

	/**
	 * Executes the queries in a block of rules (which may contain nested blocks) throwing an Exception
	 * if one of the SimpleRule(s) throws an unrecoverable exception.
	 * @param block the block of SimpleRule-s and BlockOfRules-s
	 * @param parameters the array of scheme OIDs (or instance OIDs) that must be replaced in the pseudo SQL statements.
	 * @throws Exception if a SimpleRule throws an unrecoverable Exception during its execution.
	 */
	private static boolean applyBlockOfRules(BlockOfRules block, String[] parameters) throws Exception {
		LinkedList<Rule> items = block.getItems();
		boolean thereWereChanges = false;
		int iNumberOfIterations = 0;
		
		do {
			thereWereChanges = false;
			for (Rule rule : items) {
				if (rule instanceof BlockOfRules) //Go recursion, go!
					if (RuleApplicator.applyBlockOfRules((BlockOfRules) rule, parameters))
						thereWereChanges = true;
				else if (rule instanceof SimpleRule) 
					if (RuleApplicator.applySimpleRule((SimpleRule) rule, parameters)) //This can throw an exception.
						thereWereChanges = true;
			} //end for
			iNumberOfIterations++;
		}
		while (thereWereChanges);
		
		if (iNumberOfIterations > 1) //N = 1, no changes. N > 1, there was (at least) one change. 
			return true;
		return false;
	}

	/**
	 * Executes the Statements contained in the SimpleRule <b>rule</b>.
	 * @param rule
	 * @throws Exception if an unrecoverable error happens.
	 */
	private static boolean applySimpleRule(SimpleRule rule, String[] parameters) throws Exception {
		int iNumberOfIterations = 0;
		boolean thereWereChanges = false;
		
		LinkedList<Statement> statements = rule.getStatements();
		java.sql.Statement st = con.createStatement();
		String realSQL = null;

		//Auxiliary objects related to results.
		//ResultSet rsResult = null; //Only needed for SELECT queries.
		boolean bResult = false;
		int iUpdateCount = 0;
				
		/*
		 * This code will be executed one time only (if the SimpleRule must not be iterated)
		 * or as many times as needed to reach the still-point.
		 */
		log.debug("going to apply a SimpleRule.");
		boolean result = false;
		do {
			log.debug("applying a SimpleRule w/ " + statements.size() + " statements.");
			thereWereChanges = false; //This has a Rule-granularity and not Statement-granularity, so it's here.
			for (Statement s : statements) {
				log.debug("preparing (norollback = " + s.isNoRollback() + ") statement:\n" + s.getPseudoSQL());
				
				realSQL = RuleApplicator.applyParameters(s.getPseudoSQL(), parameters);
				log.debug("going to execute statement:\n" + realSQL);
				//And now we can really execute it.
				try {
					bResult = st.execute(realSQL);
					log.debug("executed statement:\n" + realSQL);
					//Did the statement make any changes to the DB?
					if (!bResult) { //There is an update count.
						iUpdateCount = st.getUpdateCount();
						//System.out.println("the statement has an update count: " + iUpdateCount);
						if (iUpdateCount > 0) {
								thereWereChanges = true;
								result = true;
						}
					}
					else { //There is a resultset object. This is the case of a SELECT query.
						log.warn("'SELECT' statement found. Executed harmlessly and moving on to the next statement.");
						//SELECTs don't change the state of the database, so just move on.
					}
				}
				catch (SQLException sqle) {
					log.error("failed execution of (norollback = " + s.isNoRollback() + ") statement found in file '" + rule.getFilename() + "':\n" + realSQL + "\nException: " + sqle);
					if (!s.isNoRollback()) //We cannot ignore this error.
						throw new Exception(sqle);
					//This throws a new exception which breaks the for, the while and is caught
					//where a rollback will be issued.
				}
			} //end for
			iNumberOfIterations++; //A single SimpleRule completed an iteration.
		} while (rule.isMustBeIterated() && thereWereChanges);
		log.debug("completed execution of a SimpleRule (iterations: " + iNumberOfIterations + ").");
//		if (iNumberOfIterations > 1) { //Only one iteration means no changes happened.
//		if (iUpdateCount > 1) { //Only one iteration means no changes happened.
//			return true;
//		}
		return result;
	}

	/**
	 * Applies a series of parameters in place of the ?1, ?2, ... in the statement.
	 * @param statement the pseudo sql
	 * @param parameters the array of scheme OIDs (or instance OIDs) that must be replaced in the pseudo SQL statements.
	 * @return the real sql
	 */
	private static String applyParameters(String statement, String[] parameters) {
		if (parameters == null) //There are no parameters to apply.
			return statement;
		for (int i = 0; i < parameters.length; i++) {
			statement = statement.replaceAll("[?]" + (i + 1), parameters[i]);
		}
		return statement;
	}

	/**
	 * Private method which creates a connection object and begins a transaction on it.<br/>
	 * This method is automatically called by applyRule(MacroRule, long[]) and some testers.  
	 */
	private static void prepareConnection() throws ClassNotFoundException, SQLException {
		//Load the driver...
		Class.forName(ModelgenConfigurationManager.getIstance().getProperty("driver", null, "jdbc"));
		//... open the connection...
		con = DriverManager.getConnection(
				ModelgenConfigurationManager.getIstance().getProperty("url", null, "jdbc"),
				ModelgenConfigurationManager.getIstance().getProperty("username", null, "jdbc"),
				ModelgenConfigurationManager.getIstance().getProperty("password", null, "jdbc"));
		//Now let's configure our connection object.
		con.setAutoCommit(false); //We desire a transaction.
	}

	/**
	 * Private method which closes the connection.<br/>
	 * commit() and rollback() must be called explicitly during the execution of queries in applyRule().<br/>
	 * This method is automatically called by applyRule(MacroRule, long[]).
	 */
	private static void closeConnection() {
		/*
		 * con.commit() and con.rollback() must be used in the context of the queries.
		 * Attempting to close a connection whose transaction status is not clear (i.e.: committed/rollbacked)
		 * will produce an exception.
		 */ 
		try {
			con.close();
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Tester. Reads informations from the configuration file and prints them to stdout.<br/>
	 *
	private static void testConfiguration() {
		java.io.PrintStream o = System.out;
		o.println("**** Configuration test ****");
		o.println("++++ database informations ++++");
		o.println("++++ Driver: " + config.getProperty("driver", null, "jdbc"));
		o.println("++++ URL: " + config.getProperty("url", null, "jdbc"));
		o.println("++++ Username: " + config.getProperty("username", null, "jdbc"));
		o.println("++++ Password: " + config.getProperty("password", null, "jdbc"));
		o.println("**** End of configuration test ****");
	}*/
	
	/**
	 * Tester. Shows how commit() and rollback() work together. 
	 * @param withCommit set to true, con.commit() is called before the last SELECT FROM.
	 * Set to false, con.rollback() is called (and an exception should be thrown, indicating that the
	 * table does not exist). 
	 *
	private static void testTransaction(boolean withCommit) {
		java.io.PrintStream o = System.out;
		try {
			RuleApplicator.prepareConnection();
			PreparedStatement pst = null;
			boolean bResult = false; int iResult = 0; ResultSet result = null;
			
			try {
				pst = con.prepareStatement("DROP TABLE testTable;");
				bResult = pst.execute();
				if (!bResult) {
					int i = pst.getUpdateCount();
					o.println("Results for DROP TABLE: " + i + " update cunts.");
					con.commit(); //DROP TABLE gets always executed and stored. The DB is initially made empty.
				}
			}
			catch (SQLException e) { //Damned (DROP && MSAccess) 
			}
			
			pst = con.prepareStatement("CREATE TABLE testTable(" +
				"an_id int primary key," +
				"a_number int" +
				");");
			bResult = pst.execute();
			if (!bResult)
				o.println("Results for CREATE TABLE: " + pst.getUpdateCount() + " update counts.");
			
			pst = con.prepareStatement("INSERT INTO testTable VALUES(1, 1);");
			bResult = pst.execute();
			if (!bResult) {
				o.println("Results for INSERT INTO (first one): " + pst.getUpdateCount() + " update counts.");
			}

			pst = con.prepareStatement("INSERT INTO testTable VALUES(2, 2);");
			iResult = pst.executeUpdate();
			o.println("Results for INSERT INTO (second one): " + iResult + " update counts.");

			pst = con.prepareStatement("UPDATE testTable SET a_number = 3 WHERE an_id = 2;");
			bResult = pst.execute();
			if (!bResult) {
				o.println("Results for UPDATE: " + pst.getUpdateCount() + " update counts.");
			}
			
			pst = con.prepareStatement("SELECT * FROM testTable;");
			result = pst.executeQuery();
			o.println("Results for SELECT FROM: ");
			while (result.next()) {
				o.println("an_id: " + result.getInt(1) + ", a_number: " + result.getInt(2));
			}
			
			//What are we testing?
			if (withCommit)
				con.commit(); //-> stores the change. The next SELECT must read from the DB.
			else
				con.rollback(); //-> reverts every change (CREATE, INSERT). The next SELECT must fail.
			
			RuleApplicator.closeConnection();
			RuleApplicator.prepareConnection(); //let's reopen it and read from the DB.
			pst = con.prepareStatement("SELECT * FROM testTable;");
			result = pst.executeQuery();
			o.println("Results for SELECT FROM (after commit/rollback): ");
			while (result.next()) {
				o.println("an_id: " + result.getInt(1) + ", ++ a_number: " + result.getInt(2));
			}
			//The following show that commit-ing or rollback-ing a connection with no changes causes no exceptions.
			con.rollback();
			con.commit();
			con.rollback();
			con.commit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}*/
}
