/*
 * Created on 10-dic-2003
 *
 */
package org.modelgen.applyrule;

/**
 * Contains the info about a SimpleRule's statement.
 * 
 * A Statement contains a so-called "PseudoSQL".
 * The PseudoSQL is a string containing a query in a SQL-like language. The only difference with SQL is
 * the presence of some variables (?1, ?2, ?3, ...) which constitute the parameters on which to call the
 * statement. Every other sintax element (such as 'NOROLLBACK') has been removed.  
 * 
 * @author Luca
 *
 */
public class Statement {

	private String pseudoSQL = null;
	private boolean noRollback = false;
	
	/**
	 * Creates a new Statement
	 * 
	 * @param pseudoSQL the parametric (?1, ?2, ...) SQL query
	 * @param noRollback if errors during the statement's execution can be ignored
	 */
	public Statement(String pseudoSQL, boolean noRollback) {
		this.pseudoSQL = pseudoSQL;
		this.noRollback = noRollback;
	}

	
	/**
	 * A statement with the "noRollback" property set to 'true' does not cause an abort of the translation
	 * should it fail. It's mainly designed to overcome some pickyness with some DBMSs and the "DROP TABLE"
	 * instruction. 
	 * @return true if errors during the statement's execution can be ignored. 
	 */
	public boolean isNoRollback() {
		return noRollback;
	}

	/**
	 * The PseudoSQL is a string containing a query in a SQL-like language. The only difference with SQL is
	 * the presence of some variables (?1, ?2, ?3, ...) which constitute the parameters on which to call the
	 * statement. Every other sintax element (such as 'NOROLLBACK') has been removed.  
	 * @return the parametric (?1, ?2, ...) SQL query.  
	 */
	public String getPseudoSQL() {
		return pseudoSQL;
	}
}
