/*
 * Created on 12-dic-2003
 *
 */

package org.modelgen.applyrule.reader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * This class provides three static methods to remove comments from a String object.
 * It is currently used in SimpleRuleReader, as I couldn't make it recognize comments inside
 * SQL statements. 
 * @author Luca
 *
 */
public class CommentRemover {

	/**
	 * Runs some tests
	 * @param args unused
	 */
	public static void main(String[] args) {
		CommentRemover.test();
	}
	
	/**
	 * Parses a text file, optionally removing line comments and block comments.
	 * @param filename the absolute pathname of the file to read.
	 * @param stripComments true, if the parser must remove single lined and block comments.
	 * @return a String object containing the text of the file, keeping the line breaks.
	 */
	public static String readFile(String filename, boolean stripComments) {
		String result = new String(""), aux = null;
		StringBuffer sb = new StringBuffer();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while ((aux = reader.readLine()) != null) {
				if (stripComments) {
					aux = CommentRemover.removeLineComments(aux);
				}
				//Note: the final \n is stripped from the original line, so we must put it back there.
				sb.append(aux).append("\n");
			}
		}
		catch (FileNotFoundException fnfe) {
			fnfe.printStackTrace();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
		result = sb.toString();
		if (stripComments) result = CommentRemover.removeBlockComments(result);
		return result;
	}
	
	/**
	 * Removes the end-of-line comments from the given String object.
	 * @param source the source string
	 * @return the source string, stripped of the line comments. 
	 */
	public static String removeLineComments(String source) {
		int slashPosition = -1, lfPosition = -1;
		do {
			slashPosition = source.indexOf("//");
			lfPosition = source.indexOf("\n", slashPosition);
			if (slashPosition != -1) { //There is a comment, remove it.
				if (lfPosition == -1) //There are more than one lines in this string.
					source = source.substring(0, slashPosition);
				else
					source = (source.substring(0, slashPosition) + source.substring(lfPosition)).intern();
			}
		} while (slashPosition != -1);
		return source;
	}
	
	/**
	 * Removes the end-of-line or block comments from the given String object.
	 * @param source the source string
	 * @return the source string, stripped of the line or block comments. 
	 */
	public static String removeComments(String source) {
		source = CommentRemover.removeBlockComments(source);
		source = CommentRemover.removeLineComments(source);
		return source;
	}
	
	/**
	 * Removes the block comments from the given String object.
	 * @param source the source string
	 * @return the source string, stripped of the block comments. 
	 */
	public static String removeBlockComments(String source) {
		StringBuffer sb = new StringBuffer();
		boolean appending = true;
		char c = '\0'; //To initialize it.
		char firstToken, secondToken; //Auxiliary characters
		for (int i = 0; i < source.length(); i++) {
			//Initializations and configurations
			c = source.charAt(i);
			if (appending) {
				firstToken = '/'; secondToken = '*';
			}
			else {
				firstToken = '*'; secondToken = '/';
			}
			//Examine the character (and eventually the following one)
			if (c == firstToken) { //A possible change of behaviour.
				if (i + 1 < source.length()) { //There's another character, so let's keep going.
					if (source.charAt(i + 1) == secondToken) {
						appending = !appending; //Change behaviour.
						i += 1; //Skip tokens.
					}
					else //It wasn't the beginning of a change of behaviour but a proper character.
						if (appending) sb.append(c);
				}
				else //It was the end of the string.
					if (appending) sb.append(c);
			}
			else //No change of behaviour.
				if (appending) sb.append(c);
		} //end for
		source = sb.toString();
		return source;
	}

	private static void test() {
		String test = CommentRemover.readFile("doc/test/testLineBreaks.txt", false);
		System.out.println(test);
		test = CommentRemover.readFile("doc/test/testLineBreaks.txt", true);
		System.out.println(test);
	}
}
