package org.modelgen.commons.beans;


import static org.junit.Assert.assertEquals;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;

import org.junit.Before;
import org.junit.Test;

/**
 * Test class for AbstractBean
 * 
 * @author Riccardo Pietrucci
 *
 */
class AbstractBeanTest {
	Model superModel;
	Scheme romaTre;

	Construct aggregation,lexical,struct;
	SchemeObject university, student;

	Property optional;
	SchemeObjectProperty optionalValue;

	Reference lexical2aggregation;

	SchemeObjectReference student2university;


	/**
	 * Set up some shared property for the tests  
	 */
	@Before
	public void setUp() {
		/* --------- Property --------- */
		this.optional = new Property(1, "isOptional");
		this.optionalValue = new SchemeObjectProperty(2, "isOptionalValue", this.optional);

		/* --------- Reference --------- */
		this.lexical2aggregation = new Reference(1, "lexical2aggregation");
		this.student2university = new SchemeObjectReference(2, "student2university", lexical2aggregation);

		/* --------- Construct --------- */
		this.aggregation = new Construct(0, "Aggregation");

		this.lexical = new Construct(1,"Lexical");
		this.lexical.addProperty(this.optional);
		this.optional.setOwnerConstruct(this.lexical); // necessario?

		this.lexical.addReference(this.lexical2aggregation);
		this.lexical2aggregation.setOwnerConstruct(this.lexical); // necessario?
		this.lexical2aggregation.setTargetConstruct(this.aggregation);

		this.struct = new Construct(2,"Struct");

		/* --------- SchemeObject --------- */
		this.university = new SchemeObject(1,"University", this.lexical);
		this.student = new SchemeObject(2,"Student", this.aggregation);
		this.university.setReference("lexical2aggregation", this.student);

		/* --------- Model --------- */
		this.superModel = new Model(3, "SuperModel");

		/* --------- Scheme --------- */
		this.romaTre = new Scheme(4, "RomaTre", this.superModel, true);
	}

	/**
	 * Test if the oid and name are setted correctly
	 */
	@Test
	public void testBeansInfo() {
		this.testOidAndName(this.aggregation, 0, "Aggregation");
		this.testOidAndName(this.lexical, 1, "Lexical");
		this.testOidAndName(this.struct, 2, "Struct");
		this.testOidAndName(this.university, 1, "University");
		this.testOidAndName(this.superModel, 3, "SuperModel");
		this.testOidAndName(this.romaTre, 4, "RomaTre");
		this.testOidAndName(this.optional, 1, "isOptional");
		this.testOidAndName(this.optionalValue, 2, "isOptionalValue");
		this.testOidAndName(this.lexical2aggregation, 1, "lexical2aggregation");
		this.testOidAndName(this.student2university, 2, "student2university");

	}

	private void testOidAndName(AbstractBean bean, int oidExpected, String nameExpected) {
		assertEquals(oidExpected,bean.getOID());
		assertEquals(nameExpected, bean.getName());
	}

	/**
	 * Test if the metainfo is setted correctly
	 */
	@Test
	public void testMultinavigability() {
		/* superModel ha il figlio romaTre? */
		assertEquals(this.superModel, this.romaTre.getMetaInfo());

		/* lexical ha il figlio university? */
		assertEquals(this.lexical, this.university.getMetaInfo());

		/* cambio padre di university a struct: verifica cambio */
		this.university.setMetaInfo(struct);
		assertEquals(this.struct, this.university.getMetaInfo());
	}

}
