/*
 * Created on Jan 25, 2004
 *
 */
package org.modelgen.commons.beans.model;

import org.modelgen.commons.beans.AbstractBean;

/**
 * The Property is a generic property of a {@link Construct}.<p>
 * 
 * If the property hasn't a referring property containing its metainfos we have
 * a <i>MetaProperty</i> belonging to the SuperModel.<p>
 * 
 * Therefore setting <t>null</t> to {@link #setMetaInfo(Property)} brings
 * this property to act as a MetaProperty. Vice versa, setting a non null value 
 * to {@link #setMetaInfo(Property)} sets the referring MetaProperty to this property.
 * 
 * @author Luca Santarelli [lucachan@tin.it - hrk@users.sourceforge.net]
 * Jan 25, 2004
 */
public class Property extends AbstractBean<Property> {
	private String type; //This is good as string, since it's the user-representation of the type.
	//In the table we'll use the super model type (which is a valid type, no matter if SQL or Java)
	private Construct ownerConstruct;
	private String defaultValue; //This is the default value of the MSM_Propery
	
	/**
	 * Creates the MetaProperty with the oid and the name specified.
	 * 
	 * @param oid the metaProperty's oid
	 * @param name the metaProperty's name
	 */
	public Property(int oid, String name) {
		super(oid, name);
	}
	
	/**
	 * Creates a Property with the oid, the name and the
	 * MetaProperty specified.
	 * 
	 * @param oid the property's oid
	 * @param name the property's name
	 * @param metaProperty the property's metaProperty
	 */
	public Property(int oid, String name, Property metaProperty) {
		super(oid, name, metaProperty);
	}
	
	/**
	 * @return the default value
	 */
	public String getDefaultValue() {
		return this.defaultValue;
	}

	/**
	 * @param defaultValue property's default value
	 */
	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}

	/**
	 * @return the property's type
	 */
	public String getType() {
		return type;
	}

	/** 
	 * @param type the property's type
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the owner construct of this property
	 */
	public Construct getOwnerConstruct() {
		return ownerConstruct;
	}

	/**
	 * @param construct the owner construct of this property
	 */
	public void setOwnerConstruct(Construct construct) {
		// TODO aggiungi property a construct? 
		ownerConstruct = construct;
	}

	/**
	 * Returns true if the given name is valid for a Property.<p>
	 * The check is formal only (i.e.: not null, not empty, ...)
	 * @param propertyName the name to check
	 * @return if the given name is valid
	 */
	public static boolean isValidName(String propertyName) {
		if (propertyName == null || propertyName.equals(""))
			return false;
		return true;
	}

	/**
	 * Returns true if the given name is valid for a Property.<p>
	 * The check is formal only (i.e.: not null, not empty, ...)
	 * @param propertyName the name to check
	 * @return if the given name is valid
	 */
	public static boolean isValidType(String propertyName) {
		if (propertyName == null || propertyName.equals(""))
			return false;
		return true;
	}

}
