/*
 * Created on 9-apr-2004
 *
 */
package org.modelgen.commons.beans.scheme;

import org.modelgen.commons.beans.AbstractStructure;
import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;
import org.modelgen.commons.utils.Validator;
import org.modelgen.signature.ConstructSignature;
import org.modelgen.signature.ModelSignature;
import org.modelgen.signature.util.ToolTableaux;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
/**
 * Basic class representing a scheme.<p>
 * A scheme is made out of a name, a referring model and a collection of (scheme) objects.<p>
 * 
 * A scheme is a structure containing an organization of elements that
 * contains the properties defined in its 
 * {@link org.modelgen.commons.beans.model.Model}. 
 * We can set the referring model through {@link #setMetaInfo(Model)}.<p>
 * 
 * <p>The specific datas of a scheme is reported as a list of
 * {@link org.modelgen.commons.beans.instance.Instance}.
 * 
 * @author Luca
 *
 */
public class Scheme extends AbstractStructure<Model,SchemeObject> {
	private boolean isSuperModelScheme = false; // riccardop
	private LinkedList<SchemeObject> objects = null;

	/**
	 * Creates a scheme with the oid, the name and the
	 * referring model specified.
	 * 
	 * <p>If it's a super model scheme, the referring model it's the
	 * <i>source model</i>, useful with rule transformations.
	 * 
	 * @param oid the scheme's oid
	 * @param name the scheme's name
	 * @param parent the referring model (or the source model if supermodel scheme)
	 * @param isSuperModelScheme if it's a supermodel scheme
	 */
	public Scheme(int oid, String name, Model parent, boolean isSuperModelScheme) {
		super(oid, name, parent);
		this.setSuperModelScheme(isSuperModelScheme);
	}
	
	/**
	 * Restituisce la segnatura del modello istanziato da tale schema. Il nome della
	 * segnatura  quello del modello a cui lo schema si riferisce. Per quanto riguarda
	 * invece le ConstructSignature, per ciascuna di esse il nome  quello del
	 * Construct a cui si riferisce un singolo SchemeObject di questo schema: la formula
	 * associata invece  un AND dei nomi delle Property contenute nelle SchemeObjectProperty
	 * di ciascun SchemeObject, eventualmente negati. Se ci sono pi SchemeObject con lo
	 * stesso nome, le loro formule sono messe in OR
	 * @return the model's signature
	 */
	public ModelSignature getModelSignature(){
		if(this.isSuperModelScheme)
			return this.getModelSignatureSM();
		else
			return this.getModelSignatureMM();
	}
	
	private ModelSignature getModelSignatureSM(){
		String modelName = this.getMetaInfo().getName();
		/////System.out.println("modelName "+modelName);
		ModelSignature sig = new ModelSignature(modelName);
		Map<String,ConstructSignature> segnatura = new HashMap<String,ConstructSignature>();
		
		//scandisco gli SchemeObject
		/////System.out.println("costrutti "+this.getAbstractConstructs().size());
		for(SchemeObject obj: this.getAbstractConstructs()){
			//nome della ConstructSignature
			String nameConstruct = obj.getMetaInfo().getName();
			/////System.out.println("nameConstruct "+nameConstruct);
			
			//propriet dello schema
			Collection<SchemeObjectProperty> properties = obj.getProperties();
			//formula della ConstructSignature
			String formula = "";
			//Scandisco le SchemeObjectProperty
			for(SchemeObjectProperty prop: properties){
				String propName = nameConstruct+"."+prop.getMetaInfo().getName();
				String segno = prop.getValue();
				if(segno.equalsIgnoreCase("false"))
					propName = "(NOT "+propName+")";
				else
					//se il segno  il default "" ignoro la propriet
					if(!segno.equalsIgnoreCase("true"))
						propName = null;
				//aggiorno la formula
				if(propName!=null){
					if(formula.length()==0)
						formula += propName;
					else 
						formula = "("+formula+" AND "+propName+")";
				}
			}
			/////System.out.println("formula "+formula);
			
			//se il costrutto non  ancora nella mappa
			ConstructSignature presente = segnatura.get(nameConstruct);
			////System.out.println("segnatura "+segnatura);
			////System.out.println("presente "+nameConstruct+" : "+presente);
			if(presente==null){
				ConstructSignature constrSig = new ConstructSignature(nameConstruct);
				constrSig.setFormula(formula);
				segnatura.put(nameConstruct, constrSig);
			}
			else{
				String formulaPresente = presente.getFormula();
				if(formula!=""){
					if(formulaPresente.length()==0)
						formulaPresente += formula;
					else 
						formulaPresente = "("+formulaPresente+" OR "+formula+")";
					ToolTableaux tool = new ToolTableaux(formulaPresente);
					presente.setFormula(tool.formaNormDisgiuntiva());
				}
				else
					presente.setFormula(formula);
			}

		}
		sig.setCostrutti(segnatura);
		return sig;
	}
	
	private ModelSignature getModelSignatureMM(){
		String modelName = this.getMetaInfo().getName();
		ModelSignature sig = new ModelSignature(modelName);
		Map<String,ConstructSignature> segnatura = new HashMap<String,ConstructSignature>();
		//scandisco gli SchemeObject
		for(SchemeObject obj: this.getAbstractConstructs()){
			//nome della ConstructSignature
			String nameConstruct = obj.getMetaInfo().getMetaInfo().getName();
			//propriet dello schema
			Collection<SchemeObjectProperty> properties = obj.getProperties();
			//formula della ConstructSignature
			String formula = "";
			//Scandisco le SchemeObjectProperty
			for(SchemeObjectProperty prop: properties){
				String propName = nameConstruct+"."+prop.getMetaInfo().getMetaInfo().getName();
				String segno = prop.getValue();
				if(segno.equalsIgnoreCase("false"))
					propName = "(NOT "+propName+")";
				else
					//se il segno  il default "" ignoro la propriet
					if(!segno.equalsIgnoreCase("true"))
						propName = null;
				//aggiorno la formula
				if(propName!=null){
					if(formula.length()==0)
						formula += propName;
					else 
						formula = "("+formula+" AND "+propName+")";
				}
			}
			//se il costrutto non  ancora nella mappa
			ConstructSignature presente = segnatura.get(nameConstruct);
			if(presente==null){
				ConstructSignature constrSig = new ConstructSignature(nameConstruct);
				constrSig.setFormula(formula);
				segnatura.put(nameConstruct, constrSig);
			}
			else{
				String formulaPresente = presente.getFormula();
				if(formula!=""){
					if(formulaPresente.length()==0)
						formulaPresente += formula;
					else 
						formulaPresente = "("+formulaPresente+" OR "+formula+")";
					ToolTableaux tool = new ToolTableaux(formulaPresente);
					presente.setFormula(tool.formaNormDisgiuntiva());
				}
				else
					presente.setFormula(formula);
			}

		}
		sig.setCostrutti(segnatura);
		return sig;
	}
	
	/**
	 * Modifies the scheme objects internal sorting according to the
	 * constructs sorting.
	 */
	public void sort() {
		LinkedList<SchemeObject> sortedObjects = new LinkedList<SchemeObject>();
		for (Construct construct : getMetaInfo().getAbstractConstructs()) {
			//Now over the objects.
			for (SchemeObject object : this.getAbstractConstructs()) {
				if (object.getMetaInfo().getName().equals(construct.getName())) {
					//Okay.
					sortedObjects.addLast(object);
				}
			}
			//All the instances of that construct have been added.
		} //All the constructs have been checked.
		this.objects = sortedObjects;
	}

	/**
	 * Adds the specified scheme object
	 * @param o the scheme object to add
	 */
	public void addObject(SchemeObject o) {
		if(objects == null) {
			objects = new LinkedList<SchemeObject>();
		}
		objects.addLast(o);
	}
	
	/**
	 * Performs a basic String checking (i.e.: not null, not empty, doesn't start with a number).
	 * @param name the name to check
	 * @return if is a valid name
	 */
	public static boolean isValidName(String name) {
		if (!Validator.isValidString(name))
			return false;
		/* Is this needed? If yes, set the seserved characters.
		String[] reservedChars = { "_", "-" };
		if (!Validator.isValidString(name, reservedChars))
			return false;
		*/
		return true;
	}
	
	/**
	 * The key for a SchemeObject is a superkey made of its name and all the constructs which describe it.<br/>
	 * Description is known by looking at the attribute 'type' of its Reference-s.<br/>
	 * Example: we want to add a new instance of AttributeOfAbstract called "name" for the (instance of) Abstract "Person".<br/>
	 * The key is composed of "name" and of "Person" (the single Reference.DESCRIPTION reference that AttributeOfAbstract has).<br/>
	 * Now we can add another AttributeOfAbstract called "name" to the scheme, provided that its "abstract" is not the Abstract "Person".<br/>
	 * to a Scheme  
	 * [note mie: dato un nome, lo scheme object pu? essere creato solo se non ne esistono altri con lo stesso nome E afferenti allo stesso scheme object.
	 * le afferenze sono date dalle relazioni di tipo DESCRIPTION.
	 * Esempio: posso aggiungere uno SO(Entit?) "Professore" se e solo se non esistono altri SO di nome "Professore".
	 * Posso aggiungere uno SCO(AttributoDiEntit?) "nome" se e solo se non esistono altri SO che:
	 * 	- hanno nome "nome"
	 * 	- hanno un riferimento (di tipo descrittivo) allo SO "Professore".
	 * Questa "super chiave" si estende al livello immediatamente superiore e non oltre: se esistesse un particolare costrutto
	 * tale da avere un riferimento descrittivo all'AttributoDiEntit?, non dovrei proseguire fino all'Entit? cui questo afferisce, ma mi fermerei
	 * direttamente all'AttributoDiEntit?. 
	 * 
	 * NOTA ULTERIORE: chiave dovrebbe essere anche il particolare costrutto!
	 * Es: Posso avere una entit? "Professore" ed un attributo "Professore"!
	 * @param key the scheme object key
	 * @return the corresponding scheme object value
	 */
	public SchemeObject getObject(SchemeObjectKey key) {
		for (SchemeObject o : this.getAbstractConstructs()) {
			if (o.getKey().equals(key)) //Got it!
				return o;
		}
		//Got nothing, it's not present.
		return null;
	}
	/**
	 * Removes the scheme object corresponding to the key specified from
	 * this scheme
	 * @param objectKey the scheme object key
	 */
	public void removeObject(SchemeObjectKey objectKey) {
		LinkedList<SchemeObjectKey> dependencies = new LinkedList<SchemeObjectKey>();
		for (SchemeObject o : this.getAbstractConstructs()) {
			//If there's a reference pointing to the object we're going to delete, add it to the dependencies list.
			for (SchemeObjectReference sor : o.getReferences()) {
				if (sor.getValue() != null && sor.getValue().getKey().equals(objectKey)) {
					dependencies.add(o.getKey());
				}
			}
		}
		//The dependency list is complete, remove them.
		for (SchemeObjectKey dep : dependencies) {
			this.removeObject(dep);
		}
		//And now we can safely remove the real object.
		for (int i = 0; i <= objects.size(); i++) {
			SchemeObject o = objects.get(i);
			if (o.getKey().equals(objectKey)) {
				objects.remove(i);
				return;
			}
		}
	}

	/**
	 * Finds and obtains the scheme object with the specified
	 * oid and name
	 * @param objectOID the scheme oiject's oid to obtain
	 * @param constructName the scheme oiject's name to obtain
	 * @return the scheme object found, <code>null</code> otherwise
	 */
	public SchemeObject getObject(int objectOID, String constructName) {
		for (SchemeObject o : this.getAbstractConstructs()) {
			if (o.getOID() == objectOID && o.getMetaInfo().getName().equals(constructName))
				return o;
		}
		return null;
	}
	
	/**
	 * Returns a collection of all the constructs currently available for instantiation.<p>
	 * Note that this is different from <code>Model.getAvailableConstructs()</code>, since the
	 * policy is different: in <code>Model.getAvailableConstructs()</code>, the returned collection contains
	 * all the constructs which have not yet been mapped and whose dependencies are all met. They are
	 * the constructs which can be mapped.<p>
	 * The collection returned by this method, instead, returns all the constructs which can be istantiated;
	 * they are all the constructs whose dependencies are all met.  
	 * @return a collection of all the constructs currently available for instantiation.
	 */
	public Collection<Construct> getAvailableConstructs() {
		LinkedList<Construct> availableConstructs = new LinkedList<Construct>();
		for (Construct c : this.getMetaInfo().getAbstractConstructs()) {
			//For each construct, check if its dependencies are fulfilled.
			boolean allAreFulfilled = true;
			//This construct, c, can be added only if all the constructs it depends on have been istantiated at least once.
			for (Construct smc : c.getSuperModelConstructs()) {
				for (Construct dependency : smc.getDependencies()) {
					//In order for the dependency to be met, there must be at least one object instance of this construct.
					boolean atLeastOneInstance = false;
					for (SchemeObject so : this.getAbstractConstructs()) {
						for (Construct element : so.getMetaInfo().getSuperModelConstructs()) {
							if (element.getName().equals(dependency.getName())) {
								atLeastOneInstance = true;
								break; //We have found proof of at least one instance, exit from the loop. 
							}
						}
					}
					if (atLeastOneInstance == false) {
						//There is no object instance of this construct, the dependencies are not met.
						allAreFulfilled = false;
						break; //We can exit from this 'for'.
					}
				}
			}
			if (allAreFulfilled == true) {//This construct is istantiable.
				availableConstructs.addLast(c);
			}
		}
		return availableConstructs;
	}
	
	/**
	 * Restituisce una percentuale di matching tra il modello passato come parametro, e lo schema stesso.<p>
	 * Si basa sull'enumerazione dei costrutti presenti nello schema e quelli presenti nel modello destinazione.<p>
	 * Esempio: dato uno schema espresso nel modello "ER con attributi su relazioni ma senza gerarchie" con
	 * tutti i costrutti istanziati almeno una volta (quindi con almeno una entit, una relazione, un attributo di entit
	 * ed un attributo di relazione) ed il modello "ER senza gerarchie e senza attributi sulle relazioni", 
	 * l'algoritmo restituisce 0.75 come valore, poich dallo schema risultano 4 super-costrutti (presi in
	 * base alle corrispondenze tra quelli enumerati dallo schema ed il supermodello) di cui solo
	 * tre mappati a costrutti del modello destinazione.<p>
	 * <p>
	 * Se lo schema iniziale avesse avuto solamente una entit, una relazione ed un attributo di entit,
	 * il risultato dell'algoritmo sarebbe stato 1.00, perch lo schema  totalmente compatibile col modello
	 * destinazione.<p>
	 * L'algoritmo, ovviamente, non tiene conto delle regole di traduzione che permettono di far corrispondere
	 * (per dire) l'Abstract all'AggregationOfLexicals.
	 * @param destModel modello da analizzare
	 * @param isSuperScheme va impostato a true se lo schema  un superschema, false se  uno schema.
	 * @return percentuale di matching
	 */
	public double getMatchingPercentage(Model destModel, boolean isSuperScheme) {
		ArrayList<Construct> usedSuperModelConstructs = new ArrayList<Construct>(); //Contains the super model objects.
		//Let's prepare a list of all the used super model constructs.
		for (SchemeObject so : this.getAbstractConstructs()) {
			Construct smConstruct = null; //Is this a scheme or super scheme?
			if (isSuperScheme) { //A super scheme: the objects has a direct mapping to the construct.
				smConstruct = so.getMetaInfo();
			}
			else { //A (model) scheme, the object has a mapping to a model construct. 
				smConstruct = so.getMetaInfo().getSuperModelConstructs().iterator().next();
			}
			if (!usedSuperModelConstructs.contains(smConstruct)) {
					usedSuperModelConstructs.add(smConstruct);
			}
		}
		//Let's count how many of these super constructs are available in the destination model.
		int count = 0; //As starting condition, the model doesn't match. 
		for (Construct usedSuperModelConstruct : usedSuperModelConstructs) {
			if (destModel.getConstructMappedTo(usedSuperModelConstruct) != null) {
				//The destination model contains a construct mapped to the (super) construct used in the scheme.
				count++;
			}
		}
		//We can now know the matching value.
		//Note: we need to do an explicit cast to double in order to have a number between 0.00 and 1.00 
		double matching =  ((double)count / (double)(usedSuperModelConstructs.size()));
		return matching;
	}

	@Override
	protected LinkedList<SchemeObject> doGetAbstractConstructs() {
		return this.objects;
	}

	@Override
	protected void doLoadAbstractConstructs() throws ClassNotFoundException, Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler();
		//Model superModel = ph.getSuperModel();
		
		Scheme schemeLoaded;
		if(this.isSuperModelScheme) {
			schemeLoaded = ph.loadSuperModelSchemeInternally(this.getOID());
		} else {
			schemeLoaded = ph.loadSchemeInternally(this.getOID());
		}
		
		if(schemeLoaded != null) {
			this.objects = schemeLoaded.getAbstractConstructs();
		} else {
			this.objects = new LinkedList<SchemeObject>();
		}
	}

	/**
	 * @return if is a supermodel scheme
	 */
	public boolean isSuperModelScheme() {
		return this.isSuperModelScheme;
	}

	/**
	 * Sets if it's a supermodel scheme
	 * @param isSuperModelScheme if it's a supermodel scheme
	 */
	public void setSuperModelScheme(boolean isSuperModelScheme) {
		this.isSuperModelScheme = isSuperModelScheme;
	}

	public void initSchemeObjects() {
		if(objects == null) {
			objects = new LinkedList<SchemeObject>();
		}
	}

}
