/*
 * Created on 28-apr-2004
 *
 */
package org.modelgen.commons.beans.scheme;

/**
 * This class represents the key for a {@link SchemeObject}. 
 * It's made up by its name and its external identifiers.
 * 
 * @author Luca
 *
 */
public class SchemeObjectKey {
	
	/**
	 * Contatore (static) per la generazione dei valori (numerici) delle chiavi. Avrei voluto
	 * utilizzare qualche pattern un po' pi "elegante", ma alla fin fine il risultato  lo stesso,
	 * e in questo modo nessuna classe si accorge del cambiamento di rappresentazione delle chiavi.
	 * L'information hiding  sicuramente pi importante dell'eleganza di scrittura di un pattern.
	 */
	private static Integer keyCounter = new Integer(1); //Le chiavi partono da 1.
	
	private Integer internalKey = null;
	
	/**
	 * Creates a unique generated key for a scheme object
	 */
	public SchemeObjectKey() {
		this.internalKey = SchemeObjectKey.getNewKey();
	}
	
	/**
	 * Creates a key with the specified value for a scheme object
	 * @param name an integer rappresentation of key value
	 */
	public SchemeObjectKey(String name) {
		this.internalKey = Integer.decode(name);//SchemeObjectKey.getNewKey();
	}
	
	/**
	 * @return the new key in a thread-safe manner
	 */
	private synchronized static Integer getNewKey() {
		Integer newKey = SchemeObjectKey.keyCounter; //Impostiamo la chiave da restituire al valore del contatore.
		SchemeObjectKey.keyCounter = new Integer(SchemeObjectKey.keyCounter.intValue() + 1); //Incrementiamo il contatore
		return newKey;
	}
	
	/**
	 * Compares two keys, returning true if and only if the two keys are the same.<br/>
	 * It works recursively, calling the <code>equals()</code> method on the external keys.
	 * @param anotherKey the other key to analyze
	 * @return if the two keys are the same
	 */
	public boolean equals(SchemeObjectKey anotherKey) {
		if (!(this.internalKey.equals(anotherKey.internalKey))) //If the internal keys differ, they are different objects.
			return false;
		return true;
	}

	/**
	 * @return a String rappresentation of the wrapped key value
	 */
	public String toString() {
		return this.internalKey.toString();
	}
}
