/*
 * Created on 27-gen-2005
 *
 */
package org.modelgen.commons.beans.skolem;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.utils.Validator;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
/**
 * A injective function useful to generate new unique oids in rules execution.
 * 
 * <p>They are also useful to represent the mapping between the source
 * and the target schemes.
 * 
 * <p>Every SkolemFunction is composed by different {@link SkolemFunctionArgument}.
 * 
 * @author lucasantarelli
 *
 */
public class SkolemFunction {

	private int OID;
	private String name;
	private Construct ownerConstruct;
	private Collection<SkolemFunctionArgument> arguments = new LinkedList<SkolemFunctionArgument>();
	
	/**
	 * Returns true if the given name is valid for a SkolemFunction.<p>
	 * The check is formal only (i.e.: not null, not empty, ...)
	 * @param name the name to check
	 * @return if the given name is valid
	 */
	public static boolean isValidName(String name) {
		if (!Validator.isValidString(name)) return false;
		return true;
	}
	
	/**
	 * Check if this skolem function contains the specified
	 * argument (in case-insensitive manner)
	 * @param argument the argument to find
	 * @return if the argument exists in this skolem function
	 */
	public boolean contains(SkolemFunctionArgument argument) {
		for (Iterator<SkolemFunctionArgument> itArguments = arguments.iterator(); itArguments.hasNext();) {
			SkolemFunctionArgument element = itArguments.next();
			if (element.getName().equalsIgnoreCase(argument.getName()))
				return true;
		}
		return false;
	}
	/**
	 * @return Returns the oid.
	 */
	public int getOID() {
		return OID;
	}
	/**
	 * @param oid The oid to set.
	 */
	public void setOID(int oid) {
		OID = oid;
	}
	/**
	 * @return Returns the arguments.
	 */
	public Collection<SkolemFunctionArgument> getArguments() {
		return arguments;
	}
	/**
	 * @param arguments The arguments to set.
	 */
	public void setArguments(Collection<SkolemFunctionArgument> arguments) {
		this.arguments = arguments;
	}
	/**
	 * @return the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		if (name == null) throw new IllegalArgumentException("The construct can't be null.");
		this.name = name;
	}
	/**
	 * @return the ownerConstruct.
	 */
	public Construct getOwnerConstruct() {
		return ownerConstruct;
	}
	/**
	 * @param ownerConstruct The ownerConstruct to set.
	 */
	public void setOwnerConstruct(Construct ownerConstruct) {
		this.ownerConstruct = ownerConstruct;
	}
	
	/**
	 * @param argument the argument to add
	 */
	public void addArgument(SkolemFunctionArgument argument) {
		arguments.add(argument);
	}
}
