/*
 * Created on 27-gen-2005
 *
 */
package org.modelgen.commons.beans.skolem;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.utils.Validator;
/**
 * An argument of a {@link SkolemFunction}
 * 
 * @author lucasantarelli
 *
 */
public class SkolemFunctionArgument {

	private int OID;
	private String name = null;
	private String type = null;
	private Construct sourceConstruct = null;
	
	/**
	 * Create a new argument with an oid = -1 and
	 * a null name and type
	 */
	public SkolemFunctionArgument() {
		this.OID = -1;
		this.name = null;
		this.type = null;
	}
	
	/**
	 * Create a new argument
	 * @param OID the argument's oid
	 * @param name the argument's name
	 * @param type the argument's type
	 * @param sourceConstruct the argument's source construct
	 */
	public SkolemFunctionArgument(int OID, String name, String type, Construct sourceConstruct) {
		this.OID = OID;
		this.name = name;
		this.type = type;
		this.sourceConstruct = sourceConstruct;
	}
	
	/**
	 * Returns true if the given name is valid for a SkolemFunctionArgument.<p>
	 * The check is formal only (i.e.: not null, not empty, ...)
	 * @param name the name to check
	 * @return if the given name is valid
	 */
	public static boolean isValidName(String name) {
		if (!Validator.isValidString(name)) return false;
		return true;
	}
	
	/**
	 * Returns true if the given name is valid for a SkolemFunctionArgument.<p>
	 * The check is formal (i.e.: not null, not empty, ...) and check
	 * if the type is well known.
	 * @param type the name to check
	 * @return if the given name is valid
	 */
	public static boolean isValidType(String type) {
		if (!Validator.isValidString(type)) return false;
		if (!type.equalsIgnoreCase("integer") && !type.equalsIgnoreCase("varchar")) return false;
		return true;
	}
	/**
	 * @return Returns the oID.
	 */
	public int getOID() {
		return OID;
	}
	/**
	 * @param oid The oID to set.
	 */
	public void setOID(int oid) {
		OID = oid;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the type.
	 */
	public String getType() {
		return type;
	}
	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the source construct
	 */
	public Construct getSourceConstruct() {
		return sourceConstruct;
	}

	/**
	 * @param sourceConstruct the source construct
	 */
	public void setSourceConstruct(Construct sourceConstruct) {
		this.sourceConstruct = sourceConstruct;
	}
}
