/*
 * Created on 5-feb-2004
 *
 */
package org.modelgen.commons.persistence.database;

import org.modelgen.commons.utils.ModelgenConfigurationManager;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.log4j.Logger;

/**
 * Establishes a connection with the database specified in the configuration
 * file.
 * 
 * @author Luca Santarelli [lucachan@tin.it - hrk@users.sourceforge.net]
 * Feb 5, 2004
 */
public class DBConnecter {
	private Connection con = null;
	private Logger log = Logger.getLogger(DBConnecter.class);
	//DB + DBMS parameters
	
	private String driver = ModelgenConfigurationManager.getIstance().getProperty("driver", "[driver not found in config file]", "jdbc");
	private String url = ModelgenConfigurationManager.getIstance().getProperty("url", "[url not found in config file]", "jdbc");
	private String username = ModelgenConfigurationManager.getIstance().getProperty("username", "[username not found in config file]", "jdbc");
	private String password = ModelgenConfigurationManager.getIstance().getProperty("password", "[password not found in config file]", "jdbc");
	
	/**
	 * @return the connection established (or create it)
	 * @throws ClassNotFoundException if the driver specified in the configuration file can't be found
	 * @throws SQLException if the connection can't be established
	 */
	public synchronized Connection getConnection() throws ClassNotFoundException, SQLException {
		if (con == null) //If there isn't already an active connection, create it.
			con = this.createConnection();
		return con;
	}

	/**
	 * Close the connection
	 * @throws SQLException if a database access error occurs
	 */
	public void closeConnection() throws SQLException {
		con.close();
		con = null;
	}

	private synchronized Connection createConnection() throws ClassNotFoundException, SQLException {
		if (con == null) { //If there isn't a connection, create it.
			if (url.indexOf("not found") != -1) { //The configuration wasn't read successfully. :-(
				log.fatal("jConfig could not load modelgen_config.xml.");
			}
			Class.forName(driver);
			con = DriverManager.getConnection(url, username, password);
			//con.setTransactionIsolation(Connection.TRANSACTION_SERIALIZABLE);
		}
		return con;
	}
}