/*
 * Created on May 6, 2004 by Luca Santarelli [hrk@users.sourceforge.net | lucachan@tin.it]
 *
 */
package org.modelgen.commons.persistence.database;

import org.modelgen.applyrule.RuleApplicator;
import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;

import java.util.LinkedList;

/**     
 * Initializes the DB for ModelGen by calling RuleApplicator on the
 * DDL (+ basic DML) queries, which build the required tables and
 * populate the super model, and then optionally building a complete
 * Relational model and a complete Entity-Relationship (with binary
 * relationships) model.  
 * 
 * @author harlock
 */
public class Initializer {
	/**
	 * Initialize the DB for ModelGen
	 * @param args possible args: 
	 * <ul>
	 * <li><tt>--help</tt>: prints the help screen</li>
	 * <li><i>NO-ARGS</i>: creates only the Metadata tables and the Super Model data/tables
	 * <li><tt>--relational</tt>: adds a Relational model without foreign keys to the database</li>
	 * <li><tt>--erb</tt>: adds a Binary ER model without Generalisation to the database</li>
	 * <li><tt>--ern</tt>: adds a N-ary ER model without Generalisation to the database</li>
	 * <li><tt>--skolem</tt>: adds a set of predefined skolem functions to the database</li>
	 * </ul>
	 */
	public static void main(String[] args) {
		//Parse the command line for arguments:
		boolean bCreateRelational = false, bCreateBinaryER = false, bCreateNaryER = false;
		for (int i = 0; i < args.length; i++) {
			String param = args[i];
			//System.out.println("Parametro: " + args[i]);
			if (param.equalsIgnoreCase("--relational")) {
				bCreateRelational = true;
			}
			else if (param.equalsIgnoreCase("--erb")) {
				bCreateBinaryER = true;
			}
			else if (param.equalsIgnoreCase("--ern")) {
				bCreateNaryER = true;
			}
			else if (param.equalsIgnoreCase("--help")) {
				printUsage();
				return;
			}
		}
		//1. Populate the MSM_* Tables via the ApplyRule tool.
		String[] applyruleParameters = new String[1];
		applyruleParameters[0] = "GoodRules/ModelGen.txt";
		RuleApplicator.main(applyruleParameters);
		applyruleParameters[0] = "GoodRules/ModelGen_skolem.txt";
		RuleApplicator.main(applyruleParameters);
		//2. Create the tables of the SuperModel.
		try {
			DBHandler dbh = (DBHandler) PersistenceHandlerFactory.getPersistenceHandler("DBHandler");
			StructureCreator sc = new StructureCreator();
			sc.createSMStructures(dbh.getSuperModel(), dbh.getConnection());
			//3. Optionally, populate with some models.
			if (bCreateRelational)
				createRelationalModel();
			if (bCreateBinaryER)
				createBinaryERModel();
			if (bCreateNaryER)
				createNaryERModel();
//			if (bCreateSkolemFunctions)
//				createSkolemFunctions();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
//	private static void createSkolemFunctions() throws Exception {
//		DBHandler dbh = (DBHandler) PersistenceHandlerFactory.getPersistenceHandler("DBHandler");
//		Model superModel = dbh.getSuperModel();
//		
//		SkolemFunction skf = new SkolemFunction();
//		SkolemFunctionArgument skfa = null;
//		
//		skf.setName("attributeOID_1");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAggregationOfLexicals"));
//		//skfa = new SkolemFunctionArgument(-1, "OID_old", "integer");
//		skfa = new SkolemFunctionArgument(-1, "attributeOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		skf = new SkolemFunction();
//		skf.setName("attributeOID_2");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAggregationOfLexicals"));
//		skfa = new SkolemFunctionArgument(-1, "aggregationOID", "integer");
//		skf.addArgument(skfa);
//		skfa = new SkolemFunctionArgument(-1, "role", "varchar");
//		skf.addArgument(skfa);
//		skfa = new SkolemFunctionArgument(-1, "keyAttributeOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		skf = new SkolemFunction();
//		skf.setName("aggregationOID_2");
//		skf.setOwnerConstruct(superModel.getConstruct("AggregationOfLexicals"));
//		skfa = new SkolemFunctionArgument(-1, "aggregationOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//		
//		skf = new SkolemFunction();
//		skf.setName("aggregationOID_1");
//		skf.setOwnerConstruct(superModel.getConstruct("AggregationOfLexicals"));
//		skfa = new SkolemFunctionArgument(-1, "abstractOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//		
//		skf = new SkolemFunction();
//		skf.setName("attributeOID_3");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAggregationOfLexicals"));
//		skfa = new SkolemFunctionArgument(-1, "attAbstractOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare abstract in copia PA e PC 29/3/2004
//		skf = new SkolemFunction();
//		skf.setName("abstractOID_0");
//		skf.setOwnerConstruct(superModel.getConstruct("Abstract"));
////		skfa = new SkolemFunctionArgument(-1, "OID_old", "integer");
//		skfa = new SkolemFunctionArgument(-1, "abstractOID", "integer");
//		skf.addArgument(skfa);
//		//dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare attributi di abstract in copia PA e PC
//		// 29/3/2004
//		skf = new SkolemFunction();
//		skf.setName("attributeOID_0");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAbstract"));
//		skfa = new SkolemFunctionArgument(-1, "OID_old", "integer");
//		skf.addArgument(skfa);
//		//dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare binary aggrgegation of abstract in copia
//		// PA e PC 1/4/2004
//		skf = new SkolemFunction();
//		skf.setName("binaryAggregationOfAbstractOID_0");
//		skf.setOwnerConstruct(superModel.getConstruct("BinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "OID_old", "integer");
//		skf.addArgument(skfa);
//		//dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare abstract da aggregation PA e PC 29/3/2004
//		skf = new SkolemFunction();
//		skf.setName("abstractOID_1");
//		skf.setOwnerConstruct(superModel.getConstruct("Abstract"));
//		skfa = new SkolemFunctionArgument(-1, "aggregationOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare attributi di abstract che vengono da
//		// attributi di aggregation non-binarie PA e PC 30/3/2004
//		skf = new SkolemFunction();
//		skf.setName("attributeOID_4");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAbstract"));
//		skfa = new SkolemFunctionArgument(-1, "attOfAggOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare aggregation da component di aggregation
//		// // PA e PC 29/3/2004
//		skf = new SkolemFunction();
//		skf.setName("binaryAggregationOfAbstractOID_1");
//		skf.setOwnerConstruct(superModel.getConstruct("BinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "componentOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare aggregation da aggregation // PA e PC
//		// 29/3/2004
//		skf = new SkolemFunction();
//		skf.setName("binaryAggregationOfAbstractOID_2");
//		skf.setOwnerConstruct(superModel.getConstruct("BinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "nAryAggregationOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		// nuova funzione per generare attributi di aggregation da attributi di
//		// aggregation PA e PC 30/3/2004 paolo c 20041025 cambiato suffisso da
//		// _5 a _1
//		skf = new SkolemFunction();
//		skf.setName("attributeOfBinaryAggregationOfAbstractOID_1");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfBinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "attributeOfAggregationOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		//	nuova funzione per generare abstract da aggregazioniDiLessicali
//		//	(traduzione dal RelazionaleConFK al ERbinario) PC 06/10/2004
//		skf = new SkolemFunction();
//		skf.setName("abstractOID_2");
//		skf.setOwnerConstruct(superModel.getConstruct("Abstract"));
//		skfa = new SkolemFunctionArgument(-1, "aggregationOfLexicalsOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		//	nuova funzione per generare attributiDiAstratto da
//		// attributiDiAggregazioniDiLessicali
//		//	(traduzione dal RelazionaleConFK al ERbinario) PC 06/10/2004
//		skf = new SkolemFunction();
//		skf.setName("attributeOfAbstractOID_3");
//		skf.setOwnerConstruct(superModel.getConstruct("AttributeOfAbstract"));
//		skfa = new SkolemFunctionArgument(-1, "attributeOfAggregationOfLexicalsOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		skf = new SkolemFunction();
//		skf.setName("binaryAggregationOfAbstractOID_3");
//		skf.setOwnerConstruct(superModel.getConstruct("BinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "aggregation1OID", "integer");
//		skf.addArgument(skfa);
//		skfa = new SkolemFunctionArgument(-1, "abstractOID", "integer");
//		skf.addArgument(skfa);
//		skfa = new SkolemFunctionArgument(-1, "role", "varchar");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//
//		//	nuova funzione per generare attributiDiAstratto da
//		// attributiDiAggregazioniDiLessicali
//		//	(traduzione dal RelazionaleConFK al ERbinario) PC 06/10/2004
//		skf = new SkolemFunction();
//		skf.setName("binaryAggregationOfAbstractOID_4");
//		skf.setOwnerConstruct(superModel.getConstruct("BinaryAggregationOfAbstracts"));
//		skfa = new SkolemFunctionArgument(-1, "foreignKeyOID", "integer");
//		skf.addArgument(skfa);
//		dbh.storeSkolemFunction(superModel, skf);
//	}
	/**
	 * 
	 */
	private static void createNaryERModel() throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler();
		
		Model superModel = ph.getSuperModel();
		Model model = new Model(0,"ERNarioFULL");
		
		model.setShortName("ERNFULL");
		
		//Let's add the constructs.
		Construct entity = buildConstruct("Entita", model, "Abstract", superModel);
		model.addConstruct(entity);
		//The entity is in.
		Construct attributeOfEntity = buildConstruct("AttributoDiEntita", model, "AttributeOfAbstract", superModel);
		model.addConstruct(attributeOfEntity);
		//The attribute of entity is in, let's add its properties.
		attributeOfEntity.addProperty(buildProperty("ammetteNullo", "booleano", "AttributoDiEntita", model, "isNullable", "AttributeOfAbstract", superModel));
		attributeOfEntity.addProperty(buildProperty("eChiave", "booleano", "AttributoDiEntita", model, "isKey", "AttributeOfAbstract", superModel));
		attributeOfEntity.getReference("abstractOID").setName("entita");
		//Ok, let's move on to the next construct.
		Construct relationship = buildConstruct("Relazione", model, "AggregationOfAbstracts", superModel);
		model.addConstruct(relationship);
		//In the Nary ER model, relationships have no properties (since they evolved into the ComponentOfAggregationOfAbstracts)
		Construct componentOfRelationship = buildConstruct("PartecipanteInRelazione", model, "ComponentOfAggregationOfAbstracts", superModel);
		model.addConstruct(componentOfRelationship);
		//This construct has the properties which belong to the relationship in the Binary ER model:
		componentOfRelationship.addProperty(buildProperty("eOpzionale", "booleano", "PartecipanteInRelazione", model, "isOpt", "ComponentOfAggregationOfAbstracts", superModel));
		componentOfRelationship.addProperty(buildProperty("eFunzione", "booleano", "PartecipanteInRelazione", model, "isFunct", "ComponentOfAggregationOfAbstracts", superModel));
		componentOfRelationship.addProperty(buildProperty("eIdentificato", "booleano", "PartecipanteInRelazione", model, "isIdentified", "ComponentOfAggregationOfAbstracts", superModel));
		//Fix the references.
		componentOfRelationship.getReference("abstractOID").setName("entita");
		componentOfRelationship.getReference("aggregationOID").setName("relazione");
		//Ok, the ComponentOf[] is complete.
		Construct attributeOfRelationship = buildConstruct("AttributoDiRelazione", model, "AttributeOfAggregationOfAbstracts", superModel);
		model.addConstruct(attributeOfRelationship);
		attributeOfRelationship.addProperty(buildProperty("ammetteNullo", "booleano", "AttributoDiRelazione", model, "isNullable", "AttributeOfAggregationOfAbstracts", superModel));
		attributeOfRelationship.getReference("aggregationOID").setName("relazione");
		//We're done with this model.
		ph.storeModel(model);
		
	}
	/**
	 * 
	 */
	private static void printUsage() {
		java.io.PrintStream o = System.out;
		o.println("Usage: java org.modelgen.commons.persistence.database.Initiaqlizer [option(s)]");
		o.println("Available options:\n\t--help (prints this help screen)\n\t--relational (adds a Relational model without foreign keys to the database)\n\t--erb (adds a Binary ER model without Generalisation to the database)\n" +
				"\t--ern (adds a N-ary ER model without Generalisation to the database) --skolem (adds a set of predefied skolem functions to the database)");
		o.println("Without arguments, only the Metadata tables and the Super Model data/tables are created.");
	}
	/**
	 * 
	 */
	private static void createBinaryERModel() throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler();
		
		Model superModel = ph.getSuperModel();
		Model model = new Model(0,"ERBinarioFULL");
		
		model.setShortName("ERBFULL");
		
		//Let's add the constructs.
		Construct entity = buildConstruct("Entita", model, "Abstract", superModel);
		model.addConstruct(entity);
		//The entity is in.
		Construct attributeOfEntity = buildConstruct("AttributoDiEntita", model, "AttributeOfAbstract", superModel);
		model.addConstruct(attributeOfEntity);
		//The attribute of entity is in, let's add its properties.
		attributeOfEntity.addProperty(buildProperty("ammetteNullo", "booleano", "AttributoDiEntita", model, "isNullable", "AttributeOfAbstract", superModel));
		attributeOfEntity.addProperty(buildProperty("eChiave", "booleano", "AttributoDiEntita", model, "isKey", "AttributeOfAbstract", superModel));
		attributeOfEntity.getReference("abstractOID").setName("entita");
		//Ok, let's move on to the next construct.
		Construct relationship = buildConstruct("Relazione", model, "BinaryAggregationOfAbstracts", superModel);
		model.addConstruct(relationship);
		//Some properties...
		relationship.addProperty(buildProperty("ruolo1", "testuale", "Relazione", model, "role1", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("ruolo2", "testuale", "Relazione", model, "role2", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("eOpzionale1", "booleano", "Relazione", model, "isOpt1", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("eOpzionale2", "booleano", "Relazione", model, "isOpt2", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("eFunzione1", "booleano", "Relazione", model, "isFunct1", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("eFunzione2", "booleano", "Relazione", model, "isFunct2", "BinaryAggregationOfAbstracts", superModel));
		relationship.addProperty(buildProperty("eIdentificato", "booleano", "Relazione", model, "isIdentified", "BinaryAggregationOfAbstracts", superModel));
		//Fix th references.
		relationship.getReference("abstract1OID").setName("entita1");
		relationship.getReference("abstract2OID").setName("entita2");
		//Ok, the "Relation" construct is complete.
		Construct attributeOfRelationship = buildConstruct("AttributoDiRelazione", model, "AttributeOfBinaryAggregationOfAbstracts", superModel);
		model.addConstruct(attributeOfRelationship);
		attributeOfRelationship.addProperty(buildProperty("ammetteNullo", "booleano", "AttributoDiRelazione", model, "isNullable", "AttributeOfBinaryAggregationOfAbstracts", superModel));
		attributeOfRelationship.getReference("aggregationOID").setName("relazione");
		//We're done with this model.
		ph.storeModel(model);
	}
	
	private static void createRelationalModel() throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler();
		//LinkedList smConstructs = null;
		//Construct smConstruct = null;
		
		Model superModel = ph.getSuperModel();
		Model model = new Model(0,"RelazionaleFULL");
		
		model.setShortName("RelaFULL");
		//Table:
		Construct table = buildConstruct("Tabella", model, "AggregationOfLexicals", superModel);
		model.addConstruct(table);
		//Column:
		Construct column = buildConstruct("Colonna", model, "AttributeOfAggregationOfLexicals", superModel);
		//column has two properties, isNullable and isKey
		column.addProperty(buildProperty("ammetteNullo", "booleano", "Colonna", model, "isNullable", "AttributeOfAggregationOfLexicals", superModel));
		column.addProperty(buildProperty("eChiave", "booleano", "Colonna", model, "isKey", "AttributeOfAggregationOfLexicals", superModel));
		//column has a single reference to the Table/AggregationOfLexicals: set its name.
		column.getReference("aggregationOID").setName("tabella");
		model.addConstruct(column);
		/*
		//Add new construct: table.
		Construct table = new Construct();
		table.setName("Tabella");
		smConstruct = superModel.getConstruct("AggregationOfLexicals");
		smConstructs = new LinkedList();
		smConstructs.add(smConstruct);
		table.setSuperModelConstructs(smConstructs);
		
		model.addConstruct(table);
		//Add new construct: column.
		Construct column = new Construct();
		column.setName("Colonna");
		smConstruct = superModel.getConstruct("AttributeOfAggregationOfLexicals");
		smConstructs = new LinkedList();
		smConstructs.add(smConstruct);
		column.setSuperModelConstructs(smConstructs);
		//Its reference:
		Reference smReference = (Reference) smConstruct.getReferences().iterator().next();
		Reference reference = new Reference();
		reference.setName("tabella");
		reference.setType(smReference.getType());
		reference.setOwnerConstruct(column);
		reference.setTargetConstruct(model.getConstructMappedTo(smReference.getTargetConstruct()));
		reference.setSuperModelReference(smReference);
		column.addReference(reference);
		
		Property property = new Property();
		property.setName("ammetteNullo");
		property.setType("booleano");
		property.setOwnerConstruct(column);
		property.setSuperModelProperty(smConstruct.getProperty("isNullable"));
		column.addProperty(property);
		
		property = new Property();
		property.setName("eChiave");
		property.setType("booleano");
		property.setOwnerConstruct(column);
		property.setSuperModelProperty(smConstruct.getProperty("isKey"));
		column.addProperty(property);
		
		model.addConstruct(column);
		//The column is done.
		*/
		ph.storeModel(model);
	}
	
	/**
	 * Builds a construct with the given parameters.
	 * @param constructName
	 * @param model
	 * @param smConstructName
	 * @param superModel
	 * @return the construct builded
	 */
	private static Construct buildConstruct(String constructName, Model model, String smConstructName, Model superModel) {
		Construct newConstruct = new Construct(0,constructName);
		LinkedList<Construct> mappings = new LinkedList<Construct>();
		Construct smConstruct = superModel.getConstruct(smConstructName); 
		mappings.add(smConstruct);
		newConstruct.setSuperModelConstructs(mappings);
		//Now the references.
		for (Reference smReference : smConstruct.getReferences()) {
			Reference r = new Reference(0,smReference.getName(),smReference);
			r.setOwnerConstruct(newConstruct);
			r.setTargetConstruct(model.getConstructMappedTo(smReference.getTargetConstruct()));
			r.setType(smReference.getType());
			newConstruct.addReference(r);
		}
		return newConstruct;
	}
	/**
	 * Builds a Property object which corresponds to the given parameters.
	 * @param propertyName
	 * @param propertyType
	 * @param constructName
	 * @param model
	 * @param smPropertyName
	 * @param smConstructName
	 * @param superModel
	 * @return the porperty builded
	 */
	private static Property buildProperty(String propertyName, String propertyType, String constructName, Model model, String smPropertyName, String smConstructName, Model superModel) {
		Property p = new Property(0, propertyName, superModel.getConstruct(smConstructName).getProperty(smPropertyName));
		p.setType(propertyType);
		p.setOwnerConstruct(model.getConstruct(constructName));
		return p;
	}
}
