/*
 * Created on 5-feb-2004
 *
 */
package org.modelgen.commons.persistence.database;

import org.modelgen.commons.beans.instance.Instance;
import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.commons.beans.skolem.SkolemFunctionArgument;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.commons.utils.TypeTranslator;
import org.modelgen.makecopyrules.CopyRulesMaker;
import org.modelgen.regole.SkolemFunction;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
/**
 * Implements {@link DBHandler} methods: for every
 * declared method perform a specific query in the working database.
 * 
 * @author Luca Santarelli [lucachan@tin.it - hrk@users.sourceforge.net]
 * Feb 5, 2004
 */
public class RealDBHandler implements DBHandler {

	protected DBConnecter dbc;
	protected Connection con;
	private Logger log = Logger.getLogger(RealDBHandler.class);
	private CacheProxyDBHandler cache; // riccardop: per la consistenza

	/**
	 * @throws ClassNotFoundException  if the driver specified in the configuration file can't be found
	 * @throws SQLException if the connection can't be established
	 * 
	 */
	public RealDBHandler() throws ClassNotFoundException, SQLException {
		dbc = new DBConnecter();
		con = dbc.getConnection();
	}

	protected void setCache(CacheProxyDBHandler cache) {
		this.cache = cache;
	}

	/**
	 * Returns the next valid OID to be used in an INSERT statement for the table and field passed as arguments.
	 * @param table
	 * @param field
	 */
	public int getNewOID(String table, String field) throws PersistenceException {
		try {
			//Prepare the query.
			String query = "SELECT max(" + field + ") FROM " + table + " ;";
			Statement st = con.createStatement();
			//Execute the query.
			ResultSet result = st.executeQuery(query);
			//Return the result.
			int newOID = 0;
			while (result.next()) {
				newOID  = result.getInt(1) + 1;
			}
			return newOID;
		} catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}
	/**
	 * Returns true if there is already a model with the given name, false otherwise (which means it returns false on a null string too)
	 * @param modelName
	 * @return true, if a model with the name modelName does exist already.
	 */
	public boolean doesModelExist(String modelName) throws PersistenceException {
		try {
			//We're using a prepared statement in order not have Java deal with string weirdness.
			String sqlQuery = "SELECT * FROM Model WHERE name = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);
			pst.setString(1, modelName);
			boolean bFound = false;
			ResultSet result = pst.executeQuery();
			//Ok, is it there?
			while (result.next())
				bFound = true;

			return bFound;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Returns true if the shortName is already used for a model, false otherwise (which means it returns false on a null string too)
	 * @return true, if a model with the name modelName does exist already.
	 */
	public boolean doesModelExistWithShortname(String shortName) throws PersistenceException {
		try {
			//We're using a prepared statement in order not have Java deal with string weirdness.
			String sqlQuery = "SELECT * FROM Model WHERE shortname = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);
			pst.setString(1, shortName);
			boolean bFound = false;
			ResultSet result = pst.executeQuery();
			//Ok, is it there?
			while (result.next())
				bFound = true;

			return bFound;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Returns a collection of the currently available models.
	 */
	public Collection<Model> getModels() throws PersistenceException {
		try {
			String sqlQuery = "SELECT * FROM Model;";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);

			LinkedList<Model> collection = new LinkedList<Model>();
			Model model = null;
			while (result.next()) {
				model = setupModel(result);
				collection.addLast(model);
			}
			return collection;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	public Model getSuperModel() throws PersistenceException {
		try {
			Model superModel = new Model(0,"SuperModel");
			superModel.setShortName("SM");
			//SQL queries:

//			stessa aggiunta che per load model: carico anche isLexical
//			paolo c 20041022
			String queryConstructs =
//				"SELECT C.OID, C.name " +
				"SELECT C.OID, C.name, C.isLexical " +
				"FROM MSM_Construct C ;";
			String queryProperties =
				"SELECT P.OID, P.name, P.type, P.default " +
				"FROM MSM_Property P " +
				"WHERE P.constructOID = ?;";
			String queryReferences = 
				"SELECT R.OID, R.name, R.type, C.name " +
				"FROM MSM_Reference R, MSM_Construct C " +
				"WHERE R.targetConstructOID = C.OID AND R.ownerConstructOID = ?;";
			String queryDependencies = 
				"SELECT D.parent " +
				"FROM MSM_Dependency D " +
				"WHERE D.child = ?;";
			//Statements to execute queries:
			Statement st = con.createStatement();
			PreparedStatement pstProperties = con.prepareStatement(queryProperties);
			PreparedStatement pstReferences = con.prepareStatement(queryReferences);
			PreparedStatement pstDependencies = con.prepareStatement(queryDependencies);
			//ResultSet objects:
			ResultSet rsConstructs = null, rsProperties = null, rsReferences = null, rsDependencies;
			//Auxiliary objects:
			Property p = null;
			Reference r = null;
			//Let's start!
			//A super model is made of constructs:
			rsConstructs = st.executeQuery(queryConstructs);
			while (rsConstructs.next()) {
				int oid = rsConstructs.getInt(1);
				String name = rsConstructs.getString(2);
				Construct c = new Construct(oid, name);
//				paolo c 20041022
				c.setLexical(rsConstructs.getBoolean(3));
				//Now a construct has properties...
				pstProperties.setInt(1, c.getOID());
				rsProperties = pstProperties.executeQuery();
				while (rsProperties.next()) {
					int oidProp = rsProperties.getInt(1);
					String nameProp = rsProperties.getString(2);
					p = new Property(oidProp, nameProp);
					p.setType(rsProperties.getString(3));
					p.setOwnerConstruct(c);
					p.setDefaultValue(rsProperties.getString(4));
					//The property object is complete.
					c.addProperty(p);
				}
				//A construct has references and dependences too... but they depend on the construct list, so we load them later.
				superModel.addConstruct(c);
			}
			//Now we have the superModel w/ constructs and properties, we lack the references and dependencies.
			for (Construct c : superModel.getAbstractConstructs()) {
				pstReferences.setInt(1, c.getOID());
				rsReferences = pstReferences.executeQuery();
				while (rsReferences.next()) {
					int oid = rsReferences.getInt(1);
					String name = rsReferences.getString(2);
					r = new Reference(oid, name);
					r.setType(rsReferences.getString(3));
					r.setOwnerConstruct(c);
					r.setTargetConstruct(superModel.getConstruct(rsReferences.getString(4)));
					c.addReference(r);
				}
				pstDependencies.setInt(1, c.getOID());
				rsDependencies = pstDependencies.executeQuery();
				while (rsDependencies.next()) {
					int dependedConstructOID = rsDependencies.getInt(1);
					c.addDependency(superModel.getConstruct(dependedConstructOID));
				}
			}
			//The super model object is complete.
			return superModel;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Returns a Model object whose OID in the database is modelOID.
	 * @param modelOID
	 */
	public Model getModel(int modelOID) throws PersistenceException {
		try {
			Statement st = con.createStatement(); //Why not a PreparedStatement? Because:
			//1. This query is executed once per call, not multiple times per call.
			//2. the variable type is an int, which is consistent in the different sintaxes (unlike dates, strings, ...)  
			ResultSet result = st.executeQuery("SELECT * FROM Model WHERE oid = " + modelOID + ";");

			result.next(); //To move on the first tuple...

			return setupModel(result);
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Private helper method. Builds a Model object from a ResultSet object.
	 * @param result the tuple representing this model in the table "Model"
	 * @throws SQLException
	 */
	private Model setupModel(ResultSet result) throws SQLException, PersistenceException {
		int oid = result.getInt("OID");
		String name = result.getString("name");
		boolean isAncestor = result.getBoolean("isAncestor");
		int familyOid = result.getInt("familyOID");
		Model model = new Model(oid, name);
		model.setShortName(result.getString("shortname"));
		model.setAncestor(isAncestor);
		model.setFamilyOid(familyOid);
		if(familyOid!=0 && !isAncestor)
			model.setFamily(this.getFamilyAncestor(familyOid));
		return model;
	}

	public List<Model> getAncestors() throws PersistenceException{
		List<Model> families = new LinkedList<Model>();

		try{
			Statement st = con.createStatement(); //Why not a PreparedStatement? Because:
			//1. This query is executed once per call, not multiple times per call.
			//2. the variable type is an int, which is consistent in the different sintaxes (unlike dates, strings, ...)  
			ResultSet result = st.executeQuery("SELECT modelOID FROM Family WHERE OID>0");

			while(result.next()){
				int oid = result.getInt("modelOID");
				families.add(this.getModel(oid));
			}

			return families;
		} catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}

	}

	/**
	 * The family ancestor
	 * @param familyOID
	 * @return the Model that is tha family's ancestor
	 * @throws PersistenceException
	 */
	public Model getFamilyAncestor(int familyOID) throws PersistenceException{
		int modelOid;
		try {
			modelOid = this.getFamilyAncestorOid(familyOID);
			if(modelOid!=-1)
				return this.getModel(modelOid);
			else
				return null;
		} catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Private helper method. Return the oid of the family's ancestor
	 * @param familyOID
	 * @return the oid of the family's ancestor
	 * @throws PersistenceException
	 */
	private int getFamilyAncestorOid(int familyOID) throws SQLException{
		if(familyOID==0)
			return -1;
		else{
			Statement st = con.createStatement(); //Why not a PreparedStatement? Because:
			//1. This query is executed once per call, not multiple times per call.
			//2. the variable type is an int, which is consistent in the different sintaxes (unlike dates, strings, ...)  
			ResultSet result = st.executeQuery("SELECT modelOID FROM Family WHERE oid = " + familyOID + ";");

			result.next(); //To move on the first tuple...

			return result.getInt("modelOID");
		}
	}

	/**
	 * Completely loads a Model object given its database OID and an OO representation of the super model.
	 * @param modelOID
	 * @param superModel
	 */
	public Model loadModelInternally(int modelOID, Model superModel) throws PersistenceException {
		log.debug("Loading model " + modelOID);
		try {
			if (superModel == null) //Just in case people forget about it... 
				superModel = cache.getSuperModel();

			Model model;
			try {
				model = this.getModel(modelOID);
			} catch (PersistenceException e) { return null; }
			/*
			 * How do we load everything else?
			 * 1. We have to take every Construct belonging to this model => MM_Tables JOIN Model2ModelTables W/ modelOID
			 * Now that we have all the constructs which belong to the model? The properties and references of each construct.
			 * 2.1 Properties: MM_Proeprty WHERE constructOID = Construct.getOID() => build a list/collection and add it to the construct.
			 * [explode: for each property we need the mapping to the SM]
			 * 2.2 References: just the same. => Build a collection and add it to the construct.
			 * [explode: the same!]
			 * 2.3 Mappings to the super model => C_MT_SMT.
			 * This means we need to already have a construct object (although it may be not completely initialized)
			 * At the end, we have a complete model in memory.
			 */
			//A model is made of constructs:
//			C.isLexical aggiunto da paolo c 20041022
			String queryConstructs =
				"SELECT C.OID, C.name, C.signature, C.bonds, C.isLexical " +
				"FROM MM_Construct C " +
				"WHERE C.modelOID = " + modelOID + " ORDER BY C.OID;";
			String queryProperties = 
				"SELECT P.OID, P.name, P.type " +
				"FROM MM_Property P " +
				"WHERE P.constructOID = ? ORDER BY P.OID;"; //Parametric on the Construct OID
			String queryReferences = 
				"SELECT R.OID, R.name, R.type, R.targetConstructOID " +
				"FROM MM_Reference R " +
				"WHERE R.ownerConstructOID = ? ORDER BY R.OID;"; //Parametric on the Construct OID.
			String queryCMappings = //Mappings for a Construct. 
				"SELECT Con.name " +
				"FROM Mapping_MC_SMC Map, MSM_Construct Con " +
				"WHERE Map.constructOID_SM = Con.OID AND Map.constructOID_M = ?;"; //Returns the name of the SM constructs mapped to this construct.
			String queryPMappings = //Mapping(s) for a Property. Currently, there's a 1:1 mapping.
				"SELECT propertyOID_SM " +
				"FROM Mapping_MP_SMP " +
				"WHERE propertyOID_M = ?;";
			String queryRMappings = 
				"SELECT referenceOID_SM " +
				"FROM Mapping_MR_SMR " +
				"WHERE referenceOID_M = ?;"; //Same as above.

			Property property = null;
			Reference reference = null;

			PreparedStatement pstProperties = con.prepareStatement(queryProperties);
			PreparedStatement pstReferences = con.prepareStatement(queryReferences);
			PreparedStatement pstCMappings = con.prepareStatement(queryCMappings);
			PreparedStatement pstPMappings = con.prepareStatement(queryPMappings);
			PreparedStatement pstRMappings = con.prepareStatement(queryRMappings);

			ResultSet rsProperties = null, rsReferences = null,
			rsCMappings = null, rsPMappings = null, rsRMappings = null;

			//Let's begin the fun.
			Statement st = con.createStatement();
			ResultSet rsConstructs = st.executeQuery(queryConstructs);

			/* Inizializza la lista dei construct */
			model.initConstruct();
			while (rsConstructs.next()) {
				int oid = rsConstructs.getInt(1);
				String name = rsConstructs.getString(2);
				Construct construct = new Construct(oid, name);

//				aggiunto paolo c 20041022: carico anche isLex
				construct.setLexical(rsConstructs.getBoolean(5));

				//aggiunto fabrizio celli: carico anche segnatura e bond
				construct.setSignature(rsConstructs.getString(3));
				construct.setUserBonds(rsConstructs.getString(4));

				//Now: mapping(s) and properties. Mappings go first.
				pstCMappings.setInt(1, construct.getOID());
				rsCMappings = pstCMappings.executeQuery();
				LinkedList<Construct> smConstructs = new LinkedList<Construct>();
				while (rsCMappings.next()) {
					String smConstructName = rsCMappings.getString(1); //Get its name...
					smConstructs.add(superModel.getConstruct(smConstructName)); //Get the object and put it in the list...
					construct.setSuperModelConstructs(smConstructs); //Associate the list to the original Construct
				}
				//The mappings are done, let's do the properties.
				pstProperties.setInt(1, construct.getOID());
				rsProperties = pstProperties.executeQuery();

				while (rsProperties.next()) {
					int oidProp = rsProperties.getInt(1);
					String nameProp = rsProperties.getString(2);
					property = new Property(oidProp, nameProp);
					property.setType(rsProperties.getString(3));
					property.setOwnerConstruct(construct);
					//And now the mapped property.
					//It is one of the properties of one of the constructs mapped to the construct which owns this property.
					//What is the OID of the property?
					pstPMappings.setInt(1, property.getOID());
					rsPMappings = pstPMappings.executeQuery();
					rsPMappings.next(); //NOTE: Currently there's a 1:1 relation.
					int smPropertyOID = rsPMappings.getInt(1);
					//Ok, look for it in the super model.

					for (Construct c : construct.getSuperModelConstructs()) {
						Property p = c.getProperty(smPropertyOID);
						if (p != null) {
							property.setMetaInfo(p);
							break;
						}
					}
					//The property object is complete, we just need to add it to the Construct object.
					construct.addProperty(property);
				}
				//The construct object is almost complete (it just lacks the references). We add it to the model.
				model.addConstruct(construct);
			}
			//We have created the Construct objects. They're not really good yet, since they lack the references, which we weren't able to load.
			//Now that the construct tree has been built, we can load the references.
			for (Construct construct : model.getAbstractConstructs()) { //For each construct in the model... <begin>
				pstReferences.setInt(1, construct.getOID());
				rsReferences = pstReferences.executeQuery(); //pstReferences selects all the references belonging to this construct (R.tableOID = T.OID) 
				while (rsReferences.next()) { //For each reference belonging to this construct... <begin>
					//Make a new reference and fill it with the values we already have for sure:
					int oid = rsReferences.getInt(1);
					String name = rsReferences.getString(2);
					reference = new Reference(oid, name);
					reference.setType(rsReferences.getString(3));
					reference.setOwnerConstruct(construct); //This one is obvious and comes from the 2-way navigation.
					//Find the target construct: it's the one with tableOID_to as OID:
					reference.setTargetConstruct(model.getConstruct(rsReferences.getInt(4))); //Find by OID; it exists (or the db is broken => we don't validate)
					//The basic object is done, we need the mapping towards the super model.
					//1. Find the OID of the super model reference mapped to this model reference: 
					pstRMappings.setInt(1, reference.getOID());
					rsRMappings = pstRMappings.executeQuery();
					if (rsRMappings.next()) { //This should have a single item (if 0 || 2, the db is broken => remember we don't validate (not our business)
						int smReferenceOID = rsRMappings.getInt(1);
						//Now iterate on the references, which means iterate on the super model constructs and for each one, iterate on his references.
						for (Construct c : construct.getSuperModelConstructs()) {
							Reference r = c.getReference(smReferenceOID); //This is iterating internally on the references.
							if (r != null) { //It was found in the construct c! Good!
								reference.setMetaInfo(r);
								break; //Don't lose other cycles.
							}
						}
					}
					//Now that the item is ready, add it to the construct.
					construct.addReference(reference);
				} //For each reference belonging to this construct... <end>
			} //For each construct in the model... <end> 
			return model;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Saves a complete Model object into the database.
	 * @param model
	 */
	public void storeModel(Model model) throws PersistenceException {
		try {
			con.setAutoCommit(false);
			//STEP 1: DML to insert data: (OIDs are parametric and obtained via getNextOID(table, field)).
			try {
				String addModel = "INSERT INTO Model(name, shortname, familyOID, isAncestor) VALUES(?, ?, ?, ?);";
				String addConstruct = "INSERT INTO MM_Construct(name, modelOID, modelName, signature, bonds) VALUES(?, ?, ?, ?, ?);";
				String addProperty = "INSERT INTO MM_Property(constructOID, name, type) VALUES(?, ?, ?);";
				String addReference = "INSERT INTO MM_Reference(ownerConstructOID, name, type, targetConstructOID) VALUES(?, ?, ?, ?);";
				String addMappingMC_SMC = "INSERT INTO Mapping_MC_SMC(constructOID_M, constructOID_SM) VALUES(?, ?);";
				String addMappingMP_SMP = "INSERT INTO Mapping_MP_SMP(propertyOID_M, propertyOID_SM) VALUES(?, ?);";
				String addMappingMR_SMR = "INSERT INTO Mapping_MR_SMR(referenceOID_M, referenceOID_SM) VALUES(?, ?);";

				String getModelOID = "SELECT OID FROM Model WHERE name = ?;";
				String getConstructOID = "SELECT OID FROM MM_Construct WHERE name = ? AND modelOID = ?;";
				String getPropertyOID = "SELECT OID FROM MM_Property WHERE name = ? AND constructOID = ?;";
				String getReferenceOID = "SELECT OID FROM MM_Reference WHERE name = ? AND ownerConstructOID = ?;";

				int modelOID = 0, constructOID = 0, propertyOID = 0, referenceOID = 0;

				//The prepared statements.
				PreparedStatement pstAddModel = con.prepareStatement(addModel);
				PreparedStatement pstAddConstruct = con.prepareStatement(addConstruct);
				PreparedStatement pstAddProperty = con.prepareStatement(addProperty);
				PreparedStatement pstAddReference = con.prepareStatement(addReference);
				PreparedStatement pstAddMappingMC_SMC = con.prepareStatement(addMappingMC_SMC);
				PreparedStatement pstAddMappingMP_SMP = con.prepareStatement(addMappingMP_SMP);
				PreparedStatement pstAddMappingMR_SMR = con.prepareStatement(addMappingMR_SMR);
				PreparedStatement pstGetModelOID = con.prepareStatement(getModelOID);
				PreparedStatement pstGetConstructOID = con.prepareStatement(getConstructOID);
				PreparedStatement pstGetPropertyOID = con.prepareStatement(getPropertyOID);
				PreparedStatement pstGetReferenceOID = con.prepareStatement(getReferenceOID);
				ResultSet rsQueryGet = null;

				pstAddModel.setString(1, model.getName());
				pstAddModel.setString(2, model.getShortName());
				pstAddModel.setInt(3, model.getFamilyOid());
				pstAddModel.setBoolean(4, model.isAncestor());

				//System.out.println("*******************************");
				//System.out.println(pstAddModel);
				pstAddModel.execute(); //throws SQLException if fails.

				//The model is in. Let's read its key.
				pstGetModelOID.setString(1, model.getName());

				//System.out.println("*******************************");
				//System.out.println(pstGetModelOID);
				rsQueryGet = pstGetModelOID.executeQuery();

				if (rsQueryGet.next()) //If the insert was good, this will be good, so this check is redundant...
					modelOID = rsQueryGet.getInt(1);
				else //... and this line has debug value only.
					modelOID = -1;
				model.setOID(modelOID);
				rsQueryGet.close();
				for (Construct construct : model.getAbstractConstructs()) {
					//Insert the construct:
					pstAddConstruct.setString(1, construct.getName());
					pstAddConstruct.setInt(2, model.getOID());
					pstAddConstruct.setString(3, model.getName()); //TODO: (pri: LOW) remove this when the DB goes into 'stable'. (Note: it violates Boyce-Codd normal form)
					String signature = construct.getSignature();
					if(signature==null || signature.length()==0)
						signature = "true";
					String bond = construct.getUserBonds();
					if(bond==null || bond.length()==0)
						bond = "true";
					pstAddConstruct.setString(4, signature);
					pstAddConstruct.setString(5, bond);
					//The query is ready.

					//System.out.println("*******************************");
					//System.out.println(pstAddConstruct);
					pstAddConstruct.execute();

					pstGetConstructOID.setString(1, construct.getName());
					pstGetConstructOID.setInt(2, model.getOID());

					//System.out.println("*******************************");
					//System.out.println(pstGetConstructOID);
					rsQueryGet = pstGetConstructOID.executeQuery();

					if (rsQueryGet.next()) //Same as before.
						constructOID = rsQueryGet.getInt(1);
					else
						constructOID = -1;
					construct.setOID(constructOID);
					rsQueryGet.close();
					//What comes with a Construct? The corresponding SM construct(s).
					for (Construct smConstruct : construct.getSuperModelConstructs()) {
						pstAddMappingMC_SMC.setInt(1, constructOID);
						pstAddMappingMC_SMC.setInt(2, smConstruct.getOID());

						//System.out.println("*******************************");
						//System.out.println(pstAddMappingMC_SMC);
						pstAddMappingMC_SMC.execute();

					}
					//What else? Properties and references.
					for (Property property : construct.getProperties()) {
						//propertyOID = this.getNewOID("MM_Property", "OID");
						//pstAddProperty.setInt(1, propertyOID);
						pstAddProperty.setInt(1, constructOID);
						pstAddProperty.setString(2, property.getName());
						pstAddProperty.setString(3, property.getType());
						//The query is ready.
//////////////////////	//
						//System.out.println("*******************************");
						//System.out.println(pstAddProperty);
						pstAddProperty.execute();

						pstGetPropertyOID.setString(1, property.getName());
						pstGetPropertyOID.setInt(2, constructOID);
						rsQueryGet = pstGetPropertyOID.executeQuery();

						rsQueryGet.next();
						propertyOID = rsQueryGet.getInt(1);
						rsQueryGet.close();
						//The property is in, now go for the mappings.
						pstAddMappingMP_SMP.setInt(1, propertyOID);
						pstAddMappingMP_SMP.setInt(2, property.getMetaInfo().getOID());
//////////////////////	//
						//System.out.println("*******************************");
						//System.out.println(pstAddMappingMP_SMP);
						pstAddMappingMP_SMP.execute();

						//The mapping is done.
					}
					//References: they can't be done right now because we don't have all the construct OIDs we need!
					//We're done with this construct.
				}
				//We now have inserted all the constructs w/ their mappings, all their properties w/ their own mappings.
				//It's time we add the references, since we now have all the data we needed.
				for (Construct construct : model.getAbstractConstructs()) {
					//Ok, let's go.
					for (Reference reference : construct.getReferences()) {
						// paoloc 20060516
						// "if" added in order to handle optional (nullable) references
						if (reference.getTargetConstruct()!=null)
						{
							pstAddReference.setInt(1, reference.getOwnerConstruct().getOID()); //The same of construct.getOID() due to scope.
							pstAddReference.setString(2, reference.getName());
							pstAddReference.setString(3, reference.getTypeAsString());
							pstAddReference.setInt(4, reference.getTargetConstruct().getOID());
							//We're ready to go!
//////////////////////		//
							//System.out.println("*******************************");
							//System.out.println(pstAddReference);
							pstAddReference.execute(); //The reference is in.

							pstGetReferenceOID.setString(1, reference.getName());
							pstGetReferenceOID.setInt(2, reference.getOwnerConstruct().getOID());
							rsQueryGet = pstGetReferenceOID.executeQuery();

							rsQueryGet.next();
							referenceOID = rsQueryGet.getInt(1);
							rsQueryGet.close();
							reference.setOID(referenceOID);

							pstAddMappingMR_SMR.setInt(1, referenceOID);
							pstAddMappingMR_SMR.setInt(2, reference.getMetaInfo().getOID());
//////////////////////		//
							//System.out.println("*******************************");
							//System.out.println(pstAddMappingMR_SMR);
							pstAddMappingMR_SMR.execute(); //The mapping is in, too.

						}
					}
				}
				//STEP 2: create the tables which will hold the schemes.
				this.createModelTables(model);
				//Make it so.
				con.commit();
				con.setAutoCommit(true);
				//Okay, everything is inside the database!
				//2005-01-31: luca s: let's add the automatic generation of the copyrules.
				log.debug("calling CopyRuleMaker on the newly stored model " + model.getName());
				//String args[] = { "--model", Integer.toString(model.getOID()) };
				//CopyRulesMaker.main(args);
				CopyRulesMaker.makeCopyRules(model.getOID());
			}
			catch (SQLException sqle) {
				//Rollback any changes made to the DB, then propagate the exception.
				con.rollback();
				con.setAutoCommit(true);
				throw sqle;
			}
			catch (Exception e) { //CopyRulesMaker threw an exception.
				throw new PersistenceException(e);
			}
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Deletes the model which maps to the given modelOID.
	 * @param modelOID
	 */
	public void deleteModel(int modelOID) throws PersistenceException {
		try {
			Statement st = con.createStatement();
			//Step 1: the schemes associated to this model:
			String querySchemes = "SELECT OID FROM Scheme WHERE modelOID = " + modelOID + ";";
			ResultSet rsSchemes = st.executeQuery(querySchemes);
			while (rsSchemes.next()) {
				this.deleteScheme(rsSchemes.getInt(1)); //This call internally handles tables and instances. 
			}
			System.out.println("cancellati gli schemi");
			rsSchemes.close();
			//Step 2: the model tables:
			this.deleteModelTables(this.getModel(modelOID));
			//Step 3: the model data:
			//In order to delete it, we need some data first.
			String queryConstructs = "SELECT OID FROM MM_Construct WHERE modelOID = " + modelOID + ";";

			ResultSet constructs = st.executeQuery(queryConstructs);
			//We now have all the construct OIDs of the constructs belonging to the model identified by modelOID.
			//It's all we need! Let's rumble!
			String deleteCMappings = "DELETE FROM Mapping_MC_SMC WHERE constructOID_M = ?;";
			String deletePMappings = "DELETE FROM Mapping_MP_SMP WHERE propertyOID_M IN (" +
			"SELECT OID FROM MM_Property WHERE constructOID = ?);";
			String deleteRMappings = "DELETE FROM Mapping_MR_SMR WHERE referenceOID_M IN (" +
			"SELECT OID FROM MM_Reference WHERE ownerConstructOID = ?);";
			String deleteProperties = "DELETE FROM MM_Property WHERE constructOID = ?;";
			String deleteReferences = "DELETE FROM MM_Reference WHERE ownerConstructOID = ?;";
			String deleteConstructs = "DELETE FROM MM_Construct WHERE modelOID = " + modelOID + ";";
			String deleteModel = "DELETE FROM Model WHERE OID = " + modelOID + ";";
			//Now the prepared statements...
			PreparedStatement pstDeleteCMappings = con.prepareStatement(deleteCMappings);
			PreparedStatement pstDeletePMappings = con.prepareStatement(deletePMappings);
			PreparedStatement pstDeleteRMappings = con.prepareStatement(deleteRMappings);
			PreparedStatement pstDeleteProperties = con.prepareStatement(deleteProperties);
			PreparedStatement pstDeleteReferences = con.prepareStatement(deleteReferences);
			//Constructs and the model queries are executed with a simple Statement. 

			int constructOID = 0;
			while (constructs.next()) {
				constructOID = constructs.getInt(1);
				//The mappings go first.
				pstDeleteCMappings.setInt(1, constructOID);
				pstDeleteCMappings.execute();
				pstDeletePMappings.setInt(1, constructOID);
				pstDeletePMappings.execute();
				pstDeleteRMappings.setInt(1, constructOID);
				pstDeleteRMappings.execute();
				//Now properties and references:
				pstDeleteProperties.setInt(1, constructOID);
				pstDeleteProperties.execute();
				pstDeleteReferences.setInt(1, constructOID);
				pstDeleteReferences.execute();
			}
			//We can now delete the constructs in a single shot!
			st.execute(deleteConstructs);
			//And finally the model.. bye bye!
			st.execute(deleteModel);
			//We're done.
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * Renames the model with OID modelOID to the name modelName
	 * @param modelOID
	 * @param modelName
	 * @throws PersistenceException if the model does not exist or if the name is already taken.
	 */
	public void renameModel(int modelOID, String modelName, String shortName) throws PersistenceException {
		try {
			//Step 1: rename the tables of this model.
			con.setAutoCommit(false);
			Model model = cache.getModel(modelOID);
			String queryConstructs = "SELECT name FROM MM_Construct WHERE modelOID = " + modelOID + " ORDER BY OID;";
			Statement stConstructs = con.createStatement(), stTables = con.createStatement();
			ResultSet rsConstructs = stConstructs.executeQuery(queryConstructs);
			String constructName = null, queryRename = null;
			while (rsConstructs.next()) {
				constructName = rsConstructs.getString(1);
				queryRename = "ALTER TABLE " + model.getShortName() + "_" + constructName + " RENAME TO " + shortName + "_" + constructName + ";";
				stTables.execute(queryRename);
				//The skolem tables too:
				queryRename = "ALTER TABLE " + model.getShortName() + "_" + constructName + "_SK RENAME TO " + shortName + "_" + constructName + "_SK;";
				stTables.execute(queryRename);
			}
			/*
			//Unfortunately, MSAccess doesn't support ALTER TABLE <tablename> RENAME TO <new name> :-(
			//This is a workaround for MSAccess. Note that it destroys data!
			Model model = this.loadModel(modelOID, null);
			this.deleteModelTables(model);
			model.setName(modelName);
			model.setShortName(shortName);
			this.createModelTables(model);
			 */
			//The model tables have been renamed.
			//Step 2: rename the model itself. 
			String sqlQuery = "UPDATE Model SET name = ?, shortname = ? WHERE OID = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);

			pst.setString(1, modelName);
			pst.setString(2, shortName);
			pst.setInt(3, modelOID);

			pst.executeUpdate();
			//We're done (or an exception was thrown)
			con.commit();
			con.setAutoCommit(true);
			//2005-01-31: luca s: let's add the automatic generation of the copyrules.
			log.debug("calling CopyRuleMaker on the renamed  model " + model.getName());
			String args[] = { "--model", Integer.toString(modelOID) };
			CopyRulesMaker.main(args);

		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * @param model
	 */
	private void deleteModelTables(Model model) throws SQLException {
		LinkedList<Construct> list = new LinkedList<Construct>();
		//Build a reverse order list of the constructs. This should avoid reference violations.
		for (Construct construct : model.getAbstractConstructs()) {
			list.addFirst(construct);
		}
		//Now iterate on the list deleting tables.
		for (Construct construct : list) {
			this.deleteTable(model.getShortName() + "_" + construct.getName());
			this.deleteTable(model.getShortName() + "_" + construct.getName() + "_SK");
		}
	}

	private void deleteTable(String tableName) throws SQLException {
		Statement st = con.createStatement();
		/*
		ResultSet rsTables = con.getMetaData().getTables(null, null, tableName, null);
		if (rsTables.next()) {
			log.debug("dropping table " + tableName);
			st.execute("DROP TABLE " + tableName + ";");
			log.debug("dropped table " + tableName);
		}
		 */
		try {
			st.execute("DROP TABLE " + tableName + ";");
		}
		catch (SQLException sqle) {
		}
	}

	/**
	 * Call this inside a con.setAutoCommit(false) - call - con.commit()/con.rollback() - con.setAutoCommit(true).
	 * This method does not take action regarding auto commit. 
	 * @param model
	 * @throws SQLException
	 */
	private void createModelTables(Model model) throws SQLException {
		this.createModelTables(model, false);
	}
	protected void createSuperModelTables(Model superModel) throws SQLException {
		this.createModelTables(superModel, true);
	}
	private void createModelTables(Model model, boolean modelIsSuperModel) throws SQLException {
		String sqlQuery = null;
		Statement st = con.createStatement();
		for (Construct construct : model.getAbstractConstructs()) {
			/*
			 * Ok, what is this query supposed to look like?
			 * CREATE TABLE <modelShortName>_<constructName> (
			 * 	OID int identity primary key,
			 * 	schemeOID int not null,
			 * 	name varchar not null,
			 * 	[propertyName smPropertyType,]*
			 * 	[referenceName int,
			 * 	foreign key (referenceName) references <modelShortName>_<targetConstructName>(OID)]*
			 * 	unique(schemeOID, name)
			 */
			sqlQuery = "CREATE TABLE " + model.getShortName() + "_" + construct.getName() + " (\n";
			// 2005-01-29, luca s: niente pi? autoincrement, si usano le skolem.
			sqlQuery += "\tOID int primary key,\n";//"\tOID int identity primary key,\n";
			sqlQuery += "\tschemeOID int not null,\n";
			sqlQuery += "\tname varchar not null,\n";
			//Now the properties.
			for (Property property : construct.getProperties()) {
				if (modelIsSuperModel == false) //It's a model.
					sqlQuery += "\t" + property.getName() + " " + property.getMetaInfo().getType() + ",\n";
				else //It's the super model.
					sqlQuery += "\t" + property.getName() + " " + property.getType() + ",\n";
			}
			//The references are made of two lines, e.g.:
			//	abstractOID int, -> the "declaration"
			//	foreign key (abstractOID) references ER2_Entita(OID) -> the "referenced field"
			//All the "declarations" must be done before any "referenced field", otherwise a SQL exception will be thrown.
			String declarations = "", referencedFields = ""; //Buffer-like strings to avoid looping twice.
			for (Reference reference : construct.getReferences()) {
				// paoloc 20060516
				// "if" added in order to handle optional (nullable) references
				if (reference.getTargetConstruct()!=null)
				{
					declarations += "\t" + reference.getName() + " int,\n";
					referencedFields += "\tforeign key (" + reference.getName() + ") references "
					+ model.getShortName() + "_" + reference.getTargetConstruct().getName() + "(OID),\n";
				}
			}
			//Now that we have them, let's put 'em in the query.
			sqlQuery += declarations;
			sqlQuery += referencedFields;
			if (modelIsSuperModel == false) //It's a model, the schemeOID reference points to Scheme.
				sqlQuery += "\tforeign key (schemeOID) references Scheme(OID)\n";
			else //It's the super model, it points to a different table.
				sqlQuery += "\tforeign key (schemeOID) references SchemeSM(OID)\n";
			//sqlQuery += "\tunique(schemeOID, name)\n"; //Note: this breaks constructs with descriptive references.
			sqlQuery += ");";
			//The query is ready.
			st.execute(sqlQuery);
			//Now the skolem table:
			sqlQuery = "CREATE TABLE " + model.getShortName() + "_" + construct.getName() + "_SK (\n";
			sqlQuery += "\tOID int identity,\n";
			sqlQuery += "\tschemeOID int not null,\n";

			//sqlQuery += "\tOID_old int not null\n";
			//modificato da paolo c 11-06-2004 ore 18,19
			//sqlQuery += "\tOID_old int not null\n";
			//modificato da luca s 2005-01-29: non capisco la precedente modifica, a me sembra che le due righe siano uguali O_O;
			//comunque a me ora serve che OID_old possa essere nullo per poter usare le funzioni di skolem per ottenere gli OID
			//nel salvataggio degli schemi "nuovi" (non ottenuti da traduzioni, ma inseriti da tool)
			sqlQuery += "\tOID_Copy int\n";

			sqlQuery += ");";
			st.execute(sqlQuery);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#doesSchemeExist(java.lang.String)
	 */
	public boolean doesSchemeExist(String schemeName) throws PersistenceException {
		try {
			String sqlQuery = "SELECT oid FROM Scheme WHERE name = ?;";
			PreparedStatement pst = con.prepareStatement(sqlQuery);
			pst.setString(1, schemeName);
			ResultSet result = pst.executeQuery();

			return result.next();
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#getSchemes()
	 */
	public Collection<Scheme> getSchemes() throws PersistenceException {
		LinkedList<Scheme> schemes = new LinkedList<Scheme>();
		try {
			String sqlQuery = "SELECT S.OID AS schemeOID, S.name AS schemeName, M.OID AS modelOID, M.name AS modelName, M.shortname AS modelShortName FROM Scheme S, Model M WHERE S.modelOID = M.OID;";
//			String sqlQuery = "SELECT S.OID AS schemeOID, S.name AS schemeName FROM Scheme S;";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			while (result.next()) {
				/*Model model = new Model();
				model.setName(result.getString("modelName"));
				model.setShortName(result.getString("modelShortName"));
				model.setOID(result.getInt("modelOID"));*/

				/* riccardop: ora punta al modello condiviso: creato solo se necessario */
				Model model = this.cache.getModel(result.getInt("modelOID"));
				int oid = result.getInt("schemeOID");
				String name = result.getString("schemeName");
				Scheme scheme = new Scheme(oid, name, model, false);

				schemes.add(scheme);
			}
			return schemes;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#getSuperModelSchemes()
	 */
	public Collection<Scheme> getSuperModelSchemes() throws PersistenceException {
		LinkedList<Scheme> schemes = new LinkedList<Scheme>();
		try {
			String sqlQuery = "SELECT S.OID AS schemeOID, S.name AS schemeName, M.OID AS modelOID, M.name AS modelName, M.shortname AS modelShortName FROM SchemeSM S, Model M WHERE S.modelOID = M.OID;";
//			String sqlQuery = "SELECT S.OID AS schemeOID, S.name AS schemeName FROM Scheme S;";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			while (result.next()) {
				/*Model model = new Model();
				model.setName(result.getString("modelName"));
				model.setShortName(result.getString("modelShortName"));
				model.setOID(result.getInt("modelOID"));*/

				/* riccardop: ora punta al modello condiviso: creato solo se necessario */
				Model model = this.cache.getModel(result.getInt("modelOID"));
				int oid = result.getInt("schemeOID");
				String name = result.getString("schemeName");
				Scheme scheme = new Scheme(oid, name, model, true);

				schemes.add(scheme);
			}
			return schemes;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#deleteScheme(int)
	 */
	public void deleteScheme(int schemeOID) throws PersistenceException {
		/* reset degli schemi e modelli memorizzati. Si forza il loro reload dal db */
		/* per cancellare solo le tabelle che effettivamente esistono */
		cache.clean("schemes");
		cache.clean("models");
		Scheme scheme = cache.getScheme(schemeOID);
		//Add table deletion when we'll store instances.
		try {
			con.setAutoCommit(false);
			//For each construct in the model (-> for each table which stores scheme data) delete what belongs to this scheme. 
			String tableName = null, queryDelete = null;//It's parametric.
			Statement st = con.createStatement();
			if(scheme!=null){
				LinkedList<String> queue = new LinkedList<String>();
				for (Construct c : scheme.getMetaInfo().getAbstractConstructs()) {
					tableName = scheme.getMetaInfo().getShortName() + "_" + c.getName();
					//queryDelete = "DELETE FROM " + tableName + " WHERE schemeOID = " + schemeOID + ";";
					//st.execute(queryDelete);
					queue.addFirst(tableName); //Build up a queue.
					//Bye bye...
				}
				for (int i = 0; i < queue.size(); i++) { //By using a reverse order, we should avoid a breakage due to dependencies.
					//TODO: (pri: medium) switch to code which relies on true dependencies, and not list ordering.
					tableName = queue.get(i);

					queryDelete = "DELETE FROM " + tableName + " WHERE schemeOID = " + schemeOID + ";";
					st.execute(queryDelete);
					//Bye bye...
				}
				//Every construct (-> table) has been cleared. Now the "master" table.
			}
			queryDelete = "DELETE FROM Scheme WHERE OID = " + schemeOID + ";";
			st.execute(queryDelete);
			con.commit();
			con.setAutoCommit(true);
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#renameScheme(int, java.lang.String)
	 */
	public void renameScheme(int schemeOID, String schemeName) throws PersistenceException {
		//TODO: add table handling, when tables will be required (e.g.: we're storing instances).
		try {
			String queryRename = "UPDATE Scheme SET name = ? WHERE OID = ?;";
			PreparedStatement pstRename = con.prepareStatement(queryRename);
			pstRename.setString(1, schemeName);
			pstRename.setInt(2, schemeOID);
			pstRename.execute(); 
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	public Scheme loadSuperModelSchemeInternally(int schemeOID) throws PersistenceException {
		log.debug("Loading supermodel's scheme " + schemeOID);
		try {
			String sqlQuery = "SELECT * FROM SchemeSM WHERE OID = " + schemeOID + ";";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			result.next();
			int oid = result.getInt("OID");
			String name = result.getString("name");
			Model superModel = cache.getSuperModel();
			Scheme scheme = new Scheme(oid, name, superModel, true);
			Model referenceModel = cache.getModel(result.getInt("modelOID"));
			result.close();
			//The basic data, the model (with the supermodel) have been loaded, let's now proceed with the objects.
			//They depend on the model.
			String tableName = null;
			/*
			 * Since references can be cascading to many objects, we're going to load them in a second step,
			 * using their id and scheme.getObject(int oid).
			 */
			scheme.initSchemeObjects();
			for (Construct c : scheme.getMetaInfo().getAbstractConstructs()) {
				tableName = scheme.getMetaInfo().getShortName() + "_" + c.getName();
				sqlQuery = "SELECT * FROM " + tableName + " WHERE schemeOID = " + scheme.getOID() + ";";
				result  = st.executeQuery(sqlQuery);
				//result now contains ALL THE OBJECTS mapped to the construct c.
				while (result.next()) {
					int oidObj = result.getInt("OID");
					String nameObj = result.getString("name");
					SchemeObject object = new SchemeObject(oidObj, nameObj, c);
					//Now the properties: they are columns in the resultset.
					for (Property p : c.getProperties()) {
						if (TypeTranslator.getJavaType(p.getType()).equals(Boolean.class)) {
								object.setProperty(
										p.getName(),
										result.getBoolean(
												p.getName()));
						}
						else {
							object.setProperty(p.getName(), result.getString(p.getName()));
						}
						//The property has been set.
					}
					//All the properties have been set. We're not adding the references yet.
					scheme.addObject(object);
				}
				//All the objects mapped to the construct c have been loaded (except for their references).
			}
			//All the objects from all the constructs have been loaded (except for their references)
			//Now we can do a second loop to match the references (since all the objects now do exist).

			for (SchemeObject object : scheme.getAbstractConstructs()) {
				for (SchemeObjectReference sor : object.getReferences()) {
					//Ok, this is a reference to a construct of type sor.getModelReference().getTargetConstruct().
					//It is stored in a table of object.getConstruct().getName();
					tableName = scheme.getMetaInfo().getShortName() + "_" + object.getMetaInfo().getName();
					sqlQuery = "SELECT " + sor.getMetaInfo().getName()
					+ " FROM " + tableName
					+ " WHERE OID = " + object.getOID() + ";";
					result = st.executeQuery(sqlQuery);
					result.next();
					int referenceOID;
					if (result.getObject(1) == null) referenceOID = -1;
					else referenceOID = result.getInt(1);
//					int referenceOID = result.getInt(1);
					//We have the OID of the target object, let's get it from the scheme.
					SchemeObject target = scheme.getObject(referenceOID, sor.getMetaInfo().getTargetConstruct().getName());
					object.setReference(sor.getMetaInfo().getName(), target);
				}
			}
			scheme.setMetaInfo(referenceModel);
			return scheme;
		}
		catch (SQLException sqle) {
			sqle.printStackTrace(System.out);
			throw new PersistenceException(sqle);
		}

	}
	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#loadScheme(int)
	 */
	public Scheme loadSchemeInternally(int schemeOID) throws PersistenceException {
		log.debug("Loading scheme " + schemeOID);
		try {
			String sqlQuery = "SELECT * FROM Scheme WHERE OID = " + schemeOID + ";";
			Statement st = con.createStatement();
			ResultSet result = st.executeQuery(sqlQuery);
			boolean hasRow = result.next();

			if(hasRow == false) {
				return null;
			}

			int oid = result.getInt("OID");
			String name = result.getString("name");
			Model model = cache.getModel(result.getInt("modelOID"));
			Scheme scheme = new Scheme(oid, name, model, false);
			result.close();
			//The basic data, the model (with the supermodel) have been loaded, let's now proceed with the objects.
			//They depend on the model.
			String tableName = null;
			/*
			 * Since references can be cascading to many objects, we're going to load them in a second step,
			 * using their id and scheme.getObject(int oid).
			 */
			scheme.initSchemeObjects();
			for (Construct c : scheme.getMetaInfo().getAbstractConstructs()) {
				tableName = scheme.getMetaInfo().getShortName() + "_" + c.getName();
				sqlQuery = "SELECT * FROM " + tableName + " WHERE schemeOID = " + scheme.getOID() + ";";
				result  = st.executeQuery(sqlQuery);
				//result now contains ALL THE OBJECTS mapped to the construct c.
				while (result.next()) {
					int oidObj = result.getInt("OID");
					String nameObj = result.getString("name");
					SchemeObject object = new SchemeObject(oidObj, nameObj ,c);
					//Now the properties: they are columns in the resultset.
					for (Property p : c.getProperties()) {
						if (TypeTranslator.getJavaType(p.getMetaInfo().getType()).equals(Boolean.class)) {
							object.setProperty(p.getName(), result.getBoolean(p.getName()));
						}
						else {
							object.setProperty(p.getName(), result.getString(p.getName()));
						}
						//The property has been set.
					}
					//All the properties have been set. We're not adding the references yet.
					scheme.addObject(object);
				}
				//All the objects mapped to the construct c have been loaded (except for their references).
			}
			//All the objects from all the constructs have been loaded (except for their references)
			//Now we can do a second loop to match the references (since all the objects now do exist).

			for (SchemeObject object : scheme.getAbstractConstructs()) {
				for (SchemeObjectReference sor : object.getReferences()) {
					//Ok, this is a reference to a construct of type sor.getModelReference().getTargetConstruct().
					//It is stored in a table of object.getConstruct().getName();
					tableName = scheme.getMetaInfo().getShortName() + "_" + object.getMetaInfo().getName();
					sqlQuery = "SELECT " + sor.getMetaInfo().getName()
					+ " FROM " + tableName
					+ " WHERE OID = " + object.getOID() + ";";
					result = st.executeQuery(sqlQuery);
					result.next();
					int referenceOID;
					if (result.getObject(1) == null) referenceOID = -1;
					else referenceOID = result.getInt(1);
					//We have the OID of the target object, let's get it from the scheme.
					SchemeObject target = scheme.getObject(referenceOID, sor.getMetaInfo().getTargetConstruct().getName());
					object.setReference(sor.getMetaInfo().getName(), target);
				}
			}
			return scheme;
		}
		catch (SQLException sqle) {
			sqle.printStackTrace(System.out);
			throw new PersistenceException(sqle);
		}
	}

	// this method is very bad: it is a cross M to SM store method. 
	// It should be replaced by a converter M to SM and viceversa
	// and then use a normal storeScheme method.
	public void storeModelSchemeIntoSuperModel(Scheme scheme) throws PersistenceException {
		try {
			con.setAutoCommit(false); //We're going to rollback in case of errors.

			String insertScheme = "INSERT INTO SchemeSM(name, modelOID, OID, description, sourceSchemeOID) VALUES (?, ?, ?,  ?, ?);"; 

			String tableName = null, insertQuery = null, updateField = null; //Will be set up later.

			PreparedStatement pstInsertScheme = con.prepareStatement(insertScheme);
			//PreparedStatement pstGetSchemeOID = con.prepareStatement(getSchemeOID); //2004-06-07
			PreparedStatement pstInsertQuery = null; //Will be set up later due to being parametric on the table.
			PreparedStatement pstUpdateField = null; //Guess what?

			pstInsertScheme.setString(1, scheme.getName());
			pstInsertScheme.setInt(2, scheme.getMetaInfo().getOID());
			scheme.setOID(getNewOID("SchemeSM", "OID")); 
			pstInsertScheme.setInt(3, scheme.getOID());
			pstInsertScheme.setString(4, null);
			pstInsertScheme.setInt(5, scheme.getOID());

			pstInsertScheme.execute();

			// 2005-01-29: luca s: we need to use the skolem function to obtain the OIDs.
			for (SchemeObject object : scheme.getAbstractConstructs()) {
				Construct supermodelConstruct = object.getMetaInfo().getSuperModelConstructs().iterator().next();
				tableName = "SM" + "_" + supermodelConstruct.getName();
				//We need to use the skolem functions to obtain a new OID.
				String skolemInsert = "INSERT INTO " + tableName + "_SK(schemeOID) VALUES(" + scheme.getOID() + ");";

				con.createStatement().execute(skolemInsert); //This one creates the tuple, OID_old has been left null.
				String skolemQuery = "SELECT max(OID) FROM " + tableName + "_SK;";
				ResultSet rsSkolem = con.createStatement().executeQuery(skolemQuery);
				rsSkolem.next(); //There can be only one tuple here.
				object.setOID(rsSkolem.getInt(1)); //Okay, the object contains the valid id, we can save it.
//				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID) VALUES(?, ?);";
				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID, OID) VALUES(?, ?, ?);";
				pstInsertQuery = con.prepareStatement(insertQuery);
				pstInsertQuery.setString(1, object.getName());
				pstInsertQuery.setInt(2, scheme.getOID());
				pstInsertQuery.setInt(3, object.getOID()); //2005-01-29, luca s.

				pstInsertQuery.execute(); //Ok, the object is in.

				//Okay, the object is stored and its id is known. Let's start with the small/simple queries for the properties.
				for (SchemeObjectProperty sop : object.getProperties()) {
					updateField = "UPDATE " + tableName + " SET " + sop.getMetaInfo().getMetaInfo().getName() + " = ? WHERE OID = ?;";
					pstUpdateField = con.prepareStatement(updateField); //This step is needed! In each loop, the string is different!
					if (TypeTranslator.getJavaType(sop.getMetaInfo().getMetaInfo().getType()).equals(Boolean.class)) {
						if (sop.getValue().equals("true"))
							pstUpdateField.setBoolean(1, true);
						else
							pstUpdateField.setBoolean(1, false);
					}
					else //It's a string.
						pstUpdateField.setString(1, sop.getValue());
					pstUpdateField.setInt(2, object.getOID());
					//It's ready.
					pstUpdateField.execute();
				}

				//Now the references. They are similar, but different.
				for (SchemeObjectReference sor : object.getReferences()) {
					updateField = "UPDATE " + tableName + " SET " + sor.getMetaInfo().getMetaInfo().getName() + " = ? WHERE OID = ?;";
					pstUpdateField = con.prepareStatement(updateField);
//					paoloc 20060511
					// handling the null (optional) references
					if (sor.getValue() == null) pstUpdateField.setObject(1, null);
					else pstUpdateField.setInt(1, sor.getValue().getOID());
					pstUpdateField.setInt(2, object.getOID());
					pstUpdateField.execute();
				}
				//Everything is in for this particular object.
			}
			//All the objects are in.
			con.commit();
			con.setAutoCommit(true);
		}
		catch (SQLException sqle) {
			//First try to rollback any change...
			try {
				con.rollback();
				con.setAutoCommit(true);
			}
			catch (SQLException sqle2) { //I guess if this will ever happen!!
				throw new PersistenceException(sqle2);
			}
			throw new PersistenceException(sqle);
		}

	}

	public void storeSuperModelSchemeIntoModel(Scheme superModelScheme) throws PersistenceException {
		try {
			con.setAutoCommit(false); //We're going to rollback in case of errors.

			String insertScheme = "INSERT INTO Scheme(name, modelOID, OID) VALUES (?, ?, ?);"; 

			String tableName = null, insertQuery = null, updateField = null; //Will be set up later.

			Model targetModel = superModelScheme.getMetaInfo();

			PreparedStatement pstInsertScheme = con.prepareStatement(insertScheme);
			//PreparedStatement pstGetSchemeOID = con.prepareStatement(getSchemeOID); //2004-06-07
			PreparedStatement pstInsertQuery = null; //Will be set up later due to being parametric on the table.
			PreparedStatement pstUpdateField = null; //Guess what?

			pstInsertScheme.setString(1, superModelScheme.getName());
			pstInsertScheme.setInt(2, targetModel.getOID());
			superModelScheme.setOID(getNewOID("Scheme", "OID")); 
			pstInsertScheme.setInt(3, superModelScheme.getOID());
			pstInsertScheme.execute();

			SchemeObjectReference sor = null;
			// 2005-01-29: luca s: we need to use the skolem function to obtain the OIDs.
			for (SchemeObject object : superModelScheme.getAbstractConstructs()) {
				Construct modelConstruct = null;
//				TODO check delle references: non basta il nome
//				attualemente il controllo si ferma al primo costrutto del modello
//				target associato al costrutto del sm di cui lo schema-object
//				corrente e' associato. Nel nostro caso, dato che ad es. Lexical
//				puo' essere associato a piu' costrutti (attrOfEnt, attrOfRel, ...)
//				questo equivale a non scegliere il costrutto corretto, ma solo 
//				il primo che capita
//				MODIFICA: fare un check sulle ref associate al costrutto del
//				modello target.
				for (Construct construct : superModelScheme.getMetaInfo().getAbstractConstructs()) {
					if ((construct.getSuperModelConstructs().iterator().next()).getName().equals(object.getMetaInfo().getName()))
					{
						modelConstruct = construct;
						tableName =  targetModel.getShortName() + "_" + modelConstruct.getName();
						break;
					}
				}
				//We need to use the skolem functions to obtain a new OID.
				String skolemInsert = "INSERT INTO " + tableName + "_SK(schemeOID) VALUES(" + superModelScheme.getOID() + ");";
				con.createStatement().execute(skolemInsert); //This one creates the tuple, OID_old has been left null.
				String skolemQuery = "SELECT max(OID) FROM " + tableName + "_SK;";
				ResultSet rsSkolem = con.createStatement().executeQuery(skolemQuery);
				rsSkolem.next(); //There can be only one tuple here.
				object.setOID(rsSkolem.getInt(1)); //Okay, the object contains the valid id, we can save it.
//				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID) VALUES(?, ?);";
				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID, OID) VALUES(?, ?, ?);";
				pstInsertQuery = con.prepareStatement(insertQuery);
				pstInsertQuery.setString(1, object.getName());
				pstInsertQuery.setInt(2, superModelScheme.getOID());
				pstInsertQuery.setInt(3, object.getOID()); //2005-01-29, luca s.
				pstInsertQuery.execute(); //Ok, the object is in.

				//Okay, the object is stored and its id is known. Let's start with the small/simple queries for the properties.

				if (modelConstruct != null) {
					for (Property property : modelConstruct.getProperties()) {
						Property modelProperty = null;
						for (SchemeObjectProperty sop : object.getProperties()) {
							if (property.getMetaInfo().getName().equals(sop.getMetaInfo().getName()))
							{
								modelProperty = property;
								updateField = "UPDATE " + tableName + " SET " + modelProperty.getName() + " = ? WHERE OID = ?;";
								pstUpdateField = con.prepareStatement(updateField); //This step is needed! In each loop, the string is different!
								if (TypeTranslator.getJavaType(modelProperty.getType()).equals(Boolean.class)) {
									if (sop.getValue().equals("true"))
										pstUpdateField.setBoolean(1, true);
									else
										pstUpdateField.setBoolean(1, false);
								}
								else //It's a string.
									pstUpdateField.setString(1, sop.getValue());
								pstUpdateField.setInt(2, object.getOID());
								//It's ready.
								pstUpdateField.execute();
								break;
							}
						}
					}
				}

				//Now the references. They are similar, but different.

				if(modelConstruct != null) {
					for(Reference reference : modelConstruct.getReferences()) {
						Reference modelReference = null;
						Iterator<SchemeObjectReference> j = object.getReferences().iterator();
						boolean done = false;
						while ((!done)&&j.hasNext())
						{
							sor = j.next();
							if (reference.getMetaInfo().getName().equals(sor.getMetaInfo().getName()))
							{
								modelReference = reference;
								updateField = "UPDATE " + tableName + " SET " + modelReference.getName() + " = ? WHERE OID = ?;";
								pstUpdateField = con.prepareStatement(updateField);
								//							 paoloc 20060511
								// handling the null (optional) references
								if (sor.getValue() == null) pstUpdateField.setObject(1, null);
								else pstUpdateField.setInt(1, sor.getValue().getOID());
								pstUpdateField.setInt(2, object.getOID());
								pstUpdateField.execute();
								done = true;
							}
						}
					}
				}
			}
			//All the objects are in.
			con.commit();
			con.setAutoCommit(true);
		}
		catch (SQLException sqle) {
			//First try to rollback any change...
			try {
				con.rollback();
				con.setAutoCommit(true);
			}
			catch (SQLException sqle2) { //I guess if this will ever happen!!
				throw new PersistenceException(sqle2);
			}
			throw new PersistenceException(sqle);
		}

	}


	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#storeScheme(org.modelgen.commons.beans.scheme.Scheme)
	 */
	public void storeScheme(Scheme scheme) throws PersistenceException {
		try {
			con.setAutoCommit(false); //We're going to rollback in case of errors.
			/*
			 * 2004-06-07: After a discussion about MakeCopyRules and the Translation rules with Paolo C.
			 * we have decided to remove the "auto_increment" property from the Scheme (and SchemeSM) tables.
			 * As a consequence, we now need to use the getNewOID() function. The original code will be
			 * left commented out in case of a change back to the auto_increment property.  
			 */
			//String insertScheme = "INSERT INTO Scheme(name, modelOID) VALUES (?, ?);";
			String insertScheme = "INSERT INTO Scheme(name, modelOID, OID) VALUES (?, ?, ?);"; //2004-06-07
			//String getSchemeOID = "SELECT OID FROM Scheme WHERE name = ? AND modelOID = ?;"; //2004-06-07
			String tableName = null, insertQuery = null, updateField = null; //Will be set up later.

			PreparedStatement pstInsertScheme = con.prepareStatement(insertScheme);
			//PreparedStatement pstGetSchemeOID = con.prepareStatement(getSchemeOID); //2004-06-07
			PreparedStatement pstInsertQuery = null; //Will be set up later due to being parametric on the table.
			PreparedStatement pstUpdateField = null; //Guess what?

			pstInsertScheme.setString(1, scheme.getName());
			pstInsertScheme.setInt(2, scheme.getMetaInfo().getOID());
			scheme.setOID(getNewOID("Scheme", "OID")); //2004-06-07
			pstInsertScheme.setInt(3, scheme.getOID()); //2004-06-07
			pstInsertScheme.execute();
			//The scheme has been saved and its OID been read, let's proceed.
			/*
			 * Okay, let's stop for a while and think about how to make the next query.
			 * A possible way would be: loop on all the objects, for each one build a query like
			 * "INSERT INTO <modelShortname>_<constructName>(field1, field2, ...) VALUES(value1, value2, ...);"
			 * where the fields and the values are obtained looping on the properties (and references).
			 * This would be nice, but it's not going to work fine: the "values" are all strings and this query would not
			 * allow us to use a PreparedStatement unless we build a complicated way to remember the index of each field...
			 * 
			 * Let's try a simpler approach: we know that the key of an object is "schemeOID + name" and that the name surely exists.
			 * We can use a simple query like "INSERT INTO <tableName>(name, schemeOID) VALUES(<name>, <schemeOID>);"
			 * and then, looping on properties (and references), a simple query realized with a PreparedStatement: 
			 * "UPDATE <tableName> SET <field> = ? WHERE name = ? AND schemeOID = ?;"
			 * Or... put a "SELECT OID FROM <tableName> WHERE name = ? AND schemeOID = ?;" to get the OID
			 * and use it as superkey. It's just like the same, we will need that query nevertheless. :-)
			 */
			// 2005-01-29: luca s: we need to use the skolem function to obtain the OIDs.
			for (SchemeObject object : scheme.getAbstractConstructs()) {
				tableName = scheme.getMetaInfo().getShortName() + "_" + object.getMetaInfo().getName();
				//We need to use the skolem functions to obtain a new OID.
				String skolemInsert = "INSERT INTO " + tableName + "_SK(schemeOID) VALUES(" + scheme.getOID() + ");";
				con.createStatement().execute(skolemInsert); //This one creates the tuple, OID_old has been left null.
				String skolemQuery = "SELECT max(OID) FROM " + tableName + "_SK;";
				ResultSet rsSkolem = con.createStatement().executeQuery(skolemQuery);
				rsSkolem.next(); //There can be only one tuple here.
				object.setOID(rsSkolem.getInt(1)); //Okay, the object contains the valid id, we can save it.
//				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID) VALUES(?, ?);";
				insertQuery = "INSERT INTO " + tableName + "(name, schemeOID, OID) VALUES(?, ?, ?);";
				pstInsertQuery = con.prepareStatement(insertQuery);
				pstInsertQuery.setString(1, object.getName());
				pstInsertQuery.setInt(2, scheme.getOID());
				pstInsertQuery.setInt(3, object.getOID()); //2005-01-29, luca s.
				pstInsertQuery.execute(); //Ok, the object is in.

				/*
				 * The following isn't needed anymore having used the skolem functions to obtain the oid earlier.
				getObjectOID = "SELECT OID FROM " + tableName + " WHERE name = ? AND schemeOID = ?;";
				pstGetObjectOID = con.prepareStatement(getObjectOID);
				pstGetObjectOID.setString(1, object.getName());
				pstGetObjectOID.setInt(2, scheme.getOID());
				rsObjectOID = pstGetObjectOID.executeQuery(); //Okay, got it.
//				TODO: this is not safe, due to the (current) absence of a check on the SchemeObjectKey.
				while (rsObjectOID.next()) {
					 object.setOID(rsObjectOID.getInt(1)); //This way, the latest OID is used. Yes, it's unsafe.
				}
				 */
				//Okay, the object is stored and its id is known. Let's start with the small/simple queries for the properties.
				for (SchemeObjectProperty sop : object.getProperties()) {
					updateField = "UPDATE " + tableName + " SET " + sop.getMetaInfo().getName() + " = ? WHERE OID = ?;";
					pstUpdateField = con.prepareStatement(updateField); //This step is needed! In each loop, the string is different!
					if (TypeTranslator.getJavaType(sop.getMetaInfo().getMetaInfo().getType()).equals(Boolean.class)) {
						if (sop.getValue().equals("true"))
							pstUpdateField.setBoolean(1, true);
						else
							pstUpdateField.setBoolean(1, false);
					}
					else //It's a string.
						pstUpdateField.setString(1, sop.getValue());
					pstUpdateField.setInt(2, object.getOID());
					//It's ready.
					pstUpdateField.execute();
				}
				//Now the references. They are similar, but different.
				for (SchemeObjectReference sor : object.getReferences()) {
					updateField = "UPDATE " + tableName + " SET " + sor.getMetaInfo().getName() + " = ? WHERE OID = ?;";
					pstUpdateField = con.prepareStatement(updateField);
//					paoloc 20060511
					// handling the null (optional) references
					if (sor.getValue() == null) pstUpdateField.setObject(1, null);
					else pstUpdateField.setInt(1, sor.getValue().getOID());
					pstUpdateField.setInt(2, object.getOID());
					pstUpdateField.execute();
				}
				//Everything is in for this particular object.
			}
			//All the objects are in.
			con.commit();
			con.setAutoCommit(true);
		}
		catch (SQLException sqle) {
			//First try to rollback any change...
			try {
				con.rollback();
				con.setAutoCommit(true);
			}
			catch (SQLException sqle2) { //I guess if this will ever happen!!
				throw new PersistenceException(sqle2);
			}
			throw new PersistenceException(sqle);
		}

	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#getSkolemFunctions()
	 */
	public Map<String,SkolemFunction> getMappaFuntoriSkolem() throws PersistenceException {
		HashMap<String,SkolemFunction>  map = new HashMap<String,SkolemFunction>();
		//Read all the available Skolem Functions
		String queryFunctions = "SELECT F.OID, F.name AS functionName, C.name AS constructName FROM MSM_SkolemFunction AS F JOIN MSM_Construct AS C ON F.constructOID = C.OID;";
		String queryArguments = "SElECT * FROM MSM_SkolemFunctionArgument WHERE functionOID = ?;";
		ResultSet rsFunctions = null, rsArguments = null;
		try {
			Statement st = con.createStatement();
			PreparedStatement pstArguments = con.prepareStatement(queryArguments);
			rsFunctions = st.executeQuery(queryFunctions);
			while (rsFunctions.next()) {
				//Create a SkolemFunction object, fill it with the basic data, then load its arguments.
				String targetTable = "SM_" + rsFunctions.getString(/*"C.name"*/"constructName");
				String functionName = rsFunctions.getString(/*"F.name"*/"functionName");
				int functionOID = rsFunctions.getInt("OID");
				SkolemFunction skf = new SkolemFunction(
						targetTable,
						"OID", //Skolem function target oid. It is always "OID". Historically, it could have different values, but it has been standardized into "OID".
						functionName);
				//Now loop on its arguments.
				pstArguments.setInt(1, functionOID);
				rsArguments = pstArguments.executeQuery();
				while (rsArguments.next()) {
					skf.addArgumentName(rsArguments.getString("name"));
				}
				//Okay, the skolem function is ready.
				map.put(functionName, skf);
			}
			//All the skolem functions have been loaded.
			if (map.isEmpty())
				return null;
			else
				return map;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#storeSkolemFunctions(java.util.Map)
	 */
	public void storeSkolemFunctions(Map<String,SkolemFunction> map) throws PersistenceException {
		/*
		 * luca s 2004-11-15:
		 * I do not like the current implementation of the SkolemFunction since it contains derived
		 * data (e.g.: skolemFunctionTableName) in place of object oriented code (e.g.: a reference
		 * to the Construct object).
		 * To cope with the current implementation, this method will have some code which can be removed when (and if)
		 * the SkolemFunction class gets a "facelift".
		 */
		Model superModel = cache.getSuperModel();
		String insertFunction = "INSERT INTO MSM_SkolemFunction(name, constructOID) VALUES(?,?);";
		String queryGetOID = "SELECT OID FROM MSM_SkolemFunction WHERE name = ? AND constructOID = ?;";
		String insertArgument = "INSERT INTO MSM_SkolemFunctionArgument(name, functionOID, type) VALUES(?, ?, ?);";
		try {
			PreparedStatement pstInsertFunction = con.prepareStatement(insertFunction);
			PreparedStatement pstGetOID = con.prepareStatement(queryGetOID);
			PreparedStatement pstInsertArgument = con.prepareStatement(insertArgument);
			ResultSet rsOID = null;
			int functionOID = -1;

			//Iterate on the available SkolemFunctions:
			for (Iterator iter = map.values().iterator(); iter.hasNext();) {
				SkolemFunction skf = (SkolemFunction) iter.next();
				pstInsertFunction.setString(1, skf.getSkolemFunctionName());
				String constructName = skf.getTagetTable().substring(3, skf.getTagetTable().length() - 3); // modificato LF

				log.debug("The construct for the skolem function " + skf.getSkolemFunctionName() + " is: " + constructName);
				int constructOID = superModel.getConstruct(constructName).getOID();
				pstInsertFunction.setInt(2, constructOID);

				pstInsertFunction.execute();
				pstGetOID.setString(1, skf.getSkolemFunctionName());
				pstGetOID.setInt(2, constructOID);
				rsOID = pstGetOID.executeQuery();
				rsOID.next();
				functionOID = rsOID.getInt("OID");

				//Loop on the arguments.

				for (Iterator iterator = skf.getArgomenti().iterator(); iterator.hasNext();) { //modificato LF
					//TODO: change the "arguments" in the SkolemFunction to be classes with name and type.
					String argumentName = (String) iterator.next();
					String argumentType = "integer";
					//When the above change is done, we can replace "integer" with the real type.
					pstInsertArgument.setString(1, argumentName);
					pstInsertArgument.setInt(2, functionOID);
					pstInsertArgument.setString(3, argumentType);

					pstInsertArgument.execute();

				}
				//All the arguments have been inserted.
			}
			//All the available SkolemFunctions have been stored.
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#getSkolemFunction(org.modelgen.commons.beans.model.Construct, java.lang.String)
	 */
	public org.modelgen.commons.beans.skolem.SkolemFunction getSkolemFunction(Model model, Construct construct, String skfName) throws PersistenceException {
		String sqlQuery = null;
		if (model.getShortName().equals("SM")) {
			sqlQuery = "SELECT * FROM MSM_SkolemFunction WHERE constructOID = ? AND name = ?;";
		}
		else {
			//TODO: do this.
			throw new PersistenceException("Model skolem functions aren't supported yet.");
		}
		try {
			PreparedStatement pstFunction = con.prepareStatement(sqlQuery);

			pstFunction.setInt(1, construct.getOID());
			pstFunction.setString(2, skfName);

			ResultSet result = pstFunction.executeQuery();
			org.modelgen.commons.beans.skolem.SkolemFunction skf = null;
			while (result.next()) {
				skf = new org.modelgen.commons.beans.skolem.SkolemFunction();
				skf.setOID(result.getInt("OID"));
				skf.setName(result.getString("name"));
				skf.setOwnerConstruct(construct);
				break;
			}
			if (skf != null) {
				sqlQuery = "SELECT * FROM MSM_SkolemFunctionArgument WHERE functionOID = ?;";
				PreparedStatement pstArguments = con.prepareStatement(sqlQuery);

				pstArguments.setInt(1, skf.getOID());

				result = pstArguments.executeQuery();
				while (result.next()) {
					org.modelgen.commons.beans.skolem.SkolemFunctionArgument skfa = new org.modelgen.commons.beans.skolem.SkolemFunctionArgument();
					skfa.setOID(result.getInt("oid"));
					skfa.setName(result.getString("name"));
					skfa.setType(result.getString("type"));
					skfa.setSourceConstruct(getSuperModel().getConstruct(result.getString("sourceConstructOID")));
					skf.addArgument(skfa);
				}
			}
			return skf;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#storeSkolemFunction(org.modelgen.commons.beans.model.Model, org.modelgen.commons.beans.skolem.SkolemFunction)
	 */
	public void storeSkolemFunction(Model model, org.modelgen.commons.beans.skolem.SkolemFunction skolemFunction) throws PersistenceException {
		//Add a tuple to M(S)M_SkolemFunction and M(S)M_SkolemFunctionargument
		//Do N ALTER TABLE on modelShortName_Constructname_SK
		String tablePrefix = null;
		if (model.getShortName().equals("SM")) {
			tablePrefix = "MSM_";
		}
		else {
			tablePrefix = "MM_";
		}
		String sqlQuery = "INSERT INTO " + tablePrefix + "SkolemFunction(name, constructOID) VALUES(?, ?);";

		try {
//			storing function metadata
			PreparedStatement pstFunction = con.prepareStatement(sqlQuery);
			pstFunction.setString(1, skolemFunction.getName());
			pstFunction.setInt(2, skolemFunction.getOwnerConstruct().getOID());
			pstFunction.executeUpdate();
			//The function is in, we must get its OID. I'd like to have cursors! :(
			sqlQuery = "SELECT OID FROM " + tablePrefix + "SkolemFunction WHERE constructOID = ? AND name = ?;";
			PreparedStatement pstgetOID = con.prepareStatement(sqlQuery);
			pstgetOID.setInt(1, skolemFunction.getOwnerConstruct().getOID());
			pstgetOID.setString(2, skolemFunction.getName());
			ResultSet result = pstgetOID.executeQuery();
			result.next();
			skolemFunction.setOID(result.getInt(1));
			//Okay, let's add the arguments.
			sqlQuery = "INSERT INTO " + tablePrefix + "SkolemFunctionArgument(name, type, functionOID, sourceConstructOID) VALUES(?, ?, ?, ?);";
			PreparedStatement pstArgument = con.prepareStatement(sqlQuery);
			for (SkolemFunctionArgument argument : skolemFunction.getArguments()) {
				pstArgument.setString(1, argument.getName());
				pstArgument.setString(2, argument.getType());
				pstArgument.setInt(3, skolemFunction.getOID());
				pstArgument.setInt(4, argument.getSourceConstruct().getOID());

				pstArgument.executeUpdate();
				//The argument is in!
			}

			//Okay, both the function and argument metadata have been stored, let's now add the functions to the model tables.
			Statement st = con.createStatement();
			String firstPart = "ALTER TABLE " + model.getShortName() + "_"
			+ skolemFunction.getOwnerConstruct().getName() + "_SK " + " ADD COLUMN ";

			// modifica LF
			// Corrispondente query per l'aggiunta degli argomenti alle tabelle delle istanze
			String firstPartForInstance = "ALTER TABLE " + "i_" +model.getShortName() + "_"   
			+ skolemFunction.getOwnerConstruct().getName().toUpperCase() + "_SK" + " ADD COLUMN "; 

			for (SkolemFunctionArgument argument : skolemFunction.getArguments()) {
				sqlQuery = firstPart + argument.getName().toUpperCase() + " " + argument.getType() + ";";
				st.execute(sqlQuery);
				// modifica LF
				// Aggiunta degli argomenti alla tabella di istanza
				sqlQuery = firstPartForInstance + argument.getName() + " " + argument.getType(); 

				st.executeQuery(sqlQuery);
			}
			//The model tables have been updated, we have finished our job! Banzai!

		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/**
	 * @param schemeOID
	 * @param modelOID
	 * @param newName
	 */
	public void createSuperModelSchemeUgly(int schemeOID, int modelOID, String newName) throws SQLException {
		String sqlQuery = "INSERT INTO SchemeSM(OID, modelOID, name) VALUES(?, ?, ?);";
		PreparedStatement pst = con.prepareStatement(sqlQuery);

		pst.setInt(1, schemeOID);
		pst.setInt(2, modelOID);
		pst.setString(3, newName);

		pst.executeUpdate();
		return;
	}

	public void createSuperModelInstanceUgly(int instanceOID, int schemaOID, String newName) throws SQLException {
		String sqlQuery = "INSERT INTO InstanceSM(OID, schemeOID, name) VALUES(?, ?, ?);";
		PreparedStatement pst = con.prepareStatement(sqlQuery);

		pst.setInt(1, instanceOID);
		pst.setInt(2, schemaOID);
		pst.setString(3, newName);

		pst.executeUpdate();
		return;
	}

	/**
	 * Returns the map of the user defined Skolem function (i.e: those not 
	 * created at initialization time by the tool).
	 * 
	 * @return functions
	 */
	public Collection<org.modelgen.commons.beans.skolem.SkolemFunction> getUserDefinedSkolemFunctions() throws PersistenceException {
		Collection<org.modelgen.commons.beans.skolem.SkolemFunction> userDefinedFunctions = new LinkedList<org.modelgen.commons.beans.skolem.SkolemFunction>();
		for (org.modelgen.commons.beans.skolem.SkolemFunction element : getSkolemFunctions()) {
			if (!element.getName().endsWith("_0")) userDefinedFunctions.add(element);
		}
		return userDefinedFunctions;
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#getSkolemFunctions()
	 */
	public Collection<org.modelgen.commons.beans.skolem.SkolemFunction> getSkolemFunctions() throws PersistenceException {
		LinkedList<org.modelgen.commons.beans.skolem.SkolemFunction> functions = new LinkedList<org.modelgen.commons.beans.skolem.SkolemFunction>();
		//Read all the available Skolem Functions
		String queryFunctions = "SELECT F.OID, F.name AS functionName, C.name AS constructName FROM MSM_SkolemFunction AS F JOIN MSM_Construct AS C ON F.constructOID = C.OID;";
		String queryArguments = "SElECT * FROM MSM_SkolemFunctionArgument WHERE functionOID = ?;";
		ResultSet rsFunctions = null, rsArguments = null;
		Model superModel = cache.getSuperModel();
		try {
			Statement st = con.createStatement();
			PreparedStatement pstArguments = con.prepareStatement(queryArguments);
			rsFunctions = st.executeQuery(queryFunctions);
			while (rsFunctions.next()) {
				//Create a SkolemFunction object, fill it with the basic data, then load its arguments.
				org.modelgen.commons.beans.skolem.SkolemFunction function = new org.modelgen.commons.beans.skolem.SkolemFunction();
				function.setOID(rsFunctions.getInt("OID"));
				function.setName(rsFunctions.getString("functionName"));
				function.setOwnerConstruct(superModel.getConstruct(rsFunctions.getString("constructName")));
				//Now loop on its arguments.
				pstArguments.setInt(1, function.getOID());
				rsArguments = pstArguments.executeQuery();
				while (rsArguments.next()) {
					SkolemFunctionArgument skfa = new SkolemFunctionArgument();
					skfa.setName(rsArguments.getString("name"));
					skfa.setType(rsArguments.getString("type"));
					skfa.setSourceConstruct(superModel.getConstruct(rsArguments.getInt("sourceConstructOID")));
					function.addArgument(skfa);
				}
				//Okay, the skolem function is ready.
				functions.add(function);
			}
			//All the skolem functions have been loaded.
			if (functions.isEmpty())
				return null;
			else
				return functions;
		}
		catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#deleteSkolemFunction(java.lang.String, java.lang.String)
	 */
	public void deleteSkolemFunction(Model model, Construct construct, String functionName) throws PersistenceException {
		//We must delete from MSM_SkolemFunction, MSM_SkolemFunctionargument and the *_construct_SK tables.
		//We must first load the skolem function.
		org.modelgen.commons.beans.skolem.SkolemFunction skf = this.getSkolemFunction(model, construct, functionName);
		String schemeTableName = model.getShortName() + "_" + construct.getName() + "_SK";
		//String instanceTableName = model.getShortName() + "_InstanceOf_" + construct.getName() + "_SK";
		//For each argument in the skolem function, we must remove a column in the table.
		//TODO: put instance tables here, before scheme tables.
		for (SkolemFunctionArgument argument : skf.getArguments()) {
			String queryDropColumn = "ALTER TABLE " + schemeTableName + " DROP COLUMN " + argument.getName() + ";";
			try {
				con.createStatement().execute(queryDropColumn);
			}
			catch (SQLException sqle) {
				throw new PersistenceException(sqle);
			}
			//next argument
		} //we deleted all the columns, let's move on to the metadata
		try {
			String queryMetadata = "DELETE FROM MSM_SkolemFunctionArgument WHERE functionOID = " + skf.getOID() + ";";
			con.createStatement().execute(queryMetadata);
			queryMetadata = "DELETE FROM MSM_SkolemFunction WHERE OID = " + skf.getOID() + ";";
			con.createStatement().execute(queryMetadata);
		} catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
		//Done!
		return;
	}
	/**
	 * Author: Gio
	 */
	public int getModelOID(Model model) throws PersistenceException {
		int modelOID;
		try {
			String getModelOID = "SELECT OID FROM Model WHERE name = ?;";				
			//The prepared statements.
			PreparedStatement pstGetModelOID = con.prepareStatement(getModelOID);

			ResultSet rsQueryGet = null;
			pstGetModelOID.setString(1, model.getName());
			rsQueryGet = pstGetModelOID.executeQuery();

			if (rsQueryGet.next()) 
				modelOID = rsQueryGet.getInt(1);
			else 
				modelOID = -1;

			rsQueryGet.close();
		}catch (SQLException sqle) {
			throw new PersistenceException(sqle);
		}
		return modelOID;
	}

	public boolean isInitialized(){
		boolean isInitialized = true;
		try {
			String exist = "SELECT * FROM MSM_Construct;";				
			PreparedStatement pstExist = con.prepareStatement(exist);
			pstExist.executeQuery();	
		}catch (SQLException sqle) {
			//Se da errore il db non � stato inizializzato
			isInitialized = false;
		}
		return isInitialized;
	}

	public Connection getConnection() {
		return this.con;
	}

	public List<Instance> getSuperModelInstances() throws PersistenceException {

		List<Instance> result = new LinkedList<Instance>();

		String sqlQuery = "SELECT I.OID AS iOID, I.Name AS iName, S.OID AS sOID, S.Name AS sName " +
		"FROM INSTANCESM I JOIN SCHEMESM S ON I.SchemeOID = S.OID;";
		PreparedStatement pst;

		try {
			pst = con.prepareStatement(sqlQuery);
			ResultSet resultQuery = pst.executeQuery();

			while (resultQuery.next()) {
				Instance istanza = new Instance();
				istanza.setOID(resultQuery.getInt("iOID"));
				istanza.setName(resultQuery.getString("iName"));
				istanza.setSOID(resultQuery.getInt("sOID"));
				istanza.setSName(resultQuery.getString("sName"));

				result.add(istanza);

			}
		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out); 
			throw new PersistenceException(sqle);
		}

		return result;
	}

	/* (non-Javadoc)
	 * @see org.modelgen.commons.persistence.PersistenceHandler#loadSuperModelInstance(int)
	 */
	public Instance loadSuperModelInstanceInternally(int instanceOID) throws PersistenceException {
		log.debug("Loading instance " + instanceOID);

		String sqlQuery = "SELECT I.OID AS iOID, I.Name AS iName, S.OID AS sOID, S.Name AS sName " +
		"FROM INSTANCESM I JOIN SCHEMESM S ON I.SchemeOID = S.OID" +
		"WHERE iOID = ?;";

		PreparedStatement pst;
		Instance istanza = new Instance();

		try {
			pst = con.prepareStatement(sqlQuery);
			pst.setInt(1, instanceOID);
			ResultSet resultQuery = pst.executeQuery();
//			there should be just one result!

			while (resultQuery.next()) {
				istanza.setOID(resultQuery.getInt("iOID"));
				istanza.setName(resultQuery.getString("iName"));
				istanza.setSOID(resultQuery.getInt("sOID"));
				istanza.setSName(resultQuery.getString("sName"));							
			}
		} catch (SQLException sqle) {
			sqle.printStackTrace(System.out); 
			throw new PersistenceException(sqle);
		}

		return istanza;
	}

	/**
	 * @deprecated use {@link CacheProxyDBHandler#getSuperModelInstance(int)} instead
	 */
	public Instance getSuperModelInstance(int instanceOID) throws PersistenceException {
		return cache.getSuperModelInstance(instanceOID);
	}

	/**
	 * @deprecated use {@link CacheProxyDBHandler#getScheme(int)} instead
	 */
	public Scheme getScheme(int schemeOID) throws PersistenceException {
		return cache.getScheme(schemeOID);
	}

	/**
	 * @deprecated use {@link CacheProxyDBHandler#getSuperModelScheme(int)} instead
	 */
	public Scheme getSuperModelScheme(int schemeOID) throws PersistenceException {
		return cache.getSuperModelScheme(schemeOID);
	}
}