/*
 * Created on 20-ott-2004
 *
 */
package org.modelgen.commons.persistence.database;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
/**
 * Creates the tables for every super model's construct
 * required to create supermodel schemes and instances. 
 * 
 * @author lucasantarelli
 *
 */
public class StructureCreator {
	private static Logger log = Logger.getLogger(StructureCreator.class); 
	
	/**
	 * Creates the tables for every super model's construct
	 * required to create supermodel schemes and instances. 
	 * @param args unused
	 */
	public static void main(String [] args) {
		StructureCreator sc = new StructureCreator();
		log.debug("Starting structureCreator...");
		try {
			RealDBHandler dbh = new RealDBHandler();
			Model sm = dbh.getSuperModel();
			sc.createSMStructures(sm, dbh.con);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	protected void createSMStructures(Model sm, Connection con) throws SQLException {
		for (Construct element : sm.getAbstractConstructs()) {
			log.debug("Calling build routine on construct " + element.getName());
			createSMStructures(element, con);
		}
	}
	private String buildSMSchemeTableName(Construct construct) {
		return "SM_" + construct.getName();
	}
	private String buildSMInstanceTableName(Construct construct) {
		
		/* attenzione aprire e chiude le virgolette quando se ne fa uso */
		/* altrimenti hsqldb non riconosce il token '-' */
		return "i_SM_" + construct.getName().toUpperCase()+""; // modifica LF
	}
	
	protected void createSMStructures(Construct construct, Connection con) throws SQLException {
		/*
		 * For each construct, we build four tables:
		 * SM_<constructName> (contains the scheme objects)
		 * SM_<constructName>_SK (its skolem table)
		 * i_SM_<constructName> (contains the instance objects)
		 * i_SM_<constructName>_SK (its skolem table)
		 */
		Statement st = con.createStatement();
		final String nl = "\n", tab = "\t";
		
		
		String sqlQuery = "", tableName;
		//Ok, let's begin with the first table: let's see if it already exists.
		tableName = buildSMSchemeTableName(construct);
		sqlQuery = "SELECT * FROM " + tableName;
		try {
			st.executeQuery(sqlQuery);
			log.debug("The table " + tableName + " already exists.");
		}
		catch (SQLException sqle) { //The table does not exist yet, we can create it.
			/*
			 * Ok, what is this query supposed to look like?
			 * CREATE TABLE <modelShortName>_<constructName> (
			 * 	OID int identity primary key,
			 * 	schemeOID int not null,
			 * 	name varchar not null,
			 * 	[propertyName propertyType,]*
			 * 	[referenceName int,
			 * 	foreign key (referenceName) references <modelShortName>_<targetConstructName>(OID)]*
			 * 	unique(schemeOID, name);
			 * Note that the references must be declared together before the foreing key constraint.
			 */
			sqlQuery = "CREATE TABLE " + tableName + " (" + nl;
			sqlQuery += tab + "OID int identity primary key," + nl;
			sqlQuery += tab + "schemeOID int," + nl;
			sqlQuery += tab + "name varchar," + nl; //TODO: should this be dropped?
			//Iterate on the properties:
			for (Property property : construct.getProperties()) {
				sqlQuery += tab + property.getName() + " " + property.getType() + "," + nl;
			}
			//The references are made of two lines, e.g.:
			//	abstractOID int, -> the "declaration"
			//	foreign key (abstractOID) references SM_Abstract(OID) -> the "referenced field"
			//All the "declarations" must be done before any "referenced field", otherwise a SQL exception will be thrown.
			String declarations = "", referencedFields = ""; //Buffer-like strings to avoid looping twice.
			for (Reference reference : construct.getReferences()) {
				declarations += tab + reference.getName() + " int," + nl;
				referencedFields += tab + "foreign key (" + reference.getName() + ") references SM_"
					+ reference.getTargetConstruct().getName() + "(OID)," + nl;
			}
			//Now that we have them, let's put 'em in the query.
			sqlQuery += declarations;
			sqlQuery += referencedFields;
			sqlQuery += tab + "foreign key (schemeOID) references SchemeSM(OID)" + nl;
			sqlQuery += ");";
			//The query is over, let's launch it.
			log.debug(sqlQuery);
			st.execute(sqlQuery);
			//Now the skolem table:
			//The first query adds the metadata, the second query builds the columns.
			//We have to basic skolem functions: _copy (implicit, we build the columns, but not the metadata) and OID_0 (we build both columns and metadata)
			log.debug("inserting skolem function metadata for construct " + construct.getName());
			sqlQuery = "INSERT INTO MSM_SkolemFunction(name, constructOID) VALUES(?, ?);";
			PreparedStatement pstSkf = con.prepareStatement(sqlQuery);
			pstSkf.setString(1, construct.getName() + "OID_0");
			pstSkf.setInt(2, construct.getOID());
			int iUpdateCount = pstSkf.executeUpdate();
			log.debug("self copy skolem function insert update count: " + iUpdateCount);
			String querySkfOID = "SELECT max(OID) AS newOID FROM MSM_SkolemFunction;";
			ResultSet rsOID = st.executeQuery(querySkfOID);
			rsOID.next();
			int functionOID = rsOID.getInt("newOID");
			log.debug("skolem function OID for self copy function: " + functionOID);
//			sqlQuery = "INSERT INTO MSM_SkolemFunctionArgument(name, functionOID, type) VALUES(?, ?, ?);";
			sqlQuery = "INSERT INTO MSM_SkolemFunctionArgument(name, functionOID, type, sourceConstructOID) VALUES(?, ?, ?, ?);";
			PreparedStatement pstSkfa = con.prepareStatement(sqlQuery);
			pstSkfa.setString(1, "OID_old");
			pstSkfa.setInt(2, functionOID);
			pstSkfa.setString(3, "integer");
			pstSkfa.setInt(4, construct.getOID());
			pstSkfa.executeUpdate();
			//The second skolem function: we don't add metadata, so we don't do anything now Let's move to the tables.
			//Let's make the _SK table.
			/*
			 * CREATE TABLE <modelShortName>_<constructName>_SK(
			 *    OID int identity,
			 *    OID_old int,
			 *    schemeOID int);
			 */
			sqlQuery = "CREATE TABLE " + tableName + "_SK (" + nl;
			sqlQuery += tab + "OID int identity," + nl;
			sqlQuery += tab + "schemeOID int," + nl;
			sqlQuery += tab + "OID_copy int," + nl;
			sqlQuery += tab + "OID_old int" + nl;
			sqlQuery += ");";
			log.debug(sqlQuery);
			st.execute(sqlQuery);			
		}
		//The tables which will deal with the schemes are done, let's move to the ones which will deal with the instances.
		//NOTE: this block of code should be contained in the previous block, since this should be the only tool which creates
		//structures. In order to preserve some backup data, we are moving the code to its own block.
		/**
		 * CREATE TABLE i_SM_<constructName> (
		 *    OID int identity primary key,
		 *    <constructName>OID int, -> foreign key to the scheme table
		 *    [i_<referenceName>OID int,]* -> foreign key to the INSTANCE reference
		 *    value varchar, -> if it's a Lexical construct
		 * );
		 */
		tableName = buildSMInstanceTableName(construct);
		sqlQuery = "SELECT * FROM " + ""+tableName+"";
		try {
			st.execute(sqlQuery);
		}
		catch (SQLException sqle) {
			//The table does not exist, we need to add it.
			sqlQuery = "CREATE TABLE " + ""+ tableName + " (" + nl;
			sqlQuery += tab + "OID int identity primary key," + nl;
			sqlQuery += tab + construct.getName() + "OID int," + nl;
			sqlQuery += tab + "instanceOID int, " + nl;
			if (construct.isLexical()) sqlQuery += "value varchar," + nl;
			String declarations = "", referencedFields = "";
			for (Reference reference : construct.getReferences()) {
				declarations += tab + "i_" + reference.getName().toUpperCase() +"" + " int," + nl; // modifica LF
				referencedFields += tab + "foreign key (i_" + reference.getName().toUpperCase() +""+ ") references " // modifica LF
					+ ""+buildSMInstanceTableName(reference.getTargetConstruct())+"" + "(OID)," + nl;
			}
			sqlQuery += declarations;
			sqlQuery += referencedFields;
			sqlQuery += tab + "foreign key (instanceOID) references InstanceSM(OID)" + nl;
			sqlQuery += ");";
			
			log.debug(sqlQuery);
			st.execute(sqlQuery);
			//Its skolem table.
			sqlQuery = "CREATE TABLE "+"" + tableName + "_SK (" + nl; // modifica LF
			sqlQuery += tab + "OID int identity," + nl;
			sqlQuery += tab + "instanceOID int," + nl;
			sqlQuery += tab + "OID_copy int," + nl; // modifica LF
			sqlQuery += tab + "OID_old int" + nl;
			sqlQuery += ");";
			log.debug(sqlQuery);
			st.execute(sqlQuery);
		}	
	}
}
