/*
 * Created on 9-gen-2005
 *
 */
package org.modelgen.commons.persistence.xml;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.commons.beans.skolem.SkolemFunctionArgument;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;
import org.modelgen.commons.persistence.xml.jaxb.model.BasicConstructType;
import org.modelgen.commons.persistence.xml.jaxb.model.Model;
import org.modelgen.commons.persistence.xml.jaxb.model.ModelConstructType;
import org.modelgen.commons.persistence.xml.jaxb.model.ModelPropertyType;
import org.modelgen.commons.persistence.xml.jaxb.model.ModelReferenceType;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.BasicConstructTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.BasicPropertyTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.BasicReferenceTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.ModelConstructTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.ModelImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.ModelPropertyTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.ModelReferenceTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.model.impl.ModelTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.scheme.SchemeObjectPropertyType;
import org.modelgen.commons.persistence.xml.jaxb.scheme.SchemeObjectReferenceType;
import org.modelgen.commons.persistence.xml.jaxb.scheme.SchemeObjectType;
import org.modelgen.commons.persistence.xml.jaxb.scheme.impl.SchemeImpl;
import org.modelgen.commons.persistence.xml.jaxb.scheme.impl.SchemeObjectPropertyTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.scheme.impl.SchemeObjectReferenceTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.scheme.impl.SchemeObjectTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.scheme.impl.SchemeTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.ArgumentType;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.FunctionType;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.Functions;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.impl.ArgumentTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.impl.FunctionTypeImpl;
import org.modelgen.commons.persistence.xml.jaxb.skolemnew.impl.FunctionsImpl;

import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Collection;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.apache.ws.jaxme.impl.JMMarshallerImpl;
/**
 * XML export functions for {@link org.modelgen.commons.beans.model.Model},
 * {@link org.modelgen.commons.beans.scheme.Scheme} and
 * {@link org.modelgen.commons.beans.skolem.SkolemFunction}
 * 
 * @author lucasantarelli
 *
 */
public class Exporter {

	private static void printUsage() {
		System.out.println("Usage:");
		System.out.println("To query the framework:");
		System.out.println("\t--list-models, lists all the available models");
		System.out.println("\t--list-schemes, lists all the available schemes");
		System.out.println("To perform an export: (--model modelOID | --scheme schemeOID | --skolem) [outputFilename]");
		System.out.println("\t--model modelOID [outputFilename], exports the requested model to outputFilename");
		System.out.println("\t--nestedmodel modelOID [outputFilename], exports the requested model to outputFilename");
		System.out.println("\t--scheme schemeOID [outputFilename], exports the requested scheme to outputFilename");
		System.out.println("\t--skolem [outputFilename], exports all the Skolem functions");
		System.out.println("If no filename is specified, the key of the requested object will be used.");
		return;
	}

	/**
	 * ModelGen Exporter.
	 * 
	 * @param args
	 * To query the framework:
	 * <ul>
	 * <li><tt>--list-models</tt>: lists all the available models</li>
	 * <li><tt>--list-schemes</tt>: lists all the available schemes</li>
	 * </ul>
	 * To perform an export:
	 * <ul>
	 * <li><tt>--model modelOID [outputFilename]</tt>: exports the requested model to outputFilename</li>
	 * <li><tt>--nestedmodel modelOID [outputFilename]</tt>: exports the requested model to outputFilename</li>
	 * <li><tt>--scheme schemeOID [outputFilename]</tt>: exports the requested scheme to outputFilename</li>
	 * <li><tt>--skolem [outputFilename]</tt>: exports all the Skolem functions</li>
	 * </ul>
	 * If no filename is specified, the key of the requested object will be used.
	 * @throws Exception errors in db connection or in export
	 */
	public static void main(String[] args) throws Exception {
		if (args.length < 1 || args[0].equals("--help")) {
			printUsage();
		} else {
			PersistenceHandlerFactory.getPersistenceHandler("DBHandler");
			if (args[0].equals("--list-models")) {
				System.out.print(listModels());
			} else if (args[0].equals("--list-schemes")) {
				System.out.print(listSchemes());
			} else if (args[0].equals("--model")) {
				if (args.length > 2) {
					exportModel(Integer.parseInt(args[1]), args[2]);
				}
				else {
					exportModel(Integer.parseInt(args[1]), null);
				}
			} else if (args[0].equals("--nestedmodel")) {
				if (args.length > 2) {
					exportNestedModel(Integer.parseInt(args[1]), args[2]);
				}
				else {
					exportNestedModel(Integer.parseInt(args[1]), null);
				}
			} else if (args[0].equals("--scheme")) {
				if (args.length > 2) {
					exportScheme(Integer.parseInt(args[1]), args[2]);
				}
				else {
					exportScheme(Integer.parseInt(args[1]), null);
				}
			} else if (args[0].equals("--skolem")) {
				if (args.length > 1) {
					exportSkolem(args[1]);
				}
				else {
					exportSkolem(null);
				}
			} else {
				System.out.println("Wrong usage.");
				printUsage();
			}
		}
	}

	/**
	 * Export the model specified in a xml file
	 * @param modelOID the model to export
	 * @param filename the output xml file. If null use default name.
	 * @throws Exception errors in db connection or in exportation routine
	 */
	public static void exportModel(int modelOID, String filename) throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");

		org.modelgen.commons.beans.model.Model model = ph.getModel(modelOID);
		String xmlOutput = model2XML(model);
		System.out.println(xmlOutput); //DEBUG
		
		String fileName = model.getShortName() + ".xml";
		if (filename != null && !filename.equals("")) {
			fileName = filename;
		}
		FileWriter fw = new FileWriter(fileName);
		fw.write(xmlOutput);
		fw.close();

	}

	/**
	 * Export the model specified in a xml nested file
	 * @param modelOID the model to export
	 * @param filename the output xml file. If null use default name.
	 * @throws Exception errors in db connection or in exportation routine
	 */
	public static void exportNestedModel(int modelOID, String filename) throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");

		org.modelgen.commons.beans.model.Model model = ph.getModel(modelOID);
		String xmlOutput = model2NestedXML(model);
		System.out.println(xmlOutput); //DEBUG
		
		String fileName = model.getShortName() + ".xml";
		if (filename != null && !filename.equals("")) {
			fileName = filename;
		}
		FileWriter fw = new FileWriter(fileName);
		fw.write(xmlOutput);
		fw.close();
	}

	/**
	 * Export the scheme specified in a xml file
	 * @param schemeOID the scheme to export
	 * @param filename the output xml file (without extension ".xml") 
	 * @throws Exception errors in db connection or in exportation routine
	 */
	public static void exportScheme(int schemeOID, String filename) throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");

		org.modelgen.commons.beans.scheme.Scheme scheme = ph.getScheme(schemeOID);
		String xmlOutput = scheme2XML(scheme);
		System.out.println(xmlOutput); //DEBUG
		
		String fileName = scheme.getMetaInfo().getShortName() + "_" + scheme.getOID() + ".xml";
		if (filename != null && !filename.equals("")) {
			fileName = filename;
		}
		FileWriter fw = new FileWriter(fileName);
		fw.write(xmlOutput);
		fw.close();

	}

	/**
	 * Export the skolem functions in a xml file
	 * @param filename filename the output xml file. If null use default name.
	 * @throws Exception errors in db connection or in exportation routine
	 */
	public static void exportSkolem(String filename) throws Exception {
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");

		System.out.println("going to export!");
		Collection<SkolemFunction> functions = ph.getSkolemFunctions();
		String xmlOutput = skolem2XML(functions);
		System.out.println(xmlOutput); //DEBUG
		
		String fileName = "functions.xml";
		if (filename != null && !filename.equals("")) {
			fileName = filename;
		}
		FileWriter fw = new FileWriter(fileName);
		fw.write(xmlOutput);
		fw.close();

	}

	/**
	 * Computes the xml form of the specified skolem functions 
	 * @param srcFunctions the skolem functions to transform
	 * @return the xml rappresentation of the specified skolem functions
	 * @throws JAXBException errors in the XML/Java binding
	 */
	public static String skolem2XML(Collection<SkolemFunction> srcFunctions) throws JAXBException  {
		StringWriter sw = new StringWriter();
		Functions functions = new FunctionsImpl();
		System.out.println("TEST1");
		for (SkolemFunction srcFunction : srcFunctions) {
			System.out.println("TEST2");
			FunctionType xmlFunction = new FunctionTypeImpl();
			xmlFunction.setName(srcFunction.getName());
			xmlFunction.setTargetConstruct(srcFunction.getOwnerConstruct().getName());
			xmlFunction.setArguments(new FunctionTypeImpl.ArgumentsTypeImpl());
			for (SkolemFunctionArgument srcArgument : srcFunction.getArguments()) {
				System.out.println("TEST3");
				System.out.println("srcArgument Name: " + srcArgument.getName());
				System.out.println("srcArgument Type: " + srcArgument.getType());
				System.out.println("srcArgument Construct: " + srcArgument.getSourceConstruct().getName());
				ArgumentType xmlArgument = new ArgumentTypeImpl();
				xmlArgument.setName(srcArgument.getName());
				xmlArgument.setType(srcArgument.getType());
				xmlArgument.setSourceConstruct(srcArgument.getSourceConstruct().getName());
				xmlFunction.getArguments().getArgument().add(xmlArgument);
				System.out.println("TEST4");
			}
			functions.getFunction().add(xmlFunction);
		}
		//Now that the object tree is ready, we write it in out stringwriter.
		//try {
			JAXBContext context = JAXBContext.newInstance("org.modelgen.commons.persistence.xml.jaxb.skolemnew", org.modelgen.commons.persistence.xml.jaxb.skolemnew.ObjectFactory.class.getClassLoader());
			Marshaller marshaller = context.createMarshaller();
			marshaller.setProperty(JMMarshallerImpl.JAXME_XML_DECLARATION, Boolean.TRUE);
			marshaller.marshal(functions, sw);
			return sw.toString();
		/*}
		catch (JAXBException je) {
			je.printStackTrace();
			return null;
		}*/

	}
	
	/**
	 * Computes the xml form of the specified scheme 
	 * @param srcScheme the scheme to transform
	 * @return the xml rappresentation of the specified scheme
	 * @throws JAXBException errors in the XML/Java binding
	 */
	public static String scheme2XML(org.modelgen.commons.beans.scheme.Scheme srcScheme) throws JAXBException {
		StringWriter sw = new StringWriter();
		
		org.modelgen.commons.persistence.xml.jaxb.scheme.Scheme s = new SchemeImpl();
		
		//Let's first prepare the basic container:
		s.setName(srcScheme.getName());
		s.setModel(srcScheme.getMetaInfo().getShortName());
		s.setObjects(new SchemeTypeImpl.ObjectsTypeImpl());
		
		//Now the objects in the scheme.
		for (SchemeObject so : srcScheme.getAbstractConstructs()) {	
			SchemeObjectType xmlso = new SchemeObjectTypeImpl();
			xmlso.setName(so.getName());
			xmlso.setConstruct(so.getMetaInfo().getName());
			xmlso.setKey(so.getKey().toString());
			
			xmlso.setProperties(new SchemeObjectTypeImpl.PropertiesTypeImpl());
			for (SchemeObjectProperty sop : so.getProperties()) {
				
				SchemeObjectPropertyType xmlsop = new SchemeObjectPropertyTypeImpl();
				xmlsop.setName(sop.getMetaInfo().getName());
				xmlsop.setValue(sop.getValue());
				
				xmlso.getProperties().getProperty().add(xmlsop);
			}
			xmlso.setReferences(new SchemeObjectTypeImpl.ReferencesTypeImpl());
			for (SchemeObjectReference sor : so.getReferences()) {				
				SchemeObjectReferenceType xmlsor = new SchemeObjectReferenceTypeImpl();
				xmlsor.setName(sor.getMetaInfo().getName());
				if (sor.getValue()!=null) 
					xmlsor.setValue(sor.getValue().getKey().toString());
				else 
					xmlsor.setValue(SchemeObject.MDM_UNSPECIFIED);
				xmlsor.setType(sor.getMetaInfo().getTypeAsString());
				
				xmlso.getReferences().getReference().add(xmlsor);
			}
			//And now let's add the object:
			s.getObjects().getSchemeobject().add(xmlso);
		}
		
		//Now that the object tree is ready, we write it in out stringwriter.
		//try {
			JAXBContext context = JAXBContext.newInstance("org.modelgen.commons.persistence.xml.jaxb.scheme", org.modelgen.commons.persistence.xml.jaxb.scheme.ObjectFactory.class.getClassLoader());
			Marshaller marshaller = context.createMarshaller();
			marshaller.setProperty(JMMarshallerImpl.JAXME_XML_DECLARATION, Boolean.TRUE);
			marshaller.marshal(s, sw);
			return sw.toString();
		/*}
		catch (JAXBException je) {
			je.printStackTrace();
			return null;
		}*/
	}
	
	/**
	 * Computes the xml form of the specified model 
	 * @param srcModel the model to transform
	 * @return the xml rappresentation of the specified model
	 * @throws JAXBException errors in the XML/Java binding
	 */
	public static String model2XML(org.modelgen.commons.beans.model.Model srcModel) throws JAXBException {
		StringWriter sw = new StringWriter();
		
		Model m = new ModelImpl();
		ModelConstructType c = null;
		BasicConstructType b = null;
		ModelPropertyType p = null;
		ModelReferenceType r = null;
		
		//Prepare the basic containers:
		m.setName(srcModel.getName());
		m.setShortName(srcModel.getShortName());
		m.setConstructs(new ModelTypeImpl.ConstructsTypeImpl());
		m.setProperties(new ModelTypeImpl.PropertiesTypeImpl());
		m.setReferences(new ModelTypeImpl.ReferencesTypeImpl());
		//Iterate on the model constructs, adding each one (and iterate on its properties and references too!)
		for (Construct srcConstruct : srcModel.getAbstractConstructs()) {
			c = new ModelConstructTypeImpl();
			c.setName(srcConstruct.getName());
			c.setBonds(srcConstruct.getUserBonds());
			c.setSignature(srcConstruct.getSignature());
			b = new BasicConstructTypeImpl();
			Construct smConstruct = srcConstruct.getSuperModelConstructs().iterator().next(); //TODO: fix this when the beans lose the 1:N mapping.
			b.setName(smConstruct.getName());
			c.setMappingToConstruct(b); //We tie the mapping to the right construct in this call. Check how we do it for the property or reference for the future way to do it. (see previous TODO)
			
			m.getConstructs().getConstruct().add(c);
			//The construct is in, now let's add the properties.
			for (Property srcProperty : srcConstruct.getProperties()) {
				p = new ModelPropertyTypeImpl();
				p.setName(srcProperty.getName());
				p.setBelongingTo(srcConstruct.getName()); //NOTE: another way could be getOnwerConstruct().getName(). It's the same.
				p.setType(srcProperty.getType());
				p.setMappingToProperty(new BasicPropertyTypeImpl()); //We tie the mapping directly with this call.
				p.getMappingToProperty().setName(srcProperty.getMetaInfo().getName());
				p.getMappingToProperty().setBelongingTo(srcProperty.getMetaInfo().getOwnerConstruct().getName());
				p.getMappingToProperty().setType(srcProperty.getMetaInfo().getType());
				//Ok, the property is done: let's put it in the JAXB tree.
				m.getProperties().getProperty().add(p);
			}
			//References!
			for (Reference srcReference : srcConstruct.getReferences()) {
				r = new ModelReferenceTypeImpl();
				r.setName(srcReference.getName());
				r.setBelongingTo(srcConstruct.getName());
				r.setTargeting(srcReference.getTargetConstruct().getName());
				r.setType(srcReference.getTypeAsString());
				r.setMappingToReference(new BasicReferenceTypeImpl());
				r.getMappingToReference().setName(srcReference.getMetaInfo().getName());
				r.getMappingToReference().setBelongingTo(srcReference.getMetaInfo().getOwnerConstruct().getName());
				r.getMappingToReference().setTargeting(srcReference.getMetaInfo().getTargetConstruct().getName());
				r.getMappingToReference().setType(srcReference.getMetaInfo().getTypeAsString());
				m.getReferences().getReference().add(r);
			}
			//Ready for the next construct...
		}
		//Ok, the JAXB object tree is ready.
		
		//try {
			JAXBContext context = JAXBContext.newInstance("org.modelgen.commons.persistence.xml.jaxb.model", org.modelgen.commons.persistence.xml.jaxb.model.ObjectFactory.class.getClassLoader());
			Marshaller marshaller = context.createMarshaller();
			//Vogliamo che il documento in output abbia una XML declaration, quindi aggiungiamo:
			marshaller.setProperty(JMMarshallerImpl.JAXME_XML_DECLARATION, Boolean.TRUE);
			marshaller.marshal(m, sw);
			return sw.toString();
		/*}
		catch (JAXBException je) {
			je.printStackTrace();
			return null;
		}*/
	}
	
	/**
	 * Computes the xml nested form of the specified model 
	 * @param srcModel the model to transform
	 * @return the xml nested rappresentation of the specified model
	 * @throws JAXBException errors in the XML/Java binding
	 */
	public static String model2NestedXML(org.modelgen.commons.beans.model.Model srcModel) throws JAXBException{
		StringWriter sw = new StringWriter();
		
		org.modelgen.commons.persistence.xml.jaxb.nestedmodel.Model m =
			new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.ModelImpl();
		org.modelgen.commons.persistence.xml.jaxb.nestedmodel.ConstructType c = null;
		org.modelgen.commons.persistence.xml.jaxb.nestedmodel.MetaConstructType mc = null;
		org.modelgen.commons.persistence.xml.jaxb.nestedmodel.PropertyType p = null;
		org.modelgen.commons.persistence.xml.jaxb.nestedmodel.ReferenceType r = null;
		
		//Prepare the basic containers:
		m.setName(srcModel.getName());
		m.setShortName(srcModel.getShortName());
		m.setConstructs(new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.ConstructsTypeImpl());
		//Iterate on the model constructs, adding each one (and iterate on its properties and references too!)
		for (Construct srcConstruct : srcModel.getAbstractConstructs()) {
			
			c = new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.ConstructTypeImpl();
			c.setName(srcConstruct.getName());
			c.setBonds(srcConstruct.getUserBonds());
			c.setSignature(srcConstruct.getSignature());
			
			mc = new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.MetaConstructTypeImpl();
			Construct smConstruct = srcConstruct.getSuperModelConstructs().iterator().next(); //TODO: fix this when the beans lose the 1:N mapping.
			mc.setName(smConstruct.getName());
			c.setMetaConstruct(mc);
			
			c.setProperties(new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.PropertiesTypeImpl());
			
			//The construct is in, now let's add the properties.
			for (Property srcProperty : srcConstruct.getProperties()) {
				p = new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.PropertyTypeImpl();
				p.setName(srcProperty.getName());
				p.setMetaProperty(new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.MetaPropertyTypeImpl()); //We tie the mapping directly with this call.
				p.getMetaProperty().setName(srcProperty.getMetaInfo().getName());
				//Ok, the property is done: let's put it in the JAXB tree.
				c.getProperties().getProperty().add(p);
			}
			//References!
			c.setReferences(new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.ReferencesTypeImpl());
			for (Reference srcReference : srcConstruct.getReferences()) {
				r = new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.ReferenceTypeImpl();
				r.setName(srcReference.getName());
				r.setTo(srcReference.getTargetConstruct().getName());
				r.setMetaReference(new org.modelgen.commons.persistence.xml.jaxb.nestedmodel.impl.MetaReferenceTypeImpl());
				r.getMetaReference().setName(srcReference.getMetaInfo().getName());
				c.getReferences().getReference().add(r);
			}
			m.getConstructs().getConstruct().add(c);
			//Ready for the next construct...
		}
		//Ok, the JAXB object tree is ready.
		
		//try {
			JAXBContext context = JAXBContext.newInstance("org.modelgen.commons.persistence.xml.jaxb.nestedmodel", org.modelgen.commons.persistence.xml.jaxb.nestedmodel.ObjectFactory.class.getClassLoader());
			Marshaller marshaller = context.createMarshaller();
			marshaller.setProperty(JMMarshallerImpl.JAXME_XML_DECLARATION, Boolean.TRUE);
			marshaller.marshal(m, sw);
			return sw.toString();
		/*}
		catch (JAXBException je) {
			je.printStackTrace();
			return null;
		}*/
	}
	
	private static String listModels() throws Exception {
		String output = null;
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");
		Collection<org.modelgen.commons.beans.model.Model> models = ph.getModels();
		if (models.isEmpty())
			output = "There are no models.";
		else {
			output = "These are the available models:\n";
			for (org.modelgen.commons.beans.model.Model model : models) {
				output += "model '" + model.getName() 	+ "', shortname: " + model.getShortName() + ", oid: " + model.getOID() + "\n";
			}			
		}
		return output;
	}
	private static String listSchemes() throws Exception {
		String output = null;
		PersistenceHandler ph = PersistenceHandlerFactory.getPersistenceHandler("DBHandler");
		Collection<Scheme> schemes = ph.getSchemes();
		if (schemes.isEmpty())
			output = "There are no schemes.";
		else {
			output = "These are the available schemes:\n";
			for (org.modelgen.commons.beans.scheme.Scheme scheme : schemes) {
				output += "scheme '" + scheme.getName() 	+ "', model: " + scheme.getMetaInfo().getShortName() + ", oid: " + scheme.getOID() + "\n";
			}			
		}
		return output;
	}
	
}
