package org.modelgen.commons.utils;

import java.io.Serializable;

/**
 * Keep pairs of values. Useful when you want
 * to use/return pairs of values without defining new classes.
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 * @param <A> the first element's type
 * @param <B> the second element's type
 */
public class Pair<A,B> implements Serializable {
	private static final long serialVersionUID = 1L;

	private A first;
	private B second;
	/** 
	 * Creates a new <code>Edge</code>
	 * @param first
	 * @param second
	 */
	public Pair(A first, B second) {
		this.first = first;
		this.second = second;
	}
	/** 
	 * Returns first
	 * @return first
	 */
	public A getFirst() {
		return this.first;
	}
	/** 
	 * Set first
	 * @param first first to set
	 */
	public void setFirst(A first) {
		this.first = first;
	}
	/** 
	 * Returns second
	 * @return second
	 */
	public B getSecond() {
		return this.second;
	}
	/** 
	 * Set second
	 * @param second second to set
	 */
	public void setSecond(B second) {
		this.second = second;
	}
	
	/**
	 * @return ({@link A#toString()},{@link B#toString()})
	 */
	@Override
	public String toString() {
		return "( "+this.first+", "+this.second+" )";
	}
	
	/**
	 * @return if the two pairs contains equal elements 
	 */
	@Override
	public boolean equals(Object obj) {
		Pair<?,?> otherPair = (Pair<?, ?>) obj;
		return first.equals(otherPair.first) && second.equals(otherPair.second);
	}
	
	/**
	 * @return a XOR hashcode of first and second element
	 */
	@Override
	public int hashCode() {
		int h = (first == null ? 0 : first.hashCode());
		if ( second != null )
			h ^= second.hashCode();
		return h;
	}

	/**
	 * Compares two pairs composed of comparable elements
	 * @param <X> the first element's type
	 * @param <Y> the second element's type
	 * @param current the current pair
	 * @param other the other pair to compare
	 * @return the comparation between the two pairs.
	 */
	public static <X extends Comparable<X>, Y extends Comparable<Y>> int compareTo(Pair<X,Y> current, Pair<X,Y> other) {
		// check
		int compare = current.first.compareTo(other.first);
		if (compare == 0) { // se sono uguali vai ai secondi
			compare = current.second.compareTo(other.second); 
		}
		return compare;
	}
}
