/*
 * Created on 16-feb-2004
 *
 */
package org.modelgen.commons.utils;

/**
 * Utility functions to perform basic String checking. 
 * 
 * @author Luca
 *
 */
public class Validator {
	/**
	 * Performs a basic String checking (i.e.: not null, not empty, doesn't start with a number).
	 * @param s - the String object to check.
	 * @return true - if the String object is valid, else otherwise.
	 */
	public static boolean isValidString(String s) {
		if (s == null || s.equals("")) return false;
		if (Character.isDigit(s.charAt(0))) return false; //Our strings cannot start with numbers.
		return true;
	}
	
	/**
	 * Performs a basic String checking (see: isValidString(String)) and checks if the lenght is valid.
	 * @param s - the String object to check.
	 * @param minLength - the minimum lenght the String must have.
	 * @param maxLength - the maximum lenght the String can have.
	 * @return true - if the String object is valid, else otherwise.
	 */
	public static boolean isValidString(String s, int minLength, int maxLength) {
		if (!isValidString(s))
			return false;
		if (s.length() < minLength || s.length() > maxLength)
			return false;
		return true;
	}
	
	/**
	 * Returns true if the given String object does not contain any reserved character. 
	 * @param s - the String object to check
	 * @param reservedChars - an array of reserved characters (as String ojects)
	 * @return true - if the String object is valid, else otherwise.
	 */
	public static boolean isValidString(String s, String[] reservedChars) {
		if (!isValidString(s)) return false;
		for (int i = 0; i < reservedChars.length; i++)
			if (s.indexOf(reservedChars[i]) != -1)
				return false;
		return true;
	}
}
