/**
 * FORNISCE FUNZIONALITA' GENERICHE PER LA GESTIONE DEL SERVER DEL DATABASE
 * */
package org.modelgen.facade;

import org.modelgen.applyrule.MacroRule;
import org.modelgen.applyrule.RuleApplicator;
import org.modelgen.applyrule.reader.MacroRuleReader;
import org.modelgen.commons.beans.instance.Instance;
import org.modelgen.commons.beans.model.Construct;
import org.modelgen.commons.beans.model.Model;
import org.modelgen.commons.beans.model.Property;
import org.modelgen.commons.beans.model.Reference;
import org.modelgen.commons.beans.scheme.Scheme;
import org.modelgen.commons.beans.scheme.SchemeObject;
import org.modelgen.commons.beans.scheme.SchemeObjectProperty;
import org.modelgen.commons.beans.scheme.SchemeObjectReference;
import org.modelgen.commons.beans.skolem.SkolemFunction;
import org.modelgen.commons.beans.skolem.SkolemFunctionArgument;
import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.commons.persistence.PersistenceHandler;
import org.modelgen.commons.persistence.PersistenceHandlerFactory;
import org.modelgen.commons.persistence.PersistenceHandlerFactoryReloadListener;
import org.modelgen.commons.persistence.database.DBHandler;
import org.modelgen.commons.persistence.database.Initializer;
import org.modelgen.commons.persistence.xml.Exporter;
import org.modelgen.commons.persistence.xml.Importer;
import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.regole.MacroRegola;
import org.modelgen.regole.Exception.InstanceTranslationException;
import org.modelgen.regole.reader.GenerateInstanceRuleRepository;
import org.modelgen.regole.reader.ParametricDatalogRulesReader;
import org.modelgen.regole.utilities.InstanceRulesDuplicateGenerationException;
import org.modelgen.signature.MacroRuleSignature;
import org.modelgen.signature.ModelSignature;
import org.modelgen.signature.application.macroruleapplication.SignatureApplicationUtility;
import org.modelgen.tmphandler.TmpHandler;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JComboBox;
import javax.xml.bind.JAXBException;

import org.hsqldb.Server;
import org.hsqldb.util.ShutdownServer;

/**
 * Represents the unified facade of all useful functions.
 * 
 * @author Undefined...
 */
public class GeneralManagementFacade implements PersistenceHandlerFactoryReloadListener {
	private static GeneralManagementFacade instance;
	/**
	 * Used to distinguish a model created but not yet stored. 
	 */
	public static final int NEWMODELOID = -1;

	/**
	 * Used to distinguish a scheme created but not yet stored. 
	 */
	public static final int NEWSCHEMEOID = -2;

	private String serverUrl;
	private List<Scheme> ss = null;
	private Model superModel = null; 
	private PersistenceHandler ph;
	private SkolemFunction currentSkolemFunction;
	private static List<Model> ms = null;
	private Connection connection;

	/**
	 * Returns the unique shared instance associated
	 * @return the unique shared instance associated
	 */
	public synchronized static GeneralManagementFacade getInstance() {
		if (instance==null) {
			instance = new GeneralManagementFacade();
		}
		return instance;
	}

	private GeneralManagementFacade(){
		this.serverUrl = ModelgenConfigurationManager.getIstance().getProperty("serverUrl","","jdbc");
		PersistenceHandlerFactory.addReloadListener(this);
		//initPesistenceHandler();
	}

	/**
	 * Start the database server
	 * @return if errors happens in the possible initialization
	 */
	public boolean startServer(){
		Server.main(new String[]{"-database",serverUrl,"-no_system_exit","true"});			
		//Controllo inizializzazione database
		//Il significato di "isInitialized" non � solo quello di database inizializzato
		//con le strutture del metamodello, ma anche di server correttamente attivato 
		boolean isInitalized = false;
		try {
			this.initPesistenceHandler(); // reload
			isInitalized = ph.isInitialized();
			if(!isInitalized){
				System.out.println("*********************************");
				System.out.println("  Start DataBase Initialization");
				System.out.println("*********************************");
				Initializer.main(new String[]{});
				isInitalized = true;
			}else{
				System.out.println("*********************************");
				System.out.println("  DataBase already initialized");
				System.out.println("*********************************");
			}
		} catch (Exception e) {
			System.out.println();
			System.out.println("Errore: "+e);
			isInitalized = false;
		}
		return isInitalized;
	}

	/**
	 * Stop the server
	 */
	public void stopServer(){
		ShutdownServer.main(new String[]{"-database",serverUrl});
	}

	/**
	 * Delete the database
	 */
	public void deleteDataBase(){
		System.out.println("********");
		File dataBase = (new File(serverUrl)).getParentFile();
		String[] DBfiles = dataBase.list();
		int i = 0;
		while(i<DBfiles.length){
			String file = DBfiles[i];
			System.out.println(" - "+file + " deleted["+(new File(dataBase,file)).delete()+"]");
			i++;
		}
		System.out.println("********");
		
		/* cancella i modelli salvati nella GeneralManagementFacade */
		this.unLoadModel();
		this.unLoadSchemes();
		/* resetta la cache */
		PersistenceHandlerFactory.resetCache();
		
		
	}

	/**
	 * Get the unique persistence handler associated
	 * @return the unique persistence handler associated
	 */
	public PersistenceHandler getPesistenceHadler(){
		return this.ph;
	}

	/**
	 * Initialize the shared unique persistence handler
	 * @throws PersistenceException errors in the persistence handler's initialization
	 */
	public void initPesistenceHandler() throws PersistenceException{
		/*Sfuttiamo la veridica del ph per inizializzarlo (in questo modo si evita 
		 * supportare nuovamente il costo della creazione del ph, con il metodo getPh)
		 */
		try {
			this.ph = PersistenceHandlerFactory.getPersistenceHandler();
			this.connection = this.ph.getConnection();
		} catch (Exception e) {
			e.printStackTrace();
			throw new PersistenceException("Impossibile inizializzare il PersistenceHandler");
		}
	}

	/**
	 * Populate instance combo box.
	 * @return String[] the instances to insert in the combo box
	 * @throws PersistenceException if a persistence error occurs
	 * @used ApplyInstanceRule-ApplyRules
	 */
	public String[] populateCmbInstance() throws PersistenceException {
		Collection instances = ph.getSuperModelInstances();
		int max = instances.size();
		String[] visibleInstances = new String[max+1];
		visibleInstances[0] =".: Select a source instance from the list:.";
		Iterator i = instances.iterator();
		int k = 1;
		Instance istanza = null;
		while (i.hasNext()) {
			istanza = (Instance)i.next();
			visibleInstances[k] = istanza.getOID() + ": " + istanza.getName() 
			+ " (" + istanza.getSOID() + ": " + istanza.getSName()+")";
			k++;
		}
		return visibleInstances;
	}

	/**
	 * Populate scheme combo box.
	 * @param type if "model" gets all model specific schemes.
	 * Otherwise gets all super model's schemes.
	 * @return String[] the schemes to insert in the combo box
	 * @throws PersistenceException if a persistence error occurs
	 * @used ApplyInstanceRule-ApplyRules-SignatureApplication 
	 */
	public String[] populateCmbScheme(String type) throws 
	PersistenceException {
		Collection<Scheme> schemes;
		if (type.equals("model")) 
			schemes = ph.getSchemes();
		else
			schemes = ph.getSuperModelSchemes();
		int max = schemes.size();
		String[] visibleSchemes = new String[max+1];
		visibleSchemes[0] =".: Select a source scheme from the list :.";
		int k = 1;
		for (Scheme s : schemes) {
			visibleSchemes[k] = s.getOID() + ": " + s.getName() + " ("+s.getMetaInfo().getName()+") ";
			k++;
		}
		return visibleSchemes;
	}

	/**
	 * Return the instance with the specified oid 
	 * @param instanceOID the instance's oid to search
	 * @return the instance founded, null otherwise
	 * @throws PersistenceException if a persistence error occurs
	 * @used ApplyInstanceRule-ApplyRules
	 */
	public Instance getInstanceByOID(int instanceOID) throws PersistenceException {
		// Take the instances' collection 
		Collection instances = ph.getSuperModelInstances();
		Iterator i = instances.iterator();
		// Model to search 
		Instance instanceToSearch;
		// If found 
		Instance found = null;
		while(i.hasNext()) {
			instanceToSearch = (Instance)i.next();
			if (instanceOID == instanceToSearch.getOID())
				found = instanceToSearch;
		}
		return found;
	}


	/**
	 * Populate model combo box.
	 * @return String[] the models to insert in the combo box
	 * @throws PersistenceException if a persistence error occurs
	 * @used SignatureApplication-ApplyRules
	 */
	public String[] populateCmbModel() throws PersistenceException {
		//Collection<Model> models = ph.getModels();
		List<Model> models = new ArrayList<Model>();
		models.addAll(ph.getModels());
		int max = models.size();
		String[] visibleModels = new String[max+1];
		//visibleModels[0] =".: target model :.";
		visibleModels[0] =".: Select a target model from the list :.";
		int k = 1;
		for (Model m : models) {
			visibleModels[k] = m.getName() + " ("+ m.getShortName()+")";
			k++;
		}
		return visibleModels;
	}

	/**
	 * Obtain the supermodel's scheme with the specified oid 
	 * and get its signature. 
	 * @param oid the suoermodel's scheme oid
	 * @return the supermodel scheme signature
	 * @throws PersistenceException if a persistence error occurs
	 * @used SignatureApplication
	 */
	public ModelSignature getSignatureSuperModelSchema(int oid) throws PersistenceException{
		Scheme schema = ph.getSuperModelScheme(oid);
		return schema.getModelSignature();
	}
	
	/**
	 * Obtain the supermodel's scheme with the specified oid 
	 * @param oid the suoermodel's scheme oid
	 * @return the supermodel scheme
	 * @throws PersistenceException if a persistence error occurs
	 */
	public Scheme getSuperModelSchema(int oid) throws PersistenceException{
		return ph.getSuperModelScheme(oid);
	}

	/**
	 * Copy a scheme
	 * @param from Source of scheme to copy (model or supermodel)
	 * @param schemeOID The OID of the scheme
	 * @param cmbBox the GUI to refresh (ugly, TODO: observer pattern...)
	 * @throws PersistenceException if a persistence error occurs
	 * @used ApplyRules
	 */
	public void copyScheme(String from, int schemeOID, JComboBox cmbBox) throws
	PersistenceException {
		Scheme srcScheme = null;
		if(from.equals("superModel")) {
			srcScheme = ph.getSuperModelScheme(schemeOID);
			ph.storeSuperModelSchemeIntoModel(srcScheme);
		} else {
			srcScheme = ph.getScheme(schemeOID);
			ph.storeModelSchemeIntoSuperModel(srcScheme);
		}
	}

	/**
	 * Return a scheme to view in report
	 * @param schemeOID The OID
	 * @return Scheme the scheme to view
	 * @throws PersistenceException if a persistence error occurs
	 * @used ApplyRules
	 */
	public Scheme getSchemeForReport(int schemeOID) throws PersistenceException {
		// FIXME: motivo?
		Scheme srcScheme = ph.getSuperModelScheme(schemeOID);
		if(srcScheme == null)
			srcScheme = ph.getScheme(schemeOID);
		return srcScheme;
	}

	/**
	 * Return a model given the name
	 * @param modelName The model to search
	 * @return Model the model founded, null otherwise
	 * @throws PersistenceException if a persistence error occurs
	 * @used SignatureApplication-ApplyRules
	 */
	public Model getModelForName(String modelName) throws PersistenceException {
		/* If found */
		Model found = null;
		for(Model modelToSearch : ph.getModels()) {
			if (modelName.equals(modelToSearch.getName()))
				found = modelToSearch;
		}
		return found;
	}


	/**
	 * Apply instance rules.
	 * 
	 * For every rule, creates a target instance with the
	 * specified name.
	 * 
	 * @param rules the rules to apply
	 * @param name the target instance's name to store
	 * @param sourceSchemaOID the source schema's oid
	 * @param targetSchemaOID the target schema's oid
	 * @param sourceInstanceOID the source instance's oid
	 * @throws Exception errors in persistence or in the execution of the rule
	 * @used ApplyInstanceRule
	 */
	public void applyInstanceRules(String[] rules, String name, int sourceSchemaOID, 
			int targetSchemaOID, int sourceInstanceOID) throws Exception {
		// Check the passed name, because it's optional in form 
		if (name.length() == 0) { //We use the same name.
			Scheme srcScheme = ph.getSuperModelScheme(sourceSchemaOID);
			name = srcScheme.getName();
		}
		int targetInstanceOID = 0;
//		WARNING! Currently, just one rule is supported!
//		this is because translation over instances rely on populated
//		temporary tables. Applying more than one schema translation implies
//		to loose the temporary tables for the first translation. As a consequence
//		translation on instances will not work properly.
		for (int i = 0; i < rules.length; i++) {
			//3. Le'ts get the target instance oid.
			targetInstanceOID = ph.getNewOID("InstanceSM", "OID");
			System.out.println("ARGUMENTS: " + sourceSchemaOID + " " + targetSchemaOID + " " + sourceInstanceOID + " " + targetInstanceOID);
			//Okay, 3 is done. Now 3.5 is the INSERT INTO InstanceSM that should be in the datalog rule itself.
			//It's here just because of time reasons.
			ph.createSuperModelInstanceUgly(targetInstanceOID, targetSchemaOID, name);
			//4. the rule.
			String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
			String macrorulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
			String instanceMacroRules = ModelgenConfigurationManager.getIstance().getProperty("instanceMacroRules", "", "paths");
			//String instancerulePath = config.getProperty("InstanceMacroRules", "", "paths");
			String macroRuleFullPath = rootnode + macrorulePath + instanceMacroRules + rules[i];

			//And 4 is done too, now we must execute the rule. What are the parameters?
			String[] arguments = { Integer.toString(targetSchemaOID), Integer.toString(sourceSchemaOID),
					Integer.toString(targetInstanceOID), Integer.toString(sourceInstanceOID) };

			//Let's rock!
			System.out.println("macrorule: " + macroRuleFullPath);

			MacroRegola mrdp = 
				ParametricDatalogRulesReader.read(macroRuleFullPath);
			//Before we rock, we must create some temporary directories.

//			MacroRule temporaryTables = 
//			MacroRuleReader.read(rootnode + macrorulePath + 
//			"GoodRules/ModelGen_temporaryTables.txt");
////			System.out.println("BRASA le TMP: " + rootnode + macrorulePath + 
////			"GoodRules/ModelGen_temporaryTables.txt");
//			RuleApplicator.applyRule(temporaryTables, null);

//			remind that an instance rule relies on TMP tables populated by
//			the (related) schema translation

			RuleApplicator.applyRule(mrdp.istanziaMacroRuleSQL(macroRuleFullPath),
					arguments);

//			srcOID = targetInstanceOID; 
		}

	}

	/**
	 * Applies scheme rules.
	 * 
	 * For every rule, creates a target scheme with the
	 * specified name.
	 * 
	 * @param rules the rules to apply
	 * @param name the target scheme's name to store
	 * @param schemeOID the source scheme's oid
	 * @param modelOID the target model's oid
	 * @throws Exception if an error occurs
	 */
	public void applySchemeRules(String[] rules, String name, int schemeOID, 
			int modelOID) throws Exception {
		List<String> namesList = new LinkedList<String>();
		List<Integer> schemeOIDList = new LinkedList<Integer>();
		List<Integer> modelOIDList = new LinkedList<Integer>();

		namesList.add(name);
		schemeOIDList.add(schemeOID);
		modelOIDList.add(modelOID);

		// TODO: per n volte
		this.applySchemeRules(rules, namesList, schemeOIDList, modelOIDList);
	}

	/**
	 * Applies scheme rules with N sources and M targets.
	 * 
	 * For every rule, creates M target scheme with the
	 * specified name. 
	 * 
	 * <p>The number M depends on the result of
	 * {@link org.modelgen.regole.MacroRegola#getDestSchemesNumber()},
	 * returned internally.
	 * 
	 * @param rules the rules to apply
	 * @param namesList the target scheme's name to store
	 * (this method uses for now only the first element)
	 * @param schemeOIDList the N source scheme's oids
	 * @param modelOIDList the target model's oids
	 * (this method uses for now only the first element)
	 * @throws Exception if an error occurs
	 */
	public void applySchemeRules(String[] rules, List<String> namesList, List<Integer> schemeOIDList,	
			List<Integer> modelOIDList) throws Exception {

		/* Check parameters consistency. FIXME: con le nuove convenzioni inutile?
		boolean isSchemeConsistent = namesList.size() == schemeOIDList.size() && 
		schemeOIDList.size() == modelOIDList.size();
		if(!isSchemeConsistent) {
			throw new IOException("Parameters are inconsistent!");
		}*/

		/* Check the passed name, because it's optional in form */
		for(int i = 0; i < namesList.size(); i++) {
			String schemaName = namesList.get(i);
			if (schemaName.length() == 0) { //We use the same name.
				Scheme srcScheme = ph.getSuperModelScheme(schemeOIDList.get(i));
				schemaName = srcScheme.getName();
				namesList.set(i, schemaName);
			}
		}

		// FIXME: come fare?
		//int srcOID = schemeOID;


		/* per tutte le regole */
		for (int i = 0; i < rules.length; i++) {

			/* crea lista delle destinazioni */
			List<Integer> destOIDList = new LinkedList<Integer>();

			/* path della macroregola */
			String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
			String macrorulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
			String macroRuleFullPath = rootnode + macrorulePath + rules[i];

			/* parsing della macroregola */
			MacroRegola mrdp = 
				ParametricDatalogRulesReader.read(macroRuleFullPath);

			/* inserimento di tutte le destinazioni */
			int destOID = ph.getNewOID("SchemeSM", "OID"); // FIXME: eliminare perche'...
			for(int q=0;q<mrdp.getDestSchemesNumber();q++) {
				// ... FIXME: non dovrebbe essere add(ph.getNewOID("SchemeSM","OID"))?
				// il risultato e' lo stesso, ma gli effetti collaterali sul ph no
				destOIDList.add(destOID+q);
			}


			/* registrazione di tutti gli schemi sorgente */
			/* con relativo modello */
			for(int j = 0; j < destOIDList.size(); j++) {

				/* per convenzione (ad esempio) gli schemi destinazione hanno tutti il nome */
				/* del primo schema sorgente */
				String name = namesList.get(0);
				/* per convenzione (ad esempio) gli schemi destinazione hanno tutti il modello */
				/* del primo schema sorgente */
				int modelOID = modelOIDList.get(0); 

				//Okay, 3 is done. Now 3.5 is the INSERT INTO SchemeSM that should be in the datalog rule itself.
				//It's here just because of time reasons.
				ph.createSuperModelSchemeUgly(destOIDList.get(j), modelOID, name);
			}

			//And 4 is done too, now we must execute the rule. What are the parameters?
			//String[] arguments = { Integer.toString(destOID), Integer.toString(srcOID) };
			List<Integer> argumentsList = new LinkedList<Integer>();
			argumentsList.addAll(destOIDList);
			argumentsList.addAll(schemeOIDList);

			// copy argumentsList in a String array
			String[] arguments = new String[argumentsList.size()];
			for(int j = 0; j < argumentsList.size(); j++) {
				int arg = argumentsList.get(j);
				String stringArg = Integer.toString(arg);
				arguments[j] = stringArg;
			}

			//Before we rock, we must create some temporary tables.
			// NOTE: this file is the macro file for the temporary table
			// the tmphandler create the file simplerules/ModelGen/hsqldb/ddl_temporaryTablesHandler.sql

			TmpHandler.tmpHandlerMainRoutine(rules[i]);
			MacroRule temporaryTables = 
				MacroRuleReader.read(rootnode + macrorulePath + 
						TmpHandler.TEMPORARY_TABLES_FILE);
			RuleApplicator.applyRule(temporaryTables, null);

			RuleApplicator.applyRule(mrdp.istanziaMacroRuleSQL(macroRuleFullPath),
					arguments);
			// Moshi moshi? If you can read this, we survived the application of the 
			// rules.
			// The next step will require the source OID to be our current destination 
			// OID
			// FIXME: come fare?
			// srcOID = destOID; 
		}
		// Let's see the results of our hard work.
		// The last destOID is the good one
		// Scheme newScheme = ph.loadSuperModelScheme(destOID);
	}


	/**
	 * Applies both scheme and instance rules.
	 * 
	 * @param rules the rules to apply
	 * @param schemaName the target scheme's name to store
	 * @param instanceName the target instance's name to store
	 * @param sourceSchemaOID the source scheme's oid
	 * @param sourceInstanceOID the source instance's oid
	 * @param modelOID the target model's oid
	 * @throws Exception if an error occurs
	 */
	public void applyBothSchemaAndDataTranslations(String[] rules, String schemaName, String instanceName, int sourceSchemaOID, 
			int sourceInstanceOID, int modelOID) throws Exception {
		List<String> schemaNameList = new LinkedList<String>();
		List<String> instanceNameList = new LinkedList<String>();
		List<Integer> sourceSchemaOIDList = new LinkedList<Integer>();
		List<Integer> sourceInstanceOIDList = new LinkedList<Integer>();
		List<Integer> modelOIDList = new LinkedList<Integer>();

		schemaNameList.add(schemaName);
		instanceNameList.add(instanceName);
		sourceSchemaOIDList.add(sourceSchemaOID);
		sourceInstanceOIDList.add(sourceInstanceOID);
		modelOIDList.add(modelOID);

		this.applyBothSchemaAndDataTranslations(rules, schemaNameList, instanceNameList, sourceSchemaOIDList, sourceInstanceOIDList, modelOIDList);
	}

	/**
	 * Applies both scheme and instance rules with N sources and M targets.
	 * 
	 * For every rule, creates M target scheme with the
	 * specified name. 
	 * 
	 * <p>The number M depends on the result of
	 * {@link org.modelgen.regole.MacroRegola#getDestSchemesNumber()},
	 * returned internally.
	 * 
	 * @param rules the rules to apply
	 * @param schemaNameList the target scheme's name to store
	 * (this method uses for now only the first element)
	 * @param instanceNameList the target instance's name to store
	 * (this method uses for now only the first element)
	 * @param sourceSchemaOIDList the N source scheme's oids
	 * @param sourceInstanceOIDList the N source instance's oids
	 * @param modelOIDList the target model's oids
	 * (this method uses for now only the first element)
	 * @throws Exception if an error occurs
	 */
//	FIXME: secondo le convenzioni, non dovrei avere un solo modelOID e un solo name?
	public void applyBothSchemaAndDataTranslations(String[] rules, List<String> schemaNameList, List<String> instanceNameList, List<Integer> sourceSchemaOIDList, 
			List<Integer> sourceInstanceOIDList, List<Integer> modelOIDList) throws Exception {

		// Check parameters consistency FIXME: con le nuove convenzioni inutile?
		boolean isSchemeConsistent = schemaNameList.size() == sourceSchemaOIDList.size() && 
		sourceSchemaOIDList.size() == modelOIDList.size();
		boolean isInstanceConsistent = instanceNameList.size() == sourceInstanceOIDList.size();
		if(!isSchemeConsistent || !isInstanceConsistent) {
			throw new IOException("Parameters are inconsistent!");
		}

//		the last parameter should be removed. however, we cannot infer the target
//		model by the translation rules. we have to specify it at the beginnig
//		of the translation. this also means that if the translation has more
//		than one step (rule) the the intermediate schemas will be associated
//		with the wrong model

		// Check the passed name, because it's optional in form
		for(int i = 0; i < schemaNameList.size(); i++) {
			String schemaName = schemaNameList.get(i);
			if (schemaName.length() == 0) { //We use the same name.
				Scheme srcScheme = ph.getSuperModelScheme(sourceSchemaOIDList.get(i));
				schemaName = srcScheme.getName();
				schemaNameList.set(i, schemaName);
			}
		}

		for(int i = 0; i < instanceNameList.size(); i++) {
			String instanceName = instanceNameList.get(i);
			if (instanceName.length() == 0) { //We use the same name.
				Instance instance = ph.getSuperModelInstance(sourceInstanceOIDList.get(i));
				instanceName = instance.getName();
				instanceNameList.set(i, instanceName);
			}
		}

		boolean applyMacroRuleAgain = false;
		int macroruleApplicationCount = 0;
		do {
			macroruleApplicationCount++;
			for (int i = 0; i < rules.length; i++) {
				//System.out.println("ARGUMENTS: " + sourceSchemaOID + " " + targetSchemaOID + " " + sourceInstanceOID + " " + targetInstanceOID);
				/* crea lista delle destinazioni */
				List<Integer> targetSchemaOIDList = new LinkedList<Integer>();
				List<Integer> targetInstanceOIDList = new LinkedList<Integer>();

				/* path della macroregola */
				String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
				String macrorulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
				String macroRuleFullPath = rootnode + macrorulePath + rules[i];

				/* parsing della macroregola */
				MacroRegola mrdp = 
					ParametricDatalogRulesReader.read(macroRuleFullPath);



				/* inserimento di tutte le destinazioni */
				int destOID = ph.getNewOID("SchemeSM", "OID"); // FIXME: eliminare perche'...
				for(int q=0;q<mrdp.getDestSchemesNumber();q++) {
					// ... FIXME: non dovrebbe essere add(ph.getNewOID("SchemeSM","OID"))?
					// il risultato e' lo stesso, ma gli effetti collaterali sul ph no
					targetSchemaOIDList.add(destOID+q);
				} // FIXME: perche' uscire dal ciclo? posso eseguire ugly query da qui


				/* registrazione di tutti gli schemi */
				/* con relativi modelli */
				for(int j = 0; j < targetSchemaOIDList.size(); j++) {
					/* per convenzione (ad esempio) gli schemi destinazione hanno tutti il nome
					 * del primo schema sorgente */
					String name = schemaNameList.get(0);
					/* per convenzione (ad esempio) gli schemi destinazione hanno tutti il modello
					 * del primo schema sorgente */
					int modelOID = modelOIDList.get(0); 

					/* registrazione schema */
					ph.createSuperModelSchemeUgly(targetSchemaOIDList.get(j), modelOID, name);
				}

				/* registrazione di tutte le istanze destinazione (una per ogni schema sorgente) */
				// Okay, 3 is done. Now 3.5 is the INSERT INTO InstanceSM that should be in the datalog rule itself.
				// It's here just because of time reasons.
				for(int k=0;k<targetSchemaOIDList.size();k++) {
					int targetInstanceOID = ph.getNewOID("InstanceSM", "OID");
					targetInstanceOIDList.add(targetInstanceOID);
					/* per convenzione le istanze hanno un nome */
					/* derivato dal nome del primo schema */
					ph.createSuperModelInstanceUgly(targetInstanceOID, targetSchemaOIDList.get(k), schemaNameList.get(0) + "_instance");
				}


				/* lettura macroregola di istanza */
				String instanceMacroRules = ModelgenConfigurationManager.getIstance().getProperty("instanceMacroRules", "", "paths");
				//String instancerulePath = config.getProperty("InstanceMacroRules", "", "paths");

				String macroRuleSchemaPath = rootnode + macrorulePath + rules[i];
				String macroRuleInstancePath = rootnode + macrorulePath + instanceMacroRules + rules[i];

				//Let's rock!
				MacroRegola schemaMrdp = 
					ParametricDatalogRulesReader.read(macroRuleSchemaPath);

				//And 4 is done too, now we must execute the rule. What are the parameters?
				//			parameters for the schema translation
				//String[] schemaArguments = { Integer.toString(targetSchemaOID), Integer.toString(sourceSchemaOID) };
				List<Integer> argumentsList = new LinkedList<Integer>();
				argumentsList.addAll(targetSchemaOIDList);
				argumentsList.addAll(sourceSchemaOIDList);
				// copy argumentsList in a String array
				String[] schemaArguments = new String[argumentsList.size()];
				for(int j = 0; j < argumentsList.size(); j++) {
					int arg = argumentsList.get(j);
					String stringArg = Integer.toString(arg);
					schemaArguments[j] = stringArg;
				}

				TmpHandler.tmpHandlerMainRoutine(rules[i]);

				MacroRule temporaryTables = 
					MacroRuleReader.read(rootnode + macrorulePath + 
							TmpHandler.TEMPORARY_TABLES_FILE);
				RuleApplicator.applyRule(temporaryTables, null);

				//			Schema translation
				RuleApplicator.applyRule(schemaMrdp.istanziaMacroRuleSQL(macroRuleSchemaPath),
						schemaArguments);

				//String[] instanceArguments = { Integer.toString(targetSchemaOID), Integer.toString(sourceSchemaOID),
				//	Integer.toString(targetInstanceOID), Integer.toString(sourceInstanceOID) };
				argumentsList.addAll(targetInstanceOIDList);
				argumentsList.addAll(sourceInstanceOIDList);
				// copy argumentsList in a String array
				String[] instanceArguments = new String[argumentsList.size()];
				for(int j = 0; j < argumentsList.size(); j++) {
					int arg = argumentsList.get(j);
					String stringArg = Integer.toString(arg);
					instanceArguments[j] = stringArg;
				}
				//Let's rock!
				//			System.out.println("macrorule: " + macroRuleInstancePath);

				MacroRegola instanceMrdp = 
					ParametricDatalogRulesReader.read(macroRuleInstancePath);
				//Before we rock, we must create some temporary directories.

				//			RuleApplicator.applyRule(temporaryTables, null);

				//			remind that an instance rule relies on TMP tables populated by
				//			the (related) schema translation
				//			Instance translation
				RuleApplicator.applyRule(instanceMrdp.istanziaMacroRuleSQL(macroRuleInstancePath),
						instanceArguments);

				//			check if the macrorule have to be reapplied
				//			It has to be re applied if it

				// FIXME: come fare?
				//sourceSchemaOID = targetSchemaOID;
				//sourceInstanceOID = targetInstanceOID;
			}
//			the macro must be applied at least 2 times. This is to see if
//			the second application return a different schema wrt the first one.
//			If we are at the end of the first application, then do nothing
//			and re-apply the first one
//			if (macroruleApplicationCount>1) {
//			now check if the macro should be applied again
//			}



//			collect the instances oids to be deleted: they are associated to
//			temporary instances (and schemas)
//			List<Integer> instancesToDelete = new LinkedList<Integer>();
//			instancesToDelete.add(sourceInstanceOID);

		} while (applyMacroRuleAgain);

	}

	/**
	 * Translate the schemes macrorule files to the corresponding instance rules, creating the corresponding files.
	 * @param rules the schema macrorule files
	 * @param overwrite if overwrite existing target files
	 * @throws IOException errors in file writing
	 * @throws InstanceRulesDuplicateGenerationException if (overwrite==false) and macrorule or simplerule for instance data already exists
	 * @throws InstanceTranslationException errors in translation from scheme rule to instance rule 
	 */
	public void generateInstanceRules(String[] rules, boolean overwrite) throws IOException, InstanceRulesDuplicateGenerationException, InstanceTranslationException {
		GenerateInstanceRuleRepository dataRulesGenerator = new GenerateInstanceRuleRepository();
		for (int i = 0; i < rules.length; i++) {
			dataRulesGenerator.createInstanceMacroRuleFile(rules[i], overwrite);
		}
	}
	
	/**
	 * Dato il nome del file in cui � contenuta la macroregola, restituisce l'oggetto
	 * MacroRuleSignature desiderato
	 * @param path il nome del file in cui � contenuta la macroregola
	 * @return la MacroRuleSignature della MacroRegola contenuta nel file
	 */
	public MacroRuleSignature getMacroRegolaByfileName(String path){
		String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
	    String macrorulePath = ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
	    String macroRuleFullPath = rootnode + macrorulePath + path;
		MacroRegola mrdp = ParametricDatalogRulesReader.read(macroRuleFullPath);
		return mrdp.getSignature(path);
	}

	/**
	 * Get all available schemes. It obtains the schemes from the db
	 * or retries them from cache. The method {@link #unLoadSchemes()}
	 * cleans the cache and forces to reload these values from the db.
	 * 
	 * @return the available schemes
	 * @throws PersistenceException errors in db access
	 */
	public List<Scheme> getSchemesLoaded() throws PersistenceException{
		//Se i modelli sono gi� stati caricati
		if (ss!=null)
			return ss;
		//altrimenti caricamente e restituzione modelli
		ss = new LinkedList<Scheme>();

		for(Scheme schemeParziale : ph.getSchemes()){
			//int oid = schemeParziale.getOID();
			//Scheme schemeCompleto = ph.loadScheme(oid);

			// ora e' sufficiente il solo schema parziale (caricamento trasparente)
			ss.add(schemeParziale);
		}	
		List<Scheme> list = ss;
		return list;
	}

	/**
	 * Reset cached values for {@link #getSchemesLoaded()} method
	 */
	public void unLoadSchemes(){
		ss = null;
	}

	/**
	 * Searches in schemes cached in {@link #getSchemesLoaded()} the
	 * specified scheme's name.
	 * @param name the scheme's name to find
	 * @return the scheme found, null otherwise
	 */
	public Scheme getSchemeByName(String name) {
		if (ss==null){
			try {
				ss = getSchemesLoaded();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		Scheme schemeByName = null;
		for(Scheme scheme : ss){
			if (scheme.getName().equals(name)){
				schemeByName = scheme; 
			}
		}
		return schemeByName;
	}

	/**
	 * Creates (but don't store!!) a new scheme.
	 * 
	 * The new scheme created is available in {@link #getSchemesLoaded()}
	 * 
	 * @param name the scheme's name
	 * @param m the scheme's model
	 * @return always true (??)
	 */
	public boolean newScheme(String name, Model m){
		boolean created = false; 
		Scheme s = new Scheme(NEWSCHEMEOID, name, m, false);
		created = ss.add(s);

		return created;
	}

	/**
	 * Add the specified scheme object to the scheme.
	 * 
	 * This is not stored: it's only available in {@link #getSchemesLoaded()}.
	 * 
	 * @param s the working scheme
	 * @param objContruct the referring construct
	 * @param objName the scheme object's name
	 * @param objPropertyMap the scheme object's property (pairs property-value)
	 * @param objReferenceMap the scheme object's reference (pairs reference-value pointed)
	 * @return always true (??)
	 */
	public boolean addSchemeObject(Scheme s, Construct objContruct, String objName, Map<Property,String> objPropertyMap, Map<Reference,SchemeObject> objReferenceMap){
		boolean created = true;
		SchemeObject obj = new SchemeObject(0, objName, objContruct);

		Iterator<Property> it1 = objPropertyMap.keySet().iterator();
		Property p;
		String type;
		while(it1.hasNext()){
			p = it1.next();
			type = p.getType();
			if(type.equals("boolean")||type.equals("bit")){
				obj.setProperty(p.getName(),(new Boolean(objPropertyMap.get(p)).booleanValue()));
			}else{
				obj.setProperty(p.getName(),objPropertyMap.get(p));
			}
		}

		Iterator<Reference> it2 = objReferenceMap.keySet().iterator();
		Reference r;
		while(it2.hasNext()){
			r = it2.next();
			obj.setReference(r.getName(),objReferenceMap.get(r));
		}

		s.addObject(obj);

		return created;
	}

	/**
	 * Modifies the specified scheme object.
	 * 
	 * This is not stored: it's only available in {@link #getSchemesLoaded()}.
	 * 
	 * @param obj the working scheme
	 * @param objName the scheme object's name
	 * @param objPropertyMap the scheme object's property (pairs property-value)
	 * @param objReferenceMap the scheme object's reference (pairs reference-value pointed)
	 * @return always true (??)
	 */
	public boolean modifySchemeObject(SchemeObject obj, String objName, Map<Property, String> objPropertyMap, Map<Reference, SchemeObject> objReferenceMap) {
		boolean modified = true;

		obj.setName(objName);

		Iterator it = objPropertyMap.keySet().iterator();
		Property p;
		String type;
		while(it.hasNext()){
			p = (Property) it.next();
			type = p.getType();
			if(type.equals("boolen")||type.equals("bit")){
				obj.setProperty(p.getName(),(new Boolean(objPropertyMap.get(p)).booleanValue()));
			}else{
				obj.setProperty(p.getName(),objPropertyMap.get(p));
			}
		}

		it = objReferenceMap.keySet().iterator();
		Reference r;
		while(it.hasNext()){
			r = (Reference)it.next();
			obj.setReference(r.getName(),objReferenceMap.get(r));
		}


		return modified;

	}	

	/**
	 * Dato uno schema e un nuovo nome, clona lo schema
	 * ma NON lo SALVA nel DB.
	 * @param schema the scheme to clone
	 * @param name the name of the new scheme
	 */
	public boolean cloneScheme(Scheme schema, String name){
		/*Settando l'OID a NEWMODELOID sappiamo che il modello
		 *� stato creato in questa sessione di lavoro*/
		Scheme s = new Scheme(NEWSCHEMEOID, name, schema.getMetaInfo(), false);

		//itero sugli schemeObject dello schema
		for (SchemeObject so : schema.getAbstractConstructs()) {	
			SchemeObject obj = new SchemeObject(0, so.getName(), so.getMetaInfo());

			for (SchemeObjectProperty sop : so.getProperties()) {
				obj.setProperty(sop.getMetaInfo().getName(),sop.getValue());
			}

			for (SchemeObjectReference sor : so.getReferences()) {
				/*
				 * Cerchiamo, tra gli SchemeObject gi� aggiunti allo schema, quello a cui si riferisce
				 * la SchemeObjectReference: infatti la SchemeObjectReference punta allo SchemeObject dello
				 * schema da clonare, mentre noi vogliamo il corrispondente in quello clonato.
				 * Siamo inoltre sicuri che lo SchemeObject sia gi� presente nello schema perch� nella Collection
				 * di SchemeObject di uno schema, viene prima.
				 */
				obj.setReference(sor.getMetaInfo().getName(),findSObjectReference(sor,s.getAbstractConstructs()));
			}

			s.addObject(obj);
		}

		return ss.add(s);
	}

	/**
	 * Di supporto al metodo {@link #cloneScheme(Scheme schema, String name)}.
	 * Cerca lo SchemeObject (tra una lista di SchemeObject) che ha lo stesso nome di quello
	 * di una SchemeObjectReference passata per parametro.
	 */
	private SchemeObject findSObjectReference(SchemeObjectReference sor, Collection<SchemeObject> objs){
		if(sor.getValue()!=null){
			for (SchemeObject so : objs) {
				if(so.getName().equals(sor.getValue().getName())){
					return so;
				}
			}
		}
		return null;
	}

	/**
	 * Import (store) a scheme from its specified xml version.
	 * 
	 * This is stored and it's also available in {@link #getSchemesLoaded()}.
	 * 
	 * @param xmlScheme the xml file to import
	 * @return always true (??)
	 * @throws Exception errors in db access or in import function
	 */
	public boolean importScheme(String xmlScheme) throws Exception{
		boolean imported = false;
		Scheme importedScheme = null;
		Scheme reloadedScheme = null;

		importedScheme = Importer.importScheme(xmlScheme);

		if(importedScheme!=null){
			reloadedScheme = ph.getScheme(importedScheme.getOID());
		}
		if(reloadedScheme!=null){
			imported = ss.add(reloadedScheme);
		}
		return imported;		 
	}

	/**
	 * Deletes (from the db) the specified scheme if not new. It also removes
	 * the cached available schemes in {@link #getSchemesLoaded()}
	 * @param s the scheme to delete
	 * @throws PersistenceException errors in db remove
	 */
	public void deleteScheme(Scheme s) throws PersistenceException{

		/*Se il modello era gia presente nel database va eliminato dal db e non solo dalla mappa*/

		int sOID = s.getOID();
		if(s.getOID()!=NEWSCHEMEOID){
			/*Nota: se il modello � stato prelevato dal db l'OID � presente nel modello e quindi 
			 *pu� essere sfruttato per l'eliminazione dalla base di dati*/
			ph.deleteScheme(sOID);
		}
		ss.remove(s);
	}

	/**
	 * Store the specified scheme
	 * @param s the scheme info to save
	 * @throws PersistenceException errors in db access
	 */
	public void storeScheme(Scheme s) throws PersistenceException{
		boolean isNew= s.getOID()==NEWSCHEMEOID;
		/* qui la cancellazione dovrebbe andare bene perché */
		/* uno schema non definisce tabelle apposite, sono definite già */
		/* dal suo modello */
		if(!isNew){
			int oldOID = s.getOID();
			ph.deleteScheme(oldOID);
		}

		//Si vedano i cambiamenti apportati a questo metodo nel ph (si setta l'oid)
		ph.storeScheme(s);
	}

	/**
	 * Export a scheme from its xml version.
	 * 
	 * @param file the output file
	 * @param xmlScheme the xml file to save in the output file
	 * @throws IOException errors in file writing
	 */
	public void exportScheme(File file, String xmlScheme) throws IOException{
		FileWriter writer = null;
		try {
			writer = new FileWriter(file,false);
			writer.write(xmlScheme);		        	
		} catch (IOException e) {
			throw e;
		}finally{
			if(writer != null) {
				writer.close();
			}
		}
	}

	/**
	 * Deletes (not from the db!!) a scheme object from the scheme
	 * @param obj the scheme object to remove
	 * @param scheme the context scheme
	 */
	public void deleteSchemeObject(SchemeObject obj, Scheme scheme) {
		scheme.removeObject(obj.getKey());
	}

	/**
	 * Renames from the db the scheme
	 * @param schemeOID the scheme's oid to rename
	 * @param schemeName the new scheme's name
	 * @throws PersistenceException errors in db renaming
	 */
	public void renameScheme(int schemeOID, String schemeName) throws PersistenceException {
		ph.renameScheme(schemeOID, schemeName);
	}

	/**
	 * Returns the super model
	 * @return the super model
	 * @throws PersistenceException db access error
	 */
	public Model getSuperModel() throws PersistenceException {
		if(superModel==null)
			superModel = ph.getSuperModel();
		return superModel;		
	}

	/**
	 * Returns the supermodel's constructs collection
	 * @param filtered set it to false (bug) ...
	 * @return Collection the supermodel's constructs
	 * @throws PersistenceException db access error
	 */
	public Collection<Construct> populateCmbConstruct(boolean filtered) throws 
	PersistenceException {
		Model superModel = ph.getSuperModel();
		Collection<Construct> constructs = null;
		if(filtered==false)
			constructs = superModel.getAbstractConstructs();
		/*else {
			constructs = model.getAvailableConstructs(superModel);
		}*/
		return constructs;
	}

	/**
	 * Returns the skolem functions owning to the specified construct
	 * @param constructName the owning construct
	 * @return the skolem functions owning to the specified construct
	 * @throws PersistenceException db access error
	 */
	public Collection<String> populareCmbFunction(String constructName) throws 
	PersistenceException {
		// The collection with the interesting function
		Collection<String> visibleFunctions = new ArrayList<String>();
		/* Look for interesting function */
		for(SkolemFunction function : ph.getSkolemFunctions()) {
			String functionConstructName = function.getOwnerConstruct().getName();
			if(functionConstructName.equals(constructName)) {
				visibleFunctions.add(function.getName());
			}
		}
		return visibleFunctions;
	}

	/**
	 * Return all available skolem functions
	 * @return Collection all available skolem functions
	 * @throws PersistenceException db access error
	 */
	public Collection<SkolemFunction> getSkolemFunctions() throws PersistenceException {
		Collection<SkolemFunction> functions = ph.getSkolemFunctions();
		return functions;
	}

	/**
	 * Check if the specified function name is already in the list 
	 * {@link #getSkolemFunctions()}
	 * @param name the function name's to search
	 * @return if the specified function name is already in the list 
	 * {@link #getSkolemFunctions()}
	 */
	public boolean checkSkolemFunction(String name) {
		boolean exist = false;
		Iterator it = null;
		try {
			it = getSkolemFunctions().iterator();
			while(it.hasNext()) {
				SkolemFunction function = (SkolemFunction)it.next();
				/* If function is already in the list, the background of txtFunctionName
				 * will be RED */
				if(name.equals(function.getName())) 
					exist = true;
			}
		} catch (PersistenceException e) {} 
		return exist;
	}

	/**
	 * Check if the skolem function name is already in the specified list
	 * of arguments
	 * @param name the function name to search in the arguments list
	 * @param arguments the arguments list
	 * @return if the skolem function name is already in the specified list
	 * of arguments
	 */
	public boolean checkSkolemFunctionArgument(String name, Collection<String> arguments) {
		boolean exist = false;
		Iterator<String> it = arguments.iterator(); 
		while(it.hasNext()) {
			String argument = it.next();
			String effectiveName = null;
			/* Parser over argument name*/
			for(int i = 0; i < argument.length(); i++) {
				if(argument.charAt(i) == ' ')
					effectiveName = argument.substring(0, i);
			}
			/* If function is already in the list, the background of txtFunctionName
			 * will be RED */
			if(effectiveName != null && effectiveName.equals(name)) 
				exist = true;
		}
//		moreover, the name should not exists as an argument name for 
//		any skolem with the same target consturct

		return exist;
	}

	/**
	 * Creates (but don't store!!) a new Skolem function.
	 * 
	 * <p>The function skolem created becomes the working function
	 * skolem in functions {@link #addSkolemArgument(String, String, String)},
	 * and in {@link #storeSkolemFunction()}.
	 * 
	 * <p>To reset the working function skolem, see
	 * {@link #resetSkolemFunction()}
	 * 
	 * @param functionName The function name
	 * @param constructName The construct name
	 * @return SkolemFunction The current Skolem function
	 * @throws PersistenceException
	 */
	public SkolemFunction createNewSkolemFunction(String functionName, 
			String constructName) throws PersistenceException {
		// Take model and construct
		Model model = 
			getSuperModel();
		Construct ownerConstruct = model.getConstruct(constructName);
		if(this.currentSkolemFunction != null) {
			/* Change the name of the function */
			this.currentSkolemFunction.setName(functionName);
			this.currentSkolemFunction.setOwnerConstruct(ownerConstruct);
		} else {
			this.currentSkolemFunction = new SkolemFunction();
			// Setparameters
			this.currentSkolemFunction.setName(functionName);
			this.currentSkolemFunction.setOwnerConstruct(
					ownerConstruct);
		}
		return this.currentSkolemFunction;
	}

	/**
	 * Resets the working skolem function setted in
	 * {@link #createNewSkolemFunction(String, String)}
	 *
	 */
	public void resetSkolemFunction() {
		this.currentSkolemFunction = null;
	}

	/**
	 * Add argument to the working skolem function
	 * setted in {@link #createNewSkolemFunction(String, String)}
	 * @param argumentName The argument's name
	 * @param type The argument's type
	 * @param constructName the source construct's name
	 * @throws PersistenceException db access error
	 */
	public void addSkolemArgument(String argumentName, String type, String constructName) throws PersistenceException {
		// Create argument
		SkolemFunctionArgument argument = new SkolemFunctionArgument();
		argument.setName(argumentName);
		argument.setType(type);
		if (superModel==null) {
			superModel = ph.getSuperModel();
		}
		Construct sourceConstruct = superModel.getConstruct(constructName);
		argument.setSourceConstruct(sourceConstruct);
		// Add to Skolem function
		this.currentSkolemFunction.addArgument(argument);
	}

	/**
	 * Stores the working skolem function setted in 
	 * {@link #createNewSkolemFunction(String, String)}
	 * @throws PersistenceException db access error
	 */
	public void storeSkolemFunction() throws PersistenceException {
		ph.storeSkolemFunction(getSuperModel(), this.currentSkolemFunction);
		this.currentSkolemFunction = null;
	}

	/**
	 * Deletes Skolem function
	 * @param constructName the skolem function's owner construct
	 * @param functionName the skolem function's name
	 * @throws PersistenceException db access error
	 */
	public void deleteSkolemFunction(String constructName, 
			String functionName) throws PersistenceException {
		Model superModel = ph.getSuperModel();
		Construct construct = superModel.getConstruct(constructName);
		ph.deleteSkolemFunction(superModel, construct, functionName);
	}

	/**
	 * Import from specify xml file
	 * @param xmlFile The file name
	 * @return the imported skolem functions
	 * @throws Exception errors in db connection or in import routine
	 */
	public Collection<SkolemFunction> importSkolemFunctions(String xmlFile) throws Exception {
		return Importer.importSkolemFunctions(xmlFile);
	}

	/**
	 * Return the xml rappresentation of the specified skolem functions of Skolem function to export
	 * @return  the xml rappresentation of the specified skolem functions of Skolem function to export
	 * @throws PersistenceException db access error
	 * @throws JAXBException errors in the XML/Java binding
	 */
	public String getXmlSkolemFunctions() throws PersistenceException, JAXBException {
		Collection<SkolemFunction> functions = ph.getUserDefinedSkolemFunctions();
		String xmlFunctions = Exporter.skolem2XML(functions);
		return xmlFunctions;
	}

	/**
	 * Checks if the specified skolem function's name or arguments names 
	 * already exists in known skolem functions
	 * @param currentFunction the skolem function to check
	 * @return if the specified skolem function's name or arguments names 
	 * already exists in known skolem functions
	 * @throws PersistenceException db access error
	 */
	public boolean isSkolemFunctionSafe(SkolemFunction currentFunction) throws PersistenceException {
		boolean isSkolemSafe = true;
//		there should not exists another function with the same name
		Collection<SkolemFunction> skolems = getSkolemFunctions();
		for (SkolemFunction function : skolems) {
			if (function.getName().equals(currentFunction.getName())) {
				isSkolemSafe = false;
				break;
			}
			for (SkolemFunctionArgument argument : function.getArguments()) {
				for (SkolemFunctionArgument argument2 : currentFunction.getArguments()) {
					if (argument2.equals(argument)) isSkolemSafe = false;
					break;
				}
				if (!isSkolemSafe) break;
			}
		}

		return isSkolemSafe;
	}


	/**
	 * Get all available models. It obtains the models the db
	 * or retries them from cache. The method {@link #unLoadModel()}
	 * cleans the cache and forces to reload these values from the db.
	 * 
	 * @return the available models
	 * @throws PersistenceException errors in db access
	 */
	public List<Model> getModelsLoaded() throws PersistenceException {
		//Se i modelli sono gi� stati caricati
		if (ms!=null)
			return ms;
		//altrimenti caricamente e restituzione modelli
		ms = new LinkedList<Model>();

		//Model superModel = getSuperModel();
		for (Model modParziale : ph.getModels()){
			//int oid = modParziale.getOID();
			//Model modCompleto = ph.loadModel(oid,superModel);

			// riccardop: ora e' sufficiente il solo modello parziale (caricamento trasparente)
			ms.add(modParziale);
		}

		List<Model> list = ms;
		return list;
	}

	/**
	 * Cleans the models stored in {@link #getModelsLoaded()}
	 *
	 */
	public void unLoadModel(){
		ms = null;
	}

	/**
	 * Searches in models cached in {@link #getModelsLoaded()} the
	 * specified model's name.
	 * @param name the model's name to find
	 * @return the model found, null otherwise
	 */
	public Model getModelCachedByName(String name) {
		if (ms==null){
			try {
				ms = getModelsLoaded();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		Model modelByName = null;
		for(Model mod : ms){
			if (mod.getName().equals(name)){
				modelByName = mod;
			}
		}
		return modelByName;
	}

	/**
	 * Searches in models cached in {@link #getModelsLoaded()} the
	 * specified model's short name.
	 * @param shortName the model's short name to find
	 * @return the model found, null otherwise
	 */
	public Model getModelCachedByShortName(String shortName) {
		if (ms==null){
			try {
				ms = getModelsLoaded();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		Model modelByName = null;
		for(Model mod : ms){
			if (mod.getShortName().equals(shortName)){
				modelByName = mod;
			}
		}
		return modelByName;
	}

	/**
	 * Creates (but don't store!!) a new scheme.
	 * 
	 * The new scheme created is available in {@link #getModelsLoaded()}
	 * 
	 * @param name the model's name
	 * @param shortName the model's short name
	 * @return always true (??)
	 */
	public boolean newModel(String name, String shortName){
		boolean created = false;
		Model m = new Model(NEWMODELOID, name);
		m.setShortName(shortName);
		/*Settando l'OID a NEWMODELOID sappiamo che il modello
		 *� stato creato in questa sessione di lavoro*/
		created = ms.add(m);

		return created;
	}

	/**
	 * Imports a model from an xml file
	 * @param xmlModel the model to import
	 * @param mode if "flat" imports a flat xml file, nested otherwise
	 * @return if the new model has been added
	 * @throws Exception db access errors or import errors
	 */
	public boolean importedModel(String xmlModel, String mode) throws Exception{
		boolean imported = false;
		String FLATMODE = "flat";	 
		Model importedModel = null;
		Model reloadedModel = null;

		if (mode.equals(FLATMODE))
			importedModel = Importer.importModel(xmlModel);
		else
			importedModel = Importer.importNestedModel(xmlModel);

		if(importedModel!=null){
			reloadedModel = ph.getModel(importedModel.getOID());
		}

		if(reloadedModel!=null){
			imported = ms.add(reloadedModel);
		}
		return imported;		 

	}

	/** 
	 * Deletes a model from the model's list and from the database if the model
	 * is not new.
	 *
	 * @param m the model to delete
	 */
	public void deleteModel(Model m) throws PersistenceException{

		/*Se il modello era gia presente nel database va eliminato dal db e non solo dalla mappa*/

		int mOID = m.getOID();

		if(m.getOID()!=NEWMODELOID){
			/*Nota: se il modello � stato prelevato dal db l'OID � presente nel modello e quindi 
			 *pu� essere sfruttato per l'eliminazione dalla base di dati*/
			ph.deleteModel(mOID);
		}
		ms.remove(m);

	}

	/**
	 * Adds (but don't store!!) a construct to the model.
	 * @param model the working model
	 * @param constructName the construct�s name to add
	 * @param sup_construct the referring meta-construct
	 */
	public void addConstructToModel(Model model, String constructName, Construct sup_construct){
		Construct newConstruct = this.createConstruct(constructName, sup_construct);
		Construct smConstruct = sup_construct;

		//The references are already done: the construct is complete.
		model.addConstruct(newConstruct);

		System.out.println("Sto per iterare sulle ref");
		for (Reference smReference : smConstruct.getReferences()) {

			//We do the references here since they depend on the super model constructs.
			//For each reference in the sm construct, do a simple copy of it in the model!
			//NOTE: this because the only thing which should be changeable in a reference is the name, and it will be changeable later.
			System.out.println("Ref di turno: " + smReference);
			Reference reference = new Reference(0, smReference.getName(), smReference);
			reference.setType(smReference.getType()); //This will not be modifiable.
			reference.setOwnerConstruct(newConstruct); //This cannot be otherwise (see: two way navigation)

			//NOTA:I RIFERIMENTI VENGONO AGGIUNTI ALLA BASE DI DATI SOLO SE IL TARGETCONSTRUCT DEL CONSTRUCT E' DIVERSO DA NULL
			reference.setTargetConstruct(model.getConstructMappedTo(smReference.getTargetConstruct()));

			//Come nella versione precedente
			System.out.println("IF " + model.getConstructMappedTo(smReference.getTargetConstruct()));
			if(model.getConstructMappedTo(smReference.getTargetConstruct())!=null){ //momentaneo
				System.out.println("aggiungo al modello");
				newConstruct.addReference(reference);

			}
		}
		System.out.println("Ho iterato sulle ref");

	}

	/**
	 * Dato il nome del costrutto e il costrutto di riferimento nel supermodello,
	 * crea un nuovo oggetto Construct. Non gestisce le Reference, che invece sono
	 * gestite da {@link #addConstructToModel(Model model, String constructName, Construct sup_construct)}
	 */
	private Construct createConstruct(String constructName, Construct sup_construct){
		Construct newConstruct = new Construct(0, constructName);

		LinkedList<Construct> mappings = new LinkedList<Construct>();
		Construct smConstruct = sup_construct;
		mappings.addLast(smConstruct);

//		aggiung preventivamente il costrutto al modello, poi completo
//		la lista delle referenze sul costrutto
		newConstruct.setSuperModelConstructs(mappings);
		return newConstruct;
	}

	/**
	 * Adds (but don't store!!) a reference to a construct.
	 * @param construct the working construct
	 * @param referenceName the reference's name
	 * @param superReference the referring meta-reference
	 */
	public void addReferenceToConstruct(Construct construct, String referenceName, Reference superReference){
		Reference reference = new Reference(0, referenceName, superReference);
		reference.setOwnerConstruct(construct);
		reference.setType(superReference.getType());
		construct.addReference(reference);
	}

	/**
	 * Adds (but don't store!!) a property to a construct.
	 * @param construct the working construct
	 * @param propertyName the property's name
	 * @param superProperty the referring meta-property
	 */
	public void addPropertyToConstruct(Construct construct, String propertyName, Property superProperty){
		Property property = new Property(0,propertyName, superProperty);
		property.setOwnerConstruct(construct);
		property.setType(superProperty.getType());	
		construct.addProperty(property);
	}

	/**
	 * Adds (but don't store!!) a bond to a Construct
	 * @param construct
	 * @param bond
	 */
	public void addBondToConstruct(Construct construct, String bond){
		construct.setUserBonds(bond);
	}

	/**
	 * Adds (but don't store!!) a signature to a Construct
	 * @param construct
	 * @param signature
	 */
	public void addSignatureToConstruct(Construct construct, String signature){
		construct.setSignature(signature);
	}

	/**
	 * Removes (but don't store!!) a bond to a Construct
	 * @param construct
	 */
	public void removeBondToConstruct(Construct construct){
		construct.setUserBonds("true");
	}

	/**
	 * Removes (but don't store!!) a signature to a Construct
	 * @param construct
	 */
	public void removeSignatureToConstruct(Construct construct){
		construct.setSignature("true");
	}


	/**
	 * Deletes (not in the db!!) a construct from a model.
	 * 
	 * @param construct the construct to remove
	 * @param m the working model
	 * 
	 */
	public void deleteConstructFromModel(Construct construct, Model m){
		/* Sebbene l'azione sia atomica (come quella ri rinominazione di un modello),
		 * si � comunque definito tale metodo perch� si ritiene che 
		 * controlli  specifici possano interessarla
		 * */
		m.removeConstruct(construct);
	}

	/**
	 * Export a model to its xml version.
	 * 
	 * @param file the output file
	 * @param xmlModel the xml file to save in the output file
	 * @throws IOException errors in file writing
	 */
	public void exportModel(File file, String xmlModel) throws IOException{
		FileWriter writer = null;
		try {
			writer = new FileWriter(file,false);
			writer.write(xmlModel);		        	
		} catch (IOException e) {
			throw e;
		}finally{
			if(writer != null) {
				writer.close();
			}
		}
	}

	/**
	 * Store the specified model, deleting the old version if exists
	 * @param model the model info to save
	 * @throws PersistenceException errors in db access
	 */
	public void storeModel(Model model) throws PersistenceException{
		boolean isNew = (model.getOID()== NEWMODELOID);	
		
		if(!isNew){
			String name = model.getName(); // estraggo il nuovo nome
			String shortName = model.getShortName();
			int oldOid = model.getOID();
			
			if(model.isRenamed()) {
				model.setName(model.getOldName());
				model.setShortName(model.getOldShortName());
			}
			ph.deleteModel(oldOid);
			model.setName(name);
			model.setShortName(shortName);
			model.clearOldNamesOnSave();
		}

		//Si vedano i cambiamenti apportati a questo metodo nel ph (si setta l'oid)
		ph.storeModel(model);

	}


	/**
	 * Dato un modello, un nuovo nome e un nuovo short-name, clona il modello
	 * ma NON lo SALVA nel DB.
	 * @param model the model to clone
	 * @param name the name of the new model
	 * @param shortname the shortname of the new model
	 */
	public boolean cloneModel(Model model, String name, String shortname){
		/*Settando l'OID a NEWMODELOID sappiamo che il modello
		 *� stato creato in questa sessione di lavoro*/
		Model m = new Model(NEWMODELOID, name);
		m.setShortName(shortname);
		m.setAncestor(model.isAncestor());
		m.setFamilyOid(model.getFamilyOid());
		m.setFamily(model.getFamily());

		//itero sui costrutti di model
		for(Construct srcConstruct: model.getAbstractConstructs()){
			//cerco il costrutto del supermodello
			Collection<Construct> sup_constructs = srcConstruct.getSuperModelConstructs();
			Iterator<Construct> it = sup_constructs.iterator();
			Construct supCons = null;
			while(it.hasNext())
				supCons = it.next();

			//clono il costrutto e lo aggiungo al modello
			Construct c = this.createConstruct(srcConstruct.getName(), supCons);
			c.setSignature(srcConstruct.getSignature());
			c.setUserBonds(srcConstruct.getUserBonds());
			m.addConstruct(c);

			//itero sullepropriet� del costrutto
			for (Property srcProperty : srcConstruct.getProperties()) {
				this.addPropertyToConstruct(c, srcProperty.getName(), srcProperty.getMetaInfo());
			}

			//itero sulle reference del costrutto
			for (Reference srcReference : srcConstruct.getReferences()) {
				this.addReferenceToConstruct(c, srcReference.getName(), srcReference.getMetaInfo());
			}
		}

		return ms.add(m);
	}
	
	/**
	 * Retrieve the ancestor of the family
	 * @param familyOID
	 * @return the Model that is the ancestor of the given family
	 * @throws PersistenceException
	 */
	public Model getFamilyAncestor(int familyOID) throws PersistenceException{
		return ph.getFamilyAncestor(familyOID);
	}
	
	/**
	 * Retrieve all families ancestors
	 * @return all Model that are families ancestors
	 * @throws PersistenceException
	 */
	public List<Model> getAllFamilies() throws PersistenceException{
		return ph.getAncestors();
	}
	
	/**
	 * Retrieve all families ancestors signatures
	 * @return all ModelSignature of all families ancestors
	 * @throws PersistenceException
	 */
	public List<ModelSignature> getAllFamiliesSignatures() throws PersistenceException{
		List<Model> models = this.getAllFamilies();
		List<ModelSignature> result = new ArrayList<ModelSignature>();
		for(Model model: models)
			result.add(model.getModelSignature());
		return result;
	}
	
	/**
	 * Given a model signature, finds the model's family
	 * @param model the model signature
	 * @return the oid of the model's family, 0 if the model hasn't family
	 * @throws PersistenceException
	 */
	public int findFamily(ModelSignature model) throws PersistenceException{
		//carico le famiglie
		List<Model> families = GeneralManagementFacade.getInstance().getAllFamilies();
		
		//scandisco le famiglie
		for(Model ancestor: families){
			//se sono compatibili
			if(SignatureApplicationUtility.getInstance().isCompatibile(ancestor.getModelSignature(), model))
				return ancestor.getFamilyOid();
		}
		
		return 0;
	}

	/**
	 * Returns the dtabase connection.
	 * @return the db connection
	 */
	public Connection getConnection() {
		return connection;
	}

	/**
	 * On every db reload notified from 
	 * {@link org.modelgen.commons.persistence.PersistenceHandlerFactory#reload()}
	 * or from {@link org.modelgen.commons.persistence.PersistenceHandlerFactory#resetDBHandler(DBHandler)},
	 * calls automatically {@link #initPesistenceHandler()},
	 * {@link #unLoadModel()} and {@link #unLoadSchemes()}.
	 * 
	 */
	public void dbReloaded() {
		try {
			this.initPesistenceHandler();
			this.unLoadModel();
			this.unLoadSchemes();
		} catch (PersistenceException e) {
			e.printStackTrace();
		}
	}

}
