package org.modelgen.jpf.core;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.jpf.core.actions.StartDbServerAction;
import org.modelgen.jpf.core.actions.StopDbServerAction;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.swing.JOptionPane;
import javax.swing.UIManager;

import org.apache.log4j.PropertyConfigurator;
import org.java.plugin.PluginManager;
import org.java.plugin.boot.Application;
import org.java.plugin.boot.ApplicationPlugin;
import org.java.plugin.boot.Boot;
import org.java.plugin.registry.Extension;
import org.java.plugin.registry.ExtensionPoint;
import org.java.plugin.registry.PluginDescriptor;
import org.java.plugin.util.ExtendedProperties;


/**
 * This class represents the core plugin: the root plugin of the hierarchy.
 *
 */
public final class CorePlugin extends ApplicationPlugin implements Application {
	/**
	 * This plug-in ID.
	 */
	public static final String PLUGIN_ID = "org.modelgen";

	private File dataFolder;

	private ToolBoxFrame toolBox;


	/***************************************************************************
	 * Questa mappa viene predisposta per contenere informazioni (vedi ToolActivetionAction.java)
	 * condivise da tutti i plug-ins */    
	private Map<String,Object> applObjects = new HashMap<String,Object>();
	/*********************************************************************/

	private boolean manual_server;


	/**
	 * Configure all plugins log paths.
	 * 
	 * @see org.java.plugin.Plugin#doStart()
	 */
	protected void doStart() throws Exception {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Couldn't use system look and feel.");
		}
		
		configurePluginsLog(); // riccardop

		dataFolder = new File("data");
	}

	/**
	 * Stop the db server and save the modelgen configuration.
	 * 
	 * @see org.java.plugin.Plugin#doStop()
	 */
	protected void doStop() throws Exception {
//		if (manual_server){
		StopDbServerAction.getIstance().performsAction();
//		}
		/*Questo metodo consente il salvataggio delle modiche apportate a modelgen.xml*/
		ModelgenConfigurationManager.getIstance().saveConfiguration();
		/**************************************************************/
	}

	/**
	 * If requested in modelgen configuration, starts the db server.
	 * 
	 * @see org.java.plugin.boot.ApplicationPlugin#initApplication(
	 *      ExtendedProperties, String[])
	 */
	protected Application initApplication(final ExtendedProperties config, final String[] args) throws Exception {
		manual_server = ModelgenConfigurationManager.getIstance().getProperty("manual_server","","options").equals("true");
		if (manual_server){
			StartDbServerAction.getIstance().performsAction();
		}
		return this;
	}

	/**
	 * Creates and shows the GUI.
	 * 
	 * @see org.java.plugin.boot.Application#startApplication()
	 */
	public void startApplication() {  

		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});

	}

	/**
	 * Set plugin's specific logger info for log4j
	 * 
	 * @author Riccardo Pietrucci
	 *
	 */
	@SuppressWarnings("unchecked")
	private void configurePluginsLog() {
		PluginManager pm = PluginManager.lookup(ToolBoxFrame.class); 
		ExtensionPoint toolExtPoint =  pm.getRegistry().getExtensionPoint(CorePlugin.PLUGIN_ID, "Tool");
		Collection<Extension> extensions = toolExtPoint.getConnectedExtensions();
		if (extensions == null)
			return;
		for(Extension ext : extensions) {
			String plugin_id = ext.getDeclaringPluginDescriptor().getId();
			//String appender_id = plugin_id.replace('.', '_');
			String appender_id = ext.getId();
			Properties props = new Properties();
			props.setProperty("log4j.logger."+plugin_id, "ALL,"+appender_id);
			props.setProperty("log4j.appender."+appender_id, "org.apache.log4j.FileAppender");
			props.setProperty("log4j.appender."+appender_id+".file", "./logs/plugins/"+plugin_id+".log");
			props.setProperty("log4j.appender."+appender_id+".layout", "org.apache.log4j.PatternLayout");
			props.setProperty("log4j.appender."+appender_id+".layout.conversionPattern", "%d [%t] %-5p %c %m%n");
			props.setProperty("log4j.appender."+appender_id+".append", "false");
			// Set the log4j properties
	        PropertyConfigurator.configure(props);
		}
	}

	private void createAndShowGUI() {       
		//NOTA: construttore con la mappa condivisa
		toolBox = new ToolBoxFrame(applObjects, this);

		toolBox.addWindowListener(new WindowAdapter() {
			public void windowClosing(final WindowEvent e) {
				try {
					JOptionPane.getRootFrame().dispose();
					Boot.stopApplication(CorePlugin.this);
				} catch (Exception ex) {
					// ignore
				}
				System.gc();
				System.exit(0);
			}
		});
	}

	/**
	 * Returns folder where given plug-in can store it's data.
	 * @param descr plug-in descriptor
	 * @return plug-in data folder
	 */
	public File getDataFolder(final PluginDescriptor descr) {
		File result = new File(dataFolder, descr.getId());
//		if (!result.isDirectory() && !result.mkdirs()) {
//		throw new IOException("can't create data folder " + result + " for plug-in " + descr.getId());
//		}
		return result;
	}
}
