package org.modelgen.jpf.core;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


/**
 * Exception, warning and info dialog utility functions.
 * 
 * @author Riccardo Pietrucci
 * @version 0.1
 */
public class ErrorHandler {

	/**
	 * The shared instance
	 */
	public final static ErrorHandler instance = new ErrorHandler();

	/**
	 * Shows the specified exception in a new dialog as a severe error
	 * @param e the exception to show as a severe error
	 */
	public void showError(Exception e) {
		e.printStackTrace();
		final String title = e.getClass().getSimpleName();
		final String message = e.getMessage() + "\n" +
		"Show stack trace?";
		// Custom button text
		JOptionPane.setRootFrame(ToolBoxFrame.getIstance());
		int value = JOptionPane.showConfirmDialog(ToolBoxFrame.getIstance(), 
				message, title,
				JOptionPane.YES_NO_OPTION,
				JOptionPane.ERROR_MESSAGE);
		if (value == JOptionPane.YES_OPTION) { // show stack trace
			JFrame areaFrame = new JFrame(title+" Stack Trace");
			StringWriter textOut = new StringWriter();
			PrintWriter out = new PrintWriter(textOut);
			e.printStackTrace(out);
			BufferedReader in = new BufferedReader(new StringReader(textOut.getBuffer().toString()));
			JTextArea textArea = new JTextArea();
			String line;
			try {
				while((line = in.readLine()) != null) {
					textArea.append(line + "\n");
				}
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			JScrollPane areaScrollPane = new JScrollPane(textArea);
			areaFrame.add(areaScrollPane);
			areaFrame.setMinimumSize(new Dimension(800,600));
			areaFrame.setVisible(true);
			areaFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			areaFrame.toFront();
			areaFrame.setAlwaysOnTop(true);
		}
		ToolBoxFrame.getIstance().repaint();
	}

	/**
	 * Shows the specified exception in a new dialog as a warning error
	 * @param e the exception to show as a warning error
	 */
	public void showWarning(Exception e) {
		final String title = e.getClass().getSimpleName();
		final String message = e.getMessage();
		// Custom button text
		JOptionPane.setRootFrame(ToolBoxFrame.getIstance());
		JOptionPane.showConfirmDialog(ToolBoxFrame.getIstance(), 
				message, title,
				JOptionPane.CLOSED_OPTION,
				JOptionPane.WARNING_MESSAGE);
		ToolBoxFrame.getIstance().repaint();
	}

	/**
	 * Shows the specified message
	 * @param message the message to show
	 */
	public void showMessage(String message) {
		final String title = "Message";
		// Custom button text
		JOptionPane.setRootFrame(ToolBoxFrame.getIstance());
		JOptionPane.showConfirmDialog(ToolBoxFrame.getIstance(), 
				message, title,
				JOptionPane.CLOSED_OPTION,
				JOptionPane.INFORMATION_MESSAGE);
		ToolBoxFrame.getIstance().repaint();
	}

}
