package org.modelgen.jpf.core;


import org.modelgen.jpf.core.actions.ManageOptionsAction;
import org.modelgen.jpf.core.actions.ReInitializeDataBaseAction;
import org.modelgen.jpf.core.actions.StartDbServerAction;
import org.modelgen.jpf.core.actions.StopDbServerAction;
import org.modelgen.jpf.core.actions.ToolActivetionAction;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyVetoException;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import javax.swing.plaf.basic.BasicBorders;

import org.java.plugin.PluginManager;
import org.java.plugin.registry.Extension;
import org.java.plugin.registry.ExtensionPoint;
import org.java.plugin.registry.Extension.Parameter;

/**
 * The main frame containing all plugins.
 * 
 */
public class ToolBoxFrame extends JFrame {
	private static final long serialVersionUID = 1L;

	/*Componenti della Frame*/
	private JMenuBar menuBar;
	private JMenu strumenti;
	private JMenu options;
	private JMenu help;
	private JToolBar toolBar;
	private JDesktopPane desktop;
	private JLabel statusBar;

	/*Parametri */
	private Map<String,Object> applObjects;

	private final CorePlugin corePlugin;

	/*Singleton*/
	private static ToolBoxFrame istance = null;

	/**
	 * @return the unique shared instance
	 */
	public synchronized static ToolBoxFrame getIstance(){
		/*Questo metodo deve essere eseguito solo dopo che l'oggetto  gi stato costruito*/
		return istance;
	}

	/*Publico (nonostante il pattern singleton)perch deve essere invocato nello scenario  
	di avvio per poi consentire l'uso del metodo getIstance */
	protected ToolBoxFrame(Map<String,Object> applicationObjects, CorePlugin corePlugin){
		super("ModelGen");
		//Mappa di informazioni condivise
		this.applObjects = applicationObjects;
		this.corePlugin = corePlugin;
		defineStructure();
		setComponetsProperties();
		setProperties();
		istance = this;
	}

	private void defineStructure() {
		/*Struttura*/
		menuBar = new JMenuBar();
		strumenti = new JMenu("Tools");	
		options = new JMenu("Options");
		loadOptionItems();
		help = new JMenu("Help");
		toolBar = new JToolBar("ToolBar");
		loadButtons();
		menuBar.add(strumenti);
		menuBar.add(options);
		menuBar.add(help);
		desktop = new JDesktopPane(); 

		statusBar = new JLabel("<html><b>---</b><html>"+"<html><font color=#2F72FB><b>ModelGen ToolBox Application</b></html>");


		setJMenuBar(menuBar);
		add(toolBar,BorderLayout.PAGE_START);
		add(desktop,BorderLayout.CENTER);
		add(statusBar,BorderLayout.SOUTH);
	}


	private void loadOptionItems() {
		options.add(StartDbServerAction.getIstance());
		options.add(StopDbServerAction.getIstance());
		options.addSeparator();
		options.add(ReInitializeDataBaseAction.getIstance());
		options.addSeparator();

		AbstractAction manageOptions = ManageOptionsAction.getIstance();
		JMenuItem preferences = new JMenuItem(manageOptions);
		preferences.setName(Action.NAME);
		options.add(preferences);	
	}
	@SuppressWarnings("unchecked")
	private void loadButtons() {
		toolBar.add(ManageOptionsAction.getIstance());
		toolBar.add(StartDbServerAction.getIstance());
		toolBar.add(StopDbServerAction.getIstance());
		toolBar.addSeparator(new Dimension(35,0));
		toolBar.add(ReInitializeDataBaseAction.getIstance());	

		toolBar.addSeparator(new Dimension(70,0));

		PluginManager pm = PluginManager.lookup(ToolBoxFrame.class); 
		ExtensionPoint toolExtPoint =  pm.getRegistry().getExtensionPoint(CorePlugin.PLUGIN_ID, "Tool");
		Collection extensions = toolExtPoint.getConnectedExtensions();
		if (extensions == null)
			return;

//		Vector<Extension> v = new Vector<Extension>(10);
//		Vector<Extension> vOther = new Vector<Extension>(); 
//		Iterator<Extension> itX = extensions.iterator();
//		extensions.
//		for (Object object : extensions) {
//		Extension ext = (Extension)object;
//		System.out.println("Extension: " + ext.getParameter("name").valueAsString() + "|" + v.capacity() + "|" + v.size());
//		if (ext.getParameter("name").valueAsString().equals("Define Model")) v.add(0, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Define Schema")) v.add(1, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Define Skolem")) v.add(2, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Apply Schema Rule")) v.add(3, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Apply Instance Rules")) v.add(4, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Import and Export")) v.add(5, ext);
//		else if (ext.getParameter("name").valueAsString().equals("Database Visualizer and Translation Debugger")) v.add(6, ext);
//		else vOther.add(ext);
//		}
//		System.out.println("addAll");
//		v.addAll(vOther);
		List<Extension> extensionsSorted = new LinkedList<Extension>();
		extensionsSorted.addAll(extensions);
		//ordina la visualizzazione dei bottoni dei plugin - FCelli
		Collections.sort(extensionsSorted, new PluginNameComparator());

		/* For each extension, build the corresponding menu items */
		for(Extension ext : extensionsSorted) {
			String name = ext.getParameter("name").valueAsString();
			String description = ext.getParameter("description").valueAsString();
			System.out.println("plugin description: " + description);


			Map<String,Object> appObjs = new HashMap<String,Object>();
			//Informazioni comuni
			appObjs.putAll(applObjects);
			//Inoformazioni per lo specifico plug-in
			appObjs.put("extension",ext);

			AbstractAction action = new ToolActivetionAction(appObjs);
			JButton toolButton = new JButton(action);
			toolButton.setName(name);
			toolButton.setToolTipText(description);
			ImageIcon icon = getImageIcon(ext, "icon");
			toolButton.setIcon(icon);

			JMenuItem toolItem = new JMenuItem(action);
			toolItem.setText(name);
			toolItem.setToolTipText(description);
			ImageIcon itemIcon = getImageIcon(ext, "itemIcon");
			toolItem.setIcon(itemIcon);	

			/*Info da passare al Tool*/
			toolBar.add(toolButton);	
			strumenti.add(toolItem);
		}
	}

	private ImageIcon getImageIcon(Extension ext, String parameterName) {
		Parameter iconParam = ext.getParameter(parameterName);
		URL iconUrl = null;
		if (iconParam != null) {
			iconUrl = this.corePlugin.getManager().getPluginClassLoader(
					ext.getDeclaringPluginDescriptor())
					.getResource(iconParam.valueAsString());
		}
		return (iconUrl != null) ? new ImageIcon(iconUrl) : null;
	}

	private void setComponetsProperties() {
		toolBar.setFloatable(false);
		toolBar.setBorder(BasicBorders.getMenuBarBorder());
		desktop.setBackground(Color.WHITE);
	}

	private void setProperties() {
		/*Dimensioni finestra*/
		Dimension dim = getToolkit().getScreenSize();
		Dimension frameDim = new Dimension((int)dim.getWidth(),(int)dim.getHeight()-30);
		this.setSize(frameDim);
		this.setMaximumSize(frameDim);

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setVisible(true);
	}

	/**
	 * Adds and shows the specified toolFrame
	 * @param toolFrame the internal frame to show
	 */
	public void addTool(JInternalFrame toolFrame){
		desktop.add(toolFrame);
		toolFrame.setVisible(true);
	}

	/**
	 * Removes and unshows the specified toolFrame
	 * @param toolFrame the internal frame to unshow
	 */
	public void removeTool(JInternalFrame toolFrame){
		desktop.remove(toolFrame);
		desktop.updateUI();
	}

	/**
	 * Closes all tools
	 */
	public void closeTools(){
		JInternalFrame[] frames = desktop.getAllFrames();
		int i = 0;
		while(i<frames.length){
			try {
				frames[i].setClosed(true);
			} catch (PropertyVetoException e) {
				e.printStackTrace();
			}
			i++;
		}
		desktop.updateUI();
	}

	/**
	 * Set the main frame status bar.
	 * 
	 * <p>Useful for plugins that have to show status message.
	 * 
	 * @param newStatus the status message
	 */
	public void setStatusBar(JLabel newStatus){
		this.statusBar = newStatus;
		statusBar.updateUI();
	}

}
