package org.modelgen.jpf.core.actions;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.facade.ResourceFunctions;
import org.modelgen.jpf.core.ToolBoxFrame;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

/**
 * The Manage Options Dialog (protected class)
 *
 * @see org.modelgen.jpf.core.actions.ManageOptionsAction
 */
class ManageOptionsDialog extends JDialog implements ActionListener{
	private static final long serialVersionUID = 1290621096967554004L;

	JTabbedPane tabs;
	JRadioButton mode;
	
	/**
	 * Creates a new Manage Options Dialog.
	 * 
	 * @see org.modelgen.jpf.core.actions.ManageOptionsAction#actionPerformed(ActionEvent)
	 */
	public ManageOptionsDialog(){
		super(ToolBoxFrame.getIstance(),"Options Manager",true);
		
		
		//Setting bounds...
		Dimension dim = this.getToolkit().getScreenSize();
		double screenWidth = dim.getWidth();
		double screenHeight = dim.getHeight();
		double width = screenWidth/3;
		double height = screenHeight/4;
		double thisX = (screenWidth-width)/2;
		double thisY = (screenHeight-height)/2;
		this.setBounds((int)thisX,(int)thisY,(int)width,(int)height);
		
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e){
				super.windowClosing(e);
				((JDialog)e.getSource()).dispose();
			}
		});
		
		defineStructure(this.getContentPane());
	}

	private void defineStructure(Container contentPane) {
		tabs = new JTabbedPane();
		tabs.setBorder(new EmptyBorder(5,5,5,3));
			addDataBaseTab();
			addToolTab();
		
		contentPane.add(tabs,BorderLayout.CENTER);
		addControls(contentPane);
	}



	private void addDataBaseTab() {
		JPanel panel = new JPanel();
		defineDataBaseTabStructure(panel);
		
		ImageIcon icon = ResourceFunctions.openImageIcon(this.getClass(), "data.png");
		tabs.addTab("DataBase",icon, panel);

	}
	
	private void addToolTab() {
		ImageIcon tools = ResourceFunctions.openImageIcon(this.getClass(), "toolsOption.png");
		tabs.addTab("Tools",tools, new JPanel());
	}
	
	private void addControls(Container contentPane) {
		JPanel controlPanel = new JPanel(new FlowLayout(FlowLayout.CENTER,20,5));
		JButton okButton = new JButton("OK");
			okButton.setActionCommand("ok");
			okButton.addActionListener(this);
		JButton exitButton = new JButton("EXIT");
			exitButton.setActionCommand("exit");
			exitButton.addActionListener(this);
		controlPanel.add(okButton);
		controlPanel.add(exitButton);
		contentPane.add(controlPanel,BorderLayout.PAGE_END);
	}

	private void defineDataBaseTabStructure(JPanel panel) {
		JPanel dbInitializationPanel = new JPanel();
		dbInitializationPanel.setBorder(BorderFactory.createTitledBorder(new EmptyBorder(5,5,5,5),"Server Configuration"));
		
	
		mode = new JRadioButton();
		String manual = ModelgenConfigurationManager.getIstance().getProperty("manual_server","","options");
		if(manual.equals("true")){
			mode.setSelected(true);
		}
		
		JLabel label = new JLabel("<html><p>This option enables automatic start/shutdown server actions. If you want to </p>"+
                                        "<p>manually manage the database, you should not select this option. </p>"+
                                        "<p></p>"+"</html>");
			
		dbInitializationPanel.add(mode);
		dbInitializationPanel.add(label);
		
		panel.add(dbInitializationPanel);
		
		
	}

	/**
	 * Apply or cancel setted options, saving the modelgen configuration file
	 */
	public void actionPerformed(ActionEvent arg0) {
		JButton actionButton = (JButton) arg0.getSource();
		String actionCommand = actionButton.getActionCommand();
		if(actionCommand.equals("ok"))
			commit();
		this.setVisible(false);
	}


	private void commit() {
		if (mode.isSelected()){
			ModelgenConfigurationManager.getIstance().setProperty("manual_server","true","options");
		}else{
			ModelgenConfigurationManager.getIstance().setProperty("manual_server","false","options");
		}
		ModelgenConfigurationManager.getIstance().saveConfiguration();
	}
	
}
