package org.modelgen.jpf.core.actions;

import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.core.commons.Tool;

import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;

import org.java.plugin.PluginManager;
import org.java.plugin.registry.Extension;

/**
 * Tool Activation Action. It creates an action 
 * that activates the plugin specified in
 * {@link #ToolActivetionAction(Map)} and shows the corresponding
 * frame specified in {@link #setTool(Tool)}. 
 */
public class ToolActivetionAction extends AbstractAction{
	private static final long serialVersionUID = 2785901827864043218L;

	/*Parametro inserito come ClientPropery dalla ToolBoxFrame nel bottone associato all'azione*/
	private Map<String,Object> appObjs;
	
	/*Estensione associata all'azione*/
	private Extension ext;
	
	/*Il tool associato all'azione (e al pulsante) istanziata sulla base del parametro "ext"*/
	private Tool tool = null;
	
	/**
	 * Creates the action corresponding to the plugin's specified.
	 * 
	 * @param applicationObjects the plugin's info: it uses the
	 * key "extension" to obtain the corresponding 
	 * {@link org.java.plugin.registry.Extension} object.
	 */
	public ToolActivetionAction(Map<String,Object> applicationObjects){
		super();
		this.appObjs = applicationObjects;
		this.ext = (Extension) appObjs.get("extension");
	}

	/**
	 * Action performed: it activates the plugin specified in
	 * {@link #ToolActivetionAction(Map)} and shows the corresponding
	 * frame specified in {@link #setTool(Tool)}. 
	 * 
	 * <p>On frame closing, the tool is forced to be deactivated through
	 * {@link ToolAdapter#internalFrameClosing(javax.swing.event.InternalFrameEvent)}.
	 * 
	 * @see ToolAdapter
	 */
	public synchronized void actionPerformed(ActionEvent evt) {
		setEnabled(false);
			if (tool == null){
				tool = activateTool();
			}else{
				/*Se il tool  gi stato caricato premere il pulsante posiziona il tool in primo piano*/
				moveFrameToFront((JInternalFrame)tool);	
			}	
		setEnabled(true);
	}
	
  
	/*
     * This method activates (initializes Tool) if it wasn't activated yet.
     */
	private Tool activateTool() {
		try {			
			PluginManager manager = PluginManager.lookup(ToolActivetionAction.class);
        	manager.activatePlugin(ext.getDeclaringPluginDescriptor().getId());
        		        			
            // Get plug-in class loader.
            ClassLoader classLoader = manager.getPluginClassLoader(ext.getDeclaringPluginDescriptor());
            // Load Tool class.
            Class<?> toolCls = classLoader.loadClass(ext.getParameter("class").valueAsString());
            // Create Tool instance.
            tool = (Tool) toolCls.newInstance();
            // Initialize class instance according to interface contract.
            tool.init(appObjs);
            
    	    if (tool!=null){
    	    	JInternalFrame toolFrame = (JInternalFrame)tool;
    	    		toolFrame.addInternalFrameListener(new ToolAdapter(this,tool));
    	    	ToolBoxFrame toolBox = ToolBoxFrame.getIstance();
    	    	toolBox.addTool(toolFrame);   
    	    	moveFrameToFront(toolFrame);
    	    	System.out.println("*** Il tool "+ ext.getParameter("name") +" e' stato correttamente attivato ***");
    	    }
		}catch(Exception e){
					JOptionPane.showMessageDialog(ToolBoxFrame.getIstance(),"Error loading tool","Error",JOptionPane.WARNING_MESSAGE);
					System.err.println("----------------Error Loading Tool--------------------- \n");
					System.err.println("Causa: " + e.getCause());
					System.err.println("Message: " + e.getMessage());
					System.err.println("Trace: ");
					e.printStackTrace();
					System.err.println("------------------------------------------------------- ");
		}
	    return tool;
	}
	 
	private void moveFrameToFront(JInternalFrame frame){
		frame.moveToFront();
		try {
			frame.setSelected(true);
		} catch (PropertyVetoException e) {
			//ignore
		}
	}

	/**
	 * Sets the plugin's internal frame 
	 * @param tool the plugin's internal frame 
	 */
	public void setTool(Tool tool) {
		this.tool = tool;
	}

}