package org.modelgen.jpf.core.actions;

import org.modelgen.jpf.core.ToolBoxFrame;
import org.modelgen.jpf.core.commons.Tool;

import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import org.java.plugin.PluginManager;
import org.java.plugin.registry.Extension;

/**
 * Forces every plugin's internal frame to deactivate them in
 * a plugin's closing window action.
 * 
 * <p>Used internally in 
 * {@link ToolActivetionAction#actionPerformed(java.awt.event.ActionEvent)}:
 * every plugin's closing window action will be redirected to this class.
 * 
 * @author Riccardo Pietrucci
 *
 */
public class ToolAdapter extends InternalFrameAdapter {
	private ToolActivetionAction activetion; 
	private Tool tool;
	
	/**
	 * Creates a new tool adapter
	 * @param activetion the plugin's activation action
	 * @param tool the plugin's internal frame
	 */
	public ToolAdapter(ToolActivetionAction activetion, Tool tool){
		this.activetion = activetion;
		this.tool = tool;
	}
	
	/**
	 * Invoked when the plugin's internal frame 
	 * is in the process of being closed: 
	 * forces every plugin's internal frame to deactivate them.
	 */
	public void internalFrameClosing(InternalFrameEvent e){
		super.internalFrameClosing(e);
		
		//Dopo la chiusura se il ToolActivetionAction.tool == null  possibile reistanziare il tool
		activetion.setTool(null);
		
		//Disattiva il tool
		Extension ext = (Extension)tool.getInfo().get("extension");
		PluginManager pm = PluginManager.lookup(ToolAdapter.class);
		pm.deactivatePlugin(ext.getDeclaringPluginDescriptor().getId());
			System.out.println("*** Il tool "+ext.getParameter("name")+" e' stato correttamente disattivato ***");
		
			
		
		JInternalFrame toolFrame = (JInternalFrame)tool;
//      	EliminiNA il Frame dal JDesktopPane
			ToolBoxFrame.getIstance().removeTool(toolFrame);
			
		System.gc();
	}

}
