/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.Exception.FunctorVariableNotFoundException;
import org.modelgen.regole.Exception.InstanceFromComposedArgumentException;
import org.modelgen.regole.Exception.VarNotFoundException;

import java.util.Map;
import java.util.Set;


/**
 * Rappresenta il tipo di un oggetto che puo'comparire come secondo
 * argomento di un ArgomentoEtichettato
 * @author luigi, francesca
 * 
 *
 */
public interface Argomento {
	
	/**
	 * Genera le condizioni per l'argomento nelal traduzione da Datalog a SQLA
	 * @param var la variabile associata al letterale in cui compare l'argomento
	 * @param tabelle la mappa delle corrispondenze Letterale variabile dei letterali della Regola
	 * @param attributo associato all'argomento
	 * @return insieme delle condizioni associato all'argomento
	 */
	public Set<? extends Condizione> generaCondizioni(String var, Map<String, Tabella> tabelle, String attributo) throws FunctorVariableNotFoundException;
	
	/**
	 * 
	 * @return il nome dell'argomento
	 */
	public String getNome();
	
	/**
	 * Genera la clausola della parte select per un argomento quando esso compare nella testa della regola Datalog.
	 * @param tabelle la mappa delle corrispondenze Letterale variabile dei letterali della regola
	 * @return una clasuola per la parte select
	 */
	public CondizioneSelect getSelect(Map<String, Tabella> tabelle) throws VarNotFoundException, FunctorVariableNotFoundException;
	
	/**
	 * Genera le condizioni where per condizione not exists dei letterali di testa
	 * privi di funtori di Skolem
	 * @param var tipicamente "varNotExist", la variabile associata al letterale all'interno della condizione Not Exists  
	 * @param tabelle la mappa delle corrispondenze Letterale variabile dei letterali della regola
	 * @param attributo l'attributo associato all'argomento
	 * @return la condizione associata
	 */
	public Condizione getTMP(String var, Map<String, Tabella> tabelle, String attributo) throws VarNotFoundException;
	
	/**
	 * Genera il corrispondente argomento per istanza
	 * @param nuova la regola Datalog per istanza in creazione
	 * @param newLetterale il letterale per istanza in creazione
	 * @return l'argomento per istanza corrispondente a questo
	 */
	public Argomento getArgomentoPerIstanza(RegolaDatalog nuova, Letterale newLetterale) throws InstanceFromComposedArgumentException;
	
	/**
	 * Crea una copia dell'argomento nel contesto di una regola
	 * @param nuova la regola Datalog
	 * @param newLetterale il letterale cui appartiene
	 * @param isSpecialContext se la duplicazione avviene in un contesto particolare e richiede opzioni aggiuntive. Ad esempio 
	 * la duplicazione di un funtore pu� avvenire generando o meno un funtore target. 
	 * @return il nuovo argomento
	 */
	public Argomento duplicate(RegolaDatalog nuova, Letterale newLetterale, boolean isSpecialContext);
	
	/**
	 * imposta il letterale di appartenenza
	 * @param let letterale di appartenenza
	 */
	public void setLetterale(Letterale let);
	
	/**
	 * Genera il valore associato all'agomento: metodo utile per evitare di testare il tipo
	 * degli oggetti argomento nella costruzione della segnatura di una regola
	 * @return il valore associato all'argomento
	 */
	public String getValore();
}
