/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.commons.beans.model.Construct;
import org.modelgen.regole.Exception.FunctorVariableNotFoundException;
import org.modelgen.regole.Exception.IncompleteQualifiedAttributeException;
import org.modelgen.regole.Exception.InstanceFromComposedArgumentException;
import org.modelgen.regole.Exception.InstanceTranslationException;
import org.modelgen.regole.Exception.VarNotFoundException;

import java.util.Map;
import java.util.Set;


/**
 * Rappresenta una coppia attributo argomento presente nel Letterale
 * @author luigi, francesca
 * 
 */
public class ArgomentoEtichettato {
	
	public final static String INSTANCE_PREFIX = "i_";
	private String attributo;
	private Argomento argomento;
	
	public ArgomentoEtichettato(String attributo, Argomento argomento) {
		this.attributo = attributo;
		this.argomento = argomento;
	}
	
	public void setAttributo(String a) {
		this.attributo = a;
	}
	
	public String getAttributo() {
		return this.attributo;
		
	}
	
	public void setArgomento(Argomento a) {
		this.argomento = a;
	}
	
	public Argomento getArgomento() {
		return this.argomento;
	}
	
	/**
	 * Genera le condizioni per la parte where della regola associate a questo ArgomentoEtichettato
	 * @param v la var associata al letterale relativo a questo argomentoEtichettato
	 * @param tabelle la mappa delle corrispondenze Letterale variabile 
	 * @return l'insieme di condizioni generate dall'ArgomentoEtichettato
	 */
	public Set<? extends Condizione> generaCondizioni(String v, Map<String, Tabella> tabelle) throws IncompleteQualifiedAttributeException {
		/* le condizioni per un argomento Etichettato si differenziano a seconda dell'argomento quindi esse saranno generate dall'Argomento */
		try {
			return argomento.generaCondizioni(v, tabelle, attributo);
		} catch(FunctorVariableNotFoundException ex) {
			throw new IncompleteQualifiedAttributeException(attributo,ex);
		}
	}
	
	public CondizioneSelect getSelect(Map<String, Tabella> tabelle) throws IncompleteQualifiedAttributeException {
		/* le condizioni si differenzieranno a seconda dell'argomento quindi la generazione verr affidata all'Argomento */
		try {
			return argomento.getSelect(tabelle);
		} catch(VarNotFoundException ex) {
			throw new IncompleteQualifiedAttributeException(attributo,ex);
		}
	}
	
	public Condizione getTMP(Map<String, Tabella> tabelle) throws IncompleteQualifiedAttributeException {
		/* le condizioni si differenzieranno a seconda dell'argomento quindi la generazione verr affidata all'Argomento */
		try {
			return argomento.getTMP("varNotExist",tabelle,attributo);
		} catch(VarNotFoundException ex) {
			throw new IncompleteQualifiedAttributeException(attributo,ex);
		}
	}
	
	/**
	 * Restituisce il corrispondente argomento etichettato per istanza dati la regola Datalog per istanza 
	 * in creazione e il letterale in creazione a cui apparterr
	 * @param nuova la regola Datalog per istanza in creazione
	 * @param newLetterale il letterale per istanza in creazione
	 * @param costrutto il costrutto corrispondente al letterale origine
	 * @return l'argomento etichettato per istanza corrispondente a questo argomento, null se  una propriet
	 */
	public ArgomentoEtichettato getArgomentoEtichettatoPerIstanza(RegolaDatalog nuova, Letterale newLetterale, Construct costrutto) throws InstanceTranslationException {
		
		ArgomentoEtichettato newArgomentoEtichettato = null;
		String attributo = getAttributo();
		
		/* lo traduce per istanza se: non  una propriet e non  "name" oppure se  OID oppure costrutto nullo (TMP). Altrimenti */
		/* ritorna null */
		if(costrutto==null || (!costrutto.hasProperty(attributo) && !attributo.toLowerCase().equals("name")) || attributo.equalsIgnoreCase("OID")) {
			/* genera il relativo argomento per istanza */
			Argomento newArgomento = null;
			try {
				newArgomento = this.getArgomento().getArgomentoPerIstanza(nuova,newLetterale);
			} catch(InstanceFromComposedArgumentException ex) {
				throw new InstanceTranslationException(ex);
			}
			
			/* se  OID non si aggiunge la i- */
			newArgomentoEtichettato = new ArgomentoEtichettato(attributo.equalsIgnoreCase("OID") ? "OID" : INSTANCE_PREFIX+attributo,newArgomento);	
		}
		return newArgomentoEtichettato;
	}
	
	/**
	 * Crea una copia di questo argomento etichettato
	 * @param nuova la regola Datalog in creazione
	 * @return la copia dell'argomento etichettato
	 */
	public ArgomentoEtichettato duplicate(RegolaDatalog nuova, Letterale newLetterale) {
		/* l'argomento non  duplicato in un contesto speciale */
		return new ArgomentoEtichettato(this.attributo,this.getArgomento().duplicate(nuova,newLetterale, false));
	}
	
	public String toString() {
		return attributo + ": " + argomento.toString();
	}
}
