/*
 * Created on 20-mag-2005
 *
 */
package org.modelgen.regole;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * 
 * Insieme particolare che mantiene le condizioni
 * evitando condizioni duplicate e condizioni che
 * sono deducibili per transitivit da altre
 * preesistenti 
 * @author luigi, francesca
 *
 */
public class ConditionSet extends HashSet<Condizione> {
	private static final long serialVersionUID = 1L;

	public ConditionSet() {
		super();
				
	}
	
	/**
	 * 
	 * @return l'HashSet di tutte le condizioni
	 */
	public Set<Condizione> getCondizioni() {
		Set<Condizione> condizioni = new HashSet<Condizione>();
		Iterator<Condizione> it = super.iterator();
		while(it.hasNext()) {
			condizioni.add(it.next());
		}
		return condizioni;
	}
	
	public boolean add(Condizione c) {		
		
		/* controllo per le condizioni fittizie es: var1.a = var1.a*/
		if(c.getOperando1().equals(c.getOperando2()) && c.getOperatore().equals("="))
			return false;
		
		/* controllo per le condizioni vuote "" "" "" */
		if(c.getOperando1().equals(c.getOperando2()) && c.getOperatore().equals("") && c.getOperando1().equals(""))
			return false;
		
		boolean ok = false;
		if(!generaInferenze().contains(c))
				ok = super.add(c);
		
		return ok;
	}
	
	public boolean addAll(Collection<? extends Condizione> c) {
		boolean inserito = false;
		boolean modificato = false;		
		for (Condizione condizione : c) {
			inserito = add(condizione);
			modificato = inserito || modificato;
		}
		
		return modificato;
		
	}
	
	public Iterator<Condizione> iterator() {
		return super.iterator();
	}
	
	
	private boolean isUguaglianza(Condizione c) {
		/* condizione di uguaglianza e non sugli schemeOID */
		return c.getOperatore().equals("=") && !c.getOperando2().startsWith("?");
	}
	
	/**
	 * Restituisce l'insieme di tutte le condizioni semplici deducibili
	 * per inferenza per transitivit da quelle esistenti
	 * @return un insieme di condizioni semplici
	 */
	private Set<CondizioneSemplice> generaInferenze() {
		Iterator it = iterator();
		Iterator it1;
		String op1 = null;
		String op2 = null;
		String opA = null;
		String opB = null;
		Condizione c=null;
		Condizione c1 = null;
		Set<CondizioneSemplice> inferenze = new HashSet<CondizioneSemplice>();
		/* A = B e B = C -> A = C */
		while(it.hasNext()) {
			c = (Condizione)it.next();
			if(isUguaglianza(c)) {
				op2 = c.getOperando2();
				it1 = iterator();
				while(it1.hasNext()) {
					c1 = (Condizione)it1.next();
					if(isUguaglianza(c1)) {
						op1 = c1.getOperando1();
						if(op2.equals(op1)) {
							opA = c.getOperando1();
							opB = c1.getOperando2();
							if(!(opA.equals(opB)))
								inferenze.add(new CondizioneSemplice(opA,opB,"="));							
						}
					}
							
				}
				
			}
			
		}
		it = iterator();
		
		/* A = B e A = C -> B = C */
		String op11 = null;
		while(it.hasNext()) {
			c = (Condizione)it.next();
			if(isUguaglianza(c)) {
				op1 = c.getOperando1();
				it1 = iterator();
				while(it1.hasNext()) {
					c1 = (Condizione)it1.next();
					if(isUguaglianza(c1)) {
						op11 = c1.getOperando1();
						if(op1.equals(op11)) {
							opA = c.getOperando2();
							opB = c1.getOperando2();
							if(!(opA.equals(opB))) 
								inferenze.add(new CondizioneSemplice(opA,opB,"="));
						}
						
					}
				}
			}
		}
		
		it = iterator();
		/* B = A e C = A -> C = B */
		String op22 = null;
		while(it.hasNext()) {
			c = (Condizione)it.next();
			if(isUguaglianza(c)) {
				op2 = c.getOperando2();
				it1 = iterator();
				while(it1.hasNext()) {
					c1 = (Condizione)it1.next();
					if(isUguaglianza(c1)) {
						op22 = c1.getOperando2();
						if(op2.equals(op22)) {
							opA = c.getOperando1();
							opB = c1.getOperando1();
							if(!(opA.equals(opB))) 
								inferenze.add(new CondizioneSemplice(opA,opB,"="));
						}
						
					}
				}
			}
		}
		
		
		return inferenze;		
		
	}

}
