/*
 * Created on 19-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.Exception.TooManyFunctorArgumentsException;
import org.modelgen.regole.utilities.CollectionFormatter;
import org.modelgen.regole.utilities.VarGenerator;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * Rappresenta una clausola SELECT del tipo: funtore(varX.colonna, varY.colonna ..)
 * @author luigi, francesca
 * 
 */
public class CondizioneSelectSkolem implements CondizioneSelect {
	private List<String> argomenti;           // di tipo String
	private SkolemFunctor funtore;    // il funtore di Skolem associato
	
	public CondizioneSelectSkolem(SkolemFunctor funtore){
		this.funtore = funtore;
		argomenti = new LinkedList<String>();		
	}
	
	public void setFuntore(SkolemFunctor f){
		funtore = f;
	}
	
	public SkolemFunctor getFuntore() {
		return this.funtore;
	}
	
	public void setArgomenti(List<String> argomenti) {
		this.argomenti = argomenti;
	}
	
	public CondizioneSelect getTradotto(Map<String,Tabella> tabelle, Map<String,Tabella> tabelle1, Set<Condizione> where,Set<Condizione> where1, VarGenerator vg, List<Regola> regole, CondizioneNotExist cne, String colonnaRiferimento) throws TooManyFunctorArgumentsException {
		String var = vg.getNewVar();
		String targetTable = funtore.getTargetTableSK();
		Tabella t = new Tabella(targetTable, var);
		tabelle1.put(var,t);
		List<String> parametriFormali = new LinkedList<String>(funtore.getArgomentiFunzione());	// altrimenti effetto collaterale sulla tabella di Skolem
		/* condizione sullo schema o sull'istanza a seconda del tipo di funtore a cui la */
		/* condizione si riferisce */
		if(!funtore.isInstance())
			where1.add(new CondizioneSemplice(var + ".schemeOID",funtore.getLetterale().traduciRuolo(),"="));
		else
			where1.add(new CondizioneSemplice(var + ".instanceOID",funtore.getLetterale().traduciRuoloIstanza(),"="));
		
		Iterator<String> it = parametriFormali.iterator();
		Iterator<String> it1 = argomenti.iterator();
		String parametroAttuale = null;
		String parametroFormale = null;
		Condizione c = null;
		while(it1.hasNext()){
			try {
				parametroFormale = it.next();
				parametroAttuale = it1.next();
			} catch(NoSuchElementException ex) { // numero di parametri in eccesso rispetto alla funzione 
				throw new TooManyFunctorArgumentsException(funtore.getNome());
			}
			
			c = new CondizioneSemplice(var + "." + parametroFormale, parametroAttuale, "=");
			where1.add(c);
		}
		
		/* se il funtore � target si deve generare una nuova regola insert skolem */
		if(funtore.getIsTargetSkolem())	
			regole.add(generaRegolaSkolem(tabelle, where, where1, var, cne, colonnaRiferimento, parametriFormali));	
		
		CondizioneSelect cs = new CondizioneSelectSemplice(var + "."+funtore.getTargetOID());
		return cs;		
	}
	
	/**
	 * Genera la regola insert Skolem associata al funtore target
	 * @param tabelle la mappa delle corrispondenze letterale variabile
	 * @param where l'insieme delle condizioni where della regola
	 * @param where1 l'accumulatore delle condizioni WHERE
	 * @param var la newVar associata alla regola 
	 * @param cne la condizione NotExist su tutti i funtori target
	 * @param colonnaRiferimento la colonna della parte insert a cui fa riferimento il target oid
	 * @param parametriFormali la lista dei parametri formali della funzione
	 * @return la regola insert Skolem
	 */
	private Regola generaRegolaSkolem(Map<String,Tabella> tabelle, Set<Condizione> where, Set<Condizione> where1, String var, CondizioneNotExist cne, String colonnaRiferimento, List<String> parametriFormali) {
			
		InsertPart ip1 = new InsertPart(funtore.getTargetTableSK());
		ip1.setColonne(parametriFormali);
		
		/* aggiunta della colonna relativa allo schema o all'istanza */
		/* a seconda del tipo di funtore */
		if(!funtore.isInstance())
			ip1.addColonna("schemeOID");
		else
			ip1.addColonna("instanceOID");
		
		Regola regolaSK = new Regola(where,tabelle,ip1);
		if(onlyConstPars())
			regolaSK.setOnlyConstPars(true);
			
		regolaSK.setSelectPart(generaClausoleSelect());		
		
		CondizioneNotExist cne1 = generaNotExist(where1, var, cne, colonnaRiferimento, parametriFormali);			
		regolaSK.addCondizioneWhere(cne1);
			
		return regolaSK;
		
	}
	
	public boolean onlyConstPars() {
		return !this.funtore.haAlmenoUnaVariabile();
	}

	/**
	 * Genera la condizioneNotExist per la regolaSK, aggiunge le condizioni
	 * corrispondenti a questa nell'insieme accumulatore della regola generale e
	 * aggiorna la NotExist sui funtori target.
	 * @param where1 l'insieme accumulatore
	 * @param var la newVar associata alla tabella
	 * @param cne la condizione NotExist associata ai funtori target
	 * @param colonnaRiferimento la colonna della parte insert a cui fa riferimento il target oid
	 * @param parametriFormali la lista dei parametri formali della funzione
	 * @return la condizione NotExist da aggiungere alla regola 
	 */
	private CondizioneNotExist generaNotExist(Set<Condizione> where1, String var, CondizioneNotExist cne, String colonnaRiferimento, List<String> parametriFormali) {
		Iterator<String> it;
		Iterator<String> it1;
		String parametroAttuale;
		String parametroFormale;
		CondizioneNotExist cne1 = new CondizioneNotExist(funtore.getTargetTable());
		it = parametriFormali.iterator();
		it1 = argomenti.iterator();
		CondizioneSemplice cs = null;
		while(it1.hasNext()){
			parametroAttuale = it1.next();
			parametroFormale = it.next();
			cs = new CondizioneSemplice("varNotExist." + parametroFormale, parametroAttuale, "=");
			cne1.addCondizione(cs);
			cs = new CondizioneSemplice(var + "." + parametroFormale, parametroAttuale, "=");
			where1.add(cs);
		}
		/* condizione not exist sullo schema o sull'istanza a seconda del tipo di funtore */
		if(!funtore.isInstance())
			cne1.addCondizione(new CondizioneSemplice("varNotExist" + ".schemeOID", funtore.getLetterale().traduciRuolo(), "="));
		else
			cne1.addCondizione(new CondizioneSemplice("varNotExist" + ".instanceOID", funtore.getLetterale().traduciRuoloIstanza(),"="));
		/* se questa � la condizione generata dal funtore target */
		/* relativo alla colonna OID della parte insert si deve */
		/* inserire l'opportuna condizione nella NotExist generale */
		if(colonnaRiferimento.equals("OID"))
			cne.addCondizione(new CondizioneSemplice("varNotExist." + colonnaRiferimento,var+"."+funtore.getTargetOID(),"="));
		return cne1;
	}

	/**
	 * @return la lista di clausole Select associata alla insertSkolem 
	 */
	private List<CondizioneSelect> generaClausoleSelect() {
		List<CondizioneSelect> clausole = new LinkedList<CondizioneSelect>();	
		
		Iterator<String> it2 = argomenti.iterator();
		
		while(it2.hasNext()){
			clausole.add(new CondizioneSelectSemplice(it2.next()));
		}
		
		
		
		/* aggiunta della condizione select relativa all'istanza o allo schema */
		/* a seconda del tipo del funtore */
		if(!funtore.isInstance()) {
			if(!this.onlyConstPars())
				clausole.add(new CondizioneSelectSemplice(funtore.getLetterale().traduciRuolo() + " as schemeOID"));
			else
				clausole.add(new CondizioneSelectSemplice(funtore.getLetterale().traduciRuolo()));
			
		} else {
			if(!this.onlyConstPars())
				clausole.add(new CondizioneSelectSemplice(funtore.getLetterale().traduciRuoloIstanza() + " as instanceOID"));
			else
				clausole.add(new CondizioneSelectSemplice(funtore.getLetterale().traduciRuoloIstanza()));
		}
		
		return clausole;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(CollectionFormatter.getInstance().mettiTraSeparatore(funtore.getNome())+ "(");
		List newArgomenti = CollectionFormatter.getInstance().mettiElementiListaSeparatoreDopoPunto(argomenti);
		buf.append(CollectionFormatter.getInstance().getSequenzaComponenti(newArgomenti));
		buf.append(")");
		return buf.toString();
	}

}
