/*
 * Created on 19-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.utilities.CollectionFormatter;
import org.modelgen.regole.utilities.VarGenerator;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Rappresenta una condizione in cui compare un funtore di Skolem
 * @author luigi, francesca
 * 
 */
public class CondizioneSkolem implements Condizione {
	
	private List<String> argomenti;        // gli argomenti del funtore di Skolem di tipo String
	private String varOID;         // varOID.attributo = funtore
	private String attributo;      // come sopra
	private SkolemFunctor funtore; // il funtore di skolem associato
	private final int PRIORITY = 2;
	
	public CondizioneSkolem(String attributo, String varOID, SkolemFunctor funtore) {
		this.attributo = attributo;
		this.varOID = varOID;
		this.funtore = funtore;
		this.argomenti = new LinkedList<String>();
	}
	
	public void setVarOID(String v) {
		this.varOID = v;
	}
	
	public String getVarOID() {
		return this.varOID;
	}
	
	public void setAttributo(String a) {
		this.attributo = a;
	}
	
	public String getAttributo() {
		return this.attributo;
	}
	
	public void setFuntore(SkolemFunctor f) {
		this.funtore = f;
	}
	
	public SkolemFunctor getFuntore() {
		return this.funtore;
	}
	
	
	public Set<Condizione> generaCondizioniB(Map<String,Tabella> tabelle, VarGenerator vg){
		
		String targetTable = funtore.getTargetTable();
		String var = vg.getNewVar();
		Tabella t = new Tabella(targetTable, var);
		tabelle.put(var, t);
		Set<Condizione> condizioni = new ConditionSet();
		condizioni.addAll(getRisolto(var));
		String oid = funtore.getTargetAttribute();
		CondizioneSemplice cs = new CondizioneSemplice(var + "." + oid, varOID + "." + attributo, "=");
		
		CondizioneSemplice cs1 = null;
		/* condizione sullo schema o sull'istanza a seconda se il funtore � di istanza o di schema */
		if(!funtore.isInstance())
			cs1 = new CondizioneSemplice(var + "." + "schemeOID", "?1", "=");
		else
			cs1 = new CondizioneSemplice(var + "." + "instanceOID", "?3", "=");
		
		condizioni.add(cs1);
		condizioni.add(cs);
		return condizioni;		
	}
	
	/**
	 * 
	 * @param newVar la variabile associata alla tabella del funtore
	 * @return l'insieme delle condizioni generate dagli argomenti del funtore
	 */
	public Set<Condizione> getRisolto(String newVar){
		String parametroAttuale = null;
		String parametroFormale = null;
		List<String> parametriFormali = funtore.getArgomentiFunzione();
		Set<Condizione> s = new ConditionSet();
		Iterator<String> it = parametriFormali.iterator();
		Iterator<String> it1 = argomenti.iterator();
		while(it1.hasNext()){
			parametroFormale = it.next();
			parametroAttuale = it1.next();
			s.add(new CondizioneSemplice(newVar + "." + parametroFormale, parametroAttuale, "="));
		}
		return s;			
	}
	
	public int getPriority() {
		return this.PRIORITY;
	}
	
	public void addArgomento(String a) {
		argomenti.add(a);
	}
	
	public String getOperando1() {
		return "";
	}
	
	public String getOperando2(){
		return "";
	}
	
	public String getOperatore() {
		return "skolem";
	}
	
	
	public int compareTo(Condizione c) {
		return this.PRIORITY - c.getPriority(); // secondo le convenzioni di compareTo
	}
	
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(varOID + "." + CollectionFormatter.getInstance().mettiTraSeparatore(attributo) + "=");
		buf.append(CollectionFormatter.getInstance().mettiTraSeparatore(funtore.getNome()) + "(");
		List newArgomenti = CollectionFormatter.getInstance().mettiElementiListaSeparatoreDopoPunto(argomenti);
		buf.append(CollectionFormatter.getInstance().getSequenzaComponenti(newArgomenti));
		buf.append(")");
		return buf.toString();
	}
	
	

}
