/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;



/**
 * Rappresenta una costante presente in una RegolaDatalog
 * @author luigi, francesca
 * 
 */
public class CostanteDatalog implements Argomento, Manipolabile {
	
	private String valore;
	
	public CostanteDatalog(String valore) {
		/* La stringa ottenuta dal parser ha la forma 'stringa'
		/* oppure sar� true o false e in tal caso il valore coincider� */
		if(valore.startsWith("'"))
			this.valore = valore.substring(1,valore.length()-1); // toglie gli apici
		else
			this.valore = valore;
	}
	
	public void setLetterale(Letterale let) {
		
	}
	
	public void setNome(String valore) {
		this.valore = valore;
	}
	
	public String getNome() {
		return this.valore;
	}
	
	public Set<Condizione> generaCondizioni(String var, Map<String, Tabella> tabelle, String attributo){
		String val;
		if(valore.toLowerCase().equals("true") || valore.toLowerCase().equals("false"))
			val = valore;
		else val = "'" + valore + "'";
		/* ad una costante in un letterale � associata una condizione semplice che imponga l'uguagliazna con l'attributo */
		CondizioneSemplice cs = new CondizioneSemplice(var + "." + attributo, val, "=");
		Set<Condizione> insieme = new HashSet<Condizione>(); // di tipo Condizione
		insieme.add(cs);
		return insieme;
	}
	
	public String generaSemicondizione(Map<String, Tabella> tabelle) {
		/* la semicondizione associata con una costante sar� coincidente con la stringa rappresentante il nome della costante
		 * racchiuso o no tra apici a seconda che sia semplice o rappresentata da true o false
		 */
		if(valore.toLowerCase().equals("true") || valore.toLowerCase().equals("false"))
			return valore;
		else return "'" + valore + "'";
	}
	
	public CondizioneSelect getSelect(Map<String, Tabella> tabelle) {
		/*
		 * Una costante genera una CondizioneSelectSemplice contenente il nome della costante 
		 */
		if(valore.toLowerCase().equals("true") || valore.toLowerCase().equals("false"))
			return new CondizioneSelectSemplice(valore);
		else	
			return new CondizioneSelectSemplice("'"+valore+"'");
	}
	
	/**
	 * Una costante non contribuisce nella creazione della not exists
	 * nel caso in cui la testa non contenga funtori di Skolem
	 */
	public Condizione getTMP(String var, Map<String, Tabella> tabelle, String attributo) {
		return new CondizioneSemplice("", "", "");
	}
	
	
	
	
	public Argomento getArgomentoPerIstanza(RegolaDatalog nuova, Letterale newLetterale) {
		/* una costante per istanza � uguale a se stessa */
		/* indipendentemente dalla regola Datalog e dal letterale */
		return duplicate(nuova,newLetterale, false);
	}
	
	public Manipolabile getManipolabilePerIstanza(RegolaDatalog nuova, Letterale newLetterale, boolean isInSkolem) {
		/* una costante per istanza � uguale a se stessa */
		/* indipendentemente dalla regola Datalog e dal letterale e dal fatto di essere in un funtore */
		return (Manipolabile)duplicate(nuova,newLetterale, false); // sia come Manipolabile che come argomento � sempre costante
	}
	
	public Manipolabile getCopia(RegolaDatalog nuova, Letterale newLetterale,boolean isInSkolem) {
		return (Manipolabile)duplicate(nuova,newLetterale, false);
	}
	
	public Argomento duplicate(RegolaDatalog nuova, Letterale newLetterale, boolean isSpecialContext) {
		return new CostanteDatalog(this.getNome());
	}
	
	
	
	
	
	
	
	
	public String toString() {
		if(valore.toLowerCase().equals("true") || valore.toLowerCase().equals("false"))
			return "\""+valore+"\"";
		else return "\"'"+valore+"'\"";
	}
	
	/**
	 * Ritorna il valore di una costante, cio true o false. Ritorna la stringa vuota nel
	 * caso in cui la costante non sia di tipo booleano
	 */
	public String getValore() {
		String val;
		if(valore.toLowerCase().equals("true") || valore.toLowerCase().equals("false"))
			val = valore;
		else val = "";
		return val;
	}
	

}
