/*
 * Created on 19-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.Exception.IncompleteLiteralConditionsException;
import org.modelgen.regole.Exception.IncompleteQualifiedAttributeException;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * Rappresenta un letterale negato
 * @author luigi, francesca
 * 
 */
public class LetteraleNegato extends Letterale {
	
	
	public LetteraleNegato(String predicateName, String ruolo) {
		super(predicateName, ruolo);
		setIsNegato(true);
	}
	
	
	/**
	 * Genera l'insieme di condizioni per questo letterale negato
	 */
	public Set<Condizione> generaCondizioni(Map<String, Tabella> tabelle) throws IncompleteLiteralConditionsException {
		Set<Condizione> condizioni = new HashSet<Condizione>();
		CondizioneNotExist cne = new CondizioneNotExist(getPredicateName());
		List argomentiEtichettati = getListaArgomentiEtichettati();
		Iterator it = argomentiEtichettati.iterator();
		ArgomentoEtichettato ae = null;
		while(it.hasNext()) {
			ae = (ArgomentoEtichettato)it.next();
			try {
				cne.addSetCondizioni(ae.generaCondizioni("varNotExist",tabelle));
			} catch(IncompleteQualifiedAttributeException ex) {
				throw new IncompleteLiteralConditionsException(ex,getPredicateName());
			}
			
		}
		
		condizioni.add(cne);
		
		return condizioni;
		
		
	}
	
	/**
	 * Restituisce una copia di questo letterale. Copiare un letterale negato
	 *  diverso da copiare un letterale non negato. Questo duplicate()
	 * ridefinisce quello della superclasse
	 * @param regola la regola Datalog a cui appartiene
	 * @return una copia di questo letterale
	 */
	public Letterale duplicate(RegolaDatalog regola) {
		
		Letterale newLetterale = new LetteraleNegato(this.getPredicateName(),this.getRuolo());		
		
		newLetterale.setHasSkolem(this.getHasSkolem());
		newLetterale.setIsInTesta(this.getIsInTesta());
		newLetterale.setIsNegato(this.getIsNegato());
		
		for(ArgomentoEtichettato ae : getListaArgomentiEtichettati()) {
			newLetterale.inserisciArgomentoEtichettato(ae.getAttributo(),ae.getArgomento().duplicate(regola,newLetterale, false));
		}
		
		return newLetterale;
	}
	
	
	

}
