/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.Exception.VarNotFoundException;

import java.util.Map;

/**
 * Sono manipolabili gli oggetti che sono possibili argomenti
 * di condizioni datalog o di argomenti composti 
 * @author luigi, francesca
 * 
 *
 */
public interface Manipolabile {
	/**
	 * 
	 * @param tabelle le tabelle di corrispondenza Letterale variabile
	 * @return la semicondizione
	 */
	public String generaSemicondizione(Map<String, Tabella> tabelle) throws VarNotFoundException;
	
	/**
	 * Genera il manipolabile per istanza corrispondente a questo. IsInSkolem specifica se il
	 * manipolabile si trova in un funtore di Skolem o meno.
	 * @param nuova la regola Datalog in creazione
	 * @param newLetterale il letterale in creazione
	 * @param isInSkolem se questo manipolabile si trova in un funtore di Skolem o meno
	 * @return il manipolabile per istanza corrispondente a questo 
	 */
	public Manipolabile getManipolabilePerIstanza(RegolaDatalog nuova, Letterale newLetterale, boolean isInSkolem);
	
	/**
	 * Restituisce una copia del manipolabile
	 * @param nuova la regola in creazione
	 * @param newLetterale il letterale in creazione
	 * @param isInSkolem se il manipolabile  in un funtore di Skolem
	 * @return la copia del manipolabile
	 */
	public Manipolabile getCopia(RegolaDatalog nuova, Letterale newLetterale, boolean isInSkolem);

}
