/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;
import org.modelgen.regole.Exception.BTranslationException;
import org.modelgen.regole.Exception.IncompleteLiteralConditionsException;
import org.modelgen.regole.Exception.LiteralTranslationException;
import org.modelgen.regole.utilities.CollectionFormatter;
import org.modelgen.regole.utilities.VarGenerator;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * Rappresenta una regola SQL
 * @author luigi, francesca
 * 
 */
public class Regola {
	private boolean isApplyDistinct;   // la select � con distinct o meno
	private boolean onlyConstPars; // se è una regola insertSkolem derivata da funtore con tutte costanti
	private Map<String,Tabella> tabelle;               // di tipo Tabella
	private InsertPart insertPart;     // la parte insert
	private VarGenerator varGenerator;
	private Set<Condizione> condizioniWhere;       // di tipo Condizione
	private List<CondizioneSelect> selectPart;            // di tipo CondizioneSelect
	
	
	public Regola(boolean iad){
		this.isApplyDistinct = iad;
		this.tabelle = new HashMap<String,Tabella>();
		this.varGenerator = new VarGenerator();
		this.condizioniWhere = new ConditionSet(); // vedi toString() per l'ordinamento
		this.selectPart = new LinkedList<CondizioneSelect>();
	}
	
	/**
	 * Costruisce una particolare RegolaA: la regola di inserimento per i 
	 * funtori di Skolem. Vengono filtrate le condizioni NotExist e i parametri
	 * sono inizializzati come copia di quelli di un'altra regola.
	 * @param condizioniWhere l'insieme delle condizioni where
	 * @param tabelle la mappa delle corrispondenze Letterale variabile
	 * @param ip la parte insert
	 */
	public Regola(Set condizioniWhere, Map<String,Tabella> tabelle, InsertPart ip) {
		this.tabelle = new HashMap<String,Tabella>();
		this.varGenerator = new VarGenerator();
		this.condizioniWhere = new ConditionSet(); // vedi toString() per l'ordinamento
		this.selectPart = new LinkedList<CondizioneSelect>();
		isApplyDistinct = true; // una regola sk � sempre apply distinct!
		setCondizioniWhereforSK(condizioniWhere);
		setTabelle(tabelle);
		setInsertPart(ip);
		
	}
	
	public void setIsApplyDistinct(boolean isApplyDistinct){
		this.isApplyDistinct = isApplyDistinct;
		
	}
	
	public void setOnlyConstPars(boolean ocp) {
		this.onlyConstPars = ocp;
	}
	

	public boolean getIsApplyDistinct() {
		return this.isApplyDistinct;
	}
	
	protected void setInsertPart(InsertPart ip){
		this.insertPart = ip;
	}
	
	public InsertPart getInsertPart() {
		return this.insertPart;
	}
	
	protected void setTabelle(Map<String,Tabella> tabelle){
		// copia per mantenere la parte from con le sole condizioni
		// relative al funtore target.
		this.tabelle = new HashMap<String,Tabella>(tabelle);
	}
	
	public Map<String,Tabella> getTabelle(){
		return this.tabelle;
	}
	public VarGenerator getVarGenerator(){
		return this.varGenerator;
	}
	public void addCondizioneSelect(CondizioneSelect cs){
		selectPart.add(cs);
	}
	
	public Set<Condizione> getCondizioniWhere(){
		return this.condizioniWhere;
	}
	
	public void setCondizioniWhere(Set<Condizione> condizioni){
		this.condizioniWhere = condizioni;
	}
	
	public void addCondizioneWhere(Condizione c) {
		condizioniWhere.add(c);
	}
	
	
	/**
	 * Aggiunge un insieme di condizioni alla regola insert 
	 * generata in corrispondenza di un funtore target
	 * @param condizioni un insieme di condizioni
	 */
	private void setCondizioniWhereforSK(Set condizioni){
		Iterator it = condizioni.iterator();
		Condizione c = null;
		while(it.hasNext()){
			c = (Condizione)it.next();
			addCondizioneWhere(c);			
		}
	}

	
	public void setSelectPart(List<CondizioneSelect> sp){
		this.selectPart = sp;
	}
	
	
	/**
	 * Data la lista dei letterali del corpo di una Regola Datalog genera 
	 * da essa gli opportuni oggetti della regola sqlAlto corrispondente
	 * @param corpo la lista dei letterali del corpo della regola Datalog da cui deriva la regola
	 */
	public void traduciCorpo(List<Letterale> corpo) throws LiteralTranslationException {
		Iterator<Letterale> it = corpo.iterator();
		Letterale letterale = null;
		String idLetterale = null;
		Tabella tabella = null;
		while (it.hasNext()) {
			letterale = it.next();
			
			if (!letterale.getIsNegato()) {
				tabella = new Tabella(letterale.getPredicateName(), varGenerator);
				idLetterale = letterale.getID();
				tabelle.put(idLetterale, tabella);										
				
			}
		}
		
		
		
		it = corpo.iterator();
		while(it.hasNext()) {
			letterale = it.next();		
			/* aggiunge alle condizioni where tutte quelle relative al letterale corrente */
			try {
				condizioniWhere.addAll(letterale.generaCondizioni(tabelle));
			} catch(IncompleteLiteralConditionsException ex) {
				throw new LiteralTranslationException(ex,false);
			}
			
			
		}
				
	}
	
	/**
	 * Traduce le condizioni esplicite della regola datalog
	 * @param condizioni la lista delle condizioni esplicite della regola datalog
	 */
	public void traduciCondizioni(List<CondizioneDatalog> condizioni) {
		Iterator<CondizioneDatalog> it = condizioni.iterator();
		CondizioneDatalog cd = null;
		Condizione cs = null;
		while(it.hasNext()) {
			cd = it.next();
			cs = cd.generaCondizione(tabelle);
			condizioniWhere.add(cs);
			
		}
		
	}
	
	/**
	 * Genera le clausole derivanti dalla traduzione della testa
	 *
	 */
	public void traduciTesta(Letterale testa) throws LiteralTranslationException {
		InsertPart ip = new InsertPart(testa.getPredicateName());
		ip.setColonne(testa.getListaAttributi());
		
		/* se sta traducendo una regola per istanza mette instanceOID, altrimenti schemeOID */
		if(!testa.isInstance())
			ip.addColonna("schemeOID"); // condizione sullo schema
		else
			ip.addColonna("instanceOID"); // condizione sull'istanza
		
		insertPart = ip; // impostazione dell'attributo locale
		try {
			List<CondizioneSelect> condizioni = testa.getListaCondizioniSelect(tabelle); // di tipo select
			
			if(!this.onlyConstPars)
				condizioni.add(testa.getSchemeCondition()); // ?x as schemeOID
			else // se deriva da funtore con tutte costanti
				condizioni.add(testa.getConstantSchemeCondition()); // ?x
			
			selectPart = condizioni; // impostazione dell'attributo di istanza
		
		
			/* se la testa non ha funtori di Skolem si genera l'opportuna */
			/* condizione not Exist */
			if(!testa.getHasSkolem()) {
				Condizione cne = testa.getTMP(tabelle); 
				condizioniWhere.add(cne);
			}
		} catch(IncompleteLiteralConditionsException ex) {
			throw new LiteralTranslationException(ex,true);
			
		}
		
		
	}
	
	/**
	 * Traduce questa regola sqlAlto in una regola SqlBasso ossia con gli eventuali funtori
	 * di skolem risolti.
	 * @return la regola SqlBasso
	 *
	 */
	public RegolaB traduciSqlBasso() throws BTranslationException {
		RegolaB regolab = new RegolaB(isApplyDistinct, insertPart, tabelle);
		regolab.traduciWhere(this.condizioniWhere); // risolve i funtori delle parte where e li imposta
		try {
			regolab.traduciSelectPart(this.selectPart); // risolve i funtori di skolem presenti nella parte select
		} catch(BTranslationException ex) {
			throw ex;
		}
		
		return regolab;
		
	}
	
	public String toString() {
		/* ordinamento delle condizioni where */
		List<Condizione> condizioniOrdinate = new LinkedList<Condizione>(condizioniWhere);
		Collections.sort(condizioniOrdinate);
		
		String IND = "    ";
		StringBuffer buf = new StringBuffer();
		buf.append(insertPart.toString() + "\n");
		if(!this.onlyConstPars) {
			buf.append("SELECT" + (isApplyDistinct ? " DISTINCT" : "") +  "\n");
			buf.append(IND + CollectionFormatter.getInstance().getSequenzaComponenti(selectPart,",\n"+IND)+"\n");
			buf.append("FROM \n");
			buf.append(IND + CollectionFormatter.getInstance().getSequenzaComponenti(tabelle,",\n"+IND)+"\n");
			buf.append("WHERE \n");
			buf.append(IND + IND + "   " + CollectionFormatter.getInstance().getSequenzaComponenti(condizioniOrdinate,"\n"+IND+"AND "));
			buf.append("\n;");
		} else { // traduzione di regola di funtore con sole costanti
			buf.append("VALUES (");
			buf.append(CollectionFormatter.getInstance().getSequenzaComponenti(selectPart,", "));
			buf.append(")\n;");
		}
	
			
			return buf.toString();
	}
	
	

}
