/*
 * Created on 26-mag-2005
 *
 */
package org.modelgen.regole;
import org.modelgen.regole.Exception.BTranslationException;
import org.modelgen.regole.Exception.TooManyFunctorArgumentsException;
import org.modelgen.regole.utilities.CollectionFormatter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;




/**
 * Rappresenta una regola SQL basso cio� in cui sono stati risolti i funtori di Skolem
 * @author luigi, francesca
 *
 */
public class RegolaB extends Regola {
	private List<Regola> insertSkolem;
	
	public RegolaB(boolean isApplyDistinct, InsertPart ip, Map<String,Tabella> tabelle){
		super(isApplyDistinct);
		super.setInsertPart(new InsertPart(ip)); // copie per evitare effetti collaterali sulla regolaA
		super.setTabelle(new HashMap<String,Tabella>(tabelle));  // copie per evitare effetti collaterali sulla regolaA
		this.insertSkolem = new LinkedList<Regola>();		
	}
	
	/**
	 * Ritorna la lista ordinata di tutti gli statements da applicare
	 * @return la lista di tutti gli statements da applicare
	 */
	public List<String> getStatements() {
		String statement = null;
		List<String> statements = new LinkedList<String>();
		/* aggiunta degli statements di Skolem */
		for(Regola regola : insertSkolem) {
			statement = regola.toString();
			statements.add(statement);
		}
		/* aggiunta dello statement principale */
		statements.add(super.toString());
		return statements;		
	}
	
	/**
	 * Dato un insieme di condizioni where risolve i funtori eventualmente presenti
	 * e li imposta.
	 * @param condizioniWhere
	 */
	public void traduciWhere(Set<Condizione> condizioniWhere){
		ConditionSet s = new ConditionSet();
		for (Condizione c : condizioniWhere){
			s.addAll(c.generaCondizioniB(getTabelle(), getVarGenerator()));
		}
		getCondizioniWhere().addAll(s);		
	}

	
	/**
	 * Risolve i funtori della parte select generando le opportune condizioni where from e select
	 * @param selectPart la parte select della regola SQLA corrispondente
	 */
	public void traduciSelectPart(List<CondizioneSelect> selectPart) throws BTranslationException{
		Map<String,Tabella> tabelle1 = new HashMap<String,Tabella>();
		Set<Condizione> where1 = new ConditionSet();
		CondizioneNotExist cne = new CondizioneNotExist(getInsertPart().getNomeTabella());
		int i = 0;
		
		/* Nella condizione NotExist generale si deve mettere solo la 
		 * condizione relativa al funtore target di Skolem riferita alla 
		 * colonna OID della parte insert. Per far questo si passa volta per volta
		 * la giusta colonna al metodo addCondizioneSelect. Sar� il funtore, 
		 * a valutare se la colonna di riferimento � OID e, se s�, ad aggiungere
		 * la condizione nella NotExist. */
		
		for(CondizioneSelect cs : selectPart){
			try {
				addCondizioneSelect(cs.getTradotto(getTabelle(), tabelle1, getCondizioniWhere(), where1, getVarGenerator(), insertSkolem, cne,this.getInsertPart().getColonna(i)));
			} catch(TooManyFunctorArgumentsException ex) {
				throw new BTranslationException(ex);
			}
			i++; // considera la colonna successiva	
		}
		
		getTabelle().putAll(tabelle1); // unisce le due parti from
		getCondizioniWhere().addAll(where1); // unisce le due parti where
		if(!cne.isVuota())
			getCondizioniWhere().add(cne); // aggiunge la NotExist sui funtori target
		
		
	}
	
	public String toString() {
		StringBuffer buf = new StringBuffer();
		/* se non ci sono funtori di Skolem nella testa questa parte */
		/* non va eseguita. */
		if(insertSkolem.size()!=0) {
			buf.append(CollectionFormatter.getInstance().getSequenzaComponenti(insertSkolem,"\n \n"));
			buf.append("\n\n");
		}
		buf.append(super.toString());
		return buf.toString();		
	}

}
