/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.utilities.CollectionFormatter;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Rappresenta una funzione di skolem
 * @author luigi, francesca
 * 
 */
public class SkolemFunction {
	
	public final static String INSTANCE_PREFIX = "i_";
	private String targetTable;        // nome della tabella
	private String targetOID;          // OID risultato, da prelevare
	private String skolemFunctionName; // nome della funzione
	private List<String> argomenti;            // gli argomenti del funtore (di tipo VariabileDatalog)
	private final String SKOLEM_SUFFIX = "_SK"; // suffisso della tabella relazionale di Skolem
	private boolean isInstance;        // se  una funzione di istanza
	
	public SkolemFunction(String targetTable, String targetOID, String functionName) {
		this.targetTable = targetTable;
		this.targetOID = targetOID;
		this.skolemFunctionName = functionName;
		this.argomenti = new LinkedList<String>();
		isInstance = false;
	}
	
	public String getTagetTable() {
		return this.targetTable+SKOLEM_SUFFIX;
	}
	
	public void setTargetTable(String s) {
		this.targetTable = s;
	}
	
	public String getTargetOID() {
		return this.targetOID;
	}
	
	public void setTargetOID(String s) {
		this.targetOID = s;
	}
	
	public String getSkolemFunctionName() {
		return this.skolemFunctionName;
	}
	
	public void setSkolemFunctionName(String s) {
		this.skolemFunctionName = s;
	}
	
	public void addArgumentName(String a) {
		argomenti.add(a);
	}
	
	public List<String> getArgomenti() {
		return this.argomenti;
	}
	
	/**
	 * Ritorna se questa funzione  di istanza
	 * @return se questa funzione  di istanza o meno
	 */
	public boolean isInstance() {
		return this.isInstance;
	}
	
	public void setIsInstance(boolean b) {
		this.isInstance = b;
	}
	
	/**
	 * Restituisce la corrispondente funzione di Skolem per istanza, cambiando il nome e
	 * l'argomento SCHEMEOID con INSTANCEOID.
	 * @return la corrispondente funzione di Skolem per istanza
	 */
	public SkolemFunction getFunzionePerIstanza() {
		
		/* crea la funzione di Skolem associata alla tabella per istanza */
		/* cambia il nome e l'argomento SCHEMEOID con INSTANCEOID */
		/* per targetTable accede direttamente al campo per evitare la doppia aggiunta dello */
		/* SKOLEM_SUFFIX */
		SkolemFunction skfn = new SkolemFunction(INSTANCE_PREFIX+this.targetTable,this.getTargetOID(),INSTANCE_PREFIX+this.getSkolemFunctionName());
		skfn.setIsInstance(true);
		String argomento = null;
		Iterator<String> it = argomenti.iterator();
		while(it.hasNext()) {
			argomento = it.next();
			if(argomento.equals("SCHEMEOID"))
				argomento = "INSTANCEOID";
			skfn.addArgumentName(argomento);
		}
		return skfn;
	}
	
	public String toString() {
		StringBuffer funzione = new StringBuffer();
		funzione.append(getSkolemFunctionName()+"("+CollectionFormatter.getInstance().getSequenzaComponenti(getArgomenti())+")");
		return funzione.toString();
	}
	

}
