/*
 * Created on 14-mag-2005
 *
 */
package org.modelgen.regole;

import org.modelgen.regole.Exception.VarNotFoundException;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;


/**
 * Rappresenta una variabile di una RegolaDatalog
 * @author luigi, francesca
 * 
 *
 */
public class VariabileDatalog implements Argomento, Manipolabile {
	public final static String INSTANCE_PREFIX = "i_";
	private Set<Letterale> letterali;         // l'insieme di letterali a cui � associata la variabile
	private Set<Letterale> letteraliSk;       // i letterali in cui la variabile compare nei funtori di Skolem
	private String nome;           // il nome della variabile
	
	public void setLetterale(Letterale letterale) {
		
	}
	
	public VariabileDatalog(String nome){
		this.nome = nome;
		letterali = new HashSet<Letterale>();
		letteraliSk = new HashSet<Letterale>();
	}
	
	/**
	 * Genera il valore associato all'agomento (il nome della variabile): metodo utile 
	 * per evitare di testare il tipo degli oggetti argomento nella costruzione della 
	 * segnatura di una regola
	 * @return il valore associato all'argomento
	 */
	public String getValore() {
		return this.getNome();
	}
	
	public void addLetterale(Letterale l){
		letterali.add(l);
	}
	
	/**
	 * 
	 * @param l un letterale in cui la variabile compare in un funtore
	 */
	public void addLetteraleFuntore(Letterale l) {
		letteraliSk.add(l);
	}
	
	public String getNome() {
		return this.nome;
	}
	
	public void setNome(String nome) {
		this.nome = nome;
	}
	
	
	/**
	 * Ritorna il primo letterale della variabile che non sia negato
	 * in cui non compaia come funtore e che non sia la testa
	 * @return un letterale della variabile
	 */
	public Letterale getLetterale() throws VarNotFoundException {
		
		Iterator<Letterale> it = letterali.iterator();
				Letterale l = null;
		while(it.hasNext()) {
			l = it.next();
			if(!l.getIsInTesta() && !l.getIsNegato())
				return l;							
		}
	
		
		/* il controlo arriva qui solo per errore semantico della regola Datalog in input */
		/* eccezione, variabile non trovata  */
		throw new VarNotFoundException(this.nome);
		  
		           
					
			
	}
	
	/**
	 * 
	 * @param tabelle la mappa delle corrispondenze Letterali variabili
	 * @return la semicondizione
	 */
	public String generaSemicondizione(Map<String, Tabella> tabelle) throws VarNotFoundException {
		
		try {
			Letterale letterale = getLetterale();
		
		
			String attributo = letterale.getAttributoPerVariabile(this.nome);
			Tabella tabella = tabelle.get(letterale.getID());
			String var = tabella.getVarOID();		
			return var + "." + attributo;
		
		} catch(VarNotFoundException ex) {
			throw ex;
		}
	
		
		
	
	}
		
	public Set<CondizioneSemplice> generaCondizioni(String variabile, Map<String, Tabella> tabelle, String attributo) {
		Set<CondizioneSemplice> cs = new HashSet<CondizioneSemplice>(); // di tipo condizione semplice
		Iterator<Letterale> it = letterali.iterator();
		Letterale letterale = null;
		String attr = null;
		Tabella t = null;
		String var = null;
		CondizioneSemplice condizione = null;
		while(it.hasNext()) {
			letterale = it.next();
			
			if(!letterale.getIsNegato() &&!letterale.getIsInTesta()) {
				attr = letterale.getAttributoPerVariabile(this.nome);			
				t = tabelle.get(letterale.getID());
				var = t.getVarOID();
			
				/* controllo se � una condizione del genere di var1.a = var1.a, ovvero inutile */
				//if(!(var.equals(variabile) &&(attr.equals(attributo)))) {
					condizione = new CondizioneSemplice(variabile + "." + attributo,var + "."+ attr,"=");
					cs.add(condizione);
				//}
			
			}
			
		}
		
		return cs;
		
	}
	
	public CondizioneSelect getSelect(Map<String, Tabella> tabelle) throws VarNotFoundException {
		try {
			Letterale l = getLetterale();
			String attr = l.getAttributoPerVariabile(nome);
			Tabella tabella = tabelle.get(l.getID());
			String var = tabella.getVarOID();
			return new CondizioneSelectSemplice(var + "." + attr);
		} catch(VarNotFoundException ex) {
			throw ex;
		}
		
	}
	
	public Condizione getTMP(String var, Map<String, Tabella> tabelle, String attributo) throws VarNotFoundException {
		try {
			Letterale l = getLetterale();
			String attr = l.getAttributoPerVariabile(nome);
			Tabella t = tabelle.get(l.getID());
			String var1 = t.getVarOID();
			CondizioneSemplice cs = new CondizioneSemplice(var + "." + attributo, var1 + "." + attr,"=");
			return cs;
		} catch(VarNotFoundException ex) {
			throw ex;
		}
	}
	
	public Argomento getArgomentoPerIstanza(RegolaDatalog nuova, Letterale newLetterale) {
		/* fa aggiungere alla regola in creazione una nuova variabile rinominata con i- che non � */
		/* in un funtore di Skolem */
		VariabileDatalog newVariabile = nuova.getVariabile(INSTANCE_PREFIX+this.getNome(),newLetterale,false);
		return newVariabile;
	}
	
	public Manipolabile getManipolabilePerIstanza(RegolaDatalog nuova, Letterale newLetterale, boolean isInSkolem) {
		/* fa aggiungere alla regola in creazione una nuova variabile rinominata con i- che pu� */
		/* essere in un funtore di Skolem o in un argomento composto */
		VariabileDatalog newVariabile = nuova.getVariabile(INSTANCE_PREFIX+this.getNome(),newLetterale,isInSkolem);
		return newVariabile;
		
	}
	
	public Argomento duplicate(RegolaDatalog nuova, Letterale newLetterale, boolean isSpecialContext) {
		/* il metodo viene invocato solo al di fuori di un funtore di Skolem */
		/* quindi la variabili si crea sempre con parametro false */
		VariabileDatalog newVariabile = nuova.getVariabile(this.getNome(),newLetterale,false);
		return newVariabile;
	}
	
	public Manipolabile getCopia(RegolaDatalog nuova, Letterale newLetterale, boolean isInSkolem) {
		VariabileDatalog newVariabile = nuova.getVariabile(this.getNome(),newLetterale,isInSkolem);
		return newVariabile;
		
	}
	
	public String toString() {
		return nome;
	}

}
