/*
 * Created on 15-mag-2005
 *
 */
package org.modelgen.regole.utilities;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Contiene alcune funzioni di utilit� per la stampa di collezioni.
 * @author luigi, francesca
 * 
 */
public class CollectionFormatter {
	
	public final static String SQL_INTERVAL = ""; // separatore per relazioni e nomi colonne del dbms corrente
	public final static String INSTANCE_PREFIX = "i_"; 
	private static CollectionFormatter instance = new CollectionFormatter();
	
	
	/**
	 * Restituisce un'istanza dell'oggetto helper.
	 * @return un'istanza dell'oggetto helper
	 */
	public static CollectionFormatter getInstance() {
		return instance;
	}
	
	
	/**
	 * 
	 * @param l la lista da stampare con il separatore tra gli argomenti
	 * @param separatore il separatore da inserire tra gli elementi della lista
	 * @return la stringa rappresentante la stampa
	 */
	public String getSequenzaComponenti(List<?> l,String separatore) {
		
		
		Iterator<?> it = l.iterator();
		Object o;
		StringBuffer risultato = new StringBuffer();
		while(it.hasNext()) {
			o = it.next();
			risultato.append(o.toString() + separatore);
		}
		
		if(risultato.length()<1)
		 	return "";
        else			
        	return risultato.substring(0,risultato.length() - separatore.length());
				
	}
	
	/**
	 * 
	 * @param s l'insieme da stampare con il separatore tra gli argomenti
	 * @return la stringa rappresentante la stampa
	 */
	public <T> String getSequenzaComponenti(Set<T> s) {
		return getSequenzaComponenti(new LinkedList<T>(s));
	}
	
	/**
	 * 
	 * @param s l'insieme da stampare con il separatore tra gli argomenti
	 * @param separatore il separatore da inserire tra gli elementi della lista
	 * @return la stringa rappresentante la stampa
	 */
	public <T> String getSequenzaComponenti(Set<T> s, String separatore) {
		return getSequenzaComponenti(new LinkedList<T>(s), separatore);
	}
	

	/**
	 * 
	 * @param m la mappa da stampare con il separatore tra gli argomenti
	 * @param separatore il separatore da inserire tra gli elementi della lista
	 * @return la stringa rappresentante la stampa
	 */
	public String getSequenzaComponenti(Map<?, ?> m, String separatore) {
		Set<?> s = m.keySet();
		Iterator<?> it = s.iterator();
		StringBuffer buf = new StringBuffer();
		Object o=null;
		while(it.hasNext()) {
			o = it.next();
			buf.append(m.get(o).toString() + separatore);
		}
		
		if(buf.length()!=0)
			return buf.substring(0,buf.length() - separatore.length());	
		else
			return "";
	}
	
	/**
	 * 
	 * @param m la mappa da stampare con il separatore tra gli argomenti
	 * @return la stringa rappresentante la stampa
	 */
	public String getSequenzaComponenti(Map<?, ?> m) {
		return getSequenzaComponenti(m,",");
	}
		
	/**
	 * 
	 * @param l la lista da stampare con le virgole tra gli argomenti
	 * @return la lista rappresentante la stampa
	 */
	public String getSequenzaComponenti(List<?> l) {
		return getSequenzaComponenti(l,",");
	}
	
	/**
	 * Mette tutti gli elementi di una lista tra separatori
	 * @param l la lista
	 * @return la nuova lista
	 */
	public List<String> mettiElementiListaTraSeparatori(List<String> l) {
		List<String> nl = new LinkedList<String>();
		for (String s : l) {
			nl.add(this.mettiTraSeparatore(s));
		}
		
		return nl;
	}
	
	public List<String> mettiElementiListaSeparatoreDopoPunto(List<String> l) {
		List<String> nl = new LinkedList<String>();
		for (String s : l) {
			nl.add(this.mettiSeparatoreDopoPunto(s));
		}
		
		return nl;
	}
	
	/**
	 * Restituisce la stringa tra i separatori SQL
	 * @param a la stringa
	 * @return la stringa compresa tra i separatori SQL
	 */
	public String mettiTraSeparatore(String a) {
		return SQL_INTERVAL + a + SQL_INTERVAL;
	}
	
	/**
	 * Data una stringa contente un punto, mette tra separatori
	 * la sottostringa che segue il punto e la concatena con quella
	 * originaria. Restituisce il tutto.
	 * @param a la stringa originale
	 * @return la stringa manipolata con i separatori
	 */
	public String mettiSeparatoreDopoPunto(String a) {
		
		int i = 0;
		boolean ok = false;
		
		StringBuffer buf = new StringBuffer();
		String b = null;
		
		for(i=0;i<a.length() && !ok;i++) {
			if(a.charAt(i)=='.') {
				buf.append(".");
				b = SQL_INTERVAL +a.substring(i+1) + SQL_INTERVAL;				
				buf.append(b);
				ok = true;
			}
			if(!ok)
				buf.append(a.charAt(i));	
		}
		
		return buf.toString();
		
	}
	
}
