package org.modelgen.signature;

/**
 * La segnatura di un costrutto  data dal nome del costrutto 
 * (o dalla sua abbreviazione) e da una formula associata, che 
 * rappresenta i vincoli imposti alle propriet del costrutto. 
 * Ad esempio, per una Relationship molti a molti si avr 
 * R(NOT Funct1 AND NOT Funct2).
 * Abbiamo quindi un campo di tipo String che specifica il 
 * nome del costrutto e un altro di tipo String che indica la 
 * formula: di default la formula vale true, cio indica che 
 * non ci sono vincoli sulle propriet del costrutto.
 * @author Fabrizio Celli
 * @version 1.0
 * @since 03-05-2007
 *
 */
public class ConstructSignature {
	
	private String name;
	
	//	di default vale true (cio non ci sono vincoli)
	private String formula;	
	
	public ConstructSignature(){
		this.formula = "true";
	}
	
	public ConstructSignature(String name){
		this();
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getFormula() {
		return formula;
	}

	public void setFormula(String formula) {
		if(formula!=null && formula!="")
			this.formula = formula;
	}
	
	public String toString(){
		return this.getName()+": "+this.formulaSemplice(this.getFormula(), this.getName());
	}
	
	/*
	 * Metodo di supporto al toString: elimina dalla formula associata ad
	 * un costrutto il nome del costrutto che precede ciascuna propriet
	 */
	private String formulaSemplice(String formula, String nomeCostrutto){
		return formula.replaceAll(nomeCostrutto+".", "");
	}
	
	/**
	 * Due ConstructSignature sono uguali se hanno lo stesso nome e se la
	 * loro formula  uguale, nel senso che le due formule sono scritte allo
	 * stesso modo e non che sono logicamente equivalenti (ecco che spesso 
	 *  preferibile, prima di usare tale metodo, di trasformarne le formule 
	 * nella FND)
	 */
	public boolean equals(Object o){
		ConstructSignature cur = (ConstructSignature)o;
		return (this.getName().equals(cur.getName()) &&
				this.getFormula().equals(cur.getFormula()));
	}

}
