package org.modelgen.signature;

import java.util.Comparator;
import java.util.Set;

/**
 * Date due MacroRuleSignature P1 e P2, 
 * se [unione Hi(P1)] intersezione [unione Bj(P2)] != insiemeVuoto 
 * allora P1  minore di P2.
 * @author Fabrizio Celli
 */
public class MacroRuleSignatureComparator implements Comparator<MacroRuleSignature> {

	/**
	 * Compares its two arguments (MacroRuleSignature) for order. 
	 * Returns a negative integer, zero, or a positive integer as 
	 * the first argument is less than, equal to, or greater than 
	 * the second.
	 */
	public int compare(MacroRuleSignature sign1, MacroRuleSignature sign2) {
		//unione delle teste di sign1
		Set<String> heads1 = sign1.getUnionHeads();
		//unione dei corpi di sign2
		Set<String> bodies2 = sign2.getUnionBodies();
		
		//per verificare che (heads1 intersezione bodies2)  non vuoto, rimuoviamo
		//da heads1 gli elementi di bodies2 e verifichiamo che sia stato effettivamente
		//rimosso qualche elemento.
		int oldHeads1Size = heads1.size();
		heads1.removeAll(bodies2);
		if(heads1.size()<oldHeads1Size)
			return -1;
		//se heads1 e bodies1 avevano gli stessi elementi
		if(heads1.size()==0 && bodies2.size()==oldHeads1Size)
			return 0;
		return 1;
	}

}
