package org.modelgen.signature;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * La segnatura di un modello  data dal nome del modello e da una 
 * lista di segnature di costrutti presenti nel modello, ciascuna 
 * con i vincoli imposti da esso. 
 * Abbiamo quindi un campo di tipo String che specifica il nome 
 * del modello e una mappa che associa al nome di un costrutto 
 * la ConstructSignature.
 * @author Fabrizio Celli
 * @since 1.1.1
 *
 */
public class ModelSignature {

	//model's name
	private String name;

	//mappa che contiene la segnatura di un costrutto, con il suo nome
	private Map<String, ConstructSignature> costrutti;

	//lista di costrutti temporanei
	private List<String> TMP;

	public ModelSignature(String name){
		this.costrutti = new HashMap<String, ConstructSignature>();
		this.TMP = new LinkedList<String>();
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public Map<String, ConstructSignature> getCostrutti() {
		return costrutti;
	}

	public void setCostrutti(Map<String, ConstructSignature> costrutti){
		this.costrutti = costrutti;
	}

	public void insert(ConstructSignature costrutto){
		if(costrutto.getName().startsWith("_"))
			this.insertTMP(costrutto);
		else
			this.costrutti.put(costrutto.getName(), costrutto);
	}

	//inserisce un costrutto temporaneo
	private void insertTMP(ConstructSignature costrutto){
		this.costrutti.put(costrutto.getName(), costrutto);
		this.TMP.add(costrutto.getName());
	}

	/**
	 * Aggiorna la formula di una ConstructSignature, se gi  nella mappa
	 * @param costrutto
	 * @param newFormula
	 * @return true se l'aggiornamento  stato eseguito
	 */
	public boolean update(String costrutto, String newFormula){
		ConstructSignature construct = this.costrutti.get(costrutto);
		if(construct!=null){
			construct.setFormula(newFormula);
			return true;
		}
		return false;
	}

	//toString
	public String toString(){
		String result = "";
		Collection<ConstructSignature> collection = this.getCostrutti().values();
		for(ConstructSignature cs: collection)
			result = result + "-> "+cs + "\n";
		return result;
	}

	/**
	 * Restituisce una lista di stringhe, dove ogni stringa  il nome del costrutto
	 * seguito dalla formula associata:
	 * "-> costrutto: formula"
	 */
	public List<String> getCostruttiAsStringhe(){
		List<String> result = new LinkedList<String>();
		Collection<ConstructSignature> collection = this.getCostrutti().values();
		for(ConstructSignature cs: collection){
			String costrutto = "-> "+cs;
			result.add(costrutto);
		}
		return result;
	}

	/**
	 * Rimuove tutti i costrutti temporanei dalla ModelSignature
	 */
	public void removeTmpConstruct() {
		for(String construct: this.TMP)
			this.costrutti.remove(construct);
	}

}
