package org.modelgen.signature.application.loader;

import org.modelgen.commons.utils.ModelgenConfigurationManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Fabrizio Celli
 * This class implements a loader for available rules
 */
public class RulesLoader {
	
	//singleton
	private static RulesLoader instance = null;
	
	//macrorule's path and directory
	private String rootnode;
	private File macroRuleDir;
	
	private RulesLoader(){
		rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
	}
	
	//singleton
	public static RulesLoader getInstance(){
		if(instance==null)
			instance = new RulesLoader();
		return instance;
	}
	
	/**
	 * Return a list of available rules
	 */
	public List<String> load() {
		String macrorulePath = rootnode+ModelgenConfigurationManager.getIstance().getProperty("macrorules", "", "paths");
		macroRuleDir = new File(macrorulePath);
		return load(macroRuleDir.list(new FileFilterAll()));
	}
	
	/**
	 * Return a list of available transformations
	 * @throws IOException 
	 */
	public List<String> loadTransformations() throws IOException {
		String transFile = rootnode+ModelgenConfigurationManager.getIstance().getProperty("transformations", "", "paths");
		return readFile(new File(transFile));
	}
	
	/**
	 * Return a list of available reductions
	 * @throws IOException 
	 */
	public List<String> loadReductions() throws IOException {
		String redFile = rootnode+ModelgenConfigurationManager.getIstance().getProperty("reductions", "", "paths");
		return readFile(new File(redFile));
	}
	
	/*
	 * Read a file
	 */
	private List<String> readFile(File file) throws IOException{
		List<String> rules = new LinkedList<String>();
		BufferedReader lettore = new BufferedReader(new FileReader(file));//leggiamo il file
		while(lettore.ready()){//per ogni riga ne analizziamo il contenuto
			String riga = lettore.readLine();
			if(riga.equals("\n")||(riga.equals("")))
				break;
			else
				rules.add(riga);
		}
		return rules;
	}
	
	/*
	 * Load preferred rules
	 */
	private List<String> load(String sa_rules[]){ 
		List<String> rules = new LinkedList<String>();
		if (macroRuleDir.isDirectory()) {
			// All the .txt files are now in the array, but it should be easier to 
			// deal with a LinkedList.
			if (sa_rules != null)
				for (int i = 0; i < sa_rules.length; i++) {
					if(!(sa_rules[i].startsWith("LF")) && !(sa_rules[i].startsWith("test")))
						rules.add(sa_rules[i]);
				}
		} else 
			rules = null;
		return rules;
	}
	
	
}