package org.modelgen.signature.application.ruleapplication;

import org.modelgen.signature.ConstructSignature;
import org.modelgen.signature.ModelSignature;
import org.modelgen.signature.RuleSignature;
import org.modelgen.signature.util.ToolFormuleAsStringhe;
import org.modelgen.signature.util.ToolTableaux;

import java.util.List;
import java.util.Map;


/*
 * Sia L il nome di un costrutto e f la formula ad esso associata.
 * Se una regola H(L(f)) B<L1(f1),L2(f2),L3(f3)...> MAP  applicabile 
 * ad un modello m, la sua applicazione produce un nuovo modello, 
 * la cui segnatura :  f AND F'
 * Sia Gi la formula ottenuta ponendo in AND la formula fi del costrutto 
 * Li nel corpo della regola con la formula che corrisponde allo stesso 
 * costrutto nel modello di input. Allora F' si ottiene ponendo in 
 * AND tutti i MuMAP calcolati sulla forma normale disgiuntiva di Gi, 
 * per tutti gli Li presenti nel corpo della regola.
 */

/**
 * Oggetto che permette di calcolare il risultato dell'applicazione 
 * della segnatura di una regola a quella di un modello.
 * @author Fabrizio Celli
 * @version 1.2
 * @since 06-05-2007
 *
 */
public class ServicesApplication {

	//inizializzata nel metodo ApplicateOne
	private Map<String, String> mapping;


	/**
	 * Data la segnatura rule di una regola e quella model di un modello, 
	 * per prima cosa verifica che rule sia applicabile a model; se cos 
	 * non fosse, restituisce null; altrimenti calcola la segnatura del 
	 * costrutto che si ottiene applicando rule a model: tale segnatura avr 
	 * lo stesso nome di quella nella testa della regola, mentre la formula 
	 *  calcolata secondo le regole descritte nel commento iniziale.
	 * Le uniche ipotesi sono che sia rule che model devono avere costrutti 
	 * con una formula non nulla (il che  sempre vero, dato che il costruttore 
	 * di ConstructSignature inizializza la formula a true).
	 * 
	 * TESTATA IL 26-05-2007
	 * @param rule
	 * @param model
	 * @return il modello risultante, null se la regola non  applicabile
	 */
	public ConstructSignature applicateOne(RuleSignature rule, ModelSignature model){

		this.mapping = rule.getMap();
		String nomeCostruttoResult = rule.getHead().getName();
		ConstructSignature result = new ConstructSignature(nomeCostruttoResult);
		String formula = null;	//formula finale da settare in result

		if(this.applicable(rule, model)){
			formula = "";
			//formula contenuta nella segnatura della testa
			String fhead = rule.getHead().getFormula();
			//CostructSignature del corpo della regola e del modello
			List<ConstructSignature> bodyRuleSig = rule.getBody();
			Map<String, ConstructSignature> modelSig = model.getCostrutti();
			for(ConstructSignature c: bodyRuleSig){
				//sicuramente non nullo, visto che la regola  applicabile
				ConstructSignature cModel = modelSig.get(c.getName());
				String modelFormula = cModel.getFormula();
				//costruisco la congiunzione delle formule e calcolo la DNF
				String formulaRes;
				if(modelFormula.equals("true"))
					formulaRes = c.getFormula();
				else
					formulaRes = "("+c.getFormula()+" AND "+modelFormula+")";
				ToolTableaux t = new ToolTableaux(formulaRes);
				formulaRes = t.formaNormDisgiuntiva();

				//valutazione della formula
				String formulaDaValutare = ToolFormuleAsStringhe.removeOuterParenthesis(formulaRes);
				formulaDaValutare = ToolFormuleAsStringhe.removeNotParenthesis(formulaDaValutare);
				String formulaValutata = this.eval(formulaDaValutare);
				if(formulaValutata.equals("false")){
					formula = "false";
					break;
				}		
				if(!formulaValutata.equals("true")){
					if(formula.length()==0)
						formula += formulaValutata;
					else 
						formula = "("+formula + " AND "+formulaValutata+")";
				}
			}
			if(!fhead.equals("true")){
				if(formula.length()==0)
					formula = fhead;
				else
					formula = "("+fhead + " AND "+ formula+")";
			}
			else
				if(formula.length()==0)
					formula = fhead;
		}
		if(formula!=null){
			result.setFormula(formula);
			return result;
		}
		else
			return null;
	}

	/*
	 * Funzione ricorsiva che permette di valutare una formula. Il caso base  quello
	 * in cui la formula  nella forma l1 AND L2 oppure L1 OR L2, con L1 e L2 letterali
	 * TESTATA IL 9-05-2007
	 * @param formula
	 * @param res
	 * @return valutazione della formula senza parentesi esterne
	 */
	private String eval(String formula){
		formula = this.evalFormula(formula);
		return formula;
	}

	/*
	 * Restituisce una formula in forma normale disgiuntiva che  la valutazione
	 * della formula passata per parametro, cio calcola muMap per i letterali.
	 * Valuta anche i valori delle espressioni AND, ma pu contenere ancora
	 * true o false nelle espressioni OR
	 * TESTATA IL 9-05-2007
	 */
	private String evalFormula(String formula){
		ToolTableaux t;
		
		////System.out.println("f "+formula);
		
		int endIndex = ToolFormuleAsStringhe.findEndExpression(formula, 0);	
		////System.out.println("endIndex "+endIndex);

		if(endIndex==formula.length()-1)
			return this.evalAndFormula(formula);
		else{
			//controllo se l'indice  caduto in mezzo all'OR
			if(formula.charAt(endIndex)=='R' && formula.charAt(endIndex-1)=='O')
				endIndex = endIndex-3;
			String operatore = formula.substring(endIndex+2, endIndex+4);
			////System.out.println("operatore "+operatore);

			/*
			 * Se l'operatore  un OR, distribuire i muMap sugli AND solo
			 * se gli operandi sono soddisfacibili, altrimenti vengono
			 * scartati (cio valutati come false). Questo non accade mai se
			 * la DNF la otteniamo col metodo dei tableaux, con cui di 
			 * sicuro le congiunzioni di letterali di un ramo sono
			 * soddisfacibili
			 */
			if(operatore.equals("OR")){
				String f1 = formula.substring(0, endIndex+1);
				f1 = ToolFormuleAsStringhe.removeOuterParenthesis(f1);
				////System.out.println("f1 "+f1);
				String f2 = formula.substring(endIndex+5);
				////System.out.println("f2 "+f2);
				String evalf2 = "false";
				
				t = new ToolTableaux(f2);
				if(t.soddisfacilile()){	//f2
					f2 = ToolFormuleAsStringhe.removeOuterParenthesis(f2);
					////System.out.println("f2 "+f2);
					evalf2 = this.evalAndFormula(f2);
				}


				//se la clausula di destra  true, non devo valutare quella di
				//sinistra, perch l'OR gi vale true
				if(evalf2.equals("true"))
					return "true";

				//altrimenti valuto anche l'espressione alla sinistra dell'OR
				String evalf1 = evalFormula(f1);

				if(evalf2.equals("false"))
					return evalf1;
				else{
					if(evalf1.equals("true"))
						return "true";
					if(evalf1.equals("false"))
						return evalf2;
					return "("+evalf1+" OR "+evalf2+")";
				}
			}
			else{ 
				String evalf = "false";
				////System.out.println(formula);
				t = new ToolTableaux(formula);
				if(t.soddisfacilile())	//formula
					evalf = this.evalAndFormula(formula);
				t = new ToolTableaux(evalf);
				if(!evalf.equals("true") && (evalf.equals("false") 
						|| !t.soddisfacilile()))	//evalf
					return "false";
				//altrimenti torna true o il valore della formula
				return evalf;
			}
		}
	}

	/*
	 * Valuta una formula AND o un letterale. Abbiamo 3 casi:
	 * 1- L
	 * 2- l1 AND L2
	 * 3- (L1 AND L2) AND L3
	 * Inoltre elimina tutti i true e i false dalla formula, 
	 * anche se la formula risultante potrebbe non essere 
	 * soddisfacibile
	 * TESTATA IL 9-05-2007
	 */
	private String evalAndFormula(String formula){
		////System.out.println(formula);
		//		caso 1
		if(!formula.contains("AND"))
			return this.muMap(formula);
		int endIndex = ToolFormuleAsStringhe.findEndExpression(formula, 0);
		//caso 2
		if(endIndex==formula.length()-1){
			int operatorIndex = formula.indexOf("AND");
			String f1 = formula.substring(0, operatorIndex-1);
			f1 = ToolFormuleAsStringhe.removeOuterParenthesis(f1);
			String f2 = formula.substring(operatorIndex+4);
			f2 = ToolFormuleAsStringhe.removeOuterParenthesis(f2);
			f1 = this.evalAndFormula(f1);
			f2 = this.evalAndFormula(f2);
			if(f1.equals("false") || f2.equals("false"))
				return "false";
			if(f1.equals("true")){
				if(f2.equals("true"))
					return "true";
				else
					return f2;
			}
			if(f2.equals("true")){
				return f1;
			}
			return "("+f1+" AND "+f2+")";
		}
		//caso 3
		else{
			String f1 = formula.substring(0, endIndex+1);
			f1 = ToolFormuleAsStringhe.removeOuterParenthesis(f1);
			String f2 = formula.substring(endIndex+6);
			f2 = ToolFormuleAsStringhe.removeOuterParenthesis(f2);
			f1 = this.evalAndFormula(f1);
			f2 = this.evalAndFormula(f2);
			if(f1.equals("false") || f2.equals("false"))
				return "false";
			if(f1.equals("true")){
				if(f2.equals("true"))
					return "true";
				else
					return f2;
			}
			if(f2.equals("true")){
				return f1;
			}
			return "("+f1+" AND "+f2+")";
		}
	}


	/**
	 * Una segnatura di una regola  applicabile ad un modello m, se ogni
	 * segnatura di costrutto presente nel corpo della segnatura della
	 * regola  presente nella segnatura del modello e le proposizioni
	 * di tali segnature di costrutto non si contraddicano (cio la loro
	 * intersezione sia ancora soddisfacibile)
	 * @param rule
	 * @param model
	 * @return true se la regola  applicabile al modello, false altrimenti
	 * TESTATA IL 9-05-2007
	 */
	public boolean applicable(RuleSignature rule, ModelSignature model){

		ToolTableaux t;
		boolean result = true;

		//elenco delle segnature di costrutto nel corpo della regola
		List<ConstructSignature> bodyRule = rule.getBody();
		//mappa delle segnature di costrurro nel modello
		Map<String, ConstructSignature> costruttiModello = model.getCostrutti();

		for(ConstructSignature c: bodyRule){
			//verifichiamo che nel modello ci sia una segnatura con stesso nome
			ConstructSignature temp = costruttiModello.get(c.getName());
			if(temp==null){
				result = false;
				break;
			}
			//verifichiamo che le due segnature non si contraddicano
			//sicuramente qui temp non  null
			String formulaModello = temp.getFormula();
			/*
			 * La formulaRegola  quella del costrutto nel corpo della regola: pu valere
			 * true se non ci sono propriet associate al costrutto (cio argomenti
			 * etichettati associati al letterale nel corpo della regola)
			 */
			String formulaRegola = c.getFormula();
			if(formulaModello.equals("true")){
				//se nella formula del modello c' true, basta controllare la 
				//formula della regola
				if(!formulaRegola.equals("true")){
					t = new ToolTableaux(formulaRegola);
					if(!t.soddisfacilile()){	//formulaRegola
						result = false;
						break;
					}
				}
			}
			else 
				//se la formula del modello  false, l'AND non  soddisfacibile
				if(formulaModello.equals("false")){
					result = false;
					break;	
				}
				else{
					if(!formulaRegola.equals("true")){
						String query = formulaRegola + " AND "+ formulaModello;
						t = new ToolTableaux(query);
						if(!t.soddisfacilile()){
							result = false;
							break;
						}
					}
					else{
						String query = formulaModello;
						t = new ToolTableaux(query);
						if(!t.soddisfacilile()){
							result = false;
							break;
						}
					}
				}
		}

		return result;
	}


	/**
	 * Funzione muMap, necessaria per il calcolo della segnatura del modello
	 * che risulta dall'applicazione di una regola ad un modello.
	 * - muMap(true) = true
	 * - muMap (false) = false
	 * - muMap (x) = y se MAP(y)=x (dalla segnatura della regola)
	 *         altrimenti vale true
	 * - muMap (not x) = not y o true
	 * Nel caso in cui restituisca un letterale negato, mette le parentesi 
	 * intorno ad esso
	 * @param formula
	 * @param mapping
	 * @return stringa corrispondente al valore della funzione
	 * TESTATA IL 19-05-2007
	 */
	private String muMap(String formula){
		String result;
		////System.out.println(formula+" ");
		if(formula.equals("true"))
			result =  "true";
		else
			if(formula.equals("false"))
				result = "false";
			else
				if(!formula.contains("NOT")){
					String corrisp = mapping.get(formula);
					if(corrisp==null)
						result = "true";
					else 
						result = corrisp;
				}
				else{
					String subformula = formula.substring(4);
					String corrisp = mapping.get(subformula);
					if(corrisp==null)
						result = "true";
					else 
						result = "(NOT "+corrisp+")";
				}
		//////System.out.println("RES: "+result);
		return result;
	}

}
