package org.modelgen.signature.application.treeSearcher;

import org.modelgen.commons.persistence.PersistenceException;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.signature.MacroRuleSignature;
import org.modelgen.signature.MacroRuleSignatureComparator;
import org.modelgen.signature.ModelSignature;
import org.modelgen.signature.application.loader.RulesLoader;
import org.modelgen.signature.application.macroruleapplication.SignatureApplicationUtility;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Applicazione che, se eseguita, produce il file "riduzioniOrdinate"
 * nella cartella "macrorules" che per ogni famiglia elenca le riduzioni
 * verso le trasformazioni compatibili
 * @author Fabrizio Celli
 */
public class ApplicationTreeSearcher {

	private List<ModelSignature> families;
	private List<MacroRuleSignature> reductions;
	private List<ModelSignature> modelfromTransformation;


	//estrae dal DB tutte le families definite
	private void initFamilies() throws PersistenceException{
		this.families = GeneralManagementFacade.getInstance().getAllFamiliesSignatures();
	}

	private void initReductions() throws IOException{
		this.reductions = new LinkedList<MacroRuleSignature>();
		List<String> redNames = RulesLoader.getInstance().loadReductions();
		for(String reduction: redNames){
			reductions.add(GeneralManagementFacade.getInstance().getMacroRegolaByfileName(reduction));
		}
	}

	private void initTransformations() throws IOException{
		this.modelfromTransformation = new LinkedList<ModelSignature>();
		List<String> tranNames = RulesLoader.getInstance().loadTransformations();
		for(String tName: tranNames){
			MacroRuleSignature transformation = GeneralManagementFacade.getInstance().getMacroRegolaByfileName(tName);
			this.modelfromTransformation.add(transformation.getModelSignatureFromBodies());
		}
	}

	public void createReductionsFile() throws IOException, PersistenceException{
		//inizializzazioni
		this.initFamilies();
		System.out.println("families initialized");
		this.initReductions();
		System.out.println("reductions initialized");
		this.initTransformations();
		System.out.println("trasformations initialized");

		OrderedReductionsWriter out = new OrderedReductionsWriter();

		//per ogni famiglia
		for(ModelSignature family: families){
			System.out.println("------familY: "+family.getName());
			Map<String, List<MacroRuleSignature>> transformationToReductions = new HashMap<String, List<MacroRuleSignature>>();
			//per ogni modello ottenuto da trasformazione
			for(ModelSignature modelFromBody: this.modelfromTransformation){
				if(SignatureApplicationUtility.getInstance().isCompatibile(family, modelFromBody)){
					System.out.print("++++++ ");
					System.out.println(modelFromBody.getName());
					//cerco le riduzioni
					List<MacroRuleSignature> reduction = (new TreeSearcher()).getReductionFromFamilyToModel(family, modelFromBody, this.reductions);
					System.out.println(reduction);
					if(reduction!=null && reduction.size()>0){
						Collections.sort(reduction, new MacroRuleSignatureComparator());
						transformationToReductions.put(modelFromBody.getName(), reduction);
					}
					else
						transformationToReductions.put(modelFromBody.getName(), new LinkedList<MacroRuleSignature>());
				}
			}
			System.out.println(transformationToReductions);
			out.addFamily(family.getName(), transformationToReductions);
		}
	}

	public static void main(String[] args) throws IOException, PersistenceException{
		GeneralManagementFacade.getInstance().startServer();
		ApplicationTreeSearcher appl = new ApplicationTreeSearcher();
		appl.createReductionsFile();
		GeneralManagementFacade.getInstance().stopServer();
	}



}
