package org.modelgen.signature.application.treeSearcher;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.facade.GeneralManagementFacade;
import org.modelgen.signature.MacroRuleSignature;
import org.modelgen.signature.application.macroruleapplication.NaiveCacheMacroRuleSignature;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

/**
 * Parse the file "riduzioniOrdinate.MACRO"
 * @author Fabrizio Celli
 * @since 1.3.0
 */
public class OrderedReductionsReader {
	
	//flusso di lettura
	BufferedReader in = null;
	
	//	costruttore
	public OrderedReductionsReader() throws IOException{
		//carico il percorso del file "riduzioniOrdinate"
		String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
		String fileName = rootnode+ModelgenConfigurationManager.getIstance().getProperty("orderedReductions", "", "paths");

		//creo il flusso, cancellando il contenuto esistente del file
		in = new BufferedReader(new FileReader(fileName));
	}
	
	/**
	 * Dato il nome della famiglia, restituisce la lista di tutti i nomi di file 
	 * delle trasformazioni associabili alla famiglia, dove per "trasformazione"
	 * si intende una lista di nomi di file di riduzioni con una trasformazione 
	 * finale verso un'altra famiglia.
	 * Queste informazioni sono contenute nel file da parsare.
	 * @param familyName il nome della famiglia
	 * @return la lista di tutti i nomi di file delle trasformazioni associabili alla famiglia
	 * @throws IOException
	 */
	public List<List<String>> readFamily(String familyName) throws IOException{
		List<List<String>> allTransformations = null;
		//per ogni riga ne analizziamo il contenuto
		//se il file non  vuoto
		while(in.ready()){
			String riga = in.readLine();
			if(riga.equals("\n")||(riga.equals("")))
				continue;
			else{
				//appena troviamo la famiglia cercata
				if(riga.equalsIgnoreCase("ANCESTOR NAME: "+familyName)){
					allTransformations = new LinkedList<List<String>>();
					String nextRiga = in.readLine();
					//fino alla famiglia successiva
					while(nextRiga.startsWith("T")){	
						String transformation = nextRiga.substring(3);
						List<String> reduction = new LinkedList<String>();
						while((nextRiga = in.readLine()).startsWith("R"))
							reduction.add(nextRiga.substring(3));
						reduction.add(transformation);
						allTransformations.add(reduction);
					}
				}
			}
		}
		
		return allTransformations;
	}
	
	/**
	 * Dato il nome della famiglia, restituisce la lista di tutte le segnature 
	 * delle trasformazioni associabili alla famiglia, dove per "trasformazione"
	 * si intende una lista di segnature di riduzioni con una trasformazione 
	 * finale verso un'altra famiglia.
	 * Queste informazioni sono contenute nel file da parsare.
	 * @param familyName il nome della famiglia
	 * @return la lista di tutte le segnature delle trasformazioni associabili alla famiglia
	 * @throws IOException
	 */
	public List<List<MacroRuleSignature>> readFamilyReportingSignature(String familyName) throws IOException{
		List<List<String>> rules = this.readFamily(familyName);
		List<List<MacroRuleSignature>> result = null;
		if(rules!=null){
			result = new LinkedList<List<MacroRuleSignature>>();
			for(List<String> names: rules){
				List<MacroRuleSignature> rulesSign = new LinkedList<MacroRuleSignature>();
				for(String name:names){
					//provo a sfruttare la cache per le macroregole. Se non  inizializzata, non la inizializzo
					//perch richiederebbe almeno 10 secondi e in questo caso non serve
					if(NaiveCacheMacroRuleSignature.getInstance().getRegole()==null)
						rulesSign.add(GeneralManagementFacade.getInstance().getMacroRegolaByfileName(name));
					else
						rulesSign.add(NaiveCacheMacroRuleSignature.getInstance().getRegola(name));
				}
				result.add(rulesSign);
			}
		}
		return result;
		
	}

}
