package org.modelgen.signature.application.treeSearcher;

import org.modelgen.commons.utils.ModelgenConfigurationManager;
import org.modelgen.signature.MacroRuleSignature;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Map;

public class OrderedReductionsWriter {

	//flusso su cui scrivere
	private BufferedWriter out = null;

	//costruttore
	public OrderedReductionsWriter() throws IOException{
		//carico il percorso del file "riduzioniOrdinate"
		String rootnode = ModelgenConfigurationManager.getIstance().getProperty("rootnode", "", "paths");
		String fileName = rootnode+ModelgenConfigurationManager.getIstance().getProperty("orderedReductions", "", "paths");

		//creo il flusso, cancellando il contenuto esistente del file
		out = new BufferedWriter(new FileWriter(fileName));
	}

	/**
	 * Metodo che scrive sul file "riduzioniOrdinate.macro": prende come parametro il nome di una 
	 * famiglia e una mappa: questa mappa ha per chiave il nome delle trasformazioni da cui si
	 * estrae un modello per la famiglia, per parametro una lista di riduzioni ordinate che portano
	 * dalla famiglia al modello estratto dalla trasformazione
	 * @param familyName
	 * @param transformationToReductions
	 * @throws IOException
	 */
	public void addFamily(String familyName, Map<String, List<MacroRuleSignature>> transformationToReductions) throws IOException{
		if(transformationToReductions.keySet().size()!=0){
			out.write("ANCESTOR NAME: "+familyName);
			out.newLine();
			for(String transfName: transformationToReductions.keySet()){
				out.write("T: "+transfName);
				out.newLine();
				for(MacroRuleSignature reduct: transformationToReductions.get(transfName)){
					out.write("R: "+reduct.getFileName());
					out.newLine();
				}
			}
			out.newLine();
			out.flush();
		}
	}



}
