package org.modelgen.signature.application.treeSearcher;

import org.modelgen.signature.MacroRuleSignature;
import org.modelgen.signature.ModelSignature;
import org.modelgen.signature.application.macroruleapplication.SignatureApplicationUtility;

import java.util.List;

public class TreeSearcher {

	/**
	 * Dato un modello pi generale e uno target, restituisce la lista delle riduzioni.
	 * Ritorna null se non esiste tale lista.
	 * @param family il modello pi generale
	 * @param target il modello a cui si vuole arrivare
	 * @param reduction la lista delle MacroRuleSignature delle riduzioni disponibili
	 * @return la lista delle riduzioni
	 */
	public List<MacroRuleSignature> getReductionFromFamilyToModel(ModelSignature family, ModelSignature target, List<MacroRuleSignature> reduction){
		Node n = this.getNodeReductionFromFamilyToModel(family, target, reduction);
		if(n==null)
			return null;
		else 
			return n.getPercorso();
	}

	/**
	 * Dato un modello pi generale e uno target, restituisce il nodo dell'albero dei modelli
	 * che soddisfa il modello target. La radice dell'albero  il modello pi generale
	 * Ritorna null se non esiste tale lista.
	 * @param family il modello pi generale
	 * @param target il modello a cui si vuole arrivare
	 * @param reduction la lista delle MacroRuleSignature delle riduzioni disponibili
	 * @return il nodo del modello che soddisfa il modello target
	 */
	public Node getNodeReductionFromFamilyToModel(ModelSignature family, ModelSignature target, List<MacroRuleSignature> reduction){
		//primo nodo dell'albero
		Node n = new Node(family, reduction);
		//analizza il nodo
		return this.analizeRootSubtree(n, target);
	}

	/*
	 * Algoritmo di ricerca in profondit mutuamente ricorsivo
	 */
	private Node analizeRootSubtree(Node root, ModelSignature target){
		//System.out.println("AnalizeRootSubtree");
		if(SignatureApplicationUtility.getInstance().isCompatibile(target, root.getModel())){
			//System.out.println(root.getModel());
			//System.out.println(root.getPercorso());
			return root;
		}
		//se abbiamo raggiunto il limite di profondit, fissato a 2, usciamo
		if(root.getPercorso().size()==2)
			return null;
		//creo tutti i figli di root
		List<Node> figli = root.applicateAllApplicable();
		//System.out.println(figli);
		//se non ci sono figli, siamo arrivati ad una foglia non buona
		if(figli.size()==0)
			return null;
		//altrimenti chiamata mutuamente ricorsiva per visita in profondit
		return this.analizeRootsSubtree(figli, target);
	}

	private Node analizeRootsSubtree(List<Node> figli, ModelSignature target){
		//System.out.println("AnalizeFigli");
		//scandisco tutti i figli e per ciascuno visito in profondit
		for(Node figlio: figli){
			//System.out.println(figlio);
			//se non  stata una riduzione si esce
			if(!figlio.hasCreatedConstructs() && !figlio.hasIgnoredConstructs()){
				//analizzo il nodo
				Node result = this.analizeRootSubtree(figlio, target);
				//se ho trovato il nodo cercato lo riporto
				if(result!=null)
					return result;
			}
		}
		return null;
	}
}
