package org.modelgen.signature.util;

/**
 * Funzioni di utilit per maneggiare le formule proposizionali 
 * scritte come stringhe
 * @author Fabrizio Celli
 * @version 1.2
 * @since 04-05-2007
 * TESTATA CON SUCCESSO IL 19-05-2007
 *
 */
public class ToolFormuleAsStringhe {

	/**
	 * This method removes the outer parenthesis contained into the formula.
	 * Se la formula  la stringa nulla, restituisce null
	 **/
	public static String removeOuterParenthesis(String formula){
		if(formula!=null){
			String f =formula;
			int index;
			int len = formula.length();
			if(formula.charAt(0)=='(' && (findEndExpression(formula,0)==(len-1))){
				do{
					f = f.substring(1,len-1);
					index = findEndExpression(f,0);
					len = formula.length();
				}while(index==(len-1));
			}
			return f;  
		}
		else return null;
	}
	
	/**
	 * This method removes all the parenthesis contained into the formula.
	 * Se la formula  la stringa nulla, restituisce null
	 **/
	public static String removeAllParenthesis(String formula){
		if(formula!=null){
			String f =formula;
			int i = 0;
			while(i<f.length()){
				if(f.charAt(i)==')' || f.charAt(i)=='('){
					if(i==0)
						f = f.substring(i+1);
					else
						if(i==f.length()-1)
							f = f.substring(0, i);
						else
							f = f.substring(0, i)+f.substring(i+1);
				}
				else
					i++;
			}
			return f;  
		}
		else return null;
	}
	
	/**
	 * Organizza le parentesi della formula in modo da avere una formula del tipo:
	 * (A AND B AND ... ) OR (...) OR (...) ...
	 * NB: la stringa in input deve essere la FND della formula, cio deve essere
	 * stata filtrata dal tool tableau
	 */
	public static String mantainsOnlyOrParenthesis(String formula){
		formula = removeAllParenthesis(formula);
		if(formula!=null){
			String f =formula;
			formula = "";
			int orPos = f.indexOf("OR");
			while(orPos!=-1){
				formula += "("+f.substring(0, orPos-1)+") OR ";
				f = f.substring(orPos+3);
				orPos = f.indexOf("OR");
			}
			 formula += "("+f+")";
			if(formula.endsWith(" OR "))
				formula.substring(0, formula.length()-4);
		}
		return formula;
	}


	/**
	 * Rimuove tutte le parentesi intorno ai NOT. Se la formula 
	 * la stringa nulla, riporta null. Se non ci sono parentesi intorno
	 * ai letterali negati, restituisce la formula immutata.
	 */
	public static String removeNotParenthesis(String formula){
		////System.out.println(formula);
		if(formula!=null){
			int index = formula.indexOf("NOT");
			/////System.out.println(index);
			while(index!=-1){
				int endExpr = findEndExpression(formula, index);
				////System.out.println(endExpr);
				//se l'espressione era racchiusa tra tonde
				if(endExpr<formula.length()-1 && formula.charAt(endExpr+1)==')' && formula.charAt(index-1)=='('){
					String expression = formula.substring(index, endExpr+1);
					/////System.out.println(expression);
					if(!expression.contains("OR") && !expression.contains("AND")){
						formula = formula.substring(0,index-1) 
						+ formula.substring(index, endExpr+1)
						+ formula.substring(endExpr+2);
					}
					////System.out.println(formula);
				}
				//se invece non era racchiusa tra parentesi, cerco il prossimo NOT
				index = formula.indexOf("NOT",index+1);
				////System.out.println(index);
			}
			return formula;
		}
		else 
			return null;
	}

	/**
	 * Given a formula and the index of the beginning of an expression
	 * this method returns the index of the end of that expression.
	 * Se applicato alla stringa nulla o ad un indice fuori dai margini
	 * della stringa, ritorna -1
	 **/
	public static int findEndExpression(String formula, int startExpr){
		if(formula!=null && startExpr<formula.length() && startExpr>=0){
			char[] exprDelimiters={'(',')'};
			int endExpr = startExpr;  
			int index = endExpr; 
			/*
			 * Se la formula inizia per '(' va a cercare ')' e pone la sua posizione come
			 * endExpr: si ricerca per anche la successiva '(': se questa viene dopo ')'
			 * ritorna endExpr, altrimenti si prosegue
			 */
			if( formula.charAt(startExpr)=='('){
				do{
					endExpr = formula.indexOf(')', endExpr+1);
					index   = formula.indexOf('(', index+1);
				}while(index != -1 && index<endExpr); // finch non troviamo la fine della sottoespressione
			}
			/*
			 * Si prende la posizione del separatore che viene prima. Se non ce ne sono,
			 * endExpr  proprio l'ultimo carattere della formula, se invece  '(' 
			 *  l'indice di questa -2, altrimenti  l'indice di questa -1;
			 */
			else{
				int index0 = formula.indexOf(exprDelimiters[0], startExpr);
				int index1 = formula.indexOf(exprDelimiters[1], startExpr);
				index = index0;
				/*
				 * Prendo il pi piccolo tra index1 e index2 (-1 non  confrontabile)
				 */
				if(index==-1)
					index = index1;
				else
					if(index1!=-1 && index>index1)
						index = index1;

				if(index!=-1){
					if(formula.charAt(index)=='(') 
						endExpr=index-2;
					else 
						endExpr= index-1;
				}else 
					endExpr = formula.length() -1;
			}
			return endExpr;
		}
		else
			return -1;
	}

}
