package org.modelgen.signature.util;

import java.util.LinkedList;
import java.util.List;

/**
 * Classe che simula il metodo dei tableaux: le nostre formula sono
 * rappresentate da stringhe e contendono solo AND, OR e NOT
 * TESTATO 05-06-2007
 * @author Fabrizio Celli
 * @version 1.3
 * @since 08/05/2007
 *
 */
public class ToolTableaux {

	private List<List<String>> rami;

	public ToolTableaux(String formula){
		this.rami = new LinkedList<List<String>>();
		if(formula!=null){
			List<String> ramo = new LinkedList<String>();
			////System.out.println(formula);
			formula = ToolFormuleAsStringhe.removeOuterParenthesis(formula);
			////System.out.println(formula);
			formula = ToolFormuleAsStringhe.removeNotParenthesis(formula);
			////System.out.println(formula);
			if(!formula.equals("false")){
				ramo.add(formula);
				this.rami.add(ramo);
				this.evalTree();
			}
		}
	}

	/*
	 * Metodo getter
	 */
	public List<List<String>> getRami() {
		return rami;
	}

	/**
	 * Restituisce la forma normale disgiuntiva della formula. Se la formula
	 * non  soddisfacibile oppure se la formula  la stringa nulla (cio in
	 * entrambi i casi la listra rami non contiene niente) restituisce null.
	 * Quando si costruiscono le congiunzioni di letterali, vengono ignorati
	 * eventuali "true", dato che ( true AND x <=> x )
	 * @return fnd di una formula, null se la formula non  soddisfacibile
	 */
	public String formaNormDisgiuntiva(){
		String formulaRes = "";

		for(int i=0;i<this.rami.size(); i++){

			List<String> ramo = this.rami.get(i);
			String formula = this.getCongiunzione(ramo);

			if(formulaRes.length()==0){
				formulaRes += formula;
				//se vale true ci fermiamo
				if(formulaRes.equals("true"))
					break;
			}
			else{
				/*
				 * Se una clausula vale true, possiamo fermarci, dato che
				 * ( true OR x <=> true )
				 */
				if(formula.equals("true")){
					formulaRes = "true";
					break;
				}
				else
					formulaRes = "("+formulaRes+" OR "+formula+")";
			}
		}

		if(formulaRes.equals(""))
			return null;
		else
			return formulaRes;
	}


	/**
	 * Dato un tableau subTableau, verifica se  contenuto in questo
	 * tableau, cio se  pi restrittivo. 
	 * Il che significa che per ogni ramo di subTableau, deve esistere
	 * un ramo di questo tableau che  contenuto nel ramo di subTableau
	 * (in cui possono esserci altri letterali).
	 * @param subTableau
	 * @return true se subTableau  contenuto in questo tableau
	 * 
	 */
	public boolean isSoddisfattoDa(ToolTableaux subTableau){
		boolean result = true;
		List<List<String>> ramiSub = subTableau.getRami();
		//scandisco i rami del tableau pi piccolo
		for(List<String> ramoSub: ramiSub){
			boolean trovato = false;
			//scandisco i rami del tableau pi grande
			for(List<String> ramo: this.rami){
				//controllo se questo ramo  contenuto in quello del tableau pi grande
				if(ramoSub.containsAll(ramo)){
					trovato = true;
					break;
				}
			}
			if(!trovato){
				result = false;
				break;
			}
		}
		return result;
	}

	/**
	 * Restituisce false se la formula non  soddisfacibile,
	 * cio se tutti i rami del tableau sono chiusi
	 * @return true se la formula  soddisfacibile, false altrimenti
	 */
	public boolean soddisfacilile(){
		boolean res = true;
		if(this.rami.size() == 0) //cio se tutti i rami sono chiusi
			res = false;
		return res;
	}



	/*
	 * Una formula  espandibile se contiene almeno una operazione
	 * di AND o di OR
	 */
	private boolean isEspandibile(String formula){
		boolean result = false;
		if(formula.contains("AND") || formula.contains("OR"))
			result = true;
		return result;
	}

	/*
	 * Generazione del tableau: gli AND provocano la stesura in serie
	 * delle espressioni che legano, gli OR in parallelo
	 */
	private void evalTree(){
		//indice del ramo
		int j = 0;

		//finch ci sono rami con formule da espandere
		while(j<rami.size()){

			List<String> ramo = this.rami.get(j);
			//indice della formula nel ramo
			int i = 0;
			/*
			 * Scandisco la lista, aggiungendo e rimuovendo elementi, finch
			 * non finiscono gli elementi
			 */
			while(i<ramo.size()){
				String formula = ramo.get(i);
				if(this.isEspandibile(formula)){
					//System.out.println("??????????????????????");
					//System.out.println(formula);
					//espandere la formula
					int endIndex = ToolFormuleAsStringhe.findEndExpression(formula, 0);
					//////System.out.println("formula: "+formula);
					//////System.out.println("endIndex "+endIndex);
					/*
					 * La prima espressione nella formula pu terminare per:
					 * - ')' nel caso in cui la formula sia del tipo (F1 AND F1) OR ...
					 * - 'D' se  del tipo F1 AND (...)
					 * - 'R' se  del tipo F1 OR (...)
					 * - altro se  del tipo F1 AND F2 o F1 OR F2
					 */
					if(formula.charAt(endIndex)==')'){
						String operatore = formula.substring(endIndex+2,endIndex+4);
						//////System.out.println(operatore);
						if(operatore.equals("AN")){
							//.out.println("separazione1");
							String newFormula = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(1,endIndex));
							//System.out.println("!!!!!!!!!!!!!!!!!");
							//System.out.println(newFormula);
							ramo.add(newFormula);
							String newFormula2 = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(endIndex+6));
							//System.out.println(newFormula2);
							ramo.add(newFormula2);
							ramo.remove(formula);
						}
						else{	
							//l'operatore  sicuramente OR
							//System.out.println("separazione2");
							ramo.remove(formula);
							List<String> newRamo = new LinkedList<String>();
							newRamo.addAll(ramo);
							String newFormula = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(1,endIndex));
							//System.out.println(newFormula);
							ramo.add(newFormula);
							String newFormula2 = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(endIndex+5));
							newRamo.add(newFormula2);
							//System.out.println(newFormula2);
							this.rami.add(newRamo);
						}
					}
					else
						if(formula.charAt(endIndex)=='D'){
							//System.out.println("separazione3");
							String newFormula = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(0,endIndex-3));
							ramo.add(newFormula);
							//System.out.println("!!!!!!!!!!!!!!!!!");
							//System.out.println(newFormula);
							String newFormula2 = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(endIndex+2));
							ramo.add(newFormula2);
							//System.out.println(newFormula2);
							ramo.remove(formula);	
						}
						else
							if(formula.charAt(endIndex)=='R'){
								//System.out.println("separazione4");
								ramo.remove(formula);
								List<String> newRamo = new LinkedList<String>();
								newRamo.addAll(ramo);
								String f1 = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(0,endIndex-2));
								ramo.add(f1);
								//System.out.println("f1 _"+f1+"_");
								String f2 = ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(endIndex+2));
								newRamo.add(f2);
								//System.out.println(f2);
								this.rami.add(newRamo);
							}
							else{
								//System.out.println("separazione5");
								int index;
								index = formula.indexOf("AND");
								//////System.out.println("indexAND "+index);
								if(index!=-1){
									ramo.add(ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(0, index-1)));
									ramo.add(ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(index+4)));
									ramo.remove(formula);
								}
								else{
									//System.out.println("separazione6");
									index = formula.indexOf("OR");	//sicuramente diverso da -1
									//////System.out.println("indexOR "+index);
									ramo.remove(formula);
									List<String> newRamo = new LinkedList<String>();
									newRamo.addAll(ramo);
									ramo.add(ToolFormuleAsStringhe.removeOuterParenthesis(formula.substring(0,index-1)));
									//System.out.println(formula);
									String newFormula = formula.substring(index+3);
									//System.out.println(newFormula);
									newRamo.add(ToolFormuleAsStringhe.removeOuterParenthesis(newFormula));
									this.rami.add(newRamo);
									//////System.out.println("fatto");
								}
							}
				}
				else
					i++;
			}
			j++;
		}
		this.eliminaRamiChiusi();
	}

	/*
	 * Elimina i rami chiusi dalla lista dei rami: pu essere invocata
	 * solo dopo eval tree, perch richiede che nei rami ci siano solo
	 * letterali, cio atomi positivi o negati. 
	 * Inoltre elimina le eventuali copie dagli altri rami.
	 * Eliminare anche eventuali clausule (rami) ridondanti: se una
	 * clausula contiene tutti gli elementi di un'altra ed ha altri
	 * elementi, pu essere eliminata.
	 * Inoltre se c' un ramo e ce ne  un altro che  la negazione del
	 * primo, i due rami diventano "true"
	 */
	private void eliminaRamiChiusi(){

		//per ogni ramo
		for(int i=0; i<this.rami.size(); i++){

			List<String> ramo = rami.get(i);
			//specifica se ramo  ancora contenuto nella lista rami
			boolean isPresente = true;
			
			//calcola la negazione del ramo
			List<String> ramoNegato = this.getNegato(ramo);
			
			//possibile nuovo valore da assegnargli
			String newValue = "";

			//elimino tutti i rami uguali a questo
			//oppure che contengono questo
			for(int k=i+1; k<this.rami.size(); k++){
				List<String> succ = rami.get(k);
				//se succ contiene tutti gli elementi di ramo, o sono 
				//uguali o succ  ridondante
				if(succ.containsAll(ramo)){
					rami.remove(succ);
					k--;
				}
				else
					//se invece ramo  ridondante e ancora non  stato
					//eliminato
					if(isPresente && ramo.containsAll(succ)){
						rami.remove(ramo);
						isPresente = false;
						k--;
						i--;
					}
					else{
						//se il ramo  la negazione del precedente
						//vengono portati entrambi a true
						if(succ.containsAll(ramoNegato)){
							//se entrambi non contengono false, li metto a true, altrimenti a false
							//non faccio subito l'aggiornamento perch potrebbero esserci altri rami
							//da eliminare
							if(succ.contains("false") && ramo.contains("false") && newValue!="true"){
								rami.remove(succ);
								k--;
								newValue = "false";
							}
							else{
								rami.remove(succ);
								k--;
								newValue = "true";
							}
						}
					}
			}

			if(isPresente){
				//se serve aggiorno il valore
				if(newValue!=""){
					ramo.clear();
					ramo.add(newValue);
				}
				//per ogni formula nel ramo
				for(int j=0; j<ramo.size(); j++){
					boolean removable = false;
					String elemento = ramo.get(j);
					/*
					 * Se il letterale vale false, il ramo  chiuso
					 */
					if(elemento.equals("false"))
						removable = true;
					else{
						/*
						 * Se il letterale  negato, cerco il corrispondente
						 * positivo tra i successivi: se esite, il ramo  chiuso
						 * e quindi va eliminato
						 */
						if (elemento.contains("NOT")){
							String elemento2 = elemento.substring(4);	
							for(int k=j+1; k<ramo.size(); k++){
								String elemCurr = ramo.get(k);
								if(elemCurr.equals(elemento2)){
									removable = true;
									break;
								}
								if(elemCurr.equals(elemento)){
									ramo.remove(k);
									k--;
								}
							}
						}
						/*
						 * Se il letterale  positivo, cerco il corrispondente
						 * negativo tra i successivi: se esite, il ramo  chiuso
						 * e quindi va eliminato
						 */
						else{
							for(int k=j+1; k<ramo.size(); k++){
								String elemCurr = ramo.get(k);
								//////System.out.println(ramo.get(k));
								if(elemCurr.equals("NOT "+elemento)){
									removable = true;
									break;
								}
								if(elemCurr.equals(elemento)){
									ramo.remove(k);
									k--;
								}
							}
						}
					}
					/*
					 * Se il ramo viene eliminato, decremento i (altrimenti con
					 * l'incremento del for perdo un ramo, quello che scala per via
					 * dell'eliminazione del corrente)
					 */
					if(removable){
						rami.remove(i);
						i--;
						break;
					}
				}
			}
		}

	}
	
	/**
	 * Prende in input una formula X, che  in genere una congiunzione di letterali.
	 * Nel caso in cui sia una disgiunzione di clausule, viene considerata solo la 
	 * prima: ad esempio, se fosse c1 OR c2 OR ... CN, consideriamo solo c1.
	 * A questo punto viene eseguito il seguente algoritmo:
	 * 	- per ogni ramo R di questo tableau
	 *        - per ogni letterale L del ramo
	 *              - se X contiene NOT L restituisco R
	 *              - else se X  l'ultimo, restituisco R AND (NOT X)
	 * Se il risultato fosse "false", ritorna null.
	 * @param formulaConstRule
	 */
	public String ignoredLiteral(String formulaConstRule) {
		//formula risultante
		String output = "";
		//per ogni ramo del tableau, cio per ogni congiunzione di letterali
		for(List<String> congiunzione: this.rami){
			//per ogni letterale di tale ramo
			for(int i=0; i<congiunzione.size(); i++){
				String letterale = congiunzione.get(i);
				//tableau per la formula formulaConstRule, che per l'uso che ne facciamo  una
				//semplice congiunzione di letterali
				ToolTableaux t = new ToolTableaux(formulaConstRule);
				//se il letterale  negato
				if(letterale.contains("NOT")){
					//se il costrutto della regola lo contiene positivo
					if(t.getRami().get(0).contains(letterale.substring(4))){
						if(output.equals(""))
							output = this.getCongiunzione(congiunzione);
						else
							output = "("+output+" OR "+this.getCongiunzione(congiunzione)+")";
						break;
					}
					else {
						//se  l'ultimo letterale del ramo
						if(i==congiunzione.size()-1){
							if(output.equals(""))
								output = "("+this.getCongiunzione(congiunzione)+" AND "+t.getDisgiunzioneOfRamoNegato(this.rami.get(0))+")";
							else
								output = "("+output+" OR "+"("+this.getCongiunzione(congiunzione)+" AND "+t.getDisgiunzioneOfRamoNegato(this.rami.get(0))+"))";
							//non serve il break perch il letterale  l'ultimo
						}
					}
				}
				else{
					if(t.getRami().get(0).contains("NOT "+letterale)){
						if(output.equals(""))
							output = this.getCongiunzione(congiunzione);
						else
							output = "("+output+" OR "+this.getCongiunzione(congiunzione)+")";
						break;
					}
					else{
						//se  l'ultimo letterale del ramo
						if(i==congiunzione.size()-1){
							if(output.equals(""))
								output = "("+this.getCongiunzione(congiunzione)+" AND "+t.getDisgiunzioneOfRamoNegato(this.rami.get(0))+")";
							else
								output = "("+output+" OR "+"("+this.getCongiunzione(congiunzione)+" AND "+t.getDisgiunzioneOfRamoNegato(this.rami.get(0))+"))";
							//non serve il break perch il letterale  l'ultimo
						}
					}

				}
			}
		}
		if(output.equals(""))
				output = "false";
		ToolTableaux t = new ToolTableaux(output);
		return t.formaNormDisgiuntiva();
	}
	
	/**
	 * Restituisce la formula negata in FORMA NORMALE CONGIUNTIVA.
	 * @return la FNC della formula negata
	 */
	public String negaFormulaToCNF(){
		String formulaRes = "";

		for(int i=0;i<this.rami.size(); i++){

			List<String> ramo = this.rami.get(i);
			String formula = this.getDisgiunzioneOfRamoNegato(ramo);

			if(formulaRes.length()==0){
				//se vale true ci fermiamo
				if(formulaRes.equals("true"))
					continue;
				formulaRes += formula;
			}
			else{
				/*
				 * Se una clausula disgiuntiva vale true la ignoriamo, dato che
				 * ( true AND x <=> x )
				 */
				if(formula.equals("true")){
					formulaRes = "true";
					continue;
				}
				else
					formulaRes = "("+formulaRes+" AND "+formula+")";
			}
		}

		if(formulaRes.equals(""))
			return null;
		else
			return formulaRes;
	}

	/*
	 * Nega il ramo, riportandone la disgiunzione dei letterali
	 */
	private String getDisgiunzioneOfRamoNegato(List<String> ramo) {
		return this.getDisgiunzione(this.getNegato(ramo));
	}
	
	//nega il ramo
	private List<String> getNegato(List<String> ramo) {
		List<String> ramoNegato = new LinkedList<String>();
		for(String letterale:ramo){
			if(letterale.equals("true"))
				letterale = "false";
			else
				if(letterale.equals("false"))
					letterale = "true";
				else
					if(!letterale.contains("NOT"))
						letterale = "NOT "+letterale;
					else
						letterale = letterale.substring(4);
			ramoNegato.add(letterale);
		}
		return ramoNegato;
	}

	/*
	 * Dato un ramo, riporta la disgiunzione di letterali
	 */
	private String getDisgiunzione(List<String> ramo) {
		String formula = "";
		for(int j=0; j<ramo.size(); j++){
			String letterale = ramo.get(j);
			/*
			 * Se il letterale vale true la formula vale true
			 */
			if(letterale.equals("true")){
				formula = "true";
			}
			else{
				if(letterale.contains("NOT")){
					if(formula.length()==0)
						formula += "("+letterale+")";
					else 
						formula = "("+formula+" OR ("+letterale+"))";
				}
				else{
					if(formula.length()==0)
						formula += letterale;
					else 
						formula = "("+formula+" OR "+letterale+")";
				}
			}
		}
		return formula;
	}

	/*
	 * Dato un ramo, riporta la congiunzione di letterali
	 */
	private String getCongiunzione(List<String> ramo){
		String formula = "";
		for(int j=0; j<ramo.size(); j++){
			String letterale = ramo.get(j);
			/*
			 * Se il letterale vale true ed  l'unico letterale del ramo
			 * la formula vale true, altrimenti viene ignorato
			 */
			if(letterale.equals("true")){
				if(formula.length()==0 && j==(ramo.size()-1))
					formula = "true";
			}
			else{
				if(letterale.contains("NOT")){
					if(formula.length()==0)
						formula += "("+letterale+")";
					else 
						formula = "("+formula+" AND ("+letterale+"))";
				}
				else{
					if(formula.length()==0)
						formula += letterale;
					else 
						formula = "("+formula+" AND "+letterale+")";
				}
			}
		}
		return formula;
	}

}
