/*
 * $Id: OrthogonalLinkRouterUtils.java,v 1.3 2007/03/19 12:32:41 david Exp $
 * Copyright (c) 2005-2007, David Benson
 *
 * All rights reserved.
 *
 * This file is licensed under the JGraph software license, a copy of which
 * will have been provided to you in the file LICENSE at the root of your
 * installation directory. If you are unable to locate this file please
 * contact JGraph sales for another copy.
 */package com.jgraph.layout.orthogonal;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Map;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.GraphConstants;

import com.jgraph.layout.JGraphFacade;

final class OrthogonalLinkRouterUtils {

	//    private static final double CORNER_SIZE = 5;
	static final int SPACING = 6;

	static void calculateCollisionBounds(JGraphFacade facade, Object cell,
			Object sourcePort, Object targetPort, Collection verticalCollisions,
			Collection horizontalCollisions) {
		final Object[] vertices = facade.getVertices().toArray();
		for (int i = 0; i < vertices.length; i++) {
			final Object node = vertices[i];
			Rectangle2D bounds = facade.getBounds(node);
			bounds = ((AttributeMap) (facade.getAttributes(node)))
					.createRect(bounds);
			bounds.setRect(bounds.getX() - SPACING, bounds.getY() - SPACING,
					bounds.getWidth() + SPACING, bounds.getHeight() + SPACING);
			verticalCollisions.add(bounds);
			horizontalCollisions.add(bounds);
		}

		final Object[] edges = facade.getEdges().toArray();
		for (int i = 0; i < edges.length; i++) {
			final Object link = edges[i];
			Object otherLinkSourcePort = facade.getSourcePort(link);
			Object otherLinkTargetPort = facade.getTargetPort(link);

			if (OrthogonalLinkRouter.IGNORE_LINKS_WITH_ACTUAL_PORTS
					&& (otherLinkSourcePort == sourcePort || otherLinkTargetPort == targetPort))
				continue;
			Object[] points = facade.getPoints(link).toArray();
			if (points != null) {
				final int length = points.length - 1;
				for (int j = 0; j < length; j++) {
					final Point2D firstPoint = (Point2D) points[j];
					final Point2D secondPoint = (Point2D) points[j + 1];

					if (j < 2 && otherLinkSourcePort == sourcePort)
						continue;
					if (j > length - 3 && otherLinkTargetPort == targetPort)
						continue;

					final double differenceX = secondPoint.getX()
							- firstPoint.getX();
					final double differenceY = secondPoint.getY()
							- firstPoint.getY();

					if (differenceY == 0) { // horizontalBounds
						if (differenceX > 1) { // to-right direction
							Rectangle2D r = new Rectangle2D.Double(firstPoint
									.getX(), firstPoint.getY(), differenceX,
									0.0);
							r.setRect(r.getX() - SPACING, r.getY() - SPACING, r
									.getWidth()
									+ SPACING, r.getHeight() + SPACING);
							horizontalCollisions.add(r);
						} else if (differenceX < -1) { // to-left direction
							Rectangle2D r = new Rectangle2D.Double(secondPoint
									.getX(), secondPoint.getY(), -differenceX,
									0.0);
							r.setRect(r.getX() - SPACING, r.getY() - SPACING, r
									.getWidth()
									+ SPACING, r.getHeight() + SPACING);
							horizontalCollisions.add(r);
						}
					} else if (differenceX == 0) { // verticalBounds
						if (differenceY > 1) { // to-down direction
							Rectangle2D r = new Rectangle2D.Double(firstPoint
									.getX(), firstPoint.getY(), 0.0,
									differenceY);
							r.setRect(r.getX() - SPACING, r.getY() - SPACING, r
									.getWidth()
									+ SPACING, r.getHeight() + SPACING);
							verticalCollisions.add(r);
						} else if (differenceY < -1) { // to-up direction
							Rectangle2D r = new Rectangle2D.Double(secondPoint
									.getX(), secondPoint.getY(), 0.0,
									-differenceY);
							r.setRect(r.getX() - SPACING, r.getY() - SPACING, r
									.getWidth()
									+ SPACING, r.getHeight() + SPACING);
							verticalCollisions.add(r);
						}
					}
				}
			}
		}
	}

	static int getDirection(JGraphFacade facade, Object port) {
		Map attributes = facade.getAttributes(port);
		Point2D offset = GraphConstants.getOffset(attributes);
		if (offset != null) {
			if (offset.getX() == GraphConstants.PERMILLE) {
				return IDirectionable.RIGHT;
			} else if (offset.getX() == 0) {
				return IDirectionable.LEFT;
			} else if (offset.getY() == 0) {
				return IDirectionable.TOP;
			} else if (offset.getY() == GraphConstants.PERMILLE) {
				return IDirectionable.BOTTOM;
			} else {
				return IDirectionable.CENTER;
			}
		} else {
			return IDirectionable.CENTER;
		}
	}
}
